*DECK AFMTAV
      SUBROUTINE AFMTAV (NBURN,ITM,XBMAX,XBMIN,
     1         YS,Y,U,NBMIN,NBMAX,XB,SIGAV)
*
*---------------------------------------------------------------*
*
* TIME AVERAGE CALCULATION USING DIFFERENT APPROXIMATION.
*
*---------------------------------------------------------------*
*
* INPUT PARAMETERS:
*  NBURN  : TOTAL NUMBER OF STEPS.
*  XBMAX  : HIGHEST  VALUE.
*  XBMIN  : LOWER  VALUE.
*  YS  : PARAMETER TO BE INTEGRATED DIMENSION(NBURN)
*  XB  : STEPS DIMENSION(NBURN)
*  ITM    : TYPE OF THE APPROXIMATION
*           1-LAGRANGE
*           2-SPLINE
*           3-HERMIT
*
* OUTPUT PARAMETERS:
* SIGAV   : AVERAGE VALUE OF YS DIMENSION(1)
*
* SCRATCH STORAGE:
*  Y  : DIMENSION (NBURN)
*  U  : DIMENSION (NBURN)
*
*---------------------------------------------------------------*
*--------------------------- AUTHOR: M.T. SISSAOUI ; 20/09/96 --*
*---------------------------------------------------------------*
      DIMENSION YS(*),XB(*),SIGAV(1),U(*),Y(*),IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*
      IF(NBMAX.GT.0) THEN
        INMAX=NBMAX-2
        INMIN=NBMIN
      ELSE
        INMAX=ABS(NBMAX)-1
        INMIN=NBMIN
        IF(ITM.EQ.1)
     1  CALL XABORT('AFMTAV: MORE BURNUP STEPS ARE REQUIRED TO USE '
     1 //' LAGRANGE METHOD, CHOOSE HERMIT OR SPLINE METHOD')
      ENDIF
      IF(ABS(NBMAX).GT.NBMIN) THEN
        SIGAV(1)=0.0
*
        IF(ITM.EQ.1) THEN
* TIME AVERAGE CALCULATION USING LAGRANGE APPROXIMATION.
*
          DO 113 IR=INMIN,INMAX
            I1=IR
            I2=IR+1
            I3=IR+2
            XBI=MAX(XBMIN,XB(IR))
            XBF=MIN(XBMAX,XB(IR+1))
            TX=XBF-XBI
            TX2=XBF**2-XBI**2
            TX3=XBF**3-XBI**3
            X12=XB(I1)-XB(I2)
            X13=XB(I1)-XB(I3)
            X23=XB(I2)-XB(I3)
            XA12=XB(I1)+XB(I2)
            XA13=XB(I1)+XB(I3)
            XA23=XB(I2)+XB(I3)
            XM12=XB(I1)*XB(I2)
            XM13=XB(I1)*XB(I3)
            XM23=XB(I2)*XB(I3)
            Y1=YS(I1)/(X12*X13)
            Y2=-YS(I2)/(X12*X23)
            Y3=YS(I3)/(X13*X23)
*
            SIGAV(1)=SIGAV(1) +
     1               Y1*(TX3/3.0-XA23*TX2/2.0+XM23*TX)+
     1               Y2*(TX3/3.0-XA13*TX2/2.0+XM13*TX)+
     1               Y3*(TX3/3.0-XA12*TX2/2.0+XM12*TX)
 113      CONTINUE
*
        ELSE IF(ITM.EQ.2) THEN
* TIME AVERAGE CALCULATION USING SPLINE APPROXIMATION.
*     THE LOWER BOUNDARY CONDITION IS SET TO BE NATURAL
          Y(1)=0.0
          U(1)=0.0
*     THE UPPER BOUNDARY CONDITION IS SET EITHER TO BE NATURAL
          QN=0.0
          UN=0.0
*
          DO 103 IR=2,NBURN-1
            SIG=(XB(IR)-XB(IR-1))/(XB(IR+1)-XB(IR-1))
            P=SIG*Y(IR-1)+2.0
            Y(IR)=(SIG-1.0)/P
            U(IR)=(6.*((YS(IR+1)-YS(IR))/(XB(IR+1)-XB(IR))-
     1            (YS(IR)-YS(IR-1))/(XB(IR)-XB(IR-1)))/(XB(IR+1)-
     1            XB(IR-1))-SIG*U(IR-1))/P
 103      CONTINUE
*
          Y(NBURN)=(UN-QN*U(NBURN-1))/(QN*Y(NBURN-1)+1.0)
*
          DO 104 K=NBURN-1,1,-1
            Y(K)=Y(K)*Y(K+1)+U(K)
 104      CONTINUE
*
*  COMPUTE THE INTEGRAL OF THE X-SECTION
          INMAX=NBMAX-2
          INMIN=NBMIN
          DO 300 IR=INMIN,INMAX
            H=XB(IR+1)-XB(IR)
            XBI=MAX(XBMIN,XB(IR))
            XBF=MIN(XBMAX,XB(IR+1))
*
            DB=XBF-XBI
            HF=XB(IR+1)-XBF
            HI=XB(IR+1)-XBI
*
            AI=-0.5*(HF**2-HI**2)/H
            BI=DB-AI
            CI=-(AI/6)*H**2-(HF**4-HI**4)/(24*H)
            DI=-(BI/6)*H**2-(HF**4-HI**4)/(24*H)
*
            SIGAV(1)=SIGAV(1)+AI*YS(IR)+BI*YS(IR+1)+
     1               CI*Y(IR)+DI*Y(IR+1)
  300     CONTINUE
        ELSE IF(ITM.EQ.3) THEN
* TIME AVERAGE CALCULATION USING HERMIT APPROXIMATION.
          DO 101 I=1,NBURN
            Y(I)=YS(I)
  101     CONTINUE
* TAKE THE DERIVATIVE WITH RESPECT TO BURNUP OR NEUTRON EXPOSURE AT
* TABULATION POINTS.
          CALL SETARA(BASE,2*NBURN,IZGAR)
          CALL ALDERV(NBURN,XB,Y,BASE(IZGAR))
          CALL RLSARA(BASE(IZGAR))
*
*  COMPUTE THE INTEGRAL OF THE X-SECTION
          DO 200 IR=INMIN,INMAX
            DX=XB(IR+1)-XB(IR)
            XBI=MAX(XBMIN,XB(IR))
            XBF=MIN(XBMAX,XB(IR+1))
            UF=(XBF-0.5*(XB(IR)+XB(IR+1)))/DX
            UI=(XBI-0.5*(XB(IR)+XB(IR+1)))/DX
            H1=-(0.5-UF)**3+0.5*(0.5-UF)**4
     1         +(0.5-UI)**3-0.5*(0.5-UI)**4
            H2=-((0.5-UF)**3)/3.0+0.25*(0.5-UF)**4
     1         +((0.5-UI)**3)/3.0-0.25*(0.5-UI)**4
            H3=(0.5+UF)**3-0.5*(0.5+UF)**4
     1         -(0.5+UI)**3+0.5*(0.5+UI)**4
            H4=-((0.5+UF)**3)/3.0+0.25*(0.5+UF)**4
     1         +((0.5+UI)**3)/3.0-0.25*(0.5+UI)**4
*
            SIGAV(1)=SIGAV(1)+H1*YS(IR)*DX+H2*Y(IR)*DX**2+
     1      H3*YS(IR+1)*DX+H4*Y(IR+1)*DX**2
*
  200     CONTINUE
        ENDIF
        SIGAV(1)=SIGAV(1)/(XBMAX-XBMIN)
      ELSE
        SIGAV(1)=YS(NBMIN)
      ENDIF
*
      RETURN
      END
