*DECK AXLCAL
      SUBROUTINE AXLCAL(NGRP,NZC,NCH,NK,ZONE,FLUR,FLX,FLXZ,EPSF,KX)
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        *******************************************************       *
*        *                                                     *       *
*        *        MODULE OF AXIAL FLUX SHAPE CALCULATION       *       *
*        *                                                     *       *
*        *******************************************************       *
*                                                                      *
*   INPUT:                                                             *
*    NGRP : NUMBER OF ENERGY GROUPS                              I     *
*    NZC  : NUMBER OF COMBUSTION ZONES                           I     *
*    NCH  : NUMBER OF CHANNELS                                   I     *
*    NK   : NUMBER OF BUNDLES PER CHANNEL                        I     *
*    ZONE : COMBUSTION ZONE INDICES. ZONE(NCH)                   I     *
*    FLUR : FUEL AVERAGE FLUXES. FLUR(NCH,NK,NGRP)               R     *
*    KX   : INDICATOR FOR ZONES WHERE AXIAL FLUX SHAPE           I     *
*           ARE COMPUTED, =3 NCH OR =2 NZC                             *
*                                                                      *
*   OUTPUT:                                                            *
*    FLX  : AXIAL FLUX SHAPES.   FLX(NCH,NK,NGRP)                R     *
*    FLXZ : AXIAL FLUX SHAPES PER COMBUSTION ZONES.                    *
*                                FLXZ(NZC,NK,NGRP)               R     *
*    EPSF : ERROR RMS BETWEEN TWO FLUX SHAPE CALCULATIONS        R     *
*
*     SUBROUTINE OF OPTEX-4, INCORPORATED IN DONJON
*
*----------------:-------------------------:---------------------------*
*                                                                      *
*
      IMPLICIT NONE
*
      INTEGER   NGRP,NZC,NCH,NK,ZONE(NCH),IFLX2,I,J,IG,K,IZC,KX
      REAL      FLUR(NCH,NK,NGRP),FLX(NCH,NK,NGRP),EPSF,PN,PD,SUM,ALP,
     +          BASE,FLXZ(NZC,NK,NGRP)
      PARAMETER (ALP=.65)
      COMMON BASE(1)
*
*---- KEEPING OLD VALUES OF FLX
*
      CALL SETARA(BASE,NCH*NK*NGRP,IFLX2)
*
      DO 5 IG=1,NGRP
      DO 5 I=1,NCH
         DO 7 J=1,NK
            K = (IG-1)*NCH*NK + (J-1)*NCH + I
            BASE(IFLX2-1+K) = FLX(I,J,IG)
            FLX(I,J,IG) = 0.0
 7       CONTINUE
 5    CONTINUE
*
      DO 15 IG=1,NGRP
       DO 15 I=1,NZC
         DO 17 J=1,NK
            FLXZ(I,J,IG) = 0.0
 17      CONTINUE
 15   CONTINUE
*
      IF (KX.EQ.3) THEN
       DO 30 IG=1,NGRP
       DO 30 I=1,NCH
         DO 30 J=1,NK
            FLX(I,J,IG) =  FLUR(I,J,IG)
 30    CONTINUE
      ELSE
       DO 31 IG=1,NGRP
        DO 32 I=1,NCH
         DO 33 J=1,NK
            IZC=ZONE(I)
            FLXZ(IZC,J,IG)= FLXZ(IZC,J,IG) + FLUR(I,J,IG)
 33      CONTINUE
 32     CONTINUE
        DO 34 I=1,NCH
         DO 35 J=1,NK
            IZC=ZONE(I)
            FLX(I,J,IG)= FLXZ(IZC,J,IG)
 35      CONTINUE
 34     CONTINUE
 31    CONTINUE
      ENDIF
*
*---- NEW VALUES OF FLX(*,*,*) WITH RELAXATION PARAMETER
*
      PN = 0.0
      PD = 0.0
      DO 70 IG=1,NGRP
      DO 70 I=1,NCH
         SUM = 0.0
         DO 50 J=1,NK
            SUM = SUM + FLX(I,J,IG)
 50      CONTINUE
         DO 60 J=1,NK
            K = (IG-1)*NCH*NK + (J-1)*NCH + I
            FLX(I,J,IG) = ALP*FLX(I,J,IG)/SUM+(1.-ALP)*BASE(IFLX2-1+K)
            PN = PN + (FLX(I,J,IG)-BASE(IFLX2-1+K))**2
            PD = PD + FLX(I,J,IG)**2
 60      CONTINUE
 70   CONTINUE
      EPSF = SQRT(PN)/SQRT(PD)
      CALL RLSARA(BASE(IFLX2))
      IF(KX.EQ.2) THEN
          WRITE(6,*) ' CALCULAITON PER ZONE '
      ELSE
          WRITE(6,*) ' CALCULATION OVER EVERY CHANNEL '
      ENDIF
      DO 90 IG=1,NGRP
       DO 90 I=1,NCH
         DO 90 J=1,NK
            IZC=ZONE(I)
            FLXZ(IZC,J,IG)= FLXZ(IZC,J,IG) + FLX(I,J,IG)
 90   CONTINUE
      CALL AXLNRM(NGRP,NZC,NK,FLXZ)
      DO 81 IG=1,NGRP
          WRITE(6,*) '                  GROUP ',IG
       DO 81 I=1,NZC
          WRITE(6,100)  (FLXZ(I,J,IG),J=1,NK)
 81   CONTINUE
      RETURN
*
 100  FORMAT(/5X,'AXIAL  FORM  : ' /(5X,12(F6.4,1X))/)
      END
