*DECK BIVDFH
      SUBROUTINE BIVDFH (MAXEV,MAXKN,IMPX,ISPLH,LX,SIDE,NELEM,NUN,IHEX,
     1 NCODE,ZCODE,MAT,VOL,IDL,KN,QFR,BFR,MUW)
*
*-----------------------------------------------------------------------
*
* NUMBERING CORRESPONDING TO A MESH CENTERED FINITE DIFFERENCE
* DISCRETIZATION OF A 2-D HEXAGONAL GEOMETRY.
*
* INPUT PARAMETERS:
*  MAXEV  : MAXIMUM NUMBER OF UNKNOWNS.
*  MAXKN  : DIMENSION FOR ARRAYS KN, QFR AND BFR.
*  IMPX   : PRINT PARAMETER.
*  ISPLH  : =1 FOR COMPLETE HEXAGONS; >1 FOR TRIANGULAR MESH-SPLITTING
*           INTO 6*(ISPLH-1)**2 TRIANGLES.
*  LX     : NUMBER OF HEXAGONS.
*  SIDE   : SIDE OF AN HEXAGON.
*  NCODE  : NCODE(I) IS THE TYPE OF BOUNDARY CONDITION APPLIED ON SIDE I
*           I=1: HBC
*           NCODE(I)=1: VOID   NCODE(I)=2: REFL   NCODE(I)=5: SYME
*           NCODE(I)=7: ZERO
*  ZCODE  : ZCODE(I) IS THE ALBEDO CORRESPONDING TO BOUNDARY CONDITION
*           'VOID' ON SIDE I. ZCODE(I)=0.0 BY DEFAULT.
*           DIMENSION NCODE(6),ZCODE(6)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH HEXAGON. DIMENSION MAT(LX)
*  IHEX   : TYPE OF HEXAGONAL BOUNDARY CONDITION.
*
* OUTPUT PARAMETERS:
*  NELEM    : ORDER OF THE SYSTEM MATRICES (NUMBER OF ELEMENTS).
*  NUN    : NUMBER OF UNKNOWNS PER ENERGY GROUP.
*  VOL    : VOLUME OF EACH HEXAGON. DIMENSION VOL(LX)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(MAXKN)
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(MAXKN)
*  BFR    : ELEMENT-ORDERED SURFACE FRACTIONS. DIMENSION BFR(MAXKN)
*  MUW    : COMPRESSED STORAGE MODE INDICES. DIMENSION MUW(NELEM)
*  IDL    : POSITION OF THE AVERAGE FLUX COMPONENT ASSOCIATED WITH
*           EACH HEXAGON. DIMENSION IDL(LX)
*
*------------------------------------ AUTHOR : A. HEBERT ; 29/06/94 ----
*
      DIMENSION KN(*),QFR(*),BFR(*),VOL(*),MAT(*),IDL(*),NCODE(4),
     1 ZCODE(4),MUW(*)
      COMMON BASE(1)
      ALB(X)=0.5*(1.0-X)/(1.0+X)
*
      IF(IMPX.GT.0) WRITE(6,500)
      CALL SETARA(BASE,MAXEV,IIGAR)
      CALL SETARA(BASE,MAXKN,IKN2)
      CALL SETARA(BASE,MAXKN,IQFR2)
      CALL BIVSBH (MAXEV,MAXKN,IMPX,ISPLH,LX,SIDE,NELEM,IHEX,NCODE,MAT,
     1 VOL,KN,QFR,BASE(IIGAR),BASE(IKN2),BASE(IQFR2))
      CALL RLSARA(BASE(IQFR2))
      CALL RLSARA(BASE(IKN2))
      CALL RLSARA(BASE(IIGAR))
*
* PRODUCE STANDARD MESH CENTERED FINITE DIFFERENCE NUMBERING.
      IF(ISPLH.EQ.1) THEN
         NSURF=6
      ELSE
         NSURF=3
      ENDIF
      SURFTOT=0.0
      NUM1=0
      DO 200 KX=1,NELEM
      DO 190 IC=1,NSURF
      N1=ABS(KN(NUM1+IC))
      IF(N1.GT.NELEM) THEN
         IF(NCODE(1).EQ.1) THEN
            N1=-1
         ELSE IF(NCODE(1).EQ.2) THEN
            N1=-2
         ELSE IF(NCODE(1).EQ.7) THEN
            N1=-3
         ENDIF
      ELSE IF(N1.EQ.KX) THEN
         N1=-2
      ENDIF
      KN(NUM1+IC)=N1
*
* PROCESS BOUNDARY CONDITIONS.
      IF(NSURF.EQ.6) THEN
         BFR(NUM1+IC)=QFR(NUM1+IC)*QFR(NUM1+7)/(1.5*SQRT(3.0)*SIDE)
         IF(NCODE(1).EQ.1) THEN
            QFR(NUM1+IC)=ALB(ZCODE(1))*QFR(NUM1+IC)
         ELSE
            QFR(NUM1+IC)=0.0
         ENDIF
      ELSE
         AA=SIDE/REAL(ISPLH-1)
         BFR(NUM1+IC)=QFR(NUM1+IC)*QFR(NUM1+4)/(0.25*SQRT(3.0)*AA)
         IF(NCODE(1).EQ.1) THEN
            QFR(NUM1+IC)=ALB(ZCODE(1))*QFR(NUM1+IC)
         ELSE
            QFR(NUM1+IC)=0.0
         ENDIF
      ENDIF
      SURFTOT=SURFTOT+BFR(NUM1+IC)
  190 CONTINUE
  200 NUM1=NUM1+NSURF+1
*
* COMPUTE THE SURFACE FRACTIONS.
      IF(SURFTOT.GT.0.0) THEN
         DO 210 I=1,NUM1
  210    BFR(I)=BFR(I)/SURFTOT
      ENDIF
*
      IF((IMPX.GT.1).AND.(NSURF.EQ.6)) THEN
         WRITE(6,510)
         NUM1=0
         DO 220 I=1,NELEM
         WRITE(6,520) I,KN(NUM1+7),(KN(NUM1+J),J=1,6),(QFR(NUM1+J),
     1   J=1,7)
  220    NUM1=NUM1+7
         NUM1=0
         WRITE (6,580)
         DO 225 I=1,NELEM
         IF(MAT(I).LE.0) GO TO 225
         WRITE (6,590) I,(BFR(NUM1+J),J=1,6)
         NUM1=NUM1+7
  225    CONTINUE
      ELSE IF((IMPX.GT.1).AND.(NSURF.EQ.3)) THEN
         WRITE(6,530)
         NUM1=0
         DO 230 I=1,NELEM
         WRITE(6,540) I,KN(NUM1+4),(KN(NUM1+J),J=1,3),(QFR(NUM1+J),
     1   J=1,4),(BFR(NUM1+J),J=1,3)
  230    NUM1=NUM1+4
      ENDIF
      IF(IMPX.GT.0) WRITE(6,570) NELEM
*
* COMPUTE THE SYSTEM MATRIX BANDWIDTH.
      DO 240 I=1,NELEM
  240 MUW(I)=1
      NUM1=0
      DO 260 INW1=1,NELEM
      DO 250 I=1,NSURF
      IF(KN(NUM1+I).GT.0) THEN
         INW2=KN(NUM1+I)
         IF(INW2.LT.INW1) THEN
            MUW(INW1)=MAX(MUW(INW1),INW1-INW2+1)
         ENDIF
      ENDIF
  250 CONTINUE
      NUM1=NUM1+NSURF+1
  260 CONTINUE
      IIMAX=0
      DO 270 I=1,NELEM
      IIMAX=IIMAX+MUW(I)
  270 MUW(I)=IIMAX
      IF(IMPX.GT.6) WRITE(6,550) 'MUW :',(MUW(I),I=1,NELEM)
      IF(IMPX.GT.2) WRITE(6,560) IIMAX
*
* APPEND THE AVERAGED FLUXES AT THE END OF UNKNOWN VECTOR.
      NUN=0
      IF(ISPLH.GT.1) NUN=NELEM
      DO 280 I=1,LX
      IF(MAT(I).EQ.0) THEN
         IDL(I)=0
      ELSE
         NUN=NUN+1
         IDL(I)=NUN
      ENDIF
  280 CONTINUE
      RETURN
*
  500 FORMAT(//52H BIVDFH: NUMBERING FOR A MESH CENTERED FINITE DIFFER,
     1 42HENCE DISCRETIZATION IN HEXAGONAL GEOMETRY.)
  510 FORMAT(/31H BIVDFH: NUMBERING OF UNKNOWNS./1X,30(1H-)/9X,
     1 7HHEXAGON,3X,9HNEIGHBOUR,28X,23HVOID BOUNDARY CONDITION,15X,
     2 6HVOLUME)
  520 FORMAT (1X,2I6,2X,6I6,2X,6F6.2,5X,1P,E12.6)
  530 FORMAT(/31H BIVDFH: NUMBERING OF UNKNOWNS./1X,30(1H-)/9X,
     1 7HHEXAGON,3X,8HUNKNOWNS,11X,23HVOID BOUNDARY CONDITION,12X,
     2 6HVOLUME,13X,16HSURFACE FRACTION)
  540 FORMAT (1X,2I6,2X,3I6,2X,1P,3E11.2,5X,E12.6,5X,3E10.2)
  550 FORMAT(/1X,A5/(1X,20I6))
  560 FORMAT(/52H NUMBER OF TERMS IN THE COMPRESSED SYSTEM MATRICES =,
     > I6)
  570 FORMAT(/39H BIVDFH: NUMBER OF UNKNOWNS PER GROUP =,I6/)
  580 FORMAT (//17H SURFACE FRACTION//8H HEXAGON,5X,3HBFR)
  590 FORMAT (3X,I4,4X,1P,6E10.2)
      END
