*DECK BIVDM2
      SUBROUTINE BIVDM2(MAXMIX,ITY,NEL,L4,CYLIND,IELEM,ICOL,MAT,VOL,
     1 SGD,XX,YY,DD,KN,QFR,MU,LC,R,V,SYS)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF SYSTEM MATRICES FOR A MIXED-DUAL FORMULATION OF THE
* FINITE-ELEMENT DISCRETIZATION IN A 2-D GEOMETRY.
*
* INPUT PARAMETERS:
*  MAXMIX : NUMBER OF MATERIAL MIXTURES.
*  ITY    : =0: LEAKAGE-REMOVAL MATRIX ASSEMBLY; =1: X-SECTION MATRIX
*           ASSEMBLY.
*  NEL    : NUMBER OF FINITE ELEMENTS.
*  L4     : NUMBER OF UNKNOWNS PER ENERGY GROUP.
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
*  IELEM  : DEGREE OF THE LAGRANGIAN FINITE ELEMENTS. =1 (LINEAR);
*           =2 (PARABOLIC); =3 (CUBIC); =4 (QUARTIC).
*  ICOL   : TYPE OF QUADRATURE. =1 (ANALYTICAL INTEGRATION);
*           =2 (GAUSS-LOBATTO); =3 (GAUSS-LEGENDRE).
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  SGD    : CROSS SECTIONS. DIMENSION SGD(MAXMIX,3)
*  XX,YY  : X- AND Y-DIRECTED MESH SPACINGS.
*  DD     : USED WITH CYLINDRICAL GEOMETRY.
*           DIMENSION XX(NEL),YY(NEL),DD(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(5*NEL)
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(4*NEL)
*  MU     : COMPRESSED STORAGE MODE INDICES. DIMENSION MU(L4)
*  LC     : ORDER OF THE UNIT MATRICES.
*  R,V    : UNIT MATRICES. DIMENSION R(LC,LC),V(LC,LC-1)
*
* OUTPUT PARAMETERS:
*  SYS    : SYSTEM MATRIX. DIMENSION SYS(MU(L4))
*
*------------------------------------- AUTHOR: A. HEBERT ; 31/07/92 ----
*
      LOGICAL CYLIND
      DIMENSION SGD(MAXMIX,*),MAT(*),VOL(*),XX(*),YY(*),DD(*),KN(*),
     1 QFR(*),MU(*),R(LC,*),V(LC,*),SYS(*),QQ(5,5)
*
      IF((CYLIND).AND.((IELEM.GT.1).OR.(ICOL.NE.2)))
     1 CALL XABORT('BIVDM2: TYPE OF DISCRETIZATION NOT IMPLEMENTED.')
      IF(ITY.EQ.0) THEN
*        LEAKAGE-REMOVAL SYSTEM MATRIX ASSEMBLY.
         DO 10 I0=1,IELEM
         DO 10 J0=1,IELEM
         QQ(I0,J0)=0.0
         DO 10 K0=2,IELEM
   10    QQ(I0,J0)=QQ(I0,J0)+V(K0,I0)*V(K0,J0)/R(K0,K0)
*
         NUM1=0
         NUM2=0
         DO 80 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 80
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 70
         DX=XX(K)
         DY=YY(K)
         IF(CYLIND) THEN
            DIN=1.0-0.5*DX/DD(K)
            DOT=1.0+0.5*DX/DD(K)
         ELSE
            DIN=1.0
            DOT=1.0
         ENDIF
*
         DO 60 I0=1,IELEM
         INX1=ABS(KN(NUM1+2))+I0-1
         INX2=ABS(KN(NUM1+3))+I0-1
         INY1=ABS(KN(NUM1+4))+I0-1
         INY2=ABS(KN(NUM1+5))+I0-1
         DO 50 J0=1,IELEM
         JND1=KN(NUM1+1)+(I0-1)*IELEM+J0-1
         KEY=MU(JND1)
         SYS(KEY)=SYS(KEY)+VOL0*SGD(L,3)
         DO 20 K0=1,J0
         IF(QQ(J0,K0).EQ.0.0) GO TO 20
         KND1=KN(NUM1+1)+(I0-1)*IELEM+K0-1
         KEY=MU(JND1)-JND1+KND1
         SYS(KEY)=SYS(KEY)+VOL0*QQ(J0,K0)*SGD(L,1)/(DX*DX)
   20    CONTINUE
         IF(KN(NUM1+2).NE.0) THEN
            IF(JND1.GT.INX1) KEY=MU(JND1)-JND1+INX1
            IF(JND1.LT.INX1) KEY=MU(INX1)-INX1+JND1
            SG=REAL(SIGN(1,KN(NUM1+2)))
            SYS(KEY)=SYS(KEY)+SG*(VOL0/DX)*DIN*V(1,J0)
         ENDIF
         IF(KN(NUM1+3).NE.0) THEN
            IF(INX2.GT.JND1) KEY=MU(INX2)-INX2+JND1
            IF(INX2.LT.JND1) KEY=MU(JND1)-JND1+INX2
            SG=REAL(SIGN(1,KN(NUM1+3)))
            SYS(KEY)=SYS(KEY)+SG*(VOL0/DX)*DOT*V(IELEM+1,J0)
         ENDIF
         JND1=KN(NUM1+1)+(J0-1)*IELEM+I0-1
         DO 30 K0=1,J0
         IF(QQ(J0,K0).EQ.0.0) GO TO 30
         KND1=KN(NUM1+1)+(K0-1)*IELEM+I0-1
         KEY=MU(JND1)-JND1+KND1
         SYS(KEY)=SYS(KEY)+VOL0*QQ(J0,K0)*SGD(L,2)/(DY*DY)
   30    CONTINUE
         IF(KN(NUM1+4).NE.0) THEN
            IF(JND1.GT.INY1) KEY=MU(JND1)-JND1+INY1
            IF(JND1.LT.INY1) KEY=MU(INY1)-INY1+JND1
            SG=REAL(SIGN(1,KN(NUM1+4)))
            SYS(KEY)=SYS(KEY)+SG*(VOL0/DY)*V(1,J0)
         ENDIF
         IF(KN(NUM1+5).NE.0) THEN
            IF(INY2.GT.JND1) KEY=MU(INY2)-INY2+JND1
            IF(INY2.LT.JND1) KEY=MU(JND1)-JND1+INY2
           SG=REAL(SIGN(1,KN(NUM1+5)))
            SYS(KEY)=SYS(KEY)+SG*(VOL0/DY)*V(IELEM+1,J0)
         ENDIF
   50    CONTINUE
         IF(KN(NUM1+2).NE.0) THEN
            KEY=MU(INX1)
            SYS(KEY)=SYS(KEY)-DIN*(VOL0*R(1,1)/SGD(L,1)+QFR(NUM2+1))
         ENDIF
         IF(KN(NUM1+3).NE.0) THEN
            KEY=MU(INX2)
            SYS(KEY)=SYS(KEY)-DOT*(VOL0*R(IELEM+1,IELEM+1)/SGD(L,1)
     1      +QFR(NUM2+2))
         ENDIF
         IF(KN(NUM1+4).NE.0) THEN
            KEY=MU(INY1)
            SYS(KEY)=SYS(KEY)-VOL0*R(1,1)/SGD(L,2)-QFR(NUM2+3)
         ENDIF
         IF(KN(NUM1+5).NE.0) THEN
            KEY=MU(INY2)
            SYS(KEY)=SYS(KEY)-VOL0*R(IELEM+1,IELEM+1)/SGD(L,2)
     1      -QFR(NUM2+4)
         ENDIF
         IF(ICOL.NE.2) THEN
            IF((KN(NUM1+2).NE.0).AND.(KN(NUM1+3).NE.0)) THEN
               IF(INX2.GT.INX1) KEY=MU(INX2)-INX2+INX1
               IF(INX2.LE.INX1) KEY=MU(INX1)-INX1+INX2
               SG=REAL(SIGN(1,KN(NUM1+2))*SIGN(1,KN(NUM1+3)))
               IF(INX1.EQ.INX2) SG=2.0*SG
               SYS(KEY)=SYS(KEY)-SG*VOL0*R(IELEM+1,1)/SGD(L,1)
            ENDIF
            IF((KN(NUM1+4).NE.0).AND.(KN(NUM1+5).NE.0)) THEN
               IF(INY2.GT.INY1) KEY=MU(INY2)-INY2+INY1
               IF(INY2.LE.INY1) KEY=MU(INY1)-INY1+INY2
               SG=REAL(SIGN(1,KN(NUM1+4))*SIGN(1,KN(NUM1+5)))
               IF(INY1.EQ.INY2) SG=2.0*SG
               SYS(KEY)=SYS(KEY)-SG*VOL0*R(IELEM+1,1)/SGD(L,2)
            ENDIF
         ENDIF
   60    CONTINUE
   70    NUM1=NUM1+5
         NUM2=NUM2+4
   80    CONTINUE
      ELSE
*        CROSS SECTION SYSTEM MATRIX ASSEMBLY.
         NUM1=0
         DO 110 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 110
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 100
         DO 90 I0=1,IELEM
         DO 90 J0=1,IELEM
         JND1=KN(NUM1+1)+(I0-1)*IELEM+J0-1
         KEY=MU(JND1)
   90    SYS(KEY)=SYS(KEY)+VOL0*SGD(L,1)
  100    NUM1=NUM1+5
  110    CONTINUE
      ENDIF
      RETURN
      END
