*DECK BIVFLX
      SUBROUTINE BIVFLX(IPTRK,NEL,NUN,EVECT,MAT,VOL,IDL)
*
*-----------------------------------------------------------------------
*
* DRIVER FOR THE CALCULATION OF THE AVERAGED FLUX IN BIVAC.
*
* INPUT PARAMETERS:
*  IPTRK  : L_BIVAC POINTER TO THE BIVAC TRACKING INFORMATION.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : TOTAL NUMBER OF UNKNOWN PER ENERGY GROUP.
*  EVECT  : VARIATIONAL COEFFICIENTS OF THE FLUX. CONTAINED IN POSITIONS
*           EVECT(1) TO EVECT(LL4). DIMENSION EVECT(NUN)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION MAT(NEL)
*  IDL    : POSITION OF THE AVERAGE FLUX COMPONENT ASSOCIATED WITH
*           EACH VOLUME. DIMENSION IDL(NEL)
*
* OUTPUT PARAMETER:
*  EVECT  : AVERAGED FLUXES. CONTAINED IN POSITIONS EVECT(IDL(I))
*           DIMENSION EVECT(NUN)
*
*------------------------------------- AUTHOR: A. HEBERT ; 08/07/86 ----
*
      LOGICAL CYLIND
      DIMENSION EVECT(*),MAT(*),VOL(*),IDL(*),ITP(8),IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
* RECOVER BIVAC SPECIFIC TRACKING INFORMATION.
      CALL LCMGET(IPTRK,'BPARAM',ITP)
      ITYPE=ITP(1)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      IELEM=ITP(3)
      ISPLH=ITP(5)
      LL4=ITP(6)
      LX=ITP(7)
      LY=ITP(8)
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMLEN(IPTRK,'BFR',MAXQF,ITYLCM)
      CALL SETARA(BASE,MAXQF,IBFR)
      CALL LCMGET(IPTRK,'BFR',BASE(IBFR))
*
      IF((IELEM.LT.0).AND.(ITYPE.NE.8)) THEN
*        LAGRANGIAN FINITE ELEMENTS.
         CALL SETARA(BASE,LX*LY,IXX)
         CALL SETARA(BASE,LX*LY,IDD)
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'DD',BASE(IDD))
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC,IT)
         CALL SETARA(BASE,LC,ITS)
         CALL LCMGET(IPTRK,'T',BASE(IT))
         CALL LCMGET(IPTRK,'TS',BASE(ITS))
         CALL LCMSIX(IPTRK,' ',2)
         CALL BIVIN2(NEL,NUN,LL4,CYLIND,EVECT,BASE(IXX),BASE(IDD),MAT,
     1   VOL,IDL,IBASE(IKN),BASE(IBFR),LC,BASE(IT),BASE(ITS))
         CALL RLSARA(BASE(ITS))
         CALL RLSARA(BASE(IT))
         CALL RLSARA(BASE(IDD))
         CALL RLSARA(BASE(IXX))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.NE.8)) THEN
*        MIXED-DUAL FINITE ELEMENTS.
         CALL BIVIN1 (NEL,NUN,LL4,IELEM,EVECT,MAT,VOL,IBASE(IKN),
     1   BASE(IBFR))
      ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8)) THEN
*        MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL SETARA(BASE,36,IRH)
         CALL SETARA(BASE,9,IRT)
         CALL LCMGET(IPTRK,'RH',BASE(IRH))
         CALL LCMGET(IPTRK,'RT',BASE(IRT))
         CALL LCMSIX(IPTRK,' ',2)
         IF(ISPLH.EQ.1) THEN
            NELEM=MAXKN/7
         ELSE
            NELEM=MAXKN/4
         ENDIF
         CALL SETARA(BASE,MAXKN,IQFR)
         CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
         CALL BIVIH2(ISPLH,NEL,NUN,NELEM,EVECT,MAT,VOL,IDL,IBASE(IKN),
     1   BASE(IQFR),BASE(IBFR),BASE(IRH),BASE(IRT))
         CALL RLSARA(BASE(IQFR))
         CALL RLSARA(BASE(IRT))
         CALL RLSARA(BASE(IRH))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8)) THEN
*        MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
         CALL SETARA(BASE,MAXKN,IQFR)
         CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
         CALL BIVIH1(ISPLH,NEL,NUN,LL4,EVECT,VOL,IDL,IBASE(IKN),
     1   BASE(IQFR),BASE(IBFR))
         CALL RLSARA(BASE(IQFR))
      ENDIF
*
* RELEASE BIVAC SPECIFIC TRACKING INFORMATION.
      CALL RLSARA(BASE(IBFR))
      CALL RLSARA(IBASE(IKN))
      RETURN
      END
