*DECK BIVIH1
      SUBROUTINE BIVIH1 (ISPLH,NEL,NUN,LL4,EVECT,VOL,IDL,KN,QFR,BFR)
*
*-----------------------------------------------------------------------
*
* CALCULATION OF THE AVERAGED FLUX WITH A MESH CENTERED FINITE
* DIFFERENCE METHOD IN HEXAGONAL GEOMETRY.
*
* INPUT PARAMETERS:
*  ISPLH  : =1 FOR COMPLETE HEXAGONS; >1 FOR TRIANGULAR MESH-SPLITTING.
*  NEL    : NUMBER OF HEXAGONS.
*  NUN    : DIMENSION OF VECTOR EVECT.
*  LL4    : ORDER OF THE SYSTEM MATRICES.
*  EVECT  : VARIATIONAL COEFFICIENTS OF THE FLUX. CONTAINED IN POSITIONS
*           EVECT(1) TO EVECT(LL4). DIMENSION EVECT(NUN) WHERE NUN IS
*           THE TOTAL NUMBER OF UNKNOWN PER ENERGY GROUP.
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION MAT(NEL) WHERE NEL IS
*           THE TOTAL NUMBER OF ELEMENTS.
*  IDL    : POSITION OF THE AVERAGE FLUX COMPONENT ASSOCIATED WITH
*           EACH VOLUME. DIMENSION IDL(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(MAXKN) WHERE
*           MAXKN=(NSURF+1)*LL4
*  QFR    : ELEMENT-ORDERED INFORMATION. DIMENSION QFR(MAXKN)
*  BFR    : ELEMENT-ORDERED SURFACE FRACTIONS. DIMENSION BFR(MAXKN)
*
* OUTPUT PARAMETER:
*  EVECT  : VOLUME-AVERAGED FLUXES IN POSITIONS EVECT(IDL(I))
*           SURFACE-AVERAGED FLUX IN POSITION EVECT(NUN)
*           DIMENSION EVECT(NUN)
*
*------------------------------------- AUTHOR: A. HEBERT ; 01/07/94 ----
*
      DIMENSION EVECT(*),VOL(*),IDL(*),KN(*),QFR(*),BFR(*)
*
      IF(ISPLH.GT.1) THEN
         NSURF=3
         DO 10 K=1,NEL
         IF(IDL(K).NE.0) EVECT(IDL(K))=0.0
   10    CONTINUE
         NUM1=0
         DO 20 IND1=1,LL4
         K=KN(NUM1+NSURF+1)
         IF(VOL(K).EQ.0.0) GO TO 20
         EVECT(IDL(K))=EVECT(IDL(K))+QFR(NUM1+NSURF+1)*EVECT(IND1)/
     1   VOL(K)
   20    NUM1=NUM1+NSURF+1
      ELSE
         NSURF=6
      ENDIF
*
      EVECT(NUN)=0.0
      NUM1=0
      DO 40 IND1=1,LL4
      K=KN(NUM1+NSURF+1)
      IF(VOL(K).EQ.0.0) GO TO 40
      DO 30 IC=1,NSURF
      IF(BFR(NUM1+IC).GT.0.0) THEN
         EVECT(NUN)=EVECT(NUN)+EVECT(IND1)*BFR(NUM1+IC)
      ENDIF
   30 CONTINUE
   40 NUM1=NUM1+NSURF+1
      RETURN
      END
