*DECK BIVIH2
      SUBROUTINE BIVIH2 (ISPLH,NEL,NUN,NELEM,EVECT,MAT,VOL,IDL,KN,QFR,
     1 BFR,RH,RT)
*
*-----------------------------------------------------------------------
*
* CALCULATION OF THE AVERAGED FLUX WITH A LINEAR LAGRANGIAN FINITE
* ELEMENT OR MESH CORNER FINITE DIFFERENCE METHOD IN HEXAGONAL GEOMETRY.
*
* INPUT PARAMETERS:
*  ISPLH  : =1 FOR COMPLETE HEXAGONS; >1 FOR TRIANGULAR MESH-SPLITTING.
*  NEL    : NUMBER OF HEXAGONS.
*  NUN    : DIMENSION OF VECTOR EVECT.
*  NELEM  : NUMBER OF FINITE ELEMENTS (HEXAGONS OR TRIANGLES) EXCLUDING
*           THE VIRTUAL ELEMENTS.
*  EVECT  : VARIATIONAL COEFFICIENTS OF THE FLUX. CONTAINED IN POSITIONS
*           EVECT(1) TO EVECT(LL4) WHERE LL4 IS THE ORDER OF THE SYSTEM
*           MATRICES. DIMENSION EVECT(NUN)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH HEXAGON. DIMENSION MAT(NEL)
*  VOL    : VOLUME OF EACH HEXAGON. DIMENSION MAT(NEL)
*  IDL    : POSITION OF THE AVERAGE FLUX COMPONENT ASSOCIATED WITH
*           EACH HEXAGON. DIMENSION IDL(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN((LC+1)*NELEM)
*           WHERE LC=6 (HEXAGONS) OR 3 (TRIANGLES).
*  QFR    : ELEMENT-ORDERED VOID BOUNDARY CONDITIONS.
*  BFR    : ELEMENT-ORDERED SURFACE FRACTIONS.
*           DIMENSION QFR((LC+1)*NELEM),BFR((LC+1)*NELEM)
*  RH,RT  : UNIT MATRICES. DIMENSION RH(6,6),RT(3,3)
*
* OUTPUT PARAMETER:
*  EVECT  : VOLUME-AVERAGED FLUXES IN POSITIONS EVECT(IDL(I))
*           SURFACE-AVERAGED FLUX IN POSITION EVECT(NUN)
*           DIMENSION EVECT(NUN)
*
*------------------------------------- AUTHOR: A. HEBERT ; 13/07/94 ----
*
      DIMENSION EVECT(*),MAT(*),VOL(*),IDL(*),KN(*),QFR(*),BFR(*),
     1 RH(6,*),RT(3,*),T(6),ISR(6,2),ISRH(6,2),ISRT(3,2)
      DATA ISRH/2,1,4,5,6,3,1,4,5,6,3,2/
      DATA ISRT/1,2,3,2,3,1/
*
* COMPUTE THE LINEAR PRODUCT VECTOR T.
      IF(ISPLH.EQ.1) THEN
*        HEXAGONAL BASIS.
         LC=6
         DO 20 I=1,6
         DO 10 J=1,2
   10    ISR(I,J)=ISRH(I,J)
         T(I)=0.0
         DO 20 J=1,6
   20    T(I)=T(I)+RH(I,J)
         CONST=1.5*SQRT(3.0)
         CONSB=2.0*SQRT(3.0)/3.0
      ELSE
*        TRIANGULAR BASIS.
         LC=3
         DO 40 I=1,3
         DO 30 J=1,2
   30    ISR(I,J)=ISRT(I,J)
         T(I)=0.0
         DO 40 J=1,3
   40    T(I)=T(I)+RT(I,J)
         CONST=0.25*SQRT(3.0)
         CONSB=2.0*SQRT(3.0)
      ENDIF
*
      DO 50 KHEX=1,NEL
      IF(IDL(KHEX).NE.0) EVECT(IDL(KHEX))=0.0
   50 CONTINUE
      EVECT(NUN)=0.0
      NUM1=0
      DO 90 K=1,NELEM
      KHEX=KN(NUM1+LC+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 90
      DO 60 I=1,LC
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 60
      SS=T(I)*QFR(NUM1+LC+1)/(CONST*VOL(KHEX))
      EVECT(IDL(KHEX))=EVECT(IDL(KHEX))+SS*EVECT(IND1)
   60 CONTINUE
*
* SURFACE-AVERAGED FLUX.
      DO 80 IC=1,LC
      BFR1=BFR(NUM1+IC)*CONSB
      IF(BFR1.EQ.0.0) GO TO 80
      DO 70 I1=1,2
      IND1=KN(NUM1+ISR(IC,I1))
      IF(IND1.GT.0) EVECT(NUN)=EVECT(NUN)+T(I1)*EVECT(IND1)*BFR1
   70 CONTINUE
   80 CONTINUE
*
   90 NUM1=NUM1+LC+1
      RETURN
      END
