*DECK BIVIN1
      SUBROUTINE BIVIN1 (NEL,NUN,LL4,IELEM,EVECT,MAT,VOL,KN,BFR)
*
*-----------------------------------------------------------------------
*
* CALCULATION OF THE SURFACE-AVERAGED FLUX WITH A MIXED-DUAL FINITE
* ELEMENT METHOD.
*
* INPUT PARAMETERS:
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : DIMENSION OF VECTOR EVECT.
*  LL4    : ORDER OF THE SYSTEM MATRICES.
*  IELEM  : ORDER OF THE FLUX EXPANSION.
*  EVECT  : VARIATIONAL COEFFICIENTS OF THE FLUX. CONTAINED IN POSITIONS
*           EVECT(1) TO EVECT(LL4). DIMENSION EVECT(NUN)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(5*NEL)
*  BFR    : ELEMENT-ORDERED SURFACE FRACTIONS. DIMENSION BFR(4*NEL)
*
* OUTPUT PARAMETER:
*  EVECT  : SURFACE-AVERAGED FLUX IN POSITION EVECT(NUN)
*           DIMENSION EVECT(NUN)
*
*------------------------------------- AUTHOR: A. HEBERT ; 15/09/97 ----
*
      DIMENSION EVECT(*),MAT(*),VOL(*),KN(*),BFR(*)
*
      EVECT(NUN)=0.0
      NUM1=0
      NUM2=0
      DO 60 K=1,NEL
      IF(MAT(K).EQ.0) GO TO 60
      IF(VOL(K).EQ.0.0) GO TO 50
      IF(BFR(NUM2+1).GT.0.0) THEN
         SG=1.0
         DO 10 I0=1,IELEM
         IND1=KN(NUM1+1)+I0-1
         EVECT(NUN)=EVECT(NUN)+SG*SQRT(REAL(2*I0-1))*EVECT(IND1)*
     1   BFR(NUM2+1)
   10    SG=-SG
      ENDIF
      IF(BFR(NUM2+2).GT.0.0)  THEN
         DO 20 I0=1,IELEM
         IND1=KN(NUM1+1)+I0-1
   20    EVECT(NUN)=EVECT(NUN)+SQRT(REAL(2*I0-1))*EVECT(IND1)*
     1   BFR(NUM2+2)
      ENDIF
      IF(BFR(NUM2+3).GT.0.0)  THEN
         SG=1.0
         DO 30 I0=1,IELEM
         IND1=KN(NUM1+1)+(I0-1)*IELEM
         EVECT(NUN)=EVECT(NUN)+SG*SQRT(REAL(2*I0-1))*EVECT(IND1)*
     1   BFR(NUM2+3)
   30    SG=-SG
      ENDIF
      IF(BFR(NUM2+4).GT.0.0)  THEN
         DO 40 I0=1,IELEM
         IND1=KN(NUM1+1)+(I0-1)*IELEM
   40    EVECT(NUN)=EVECT(NUN)+SQRT(REAL(2*I0-1))*EVECT(IND1)*
     1   BFR(NUM2+4)
      ENDIF
   50 NUM1=NUM1+5
      NUM2=NUM2+4
   60 CONTINUE
      RETURN
      END
