*DECK BIVIN2
      SUBROUTINE BIVIN2 (NEL,NUN,LL4,CYLIND,EVECT,XX,DD,MAT,VOL,IDL,KN,
     1 BFR,LC,T,TS)
*
*-----------------------------------------------------------------------
*
* CALCULATION OF THE VOLUME- AND SURFACE-AVERAGED FLUX WITH A PRIMAL
* FINITE ELEMENT METHOD.
*
* INPUT PARAMETERS:
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : DIMENSION OF VECTOR EVECT.
*  LL4    : ORDER OF THE SYSTEM MATRICES.
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
*  EVECT  : VARIATIONAL COEFFICIENTS OF THE FLUX. CONTAINED IN POSITIONS
*           EVECT(1) TO EVECT(LL4). DIMENSION EVECT(NUN)
*  XX     : X-SIDE OF EACH ELEMENT. DIMENSION XX(NEL)
*  DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(NEL)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  IDL    : POSITION OF THE AVERAGE FLUX COMPONENT ASSOCIATED WITH
*           EACH VOLUME. DIMENSION IDL(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(ICOEF*NEL)
*           WHERE ICOEF=IELEM**2
*  BFR    : ELEMENT-ORDERED SURFACE FRACTIONS. DIMENSION BFR(4*NEL)
*  LC     : ORDER OF THE FINITE ELEMENT BASIS.
*  T,TS   : LINEAR PRODUCT VECTORS. DIMENSION T(LC),TS(LC)
*
* OUTPUT PARAMETER:
*  EVECT  : VOLUME-AVERAGED FLUXES IN POSITIONS EVECT(IDL(I))
*           SURFACE-AVERAGED FLUX IN POSITION EVECT(NUN)
*           DIMENSION EVECT(NUN)
*
*------------------------------------- AUTHOR: A. HEBERT ; 15/01/86 ----
*
      LOGICAL CYLIND
      DIMENSION EVECT(*),XX(*),DD(*),MAT(*),VOL(*),IDL(*),KN(*),BFR(*),
     1 T(*),TS(*),IJ1(25),IJ2(25),ISR(4,5)
*
* COMPUTE VECTORS IJ1, IJ2 AND MATRIX ISR.
      LL=LC*LC
      DO 10 I=1,LL
      IJ1(I)=1+MOD(I-1,LC)
   10 IJ2(I)=1+(I-IJ1(I))/LC
      DO 15 I=1,LC
      ISR(1,I)=(I-1)*LC+1
      ISR(2,I)=I*LC
      ISR(3,I)=I
   15 ISR(4,I)=LL-LC+I
*
      EVECT(NUN)=0.0
      NUM1=0
      NUM2=0
      DO 60 K=1,NEL
      IF(MAT(K).EQ.0) GO TO 60
      EVECT(IDL(K))=0.0
      IF(VOL(K).EQ.0.0) GO TO 50
*
* VOLUME-AVERAGED FLUXES.
      DO 20 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 20
      I1=IJ1(I)
      I2=IJ2(I)
      IF(CYLIND) THEN
         SS=(T(I1)+TS(I1)*XX(K)/DD(K))*T(I2)
      ELSE
         SS=T(I1)*T(I2)
      ENDIF
      EVECT(IDL(K))=EVECT(IDL(K))+SS*EVECT(IND1)
   20 CONTINUE
*
* SURFACE-AVERAGED FLUX.
      DO 40 IC=1,4
      BFR1=BFR(NUM2+IC)
      IF(BFR1.EQ.0.0) GO TO 40
      DO 30 I1=1,LC
      IND1=KN(NUM1+ISR(IC,I1))
      IF(IND1.GT.0) EVECT(NUN)=EVECT(NUN)+T(I1)*EVECT(IND1)*BFR1
   30 CONTINUE
   40 CONTINUE
*
   50 NUM1=NUM1+LL
      NUM2=NUM2+4
   60 CONTINUE
      RETURN
      END
