*DECK BIVMX2
      SUBROUTINE BIVMX2 (MAXMIX,ITY,ISPLH,LL4,MAT,VOL,SGD,SIDE,KN,QFR,
     1 MU,SYS)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF SYSTEM MATRICES FOR A MESH CENTERED FINITE DIFFERENCE
* DISCRETIZATION IN A 2-D HEXAGONAL GEOMETRY.
*
* INPUT PARAMETERS:
*  MAXMIX : NUMBER OF MATERIAL MIXTURES.
*  ITY    : =0: LEAKAGE-REMOVAL MATRIX ASSEMBLY; =1: X-SECTION MATRIX
*           ASSEMBLY.
*  ISPLH  : =1 FOR COMPLETE HEXAGONS. OTHERWISE, THE NUMBER OF TRIANGLES
*           PER HEXAGON IS 6*(ISPLH-1)**2
*  LL4    : ORDER OF THE SYSTEM MATRICES.
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  VOL    : VOLUMES OF THE HEXAGONS. DIMENSION VOL(LX)
*  SGD    : CROSS SECTIONS. DIMENSION SGD(MAXMIX,3)
*  SIDE   : SIDE OF AN HEXAGON.
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(MAXKN)
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(MAXKN)
*  MU     : COMPRESSED STORAGE MODE INDICES. DIMENSION MU(LL4)
*
* OUTPUT PARAMETERS:
*  SYS    : SYSTEM MATRIX. DIMENSION SYS(MU(LL4))
*
* NOTE: SYSTEM MATRIX SHOULD BE INITIALIZED BY THE CALLING PROGRAM.
*
*------------------------------------- AUTHOR: A. HEBERT ; 30/06/94 ----
*
      DOUBLE PRECISION A1,DHARM,VAR1
      DIMENSION MAT(*),VOL(*),SGD(MAXMIX,*),KN(*),QFR(*),MU(*),SYS(*)
      DHARM(X1,X2,DIF1,DIF2)=2.0D0*DIF1*DIF2/(X1*DIF2+X2*DIF1)
*
      IF(ISPLH.EQ.1) THEN
         DS=SQRT(3.0)*SIDE
         FACT=2.0/(3.0*DS)
         NSURF=6
      ELSE
         DS=SIDE/(SQRT(3.0)*REAL(ISPLH-1))
         FACT=4.0/(3.0*DS)
         NSURF=3
      ENDIF
      IF(ITY.EQ.0) THEN
*        LEAKAGE-REMOVAL SYSTEM MATRIX ASSEMBLY.
         NUM1=0
         DO 20 IND1=1,LL4
         KHEX=KN(NUM1+NSURF+1)
         IF(VOL(KHEX).EQ.0.0) GO TO 20
         L=MAT(KHEX)
         VOL0=QFR(NUM1+NSURF+1)
         SIDEB=FACT*VOL0
         VAR1=0.0D0
         KEY0=MU(IND1)-IND1
         DO 10 IX=1,NSURF
         IND2=KN(NUM1+IX)
         IF(IND2.GT.0) THEN
            LL=MAT(KN(IND2*(NSURF+1)))
            A1=DHARM(DS,DS,SGD(L,1),SGD(LL,1))*SIDEB
         ELSE IF(IND2.EQ.-1) THEN
            A1=DHARM(DS,DS,SGD(L,1),DS*QFR(NUM1+IX)/2.0)*SIDEB
         ELSE IF(IND2.EQ.-2) THEN
            A1=0.0D0
         ELSE IF(IND2.EQ.-3) THEN
            A1=2.0D0*DHARM(DS,DS,SGD(L,1),SGD(L,1))*SIDEB
         ENDIF
         VAR1=VAR1+A1
         IF(IND2.GT.0) THEN
            IF(IND2.LT.IND1) THEN
               KEY=KEY0+IND2
               SYS(KEY)=SYS(KEY)-A1
            ENDIF
         ENDIF
   10    CONTINUE
         KEY=KEY0+IND1
         SYS(KEY)=SYS(KEY)+VAR1+SGD(L,3)*VOL0
   20    NUM1=NUM1+NSURF+1
      ELSE
*        CROSS SECTION SYSTEM MATRIX ASSEMBLY.
         NUM1=0
         DO 30 IND1=1,LL4
         KHEX=KN(NUM1+NSURF+1)
         IF(VOL(KHEX).EQ.0.0) GO TO 30
         L=MAT(KHEX)
         KEY=MU(IND1)
         SYS(KEY)=SYS(KEY)+SGD(L,1)*QFR(NUM1+NSURF+1)
   30    NUM1=NUM1+NSURF+1
      ENDIF
      RETURN
      END
