*DECK BIVPM2
      SUBROUTINE BIVPM2(MAXMIX,ITY,NEL,L4,CYLIND,MAT,VOL,SGD,XX,YY,DD,
     1 KN,QFR,MU,LC,R,RS,Q,QS,SYS)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF SYSTEM MATRICES FOR A PRIMAL FORMULATION OF THE FINITE-
* ELEMENT DISCRETIZATION IN A 2-D GEOMETRY.
*
* INPUT PARAMETERS:
*  MAXMIX : NUMBER OF MATERIAL MIXTURES.
*  ITY    : =0: LEAKAGE-REMOVAL MATRIX ASSEMBLY; =1: X-SECTION MATRIX
*           ASSEMBLY.
*  NEL    : NUMBER OF FINITE ELEMENTS.
*  L4     : NUMBER OF UNKNOWNS PER ENERGY GROUP.
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  SGD    : CROSS SECTIONS. DIMENSION SGD(MAXMIX,3)
*  XX,YY  : X- AND Y-DIRECTED MESH SPACINGS.
*  DD     : USED WITH CYLINDRICAL GEOMETRY.
*           DIMENSION XX(NEL),YY(NEL),DD(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(LC*LC*NEL) WHERE
*           LC IS THE NUMBER IF POLYNOMIALS IN A COMPLETE 1D BASIS.
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(4*NEL)
*  MU     : COMPRESSED STORAGE MODE INDICES. DIMENSION MU(L4)
*  LC     : ORDER OF THE UNIT MATRICES.
*  R,RS,Q,QS : UNIT MATRICES. DIMENSION R(LC,LC),RS(LC,LC),Q(LC,LC),
*              QS(LC,LC)
*
* OUTPUT PARAMETER:
*  SYS    : SYSTEM MATRIX. DIMENSION SYS(MU(L4))
*
*------------------------------------- AUTHOR: A. HEBERT ; 31/07/92 ----
*
      LOGICAL CYLIND
      DIMENSION SGD(MAXMIX,*),XX(*),YY(*),DD(*),MAT(*),KN(*),QFR(*),
     1 VOL(*),MU(*),R(LC,*),RS(LC,*),Q(LC,*),QS(LC,*),SYS(*),
     2 Q2DP1(25,25),Q2DP2(25,25),R2DP(25,25),Q2DC1(25,25),Q2DC2(25,25),
     3 R2DC(25,25),IJ1(25),IJ2(25),ISR(4,5)
*
* COMPUTE VECTORS IJ1, IJ2 AND MATRIX ISR.
      LL=LC*LC
      DO 10 I=1,LL
      IJ1(I)=1+MOD(I-1,LC)
   10 IJ2(I)=1+(I-IJ1(I))/LC
      DO 20 I=1,LC
      ISR(1,I)=(I-1)*LC+1
      ISR(2,I)=I*LC
      ISR(3,I)=I
   20 ISR(4,I)=LL-LC+I
*
* COMPUTE THE 2D MASS AND STIFFNESS MATRICES AS TENSORIAL PRODUCT OF 1D
* MATRICES.
      DO 250 I=1,LL
      I1=IJ1(I)
      I2=IJ2(I)
      DO 250 J=1,LL
      J1=IJ1(J)
      J2=IJ2(J)
      Q2DP1(I,J)=Q(I1,J1)*R(I2,J2)
      Q2DP2(I,J)=R(I1,J1)*Q(I2,J2)
      R2DP(I,J)=R(I1,J1)*R(I2,J2)
      Q2DC1(I,J)=QS(I1,J1)*R(I2,J2)
      Q2DC2(I,J)=RS(I1,J1)*Q(I2,J2)
  250 R2DC(I,J)=RS(I1,J1)*R(I2,J2)
*
* ASSEMBLY OF SYSTEM MATRICES.
      IF(ITY.EQ.0) THEN
*        LEAKAGE-REMOVAL SYSTEM MATRIX ASSEMBLY.
         NUM1=0
         NUM2=0
         DO 360 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 360
         IF(VOL(K).EQ.0.0) GO TO 350
         DX=XX(K)
         DY=YY(K)
         DO 320 I=1,LL
         IND1=KN(NUM1+I)
         IF(IND1.EQ.0) GO TO 320
         KEY1=MU(IND1)-IND1
         DO 310 J=1,LL
         IND2=KN(NUM1+J)
         IF((IND2.EQ.0).OR.(IND2.GT.IND1)) GO TO 310
         IF(CYLIND) THEN
            QQX=(Q2DP1(I,J)+Q2DC1(I,J)*DX/DD(K))/(DX*DX)
            QQY=(Q2DP2(I,J)+Q2DC2(I,J)*DX/DD(K))/(DY*DY)
            RR=R2DP(I,J)+R2DC(I,J)*DX/DD(K)
         ELSE
            QQX=Q2DP1(I,J)/(DX*DX)
            QQY=Q2DP2(I,J)/(DY*DY)
            RR=R2DP(I,J)
         ENDIF
         IF((QQX.EQ.0.0).AND.(QQY.EQ.0.0).AND.(RR.EQ.0.0)) GO TO 310
         KEY=KEY1+IND2
         SYS(KEY)=SYS(KEY)+(QQX*SGD(L,1)+QQY*SGD(L,2)+RR*SGD(L,3))
     1   *VOL(K)
  310    CONTINUE
  320    CONTINUE
         DO 340 IC=1,4
         QFR1=QFR(NUM2+IC)
         IF(QFR1.EQ.0.0) GO TO 340
         DO 335 I1=1,LC
         IND1=KN(NUM1+ISR(IC,I1))
         IF(IND1.EQ.0) GO TO 335
         KEY1=MU(IND1)-IND1
         DO 330 J1=1,LC
         IND2=KN(NUM1+ISR(IC,J1))
         IF((IND2.EQ.0).OR.(IND2.GT.IND1)) GO TO 330
         IF(CYLIND) THEN
            IF(IC.EQ.1) THEN
               CRZ=-0.5*R(I1,J1)
            ELSE IF(IC.EQ.2) THEN
               CRZ=0.5*R(I1,J1)
            ELSE IF(IC.EQ.3) THEN
               CRZ=RS(I1,J1)
            ELSE IF(IC.EQ.4) THEN
               CRZ=RS(I1,J1)
            ENDIF
            RR=R(I1,J1)+DX*CRZ/DD(K)
         ELSE
            RR=R(I1,J1)
         ENDIF
         IF(RR.EQ.0.0) GO TO 330
         KEY=KEY1+IND2
         SYS(KEY)=SYS(KEY)+RR*QFR1
  330    CONTINUE
  335    CONTINUE
  340    CONTINUE
  350    NUM1=NUM1+LL
         NUM2=NUM2+4
  360    CONTINUE
      ELSE
*        CROSS SECTION SYSTEM MATRIX ASSEMBLY.
         NUM1=0
         DO 400 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 400
         IF(VOL(K).EQ.0.0) GO TO 390
         DX=XX(K)
         DY=YY(K)
         DO 380 I=1,LL
         IND1=KN(NUM1+I)
         IF(IND1.EQ.0) GO TO 380
         KEY1=MU(IND1)-IND1
         DO 370 J=1,LL
         IND2=KN(NUM1+J)
         IF((IND2.EQ.0).OR.(IND2.GT.IND1)) GO TO 370
         IF(CYLIND) THEN
            RR=R2DP(I,J)+R2DC(I,J)*DX/DD(K)
         ELSE
            RR=R2DP(I,J)
         ENDIF
         IF(RR.EQ.0.0) GO TO 370
         KEY=KEY1+IND2
         SYS(KEY)=SYS(KEY)+RR*SGD(L,1)*VOL(K)
  370    CONTINUE
  380    CONTINUE
  390    NUM1=NUM1+LL
  400    CONTINUE
      ENDIF
      RETURN
      END
