*DECK BIVRX2
      SUBROUTINE BIVRX2 (MAXMIX,ITY,ISPLH,NELEM,LL4,SGD,SIDE,MAT,VOL,KN,
     1 QFR,MU,R,RH,QH,RT,QT,SYS)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF SYSTEM MATRICES FOR A LINEAR LAGRANGIAN OR MESH CORNER
* FINITE DIFFERENCE DISCRETIZATION IN A 2-D HEXAGONAL GEOMETRY.
*
* INPUT PARAMETERS:
*  MAXMIX : NUMBER OF MATERIAL MIXTURES.
*  ITY    : =0: LEAKAGE-REMOVAL MATRIX ASSEMBLY; =1: X-SECTION MATRIX
*           ASSEMBLY.
*  ISPLH  : =1 : HEXAGONAL ELEMENTS ; .GT.1 : TRIANGULAR ELEMENTS.
*  NELEM  : NUMBER OF FINITE ELEMENTS (HEXAGONS OR TRIANGLES) EXCLUDING
*           THE VIRTUAL ELEMENTS.
*  LL4    : NUMBER OF UNKNOWNS PER ENERGY GROUP.
*  SGD    : CROSS SECTIONS. DIMENSION SGD(MAXMIX,3)
*  SIDE   : SIDE OF AN EQUILATERAL TRIANGLE OR HEXAGON.
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH HEXAGON. DIMENSION MAT(LX)
*           WHERE LX IS THE NUMBER IF HEXAGONS.
*  VOL    : VOLUMES OF THE HEXAGONS. DIMENSION VOL(LX)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN((LC+1)*NELEM)
*           WHERE LC=6 (HEXAGONS) OR 3 (TRIANGLES).
*  QFR    : ELEMENT-ORDERED INFORMATION. DIMENSION QFR((LC+1)*NELEM)
*  MU     : COMPRESSED STORAGE MODE INDICES. DIMENSION MU(LL4)
*  R,RH,QH,RT,QT : UNIT MATRICES. DIMENSION R(2,2),RH(6,6),QH(6,6),
*                  RT(3,3),QT(3,3)
*
* OUTPUT PARAMETER:
*  SYS    : SYSTEM MATRIX. DIMENSION SYS(MU(LL4))
*
* NOTE: SYSTEM MATRICES SHOULD BE INITIALIZED BY THE CALLING PROGRAM.
*
*------------------------------------- AUTHOR: A. HEBERT ; 11/07/94 ----
*
      DOUBLE PRECISION RR,RRH,QQH
      DIMENSION SGD(MAXMIX,*),MAT(*),VOL(*),KN(*),QFR(*),MU(*),R(2,*),
     1 RH(6,*),QH(6,*),RT(3,*),QT(3,*),SYS(*),ISR(6,2),ISRH(6,2),
     2 ISRT(3,2),RH2(6,6),QH2(6,6)
      DATA ISRH/2,1,4,5,6,3,1,4,5,6,3,2/
      DATA ISRT/1,2,3,2,3,1/
*
* RECOVER THE HEXAGONAL MASS (RH2) AND STIFFNESS (QH2) MATRICES.
      IF(ISPLH.EQ.1) THEN
*        HEXAGONAL BASIS.
         LC=6
         DO 10 I=1,6
         DO 10 J=1,2
   10    ISR(I,J)=ISRH(I,J)
         DO 20 I=1,6
         DO 20 J=1,6
         RH2(I,J)=RH(I,J)
   20    QH2(I,J)=QH(I,J)
         CONST=1.5*SQRT(3.0)
         AA=SIDE
      ELSE
*        TRIANGULAR BASIS.
         LC=3
         DO 30 I=1,3
         DO 30 J=1,2
   30    ISR(I,J)=ISRT(I,J)
         DO 40 I=1,3
         DO 40 J=1,3
         RH2(I,J)=RT(I,J)
   40    QH2(I,J)=QT(I,J)
         CONST=0.25*SQRT(3.0)
         AA=SIDE/REAL(ISPLH-1)
      ENDIF
*
* ASSEMBLY OF SYSTEM MATRICES FOR AN HEXAGONAL GEOMETRY.
      IF(ITY.EQ.0) THEN
*        LEAKAGE-REMOVAL SYSTEM MATRIX ASSEMBLY.
         NUM1=0
         DO 100 K=1,NELEM
         KHEX=KN(NUM1+LC+1)
         IF(VOL(KHEX).EQ.0.0) GO TO 100
         L=MAT(KHEX)
         VOL0=QFR(NUM1+LC+1)
         DO 60 I=1,LC
         IND1=KN(NUM1+I)
         IF(IND1.EQ.0) GO TO 60
         KEY1=MU(IND1)-IND1
         DO 50 J=1,LC
         IND2=KN(NUM1+J)
         IF((IND2.EQ.0).OR.(IND2.GT.IND1)) GO TO 50
         QQH=QH2(I,J)/(CONST*AA*AA)
         RRH=RH2(I,J)/CONST
         IF((QQH.EQ.0.0).AND.(RRH.EQ.0.0)) GO TO 50
         KEY=KEY1+IND2
         SYS(KEY)=SYS(KEY)+(QQH*SGD(L,1)+RRH*SGD(L,3))*VOL0
   50    CONTINUE
   60    CONTINUE
         DO 90 IC=1,LC
         QFR1=QFR(NUM1+IC)
         IF(QFR1.EQ.0.0) GO TO 90
         DO 80 I1=1,2
         IND1=KN(NUM1+ISR(IC,I1))
         IF(IND1.EQ.0) GO TO 80
         KEY1=MU(IND1)-IND1
         DO 70 J1=1,2
         IND2=KN(NUM1+ISR(IC,J1))
         IF((IND2.EQ.0).OR.(IND2.GT.IND1)) GO TO 70
         RR=R(I1,J1)
         IF(RR.EQ.0.0) GO TO 70
         KEY=KEY1+IND2
         SYS(KEY)=SYS(KEY)+RR*QFR1
   70    CONTINUE
   80    CONTINUE
   90    CONTINUE
  100    NUM1=NUM1+LC+1
      ELSE
*        CROSS SECTION SYSTEM MATRIX ASSEMBLY.
         NUM1=0
         DO 130 K=1,NELEM
         KHEX=KN(NUM1+LC+1)
         IF(VOL(KHEX).EQ.0.0) GO TO 130
         L=MAT(KHEX)
         VOL0=QFR(NUM1+LC+1)
         DO 120 I=1,LC
         IND1=KN(NUM1+I)
         IF(IND1.EQ.0) GO TO 120
         KEY1=MU(IND1)-IND1
         DO 110 J=1,LC
         IND2=KN(NUM1+J)
         IF((IND2.EQ.0).OR.(IND2.GT.IND1)) GO TO 110
         RRH=RH2(I,J)/CONST
         IF(RRH.EQ.0.0) GO TO 110
         KEY=KEY1+IND2
         SYS(KEY)=SYS(KEY)+RRH*SGD(L,1)*VOL0
  110    CONTINUE
  120    CONTINUE
  130    NUM1=NUM1+LC+1
      ENDIF
      RETURN
      END
