*DECK BIVSYS
      SUBROUTINE BIVSYS(IPTRK,IPMACR,IPSYS,IMPX,NGRP,NEL,NUN,MAT,VOL,
     1 IDL,TITLE,MAXMIX,SGD,WORK,IJJ,NJJ,IPOS)
*
*-----------------------------------------------------------------------
*
* RECOVER THE DIFFUSION COEFFICIENT AND CROSS-SECTION DATA IN THE LINKED
* LIST OR XSM FILE WITH POINTER IPMACR, COMPUTE AND STORE THE CORRES-
* PONDING SYSTEM MATRICES.
*
* INPUT PARAMETERS:
*  IPTRK  : L_BIVAC POINTER TO THE BIVAC TRACKING INFORMATION.
*  IPMACR : L_MACROLIB POINTER TO THE CROSS SECTIONS.
*  IPSYS  : L_SYSTEM POINTER TO SYSTEM MATRICES.
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : TOTAL NUMBER OF UNKNOWNS PER GROUP.
*  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*           DIMENSION MAT(NEL)
*  VOL    : VOLUMES. DIMENSION VOL(NEL)
*  IDL    : POSITION OF THE FLUX COMPONENT ASSOCIATED WITH EACH VOLUME.
*           DIMENSION IDL(NEL)
*  TITLE  : CHARACTER*72 TITLE.
*  MAXMIX : DIMENSION FOR SCRATCH STORAGE ALLOCATION.
*
* OUTPUT PARAMETER:
*  IPSYS : L_SYSTEM POINTER TO THE SYSTEM MATRICES.
*
* SCRATCH STORAGE:
*  SGD    : DIMENSION SGD(MAXMIX,3)
*  WORK   : DIMENSION WORK(MAXMIX*NGRP)
*  IJJ    : DIMENSION IJJ(MAXMIX)
*  NJJ    : DIMENSION NJJ(MAXMIX)
*  IPOS   : DIMENSION IPOS(MAXMIX)
*
*--------------------------------------------- A. HEBERT ; 03/01/94 ----
*
      CHARACTER CGRPNM*12,TEXT12*12
      LOGICAL LFIS
      DIMENSION MAT(*),VOL(*),IDL(*),SGD(MAXMIX,*),WORK(*),IJJ(*),
     1 NJJ(*),IPOS(*),ISPARM(7)
*
      DO 5 I=1,7
        ISPARM(I) = 0
   5  CONTINUE
      ISPARM(1) = NGRP
      DO 70 IGR=1,NGRP
*     PROCESS SECONDARY GROUP IGR.
      WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
      CALL LCMSIX(IPMACR,CGRPNM,1)
*
* PROCESS LEAKAGE AND REMOVAL TERMS.
      CALL LCMLEN(IPMACR,'TOTAL',NBMIX,ITYLCM)
      IF(NBMIX.EQ.0) THEN
         CALL XABORT('BIVSYS: NO TOTAL CROSS SECTIONS.')
      ELSE IF(NBMIX.GT.MAXMIX) THEN
         CALL XABORT('BIVSYS: INVALID LENGTH FOR TOTAL CROSS SECTIONS.')
      ENDIF
      CALL LCMGET(IPMACR,'TOTAL',SGD(1,3))
      CALL LCMLEN(IPMACR,'SIGW 0',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         IF(LENGT.GT.MAXMIX) CALL XABORT('BIVSYS: INVALID LENGTH FOR '
     1   //'''SIGW 0'' CROSS SECTIONS.')
         CALL LCMGET(IPMACR,'SIGW 0',SGD(1,1))
         DO 10 IBM=1,NBMIX
   10    SGD(IBM,3)=SGD(IBM,3)-SGD(IBM,1)
      ENDIF
      CALL LCMLEN(IPMACR,'DIFFX',NBMIX,ITYLCM)
      IF(NBMIX.EQ.0) THEN
         CALL XABORT('BIVSYS: NO DIFFUSION COEFFICIENTS.')
      ELSE IF(NBMIX.GT.MAXMIX) THEN
         CALL XABORT('BIVSYS: INVALID LENGTH FOR DIFFUSION COEFFS.')
      ENDIF
      CALL LCMGET(IPMACR,'DIFFX',SGD(1,1))
      CALL LCMLEN(IPMACR,'DIFFY',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(IPMACR,'DIFFY',SGD(1,2))
      ELSE
         CALL LCMGET(IPMACR,'DIFFX',SGD(1,2))
      ENDIF
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      CALL BIVASM(TEXT12,0,IPTRK,IPSYS,IMPX,MAXMIX,NEL,NUN,MAT,VOL,IDL,
     1 SGD,TITLE,ISPARM)
*
* PROCESS SCATTERING TERMS.
      CALL LCMLEN(IPMACR,'NJJ 0',NBMIX,ITYLCM)
      IF(NBMIX.GT.MAXMIX) CALL XABORT('BIVSYS: INVALID LENGTH FOR ''N'
     1 //'JJ 0'' INFORMATION.')
      IF(NBMIX.GT.0) THEN
         CALL LCMGET(IPMACR,'NJJ 0',NJJ)
         CALL LCMGET(IPMACR,'IJJ 0',IJJ)
         JGRMIN=IGR
         JGRMAX=IGR
         DO 40 IBM=1,NBMIX
         JGRMIN=MIN(JGRMIN,IJJ(IBM)-NJJ(IBM)+1)
   40    JGRMAX=MAX(JGRMAX,IJJ(IBM))
         CALL LCMGET(IPMACR,'IPOS 0',IPOS)
         CALL LCMGET(IPMACR,'SCAT 0',WORK)
         DO 60 JGR=JGRMAX,JGRMIN,-1
         IF(JGR.EQ.IGR) GO TO 60
         DO 50 IBM=1,NBMIX
         IF((JGR.GT.IJJ(IBM)-NJJ(IBM)).AND.(JGR.LE.IJJ(IBM))) THEN
            SGD(IBM,1)=WORK(IPOS(IBM)+IJJ(IBM)-JGR)
         ELSE
            SGD(IBM,1)=0.0
         ENDIF
   50    CONTINUE
         WRITE (TEXT12,'(1HA,I3,I3)') IGR,JGR
         CALL BIVASM(TEXT12,1,IPTRK,IPSYS,IMPX,MAXMIX,NEL,NUN,MAT,VOL,
     1   IDL,SGD,TITLE,ISPARM)
   60    CONTINUE
      ENDIF
      IF(IPMACR.NE.IPMACR) CALL LCMSIX(IPMACR,' ',2)
      CALL LCMSIX(IPMACR,' ',2)
   70 CONTINUE
*
* PROCESS FISSION SPECTRUM TERMS.
      WRITE (CGRPNM,'(9HGROUP  1/,I3)') NGRP
      CALL LCMSIX(IPMACR,CGRPNM,1)
      CALL LCMLEN(IPMACR,'CHI',NBMIX,ITYLCM)
      IF(NBMIX.GT.MAXMIX) CALL XABORT('BIVSYS: INVALID LENGTH FOR CHI'
     1 //' INFORMATION.')
      CALL LCMSIX(IPMACR,' ',2)
      IF(NBMIX.GT.0) THEN
         DO 80 IGR=1,NGRP
         WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
         CALL LCMSIX(IPMACR,CGRPNM,1)
         CALL LCMGET(IPMACR,'CHI',WORK((IGR-1)*MAXMIX+1))
         CALL LCMSIX(IPMACR,' ',2)
   80    CONTINUE
      ELSE
         NBMIX=MAXMIX
         DO 90 IBM=1,MAXMIX
         WORK(IBM)=1.0
         DO 90 IGR=2,NGRP
   90    WORK((IGR-1)*MAXMIX+IBM)=0.0
      ENDIF
*
* PROCESS FISSION NUSIGF TERMS.
      DO 130 IGR=1,NGRP
*     PROCESS SECONDARY GROUP IGR.
      LFIS=.FALSE.
      DO 100 IBM=1,NBMIX
  100 LFIS=LFIS.OR.(WORK((IGR-1)*MAXMIX+IBM).NE.0.0)
      IF(LFIS) THEN
         DO 120 JGR=1,NGRP
         WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') JGR,NGRP
         CALL LCMSIX(IPMACR,CGRPNM,1)
         CALL LCMLEN(IPMACR,'NUSIGF',LENGT,ITYLCM)
         IF(LENGT.GT.MAXMIX) CALL XABORT('BIVSYS: INVALID LENGTH FOR '
     1    //'NUSIGF INFORMATION.')
         IF(LENGT.GT.0) THEN
            CALL LCMGET(IPMACR,'NUSIGF',SGD)
            DO 110 IBM=1,LENGT
  110       SGD(IBM,1)=SGD(IBM,1)*WORK((IGR-1)*MAXMIX+IBM)
            WRITE (TEXT12,'(1HB,I3,I3)') IGR,JGR
            CALL BIVASM(TEXT12,1,IPTRK,IPSYS,IMPX,MAXMIX,NEL,NUN,MAT,
     1      VOL,IDL,SGD,TITLE,ISPARM)
         ENDIF
         CALL LCMSIX(IPMACR,' ',2)
  120    CONTINUE
      ENDIF
  130 CONTINUE
*
      CALL LCMPUT(IPSYS,'SYS-PARAM',7,1,ISPARM)
*
      RETURN
      END
