*DECK CREBUR
      SUBROUTINE CREBUR (JPLIST,NISO,NGRP,NL,IMPX,HISO,DERIV,NBURN,
     1 BURN0,BURN1,BURNUP,ITY,CONC,TOTAL,ZNUG,SNUGF,CHI,OVERV,DIFFX,
     2 DIFFY,DIFFZ,H,SCAT,FLUX,ZTOTAL,ZZNUG,ZNUGF,ZCHI,ZOVERV,ZDIFFX,
     3 ZDIFFY,ZDIFFZ,ZH,ZSCAT,ZFLUX,DENSIT,UPS)
*
*-----------------------------------------------------------------------
*
* INTERPOLATE A COMPO FOR A GIVEN BURNUP OR NEUTRON EXPOSURE VALUE.
*
* INPUT PARAMETERS:
*  JPLIST : ADDRESS OF THE COMPO LINKED LIST OR XSM FILE.
*  NISO   : 1+NUMBER OF EXTRACTED ISOTOPES.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NL     : NUMBER OF LEGENDRE ORDERS (=1 FOR ISOTROPIC SCATTERING).
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*  HISO   : HOLLERITH NAME INFORMATION FOR EXTRACTED ISOTOPES.
*           DIMENSION HISO(3*NISO)
*  DERIV  : =.TRUE.: DERIVATIVE OF MACROLIB INFO IS COMPUTED WITH
*           RESPECT TO BURN1.
*  NBURN  : NUMBER OF TABULATED BURNUP STEPS.
*  BURN0  : USER DEFINED INITIAL BURNUP OR NEUTRON EXPOSURE.
*  BURN1  : USER DEFINED FINAL BURNUP OR NEUTRON EXPOSURE.
*           IF BURN0=BURN1, A SIMPLE INTERPOLATION IS PERFORMED.
*           IF BURN0.LT.BURN1, A TIME AVERAGE CALCULATION IS PERFORMED.
*  BURNUP : BURNUP OR NEUTRON EXPOSURE TABULATION POINTS.
*           DIMENSION BURNUP(NBURN)
*  ITY    : =0: DO NOT PROCESS THE ISOTOPE; =1: USE NUMBER DENSITY
*           STORED IN CONC(I); =2: USE NUMBER DENSITY STORED IN COMPO.
*           DIMENSION ITY(NISO)
*  CONC   : USER DEFINED NUMBER DENSITY. DIMENSION CONC(NISO)
*
* INTERPOLATED OUTPUT PARAMETERS:
*  TOTAL  : TOTAL MACROSCOPIC X-SECTIONS. DIMENSION TOTAL(NGRP)
*  ZNUG   : NU * FISSION MACROSCOPIC X-SECTIONS. DIMENSION ZNUG(NGRP)
*  SNUGF   : FISSION MACROSCOPIC X-SECTIONS. DIMENSION SNUGF(NGRP)
*  CHI    : FISSION SPECTRUM. DIMENSION CHI(NGRP)
*  OVERV  : RECIPROCAL NEUTRON VELOCITIES. DIMENSION OVERV(NGRP)
*  DIFFX  : X-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFX(NGRP)
*  DIFFY  : Y-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFY(NGRP)
*  DIFFZ  : Z-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFZ(NGRP)
*  H      : H-FACTORS (KAPPA * FISSION MACROSCOPIC X-SECTIONS).
*           DIMENSION H(NGRP)
*  SCAT   : SCATTERING MACROSCOPIC X-SECTIONS.
*           DIMENSION SCAT(NL,NGRP,NGRP)
*  FLUX   : INTEGRATED FLUX. DIMENSION FLUX(NGRP)
*
* SCRATCH STORAGE:
*  ZTOTAL : DIMENSION ZTOTAL(2*NGRP*NBURN)
*  ZZNUG  : DIMENSION ZZNUG(2*NGRP*NBURN)
*  ZNUGF  : DIMENSION ZNUGF(2*NGRP*NBURN)
*  ZCHI   : DIMENSION ZCHI(2*NGRP*NBURN)
*  ZOVERV : DIMENSION ZOVERV(2*NGRP*NBURN)
*  ZDIFFX : DIMENSION ZDIFFX(2*NGRP*NBURN)
*  ZDIFFY : DIMENSION ZDIFFY(2*NGRP*NBURN)
*  ZDIFFZ : DIMENSION ZDIFFZ(2*NGRP*NBURN)
*  ZH     : DIMENSION H(2*NGRP*NBURN)
*  ZSCAT  : DIMENSION ZSCAT(2*NL*NGRP*NGRP*NBURN)
*  ZFLUX  : DIMENSION ZFLUX(2*NGRP*NBURN)
*  DENSIT : DIMENSION DENSIT(NISO)
*
*-------------------------------------- AUTHOR: A. HEBERT ; 18/02/94 ---
*-------------------------- MODIFIED : E. VARIN ; 11/09/95 ---
*
      LOGICAL DERIV,UPS
      DIMENSION HISO(*),BURNUP(*),ITY(*),CONC(*),TOTAL(*),ZNUG(*),
     1 CHI(*),OVERV(*),DIFFX(*),DIFFY(*),DIFFZ(*),H(*),SCAT(NL,NGRP,*),
     2 SNUGF(*),FLUX(*),ZTOTAL(NBURN,NGRP,*),ZZNUG(NBURN,NGRP,*),
     3 ZCHI(NBURN,NGRP,*),ZOVERV(NBURN,NGRP,*),ZDIFFX(NBURN,NGRP,*),
     4 ZDIFFY(NBURN,NGRP,*),ZDIFFZ(NBURN,NGRP,*),ZH(NBURN,NGRP,*),
     5 ZSCAT(NBURN,NL,NGRP,NGRP,*),ZFLUX(NBURN,NGRP,*),DENSIT(*),
     6 ZNUGF(NBURN,NGRP,*)
      COMMON BASE(1)
*
* RECOVER MACROSCOPIC X-SECTION INFO FROM BURNUP DIRECTORIES:
      IF(NBURN.LE.1) CALL XABORT('CREBUR: INVALID VALUE OF NBURN.')
      CALL CRETAB(JPLIST,NISO,NGRP,NL,IMPX,HISO,NBURN,
     1 BURNUP,ITY,CONC,TOTAL,ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,
     2 DIFFZ,H,SCAT,FLUX,ZTOTAL,ZZNUG,ZNUGF,ZCHI,ZOVERV,ZDIFFX,
     3 ZDIFFY,ZDIFFZ,ZH,ZSCAT,ZFLUX,DENSIT,UPS)
*
* PERFORM INTERPOLATION OR TIME AVERAGING.
      CALL CREITP(NGRP,NL,DERIV,NBURN,
     1 BURN0,BURN1,BURNUP,TOTAL,ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,
     2 DIFFZ,H,SCAT,FLUX,ZTOTAL,ZZNUG,ZNUGF,ZCHI,ZOVERV,ZDIFFX,
     3 ZDIFFY,ZDIFFZ,ZH,ZSCAT,ZFLUX)
*
      RETURN
      END
