*DECK CREINT
      SUBROUTINE CREINT (JPLIST,NISO,DERIV,NBURN,KBURN,BURN0,BURN1,NGRP,
     1 NL,IMPX,HISO,ITY,CONC,TOTAL,ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,
     2 DIFFZ,H,SCAT,FLUX,BURNUP,DENSIT,UPS)
*
*-----------------------------------------------------------------------
*
* RECOVER AND INTERPOLATE COMPO INFORMATION ACCORDING TO BURNUP AND
* EXTRACTED ISOTOPE DENSITY.
*
* INPUT PARAMETERS:
*  JPLIST : ADDRESS OF THE COMPO LINKED LIST OR XSM FILE.
*  NISO   : 1+NUMBER OF EXTRACTED ISOTOPES.
*  DERIV  : =.TRUE.: DERIVATIVE OF MACROLIB INFO IS COMPUTED WITH
*           RESPECT TO BURN1.
*  UPS    : =.TRUE.: NO UPSCATERING CROSS SECTIONS WILL BE STORED.
*  NBURN  : NUMBER OF TABULATED BURNUP STEPS.
*  KBURN  : =0: NO BURNUP PARAMETERS; =1: USE MW DAY/TONNE OF INITIAL
*           HEAVY ELEMENTS); =2: USE N/KB.
*  BURN0  : USER DEFINED INITIAL BURNUP OR NEUTRON EXPOSURE.
*  BURN1  : USER DEFINED FINAL BURNUP OR NEUTRON EXPOSURE.
*           IF BURN0=BURN1, A SIMPLE INTERPOLATION IS PERFORMED.
*           IF BURN0.LT.BURN1, A TIME AVERAGE CALCULATION IS PERFORMED.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NL     : NUMBER OF LEGENDRE ORDERS (=1 FOR ISOTROPIC SCATTERING).
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*  HISO   : HOLLERITH NAME INFORMATION FOR EXTRACTED ISOTOPES.
*           DIMENSION HISO(3*NISO)
*  ITY    : =0: DO NOT PROCESS THE ISOTOPE; =1: USE NUMBER DENSITY
*           STORED IN CONC(I); =2: USE NUMBER DENSITY STORED IN COMPO.
*           DIMENSION ITY(NISO)
*  CONC   : USER DEFINED NUMBER DENSITY. DIMENSION CONC(NISO)
*
* OUTPUT PARAMETERS:
*  TOTAL  : TOTAL MACROSCOPIC X-SECTIONS. DIMENSION TOTAL(NGRP)
*  ZNUG   : NU * FISSION MACROSCOPIC X-SECTIONS. DIMENSION ZNUG(NGRP)
*  SNUGF   : FISSION MACROSCOPIC X-SECTIONS. DIMENSION SNUGF(NGRP)
*  CHI    : FISSION SPECTRUM. DIMENSION CHI(NGRP)
*  OVERV  : RECIPROCAL NEUTRON VELOCITIES. DIMENSION OVERV(NGRP)
*  DIFFX  : X-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFX(NGRP)
*  DIFFY  : Y-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFY(NGRP)
*  DIFFZ  : Z-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFZ(NGRP)
*  H      : H-FACTORS (KAPPA * FISSION MACROSCOPIC X-SECTIONS).
*           DIMENSION H(NGRP)
*  SCAT   : SCATTERING MACROSCOPIC X-SECTIONS.
*           DIMENSION SCAT(NL,NGRP,NGRP)
*  FLUX   : INTEGRATED FLUXES. DIMENSION FLUX(NGRP)
*
* SCRATCH STORAGE:
*  BURNUP : DIMENSION BURNUP(NBURN)
*  DENSIT : DIMENSION DENSIT(NISO)
*
*-------------------------------------- AUTHOR: A. HEBERT ; 17/02/94 ---
*------------------------------------- MODIFIED : E. VARIN ; 08/08/95 --
*
      CHARACTER TEXT12*12,HISO*12
      LOGICAL DERIV,UPS
      INTEGER IBASE(1)
      DIMENSION HISO(*),ITY(*),CONC(*),TOTAL(*),ZNUG(*),SNUGF(*),CHI(*),
     1 OVERV(*),DIFFX(*),DIFFY(*),DIFFZ(*),H(*),SCAT(NL,NGRP,*),
     2 FLUX(*),BURNUP(*),DENSIT(*)
      EQUIVALENCE (BASE(1),IBASE(1))
      COMMON BASE(1)
*
      IF(KBURN.EQ.0) THEN
*        CASE WITH NO BURNUP PARAMETER.
         CALL LCMSIX(JPLIST,'BURN       1',1)
         CALL LCMGET(JPLIST,'ISOTOPESDENS',DENSIT)
         IF(DENSIT(1).NE.1.0) CALL XABORT('CREINT: DENSIT(1).NE.1.')
         DO 10 I=2,NISO
         IF(ITY(I).EQ.0) THEN
            DENSIT(I)=0.0
         ELSE IF(ITY(I).EQ.1) THEN
            DENSIT(I)=CONC(I)
         ELSE IF(ITY(I).NE.2) THEN
            CALL XABORT('CREINT: INVALID VALUE OF ITY.')
         ENDIF
   10    CONTINUE
         CALL SETARA(IBASE,NGRP,IIJJ)
         CALL SETARA(IBASE,NGRP,INJJ)
         CALL SETARA(IBASE,21+NL,INDXS)
         CALL SETARA(BASE,NISO,IENG)
         CALL SETARA(BASE,3*NGRP,IWORK1)
         CALL SETARA(BASE,NGRP*NGRP,IWORK2)
         CALL CREMAC(JPLIST,NISO,NGRP,NL,IMPX,HISO,DENSIT,TOTAL,ZNUG,
     1   SNUGF,CHI,OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,FLUX,IBASE(IIJJ),
     2   IBASE(INJJ),BASE(IENG),IBASE(INDXS),BASE(IWORK1),
     3   BASE(IWORK2),UPS)
         CALL RLSARA(BASE(IWORK2))
         CALL RLSARA(BASE(IWORK1))
         CALL RLSARA(BASE(IENG))
         CALL RLSARA(IBASE(INDXS))
         CALL RLSARA(IBASE(INJJ))
         CALL RLSARA(IBASE(IIJJ))
         CALL LCMSIX(JPLIST,' ',2)
      ELSE
*        CASE WITH BURNUP PARAMETERS.
         IF(KBURN.EQ.1) THEN
            CALL LCMGET(JPLIST,'BURNUP',BURNUP)
         ELSE IF(KBURN.EQ.2) THEN
            CALL LCMGET(JPLIST,'N/KB',BURNUP)
         ENDIF
         TEXT12=' '
         IF(BURN0.EQ.BURN1) THEN
            DO 20 I=1,NBURN
            IF(BURN0.EQ.BURNUP(I)) THEN
               WRITE(TEXT12,'(4HBURN,4X,I4)') I
               GO TO 30
            ENDIF
   20       CONTINUE
         ENDIF
   30    IF((TEXT12.NE.' ').AND.(.NOT.DERIV)) THEN
*           BURN0=BURN1 IS A TABULATION POINT.
            CALL LCMSIX(JPLIST,TEXT12,1)
            CALL LCMGET(JPLIST,'ISOTOPESDENS',DENSIT)
            IF(DENSIT(1).NE.1.0) CALL XABORT('CREINT: DENSIT(1).NE.1.')
            DO 40 I=2,NISO
            IF(ITY(I).EQ.0) THEN
               DENSIT(I)=0.0
            ELSE IF(ITY(I).EQ.1) THEN
               DENSIT(I)=CONC(I)
            ELSE IF(ITY(I).NE.2) THEN
               CALL XABORT('CREINT: INVALID VALUE OF ITY.')
            ENDIF
   40       CONTINUE
            CALL SETARA(BASE,NGRP,IIJJ)
            CALL SETARA(BASE,NGRP,INJJ)
            CALL SETARA(IBASE,21+NL,INDXS)
            CALL SETARA(BASE,NISO,IENG)
            CALL SETARA(BASE,3*NGRP,IWORK1)
            CALL SETARA(BASE,NGRP*NGRP,IWORK2)
            CALL CREMAC(JPLIST,NISO,NGRP,NL,IMPX,HISO,DENSIT,TOTAL,
     1      ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,FLUX,
     2      IBASE(IIJJ),IBASE(INJJ),BASE(IENG),IBASE(INDXS),
     3      BASE(IWORK1),BASE(IWORK2),UPS)
            CALL RLSARA(BASE(IWORK2))
            CALL RLSARA(BASE(IWORK1))
            CALL RLSARA(BASE(IENG))
            CALL RLSARA(IBASE(INDXS))
            CALL RLSARA(IBASE(INJJ))
            CALL RLSARA(IBASE(IIJJ))
            CALL LCMSIX(JPLIST,' ',2)
         ELSE
*           INTERPOLATION IS REQUIRED.
            CALL SETARA(BASE,2*NGRP*NBURN,ITOTAL)
            CALL SETARA(BASE,2*NGRP*NBURN,IZNUG)
            CALL SETARA(BASE,2*NGRP*NBURN,INUGF)
            CALL SETARA(BASE,2*NGRP*NBURN,ICHI)
            CALL SETARA(BASE,2*NGRP*NBURN,IOVERV)
            CALL SETARA(BASE,2*NGRP*NBURN,IDIFFX)
            CALL SETARA(BASE,2*NGRP*NBURN,IDIFFY)
            CALL SETARA(BASE,2*NGRP*NBURN,IDIFFZ)
            CALL SETARA(BASE,2*NGRP*NBURN,IH)
            CALL SETARA(BASE,2*NL*NGRP*NGRP*NBURN,ISCAT)
            CALL SETARA(BASE,2*NGRP*NBURN,IFLUX)
            CALL CREBUR(JPLIST,NISO,NGRP,NL,IMPX,HISO,DERIV,NBURN,BURN0,
     1      BURN1,BURNUP,ITY,CONC,TOTAL,ZNUG,SNUGF,CHI,OVERV,DIFFX,
     2      DIFFY,DIFFZ,H,SCAT,FLUX,BASE(ITOTAL),BASE(IZNUG),
     3      BASE(INUGF),BASE(ICHI),BASE(IOVERV),BASE(IDIFFX),
     4      BASE(IDIFFY),BASE(IDIFFZ),BASE(IH),BASE(ISCAT),BASE(IFLUX),
     5      DENSIT,UPS)
            CALL RLSARA(BASE(IFLUX))
            CALL RLSARA(BASE(ISCAT))
            CALL RLSARA(BASE(IH))
            CALL RLSARA(BASE(IDIFFZ))
            CALL RLSARA(BASE(IDIFFY))
            CALL RLSARA(BASE(IDIFFX))
            CALL RLSARA(BASE(IOVERV))
            CALL RLSARA(BASE(ICHI))
            CALL RLSARA(BASE(INUGF))
            CALL RLSARA(BASE(IZNUG))
            CALL RLSARA(BASE(ITOTAL))
         ENDIF
      ENDIF
      RETURN
      END
