*DECK CREITP
      SUBROUTINE CREITP (NGRP,NL,DERIV,NBURN,BURN0,BURN1,BURNUP,TOTAL,
     1 ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,FLUX,ZTOTAL,
     2 ZZNUG,ZNUGF,ZCHI,ZOVERV,ZDIFFX,ZDIFFY,ZDIFFZ,
     3 ZH,ZSCAT,ZFLUX)
*
*-----------------------------------------------------------------------
*
* INTERPOLATE BURNUP DEPENDENT TABLE FOR A GIVEN BURNUP OR NEUTRON
* EXPOSURE VALUE OR TIME-AVERAGE OR DERIVATIVES.
*
* INPUT PARAMETERS:
*  JPLIST : ADDRESS OF THE COMPO LINKED LIST OR XSM FILE.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NL     : NUMBER OF LEGENDRE ORDERS (=1 FOR ISOTROPIC SCATTERING).
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*  DERIV  : =.TRUE.: DERIVATIVE OF MACROLIB INFO IS COMPUTED WITH
*           RESPECT TO BURN1.
*  NBURN  : NUMBER OF TABULATED BURNUP STEPS.
*  BURN0  : USER DEFINED INITIAL BURNUP OR NEUTRON EXPOSURE.
*  BURN1  : USER DEFINED FINAL BURNUP OR NEUTRON EXPOSURE.
*           IF BURN0=BURN1, A SIMPLE INTERPOLATION IS PERFORMED.
*           IF BURN0.LT.BURN1, A TIME AVERAGE CALCULATION IS PERFORMED.
*  BURNUP : BURNUP OR NEUTRON EXPOSURE TABULATION POINTS.
*           DIMENSION BURNUP(NBURN)
*
* INTERPOLATED OUTPUT PARAMETERS:
*  TOTAL  : TOTAL MACROSCOPIC X-SECTIONS. DIMENSION TOTAL(NGRP)
*  ZNUG   : NU * FISSION MACROSCOPIC X-SECTIONS. DIMENSION ZNUG(NGRP)
*  SNUGF   : FISSION MACROSCOPIC X-SECTIONS. DIMENSION SNUGF(NGRP)
*  CHI    : FISSION SPECTRUM. DIMENSION CHI(NGRP)
*  OVERV  : RECIPROCAL NEUTRON VELOCITIES. DIMENSION OVERV(NGRP)
*  DIFFX  : X-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFX(NGRP)
*  DIFFY  : Y-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFY(NGRP)
*  DIFFZ  : Z-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFZ(NGRP)
*  H      : H-FACTORS (KAPPA * FISSION MACROSCOPIC X-SECTIONS).
*           DIMENSION H(NGRP)
*  SCAT   : SCATTERING MACROSCOPIC X-SECTIONS.
*           DIMENSION SCAT(NL,NGRP,NGRP)
*  FLUX   : INTEGRATED FLUX. DIMENSION FLUX(NGRP)
*
* SCRATCH STORAGE:
*  ZTOTAL : DIMENSION ZTOTAL(2*NGRP*NBURN)
*  ZZNUG  : DIMENSION ZZNUG(2*NGRP*NBURN)
*  ZNUGF  : DIMENSION ZNUGF(2*NGRP*NBURN)
*  ZCHI   : DIMENSION ZCHI(2*NGRP*NBURN)
*  ZOVERV : DIMENSION ZOVERV(2*NGRP*NBURN)
*  ZDIFFX : DIMENSION ZDIFFX(2*NGRP*NBURN)
*  ZDIFFY : DIMENSION ZDIFFY(2*NGRP*NBURN)
*  ZDIFFZ : DIMENSION ZDIFFZ(2*NGRP*NBURN)
*  ZH     : DIMENSION H(2*NGRP*NBURN)
*  ZSCAT  : DIMENSION ZSCAT(2*NL*NGRP*NGRP*NBURN)
*  ZFLUX  : DIMENSION ZFLUX(2*NGRP*NBURN)
*
*-------------------------------------- AUTHOR: A. HEBERT ; 18/02/94 ---
*-------------------------- MODIFIED : E. VARIN ; 11/09/95 ---
*
      LOGICAL DERIV
      DIMENSION BURNUP(*),TOTAL(*),ZNUG(*),SNUGF(*),CHI(*),OVERV(*),
     1 DIFFX(*),DIFFY(*),DIFFZ(*),H(*),SCAT(NL,NGRP,*),FLUX(*),
     2 ZTOTAL(NBURN,NGRP,*),ZZNUG(NBURN,NGRP,*),ZNUGF(NBURN,NGRP,*),
     3 ZCHI(NBURN,NGRP,*),ZOVERV(NBURN,NGRP,*),ZDIFFX(NBURN,NGRP,*),
     4 ZDIFFY(NBURN,NGRP,*),ZDIFFZ(NBURN,NGRP,*),ZH(NBURN,NGRP,*),
     5 ZSCAT(NBURN,NL,NGRP,NGRP,*),ZFLUX(NBURN,NGRP,*),UU(2)
*
      IF(BURN1.LT.BURN0) CALL XABORT('CREITP: ILLEGAL BURN1')
*
* PERFORM INTERPOLATION OR TIME AVERAGING.
      IF(((BURN0.EQ.BURN1).AND.(.NOT.DERIV)).OR.((BURN0.LT.BURN1)
     1   .AND.DERIV)) THEN
*        INSTANTANEOUS MACROLIB IS REQUIRED.
         DO 40 I=1,NBURN-1
         IF((BURN1.GE.BURNUP(I)).AND.(BURN1.LE.BURNUP(I+1))) THEN
            IBURN=I
            GO TO 50
         ENDIF
   40    CONTINUE
         CALL XABORT('CREITP: UNABLE TO INTERPOLATE.')
   50    DX=(BURNUP(IBURN+1)-BURNUP(IBURN))
         U=(BURN1-0.5*(BURNUP(IBURN)+BURNUP(IBURN+1)))/DX
         H1=3.0*(0.5-U)**2-2.0*(0.5-U)**3
         H2=(0.5-U)**2-(0.5-U)**3
         H3=3.0*(0.5+U)**2-2.0*(0.5+U)**3
         H4=-(0.5+U)**2+(0.5+U)**3
         DO 60 JGR=1,NGRP
         TOTAL(JGR)=H1*ZTOTAL(IBURN,JGR,1)+H2*ZTOTAL(IBURN,JGR,2)*DX+
     1   H3*ZTOTAL(IBURN+1,JGR,1)+H4*ZTOTAL(IBURN+1,JGR,2)*DX
         ZNUG(JGR)=H1*ZZNUG(IBURN,JGR,1)+H2*ZZNUG(IBURN,JGR,2)*DX+
     1   H3*ZZNUG(IBURN+1,JGR,1)+H4*ZZNUG(IBURN+1,JGR,2)*DX
         SNUGF(JGR)=H1*ZNUGF(IBURN,JGR,1)+H2*ZNUGF(IBURN,JGR,2)*DX+
     1   H3*ZNUGF(IBURN+1,JGR,1)+H4*ZNUGF(IBURN+1,JGR,2)*DX
         CHI(JGR)=H1*ZCHI(IBURN,JGR,1)+H2*ZCHI(IBURN,JGR,2)*DX+
     1   H3*ZCHI(IBURN+1,JGR,1)+H4*ZCHI(IBURN+1,JGR,2)*DX
         OVERV(JGR)=H1*ZOVERV(IBURN,JGR,1)+H2*ZOVERV(IBURN,JGR,2)*DX+
     1   H3*ZOVERV(IBURN+1,JGR,1)+H4*ZOVERV(IBURN+1,JGR,2)*DX
         DIFFX(JGR)=H1*ZDIFFX(IBURN,JGR,1)+H2*ZDIFFX(IBURN,JGR,2)*DX+
     1   H3*ZDIFFX(IBURN+1,JGR,1)+H4*ZDIFFX(IBURN+1,JGR,2)*DX
         DIFFY(JGR)=H1*ZDIFFY(IBURN,JGR,1)+H2*ZDIFFY(IBURN,JGR,2)*DX+
     1   H3*ZDIFFY(IBURN+1,JGR,1)+H4*ZDIFFY(IBURN+1,JGR,2)*DX
         DIFFZ(JGR)=H1*ZDIFFZ(IBURN,JGR,1)+H2*ZDIFFZ(IBURN,JGR,2)*DX+
     1   H3*ZDIFFZ(IBURN+1,JGR,1)+H4*ZDIFFZ(IBURN+1,JGR,2)*DX
         H(JGR)=H1*ZH(IBURN,JGR,1)+H2*ZH(IBURN,JGR,2)*DX+
     1   H3*ZH(IBURN+1,JGR,1)+H4*ZH(IBURN+1,JGR,2)*DX
         FLUX(JGR)=H1*ZFLUX(IBURN,JGR,1)+H2*ZFLUX(IBURN,JGR,2)*DX+
     1   H3*ZFLUX(IBURN+1,JGR,1)+H4*ZFLUX(IBURN+1,JGR,2)*DX
         DO 60 IGR=1,NGRP
         DO 60 IL=1,NL
         SCAT(IL,IGR,JGR)=H1*ZSCAT(IBURN,IL,IGR,JGR,1)+
     1   H2*ZSCAT(IBURN,IL,IGR,JGR,2)*DX+
     2   H3*ZSCAT(IBURN+1,IL,IGR,JGR,1)+
     3   H4*ZSCAT(IBURN+1,IL,IGR,JGR,2)*DX
  60     CONTINUE
         IF(BURN0.LT.BURN1) THEN
*           DERIVATIVE WITH RESPECT TO THE EXIT BURNUP OR EXPOSURE.
            DO 70 JGR=1,NGRP
            TOTAL(JGR)=TOTAL(JGR)/(BURN1-BURN0)
            ZNUG(JGR)=ZNUG(JGR)/(BURN1-BURN0)
            SNUGF(JGR)=SNUGF(JGR)/(BURN1-BURN0)
            CHI(JGR)=CHI(JGR)/(BURN1-BURN0)
            OVERV(JGR)=OVERV(JGR)/(BURN1-BURN0)
            DIFFX(JGR)=DIFFX(JGR)/(BURN1-BURN0)
            DIFFY(JGR)=DIFFY(JGR)/(BURN1-BURN0)
            DIFFZ(JGR)=DIFFZ(JGR)/(BURN1-BURN0)
            H(JGR)=H(JGR)/(BURN1-BURN0)
            FLUX(JGR)=FLUX(JGR)/(BURN1-BURN0)
            DO 70 IGR=1,NGRP
            DO 70 IL=1,NL
            SCAT(IL,IGR,JGR)=SCAT(IL,IGR,JGR)/(BURN1-BURN0)
  70        CONTINUE
         ENDIF
      ELSE IF((BURN0.EQ.BURN1).AND.DERIV) THEN
*        DERIVATIVE OF THE MACROLIB IS REQUIRED.
         DO 80 I=1,NBURN-1
         IF((BURN1.GE.BURNUP(I)).AND.(BURN1.LE.BURNUP(I+1))) THEN
            IBURN=I
            GO TO 90
         ENDIF
   80    CONTINUE
         CALL XABORT('CREITP: UNABLE TO INTERPOLATE.')
   90    DX=(BURNUP(IBURN+1)-BURNUP(IBURN))
         U=(BURN1-0.5*(BURNUP(IBURN)+BURNUP(IBURN+1)))/DX
         H1=-6.0*(0.5-U)+6.0*(0.5-U)**2
         H2=-2.0*(0.5-U)+3.0*(0.5-U)**2
         H3=6.0*(0.5+U)-6.0*(0.5+U)**2
         H4=-2.0*(0.5+U)+3.0*(0.5+U)**2
         DO 100 JGR=1,NGRP
         TOTAL(JGR)=(H1*ZTOTAL(IBURN,JGR,1)+H2*ZTOTAL(IBURN,JGR,2)*DX+
     1   H3*ZTOTAL(IBURN+1,JGR,1)+H4*ZTOTAL(IBURN+1,JGR,2)*DX)/DX
         ZNUG(JGR)=(H1*ZZNUG(IBURN,JGR,1)+H2*ZZNUG(IBURN,JGR,2)*DX+
     1   H3*ZZNUG(IBURN+1,JGR,1)+H4*ZZNUG(IBURN+1,JGR,2)*DX)/DX
         SNUGF(JGR)=(H1*ZNUGF(IBURN,JGR,1)+H2*ZNUGF(IBURN,JGR,2)*DX+
     1   H3*ZNUGF(IBURN+1,JGR,1)+H4*ZNUGF(IBURN+1,JGR,2)*DX)/DX
         CHI(JGR)=(H1*ZCHI(IBURN,JGR,1)+H2*ZCHI(IBURN,JGR,2)*DX+
     1   H3*ZCHI(IBURN+1,JGR,1)+H4*ZCHI(IBURN+1,JGR,2)*DX)/DX
         OVERV(JGR)=(H1*ZOVERV(IBURN,JGR,1)+H2*ZOVERV(IBURN,JGR,2)*DX+
     1   H3*ZOVERV(IBURN+1,JGR,1)+H4*ZOVERV(IBURN+1,JGR,2)*DX)/DX
         DIFFX(JGR)=(H1*ZDIFFX(IBURN,JGR,1)+H2*ZDIFFX(IBURN,JGR,2)*DX+
     1   H3*ZDIFFX(IBURN+1,JGR,1)+H4*ZDIFFX(IBURN+1,JGR,2)*DX)/DX
         DIFFY(JGR)=(H1*ZDIFFY(IBURN,JGR,1)+H2*ZDIFFY(IBURN,JGR,2)*DX+
     1   H3*ZDIFFY(IBURN+1,JGR,1)+H4*ZDIFFY(IBURN+1,JGR,2)*DX)/DX
         DIFFZ(JGR)=(H1*ZDIFFZ(IBURN,JGR,1)+H2*ZDIFFZ(IBURN,JGR,2)*DX+
     1   H3*ZDIFFZ(IBURN+1,JGR,1)+H4*ZDIFFZ(IBURN+1,JGR,2)*DX)/DX
         H(JGR)=(H1*ZH(IBURN,JGR,1)+H2*ZH(IBURN,JGR,2)*DX+
     1   H3*ZH(IBURN+1,JGR,1)+H4*ZH(IBURN+1,JGR,2)*DX)/DX
         FLUX(JGR)=(H1*ZFLUX(IBURN,JGR,1)+H2*ZFLUX(IBURN,JGR,2)*DX+
     1   H3*ZFLUX(IBURN+1,JGR,1)+H4*ZFLUX(IBURN+1,JGR,2)*DX)/DX
         DO 100 IGR=1,NGRP
         DO 100 IL=1,NL
         SCAT(IL,IGR,JGR)=(H1*ZSCAT(IBURN,IL,IGR,JGR,1)+
     1   H2*ZSCAT(IBURN,IL,IGR,JGR,2)*DX+
     2   H3*ZSCAT(IBURN+1,IL,IGR,JGR,1)+
     3   H4*ZSCAT(IBURN+1,IL,IGR,JGR,2)*DX)/DX
  100    CONTINUE
      ELSE IF((BURN0.LT.BURN1).AND.(.NOT.DERIV)) THEN
*        TIME AVERAGE OF THE MACROLIB IS REQUIRED.
         DO 110 JGR=1,NGRP
         TOTAL(JGR)=0.0
         ZNUG(JGR)=0.0
         SNUGF(JGR)=0.0
         CHI(JGR)=0.0
         OVERV(JGR)=0.0
         DIFFX(JGR)=0.0
         DIFFY(JGR)=0.0
         DIFFZ(JGR)=0.0
         H(JGR)=0.0
         FLUX(JGR)=0.0
         DO 110 IGR=1,NGRP
         DO 110 IL=1,NL
         SCAT(IL,IGR,JGR)=0.0
  110    CONTINUE
         DO 130 I=1,NBURN-1
         IF((BURN0.LT.BURNUP(I+1)).AND.(BURN1.GT.BURNUP(I))) THEN
            A=MAX(BURN0,BURNUP(I))
            B=MIN(BURN1,BURNUP(I+1))
            CC=0.5*(B-A)/(BURN1-BURN0)
            DX=(BURNUP(I+1)-BURNUP(I))
            U1=(A-0.5*(BURNUP(I)+BURNUP(I+1)))/DX
            U2=(B-0.5*(BURNUP(I)+BURNUP(I+1)))/DX
            UU(1)=0.5*(-(U2-U1)*0.577350269189626+U1+U2)
            UU(2)=0.5*((U2-U1)*0.577350269189626+U1+U2)
            DO 120 J=1,2
            H1=3.0*(0.5-UU(J))**2-2.0*(0.5-UU(J))**3
            H2=(0.5-UU(J))**2-(0.5-UU(J))**3
            H3=3.0*(0.5+UU(J))**2-2.0*(0.5+UU(J))**3
            H4=-(0.5+UU(J))**2+(0.5+UU(J))**3
            DO 120 JGR=1,NGRP
            TOTAL(JGR)=TOTAL(JGR)+(H1*ZTOTAL(I,JGR,1)+
     1      H2*ZTOTAL(I,JGR,2)*DX+H3*ZTOTAL(I+1,JGR,1)+
     2      H4*ZTOTAL(I+1,JGR,2)*DX)*CC
            ZNUG(JGR)=ZNUG(JGR)+(H1*ZZNUG(I,JGR,1)+
     1      H2*ZZNUG(I,JGR,2)*DX+H3*ZZNUG(I+1,JGR,1)+
     2      H4*ZZNUG(I+1,JGR,2)*DX)*CC
            SNUGF(JGR)=SNUGF(JGR)+(H1*ZNUGF(I,JGR,1)+
     1      H2*ZNUGF(I,JGR,2)*DX+H3*ZNUGF(I+1,JGR,1)+
     2      H4*ZNUGF(I+1,JGR,2)*DX)*CC
            CHI(JGR)=CHI(JGR)+(H1*ZCHI(I,JGR,1)+
     1      H2*ZCHI(I,JGR,2)*DX+H3*ZCHI(I+1,JGR,1)+
     2      H4*ZCHI(I+1,JGR,2)*DX)*CC
            CHI(JGR)=ZCHI(I,JGR,1)
            OVERV(JGR)=OVERV(JGR)+(H1*ZOVERV(I,JGR,1)+
     1      H2*ZOVERV(I,JGR,2)*DX+H3*ZOVERV(I+1,JGR,1)+
     2      H4*ZOVERV(I+1,JGR,2)*DX)*CC
            DIFFX(JGR)=DIFFX(JGR)+(H1*ZDIFFX(I,JGR,1)+
     1      H2*ZDIFFX(I,JGR,2)*DX+H3*ZDIFFX(I+1,JGR,1)+
     2      H4*ZDIFFX(I+1,JGR,2)*DX)*CC
            DIFFY(JGR)=DIFFY(JGR)+(H1*ZDIFFY(I,JGR,1)+
     1      H2*ZDIFFY(I,JGR,2)*DX+H3*ZDIFFY(I+1,JGR,1)+
     2      H4*ZDIFFY(I+1,JGR,2)*DX)*CC
            DIFFZ(JGR)=DIFFZ(JGR)+(H1*ZDIFFZ(I,JGR,1)+
     1      H2*ZDIFFZ(I,JGR,2)*DX+H3*ZDIFFZ(I+1,JGR,1)+
     2      H4*ZDIFFZ(I+1,JGR,2)*DX)*CC
            H(JGR)=H(JGR)+(H1*ZH(I,JGR,1)+H2*ZH(I,JGR,2)*DX+
     1      H3*ZH(I+1,JGR,1)+H4*ZH(I+1,JGR,2)*DX)*CC
            FLUX(JGR)=FLUX(JGR)+(H1*ZFLUX(I,JGR,1)+
     1      H2*ZFLUX(I,JGR,2)*DX+H3*ZFLUX(I+1,JGR,1)+
     2      H4*ZFLUX(I+1,JGR,2)*DX)*CC
            DO 120 IGR=1,NGRP
            DO 120 IL=1,NL
            SCAT(IL,IGR,JGR)=SCAT(IL,IGR,JGR)+(H1*ZSCAT(I,IL,IGR,JGR,1)+
     1      H2*ZSCAT(I,IL,IGR,JGR,2)*DX+H3*ZSCAT(I+1,IL,IGR,JGR,1)+
     2      H4*ZSCAT(I+1,IL,IGR,JGR,2)*DX)*CC
  120       CONTINUE
         ENDIF
  130 CONTINUE
      ENDIF
      RETURN
      END
