*DECK CREMAC
      SUBROUTINE CREMAC (JPLIST,NISO,NGRP,NL,IMPX,HISO,DENSIT,TOTAL,
     1 ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,FLUX,IJJ,NJJ,
     2 ENGFIS,INDXS,WORK1,WORK2,UPS)
*
*-----------------------------------------------------------------------
*
* ADD THE MICROSCOPIC X-SECTIONS OF THE EXTRACTED ISOTOPES TO THE
* MACROSCOPIC RESIDUAL.
*
* INPUT PARAMETERS:
*  JPLIST : ADDRESS OF THE COMPO LINKED LIST OR XSM FILE.
*  NISO   : 1+NUMBER OF EXTRACTED ISOTOPES.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NL     : NUMBER OF LEGENDRE ORDERS (=1 FOR ISOTROPIC SCATTERING).
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*  HISO   : HOLLERITH NAME INFORMATION FOR EXTRACTED ISOTOPES.
*           DIMENSION HISO(3*NISO)
*  DENSIT : NUMBER DENSITIES. DIMENSION DENSIT(NISO)
*  UPS    :  =.TRUE.: NO UPSCATERING CROSS SECTIONS WILL BE STORED.
*  INDXS  : INDICATOR VECTOR OF THE SAVED PROPERTIES.
*           DIMENSION INDXS(21+NL)
*
* OUTPUT PARAMETERS:
*  TOTAL  : TOTAL MACROSCOPIC X-SECTIONS. DIMENSION TOTAL(NGRP)
*  ZNUG   : NU * FISSION MACROSCOPIC X-SECTIONS. DIMENSION ZNUG(NGRP)
*  SNUGF   : FISSION MACROSCOPIC X-SECTIONS. DIMENSION SNUGF(NGRP)
*  CHI    : FISSION SPECTRUM. DIMENSION CHI(NGRP)
*  OVERV  : RECIPROCAL NEUTRON VELOCITIES. DIMENSION OVERV(NGRP)
*  DIFFX  : X-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFX(NGRP)
*  DIFFY  : Y-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFY(NGRP)
*  DIFFZ  : Z-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFZ(NGRP)
*  H      : H-FACTORS (KAPPA * FISSION MACROSCOPIC X-SECTIONS).
*           DIMENSION H(NGRP)
*  SCAT   : SCATTERING MACROSCOPIC X-SECTIONS.
*           DIMENSION SCAT(NL,NGRP,NGRP)
*  FLUX   : INTEGRATED FLUXES. DIMENSION FLUX(NGRP)
*
* SCRATCH STORAGE:
*  IJJ    : DIMENSION IJJ(NGRP)
*  NJJ    : DIMENSION NJJ(NGRP)
*  WORK1  : DIMENSION WORK1(NGRP,3)
*  WORK2  : DIMENSION WORK2(NGRP*NGRP)
*
*-------------------------------------- AUTHOR: A. HEBERT ; 21/02/94 ---
*
      CHARACTER HMICRO*12,CM*2
      LOGICAL LFISS,UPS
      INTEGER  IJJ,NJJ
      INTEGER  INDXS(*)
      DIMENSION HISO(*),DENSIT(*),TOTAL(*),ZNUG(*),SNUGF(*),CHI(*),
     1 OVERV(*),DIFFX(*),DIFFY(*),DIFFZ(*),H(*),SCAT(NL,NGRP,*),
     2 FLUX(*),IJJ(*),NJJ(*),WORK1(NGRP,*),WORK2(*),
     3 ENGFIS(*)
*
* RECOVER MACROSCOPIC RESIDUAL OF VECTORIAL X-SECTIONS.
      DO 10 IGR=1,NGRP
        TOTAL(IGR)=0.0
        DIFFX(IGR)=0.0
        DIFFY(IGR)=0.0
        DIFFZ(IGR)=0.0
        ZNUG(IGR)=0.0
        SNUGF(IGR)=0.0
        CHI(IGR)=0.0
   10 CONTINUE
      CALL LCMGET(JPLIST,'FLUX-INTG',FLUX)
      CALL LCMGET(JPLIST,'OVERV',OVERV)
      CALL LCMGET(JPLIST,'ISOTOPES-EFJ',ENGFIS)
      CALL LCMSIX(JPLIST,'MACR',1)
      CALL LCMGET(JPLIST,'XS-SAVED',INDXS)
      IF( INDXS(1).EQ.1 )
     +   CALL LCMGET(JPLIST,'TOTAL',TOTAL)
      IF( INDXS(18).EQ.1 )THEN
         CALL LCMGET(JPLIST,'STRD X',DIFFX)
         CALL LCMGET(JPLIST,'STRD Y',DIFFY)
         CALL LCMGET(JPLIST,'STRD Z',DIFFZ)
      ELSEIF( INDXS(17).EQ.1 )THEN
         CALL LCMGET(JPLIST,'STRD',DIFFX)
      ENDIF
      IF( INDXS(3).EQ.1 )THEN
         CALL LCMGET(JPLIST,'NUSIGF',ZNUG)
         CALL LCMGET(JPLIST,'NFTOT',SNUGF)
         CALL LCMGET(JPLIST,'CHI',CHI)
      ENDIF
      DO 11 IGR=1, NGRP
        H(IGR) = ENGFIS(1)*SNUGF(IGR)
   11 CONTINUE	
      CALL LCMSIX(JPLIST,' ',2)
*
* RECOVER MICROSCOPIC CONTRIBUTIONS OF VECTORIAL X-SECTIONS.
      LFISS=.FALSE.
      DO 40 ISO=2,NISO
        IF(DENSIT(ISO).EQ.0.0) GO TO 40
        WRITE(HMICRO,'(3A4)') (HISO((ISO-1)*3+I),I=1,3)
        CALL LCMLEN(JPLIST,HMICRO,ILENG,ITYLCM)
        IF(ILENG.EQ.0) GO TO 40
        IF(IMPX.GT.1) WRITE(6,'(/29H CREMAC: PROCESSING ISOTOPE '',A12,
     1  16H'' WITH DENSITY =,1P,E13.5,2H .)') HMICRO,DENSIT(ISO)
        CALL LCMSIX(JPLIST,HMICRO,1)
        CALL LCMGET(JPLIST,'XS-SAVED',INDXS)
  	IF( INDXS(1).EQ.1 )THEN
          CALL LCMGET(JPLIST,'TOTAL',WORK1(1,1))
          DO 20 IGR=1,NGRP
            TOTAL(IGR)=TOTAL(IGR)+DENSIT(ISO)*WORK1(IGR,1)
   20     CONTINUE
        ENDIF
        IF( INDXS(18).EQ.1 )THEN
          CALL LCMGET(JPLIST,'STRD X',WORK1(1,1))
          CALL LCMGET(JPLIST,'STRD Y',WORK1(1,2))
          CALL LCMGET(JPLIST,'STRD Z',WORK1(1,3))
        ELSEIF( INDXS(17).EQ.1 )THEN
          CALL LCMGET(JPLIST,'STRD',WORK1(1,1))
        ENDIF
        IF( INDXS(17).EQ.1 )THEN
         DO 21 IGR=1,NGRP
           DIFFX(IGR)=DIFFX(IGR)+DENSIT(ISO)*WORK1(IGR,1)
	    IF( INDXS(18).EQ.1 )THEN
              DIFFY(IGR)=DIFFY(IGR)+DENSIT(ISO)*WORK1(IGR,2)
              DIFFZ(IGR)=DIFFZ(IGR)+DENSIT(ISO)*WORK1(IGR,3)
	    ENDIF
   21    CONTINUE
        ENDIF
        IF( INDXS(3).EQ.1 )THEN
         CALL LCMGET(JPLIST,'NUSIGF',WORK1(1,1))
         CALL LCMGET(JPLIST,'NFTOT',WORK1(1,2))
         CALL LCMGET(JPLIST,'CHI',WORK1(1,3))
         DO 30 IGR=1,NGRP
           LFISS=LFISS.OR.(CHI(IGR).NE.WORK1(IGR,3))
           ZNUG(IGR)=ZNUG(IGR)+DENSIT(ISO)*WORK1(IGR,1)
           SNUGF(IGR)=SNUGF(IGR)+DENSIT(ISO)*WORK1(IGR,2)
           H(IGR)=H(IGR)+DENSIT(ISO)*WORK1(IGR,2)*ENGFIS(ISO)
   30    CONTINUE
        ENDIF
        CALL LCMSIX(JPLIST,' ',2)
   40 CONTINUE
*
* COMPUTE AN AVERAGE FISSION SPECTRUM.
      IF(LFISS) THEN
        CALL LCMGET(JPLIST,'FLUX-INTG',WORK1(1,1))
        CALL LCMSIX(JPLIST,'MACR',1)
	CALL LCMGET(JPLIST,'XS-SAVED',INDXS)
        IF( INDXS(3).EQ.1 )THEN
          CALL LCMGET(JPLIST,'NUSIGF',WORK1(1,2))
          CALL LCMGET(JPLIST,'CHI',WORK1(1,3))
          DO 50 JGR=1,NGRP
           DO 50 IGR=1,NGRP
             SCAT(1,IGR,JGR)=WORK1(IGR,1)*WORK1(IGR,2)*WORK1(JGR,3)
   50     CONTINUE
        ELSE
          DO 60 JGR=1,NGRP
           DO 60 IGR=1,NGRP
            SCAT(1,IGR,JGR)=0.0
   60     CONTINUE
        ENDIF
        CALL LCMSIX(JPLIST,' ',2)
        DO 80 ISO=2,NISO
          IF(DENSIT(ISO).EQ.0.0) GO TO 80
          WRITE(HMICRO,'(3A4)') (HISO((ISO-1)*3+I),I=1,3)
          CALL LCMLEN(JPLIST,HMICRO,ILENG,ITYLCM)
          IF(ILENG.EQ.0) GO TO 80
          CALL LCMSIX(JPLIST,HMICRO,1)
          CALL LCMGET(JPLIST,'XS-SAVED',INDXS)
          IF( INDXS(3).EQ.1 )THEN
            CALL LCMGET(JPLIST,'NUSIGF',WORK1(1,2))
            CALL LCMGET(JPLIST,'CHI',WORK1(1,3))
            DO 70 JGR=1,NGRP
             DO 70 IGR=1,NGRP
              SCAT(1,IGR,JGR)=SCAT(1,IGR,JGR)+DENSIT(ISO)*
     1        WORK1(IGR,1)*WORK1(IGR,2)*WORK1(JGR,3)
   70       CONTINUE
          ENDIF
          CALL LCMSIX(JPLIST,' ',2)
   80   CONTINUE
        SSUM=0.0
        DO 90 JGR=1,NGRP
         CHI(JGR)=0.0
         DO 90 IGR=1,NGRP
           SSUM=SSUM+SCAT(1,IGR,JGR)
           CHI(JGR)=CHI(JGR)+SCAT(1,IGR,JGR)
   90   CONTINUE
        DO 100 JGR=1,NGRP
          CHI(JGR)=CHI(JGR)/SSUM
  100   CONTINUE
      ENDIF
*
* RECOVER MACROSCOPIC RESIDUAL OF SCATTERING X-SECTIONS.
      CALL LCMSIX(JPLIST,'MACR',1)
      CALL LCMGET(JPLIST,'XS-SAVED',INDXS)
      DO 130 IL=1,NL
       DO 110 JGR=1,NGRP
        DO 110 IGR=1,NGRP
          SCAT(IL,IGR,JGR)=0.0
  110  CONTINUE
       WRITE (CM,'(I2)') IL-1
       IF( INDXS(20+IL).EQ.1 )THEN
         CALL LCMGET(JPLIST,'SCAT'//CM,WORK2)
         CALL LCMGET(JPLIST,'NJJ '//CM,NJJ)
         CALL LCMGET(JPLIST,'IJJ '//CM,IJJ)
         IGAR=0
         DO 120 JGR=1,NGRP
          DO 120 IGR=IJJ(JGR),IJJ(JGR)-NJJ(JGR)+1,-1
           IGAR=IGAR+1
           SCAT(IL,IGR,JGR)=WORK2(IGAR)
  120    CONTINUE
       ENDIF
  130 CONTINUE
      CALL LCMSIX(JPLIST,' ',2)
*
* RECOVER MICROSCOPIC CONTRIBUTIONS OF SCATTERING X-SECTIONS.
      DO 160 ISO=2,NISO
        IF(DENSIT(ISO).EQ.0.0) GO TO 160
        WRITE(HMICRO,'(3A4)') (HISO((ISO-1)*3+I),I=1,3)
        CALL LCMLEN(JPLIST,HMICRO,ILENG,ITYLCM)
        IF(ILENG.EQ.0) GO TO 160
        CALL LCMSIX(JPLIST,HMICRO,1)
        CALL LCMGET(JPLIST,'XS-SAVED',INDXS)
        DO 150 IL=1,NL
          WRITE (CM,'(I2)') IL-1
          IF( INDXS(20+IL).EQ.1 )THEN
            CALL LCMGET(JPLIST,'SCAT'//CM,WORK2)
            CALL LCMGET(JPLIST,'NJJ '//CM,NJJ)
            CALL LCMGET(JPLIST,'IJJ '//CM,IJJ)
            IGAR=0
            DO 140 JGR=1,NGRP
             DO 140 IGR=IJJ(JGR),IJJ(JGR)-NJJ(JGR)+1,-1
               IGAR=IGAR+1
               SCAT(IL,IGR,JGR)=SCAT(IL,IGR,JGR)
     1                          +DENSIT(ISO)*WORK2(IGAR)
  140       CONTINUE
          ENDIF
  150   CONTINUE
        CALL LCMSIX(JPLIST,' ',2)
  160 CONTINUE
*
* COMPUTE DIRECTIONAL DIFFUSION COEFFICIENTS FROM STRD X-SECTIONS.
      CALL LCMSIX(JPLIST,'MACR',1)
      CALL LCMGET(JPLIST,'XS-SAVED',INDXS)
      CALL LCMSIX(JPLIST,' ',2)
      IF( INDXS(17).EQ.1 )THEN
        DO 170 IGR=1,NGRP
          DIFFX(IGR)=1.0/(3.0*DIFFX(IGR))
	  IF( INDXS(18).EQ.1 )THEN
            DIFFY(IGR)=1.0/(3.0*DIFFY(IGR))
            DIFFZ(IGR)=1.0/(3.0*DIFFZ(IGR))
          ELSE
            DIFFY(IGR)=DIFFX(IGR)
            DIFFZ(IGR)=DIFFX(IGR)
          ENDIF
  170   CONTINUE
      ENDIF
* COMPUTE TOTAL CROSS SECTION FOR UPSCATERING CORRECTION
* EV 11/08/95
      IF ((UPS).AND.(NGRP.EQ.2)) THEN
       DO 200 IL=1,NL
        TOTAL(2) = TOTAL(2) - SCAT(IL,2,1)
        SCAT(IL,2,1) = 0.0
  200  CONTINUE
      ENDIF
      RETURN
      END
