*DECK CRERGR
      SUBROUTINE CRERGR(IPRES,JPLIST,DERIV,KFUEL,IBM,
     1   NMIXT,NBURN,KBURN,BURN0,BURN1,NGRP,NL,IMPX,
     2   TOTAL,ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,
     3   IJJ,NJJ,FLUX,ZTOTAL,ZZNUG,ZNUGF,ZCHI,ZOVERV,ZDIFFX,
     4   ZDIFFY,ZDIFFZ,ZH,ZSCAT,ZFLUX,BURNUP,INDEX,
     5   BZC,ZONEB,WINT,VOLUME,NW,NCH,NK,NZC)
*
*-----------------------------------------------------------------------
*
* COMPUTE FUEL PORPERTIES WITH RESPECT TO BURNUP.
*
* INPUT PARAMETERS:
*  JPLIST : ADDRESS OF THE COMPO LINKED LIST OR XSM FILE.
*  IPRES  : ADDRESS OF THE FUEL MAP LINKED LIST OR XSM FILE.
*  NMIXT  : NUMBER OF MATERIAL MIXTURES.
*  IBM    : NUMBER OF THE FUEL TYPE TO TREAT.
*  DERIV  : =.TRUE.: DERIVATIVE OF MACROLIB INFO IS COMPUTED WITH
*           RESPECT TO BURN1.
*  NBURN  : NUMBER OF TABULATED BURNUP STEPS.
*  KBURN  : =0: NO BURNUP PARAMETERS; =1: USE MW DAY/TONNE OF INITIAL
*           HEAVY ELEMENTS); =2: USE N/KB.
*  KFUEL  : TYPE OF FUEL PROPERTY CALCULATION
*  BURN0  : USER DEFINED INITIAL BURNUP OR NEUTRON EXPOSURE.
*  BURN1  : USER DEFINED FINAL BURNUP OR NEUTRON EXPOSURE.
*           IF BURN0=BURN1, A SIMPLE INTERPOLATION IS PERFORMED.
*           IF BURN0.LT.BURN1, A TIME AVERAGE CALCULATION IS PERFORMED.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NL     : NUMBER OF LEGENDRE ORDERS (=1 FOR ISOTROPIC SCATTERING).
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*  HISO   : HOLLERITH NAME INFORMATION FOR EXTRACTED ISOTOPES.
*           DIMENSION HISO(3*NISO)
*  ITY    : =0: DO NOT PROCESS THE ISOTOPE; =1: USE NUMBER DENSITY
*           STORED IN CONC(I); =2: USE NUMBER DENSITY STORED IN COMPO.
*           DIMENSION ITY(NISO)
*  CONC   : USER DEFINED NUMBER DENSITY. DIMENSION CONC(NISO)
*  NZC    : NUMBER OF FUEL COMBUSTION ZONES.
*  NCH    : NUMBER OF FUEL CHANNELS.
*  NK     : NUMBER OF FUEL BUNDLES PER CHANNEL.
*  NIND   : NUMBER OF MIXTURES IN L_MAP.
*  BURNUP : BURNUP OR IRRADIATION STEPS. DIMENSION BURNUP(NBURN)
*  INDEX  : FUEL TYPE NUMBER. DIMENSION INDEX(*)
*  BZC    : AVERAGE EXIT BURNUP PER COMBUSTION ZONES. DIMENSION BZC(NZC)
*  ZONEB  : COMBUSTION ZONE INDEX. DIMENSION ZONEB(NCH)
*  WINT   : FUEL BURNUP DISTRIBUTIONS.
*           DIMENSION WINT(NZC*NK,2) OR WINT(NCH*NK,2)
*
* OUTPUT PARAMETERS:
*  TOTAL  : TOTAL MACROSCOPIC X-SECTIONS. DIMENSION TOTAL(NMIXT,NGRP)
*  ZNUG   : NU * FISSION MACROSCOPIC X-SECTIONS.
*            DIMENSION ZNUG(NMIXT,NGRP)
*  SNUGF   : FISSION MACROSCOPIC X-SECTIONS.
*            DIMENSION SNUGF(NMIXT,NGRP)
*  CHI    : FISSION SPECTRUM. DIMENSION CHI(NMIXT,NGRP)
*  OVERV  : RECIPROCAL NEUTRON VELOCITIES. DIMENSION OVERV(NMIXT,NGRP)
*  DIFFX  : X-DIRECTED DIFFUSION COEFFICIENTS.
*           DIMENSION DIFFX(NMIXT,NGRP)
*  DIFFY  : Y-DIRECTED DIFFUSION COEFFICIENTS.
*           DIMENSION DIFFY(NMIXT,NGRP)
*  DIFFZ  : Z-DIRECTED DIFFUSION COEFFICIENTS.
*           DIMENSION DIFFZ(NMIXT,NGRP)
*  H      : H-FACTORS (KAPPA * FISSION MACROSCOPIC X-SECTIONS).
*           DIMENSION H(NMIXT,NGRP)
*  SCAT   : SCATTERING MACROSCOPIC X-SECTIONS.
*           DIMENSION SCAT(NMIXT,NL,NGRP,NGRP)
*  IJJ    : DIMENSION IJJ(NMIXT,NL,NGRP)
*  NJJ    : DIMENSION NJJ(NMIXT,NL,NGRP)
*  VOLUME : MERGED VOLUME. DIMENSION VOLUME(NMIXT)
*  FLUX   : INTEGRATED FLUXES. DIMENSION FLUX(NMIXT,NGRP)
*
* SCRATCH STORAGE:
*  ZTOTAL  : DIMENSION ZTOTAL(2*NGRP*NBURN)
*  ZZNUG   : DIMENSION ZZNUG(2*NGRP*NBURN)
*  ZNUGF   : DIMENSION ZNUGF(2*NGRP*NBURN)
*  ZCHI    : DIMENSION ZCHI(2*NGRP*NBURN)
*  ZOVERV  : DIMENSION ZOVERV(2*NGRP*NBURN)
*  ZDIFFX  : DIMENSION ZDIFFX(2*NGRP*NBURN)
*  ZDIFFY  : DIMENSION ZDIFFY(2*NGRP*NBURN)
*  ZDIFFZ  : DIMENSION ZDIFFZ(2*NGRP*NBURN)
*  ZH      : DIMENSION ZH(2*NGRP*NBURN)
*  ZSCAT   : DIMENSION ZSCAT(2*NL*NGRP*NGRP*NBURN)
*  ZFLUX   : DIMENSION ZFLUX(2*NGRP*NBURN)
*
*-------------------------------------- AUTHOR: E. VARIN 11/09/95 ---
C
*
*      IMPLICIT NONE
*
      INTEGER IPRES,JPLIST
      INTEGER NW,NCH,NK,NZC,KFUEL,IBM,NMIXT,NBURN,KBURN,NGRP,NL,IMPX,
     1        IDIM(4),ITAB
* LOCAL
      INTEGER I,JMIX,J,IW,IRES,ICH,IBUND,IGR,JGR,IL,IGMAX,IGMIN,IZNUG,
     1  ITOTAL,INUGF,IDIFFX,IDIFFY,IDIFFZ,ICHI,IH,IOVERV,ISCAT,IFLUX,IP
*
      REAL    TOTAL(NMIXT,*),ZNUG(NMIXT,*),SNUGF(NMIXT,*),CHI(NMIXT,*),
     1 OVERV(NMIXT,*),DIFFX(NMIXT,*),DIFFY(NMIXT,*),DIFFZ(NMIXT,*),
     2 H(NMIXT,*),SCAT(NMIXT,NL,NGRP,*),FLUX(NMIXT,*),BURNUP(*),
     3 WINT(NW,*),VOLUME(*),BZC(*),ZTOTAL(NBURN,NGRP,*),
     4 ZZNUG(NBURN,NGRP,*),ZNUGF(NBURN,NGRP,*),
     5 ZCHI(NBURN,NGRP,*),ZOVERV(NBURN,NGRP,*),ZDIFFX(NBURN,NGRP,*),
     6 ZDIFFY(NBURN,NGRP,*),ZDIFFZ(NBURN,NGRP,*),ZH(NBURN,NGRP,*),
     7 ZSCAT(NBURN,NL,NGRP,NGRP,*),ZFLUX(NBURN,NGRP,*)
      REAL    BURN0,BURN1,VOL
      LOGICAL DERIV
      INTEGER INDEX(*),ZONEB(*),IJJ(NMIXT,NL,*),
     1  NJJ(NMIXT,NL,*),ILONG,ITYP
      DIMENSION  IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*
*
*     RECUPERATION DES INFORMATIONS DANS RESEAU
*
      CALL LCMGET(IPRES,'ZONE',ZONEB)
      CALL LCMGET(IPRES,'INDEX',INDEX)
      CALL LCMGET(IPRES,'BURN-ZONE',BZC)
      DO 1 I=1,2
       DO 1 J=1,NW
         WINT(J,I) = 0.0
  1   CONTINUE
      IF((KFUEL.EQ.4).OR.(KFUEL.EQ.5)) THEN
       CALL LCMGET(IPRES,'BURN-DEB',WINT(1,1))
      ELSEIF((KFUEL.EQ.2).OR.(KFUEL.EQ.3)) THEN
       CALL LCMGET(IPRES,'BURN-DEB',WINT(1,1))
       CALL LCMGET(IPRES,'BURN-END',WINT(1,2))
      ENDIF
      CALL LCMGET(JPLIST,'VOLUME',VOL)
      IF (KBURN.EQ.1) THEN
       CALL LCMLEN(JPLIST,'BURNUP',ILONG,ITYP)
       IF(ILONG.EQ.0) CALL XABORT('CRERGR: BLOCK BURNUP DOES NOT EXIST'
     1  //' CHECK OPTION IN CRE: OR IN XSCONS: MODULES')
       CALL LCMGET(JPLIST,'BURNUP',BURNUP)
      ELSE
       CALL LCMLEN(JPLIST,'N/KB',ILONG,ITYP)
       IF(ILONG.EQ.0) CALL XABORT('CRERGR: BLOCK N/KB DOES NOT EXIST'
     1  //' CHECK OPTION IN CRE: OR IN XSCONS: MODULES')
       CALL LCMGET(JPLIST,'N/KB',BURNUP)
      ENDIF
*
*     RECUPERATION DES TABLES DE PROPRIETES
*
      CALL LCMGET(JPLIST,'TOTAL',ZTOTAL)
      CALL LCMGET(JPLIST,'NUSIGF',ZZNUG)
      CALL LCMGET(JPLIST,'NFTOT',ZNUGF)
      CALL LCMGET(JPLIST,'CHI',ZCHI)
      CALL LCMGET(JPLIST,'OVERV',ZOVERV)
      CALL LCMGET(JPLIST,'DIFFX',ZDIFFX)
      CALL LCMGET(JPLIST,'DIFFY',ZDIFFY)
      CALL LCMGET(JPLIST,'DIFFZ',ZDIFFZ)
      CALL LCMGET(JPLIST,'H-FACTORS',ZH)
      CALL LCMGET(JPLIST,'FLUX-INTG',ZFLUX)
      CALL LCMGET(JPLIST,'SCAT',ZSCAT)
      CALL LCMGET(JPLIST,'IDIM',IDIM)
      IF( IDIM(4).EQ.1 ) THEN
        WRITE(6,*) ' TABLE FROM AFM: '
        DO 20 IGR=1,NGRP
          DO 20 I=1,NBURN
            ZTOTAL(I,IGR,2)=ZTOTAL(I,IGR,1)
            ZZNUG(I,IGR,2)=ZZNUG(I,IGR,1)
            ZNUGF(I,IGR,2)=ZNUGF(I,IGR,1)
            ZCHI(I,IGR,2)=ZCHI(I,IGR,1)
            ZOVERV(I,IGR,2)=ZOVERV(I,IGR,1)
            ZDIFFX(I,IGR,2)=ZDIFFX(I,IGR,1)
            ZDIFFY(I,IGR,2)=ZDIFFY(I,IGR,1)
            ZDIFFZ(I,IGR,2)=ZDIFFZ(I,IGR,1)
            ZFLUX(I,IGR,2)=ZFLUX(I,IGR,1)
            ZH(I,IGR,2)=ZH(I,IGR,1)
            DO 20 JGR=1,NGRP
              DO 20 IL=1,NL
                ZSCAT(I,IL,JGR,IGR,2)=ZSCAT(I,IL,JGR,IGR,1)
   20   CONTINUE
        CALL SETARA(BASE,2*NBURN,ITAB)
        DO 21 JGR=1,NGRP
           CALL ALDERV(NBURN,BURNUP,ZTOTAL(1,JGR,2),BASE(ITAB))
           CALL ALDERV(NBURN,BURNUP,ZZNUG(1,JGR,2),BASE(ITAB))
           CALL ALDERV(NBURN,BURNUP,ZNUGF(1,JGR,2),BASE(ITAB))
           CALL ALDERV(NBURN,BURNUP,ZCHI(1,JGR,2),BASE(ITAB))
           CALL ALDERV(NBURN,BURNUP,ZOVERV(1,JGR,2),BASE(ITAB))
           CALL ALDERV(NBURN,BURNUP,ZDIFFX(1,JGR,2),BASE(ITAB))
           CALL ALDERV(NBURN,BURNUP,ZDIFFY(1,JGR,2),BASE(ITAB))
           CALL ALDERV(NBURN,BURNUP,ZDIFFZ(1,JGR,2),BASE(ITAB))
           CALL ALDERV(NBURN,BURNUP,ZH(1,JGR,2),BASE(ITAB))
           CALL ALDERV(NBURN,BURNUP,ZFLUX(1,JGR,2),BASE(ITAB))
           DO 30 IGR=1,NGRP
           DO 30 IL=1,NL
             CALL ALDERV(NBURN,BURNUP,ZSCAT(1,IL,IGR,JGR,2),
     1                   BASE(ITAB))
   30      CONTINUE
   21   CONTINUE
        CALL RLSARA(BASE(ITAB))
      ENDIF
*
*     CONSTRUCTION OF THE FUEL TYPE PROPERTIES WITH RESPECT TO BURNUP
*
      CALL SETARA(BASE,NGRP,ITOTAL)
      CALL SETARA(BASE,NGRP,IZNUG)
      CALL SETARA(BASE,NGRP,INUGF)
      CALL SETARA(BASE,NGRP,ICHI)
      CALL SETARA(BASE,NGRP,IOVERV)
      CALL SETARA(BASE,NGRP,IDIFFX)
      CALL SETARA(BASE,NGRP,IDIFFY)
      CALL SETARA(BASE,NGRP,IDIFFZ)
      CALL SETARA(BASE,NGRP,IH)
      CALL SETARA(BASE,NL*NGRP*NGRP,ISCAT)
      CALL SETARA(BASE,NGRP,IFLUX)
      DO 5 I=1,NCH*NK
        IF (INDEX(I).EQ.IBM) THEN
* FUEL
          IRES = I
          IP = (IRES-1)/NCH
          ICH = IRES - IP*NCH
          IF(KFUEL.EQ.1) THEN
* HOMOGENEOUS MODEL
            BURN0 = 0.0
            BURN1 = BZC(ZONEB(ICH))
          ELSEIF(KFUEL.EQ.3 .OR. KFUEL.EQ.2 ) THEN
            IW = IRES
            BURN0 = WINT(IW,1)
            BURN1 = WINT(IW,2)
          ELSEIF(KFUEL.EQ.5) THEN
            IBUND = IRES - ICH*NK
            IW = ZONEB(ICH)*IBUND
            BURN0 = WINT(IW,1)
            BURN1 = BURN0
          ELSEIF(KFUEL.EQ.4) THEN
            IW = IRES
            BURN0 = WINT(IW,1)
            BURN1 = BURN0
          ENDIF
          IF(BURN1.GT.BURNUP(NBURN)) THEN
             CALL XABORT('CRERGR: INTERPOLATION IS OUT OF BOUNDS, '
     1       //'MAYBE L_MAP DOES NOT CONTAIN THE RIGHT VALUES '
     2       //'IRRADIATION OR BURNUP')
          ENDIF
          JMIX = I
* CALCULATION OF SIGMA
          CALL CREITP(NGRP,NL,DERIV,NBURN,
     1     BURN0,BURN1,BURNUP,BASE(ITOTAL),BASE(IZNUG),
     2     BASE(INUGF),BASE(ICHI),BASE(IOVERV),
     3     BASE(IDIFFX),BASE(IDIFFY),BASE(IDIFFZ),BASE(IH),BASE(ISCAT),
     4     BASE(IFLUX),ZTOTAL,ZZNUG,ZNUGF,ZCHI,ZOVERV,ZDIFFX,ZDIFFY,
     5     ZDIFFZ,ZH,ZSCAT,ZFLUX)
          DO 110 JGR=1,NGRP
           TOTAL(JMIX,JGR)=BASE(ITOTAL+JGR-1)
           ZNUG(JMIX,JGR)=BASE(IZNUG+JGR-1)
           SNUGF(JMIX,JGR)=BASE(INUGF+JGR-1)
           CHI(JMIX,JGR)=BASE(ICHI+JGR-1)
           OVERV(JMIX,JGR)=BASE(IOVERV+JGR-1)
           DIFFX(JMIX,JGR)=BASE(IDIFFX+JGR-1)
           DIFFY(JMIX,JGR)=BASE(IDIFFY+JGR-1)
           DIFFZ(JMIX,JGR)=BASE(IDIFFZ+JGR-1)
           H(JMIX,JGR)=BASE(IH+JGR-1)
           FLUX(JMIX,JGR)=BASE(IFLUX+JGR-1)
           DO 110 IGR=1,NGRP
           DO 110 IL=1,NL
            SCAT(JMIX,IL,IGR,JGR)=
     1            BASE(ISCAT+NL*((JGR-1)*NGRP+IGR-1)+IL-1)
 110      CONTINUE
*         JGR IS THE SECONDARY GROUP.
          DO 130 JGR=1,NGRP
          DO 130 IL=1,NL
            IGMIN=JGR
            IGMAX=JGR
            DO 120 IGR=NGRP,1,-1
             IF (SCAT(JMIX,IL,IGR,JGR).NE.0.0) THEN
              IGMIN=MIN(IGMIN,IGR)
              IGMAX=MAX(IGMAX,IGR)
             ENDIF
  120       CONTINUE
            IJJ(JMIX,IL,JGR)=IGMAX
  130       NJJ(JMIX,IL,JGR)=IGMAX-IGMIN+1
          VOLUME(JMIX) = VOL
        ENDIF
 5    CONTINUE
      CALL RLSARA(BASE(ITOTAL))
      CALL RLSARA(BASE(IFLUX))
      CALL RLSARA(BASE(ISCAT))
      CALL RLSARA(BASE(IH))
      CALL RLSARA(BASE(IDIFFZ))
      CALL RLSARA(BASE(IDIFFY))
      CALL RLSARA(BASE(IDIFFX))
      CALL RLSARA(BASE(IOVERV))
      CALL RLSARA(BASE(ICHI))
      CALL RLSARA(BASE(INUGF))
      CALL RLSARA(BASE(IZNUG))
      RETURN
      END
