*DECK CRETAB
      SUBROUTINE CRETAB (JPLIST,NISO,NGRP,NL,IMPX,HISO,NBURN,
     1 BURNUP,ITY,CONC,TOTAL,ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,
     2 DIFFZ,H,SCAT,FLUX,ZTOTAL,ZZNUG,ZNUGF,ZCHI,ZOVERV,ZDIFFX,
     3 ZDIFFY,ZDIFFZ,ZH,ZSCAT,ZFLUX,DENSIT,UPS)
*
*-----------------------------------------------------------------------
*
* CREATE A BURNUP DEPENDENT TABLE WITH THE EXTRACTED ISOTOPE
*
* INPUT PARAMETERS:
*  JPLIST : ADDRESS OF THE COMPO LINKED LIST OR XSM FILE.
*  NISO   : 1+NUMBER OF EXTRACTED ISOTOPES.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NL     : NUMBER OF LEGENDRE ORDERS (=1 FOR ISOTROPIC SCATTERING).
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*  HISO   : HOLLERITH NAME INFORMATION FOR EXTRACTED ISOTOPES.
*           DIMENSION HISO(3*NISO)
*  NBURN  : NUMBER OF TABULATED BURNUP STEPS.
*  BURNUP : BURNUP OR NEUTRON EXPOSURE TABULATION POINTS.
*           DIMENSION BURNUP(NBURN)
*  ITY    : =0: DO NOT PROCESS THE ISOTOPE; =1: USE NUMBER DENSITY
*           STORED IN CONC(I); =2: USE NUMBER DENSITY STORED IN COMPO.
*           DIMENSION ITY(NISO)
*  CONC   : USER DEFINED NUMBER DENSITY. DIMENSION CONC(NISO)
*
* INTERPOLATED OUTPUT PARAMETERS:
*  TOTAL  : TOTAL MACROSCOPIC X-SECTIONS. DIMENSION TOTAL(NGRP)
*  ZNUG   : NU * FISSION MACROSCOPIC X-SECTIONS. DIMENSION ZNUG(NGRP)
*  SNUGF   : FISSION MACROSCOPIC X-SECTIONS. DIMENSION SNUGF(NGRP)
*  CHI    : FISSION SPECTRUM. DIMENSION CHI(NGRP)
*  OVERV  : RECIPROCAL NEUTRON VELOCITIES. DIMENSION OVERV(NGRP)
*  DIFFX  : X-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFX(NGRP)
*  DIFFY  : Y-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFY(NGRP)
*  DIFFZ  : Z-DIRECTED DIFFUSION COEFFICIENTS. DIMENSION DIFFZ(NGRP)
*  H      : H-FACTORS (KAPPA * FISSION MACROSCOPIC X-SECTIONS).
*           DIMENSION H(NGRP)
*  SCAT   : SCATTERING MACROSCOPIC X-SECTIONS.
*           DIMENSION SCAT(NL,NGRP,NGRP)
*  FLUX   : INTEGRATED FLUX. DIMENSION FLUX(NGRP)
*
* SCRATCH STORAGE:
*  ZTOTAL : DIMENSION ZTOTAL(2*NGRP*NBURN)
*  ZZNUG  : DIMENSION ZZNUG(2*NGRP*NBURN)
*  ZNUGF  : DIMENSION ZNUGF(2*NGRP*NBURN)
*  ZCHI   : DIMENSION ZCHI(2*NGRP*NBURN)
*  ZOVERV : DIMENSION ZOVERV(2*NGRP*NBURN)
*  ZDIFFX : DIMENSION ZDIFFX(2*NGRP*NBURN)
*  ZDIFFY : DIMENSION ZDIFFY(2*NGRP*NBURN)
*  ZDIFFZ : DIMENSION ZDIFFZ(2*NGRP*NBURN)
*  ZH     : DIMENSION H(2*NGRP*NBURN)
*  ZSCAT  : DIMENSION ZSCAT(2*NL*NGRP*NGRP*NBURN)
*  ZFLUX  : DIMENSION ZFLUX(2*NGRP*NBURN)
*  DENSIT : DIMENSION DENSIT(NISO)
*
*-------------------------------------- AUTHOR: A. HEBERT ; 18/02/94 ---
*-------------------------- MODIFIED : E. VARIN ; 11/09/95 ---
*
      CHARACTER TEXT12*12
      LOGICAL UPS
      DIMENSION HISO(*),BURNUP(*),ITY(*),CONC(*),TOTAL(*),ZNUG(*),
     1 SNUGF(*),CHI(*),OVERV(*),DIFFX(*),DIFFY(*),DIFFZ(*),H(*),
     2 SCAT(NL,NGRP,*),FLUX(*),ZTOTAL(NBURN,NGRP,*),
     3 ZZNUG(NBURN,NGRP,*),ZNUGF(NBURN,NGRP,*),ZCHI(NBURN,NGRP,*),
     4 ZOVERV(NBURN,NGRP,*),ZDIFFX(NBURN,NGRP,*),ZDIFFY(NBURN,NGRP,*),
     5 ZDIFFZ(NBURN,NGRP,*),ZH(NBURN,NGRP,*),IBASE(1),
     6 ZSCAT(NBURN,NL,NGRP,NGRP,*),ZFLUX(NBURN,NGRP,*),DENSIT(*)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
* RECOVER MACROSCOPIC X-SECTION INFO FROM BURNUP DIRECTORIES:
      CALL SETARA(IBASE,NGRP,IIJJ)
      CALL SETARA(IBASE,NGRP,INJJ)
      CALL SETARA(BASE,NISO,IENG)
      CALL SETARA(IBASE,(21+NL),INDXS)
      CALL SETARA(BASE,3*NGRP,IWORK1)
      CALL SETARA(BASE,NGRP*NGRP,IWORK2)
      DO 20 IBURN=1,NBURN
        WRITE(TEXT12,'(4HBURN,4X,I4)') IBURN
        CALL LCMSIX(JPLIST,TEXT12,1)
        CALL LCMGET(JPLIST,'ISOTOPESDENS',DENSIT)
        IF(DENSIT(1).NE.1.0) CALL XABORT('CRETAB: DENSIT(1).NE.1.')
        DO 10 I=2,NISO
          IF(ITY(I).EQ.0) THEN
            DENSIT(I)=0.0
          ELSE IF(ITY(I).EQ.1) THEN
            DENSIT(I)=CONC(I)
          ELSE IF(ITY(I).NE.2) THEN
            CALL XABORT('CRETAB: INVALID VALUE OF ITY.')
          ENDIF
   10   CONTINUE
        CALL CREMAC(JPLIST,NISO,NGRP,NL,IMPX,HISO,DENSIT,TOTAL,
     1      ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,FLUX,
     2      IBASE(IIJJ),IBASE(INJJ),BASE(IENG),IBASE(INDXS),
     3      BASE(IWORK1),BASE(IWORK2),UPS)
        CALL LCMSIX(JPLIST,' ',2)
        DO 21 I=1,2
         DO 22 JGR=1,NGRP
           ZTOTAL(IBURN,JGR,I)=TOTAL(JGR)
           ZZNUG(IBURN,JGR,I)=ZNUG(JGR)
           ZNUGF(IBURN,JGR,I)=SNUGF(JGR)
           ZCHI(IBURN,JGR,I)=CHI(JGR)
           ZOVERV(IBURN,JGR,I)=OVERV(JGR)
           ZDIFFX(IBURN,JGR,I)=DIFFX(JGR)
           ZDIFFY(IBURN,JGR,I)=DIFFY(JGR)
           ZDIFFZ(IBURN,JGR,I)=DIFFZ(JGR)
           ZH(IBURN,JGR,I)=H(JGR)
           ZFLUX(IBURN,JGR,I)=FLUX(JGR)
           DO 23 IGR=1,NGRP
             DO 24 IL=1,NL
               ZSCAT(IBURN,IL,IGR,JGR,I)=SCAT(IL,IGR,JGR)
   24        CONTINUE
   23      CONTINUE
   22    CONTINUE
   21   CONTINUE
   20 CONTINUE
      CALL RLSARA(BASE(IWORK2))
      CALL RLSARA(BASE(IWORK1))
      CALL RLSARA(BASE(IENG))
      CALL RLSARA(IBASE(INDXS))
      CALL RLSARA(IBASE(INJJ))
      CALL RLSARA(IBASE(IIJJ))
*
* TAKE THE DERIVATIVE WITH RESPECT TO BURNUP OR NEUTRON EXPOSURE AT
* TABULATION POINTS.
      CALL SETARA(BASE,2*NBURN,IZGAR)
      DO 30 JGR=1,NGRP
        CALL ALDERV(NBURN,BURNUP,ZTOTAL(1,JGR,2),BASE(IZGAR))
        CALL ALDERV(NBURN,BURNUP,ZZNUG(1,JGR,2),BASE(IZGAR))
        CALL ALDERV(NBURN,BURNUP,ZNUGF(1,JGR,2),BASE(IZGAR))
        CALL ALDERV(NBURN,BURNUP,ZCHI(1,JGR,2),BASE(IZGAR))
        CALL ALDERV(NBURN,BURNUP,ZOVERV(1,JGR,2),BASE(IZGAR))
        CALL ALDERV(NBURN,BURNUP,ZDIFFX(1,JGR,2),BASE(IZGAR))
        CALL ALDERV(NBURN,BURNUP,ZDIFFY(1,JGR,2),BASE(IZGAR))
        CALL ALDERV(NBURN,BURNUP,ZDIFFZ(1,JGR,2),BASE(IZGAR))
        CALL ALDERV(NBURN,BURNUP,ZH(1,JGR,2),BASE(IZGAR))
        CALL ALDERV(NBURN,BURNUP,ZFLUX(1,JGR,2),BASE(IZGAR))
        DO 31 IGR=1,NGRP
         DO 32 IL=1,NL
           CALL ALDERV(NBURN,BURNUP,ZSCAT(1,IL,IGR,JGR,2),BASE(IZGAR))
   32    CONTINUE
   31   CONTINUE
   30 CONTINUE
      CALL RLSARA(BASE(IZGAR))
*
      RETURN
      END
