*DECK DELDFL
      SUBROUTINE DELDFL (IPTRK,IPSYS0,IPFLUP,NUN,NGRP,ICL1,ICL2,IMPX,
     1 IMPH,TITR,EPS2,NADI,MAXX0,FKEFF,EVECT,ADECT,EASS,SOUR,AGAR,GRAD1,
     2 GRAD2,GAR1,GAR2,GAR3,WORK)
*
*-----------------------------------------------------------------------
*
* SOLUTION OF A MULTIGROUP FIXED SOURCE EIGENVALUE PROBLEM FOR THE
* CALCULATION OF A DIRECT GPT SOLUTION IN TRIVAC. USE THE PRECONDI-
* TIONED POWER METHOD WITH A TWO GROUP SVAT ACCELERATION TECHNIQUE.
*
* INPUT PARAMETERS:
*  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPSYS0 : L_SYSTEM POINTER TO UNPERTURBED SYSTEM MATRICES.
*  IPFLUP : L_FLUX POINTER TO THE GPT SOLUTION.
*  NUN    : NUMBER OF UNKNOWNS IN EACH ENERGY GROUP.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  ICL1   : NUMBER OF FREE IRETATIONS IN ONE CYCLE OF THE INVERSE POWER
*           METHOD.
*  ICL2   : NUMBER OF ACCELERATED ITERATIONS IN ONE CYCLE.
*  IMPX   : PRINT PARAMETER. =0: NO PRINT ; =1: MINIMUM PRINTING ;
*           =2: ITERATION HISTORY IS PRINTED. =3: SOLUTION IS PRINTED.
*  IMPH   : =0: NO ACTION IS TAKEN.
*           =1: THE FLUX IS COMPARED TO A REFERENCE FLUX STORED ON LCM.
*           =2: THE CONVERGENCE HISTOGRAM IS PRINTED.
*           =3: THE CONVERGENCE HISTOGRAM IS PRINTED WITH AXIS AND
*               TITLES. THE PLOTTING FILE IS COMPLETED.
*           =4: THE CONVERGENCE HISTOGRAM IS PRINTED WITH AXIS, ACCE-
*               LERATION FACTORS AND TITLES. THE PLOTTING FILE IS
*               COMPLETED.
*  TITR   : CHARACTER*72 TITLE.
*  EPS2   : CONVERGENCE CRITERIA FOR THE FLUX.
*  NADI   : NUMBER OF INNER ADI ITERATIONS PER OUTER ITERATION.
*  MAXX0  : MAXIMUM NUMBER OF OUTER ITERATIONS.
*  FKEFF  : EFFECTIVE MULTIPLICATION FACTOR
*  EVECT  : UNKNOWN VECTOR FOR THE NON PERTURBED DIRECT FLUX.
*  ADECT  : UNKNOWN VECTOR FOR THE NON PERTURBED ADJOINT FLUX.
*           DIMENSION EVECT(NUN,NGRP),ADECT(NUN,NGRP)
*  SOUR   : FIXED SOURCE. DIMENSION SOUR(NUN,NGRP)
*
* OUTPUT PARAMETERS:
*  EASS   : CONVERGED SOLUTION. DIMENSION EASS(NUN,NGRP)
*
* SCRATCH STORAGE:
*  AGAR   : DIMENSION AGAR(LL4) WHERE LL4 IS THE ORDER OF THE SYSTEM
*           MATRICES. LL4.LE.NUN
*  GRAD1  : DIMENSION GRAD1(NUN,NGRP)
*  GRAD2  : DIMENSION GRAD2(NUN,NGRP)
*  GAR1   : DIMENSION GAR1(NUN,NGRP)
*  GAR2   : DIMENSION GAR2(NUN,NGRP)
*  GAR3   : DIMENSION GAR3(NUN,NGRP)
*  WORK   : DIMENSION WORK(NUN)
*
* REFERENCE:
*  A. HEBERT, 'PRECONDITIONING THE POWER METHOD FOR REACTOR CALCU-
*  LATIONS', NUCL. SCI. ENG., 94, 1 (1986).
*
*-------------------------------------- AUTHOR: A. HEBERT ; 22/06/94 ---
*
      CHARACTER*(*) TITR
      CHARACTER*12 TEXT12,NAMLCM,NAMMY
      LOGICAL LCM,LOGTES,LMPH
      DOUBLE PRECISION AIL,BIL,EVAL,D2F(2,3),ALP,BET,DDELN1,DDELD1,
     1 ZNORM,GAZ
      DIMENSION EVECT(NUN,*),ADECT(NUN,*),EASS(NUN,*),SOUR(NUN,*),
     1 AGAR(*),GRAD1(NUN,*),GRAD2(NUN,*),GAR1(NUN,*),GAR2(NUN,*),
     2 GAR3(NUN,*),WORK(*),ERR(250),ALPH(250),BETA(250),ITITR(18)
      COMMON BASE(1)
      DATA EPS1,MMAXX/1.0E-4,250/
      MAXX=MIN(MAXX0,MMAXX)
      CALL MTOPEN(IMPX,IPTRK,LL4)
      IF(LL4.GT.NUN) CALL XABORT('DELDFL: INVALID NUMBER OF UNKNOWNS.')
      TEXT12=' '
      CALL LCMNXT(IPSYS0,TEXT12,NAMLCM,NAMMY,LCM)
*
* UNPERTURBED EIGENVALUE CALCULATION.
      AIL=0.0D0
      BIL=0.0D0
      DO 80 IGR=1,NGRP
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      CALL MTLDLM(TEXT12,IPTRK,IPSYS0,EVECT(1,IGR),GRAD1(1,IGR))
      DO 10 I=1,LL4
   10 WORK(I)=0.0
      DO 70 JGR=1,NGRP
      IF(JGR.EQ.IGR) GO TO 40
      WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 40
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS0,TEXT12)
         DO 20 I=1,LL4
   20    GRAD1(I,IGR)=GRAD1(I,IGR)-BASE(IAGAR+I-1)*EVECT(I,JGR)
      ELSE
         CALL LCMGET(IPSYS0,TEXT12,AGAR)
         DO 30 I=1,LL4
   30    GRAD1(I,IGR)=GRAD1(I,IGR)-AGAR(I)*EVECT(I,JGR)
      ENDIF
   40 WRITE(TEXT12,'(1HB,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 70
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS0,TEXT12)
         DO 50 I=1,LL4
   50    WORK(I)=WORK(I)+BASE(IAGAR+I-1)*EVECT(I,JGR)
      ELSE
         CALL LCMGET(IPSYS0,TEXT12,AGAR)
         DO 60 I=1,LL4
   60    WORK(I)=WORK(I)+AGAR(I)*EVECT(I,JGR)
      ENDIF
   70 CONTINUE
      DO 80 I=1,LL4
      AIL=AIL+ADECT(I,IGR)*GRAD1(I,IGR)
   80 BIL=BIL+ADECT(I,IGR)*WORK(I)
      EVAL=AIL/BIL
      IF(ABS(FKEFF-1.0/EVAL).GT.EPS1) CALL XABORT('DELDFL: THE COMPUTE'
     1 //'D AND PROVIDED K-EFFECTIVES ARE INCONSISTENTS.')
*
* VALIDATION OF THE FIXED SOURCE TERM.
      AIL=0.0D0
      BIL=0.0D0
      DO 90 IGR=1,NGRP
      DO 90 I=1,LL4
      GAZ=ADECT(I,IGR)*SOUR(I,IGR)
      AIL=AIL+GAZ
   90 BIL=BIL+GAZ**2
      GAZ=ABS(AIL)/SQRT(REAL(LL4)*BIL)
      IF(GAZ.GT.EPS2) CALL XABORT('DELDFL: THE SOURCE TERM IS NOT ORT'
     1 //'HOGONAL TO THE ADJOINT REFERENCE FLUX.')
*
* PRECONDITIONED POWER METHOD.
      ISTART=1
      NNADI=NADI
      IF(IMPX.GE.1) WRITE (6,600) NADI
      IF(IMPX.GE.2) WRITE (6,610)
      M=0
  100 M=M+1
*
      IF(MOD(M-ISTART+1,ICL1+ICL2).EQ.1) THEN
         DO 135 IGR=1,NGRP
         WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
         CALL MTLDLM(TEXT12,IPTRK,IPSYS0,EASS(1,IGR),GAR1(1,IGR))
         DO 135 JGR=1,NGRP
         IF(JGR.EQ.IGR) GO TO 120
         WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
         CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 120
         IF(LCM) THEN
            IAGAR=LCMIOF(IPSYS0,TEXT12)
            DO 110 I=1,LL4
  110       GAR1(I,IGR)=GAR1(I,IGR)-BASE(IAGAR+I-1)*EASS(I,JGR)
         ELSE
            CALL LCMGET(IPSYS0,TEXT12,AGAR)
            DO 115 I=1,LL4
  115       GAR1(I,IGR)=GAR1(I,IGR)-AGAR(I)*EASS(I,JGR)
         ENDIF
  120    WRITE(TEXT12,'(1HB,2I3)') IGR,JGR
         CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 135
         IF(LCM) THEN
            IAGAR=LCMIOF(IPSYS0,TEXT12)
            DO 125 I=1,LL4
  125       GAR1(I,IGR)=GAR1(I,IGR)-EVAL*BASE(IAGAR+I-1)*EASS(I,JGR)
         ELSE
            CALL LCMGET(IPSYS0,TEXT12,AGAR)
            DO 130 I=1,LL4
  130       GAR1(I,IGR)=GAR1(I,IGR)-EVAL*AGAR(I)*EASS(I,JGR)
         ENDIF
  135    CONTINUE
      ENDIF
*
* DIRECTION EVALUATION.
      DO 180 IGR=1,NGRP
      DO 140 I=1,LL4
  140 GRAD1(I,IGR)=-SOUR(I,IGR)-GAR1(I,IGR)
      DO 170 JGR=1,IGR-1
      WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 170
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS0,TEXT12)
         DO 150 I=1,LL4
  150    GRAD1(I,IGR)=GRAD1(I,IGR)+BASE(IAGAR+I-1)*GRAD1(I,JGR)
      ELSE
         CALL LCMGET(IPSYS0,TEXT12,AGAR)
         DO 160 I=1,LL4
  160    GRAD1(I,IGR)=GRAD1(I,IGR)+AGAR(I)*GRAD1(I,JGR)
      ENDIF
  170 CONTINUE
*
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      IF(NNADI.EQ.1) THEN
         CALL MTLDLS(TEXT12,IPTRK,IPSYS0,GRAD1(1,IGR))
      ELSE
         CALL TRIADI(TEXT12,IPTRK,IPSYS0,LL4,GRAD1(1,IGR),NNADI)
      ENDIF
  180 CONTINUE
*
* HOTELLING DEFLATION.
      DDELN1=0.0D0
      DDELD1=0.0D0
      DO 230 IGR=1,NGRP
      DO 190 I=1,LL4
  190 WORK(I)=0.0
      DO 220 JGR=1,NGRP
      WRITE(TEXT12,'(1HB,2I3)') JGR,IGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 220
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS0,TEXT12)
         DO 200 I=1,LL4
  200    WORK(I)=WORK(I)+BASE(IAGAR+I-1)*ADECT(I,JGR)
      ELSE
         CALL LCMGET(IPSYS0,TEXT12,AGAR)
         DO 210 I=1,LL4
  210    WORK(I)=WORK(I)+AGAR(I)*ADECT(I,JGR)
      ENDIF
  220 CONTINUE
      DO 230 I=1,LL4
      DDELN1=DDELN1+WORK(I)*EASS(I,IGR)
  230 DDELD1=DDELD1+WORK(I)*EVECT(I,IGR)
      ZNORM=DDELN1/DDELD1
      DO 240 IGR=1,NGRP
      DO 240 I=1,LL4
  240 GRAD1(I,IGR)=GRAD1(I,IGR)-ZNORM*EVECT(I,IGR)
*
* EVALUATION OF THE DISPLACEMENT AND OF THE TWO ACCELERATION PARAMETERS
* ALP AND BET.
      ALP=1.0D0
      BET=0.0D0
      DO 250 I=1,2
      DO 250 J=1,3
  250 D2F(I,J)=0.0D0
      DO 310 IGR=1,NGRP
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      CALL MTLDLM(TEXT12,IPTRK,IPSYS0,GRAD1(1,IGR),GAR2(1,IGR))
      DO 310 JGR=1,NGRP
      IF(JGR.EQ.IGR) GO TO 280
      WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 280
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS0,TEXT12)
         DO 260 I=1,LL4
  260    GAR2(I,IGR)=GAR2(I,IGR)-BASE(IAGAR+I-1)*GRAD1(I,JGR)
      ELSE
         CALL LCMGET(IPSYS0,TEXT12,AGAR)
         DO 270 I=1,LL4
  270    GAR2(I,IGR)=GAR2(I,IGR)-AGAR(I)*GRAD1(I,JGR)
      ENDIF
  280 WRITE(TEXT12,'(1HB,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 310
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS0,TEXT12)
         DO 290 I=1,LL4
  290    GAR2(I,IGR)=GAR2(I,IGR)-EVAL*BASE(IAGAR+I-1)*GRAD1(I,JGR)
      ELSE
         CALL LCMGET(IPSYS0,TEXT12,AGAR)
         DO 300 I=1,LL4
  300    GAR2(I,IGR)=GAR2(I,IGR)-EVAL*AGAR(I)*GRAD1(I,JGR)
      ENDIF
  310 CONTINUE
      IF(1+MOD(M-ISTART,ICL1+ICL2).GT.ICL1) THEN
         DO 320 IGR=1,NGRP
         DO 320 I=1,LL4
         D2F(1,1)=D2F(1,1)+GAR2(I,IGR)**2
         D2F(1,2)=D2F(1,2)+GAR2(I,IGR)*GAR3(I,IGR)
         D2F(2,2)=D2F(2,2)+GAR3(I,IGR)**2
         D2F(1,3)=D2F(1,3)-(GAR1(I,IGR)+SOUR(I,IGR))*GAR2(I,IGR)
  320    D2F(2,3)=D2F(2,3)-(GAR1(I,IGR)+SOUR(I,IGR))*GAR3(I,IGR)
         D2F(2,1)=D2F(1,2)
*        SOLUTION OF A LINEAR SYSTEM.
         CALL ALSBD(2,1,D2F,IER,2)
         IF(IER.NE.0) CALL XABORT('DELDFL: SINGULAR MATRIX.')
         ALP=D2F(1,3)
         BET=D2F(2,3)/ALP
         IF((ALP.LT.1.0D0).AND.(ALP.GT.0.0D0)) THEN
            ALP=1.0D0
            BET=0.0D0
         ELSE IF(ALP.LE.0.0D0) THEN
            ISTART=M+1
            ALP=1.0D0
            BET=0.0D0
         ENDIF
         DO 330 IGR=1,NGRP
         DO 330 I=1,LL4
         GRAD1(I,IGR)=ALP*(GRAD1(I,IGR)+BET*GRAD2(I,IGR))
  330    GAR2(I,IGR)=ALP*(GAR2(I,IGR)+BET*GAR3(I,IGR))
      ENDIF
*
      LOGTES=(M.LT.ICL1).OR.(MOD(M,ICL1+ICL2).EQ.ICL1)
      IF(LOGTES) THEN
         DELT=0.0
         DO 350 IGR=1,NGRP
         DELN=0.0
         DELD=0.0
         DO 340 I=1,LL4
         DELN=MAX(DELN,ABS(GRAD1(I,IGR)))
         EASS(I,IGR)=EASS(I,IGR)+GRAD1(I,IGR)
         DELD=MAX(DELD,ABS(EASS(I,IGR)))
         GAR1(I,IGR)=GAR1(I,IGR)+GAR2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
  340    GAR3(I,IGR)=GAR2(I,IGR)
         IF(DELD.NE.0.0) DELT=MAX(DELT,DELN/DELD)
  350    CONTINUE
         IF(IMPX.GE.2) WRITE (6,620) M,ALP,BET,ZNORM,DELT
*        COMPUTE THE CONVERGENCE HISTOGRAM.
         IF(IMPH.GE.1) THEN
            LMPH=IMPH.GE.1
            CALL XCOMP(IPFLUP,LL4,NUN,EASS(1,NGRP),LMPH,ERR(M))
            ALPH(M)=ALP
            BETA(M)=BET
         ENDIF
         IF(DELT.LT.EPS2) GO TO 370
      ELSE
         DO 360 IGR=1,NGRP
         DO 360 I=1,LL4
         EASS(I,IGR)=EASS(I,IGR)+GRAD1(I,IGR)
         GAR1(I,IGR)=GAR1(I,IGR)+GAR2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
  360    GAR3(I,IGR)=GAR2(I,IGR)
         IF(IMPX.GE.2) WRITE (6,620) M,ALP,BET,ZNORM
*        COMPUTE THE CONVERGENCE HISTOGRAM.
         IF(IMPH.GE.1) THEN
            LMPH=IMPH.GE.1
            CALL XCOMP(IPFLUP,LL4,NUN,EASS(1,NGRP),LMPH,ERR(M))
            ALPH(M)=ALP
            BETA(M)=BET
         ENDIF
      ENDIF
      IF(M.EQ.1) TEST=DELT
      IF((M.GT.20).AND.(DELT.GT.TEST)) CALL XABORT('DELDFL: CONVERGENC'
     1 //'E FAILURE.')
      IF(M.GE.MAXX) THEN
         WRITE (6,710)
         GO TO 370
      ENDIF
      IF(MOD(M,36).EQ.0) THEN
         ISTART=M+1
         NNADI=NNADI+1
         WRITE (6,720) NNADI
      ENDIF
      GO TO 100
*
* SOLUTION EDITION.
  370 IF(IMPX.EQ.1) WRITE (6,640) M
      IF(IMPX.GE.3) THEN
         DO 380 IGR=1,NGRP
         WRITE (6,690) IGR,(EASS(I,IGR),I=1,LL4)
  380    CONTINUE
      ENDIF
      IF(IMPH.GE.2) THEN
         IGRAPH=0
  390    IGRAPH=IGRAPH+1
         WRITE (TEXT12,'(5HHISTO,I3)') IGRAPH
         CALL LCMLEN (IPFLUP,TEXT12,ILENG,ITYLCM)
         IF(ILENG.EQ.0) THEN
            READ (TITR,'(18A4)') ITITR
            CALL LCMSIX (IPFLUP,TEXT12,1)
            CALL LCMPUT (IPFLUP,'HTITLE',18,3,ITITR)
            CALL LCMPUT (IPFLUP,'ALPHA',M,2,ALPH)
            CALL LCMPUT (IPFLUP,'BETA',M,2,BETA)
            CALL LCMPUT (IPFLUP,'ERROR',M,2,ERR)
            CALL LCMPUT (IPFLUP,'IMPH',1,1,IMPH)
            CALL LCMSIX (IPFLUP,' ',2)
         ELSE
            GO TO 390
         ENDIF
      ENDIF
      RETURN
*
  600 FORMAT(1H1/50H DELDFL: ITERATIVE PROCEDURE BASED ON PRECONDITION,
     1 17HED POWER METHOD (,I2,37H ADI ITERATIONS PER OUTER ITERATION)./
     2 9X,39HDIRECT FIXED SOURCE EIGENVALUE PROBLEM.)
  610 FORMAT (/29X,15HORTHONORMALIZA-/11X,5HALPHA,3X,4HBETA,6X,
     1 11HTION FACTOR,6X,8HACCURACY,12(1H.))
  620 FORMAT (1X,I3,4X,2F8.3,1PE14.2,6X,2E10.2)
  640 FORMAT(/23H DELDFL: CONVERGENCE IN,I4,12H ITERATIONS.)
  650 FORMAT(/53H DELDFL: CPU TIME USED TO SOLVE THE TRIANGULAR LINEAR,
     1 10H SYSTEMS =,F9.3/23X,34HTO COMPUTE THE BILINEAR PRODUCTS =,
     2 F9.3,20X,16HTOTAL CPU TIME =,F9.3)
  690 FORMAT(//52H DELDFL: DIRECT FIXED SOURCE PROBLEM SOLUTION CORRES,
     1 20HPONDING TO THE GROUP,I4//(5X,1P,8E14.5))
  710 FORMAT(/53H DELDFL: ***WARNING*** THE MAXIMUM NUMBER OF OUTER IT,
     1 20HERATIONS IS REACHED.)
  720 FORMAT(/53H DELDFL: INCREASING THE NUMBER OF INNER ITERATIONS TO,
     1 I3,36H ADI ITERATIONS PER OUTER ITERATION./)
      END
