*DECK DELDRV
      SUBROUTINE DELDRV (IPGPT,IPTRK,IPSYS0,IPFLU0,REC,NEL,NUN,NGRP,
     1 MAT,VOL,IDL,TITR,IPFLUP,EVECT,ADECT,EASS)
*
*-----------------------------------------------------------------------
*
* DRIVER FOR THE SOLUTION OF A PERTURBED NEUTRON FLUX AS A FIXED SOURCE
* EIGENVALUE PROBLEM.
*
* INPUT PARAMETERS:
*  IPGPT  : L_GPT POINTER TO FIXED SOURCE TERMS.
*  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPSYS0 : L_SYSTEM POINTER TO UNPERTURBED SYSTEM MATRICES.
*  IPFLU0 : L_FLUX POINTER TO THE UNPERTURBED SOLUTION.
*  REC    : .TRUE. RECOVER THE EXISTING SOLUTION AS GPT INITIAL
*           ESTIMATE; .FALSE. USE A UNIFORM GPT INITIAL ESTIMATE.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : TOTAL NUMBER OF UNKNOWNS PER GROUP.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*            DIMENSION MAT(NEL)
*  VOL    : VOLUMES. DIMENSION VOL(NEL)
*  IDL    : POSITION OF THE AVERAGE FLUX COMPONENT ASSOCIATED WITH
*           EACH VOLUME. DIMENSION IDL(NEL)
*  TITR   : TITLE.
*
* OUTPUT PARAMETER:
*  IPFLUP : L_FLUX POINTER TO THE GPT SOLUTION.
*
* SCRATCH STORAGE:
*  EVECT  : DIMENSION EVECT(NUN,NGRP)
*  ADECT  : DIMENSION ADECT(NUN,NGRP)
*  EASS   : DIMENSION EASS(NUN,NGRP)
*
* INFORMATION RECOVERED FROM L_SYSTEM AT IPSYS0:
*  'A  1  1'  : SYSTEM MATRIX RELATED TO FAST LEAKAGE AND REMOVAL.
*  'A  2  2'  : SYSTEM MATRIX RELATED TO THERMAL LEAKAGE AND REMOVAL.
*  'A  1  2'  : SYSTEM MATRIX RELATED TO UP-SCATTERING.
*  'A  2  1'  : SYSTEM MATRIX RELATED TO DOWN-SCATTERING.
*  'B  1  1'  : SYSTEM MATRIX RELATED TO FAST FISSION.
*  'B  1  2'  : SYSTEM MATRIX RELATED TO THERMAL FISSION.
*
* INFORMATION RECOVERED FROM L_SYSTEM AT IPSYSP:
*  'A  1  1' : DELTA-SYSTEM MATRIX RELATED TO FAST LEAKAGE AND REMOVAL.
*  'A  2  2' : DELTA-SYSTEM MATRIX RELATED TO THERMAL LEAKAGE AND
*              REMOVAL.
*  'A  1  2' : DELTA-SYSTEM MATRIX RELATED TO UP-SCATTERING.
*  'A  2  1' : DELTA-SYSTEM MATRIX RELATED TO DOWN-SCATTERING.
*  'B  1  1' : DELTA-SYSTEM MATRIX RELATED TO FAST FISSION.
*  'B  1  2' : DELTA-SYSTEM MATRIX RELATED TO THERMAL FISSION.
*
*---------------------------------------------- A. HEBERT 13/07/93 -----
*
      LOGICAL ADJ,REC
      DOUBLE PRECISION DFLOTT
      CHARACTER TEXT4*4,TITR*72,TEXT12*12
      DIMENSION VOL(*),MAT(*),IDL(*),EVECT(NUN,*),ADECT(NUN,*),
     1 EASS(NUN,*),IFPARM(5)
      COMMON BASE(1)
*
* SET FLUX PARAMETERS
*
      CALL LCMGET(IPGPT,'PARAM',IFPARM)
      NSRC = IFPARM(3)
      IF(REC) THEN
        CALL LCMGET(IPFLUP,'FLUX_PARAM',IFPARM)
      ELSE
        DO 1 I=1,5
          IFPARM(I) = 0
   1    CONTINUE
        IFPARM(1) = NGRP
        IFPARM(2) = NUN
        IF( NSRC.GT.1 )IFPARM(4) = NSRC
        IFPARM(5) = 1
      ENDIF
*
* READ THE INPUT DATA.
*     DEFAULT OPTIONS.
      ICL1=3
      ICL2=3
      NADI=1
      MAXX0=250
      IMPX=1
      IMPH=0
      ADJ=.FALSE.
      EPS2=1.0E-4
*
   10 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.EQ.10) GO TO 160
      IF(INDIC.NE.3) CALL XABORT('DELDRV: CHARACTER DATA EXPECTED.')
*
      IF(TEXT4.NE.'VAR1') GO TO 40
      CALL REDGET(INDIC,ICL1,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('DELDRV: INTEGER DATA EXPECTED.')
      CALL REDGET(INDIC,ICL2,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('DELDRV: INTEGER DATA EXPECTED.')
      GO TO 10
*
   40 IF(TEXT4.NE.'EDIT') GO TO 50
      CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('DELDRV: INTEGER DATA EXPECTED.')
      GO TO 10
*
   50 IF(TEXT4.NE.'ADJ') GO TO 55
      ADJ=.TRUE.
      GO TO 10
*
   55 IF(TEXT4.NE.'PREC') GO TO 60
      CALL REDGET(INDIC,NITMA,EPS2,TEXT4,DFLOTT)
      IF(INDIC.NE.2) CALL XABORT('DELDRV: REAL DATA EXPECTED.')
      GO TO 10
*
   60 IF(TEXT4.NE.'ADI') GO TO 65
      CALL REDGET(INDIC,NADI,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('DELDRV: INTEGER DATA EXPECTED.')
      GO TO 10
*
   65 IF(TEXT4.NE.'MAX') GO TO 70
      CALL REDGET(INDIC,MAXX0,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('DELDRV: INTEGER DATA EXPECTED.')
      GO TO 10
*
   70 IF(TEXT4.NE.'HIST') GO TO 150
      CALL REDGET(INDIC,IMPH,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('FLUDRV: INTEGER DATA EXPECTED(8).')
      GO TO 10
*
  150 IF(TEXT4.NE.';') CALL XABORT('DELDRV: ; EXPECTED.')
*
* RECOVER UNPERTURBED K-EFFECTIVE AND FLUXES.
  160 CALL LCMGET(IPFLU0,'K-EFFECTIVE',FKEFF)
      DO 170 IGR=1,NGRP
        WRITE(TEXT12,'(4HFLUX,I3)') IGR
        CALL LCMGET(IPFLU0,TEXT12,EVECT(1,IGR))
        WRITE(TEXT12,'(5HAFLUX,I3)') IGR
        CALL LCMGET(IPFLU0,TEXT12,ADECT(1,IGR))
  170 CONTINUE
*
* GPT FLUXES INITIALIZATION.
      CALL SETARA(BASE,NUN*NGRP,ISOUR)
      DO 20 ISRC=1,NSRC
* RECOVER FIXED SOURCE
        DO 171 IGR=1,NGRP
          IF(ADJ) THEN
            WRITE(TEXT12,'(5HASOUR,I3,I3)') IGR,ISRC
          ELSE
            WRITE(TEXT12,'(5HDSOUR,I3,I3)') IGR,ISRC
          ENDIF
          CALL LCMGET(IPGPT,TEXT12,BASE(ISOUR+(IGR-1)*NUN))
  171   CONTINUE
        IF(REC.AND.ADJ.AND.(IMPH.EQ.0)) THEN
          IF( IFPARM(3).NE.1 )CALL XABORT('DELDRV: NO ADJOINT FLUX'
     +        //' PREVIOUSLY STORED IN L_FLUX')
          DO 180 IGR=1,NGRP
            IF(NSRC.GT.1) THEN
              WRITE(TEXT12,'(6HADFLUX,I3,I3)') IGR,ISRC
            ELSE
              WRITE(TEXT12,'(6HADFLUX,I3)') IGR
            ENDIF
            CALL LCMGET(IPFLUP,TEXT12,EASS(1,IGR))
  180     CONTINUE
        ELSE IF(REC.AND.(IMPH.EQ.0)) THEN
          IF( IFPARM(3).NE.0 )CALL XABORT('DELDRV: NO DIRECT FLUX'
     +        //' PREVIOUSLY STORED IN L_FLUX')
          DO 185 IGR=1,NGRP
            IF(NSRC.GT.1) THEN
              WRITE(TEXT12,'(5HDFLUX,I3,I3)') IGR,ISRC
            ELSE
              WRITE(TEXT12,'(5HDFLUX,I3)') IGR
            ENDIF
            CALL LCMGET(IPFLUP,TEXT12,EASS(1,IGR))
  185     CONTINUE
        ELSE
*         INITIAL ESTIMATE OF THE GPT FLUXES.
          DO 190 IGR=1,NGRP
           DO 191 IND1=1,NUN
             EASS(IND1,IGR)=1.0
  191      CONTINUE
  190     CONTINUE
        ENDIF
        CALL SETARA(BASE,NUN,IAGAR)
        CALL SETARA(BASE,NUN*NGRP,IGRAD1)
        CALL SETARA(BASE,NUN*NGRP,IGRAD2)
        CALL SETARA(BASE,NUN*NGRP,IGAR1)
        CALL SETARA(BASE,NUN*NGRP,IGAR2)
        CALL SETARA(BASE,NUN*NGRP,IGAR3)
        CALL SETARA(BASE,NUN,IWORK)
        IF(ADJ) THEN
            IFPARM(3) = 1
            CALL DELAFL(IPTRK,IPSYS0,IPFLUP,NUN,NGRP,ICL1,ICL2,IMPX,
     1      IMPH,TITR,EPS2,NADI,MAXX0,FKEFF,EVECT,ADECT,EASS,
     2      BASE(ISOUR),BASE(IAGAR),BASE(IGRAD1),BASE(IGRAD2),
     3      BASE(IGAR1),BASE(IGAR2),BASE(IGAR3),BASE(IWORK))
        ELSE
            IFPARM(3) = 0
            CALL DELDFL(IPTRK,IPSYS0,IPFLUP,NUN,NGRP,ICL1,ICL2,IMPX,
     1      IMPH,TITR,EPS2,NADI,MAXX0,FKEFF,EVECT,ADECT,EASS,
     2      BASE(ISOUR),BASE(IAGAR),BASE(IGRAD1),BASE(IGRAD2),
     3      BASE(IGAR1),BASE(IGAR2),BASE(IGAR3),BASE(IWORK))
        ENDIF
        CALL RLSARA(BASE(IWORK))
        CALL RLSARA(BASE(IGAR3))
        CALL RLSARA(BASE(IGAR2))
        CALL RLSARA(BASE(IGAR1))
        CALL RLSARA(BASE(IGRAD2))
        CALL RLSARA(BASE(IGRAD1))
        CALL RLSARA(BASE(IAGAR))
        DO 200 IGR=1,NGRP
         CALL TRIFLX(IPTRK,NEL,NUN,EASS(1,IGR),MAT,VOL,IDL)
         IF(NSRC.GT.1) THEN
           IF(ADJ) THEN
              WRITE(TEXT12,'(6HADFLUX,I3,I3)') IGR,ISRC
           ELSE
              WRITE(TEXT12,'(5HDFLUX,I3,I3)') IGR,ISRC
           ENDIF
         ELSE
           IF(ADJ) THEN
              WRITE(TEXT12,'(6HADFLUX,I3)') IGR
           ELSE
              WRITE(TEXT12,'(5HDFLUX,I3)') IGR
           ENDIF
         ENDIF
         CALL LCMPUT(IPFLUP,TEXT12,NUN,2,EASS(1,IGR))
  200    CONTINUE
   20 CONTINUE
*
      CALL RLSARA(BASE(ISOUR))
      CALL LCMPUT(IPFLUP,'FLUX-PARAM',5,1,IFPARM)
      RETURN
      END
