*DECK DELPER
      SUBROUTINE DELPER (IPTRK,IPSYS0,IPSYSP,ADJ,NUN,NGRP,FKEFF,IMPX,
     1 EVECT,ADECT,DELKEF,SOUR,AGAR,WORK)
*
*-----------------------------------------------------------------------
*
* CALCULATION OF THE SOURCE TERM FOR A DIRECT OR ADJOINT FIXED SOURCE
* EIGENVALUE PROBLEM.
*
* INPUT PARAMETERS:
*  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPSYS0 : L_SYSTEM POINTER TO UNPERTURBED SYSTEM MATRICES.
*  IPSYSP : L_SYSTEM POINTER TO DELTA SYSTEM MATRICES.
*  ADJ    : IF ADJ=.TRUE., WE COMPUTE THE SOURCE TERM FOR AN ADJOINT
*           FIXED SOURCE EIGENVALUE PROBLEM.
*  NUN    : TOTAL NUMBER OF UNKNOWNS PER ENERGY GROUP.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  FKEFF  : REFERENCE K-EFFECTIVE.
*  IMPX   : DELTA K-EFFECTIVE IS PRINTED IF IMPX.GE.1.
*  EVECT  : REFERENCE SOLUTION OF THE ASSOCIATED DIRECT EIGENVALUE
*           PROBLEM. DIMENSION EVECT(NUN,NGRP)
*  ADECT  : REFERENCE SOLUTION OF THE ASSOCIATED ADJOINT EIGENVALUE
*           PROBLEM. DIMENSION ADECT(NUN,NGRP)
*
* OUTPUT PARAMETERS:
*  DELKEF : DELTA K-EFFECTIVE.
*  SOUR   : SOURCE TERM. DIMENSION SOUR(NUN,NGRP)
*
* SCRATCH STORAGE:
*  AGAR   : DIMENSION AGAR(NUN)
*  WORK   : DIMENSION WORK(NUN)
*
*------------------------------------- AUTHOR: A. HEBERT ; 21/06/94 ----
*
      PARAMETER (EPS1=1.0E-4)
      CHARACTER*12 TEXT12,NAMLCM,NAMMY
      LOGICAL ADJ,LCM0,LCMP
      DOUBLE PRECISION AIL,BIL,EVAL,DEVAL
      DIMENSION EVECT(NUN,*),ADECT(NUN,*),SOUR(NUN,*),AGAR(*),WORK(*)
      COMMON BASE(1)
*
      CALL MTOPEN(0,IPTRK,LL4)
      IF(LL4.GT.NUN) CALL XABORT('DELPER: INVALID NUMBER OF UNKNOWNS.')
      TEXT12=' '
      CALL LCMNXT(IPSYS0,TEXT12,NAMLCM,NAMMY,LCM0)
      TEXT12=' '
      CALL LCMNXT(IPSYSP,TEXT12,NAMLCM,NAMMY,LCMP)
*
* COMPUTE THE NON-PERTURBED K-EFFECTIVE.
      AIL=0.0D0
      BIL=0.0D0
      DO 80 IGR=1,NGRP
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      CALL MTLDLM(TEXT12,IPTRK,IPSYS0,EVECT(1,IGR),SOUR(1,IGR))
      DO 10 I=1,LL4
   10 WORK(I)=0.0
      DO 70 JGR=1,NGRP
      IF(JGR.EQ.IGR) GO TO 40
      WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 40
      IF(LCM0) THEN
         IAGAR=LCMIOF(IPSYS0,TEXT12)
         DO 20 I=1,LL4
   20    SOUR(I,IGR)=SOUR(I,IGR)-BASE(IAGAR+I-1)*EVECT(I,JGR)
      ELSE
         CALL LCMGET(IPSYS0,TEXT12,AGAR)
         DO 30 I=1,LL4
   30    SOUR(I,IGR)=SOUR(I,IGR)-AGAR(I)*EVECT(I,JGR)
      ENDIF
   40 WRITE(TEXT12,'(1HB,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 70
      IF(LCM0) THEN
         IAGAR=LCMIOF(IPSYS0,TEXT12)
         DO 50 I=1,LL4
   50    WORK(I)=WORK(I)+BASE(IAGAR+I-1)*EVECT(I,JGR)
      ELSE
         CALL LCMGET(IPSYS0,TEXT12,AGAR)
         DO 60 I=1,LL4
   60    WORK(I)=WORK(I)+AGAR(I)*EVECT(I,JGR)
      ENDIF
   70 CONTINUE
      DO 80 I=1,LL4
      AIL=AIL+ADECT(I,IGR)*SOUR(I,IGR)
   80 BIL=BIL+ADECT(I,IGR)*WORK(I)
      EVAL=AIL/BIL
      IF(ABS(FKEFF-1.0/EVAL).GT.EPS1) CALL XABORT('DELPER: INCOMPATIBIL'
     1 //'ITY BETWEEN THE PROVIDED AND CALCULATED KEFF.')
*
* COMPUTE THE DIRECT OR ADJOINT SOURCE TERM.
      IF(ADJ) THEN
         DO 150 IGR=1,NGRP
         WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
         CALL MTLDLM(TEXT12,IPTRK,IPSYSP,ADECT(1,IGR),SOUR(1,IGR))
         DO 150 JGR=1,NGRP
         IF(JGR.EQ.IGR) GO TO 120
         WRITE(TEXT12,'(1HA,2I3)') JGR,IGR
         CALL LCMLEN(IPSYSP,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 120
         IF(LCMP) THEN
            IAGAR=LCMIOF(IPSYSP,TEXT12)
            DO 100 I=1,LL4
  100       SOUR(I,IGR)=SOUR(I,IGR)-BASE(IAGAR+I-1)*ADECT(I,JGR)
         ELSE
            CALL LCMGET(IPSYSP,TEXT12,AGAR)
            DO 110 I=1,LL4
  110       SOUR(I,IGR)=SOUR(I,IGR)-AGAR(I)*ADECT(I,JGR)
         ENDIF
  120    WRITE(TEXT12,'(1HB,2I3)') JGR,IGR
         CALL LCMLEN(IPSYSP,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 150
         IF(LCMP) THEN
            IAGAR=LCMIOF(IPSYSP,TEXT12)
            DO 130 I=1,LL4
  130       SOUR(I,IGR)=SOUR(I,IGR)-EVAL*BASE(IAGAR+I-1)*ADECT(I,JGR)
         ELSE
            CALL LCMGET(IPSYSP,TEXT12,AGAR)
            DO 140 I=1,LL4
  140       SOUR(I,IGR)=SOUR(I,IGR)-EVAL*AGAR(I)*ADECT(I,JGR)
         ENDIF
  150    CONTINUE
         AIL=0.0D0
         DO 160 IGR=1,NGRP
         DO 160 I=1,LL4
  160    AIL=AIL+SOUR(I,IGR)*EVECT(I,IGR)
         DEVAL=AIL/BIL
         DO 210 IGR=1,NGRP
         DO 170 I=1,LL4
  170    WORK(I)=0.0
         DO 200 JGR=1,NGRP
         WRITE(TEXT12,'(1HB,2I3)') JGR,IGR
         CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 200
         IF(LCM0) THEN
            IAGAR=LCMIOF(IPSYS0,TEXT12)
            DO 180 I=1,LL4
  180       WORK(I)=WORK(I)+BASE(IAGAR+I-1)*ADECT(I,JGR)
         ELSE
            CALL LCMGET(IPSYS0,TEXT12,AGAR)
            DO 190 I=1,LL4
  190       WORK(I)=WORK(I)+AGAR(I)*ADECT(I,JGR)
         ENDIF
  200    CONTINUE
         DO 210 I=1,LL4
  210    SOUR(I,IGR)=SOUR(I,IGR)-DEVAL*WORK(I)
      ELSE
         DO 280 IGR=1,NGRP
         WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
         CALL MTLDLM(TEXT12,IPTRK,IPSYSP,EVECT(1,IGR),SOUR(1,IGR))
         DO 280 JGR=1,NGRP
         IF(JGR.EQ.IGR) GO TO 250
         WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
         CALL LCMLEN(IPSYSP,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 250
         IF(LCMP) THEN
            IAGAR=LCMIOF(IPSYSP,TEXT12)
            DO 230 I=1,LL4
  230       SOUR(I,IGR)=SOUR(I,IGR)-BASE(IAGAR+I-1)*EVECT(I,JGR)
         ELSE
            CALL LCMGET(IPSYSP,TEXT12,AGAR)
            DO 240 I=1,LL4
  240       SOUR(I,IGR)=SOUR(I,IGR)-AGAR(I)*EVECT(I,JGR)
         ENDIF
  250    WRITE(TEXT12,'(1HB,2I3)') IGR,JGR
         CALL LCMLEN(IPSYSP,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 280
         IF(LCMP) THEN
            IAGAR=LCMIOF(IPSYSP,TEXT12)
            DO 260 I=1,LL4
  260       SOUR(I,IGR)=SOUR(I,IGR)-EVAL*BASE(IAGAR+I-1)*EVECT(I,JGR)
         ELSE
            CALL LCMGET(IPSYSP,TEXT12,AGAR)
            DO 270 I=1,LL4
  270       SOUR(I,IGR)=SOUR(I,IGR)-EVAL*AGAR(I)*EVECT(I,JGR)
         ENDIF
  280    CONTINUE
         AIL=0.0D0
         DO 290 IGR=1,NGRP
         DO 290 I=1,LL4
  290    AIL=AIL+ADECT(I,IGR)*SOUR(I,IGR)
         DEVAL=AIL/BIL
         DO 340 IGR=1,NGRP
         DO 300 I=1,LL4
  300    WORK(I)=0.0
         DO 330 JGR=1,NGRP
         WRITE(TEXT12,'(1HB,2I3)') IGR,JGR
         CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 330
         IF(LCM0) THEN
            IAGAR=LCMIOF(IPSYS0,TEXT12)
            DO 310 I=1,LL4
  310       WORK(I)=WORK(I)+BASE(IAGAR+I-1)*EVECT(I,JGR)
         ELSE
            CALL LCMGET(IPSYS0,TEXT12,AGAR)
            DO 320 I=1,LL4
  320       WORK(I)=WORK(I)+AGAR(I)*EVECT(I,JGR)
         ENDIF
  330    CONTINUE
         DO 340 I=1,LL4
  340    SOUR(I,IGR)=SOUR(I,IGR)-DEVAL*WORK(I)
      ENDIF
      DELKEF=-DEVAL/(EVAL*EVAL)
      IF(IMPX.GE.1) WRITE (6,'(/21H DELPER: DELTA KEFF =,1P,E17.9/)')
     1 DELKEF
      RETURN
      END
