*DECK DELTA
      SUBROUTINE DELTA(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
* GENERALIZED PERTURBATION THEORY (GPT) SOLUTION MODULE FOR TRIVAC-3.
*
* INPUT/OUTPUT PARAMETERS:
*  NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
*  HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
*  IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
*           =4 SEQUENTIAL ASCII FILE.
*  JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
*           =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
*           =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
*  KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.
*           DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
*           KENTRY(NENTRY)
*
* LINKED LIST / XSM FILE:
*  HENTRY(1) : CREATE OR MODIFICATION TYPE(L_FLUX) => GPT SOLUTION.
*  HENTRY(2) : READ-ONLY TYPE(L_GPT) => SOURCE TERM.
*  HENTRY(3) : READ-ONLY TYPE(L_FLUX) => UNPERTURBED SOLUTION.
*  HENTRY(4) : READ-ONLY TYPE(L_SYSTEM) => UNPERTURBED MATRICES.
*  HENTRY(5) : READ-ONLY TYPE(L_TRIVAC) => TRACKING INFORMATIONS.
*
*-------------------------------------- AUTHOR: A. HEBERT ; 20/12/93 ---
*-------------------------------------- MODIF: E. VARIN ; 15/05/96 ---
*
      CHARACTER HENTRY(*)*12,TEXT12*12,TITLE*72,HSIGN*12
      LOGICAL REC
      DIMENSION IENTRY(*),JENTRY(*),KENTRY(*),IGP(4),IBASE(1),KCHAR(18),
     +          ISPARM(7),IFPARM(5),IPR(3),ITP(19)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
* PARAMETER VALIDATION.
      IF(NENTRY.LE.4) CALL XABORT('DELTA: FIVE PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('DELTA: LI'
     1 //'NKED LIST OR XSM FILE EXPECTED AT LHS.')
      IF((JENTRY(1).NE.0).AND.(JENTRY(1).NE.1)) CALL XABORT('DELTA: EN'
     1 //'TRY IN CREATE OR MODIFICATION MODE EXPECTED.')
      IF(JENTRY(1).EQ.1) THEN
         CALL LCMGET(KENTRY(1),'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_FLUX') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('DELTA: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_FLUX EXPECTED.')
         ENDIF
      ELSE
        HSIGN='L_FLUX'
        READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
        CALL LCMPUT(KENTRY(1),'SIGNATURE',3,3,KCHAR)
      ENDIF
      REC=(JENTRY(1).EQ.1)
      IPFLUP = KENTRY(1)
*
      IF((JENTRY(2).NE.2).OR.((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)))
     1 CALL XABORT('DELTA: LINKED LIST OR XSM FILE IN READ-ONLY MODE E'
     2 //'XPECTED AT FIRST RHS.')
      IF((JENTRY(3).NE.2).OR.((IENTRY(3).NE.1).AND.(IENTRY(3).NE.2)))
     1 CALL XABORT('DELTA: LINKED LIST OR XSM FILE IN READ-ONLY MODE E'
     2 //'XPECTED AT SECOND RHS.')
      IF((JENTRY(4).NE.2).OR.(IENTRY(4).NE.1))
     1 CALL XABORT('SORKEF: LINKED LIST IN READ-ONLY MODE E'
     2 //'XPECTED AT FOURTH RHS.')
      IF((JENTRY(5).NE.2).OR.(IENTRY(5).NE.1))
     1 CALL XABORT('SORKEF: LINKED LIST IN READ-ONLY MODE E'
     2 //'XPECTED AT FIFTH RHS.')
      CALL LCMGET(KENTRY(2),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_GPT') THEN
         TEXT12=HENTRY(2)
         CALL XABORT('DELTA: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_GPT EXPECTED.')
      ENDIF
      IPGPT =  KENTRY(2)
      CALL LCMGET(KENTRY(3),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_FLUX') THEN
         TEXT12=HENTRY(3)
         CALL XABORT('DELTA: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_FLUX EXPECTED.')
      ENDIF
      IPFLU0 = KENTRY(3)
*
* RECOVER IPTRK AND IPSYS0 POINTER AND VALIDATE THEM.
      CALL LCMGET(KENTRY(4),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_SYSTEM') THEN
         TEXT12=HENTRY(4)
         CALL XABORT('DELTA: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_SYSTEM EXPECTED.')
      ENDIF
      IPSYS0 = KENTRY(4)
      CALL LCMGET(KENTRY(5),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_TRIVAC') THEN
        TEXT12=HENTRY(5)
        CALL XABORT('DELTA: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1     '. L_TRIVAC EXPECTED.')
      ENDIF
      IPTRK = KENTRY(5)
*
* RECOVER GENERAL TRACKING INFORMATION.
   40 CALL LCMGET(IPTRK,'GPARAM',IGP)
      NEL=IGP(1)
      NUN=IGP(2)
      CALL SETARA(IBASE,NEL,IMAT)
      CALL SETARA(BASE,NEL,IVOL)
      CALL SETARA(IBASE,NEL,IIDL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IIDL))
      CALL LCMLEN(IPTRK,'TITLE',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(IPTRK,'TITLE',KCHAR)
         WRITE(TITLE,'(18A4)') (KCHAR(I),I=1,18)
      ELSE
         TITLE='*** NO TITLE PROVIDED ***'
      ENDIF
      CALL LCMGET(IPTRK,'TPARAM',ITP)
      LL4 = ITP(8)
      CALL LCMGET(IPSYS0,'SYS-PARAM',ISPARM)
      NGRP = ISPARM(1)
      IF( ISPARM(2).NE.LL4 )CALL XABORT('SORKEF: UNPERTURBED SYSTEM'
     + //' AND TRACKING OBJECTS HAVE DIFFERENT NUMBER OF LINEAR ORDER')
      CALL LCMGET(IPFLU0,'FLUX-PARAM',IFPARM)
      IF( IFPARM(5).NE.0 )CALL XABORT('DELTA: UNPERTURBED FLUX '
     +   //'CONTAINS SOLUTION OF A GPT PROBLEM')
      IF( IFPARM(3).NE.1 )CALL XABORT('DELTA: UNPERTURBED FLUX DOES '
     +   //'NOT CONTAIN SOLUTION OF AN ADJOINT DIFFUSION PROBLEM')
      IF( IFPARM(1).NE.NGRP )CALL XABORT('DELTA: L_SYSTEM AND L_FLUX'
     +   //'FOR UNPERTURBED STATE HAVE DIFFERENT NUMBER OF GROUPS')
      IF( IFPARM(2).NE.NUN )CALL XABORT('DELTA: TRACKING AND '
     +    //'UNPERTURBED FLUX HAVE DIFFERENT NUMBER OF UNKNOWS')
      CALL LCMGET(IPGPT,'PARAM',IPR)
      IF( IPR(1).NE.NGRP )CALL XABORT('DELTA: L_GPT AND L_FLUX'
     +   //' HAVE DIFFERENT NUMBER OF GROUPS')
      IF( IPR(2).NE.NUN )CALL XABORT('DELTA: L_GPT AND L_FLUX'
     +   //' HAVE DIFFERENT NUMBER OF UNKNOWS')
      IF (REC) THEN
        CALL LCMGET(IPFLUP,'FLUX-PARAM',IFPARM)
        IF( IFPARM(5).NE.1 )CALL XABORT('DELTA: PERTURBED FLUX '
     +     //'CONTAINS SOLUTION OF A DIRECT DIFFUSION PROBLEM')
        IF( IFPARM(1).NE.NGRP )CALL XABORT('FLU: L_SYSTEM AND L_FLUX'
     +     //' HAVE DIFFERENT NUMBER OF GROUPS')
        IF( IFPARM(2).NE.NUN )CALL XABORT('FLU: TRACKING AND PREVIOUS '
     +     //'FLUX HAVE DIFFERENT NUMBER OF UNKNOWS')
      ENDIF
*
* COMPUTE THE GPT SOLUTION.
      CALL SETARA(BASE,NUN*NGRP,IEVECT)
      CALL SETARA(BASE,NUN*NGRP,IADECT)
      CALL SETARA(BASE,NUN*NGRP,IEASS)
      CALL DELDRV(IPGPT,IPTRK,IPSYS0,IPFLU0,REC,NEL,NUN,NGRP,
     1 IBASE(IMAT),BASE(IVOL),IBASE(IIDL),TITLE,IPFLUP,BASE(IEVECT),
     2 BASE(IADECT),BASE(IEASS))
      CALL RLSARA(BASE(IEASS))
      CALL RLSARA(BASE(IADECT))
      CALL RLSARA(BASE(IEVECT))
*
* RELEASE GENERAL TRACKING INFORMATION.
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
