*DECK DETECT
      SUBROUTINE DETECT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
**********************************************************************
*
*     THIS SUBROUTINE COMPUTES DETECTOR RESPONSE.
*     GEOMETRY ALLOWED: CARTESIAN AND HEXAGONAL
*
*     CETTE SOUS ROUTINE DETERMINE LA REPONSE DES DETECTEURS.
*     LA SEULE GEOMETRIE CONSIDEREE EST 3D-CARTESIEN
*
*     INPUT/OUTPUT PARAMETERS:
*      HMODUL : CHARACTER*12 NAME OF THE MODULE.
*      NENTRY : NUMBER OF LINKED LISTS AND FILES USED BY THE MODULE.
*      HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
*      IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
*               =4 SEQUENTIAL ASCII FILE; =5 DIRECT ACCESS FILE.
*      JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
*               =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
*               =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
*      KENTRY : FILE UNIT NUMBER OR LINKED LIST ADDRESS.
*
*      DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
*      KENTRY(NENTRY)
*
*     REQUIRED LINKED LISTS :
*        KENTRY(1-4): OBJECT TYPE L_DETECT IN MODIFICATION MODE
*                                 L_TRACK  IN READ-ONLY MODE
*                                 L_GEOM   IN READ-ONLY MODE
*                          L_IQS OU L_FLU  IN READ-ONLY MODE
*
*     PROCEDURES CALLED:  INTFLU, SMVAND, SMDRPD, SMCION
*
*-------------------------------------- AUTHOR: E. VARIN  94/10/10 ---
*
      IMPLICIT NONE
*  DRVDRV VARIABLES
      INTEGER NENTRY
      CHARACTER HENTRY(*)*12
      INTEGER IENTRY,JENTRY,KENTRY
      DIMENSION IENTRY(*),JENTRY(*),KENTRY(*)
*  SETARA VARIABLES
      REAL BASE
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*  REDGET TYPES
      INTEGER ITYP,NITMA
      REAL FLOTT
      DOUBLE PRECISION DFLOTT
      CHARACTER*12 TEXT
*
*  SIGNATURE INFORMATION
*
      CHARACTER*12 SGNTUR,NAMPAR
      INTEGER ISGNTR(3)
      LOGICAL TRK,FLU,GEO,DET,LCM,REGUL,LNORM,LHEX
*
      INTEGER IPTRK,IPLDET,IPGEOM,IPFLU
      INTEGER I,I1,J
*
*  GEOM
      INTEGER STATE(13)
*  TRK
      INTEGER GPARAM(4),FPARAM(5)
*
*  DETECTEUR STRUCTURE
*
      CHARACTER NAMDET*12
      REAL DEVPOS(6),VNORM
      INTEGER INFO(2),PARAM(2)
*
*  LOCAL VARIABLES
*
      INTEGER GEOTYP,NX,NY,NZ,NXP1,NYP1,NZP1
      INTEGER ILONG,ITYLCM,IGR,NGRP
      INTEGER IMESHX,IMESHY,IMESHZ
      INTEGER NUN,NEL,IPRT
*
      CHARACTER*12  CGMPR,NXTYP,NAMMY,NXDET,FIRST,FIRST1
      INTEGER       IKEYF,IFLU,IREP,ISPEC,IFRACT,INVCST,IPDD,
     +              IAPD,IBPD,NHEX,ITHEX,IHEX
*
      INTEGER NREP,KC
      REAL    DT,RESP,REF,PLNL,TLG
*
*  INPUT VERIFICATIONS
*
      IF(NENTRY.LE.3) CALL XABORT('DETECT: FOUR PARAMETER EXPECTED.')
*
*     SEARCHING OBJECT TYPES WITH THEIR SIGNATURE
*
      TRK = .FALSE.
      FLU = .FALSE.
      GEO = .FALSE.
      DET = .FALSE.
      DO 10 I=1,NENTRY
        IF((IENTRY(I).EQ.1).OR.(IENTRY(I).EQ.2)) THEN
         CALL LCMSIX(KENTRY(I),' ',0)
         CALL LCMGET(KENTRY(I),'SIGNATURE',ISGNTR)
         WRITE(SGNTUR,'(3A4)') (ISGNTR(I1),I1=1,3)
         IF (SGNTUR.EQ.'L_DETECT') THEN
           IPLDET=KENTRY(I)
           DET = .TRUE.
           IF(JENTRY(I).NE.1) CALL XABORT('DET'
     +     //'ECT: MODIFICATION MODE EXPECTED FOR OBJECT'//SGNTUR//'.')
         ELSEIF (SGNTUR.EQ.'L_GEOM') THEN
           IPGEOM=KENTRY(I)
           GEO = .TRUE.
           IF(JENTRY(I).NE.2) CALL XABORT('DET'
     +      //'ECT: READ-ONLY MODE EXPECTED FOR OBJECT'//SGNTUR//'.')
         ELSEIF (SGNTUR.EQ.'L_TRIVAC') THEN
          IF (.NOT.TRK) THEN
           IPTRK=KENTRY(I)
           TRK = .TRUE.
           IF(JENTRY(I).NE.2) CALL XABORT('DET'
     +      //'ECT: READ-ONLY MODE EXPECTED FOR OBJECT'//SGNTUR//'.')
          ELSE
            CALL XABORT('DETECT: ONLY ONE L_TRACK FILE IS REQUIRED')
          ENDIF
         ELSEIF (SGNTUR.EQ.'L_BIVAC') THEN
          IF (.NOT.TRK) THEN
           IPTRK=KENTRY(I)
           TRK = .TRUE.
           IF(JENTRY(I).NE.2) CALL XABORT('DET'
     +      //'ECT: READ-ONLY MODE EXPECTED FOR OBJECT'//SGNTUR//'.')
          ELSE
            CALL XABORT('DETECT: ONLY ONE L_TRACK FILE IS REQUIRED')
          ENDIF
         ELSEIF (SGNTUR.EQ.'L_FLUX') THEN
          IF (.NOT.FLU) THEN
           IPFLU=KENTRY(I)
           NAMPAR = 'FLUX-PARAM'
           FLU = .TRUE.
           IF(JENTRY(I).NE.2) CALL XABORT('DET'
     +      //'ECT: READ-ONLY MODE EXPECTED FOR OBJECT'//SGNTUR//'.')
          ELSE
            CALL XABORT('DETECT: ONLY ONE L_FLUX FILE IS REQUIRED')
          ENDIF
         ELSEIF (SGNTUR.EQ.'L_IQS') THEN
          IF (.NOT.FLU) THEN
           IPFLU=KENTRY(I)
           NAMPAR = 'IQS-PARAM'
           FLU = .TRUE.
           IF(JENTRY(I).NE.2) CALL XABORT('DET'
     +      //'ECT: READ-ONLY MODE EXPECTED FOR OBJECT'//SGNTUR//'.')
          ELSE
            CALL XABORT('DETECT: ONLY ONE L_FLUX FILE IS REQUIRED')
          ENDIF
         ENDIF
        ENDIF
  10  CONTINUE
      IF (.NOT.(FLU.AND.GEO.AND.TRK.AND.DET))
     +   CALL XABORT('DETECT: MISSING OBJETCS IN CALL')
*
*     DATA INPUT READING
*
      IPRT = 1
      LNORM = .FALSE.
 15   CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
      IF(ITYP.EQ.3) THEN
        IF (TEXT.EQ.'EDIT') THEN
*   PRINT FLAG
          CALL REDGET(ITYP,IPRT,FLOTT,TEXT,DFLOTT)
          IF (ITYP.NE.1)
     +      CALL XABORT('DETECT: INTEGER DATA EXPECTED FOR EDIT')
        ELSEIF (TEXT.EQ.'TIME') THEN
*  SIMULATION TIME STEP
          CALL REDGET(ITYP,NITMA,DT,TEXT,DFLOTT)
          IF (ITYP.NE.2)
     +      CALL XABORT('DETECT: REAL DATA EXPECTED FOR TIME')
        ELSEIF (TEXT.EQ.'REF') THEN
*   REFERENCE FLAG
          CALL REDGET(ITYP,KC,FLOTT,TEXT,DFLOTT)
          IF (ITYP.NE.1)
     +      CALL XABORT('DETECT: INTEGER DATA EXPECTED FOR REF')
        ELSEIF (TEXT.EQ.'NORM') THEN
*   FIXED VALUE OF DETECTOR
          LNORM = .TRUE.
          CALL REDGET(ITYP,NITMA,VNORM,TEXT,DFLOTT)
          IF (ITYP.NE.2)
     +      CALL XABORT('DETECT: REAL DATA EXPECTED FOR NORM')
          IF( VNORM.EQ.0.0 )CALL XABORT('DETECT: ILLEGAL VALUE OF NORM')
        ELSEIF (TEXT.EQ.';') THEN
          GOTO 70
        ELSE
          CALL XABORT('DETECT: CONTROLLED TYPE EXPECTED'//TEXT)
        ENDIF
      ELSE
        CALL XABORT('DETECT: CHARACTER DATA EXPECTED')
      ENDIF
      GOTO 15
*
*     LOCALISATION OF DETECTOR ZONES
*     COMPUTATION OF MEAN FLUX ALONG EACH DETECTOR
*
*     DETERMINATION DES ZONES DETECTEURS ET CALCUL DU FLUX MOYEN
*     LE LONG DE CHAQUE DETECTEUR
*
 70   CALL LCMGET(IPLDET,'HEX',LHEX)
      CALL LCMGET(IPGEOM,'STATE',STATE)
      GEOTYP = STATE(1)
      IF((LHEX.AND.(GEOTYP.LT.8)).OR.(.NOT.LHEX.AND.(GEOTYP.GE.8)))
     +    CALL XABORT('DETECT: INCOMPATIBLE DETECT WITH GEOMETRY')
      IF(GEOTYP.LT.5.OR.GEOTYP.EQ.6)
     +    CALL XABORT('DETECT: GEOMETRY TYPE NOT SUPPORTED IN DETECT')
      NX = STATE(3)
      NY=STATE(4)
      IF(NY.EQ.0) NY=1
      NZ=STATE(5)
      IF(NZ.EQ.0) NZ=1
      NXP1 = NX+1
      NYP1 = NY+1
      NZP1 = NZ+1
      CALL SETARA(BASE,NXP1,IMESHX)
      CALL SETARA(BASE,NYP1,IMESHY)
      CALL SETARA(BASE,NZP1,IMESHZ)
      IF((GEOTYP.EQ.7).OR.(GEOTYP.EQ.5)) THEN
        CALL LCMGET(IPGEOM,'MESHX',BASE(IMESHX))
        CALL LCMGET(IPGEOM,'MESHY',BASE(IMESHY))
      ELSE
*    ARBITRARY
        BASE(IMESHY)=0.
        BASE(IMESHY+1)=1.
        BASE(IMESHX)=0.
        BASE(IMESHX+1)=1.
      ENDIF
      IF( GEOTYP.EQ.9 .OR. GEOTYP.EQ.7 )THEN
       CALL LCMGET(IPGEOM,'MESHZ',BASE(IMESHZ))
      ELSE IF( GEOTYP.EQ.5 .OR. GEOTYP.EQ.8 )THEN
*    ARBITRARY
        BASE(IMESHZ)=0.
        BASE(IMESHZ+1)=1.
      ENDIF
*
      CALL LCMGET(IPTRK,'GPARAM',GPARAM)
      NEL = GPARAM(1)
      NUN = GPARAM(2)
      CALL SETARA(IBASE,NEL,IKEYF)
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IKEYF))
*
      CALL LCMGET(IPLDET,'DET-PARAM',PARAM)
      NGRP = PARAM(1)
      CALL LCMGET(IPFLU,NAMPAR,FPARAM)
      IF( FPARAM(1).NE.NGRP )CALL XABORT('DETECT: NUMBER OF ENERGY '
     +  //'GROUPS INCOMPATIBLE BETWEEN FLUX AND DETECT')
*
      CALL SETARA(BASE,NUN*NGRP,IFLU)
      CALL LCMSIX(IPFLU,' ',0)
      DO 100 IGR=1,NGRP
         WRITE(CGMPR,'(4HFLUX,I3)') IGR
         CALL LCMGET(IPFLU,CGMPR,BASE(IFLU+(IGR-1)*NUN))
 100  CONTINUE
      CALL SETARA(BASE,NGRP,ISPEC)
*
      IF(IPRT.GE.1) THEN
        IF(KC.EQ.0) THEN
          WRITE(6,*) 'DETECT: CALCULATION TYPE REFERENCE, KC = ',KC
        ELSE
          WRITE(6,*) 'DETECT: CALCULATION TYPE NORMAL, KC = ',KC
        ENDIF
        WRITE(6,*) 'DETECT: TIME STEP USED, DT = ',DT
      ENDIF
      CALL LCMSIX(IPLDET,' ',0)
      NXTYP = ' '
      CALL LCMNXT(IPLDET,NXTYP,NAMDET,NAMMY,LCM)
      FIRST = NXTYP
 20   CALL LCMNXT(IPLDET,NXTYP,NAMDET,NAMMY,LCM)
      CALL LCMLEN(IPLDET,NXTYP,ILONG,ITYLCM)
*     DIRECTORY
      IF (ITYLCM.EQ.0) THEN
        CALL LCMSIX(IPLDET,NXTYP,1)
        CALL LCMGET(IPLDET,'INFORMATION',INFO)
        CALL LCMGET(IPLDET,'SPECTRAL',BASE(ISPEC))
        NREP = INFO(2)
        CALL SETARA(BASE,NREP,IREP)
        IF( NXTYP(1:5).EQ.'PLATN' )THEN
          CALL SETARA(BASE,NREP-1,IFRACT)
          CALL LCMGET(IPLDET,'FRACTION',BASE(IFRACT))
          CALL SETARA(BASE,NREP-2,INVCST)
          CALL LCMGET(IPLDET,'INV-CONST',BASE(INVCST))
          CALL SETARA(BASE,NREP-2,IPDD)
          CALL SETARA(BASE,NREP-2,IAPD)
          CALL SETARA(BASE,NREP-2,IBPD)
        ENDIF
        NXDET = ' '
        CALL LCMNXT(IPLDET,NXDET,NAMDET,NAMMY,LCM)
        FIRST1 = NXDET
 40     CALL LCMNXT(IPLDET,NXDET,NAMDET,NAMMY,LCM)
        CALL LCMLEN(IPLDET,NXDET,ILONG,ITYLCM)
        IF (ITYLCM.EQ.0) THEN
           IF(IPRT.GT.3) WRITE(6,*) 'NAME DETECTOR ',NXDET
           CALL LCMSIX(IPLDET,NXDET,1)
           IHEX = 1
           IF(LHEX) THEN
             CALL LCMLEN(IPLDET,'NHEX',NHEX,ITHEX)
             IF(NHEX.EQ.0) CALL XABORT('XSRES: HEXAGON NUMBERS'
     +             //' NOT PRESENT IN DEVICE')
             CALL SETARA(IBASE,NHEX,IHEX)
             CALL LCMGET(IPLDET,'NHEX',IBASE(IHEX))
           ENDIF
           CALL LCMGET(IPLDET,'POSITION',DEVPOS)
           CALL LCMGET(IPLDET,'RESPON',BASE(IREP))
*
***********************************************************************
*
* FLUX AT DETECTOR SITE AS ARITHMETIC MEAN  94/10/10
* RESPONSE CALCULATION WITH DETECTOR SPECTRAL INFORMATION 96/04/04
*
* MAUVAISE APPROXIMATION DU FLUX MAIS LA SEULE POUR LE MOMENT 94/10/10
* CALCUL DE LA REPONSE EN TENANT COMPTE DU SPECTRE DU DETECTEUR 96/04/04
*
***********************************************************************
*
           CALL INTFLU(GEOTYP,LHEX,NXP1,NYP1,NZP1,NEL,NUN,
     +       BASE(IMESHX),BASE(IMESHY),BASE(IMESHZ),IBASE(IKEYF),
     +       BASE(IFLU),NGRP,BASE(ISPEC),DEVPOS,NHEX,IBASE(IHEX),
     +       RESP,IPRT)
*
*      DETECTOR RESPONSE CALCULATION
*
           IF( .NOT.LNORM )THEN
             PLNL = BASE(IREP)
             REF  = BASE(IREP+1)
             IF( NXTYP.EQ.'VANAD_REGUL' )THEN
               IF (KC.EQ.1) THEN
                 CALL SMVAND(DT,RESP,PLNL)
                 REF = PLNL
               ELSE
                 REF = RESP
               ENDIF
             ELSEIF( NXTYP(1:5).EQ.'PLATN' )THEN
               REGUL = .FALSE.
               DO 50 J=1,NREP-2
                 BASE(IPDD+J-1) = BASE(IREP+J+1)
  50           CONTINUE
               IF( NXTYP.EQ.'PLATN_REGUL' )THEN
                 REGUL = .TRUE.
               ENDIF
               CALL SMDRPD(DT,RESP,REF,KC,BASE(IPDD),REGUL,
     +           BASE(IFRACT),BASE(INVCST),NREP-2,BASE(IAPD),
     +           BASE(IBPD))
               DO 60 J=1,NREP-2
                 BASE(IREP+J+1) = BASE(IPDD+J-1)
  60           CONTINUE
             ELSEIF (NXTYP(1:5).EQ.'CHION') THEN
               IF( NREP.NE.3 )CALL XABORT('DETECT: ION CHAMBERS MUST '
     +           //'HAVE THREE STORED VALUES FOR RESPONSES')
               IF (KC.EQ.1) THEN
                 REF = BASE(IREP+2)
                 CALL SMCION(DT,RESP,REF,PLNL,TLG)
                 REF = TLG
               ELSE
                 BASE(IREP+2) = RESP
                 RESP =LOG10(RESP/BASE(IREP+2))
               ENDIF
             ENDIF
           ELSE
             REF = VNORM/RESP
             RESP = VNORM
           ENDIF
*
*          DETECTOR RESPONSE STORAGE
           BASE(IREP) = RESP
           BASE(IREP+1)  = REF
           IF(IPRT.GT.4) WRITE(6,*) 'RESP, REF ',RESP, REF
           CALL LCMPUT(IPLDET,'RESPON',NREP,2,BASE(IREP))
*
           CALL LCMSIX(IPLDET,' ',2)
        ENDIF
        IF(LHEX)  CALL RLSARA(IBASE(IHEX))
        IF(NXDET.EQ.FIRST1) GOTO 41
        GOTO 40
 41     CALL LCMSIX(IPLDET,' ',2)
        CALL RLSARA(BASE(IREP))
        IF( NXTYP(1:5).EQ.'PLATN' )THEN
         CALL RLSARA(BASE(IFRACT))
         CALL RLSARA(BASE(INVCST))
         CALL RLSARA(BASE(IPDD))
         CALL RLSARA(BASE(IAPD))
         CALL RLSARA(BASE(IBPD))
        ENDIF
      ENDIF
      IF (NXTYP.EQ.FIRST) GOTO 30
      GOTO 20
  30  CONTINUE
      CALL RLSARA(BASE(IMESHX))
      CALL RLSARA(BASE(IMESHY))
      CALL RLSARA(BASE(IMESHZ))
      CALL RLSARA(IBASE(IKEYF))
      CALL RLSARA(BASE(IFLU))
      CALL RLSARA(BASE(ISPEC))
      RETURN
      END
