*DECK DVPOST
      SUBROUTINE DVPOST(NAMDEV,GEOTYP,LHEX,IHEX,NHEX,MIX,DEVPOS,NX,
     +              NY,NZ,MESHX,MESHY,MESHZ,HX,HY,HZ,NXP1,NYP1,NZP1,
     +              TOTAL,ZNUSIG,ZSIGF,DIFFX,DIFFY,DIFFZ,SCAT,
     +              MAPR,MAPD,XFAC,NGRP,NL,NMIXT,LSIG,IPRT)
C
C THIS SUBOUTINE CALCULATES THE CELLS THE DEVICE AFFECTS
C AND MODIFIES THE CORRESPONDING CROSS SECTIONS
C
C INPUTS - NAMDEV : CHARACTER*12 NAME OF THE DEVICE TREATED
C          GEOTYP : TYPE OF GEOMETRY
C          LHEX  : FLAG FOR HEXAGONAL 3-D GEOMETRY
C          IHEX  : HEXAGON NUMBER VECTOR. DIM: NHEX
C          NHEX  : NUMBER OF HEXAGONS AFFECTED
C          NX : NUMBER OF NODES IN X DIRECTION
C          NY : NUMBER OF NODES IN Y DIRECTION
C          NZ : NUMBER OF NODES IN Z DIRECTION
C          NXP1 : NUMBER OF COORDINATES IN X DIRECTION
C          NYP1 : NUMBER OF COORDINATES IN Y DIRECTION
C          NZP1 : NUMBER OF COORDINATES IN Z DIRECTION
C          NGRP : NUMBER OF GROUPS
C          NMIXT : NUMBER OF MIX ON L_MACROLIB2
C          NL : ORDER OF LEGENDRE EXPANSION OF SCATTERING CROSS-SECTIONS
C          XFACC : CORRECTION FACTOR OF INCREMENTS FOR TYPE CONTROLLED
C          XFACP : CORRECTION FACTOR OF INCREMENTS FOR TYPE PERT
C          IPRT : PRINTING FLAG
C          MESHX MESHY MESHZ : X Y AND Z COORDINATES OF DISCRETISATION P
C          HX HY HZ : X Y ANZD Z WIDTHS OF NODES
C          MIX :  MIX NUMBER FOR EACH NODE
C          DEVPOS :  DEVICE POSITIONS IN CM IN THE CORE. DIM: 6
C          MAPR : MIX NUMBER OF PROPERTIES WITH DEVICE
C          MAPD : MIX NUMBER OF PROPERTIES WITHOUT DEVICE
C OUTPUTS
C          TOTAL ... CHI : NEW VALUES TO PUT BACK IN L_MACROLIB3
*
*-------------------------------------- AUTHORS: J. KOCLAS 94/04/10 ---
*                                      MODIFIED BY E. VARIN-95/12/14
C
*
      IMPLICIT NONE
*
*
      CHARACTER*12 NAMDEV
      INTEGER GEOTYP,NGRP,NMIXT,NL,IPRT
      INTEGER NX,NY,NZ,NXP1,NYP1,NZP1
      REAL MESHX(NXP1),MESHY(NYP1),MESHZ(NZP1),HX(NX),HY(NY),HZ(NZ)
      INTEGER MIX(*)
*
      REAL TOTAL((NMIXT+1),NGRP),ZNUSIG((NMIXT+1),NGRP),
     +   ZSIGF((NMIXT+1),NGRP)
      REAL DIFFX((NMIXT+1),NGRP),
     +   DIFFY((NMIXT+1),NGRP)
      REAL DIFFZ((NMIXT+1),NGRP)
      REAL SCAT((NMIXT+1),NL,NGRP,NGRP)
C
C DEVICE STRUCTURE
C
      REAL DEVPOS(6)
C LOCAL
      INTEGER I,J,K,I1,I2,J1,J2,K1,K2,IAM,MAPG,MAPR,MAPD
      INTEGER NHEX,IHEX(NHEX)
      LOGICAL LSIG,LHEX
*
*
      REAL X1,X2,Y1,Y2,Z1,Z2
      REAL VFX,VFY,VFZ,VF,PREC
      PARAMETER ( PREC= 1.E-4 )
*
*     FACTEUR DE CORRECTION DES SECT. EFFICACES
      REAL XFAC
*
      X1=DEVPOS(1)
      X2=DEVPOS(2)
      Y1=DEVPOS(3)
      Y2=DEVPOS(4)
      Z1=DEVPOS(5)
      Z2=DEVPOS(6)
*
      IF(.NOT.LHEX) THEN
        IF(X1.LE.MESHX(1)) X1=MESHX(1)
        IF(X2.LE.MESHX(1)) X2=MESHX(1)
        IF(X2.GT.MESHX(NXP1)) X2=MESHX(NXP1)
        IF(X1.GT.MESHX(NXP1)) X1=MESHX(NXP1)
*
        IF(GEOTYP.NE.6) THEN
          IF(Y1.LE.MESHY(1)) Y1=MESHY(1)
          IF(Y2.LE.MESHY(1)) Y2=MESHY(1)
          IF(Y2.GT.MESHY(NYP1)) Y2=MESHY(NYP1)
          IF(Y1.GT.MESHY(NYP1)) Y1=MESHY(NYP1)
        ENDIF
      ENDIF
*
      IF(Z1.LE.MESHZ(1)) Z1=MESHZ(1)
      IF(Z2.LE.MESHZ(1)) Z2=MESHZ(1)
      IF(Z2.GT.MESHZ(NZP1)) Z2=MESHZ(NZP1)
      IF(Z1.GT.MESHZ(NZP1)) Z1=MESHZ(NZP1)
*
      IF(.NOT.LHEX) THEN
        DO 105 I=1,NX
          IF(X1.GE.MESHX(I) .AND. X1.LE.MESHX(I+1)) THEN
            I1=I
          ENDIF
          IF(X2.GE.MESHX(I) .AND. X2.LE.MESHX(I+1)) THEN
            I2=I
            GO TO 110
          ENDIF
 105    CONTINUE
 110    CONTINUE
*
        IF(GEOTYP.NE.6) THEN
          DO 115 J=1,NY
            IF(Y1.GE.MESHY(J) .AND. Y1.LE.MESHY(J+1)) THEN
              J1=J
            ENDIF
            IF(Y2.GE.MESHY(J) .AND. Y2.LE.MESHY(J+1)) THEN
              J2=J
              GO TO 120
            ENDIF
 115      CONTINUE
 120      CONTINUE
        ELSE
          J1 = 1
          J2 = 1
        ENDIF
      ELSE
         J1 = 1
         J2 = 1
         I1 = 1
         I2 = NHEX
      ENDIF
*
      DO 125 K=1,NZ
        IF(Z1.GE.MESHZ(K) .AND. Z1.LE.MESHZ(K+1)) THEN
          K1=K
        ENDIF
        IF(Z2.GE.MESHZ(K) .AND. Z2.LE.MESHZ(K+1)) THEN
          K2=K
          GOTO 130
        ENDIF
 125  CONTINUE
 130  CONTINUE
*
      IF( IPRT.GT.4 )THEN
         WRITE(6,*) 'DEVICE=', NAMDEV,
     +              ' I1 I2 J1 J2 K1 K2 ',I1,I2,J1,J2,K1,K2
      ENDIF
*
      DO 141 K=K1,K2
       DO 142 J=J1,J2
         DO 143 I=I1,I2
*
           IF(LHEX) THEN
              IAM = (K-1)*NX+IHEX(I)
           ELSE
              IAM=(K-1)*NX*NY+(J-1)*NX+I
           ENDIF
           MAPG=MIX(IAM)
           IF( IPRT.GT.4 )THEN
             WRITE(6,*) 'DEVICE=', NAMDEV,
     +                  ' MAPG MAPR MAPD ',MAPG,MAPR,MAPD
           ENDIF
           IF(MAPG.NE.0) THEN
*
            VFZ= 0.0
            IF(Z1.GE.MESHZ(K) .AND. Z2.GT.MESHZ(K+1)) THEN
              VFZ=(MESHZ(K+1)-Z1)/HZ(K)
            ELSE IF(Z1.GE.MESHZ(K) .AND. Z2.LE.MESHZ(K+1)) THEN
              VFZ=(Z2-Z1)/HZ(K)
            ELSE IF(Z1.LT.MESHZ(K) .AND. Z2.GT.MESHZ(K+1)) THEN
              VFZ=1.
            ELSE IF(Z1.LT.MESHZ(K) .AND. Z2.LE.MESHZ(K+1)) THEN
              VFZ=(Z2-MESHZ(K))/HZ(K)
            ENDIF
*
            IF(LHEX) THEN
              VFY = 1.0
              VFX = 1.0
            ELSE
              IF(GEOTYP.NE.6) THEN
                VFY= 0.0
                IF(Y1.GE.MESHY(J) .AND. Y2.GT.MESHY(J+1)) THEN
                  VFY=(MESHY(J+1)-Y1)/HY(J)
                ELSE IF(Y1.GE.MESHY(J) .AND. Y2.LE.MESHY(J+1)) THEN
                  VFY=(Y2-Y1)/HY(J)
                ELSE IF(Y1.LT.MESHY(J) .AND. Y2.GT.MESHY(J+1)) THEN
                  VFY=1.
                ELSE IF(Y1.LT.MESHY(J) .AND. Y2.LE.MESHY(J+1)) THEN
                  VFY=(Y2-MESHY(J))/HY(J)
                ENDIF
*
                VFX= 0.0
                IF(X1.GE.MESHX(I) .AND. X2.GT.MESHX(I+1)) THEN
                  VFX=(MESHX(I+1)-X1)/HX(I)
                ELSE IF(X1.GE.MESHX(I) .AND. X2.LE.MESHX(I+1)) THEN
                  VFX=(X2-X1)/HX(I)
                ELSE IF(X1.LT.MESHX(I) .AND. X2.GT.MESHX(I+1)) THEN
                  VFX=1.
                ELSE IF(X1.LT.MESHX(I) .AND. X2.LE.MESHX(I+1)) THEN
                  VFX=(X2-MESHX(I))/HX(I)
                ENDIF
              ELSE
                VFY= 1.0
                VFX= 0.0
                IF(X1.GE.MESHX(I) .AND. X2.GT.MESHX(I+1)) THEN
                  VFX=(MESHX(I+1)**2-X1**2)/HX(I)
                ELSE IF(X1.GE.MESHX(I) .AND. X2.LE.MESHX(I+1)) THEN
                  VFX=(X2**2-X1**2)/HX(I)
                ELSE IF(X1.LT.MESHX(I) .AND. X2.GT.MESHX(I+1)) THEN
                  VFX=1.
                ELSE IF(X1.LT.MESHX(I) .AND. X2.LE.MESHX(I+1)) THEN
                  VFX=(X2**2-MESHX(I)**2)/HX(I)
                ENDIF
              ENDIF
            ENDIF
*
            VF=VFX*VFY*VFZ
            IF( IPRT.GT.4 .AND. VF.GT.PREC )THEN
              WRITE(6,*) 'DEVICE=', NAMDEV,' # ELEM=',IAM,
     +                  ' VOLUME FRACTION=',VF
            ENDIF
*
            IF(VF.GT.PREC) THEN
              CALL DEVRES(TOTAL,ZNUSIG,ZSIGF,DIFFX,DIFFY,DIFFZ,
     +          SCAT,MAPG,MAPR,MAPD,VF,XFAC,NGRP,NL,NMIXT,LSIG)
            ENDIF
*           fin de  MAPG NE 0
          ENDIF
*           fin de  I,J,K
 143    CONTINUE
 142   CONTINUE
 141  CONTINUE
*
      RETURN
      END
