*DECK FLDRIV
      SUBROUTINE FLDRIV(CMODUL,IPTRK,IPSYS,REC,NEL,NUN,MAT,VOL,IDL,
     1 NGRP,TITR,IPFLUX,EVECT)
*
*-----------------------------------------------------------------------
*
* DRIVER FOR THE SOLUTION OF THE NEUTRON FLUX AS AN EIGENVALUE PROBLEM.
*
* INPUT PARAMETERS:
*  CMODUL : NAME OF THE ASSEMBLY DOOR ('BIVAC' OR 'TRIVAC').
*  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPSYS  : L_SYSTEM POINTER TO SYSTEM MATRICES.
*  REC    : .TRUE. RECOVER THE EXISTING SOLUTION AS INITIAL ESTIMATE;
*           .FALSE. USE A UNIFORM INITIAL ESTIMATE.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : TOTAL NUMBER OF UNKNOWNS PER GROUP.
*  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*           DIMENSION MAT(NEL)
*  VOL    : VOLUMES. DIMENSION VOL(NEL)
*  IDL    : POSITION OF THE AVERAGE FLUX COMPONENT ASSOCIATED WITH
*           EACH VOLUME. DIMENSION IDL(NEL)
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  TITR   : TITLE.
*
* OUTPUT PARAMETER:
*  IPFLUX : L_FLUX POINTER TO THE SOLUTION.
*
* SCRATCH STORAGE:
*  EVECT  : DIMENSION EVECT(NUN,NGRP)
*
* INFORMATION RECOVERED FROM L_SYSTEM AT IPSYS:
*  'A  1  1'  : SYSTEM MATRIX RELATED TO FAST LEAKAGE AND REMOVAL.
*  'A  2  2'  : SYSTEM MATRIX RELATED TO THERMAL LEAKAGE AND REMOVAL.
*  'A  1  2'  : SYSTEM MATRIX RELATED TO UP-SCATTERING.
*  'A  2  1'  : SYSTEM MATRIX RELATED TO DOWN-SCATTERING.
*  'B  1  1'  : SYSTEM MATRIX RELATED TO FAST FISSION.
*  'B  1  2'  : SYSTEM MATRIX RELATED TO THERMAL FISSION.
*
*---------------------------------------------- A. HEBERT 13/07/93 -----
*
      COMMON BASE(1)
      CHARACTER TEXT4*4,CMODUL*12,TITR*72,TEXT12*12
      DOUBLE PRECISION DFLOTT
      LOGICAL REC,ADJ,MONIC,RAND
      DIMENSION MAT(*),VOL(*),IDL(*),EVECT(NUN,*),IFPARM(5)
*
* SET FLUX PARAMETERS
*
      IF(REC) THEN
        CALL LCMGET(IPFLUX,'FLUX-PARAM',IFPARM)
      ELSE
        DO 1 I=1,5
          IFPARM(I) = 0
   1    CONTINUE
        IFPARM(1) = NGRP
        IFPARM(2) = NUN
      ENDIF
*
* READ THE INPUT DATA.
*     DEFAULT OPTIONS.
      MAXX0=200
      ICL1=3
      ICL2=3
      IMPX=1
      IMPH=0
      NADI=1
      EPS2=1.0E-4
      ADJ=.FALSE.
      MONIC=.FALSE.
      RAND=.FALSE.
*
   10 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.EQ.10) GO TO 170
   15 IF(INDIC.NE.3) CALL XABORT('FLDRIV: CHARACTER DATA EXPECTED(1).')
*
      IF(TEXT4.NE.'MAX') GO TO 20
      CALL REDGET(INDIC,MAXX0,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('FLDRIV: INTEGER DATA EXPECTED(1).')
      GO TO 10
*
   20 IF(TEXT4.NE.'VAR1') GO TO 30
      CALL REDGET(INDIC,ICL1,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('FLDRIV: INTEGER DATA EXPECTED(2).')
      CALL REDGET(INDIC,ICL2,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('FLDRIV: INTEGER DATA EXPECTED(3).')
      GO TO 10
*
   30 IF(TEXT4.NE.'EDIT') GO TO 35
      CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('FLDRIV: INTEGER DATA EXPECTED(4).')
      GO TO 10
*
   35 IF(TEXT4.NE.'ADI') GO TO 40
      CALL REDGET(INDIC,NADI,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('FLDRIV: INTEGER DATA EXPECTED(5).')
      GO TO 10
*
   40 IF(TEXT4.NE.'PREC') GO TO 50
      CALL REDGET(INDIC,NITMA,EPS2,TEXT4,DFLOTT)
      IF(INDIC.NE.2) CALL XABORT('FLDRIV: REAL DATA EXPECTED(1).')
      GO TO 10
*
   50 IF(TEXT4.NE.'ADJ') GO TO 60
      ADJ=.TRUE.
      GO TO 10
*
   60 IF(TEXT4.NE.'MONI') GO TO 65
      MONIC=.TRUE.
      CALL REDGET(INDIC,LMOD,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('FLDRIV: INTEGER DATA EXPECTED(7).')
      GO TO 10
*
   65 IF(TEXT4.NE.'RAND') GO TO 70
      RAND=.TRUE.
      GO TO 10
*
   70 IF(TEXT4.NE.'HIST') GO TO 160
      CALL REDGET(INDIC,IMPH,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('FLDRIV: INTEGER DATA EXPECTED(8).')
      GO TO 10
*
  160 IF(TEXT4.NE.';') CALL XABORT('FLDRIV: '//TEXT4//' IS AN INVALID '
     1 //'KEY WORD.')
*
* FLUXES INITIALIZATION.
  170 IF(REC.AND.(IMPH.EQ.0)) THEN
         DO 180 IGR=1,NGRP
           WRITE(TEXT12,'(4HFLUX,I3)') IGR
           CALL LCMGET(IPFLUX,TEXT12,EVECT(1,IGR))
  180    CONTINUE
      ELSE
*        INITIAL ESTIMATE OF THE DIRECT FLUXES.
         DO 190 IGR=1,NGRP
         DO 190 IND1=1,NUN
  190    EVECT(IND1,IGR)=1.0
      ENDIF
*
      IF(MONIC) THEN
         IF(ADJ) CALL XABORT('FLDRIV: NO FONDAMENTAL ADJOINT'
     1      //' CALCULATION WITH HARMONIC CALCULATION')
*        BI-ORTHOGONAL HARMONIC CALCULATION.
         IF(CMODUL.NE.'TRIVAC') CALL XABORT('FLDRIV: HARMONIC CALCULAT'
     1   //'ION IS ONLY POSSIBLE WITH TRIVAC.')
         IFPARM(3) = 1
         IFPARM(4) = LMOD
         CALL SETARA(BASE,LMOD,IFKEFF)
         CALL SETARA(BASE,NUN*NGRP*LMOD,IEV)
         CALL SETARA(BASE,NUN*NGRP*LMOD,IAD)
         CALL SETARA(BASE,NUN,IAGAR)
         CALL SETARA(BASE,NUN*NGRP,IGRAD1)
         CALL SETARA(BASE,NUN*NGRP,IGRAD2)
         CALL SETARA(BASE,NUN*NGRP,IVEA1)
         CALL SETARA(BASE,NUN*NGRP,IVEA2)
         CALL SETARA(BASE,NUN*NGRP,IVEA3)
         CALL SETARA(BASE,NUN*NGRP,IVEB1)
         CALL SETARA(BASE,NUN*NGRP,IVEB2)
         CALL SETARA(BASE,NUN*NGRP,IVEB3)
         CALL FLUMON(IPTRK,IPSYS,IPFLUX,NUN,NGRP,LMOD,ICL1,ICL2,IMPX,
     1   IMPH,TITR,EPS2,NADI,MAXX0,RAND,BASE(IFKEFF),BASE(IEV),
     2   BASE(IAD),BASE(IAGAR),BASE(IGRAD1),BASE(IGRAD2),BASE(IVEA1),
     3   BASE(IVEA2),BASE(IVEA3),BASE(IVEB1),BASE(IVEB2),BASE(IVEB3))
         CALL RLSARA(BASE(IVEB3))
         CALL RLSARA(BASE(IVEB2))
         CALL RLSARA(BASE(IVEB1))
         CALL RLSARA(BASE(IVEA3))
         CALL RLSARA(BASE(IVEA2))
         CALL RLSARA(BASE(IVEA1))
         CALL RLSARA(BASE(IGRAD2))
         CALL RLSARA(BASE(IGRAD1))
         CALL RLSARA(BASE(IAGAR))
         DO 200 IMOD=1,LMOD
         IOF1=(IMOD-1)*NUN*NGRP
         DO 200 IGR=1,NGRP
         IOF2=IOF1+(IGR-1)*NUN
         CALL TRIFLX(IPTRK,NEL,NUN,BASE(IEV+IOF2),MAT,VOL,IDL)
         CALL TRIFLX(IPTRK,NEL,NUN,BASE(IAD+IOF2),MAT,VOL,IDL)
         WRITE(TEXT12,'(4HFLUX,2I3)') IGR,IMOD
         CALL LCMPUT(IPFLUX,TEXT12,NUN,2,BASE(IEV+IOF2))
         WRITE(TEXT12,'(5HAFLUX,2I3)') IGR,IMOD
         CALL LCMPUT(IPFLUX,TEXT12,NUN,2,BASE(IAD+IOF2))
  200    CONTINUE
         CALL RLSARA(BASE(IAD))
         CALL RLSARA(BASE(IEV))
         CALL RLSARA(BASE(IFKEFF))
      ELSE
*        DIRECT NEUTRON FLUX CALCULATION.
         IF(CMODUL.EQ.'BIVAC') THEN
            CALL SETARA(BASE,NUN*NGRP,IGRAD1)
            CALL SETARA(BASE,NUN*NGRP,IGRAD2)
            CALL SETARA(BASE,NUN*NGRP,IGAR1)
            CALL SETARA(BASE,NUN*NGRP,IGAR2)
            CALL SETARA(BASE,NUN*NGRP,IGAR3)
            CALL SETARA(BASE,NUN*NGRP,IGAF1)
            CALL SETARA(BASE,NUN*NGRP,IGAF2)
            CALL SETARA(BASE,NUN*NGRP,IGAF3)
            CALL SETARA(BASE,NUN,IWORK)
            CALL FLUSMB(IPTRK,IPSYS,IPFLUX,NUN,NGRP,ICL1,ICL2,IMPX,
     1      IMPH,TITR,EPS2,MAXX0,EVECT,FKEFF,BASE(IGRAD1),BASE(IGRAD2),
     2      BASE(IGAR1),BASE(IGAR2),BASE(IGAR3),BASE(IGAF1),BASE(IGAF2),
     3      BASE(IGAF3),BASE(IWORK))
            CALL RLSARA(BASE(IWORK))
            CALL RLSARA(BASE(IGAF3))
            CALL RLSARA(BASE(IGAF2))
            CALL RLSARA(BASE(IGAF1))
            CALL RLSARA(BASE(IGAR3))
            CALL RLSARA(BASE(IGAR2))
            CALL RLSARA(BASE(IGAR1))
            CALL RLSARA(BASE(IGRAD2))
            CALL RLSARA(BASE(IGRAD1))
            DO 210 IGR=1,NGRP
            CALL BIVFLX(IPTRK,NEL,NUN,EVECT(1,IGR),MAT,VOL,IDL)
  210       CONTINUE
         ELSE IF(CMODUL.EQ.'TRIVAC') THEN
            CALL SETARA(BASE,NUN,IAGAR)
            CALL SETARA(BASE,NUN*NGRP,IGRAD1)
            CALL SETARA(BASE,NUN*NGRP,IGRAD2)
            CALL SETARA(BASE,NUN*NGRP,IGAR1)
            CALL SETARA(BASE,NUN*NGRP,IGAR2)
            CALL SETARA(BASE,NUN*NGRP,IGAR3)
            CALL SETARA(BASE,NUN,IGAF1)
            CALL SETARA(BASE,NUN,IGAF2)
            CALL SETARA(BASE,NUN,IGAF3)
            CALL FLUDIR(IPTRK,IPSYS,IPFLUX,NUN,NGRP,ICL1,ICL2,IMPX,IMPH,
     1      TITR,EPS2,NADI,MAXX0,EVECT,FKEFF,BASE(IAGAR),BASE(IGRAD1),
     2      BASE(IGRAD2),BASE(IGAR1),BASE(IGAR2),BASE(IGAR3),
     3      BASE(IGAF1),BASE(IGAF2),BASE(IGAF3))
            CALL RLSARA(BASE(IGAF3))
            CALL RLSARA(BASE(IGAF2))
            CALL RLSARA(BASE(IGAF1))
            CALL RLSARA(BASE(IGAR3))
            CALL RLSARA(BASE(IGAR2))
            CALL RLSARA(BASE(IGAR1))
            CALL RLSARA(BASE(IGRAD2))
            CALL RLSARA(BASE(IGRAD1))
            CALL RLSARA(BASE(IAGAR))
            DO 220 IGR=1,NGRP
            CALL TRIFLX(IPTRK,NEL,NUN,EVECT(1,IGR),MAT,VOL,IDL)
  220       CONTINUE
         ENDIF
         CALL LCMPUT(IPFLUX,'K-EFFECTIVE',1,2,FKEFF)
         DO 230 IGR=1,NGRP
         WRITE(TEXT12,'(4HFLUX,I3)') IGR
         CALL LCMPUT(IPFLUX,TEXT12,NUN,2,EVECT(1,IGR))
  230    CONTINUE
      ENDIF
*
* ADJOINT NEUTRON FLUX CALCULATION.
      IF(ADJ) THEN
         IF(CMODUL.NE.'TRIVAC') CALL XABORT('FLDRIV: ADJOINT CALCULATI'
     1   //'ON IS ONLY POSSIBLE WITH TRIVAC.')
*        ADJOINT FLUX INITIALIZATION.
         IF(REC.AND.(IMPH.EQ.0)) THEN
            IF( IFPARM(3).NE.1 )CALL XABORT('FLDRIV: L_FLUX DOES NOT '
     +         //'CONTAIN ADJOINT FLUX')
            IF( IFPARM(4).NE.0 )CALL XABORT('FLDRIV: L_FLUX CONTAINS'
     +         //'HARMONICS OF FLUX, NO ADJOINT')
            DO 240 IGR=1,NGRP
              WRITE(TEXT12,'(5HAFLUX,I3)') IGR
              CALL LCMGET(IPFLUX,TEXT12,EVECT(1,IGR))
  240       CONTINUE
         ELSE
            IFPARM(3) = 1
*           INITIAL ESTIMATE OF THE ADJOINT FLUXES.
            DO 250 IGR=1,NGRP
            DO 250 IND1=1,NUN
  250       EVECT(IND1,IGR)=1.0
         ENDIF
*
         CALL SETARA(BASE,NUN,IAGAR)
         CALL SETARA(BASE,NUN*NGRP,IGRAD1)
         CALL SETARA(BASE,NUN*NGRP,IGRAD2)
         CALL SETARA(BASE,NUN*NGRP,IGAR1)
         CALL SETARA(BASE,NUN*NGRP,IGAR2)
         CALL SETARA(BASE,NUN*NGRP,IGAR3)
         CALL SETARA(BASE,NUN,IGAF1)
         CALL SETARA(BASE,NUN,IGAF2)
         CALL SETARA(BASE,NUN,IGAF3)
         CALL FLUADJ(IPTRK,IPSYS,NUN,NGRP,ICL1,ICL2,IMPX,EPS2,NADI,
     1   MAXX0,EVECT,FKEFF,BASE(IAGAR),BASE(IGRAD1),BASE(IGRAD2),
     2   BASE(IGAR1),BASE(IGAR2),BASE(IGAR3),BASE(IGAF1),BASE(IGAF2),
     3   BASE(IGAF3))
         CALL RLSARA(BASE(IGAF3))
         CALL RLSARA(BASE(IGAF2))
         CALL RLSARA(BASE(IGAF1))
         CALL RLSARA(BASE(IGAR3))
         CALL RLSARA(BASE(IGAR2))
         CALL RLSARA(BASE(IGAR1))
         CALL RLSARA(BASE(IGRAD2))
         CALL RLSARA(BASE(IGRAD1))
         CALL RLSARA(BASE(IAGAR))
         DO 260 IGR=1,NGRP
         CALL TRIFLX(IPTRK,NEL,NUN,EVECT(1,IGR),MAT,VOL,IDL)
         WRITE(TEXT12,'(5HAFLUX,I3)') IGR
         CALL LCMPUT(IPFLUX,TEXT12,NUN,2,EVECT(1,IGR))
  260    CONTINUE
      ENDIF
      CALL LCMPUT(IPFLUX,'FLUX-PARAM',5,1,IFPARM)
      RETURN
      END
