*DECK FLUADJ
      SUBROUTINE FLUADJ(IPTRK,IPSYS,NUN,NGRP,ICL1,ICL2,IMPX,EPS2,NADI,
     1 MAXX0,ADECT,FKEFF,AGAR,GRAD1,GRAD2,GAR1,GAR2,GAR3,GAF1,GAF2,GAF3)
*
*-----------------------------------------------------------------------
*
* SOLUTION OF A MULTIGROUP EIGENVALUE SYSTEM FOR THE CALCULATION OF THE
* ADJOINT NEUTRON FLUX IN TRIVAC. USE THE PRECONDITIONNED POWER METHOD
* WITH A TWO PARAMETER SVAT ACCELERATION TECHNIQUE.
*
* INPUT PARAMETERS:
*  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPSYS  : L_SYSTEM POINTER TO SYSTEM MATRICES.
*  NUN    : NUMBER OF UNKNOWNS IN EACH ENERGY GROUP.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  ICL1   : NUMBER OF FREE IRETATIONS IN ONE CYCLE OF THE INVERSE POWER
*           METHOD.
*  ICL2   : NUMBER OF ACCELERATED ITERATIONS IN ONE CYCLE.
*  IMPX   : PRINT PARAMETER. =0: NO PRINT ; =1: MINIMUM PRINTING ;
*           =2: ITERATION HISTORY IS PRINTED. =3: SOLUTION IS PRINTED.
*  TITR   : CHARACTER*72 TITLE.
*  EPS2   : CONVERGENCE CRITERIA FOR THE FLUX.
*  NADI   : NUMBER OF INNER ADI ITERATIONS PER OUTER ITERATION.
*  MAXX0  : MAXIMUM NUMBER OF OUTER ITERATIONS.
*  ADECT  : INITIAL ESTIMATE OF THE UNKNOWN VECTOR.
*           DIMENSION ADECT(NUN,NGRP)
*
* OUTPUT PARAMETERS:
*  FKEFF  : EFFECTIVE MULTIPLICATION FACTOR
*  ADECT  : CONVERGED UNKNOWN VECTOR. DIMENSION ADECT(NUN,NGRP)
*
* SCRATCH STORAGE:
*  AGAR   : DIMENSION AGAR(LL4) WHERE LL4 IS THE ORDER OF THE SYSTEM
*           MATRICES. LL4.LE.NUN
*  GRAD1  : DIMENSION GRAD1(NUN,NGRP)
*  GRAD2  : DIMENSION GRAD2(NUN,NGRP)
*  GAR1   : DIMENSION GAR1(NUN,NGRP)
*  GAR2   : DIMENSION GAR2(NUN,NGRP)
*  GAR3   : DIMENSION GAR3(NUN,NGRP)
*  GAF1   : DIMENSION GAF1(NUN)
*  GAF2   : DIMENSION GAF2(NUN)
*  GAF3   : DIMENSION GAF3(NUN)
*
* REFERENCE:
*  A. HEBERT, 'PRECONDITIONING THE POWER METHOD FOR REACTOR CALCU-
*  LATIONS', NUCL. SCI. ENG., 94, 1 (1986).
*
*-------------------------------------- AUTHOR: A. HEBERT ; 17/06/94 ---
*
      CHARACTER*12 TEXT12,NAMLCM,NAMMY
      LOGICAL LCM,LOGTES
      DOUBLE PRECISION AEAE,AEAG,AEAH,AGAG,AGAH,AHAH,BEBE,BEBG,BEBH,
     1 BGBG,BGBH,BHBH,AEBE,AEBG,AEBH,AGBE,AGBG,AGBH,AHBE,AHBG,AHBH,
     2 X,DXDA,DXDB,Y,DYDA,DYDB,Z,DZDA,DZDB,F,D2F(2,3),EVAL,ALP,BET
      DIMENSION ADECT(NUN,*),AGAR(*),GRAD1(NUN,*),GRAD2(NUN,*),
     1 GAR1(NUN,*),GAR2(NUN,*),GAR3(NUN,*),GAF1(*),GAF2(*),GAF3(*)
      COMMON BASE(1)
      DATA EPS1,MMAXX/1.0E-5,250/
      MAXX=MIN(MAXX0,MMAXX)
      CALL MTOPEN(IMPX,IPTRK,LL4)
      IF(LL4.GT.NUN) CALL XABORT('FLUADJ: INVALID NUMBER OF UNKNOWNS.')
      TEXT12=' '
      CALL LCMNXT(IPSYS,TEXT12,NAMLCM,NAMMY,LCM)
*
* PRECONDITIONED POWER METHOD.
      EVAL=1.0D0
      VVV=0.0
      ISTART=1
      NNADI=NADI
      IF(IMPX.GE.1) WRITE (6,600) NADI
      IF(IMPX.GE.2) WRITE (6,610)
      DO 30 IGR=1,NGRP
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      CALL MTLDLM(TEXT12,IPTRK,IPSYS,ADECT(1,IGR),GAR1(1,IGR))
      DO 30 JGR=1,NGRP
      IF(JGR.EQ.IGR) GO TO 30
      WRITE(TEXT12,'(1HA,2I3)') JGR,IGR
      CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 30
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS,TEXT12)
         DO 10 I=1,LL4
   10    GAR1(I,IGR)=GAR1(I,IGR)-BASE(IAGAR+I-1)*ADECT(I,JGR)
      ELSE
         CALL LCMGET(IPSYS,TEXT12,AGAR)
         DO 20 I=1,LL4
   20    GAR1(I,IGR)=GAR1(I,IGR)-AGAR(I)*ADECT(I,JGR)
      ENDIF
   30 CONTINUE
*
      M=0
   40 M=M+1
*
* EIGENVALUE EVALUATION.
      AEBE=0.0D0
      BEBE=0.0D0
      DO 90 IGR=1,NGRP
      DO 50 I=1,LL4
   50 GAF1(I)=0.0
      DO 80 JGR=1,NGRP
      WRITE(TEXT12,'(1HB,2I3)') JGR,IGR
      CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 80
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS,TEXT12)
         DO 60 I=1,LL4
   60    GAF1(I)=GAF1(I)+BASE(IAGAR+I-1)*ADECT(I,JGR)
      ELSE
         CALL LCMGET(IPSYS,TEXT12,AGAR)
         DO 70 I=1,LL4
   70    GAF1(I)=GAF1(I)+AGAR(I)*ADECT(I,JGR)
      ENDIF
   80 CONTINUE
      DO 90 I=1,LL4
      AEBE=AEBE+GAR1(I,IGR)*GAF1(I)
      BEBE=BEBE+GAF1(I)**2
   90 GRAD1(I,IGR)=GAF1(I)
      EVAL=AEBE/BEBE
*
* DIRECTION EVALUATION.
      DO 140 IGR=NGRP,1,-1
      DO 100 I=1,LL4
  100 GRAD1(I,IGR)=EVAL*GRAD1(I,IGR)-GAR1(I,IGR)
      DO 130 JGR=NGRP,IGR+1,-1
      WRITE(TEXT12,'(1HA,2I3)') JGR,IGR
      CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 130
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS,TEXT12)
         DO 110 I=1,LL4
  110    GRAD1(I,IGR)=GRAD1(I,IGR)+BASE(IAGAR+I-1)*GRAD1(I,JGR)
      ELSE
         CALL LCMGET(IPSYS,TEXT12,AGAR)
         DO 120 I=1,LL4
  120    GRAD1(I,IGR)=GRAD1(I,IGR)+AGAR(I)*GRAD1(I,JGR)
      ENDIF
  130 CONTINUE
*
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      IF(NNADI.EQ.1) THEN
         CALL MTLDLS(TEXT12,IPTRK,IPSYS,GRAD1(1,IGR))
      ELSE
         CALL TRIADI(TEXT12,IPTRK,IPSYS,LL4,GRAD1(1,IGR),NNADI)
      ENDIF
  140 CONTINUE
*
* DISPLACEMENT EVALUATION.
      F=0.0D0
      DELS=ABS((EVAL-VVV)/EVAL)
      VVV=EVAL
*
* EVALUATION OF THE TWO ACCELERATION PARAMETERS ALP AND BET.
      ALP=1.0D0
      BET=0.0D0
      N=0
      AEAE=0.0D0
      AEAG=0.0D0
      AEAH=0.0D0
      AGAG=0.0D0
      AGAH=0.0D0
      AHAH=0.0D0
      BEBG=0.0D0
      BEBH=0.0D0
      BGBG=0.0D0
      BGBH=0.0D0
      BHBH=0.0D0
      AEBG=0.0D0
      AEBH=0.0D0
      AGBE=0.0D0
      AGBG=0.0D0
      AGBH=0.0D0
      AHBE=0.0D0
      AHBG=0.0D0
      AHBH=0.0D0
      DO 170 IGR=1,NGRP
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      CALL MTLDLM(TEXT12,IPTRK,IPSYS,GRAD1(1,IGR),GAR2(1,IGR))
      DO 170 JGR=1,NGRP
      IF(JGR.EQ.IGR) GO TO 170
      WRITE(TEXT12,'(1HA,2I3)') JGR,IGR
      CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 170
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS,TEXT12)
         DO 150 I=1,LL4
  150    GAR2(I,IGR)=GAR2(I,IGR)-BASE(IAGAR+I-1)*GRAD1(I,JGR)
      ELSE
         CALL LCMGET(IPSYS,TEXT12,AGAR)
         DO 160 I=1,LL4
  160    GAR2(I,IGR)=GAR2(I,IGR)-AGAR(I)*GRAD1(I,JGR)
      ENDIF
  170 CONTINUE
      IF(1+MOD(M-ISTART,ICL1+ICL2).GT.ICL1) THEN
         DO 220 IGR=1,NGRP
         DO 180 I=1,LL4
         GAF1(I)=0.0
         GAF2(I)=0.0
  180    GAF3(I)=0.0
         DO 210 JGR=1,NGRP
         WRITE(TEXT12,'(1HB,2I3)') JGR,IGR
         CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 210
         IF(LCM) THEN
            IAGAR=LCMIOF(IPSYS,TEXT12)
            DO 190 I=1,LL4
            GAF1(I)=GAF1(I)+BASE(IAGAR+I-1)*ADECT(I,JGR)
            GAF2(I)=GAF2(I)+BASE(IAGAR+I-1)*GRAD1(I,JGR)
  190       GAF3(I)=GAF3(I)+BASE(IAGAR+I-1)*GRAD2(I,JGR)
         ELSE
            CALL LCMGET(IPSYS,TEXT12,AGAR)
            DO 200 I=1,LL4
            GAF1(I)=GAF1(I)+AGAR(I)*ADECT(I,JGR)
            GAF2(I)=GAF2(I)+AGAR(I)*GRAD1(I,JGR)
  200       GAF3(I)=GAF3(I)+AGAR(I)*GRAD2(I,JGR)
         ENDIF
  210    CONTINUE
         DO 220 I=1,LL4
*        COMPUTE (A ,A )
         AEAE=AEAE+GAR1(I,IGR)**2
         AEAG=AEAG+GAR1(I,IGR)*GAR2(I,IGR)
         AEAH=AEAH+GAR1(I,IGR)*GAR3(I,IGR)
         AGAG=AGAG+GAR2(I,IGR)**2
         AGAH=AGAH+GAR2(I,IGR)*GAR3(I,IGR)
         AHAH=AHAH+GAR3(I,IGR)**2
*        COMPUTE (B ,B )
         BEBG=BEBG+GAF1(I)*GAF2(I)
         BEBH=BEBH+GAF1(I)*GAF3(I)
         BGBG=BGBG+GAF2(I)**2
         BGBH=BGBH+GAF2(I)*GAF3(I)
         BHBH=BHBH+GAF3(I)**2
*        COMPUTE (A ,B )
         AEBG=AEBG+GAR1(I,IGR)*GAF2(I)
         AEBH=AEBH+GAR1(I,IGR)*GAF3(I)
         AGBE=AGBE+GAR2(I,IGR)*GAF1(I)
         AGBG=AGBG+GAR2(I,IGR)*GAF2(I)
         AGBH=AGBH+GAR2(I,IGR)*GAF3(I)
         AHBE=AHBE+GAR3(I,IGR)*GAF1(I)
         AHBG=AHBG+GAR3(I,IGR)*GAF2(I)
  220    AHBH=AHBH+GAR3(I,IGR)*GAF3(I)
*
  250    N=N+1
         IF(N.GT.10) CALL XABORT('FLUADJ: FAILURE OF THE OVERRELAXATIO'
     1   //'N PARAMETERS CALCULATION.')
*        COMPUTE X(M+1)
         X=BEBE+ALP*ALP*BGBG+BET*BET*BHBH+2.0D0*(ALP*BEBG+BET*BEBH
     1   +ALP*BET*BGBH)
         DXDA=2.0D0*(BEBG+ALP*BGBG+BET*BGBH)
         DXDB=2.0D0*(BEBH+ALP*BGBH+BET*BHBH)
*        COMPUTE Y(M+1)
         Y=AEAE+ALP*ALP*AGAG+BET*BET*AHAH+2.0D0*(ALP*AEAG+BET*AEAH
     1   +ALP*BET*AGAH)
         DYDA=2.0D0*(AEAG+ALP*AGAG+BET*AGAH)
         DYDB=2.0D0*(AEAH+ALP*AGAH+BET*AHAH)
*        COMPUTE Z(M+1)
         Z=AEBE+ALP*ALP*AGBG+BET*BET*AHBH+ALP*(AEBG+AGBE)
     1   +BET*(AEBH+AHBE)+ALP*BET*(AGBH+AHBG)
         DZDA=AEBG+AGBE+2.0D0*ALP*AGBG+BET*(AGBH+AHBG)
         DZDB=AEBH+AHBE+ALP*(AGBH+AHBG)+2.0D0*BET*AHBH
*        COMPUTE F(M+1)
         F=X*Y-Z*Z
         D2F(1,1)=2.0D0*(BGBG*Y+DXDA*DYDA+X*AGAG-DZDA**2-2.0D0*Z*AGBG)
         D2F(1,2)=2.0D0*BGBH*Y+DXDA*DYDB+DXDB*DYDA+2.0D0*X*AGAH
     1   -2.0D0*DZDA*DZDB-2.0D0*Z*(AGBH+AHBG)
         D2F(2,2)=2.0D0*(BHBH*Y+DXDB*DYDB+X*AHAH-DZDB**2-2.0D0*Z*AHBH)
         D2F(2,1)=D2F(1,2)
         D2F(1,3)=DXDA*Y+X*DYDA-2.0D0*Z*DZDA
         D2F(2,3)=DXDB*Y+X*DYDB-2.0D0*Z*DZDB
*        SOLUTION OF A LINEAR SYSTEM.
         CALL ALSBD(2,1,D2F,IER,2)
         IF(IER.NE.0) CALL XABORT('FLUADJ: SINGULAR MATRIX.')
         ALP=ALP-D2F(1,3)
         BET=BET-D2F(2,3)
         IF((ABS(D2F(1,3)).LE.1.0D-4).AND.(ABS(D2F(2,3)).LE.1.0D-4))
     1   GO TO 260
         GO TO 250
  260    BET=BET/ALP
         IF((ALP.LT.1.0D0).AND.(ALP.GT.0.0D0)) THEN
            ALP=1.0D0
            BET=0.0D0
         ELSE IF(ALP.LE.0.0D0) THEN
            ISTART=M+1
            ALP=1.0D0
            BET=0.0D0
         ENDIF
         DO 270 IGR=1,NGRP
         DO 270 I=1,LL4
         GRAD1(I,IGR)=ALP*(GRAD1(I,IGR)+BET*GRAD2(I,IGR))
  270    GAR2(I,IGR)=ALP*(GAR2(I,IGR)+BET*GAR3(I,IGR))
      ENDIF
*
      LOGTES=(M.LT.ICL1).OR.(MOD(M,ICL1+ICL2).EQ.ICL1)
      IF(LOGTES.AND.(DELS.LE.EPS1))THEN
         DELT=0.0
         DO 290 IGR=1,NGRP
         DELN=0.0
         DELD=0.0
         DO 280 I=1,LL4
         DELN=MAX(DELN,ABS(GRAD1(I,IGR)))
         ADECT(I,IGR)=ADECT(I,IGR)+GRAD1(I,IGR)
         DELD=MAX(DELD,ABS(ADECT(I,IGR)))
         GAR1(I,IGR)=GAR1(I,IGR)+GAR2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
  280    GAR3(I,IGR)=GAR2(I,IGR)
         IF(DELD.NE.0.0) DELT=MAX(DELT,DELN/DELD)
  290    CONTINUE
         IF(IMPX.GE.2) WRITE (6,615) M,AEAE,AEAG,AEAH,AGAG,AGAH,AHAH,
     1   BEBE,ALP,BET,EVAL,F,DELS,DELT,N,BEBG,BEBH,BGBG,BGBH,BHBH,AEBE,
     2   AEBG,AEBH,AGBE,AGBG,AGBH,AHBE,AHBG,AHBH
         IF(DELT.LE.EPS2) GO TO 310
      ELSE
         DO 300 IGR=1,NGRP
         DO 300 I=1,LL4
         ADECT(I,IGR)=ADECT(I,IGR)+GRAD1(I,IGR)
         GAR1(I,IGR)=GAR1(I,IGR)+GAR2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
  300    GAR3(I,IGR)=GAR2(I,IGR)
         IF(IMPX.GE.2) WRITE (6,620) M,AEAE,AEAG,AEAH,AGAG,AGAH,AHAH,
     1   BEBE,ALP,BET,EVAL,F,DELS,N,BEBG,BEBH,BGBG,BGBH,BHBH,AEBE,
     2   AEBG,AEBH,AGBE,AGBG,AGBH,AHBE,AHBG,AHBH
      ENDIF
      IF(M.EQ.1) TEST=DELS
      IF((M.GT.5).AND.(DELS.GT.TEST)) CALL XABORT('FLUADJ: CONVERGENCE'
     1 //' FAILURE.')
      IF(M.GE.MAXX) THEN
         WRITE (6,690)
         GO TO 310
      ENDIF
      IF(MOD(M,36).EQ.0) THEN
         ISTART=M+1
         NNADI=NNADI+1
         WRITE (6,700) NNADI
      ENDIF
      GO TO 40
*
* SOLUTION EDITION.
  310 FKEFF=1.0D0/EVAL
      IF(IMPX.EQ.1) WRITE (6,640) M
      IF(IMPX.GE.1) THEN
         WRITE (6,670) FKEFF
      ENDIF
      IF(IMPX.EQ.3) THEN
         DO 320 IGR=1,NGRP
         WRITE (6,680) IGR,(ADECT(I,IGR),I=1,LL4)
  320    CONTINUE
      ENDIF
      RETURN
*
  600 FORMAT(1H1/50H FLUADJ: ITERATIVE PROCEDURE BASED ON PRECONDITION,
     1 17HED POWER METHOD (,I2,37H ADI ITERATIONS PER OUTER ITERATION)./
     2 9X,17HADJOINT EQUATION.)
  610 FORMAT(//5X,17HBILINEAR PRODUCTS,48X,5HALPHA,3X,4HBETA,3X,
     1 12HEIGENVALUE..,12X,8HACCURACY,11(1H.),2X,1HN)
  615 FORMAT(1X,I3,1P,7E9.1,0P,2F8.3,E14.6,3E10.2,I4/
     1 (4X,1P,7E9.1))
  620 FORMAT(1X,I3,1P,7E9.1,0P,2F8.3,E14.6,2E10.2,10X,I4/
     1 (4X,1P,7E9.1))
  640 FORMAT(/23H FLUADJ: CONVERGENCE IN,I4,12H ITERATIONS.)
  650 FORMAT(/53H FLUADJ: CPU TIME USED TO SOLVE THE TRIANGULAR LINEAR,
     1 10H SYSTEMS =,F9.3/23X,34HTO COMPUTE THE BILINEAR PRODUCTS =,
     2 F9.3,20X,16HTOTAL CPU TIME =,F9.3)
  670 FORMAT(//42H FLUADJ: EFFECTIVE MULTIPLICATION FACTOR =,1P,E17.10/)
  680 FORMAT(//53H FLUADJ: ADJOINT EIGENVECTOR CORRESPONDING TO THE GRO,
     1 2HUP,I4//(5X,1P,8E14.5))
  690 FORMAT(/53H FLUADJ: ***WARNING*** THE MAXIMUM NUMBER OF OUTER IT,
     1 20HERATIONS IS REACHED.)
  700 FORMAT(/53H FLUADJ: INCREASING THE NUMBER OF INNER ITERATIONS TO,
     1 I3,36H ADI ITERATIONS PER OUTER ITERATION./)
      END
