*DECK FLUDEF
      SUBROUTINE FLUDEF (MAX,IPTRK,IPSYS,LL4,NGRP,LCM,IMOD,EVECT,ADECT,
     1 VEC,IADJ,VEA,VEB,AGAR,GAF)
*
*-----------------------------------------------------------------------
*
* MULTIGROUP HOTELLING DEFLATION PROCEDURE. 1- MULTIPLICATION OF THE
* 'A' MATRIX BY A VECTOR; 2- MULTIPLICATION OF A DEFLATED 'B' MATRIX
* BY THE SAME VECTOR.
*
* INPUT PARAMETERS:
*  MAX    : FIRST DIMENSION OF EVECT, ADECT, VEC, VEA AND VEB.
*  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPSYS  : L_SYSTEM POINTER TO SYSTEM MATRICES.
*  LL4    : ORDER OF THE SYSTEM MATRICES.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  LCM    : =.TRUE. IS IPSYS IS A MEMORY RESIDENT LINKED LIST.
*  IMOD   : NUMBER OF THE HARMONIC TO BE DEFLATED.
*  EVECT  : DIRECT EIGENVECTOR.
*  ADECT  : ADJOINT EIGENVECTOR.
*           DIMENSION EVECT(MAX,NGRP,LMOD),ADECT(MAX,NGRP,LMOD)
*  VEC    : VECTOR TO BE MULTIPLIED. DIMENSION VEC(MAX,NGRP)
*  IADJ   : =1 FOR A DIRECT DEFLATION / =2 FOR AN ADJOINT DEFLATION.
*
* OUTPUT PARAMETERS:
*  VEA    : RESULT OF THE MULTIPLICATION TO THE 'A' MATRIX.
*  VEB    : RESULT OF THE MULTIPLICATION TO THE 'B' MATRIX.
*           DIMENSION VEA(MAX,NGRP),VEB(MAX,NGRP)
*
* SCRATCH STORAGE:
*  AGAR   : DIMENSION AGAR(LL4)
*  GAF    : DIMENSION GAF(LL4)
*
*------------------------------------- AUTHOR: A. HEBERT ; 20/06/94 ----
*
      CHARACTER*12 TEXT12
      LOGICAL LCM
      DOUBLE PRECISION DDELN1,DDELD1
      DIMENSION EVECT(MAX,NGRP,*),ADECT(MAX,NGRP,*),VEC(MAX,*),
     1 VEA(MAX,*),VEB(MAX,*),AGAR(*),GAF(*)
      COMMON BASE(1)
*
      IF(IADJ.EQ.1) THEN
*        DIRECT CASE.
         DO 40 IGR=1,NGRP
         DO 10 I=1,LL4
   10    VEB(I,IGR)=0.0
         DO 40 JGR=1,NGRP
         WRITE(TEXT12,'(1HB,2I3)') IGR,JGR
         CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 40
         IF(LCM) THEN
            IAGAR=LCMIOF(IPSYS,TEXT12)
            DO 20 I=1,LL4
   20       VEB(I,IGR)=VEB(I,IGR)+BASE(IAGAR+I-1)*VEC(I,JGR)
         ELSE
            CALL LCMGET(IPSYS,TEXT12,AGAR)
            DO 30 I=1,LL4
   30       VEB(I,IGR)=VEB(I,IGR)+AGAR(I)*VEC(I,JGR)
         ENDIF
   40    CONTINUE
         DO 130 JMOD=1,IMOD-1
         DDELN1=0.0D0
         DDELD1=0.0D0
         DO 120 IGR=1,NGRP
         WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
         CALL MTLDLM(TEXT12,IPTRK,IPSYS,EVECT(1,IGR,JMOD),VEA(1,IGR))
         DO 50 I=1,LL4
   50    GAF(I)=0.0
         DO 110 JGR=1,NGRP
         IF(JGR.EQ.IGR) GO TO 80
         WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
         CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 80
         IF(LCM) THEN
            IAGAR=LCMIOF(IPSYS,TEXT12)
            DO 60 I=1,LL4
   60       VEA(I,IGR)=VEA(I,IGR)-BASE(IAGAR+I-1)*EVECT(I,JGR,JMOD)
         ELSE
            CALL LCMGET(IPSYS,TEXT12,AGAR)
            DO 70 I=1,LL4
   70       VEA(I,IGR)=VEA(I,IGR)-AGAR(I)*EVECT(I,JGR,JMOD)
         ENDIF
   80    WRITE(TEXT12,'(1HB,2I3)') JGR,IGR
         CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 110
         IF(LCM) THEN
            IAGAR=LCMIOF(IPSYS,TEXT12)
            DO 90 I=1,LL4
   90       GAF(I)=GAF(I)+BASE(IAGAR+I-1)*ADECT(I,JGR,JMOD)
         ELSE
            CALL LCMGET(IPSYS,TEXT12,AGAR)
            DO 100 I=1,LL4
  100       GAF(I)=GAF(I)+AGAR(I)*ADECT(I,JGR,JMOD)
         ENDIF
  110    CONTINUE
         DO 120 I=1,LL4
         DDELN1=DDELN1+GAF(I)*VEC(I,IGR)
  120    DDELD1=DDELD1+ADECT(I,IGR,JMOD)*VEA(I,IGR)
         DDELN1=DDELN1/DDELD1
         DO 130 IGR=1,NGRP
         DO 130 I=1,LL4
  130    VEB(I,IGR)=VEB(I,IGR)-VEA(I,IGR)*DDELN1
         DO 160 IGR=1,NGRP
         WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
         CALL MTLDLM(TEXT12,IPTRK,IPSYS,VEC(1,IGR),VEA(1,IGR))
         DO 160 JGR=1,NGRP
         IF(JGR.EQ.IGR) GO TO 160
         WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
         CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 160
         IF(LCM) THEN
            IAGAR=LCMIOF(IPSYS,TEXT12)
            DO 140 I=1,LL4
  140       VEA(I,IGR)=VEA(I,IGR)-BASE(IAGAR+I-1)*VEC(I,JGR)
         ELSE
            CALL LCMGET(IPSYS,TEXT12,AGAR)
            DO 150 I=1,LL4
  150       VEA(I,IGR)=VEA(I,IGR)-AGAR(I)*VEC(I,JGR)
         ENDIF
  160    CONTINUE
      ELSE IF(IADJ.EQ.2) THEN
*        ADJOINT CASE.
         DO 200 IGR=1,NGRP
         DO 170 I=1,LL4
  170    VEB(I,IGR)=0.0
         DO 200 JGR=1,NGRP
         WRITE(TEXT12,'(1HB,2I3)') JGR,IGR
         CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 200
         IF(LCM) THEN
            IAGAR=LCMIOF(IPSYS,TEXT12)
            DO 180 I=1,LL4
  180       VEB(I,IGR)=VEB(I,IGR)+BASE(IAGAR+I-1)*VEC(I,JGR)
         ELSE
            CALL LCMGET(IPSYS,TEXT12,AGAR)
            DO 190 I=1,LL4
  190       VEB(I,IGR)=VEB(I,IGR)+AGAR(I)*VEC(I,JGR)
         ENDIF
  200    CONTINUE
         DO 290 JMOD=1,IMOD-1
         DDELN1=0.0D0
         DDELD1=0.0D0
         DO 280 IGR=1,NGRP
         WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
         CALL MTLDLM(TEXT12,IPTRK,IPSYS,ADECT(1,IGR,JMOD),VEA(1,IGR))
         DO 210 I=1,LL4
  210    GAF(I)=0.0
         DO 270 JGR=1,NGRP
         IF(JGR.EQ.IGR) GO TO 240
         WRITE(TEXT12,'(1HA,2I3)') JGR,IGR
         CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 240
         IF(LCM) THEN
            IAGAR=LCMIOF(IPSYS,TEXT12)
            DO 220 I=1,LL4
  220       VEA(I,IGR)=VEA(I,IGR)-BASE(IAGAR+I-1)*ADECT(I,JGR,JMOD)
         ELSE
            CALL LCMGET(IPSYS,TEXT12,AGAR)
            DO 230 I=1,LL4
  230       VEA(I,IGR)=VEA(I,IGR)-AGAR(I)*ADECT(I,JGR,JMOD)
         ENDIF
  240    WRITE(TEXT12,'(1HB,2I3)') IGR,JGR
         CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 270
         IF(LCM) THEN
            IAGAR=LCMIOF(IPSYS,TEXT12)
            DO 250 I=1,LL4
  250       GAF(I)=GAF(I)+BASE(IAGAR+I-1)*EVECT(I,JGR,JMOD)
         ELSE
            CALL LCMGET(IPSYS,TEXT12,AGAR)
            DO 260 I=1,LL4
  260       GAF(I)=GAF(I)+AGAR(I)*EVECT(I,JGR,JMOD)
         ENDIF
  270    CONTINUE
         DO 280 I=1,LL4
         DDELN1=DDELN1+GAF(I)*VEC(I,IGR)
  280    DDELD1=DDELD1+EVECT(I,IGR,JMOD)*VEA(I,IGR)
         DDELN1=DDELN1/DDELD1
         DO 290 IGR=1,NGRP
         DO 290 I=1,LL4
  290    VEB(I,IGR)=VEB(I,IGR)-VEA(I,IGR)*DDELN1
         DO 320 IGR=1,NGRP
         WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
         CALL MTLDLM(TEXT12,IPTRK,IPSYS,VEC(1,IGR),VEA(1,IGR))
         DO 320 JGR=1,NGRP
         IF(JGR.EQ.IGR) GO TO 320
         WRITE(TEXT12,'(1HA,2I3)') JGR,IGR
         CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 320
         IF(LCM) THEN
            IAGAR=LCMIOF(IPSYS,TEXT12)
            DO 300 I=1,LL4
  300       VEA(I,IGR)=VEA(I,IGR)-BASE(IAGAR+I-1)*VEC(I,JGR)
         ELSE
            CALL LCMGET(IPSYS,TEXT12,AGAR)
            DO 310 I=1,LL4
  310       VEA(I,IGR)=VEA(I,IGR)-AGAR(I)*VEC(I,JGR)
         ENDIF
  320    CONTINUE
      ENDIF
      RETURN
      END
