*DECK FLUMON
      SUBROUTINE FLUMON (IPTRK,IPSYS,IPFLUX,NUN,NGRP,LMOD,ICL1,ICL2,
     1 IMPX,IMPH,TITR,EPS2,NADI,MAXX0,RAND,FKEFF,EVECT,ADECT,AGAR,GRAD1,
     2 GRAD2,VEA1,VEA2,VEA3,VEB1,VEB2,VEB3)
*
*-----------------------------------------------------------------------
*
* SOLUTION OF MULTIGROUP EIGENVALUE SYSTEMS FOR THE CALCULATION OF THE
* LMOD FIRST BI-ORTHOGONAL HARMONICS OF THE DIFFUSION EQUATION IN
* TRIVAC. USE THE PRECONDITIONNED POWER METHOD WITH HOTELLING DEFLATION
* AND A TWO PARAMETER SVAT ACCELERATION TECHNIQUE.
*
* INPUT PARAMETERS:
*  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPSYS  : L_SYSTEM POINTER TO SYSTEM MATRICES.
*  IPFLUX : L_FLUX POINTER TO THE SOLUTION.
*  NUN    : NUMBER OF UNKNOWNS IN EACH ENERGY GROUP.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  LMOD   : NUMBER OF BI-ORTHOGONAL HARMONICS TO COMPUTE.
*  ICL1   : NUMBER OF FREE IRETATIONS IN ONE CYCLE OF THE INVERSE POWER
*           METHOD.
*  ICL2   : NUMBER OF ACCELERATED ITERATIONS IN ONE CYCLE.
*  IMPX   : PRINT PARAMETER. =0: NO PRINT ; =1: MINIMUM PRINTING ;
*           =2: ITERATION HISTORY IS PRINTED. =3: SOLUTION IS PRINTED.
*  IMPH   : =0: NO ACTION IS TAKEN.
*           =1: THE FLUX IS COMPARED TO A REFERENCE FLUX STORED ON LCM.
*           =2: THE CONVERGENCE HISTOGRAM IS PRINTED.
*           =3: THE CONVERGENCE HISTOGRAM IS PRINTED WITH AXIS AND
*               TITLES. THE PLOTTING FILE IS COMPLETED.
*           =4: THE CONVERGENCE HISTOGRAM IS PRINTED WITH AXIS, ACCE-
*               LERATION FACTORS AND TITLES. THE PLOTTING FILE IS
*               COMPLETED.
*  TITR   : CHARACTER*72 TITLE.
*  EPS2   : CONVERGENCE CRITERIA FOR THE FLUX.
*  NADI   : NUMBER OF INNER ADI ITERATIONS PER OUTER ITERATION.
*  MAXX0  : MAXIMUM NUMBER OF OUTER ITERATIONS.
*  RAND   : =.TRUE. USE INITIAL RANDOM FLUX ; =.FALSE. USE FLAT FLUX.
*
* OUTPUT PARAMETERS:
*  FKEFF  : EFFECTIVE MULTIPLICATION FACTOR OF EACH HARMONIC.
*           DIMENSION FKEFF(LMOD)
*  EVECT  : CONVERGED DIRECT HARMONIC VECTOR.
*  ADECT  : CONVERGED ADJOINT HARMONIC VECTOR.
*           DIMENSION EVECT(NUN,NGRP,LMOD),ADECT(NUN,NGRP,LMOD)
*
* SCRATCH STORAGE:
*  AGAR   : DIMENSION AGAR(LL4) WHERE LL4 IS THE ORDER OF THE SYSTEM
*           MATRICES. LL4.LE.NUN
*  GRAD1  : DIMENSION GRAD1(NUN,NGRP)
*  GRAD2  : DIMENSION GRAD2(NUN,NGRP)
*  VEA1,VEA2,VEA3 : DIMENSION VEA1(NUN,NGRP),VEA2(NUN,NGRP),
*                   VEA2(NUN,NGRP)
*  VEB1,VEB2,VEB3 : DIMENSION VEB1(NUN,NGRP),VEB2(NUN,NGRP),
*                   VEB2(NUN,NGRP)
*
* REFERENCE:
*  A. HEBERT, 'PRECONDITIONING THE POWER METHOD FOR REACTOR CALCU-
*  LATIONS', NUCL. SCI. ENG., 94, 1 (1986).
*  J. H. WILKINSON, "THE ALGEBRAIC EIGENVALUE PROBLEM", CLARENDON
*  PRESS, OXFORD, P. 584, 1965.
*
*-------------------------------------- AUTHOR: A. HEBERT ; 20/06/94 ---
*
      PARAMETER (IM=714025,IA=1366,IC=150889,RM=1.4005112E-6)
      CHARACTER*(*) TITR
      CHARACTER*12 TEXT12,NAMLCM,NAMMY
      LOGICAL LCM,LOGTES,RAND,LMPH
      DOUBLE PRECISION AEAE,AEAG,AEAH,AGAG,AGAH,AHAH,BEBE,BEBG,BEBH,
     1 BGBG,BGBH,BHBH,AEBE,AEBG,AEBH,AGBE,AGBG,AGBH,AHBE,AHBG,AHBH,
     2 X,DXDA,DXDB,Y,DYDA,DYDB,Z,DZDA,DZDB,F,D2F(2,3),EVAL,ALP,BET,Z1
      DIMENSION FKEFF(*),EVECT(NUN,NGRP,*),ADECT(NUN,NGRP,*),AGAR(*),
     1 GRAD1(NUN,*),GRAD2(NUN,*),VEA1(NUN,*),VEA2(NUN,*),VEA3(NUN,*),
     2 VEB1(NUN,*),VEB2(NUN,*),VEB3(NUN,*),ERR(250),ALPH(250),BETA(250),
     3 ITITR(18)
      COMMON BASE(1)
      DATA EPS1,MMAXX/1.0E-5,250/
      MAXX=MIN(MAXX0,MMAXX)
      CALL MTOPEN(IMPX,IPTRK,LL4)
      IF(LL4.GT.NUN) CALL XABORT('FLUMON: INVALID NUMBER OF UNKNOWNS.')
      TEXT12=' '
      CALL LCMNXT(IPSYS,TEXT12,NAMLCM,NAMMY,LCM)
*
      DO 390 IMOD=1,LMOD
      IF(IMPX.GE.1) WRITE (6,'(1H1//13H HARMONIC NB.,I3//)') IMOD
      WRITE(TEXT12,'(7HFLUX  1,I3)') IMOD
      CALL LCMLEN(IPFLUX,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.NUN) THEN
         DO 10 IGR=1,NGRP
         WRITE(TEXT12,'(4HFLUX,2I3)') IGR,IMOD
         CALL LCMGET(IPFLUX,TEXT12,EVECT(1,IGR,IMOD))
         WRITE(TEXT12,'(5HAFLUX,2I3)') IGR,IMOD
         CALL LCMGET(IPFLUX,TEXT12,ADECT(1,IGR,IMOD))
   10    CONTINUE
      ELSE IF((IMOD.EQ.1).OR.(.NOT.RAND)) THEN
*        UNIFORM UNKNOWN VECTOR.
         DO 20 IGR=1,NGRP
         DO 20 I=1,NUN
         EVECT(I,IGR,IMOD)=1.0
   20    ADECT(I,IGR,IMOD)=1.0
      ELSE
*        RANDOM UNKNOWN VECTOR.
         ISEED=0
         DO 30 IGR=1,NGRP
         DO 30 I=1,NUN
         ISEED=MOD(ISEED*IA+IC,IM)
         RAN=REAL(ISEED)*RM
         EVECT(I,IGR,IMOD)=RAN
   30    ADECT(I,IGR,IMOD)=RAN
      ENDIF
*
* PRECONDITIONED POWER METHOD FOR THE DIRECT PROBLEM.
      EVAL=1.0D0
      VVV=0.0
      ISTART=1
      NNADI=NADI
      IF(IMPX.GE.1) WRITE (6,600) NADI,'DIRECT'
      IF(IMPX.GE.2) WRITE (6,610)
      CALL SETARA(BASE,LL4,IGAF)
      CALL FLUDEF(NUN,IPTRK,IPSYS,LL4,NGRP,LCM,IMOD,EVECT,ADECT,
     1 EVECT(1,1,IMOD),1,VEA1,VEB1,AGAR,BASE(IGAF))
      CALL RLSARA(BASE(IGAF))
      M=0
   50 M=M+1
*
* EIGENVALUE EVALUATION.
      AEBE=0.0D0
      BEBE=0.0D0
      DO 60 IGR=1,NGRP
      DO 60 I=1,LL4
      AEBE=AEBE+VEA1(I,IGR)*VEB1(I,IGR)
   60 BEBE=BEBE+VEB1(I,IGR)**2
      EVAL=AEBE/BEBE
*
* DIRECTION EVALUATION.
      DO 110 IGR=1,NGRP
      DO 70 I=1,LL4
   70 GRAD1(I,IGR)=EVAL*VEB1(I,IGR)-VEA1(I,IGR)
      DO 100 JGR=1,IGR-1
      WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 100
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS,TEXT12)
         DO 80 I=1,LL4
   80    GRAD1(I,IGR)=GRAD1(I,IGR)+BASE(IAGAR+I-1)*GRAD1(I,JGR)
      ELSE
         CALL LCMGET(IPSYS,TEXT12,AGAR)
         DO 90 I=1,LL4
   90    GRAD1(I,IGR)=GRAD1(I,IGR)+AGAR(I)*GRAD1(I,JGR)
      ENDIF
  100 CONTINUE
*
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      IF(NNADI.EQ.1) THEN
         CALL MTLDLS(TEXT12,IPTRK,IPSYS,GRAD1(1,IGR))
      ELSE
         CALL TRIADI(TEXT12,IPTRK,IPSYS,LL4,GRAD1(1,IGR),NNADI)
      ENDIF
  110 CONTINUE
*
* DISPLACEMENT EVALUATION.
      F=0.0D0
      DELS=ABS((EVAL-VVV)/EVAL)
      VVV=EVAL
*
* EVALUATION OF THE TWO ACCELERATION PARAMETERS ALP AND BET.
      ALP=1.0D0
      BET=0.0D0
      N=0
      AEAE=0.0D0
      AEAG=0.0D0
      AEAH=0.0D0
      AGAG=0.0D0
      AGAH=0.0D0
      AHAH=0.0D0
      BEBG=0.0D0
      BEBH=0.0D0
      BGBG=0.0D0
      BGBH=0.0D0
      BHBH=0.0D0
      AEBG=0.0D0
      AEBH=0.0D0
      AGBE=0.0D0
      AGBG=0.0D0
      AGBH=0.0D0
      AHBE=0.0D0
      AHBG=0.0D0
      AHBH=0.0D0
      CALL SETARA(BASE,LL4,IGAF)
      CALL FLUDEF(NUN,IPTRK,IPSYS,LL4,NGRP,LCM,IMOD,EVECT,ADECT,GRAD1,
     1 1,VEA2,VEB2,AGAR,BASE(IGAF))
      CALL RLSARA(BASE(IGAF))
      IF(1+MOD(M-ISTART,ICL1+ICL2).GT.ICL1) THEN
         DO 120 IGR=1,NGRP
         DO 120 I=1,LL4
*        COMPUTE (A ,A )
         AEAE=AEAE+VEA1(I,IGR)**2
         AEAG=AEAG+VEA1(I,IGR)*VEA2(I,IGR)
         AEAH=AEAH+VEA1(I,IGR)*VEA3(I,IGR)
         AGAG=AGAG+VEA2(I,IGR)**2
         AGAH=AGAH+VEA2(I,IGR)*VEA3(I,IGR)
         AHAH=AHAH+VEA3(I,IGR)**2
*        COMPUTE (B ,B )
         BEBG=BEBG+VEB1(I,IGR)*VEB2(I,IGR)
         BEBH=BEBH+VEB1(I,IGR)*VEB3(I,IGR)
         BGBG=BGBG+VEB2(I,IGR)**2
         BGBH=BGBH+VEB2(I,IGR)*VEB3(I,IGR)
         BHBH=BHBH+VEB3(I,IGR)**2
*        COMPUTE (A ,B )
         AEBG=AEBG+VEA1(I,IGR)*VEB2(I,IGR)
         AEBH=AEBH+VEA1(I,IGR)*VEB3(I,IGR)
         AGBE=AGBE+VEA2(I,IGR)*VEB1(I,IGR)
         AGBG=AGBG+VEA2(I,IGR)*VEB2(I,IGR)
         AGBH=AGBH+VEA2(I,IGR)*VEB3(I,IGR)
         AHBE=AHBE+VEA3(I,IGR)*VEB1(I,IGR)
         AHBG=AHBG+VEA3(I,IGR)*VEB2(I,IGR)
  120    AHBH=AHBH+VEA3(I,IGR)*VEB3(I,IGR)
*
  130    N=N+1
         IF(N.GT.50) CALL XABORT('FLUMON: FAILURE OF THE OVERRELAXATIO'
     1   //'N PARAMETERS CALCULATION.')
*        COMPUTE X(M+1)
         X=BEBE+ALP*ALP*BGBG+BET*BET*BHBH+2.0D0*(ALP*BEBG+BET*BEBH
     1   +ALP*BET*BGBH)
         DXDA=2.0D0*(BEBG+ALP*BGBG+BET*BGBH)
         DXDB=2.0D0*(BEBH+ALP*BGBH+BET*BHBH)
*        COMPUTE Y(M+1)
         Y=AEAE+ALP*ALP*AGAG+BET*BET*AHAH+2.0D0*(ALP*AEAG+BET*AEAH
     1   +ALP*BET*AGAH)
         DYDA=2.0D0*(AEAG+ALP*AGAG+BET*AGAH)
         DYDB=2.0D0*(AEAH+ALP*AGAH+BET*AHAH)
*        COMPUTE Z(M+1)
         Z=AEBE+ALP*ALP*AGBG+BET*BET*AHBH+ALP*(AEBG+AGBE)
     1   +BET*(AEBH+AHBE)+ALP*BET*(AGBH+AHBG)
         DZDA=AEBG+AGBE+2.0D0*ALP*AGBG+BET*(AGBH+AHBG)
         DZDB=AEBH+AHBE+ALP*(AGBH+AHBG)+2.0D0*BET*AHBH
*        COMPUTE F(M+1)
         F=X*Y-Z*Z
         D2F(1,1)=2.0D0*(BGBG*Y+DXDA*DYDA+X*AGAG-DZDA**2-2.0D0*Z*AGBG)
         D2F(1,2)=2.0D0*BGBH*Y+DXDA*DYDB+DXDB*DYDA+2.0D0*X*AGAH
     1   -2.0D0*DZDA*DZDB-2.0D0*Z*(AGBH+AHBG)
         D2F(2,2)=2.0D0*(BHBH*Y+DXDB*DYDB+X*AHAH-DZDB**2-2.0D0*Z*AHBH)
         D2F(2,1)=D2F(1,2)
         D2F(1,3)=DXDA*Y+X*DYDA-2.0D0*Z*DZDA
         D2F(2,3)=DXDB*Y+X*DYDB-2.0D0*Z*DZDB
*        SOLUTION OF A LINEAR SYSTEM.
         CALL ALSBD(2,1,D2F,IER,2)
         IF(IER.NE.0) CALL XABORT('FLUMON: SINGULAR MATRIX.')
         ALP=ALP-D2F(1,3)
         BET=BET-D2F(2,3)
         IF((ABS(D2F(1,3)).LE.1.0D-4).AND.(ABS(D2F(2,3)).LE.1.0D-4))
     1   GO TO 140
         GO TO 130
  140    BET=BET/ALP
         IF((ALP.LT.1.0D0).AND.(ALP.GT.0.0D0)) THEN
            ALP=1.0D0
            BET=0.0D0
         ELSE IF(ALP.LE.0.0D0) THEN
            ISTART=M+1
            ALP=1.0D0
            BET=0.0D0
         ENDIF
         DO 150 IGR=1,NGRP
         DO 150 I=1,LL4
         GRAD1(I,IGR)=ALP*(GRAD1(I,IGR)+BET*GRAD2(I,IGR))
         VEA2(I,IGR)=ALP*(VEA2(I,IGR)+BET*VEA3(I,IGR))
  150    VEB2(I,IGR)=ALP*(VEB2(I,IGR)+BET*VEB3(I,IGR))
      ENDIF
*
      LOGTES=(M.LT.ICL1).OR.(MOD(M,ICL1+ICL2).EQ.ICL1)
      IF(LOGTES.AND.(DELS.LE.EPS1)) THEN
         DELT=0.0
         DO 170 IGR=1,NGRP
         DELN=0.0
         DELD=0.0
         DO 160 I=1,LL4
         DELN=MAX(DELN,ABS(GRAD1(I,IGR)))
         EVECT(I,IGR,IMOD)=EVECT(I,IGR,IMOD)+GRAD1(I,IGR)
         DELD=MAX(DELD,ABS(EVECT(I,IGR,IMOD)))
         VEA1(I,IGR)=VEA1(I,IGR)+VEA2(I,IGR)
         VEB1(I,IGR)=VEB1(I,IGR)+VEB2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
         VEA3(I,IGR)=VEA2(I,IGR)
  160    VEB3(I,IGR)=VEB2(I,IGR)
         IF(DELD.NE.0.0) DELT=MAX(DELT,DELN/DELD)
  170    CONTINUE
         IF(IMPX.GE.2) WRITE (6,615) M,AEAE,AEAG,AEAH,AGAG,AGAH,AHAH,
     1   BEBE,ALP,BET,EVAL,F,DELS,DELT,N,BEBG,BEBH,BGBG,BGBH,BHBH,
     2   AEBE,AEBG,AEBH,AGBE,AGBG,AGBH,AHBE,AHBG,AHBH
*        COMPUTE THE CONVERGENCE HISTOGRAM.
         IF(IMPH.GE.1) THEN
            LMPH=IMPH.GE.1
            CALL XCOMP(IPFLUX,LL4,NUN,EVECT(1,NGRP,IMOD),LMPH,ERR(M))
            ALPH(M)=ALP
            BETA(M)=BET
         ENDIF
         IF(DELT.LE.EPS2) GO TO 190
      ELSE
         DO 180 IGR=1,NGRP
         DO 180 I=1,LL4
         EVECT(I,IGR,IMOD)=EVECT(I,IGR,IMOD)+GRAD1(I,IGR)
         VEA1(I,IGR)=VEA1(I,IGR)+VEA2(I,IGR)
         VEB1(I,IGR)=VEB1(I,IGR)+VEB2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
         VEA3(I,IGR)=VEA2(I,IGR)
  180    VEB3(I,IGR)=VEB2(I,IGR)
         IF(IMPX.GE.2) WRITE (6,620) M,AEAE,AEAG,AEAH,AGAG,AGAH,AHAH,
     1   BEBE,ALP,BET,EVAL,F,DELS,N,BEBG,BEBH,BGBG,BGBH,BHBH,AEBE,
     2   AEBG,AEBH,AGBE,AGBG,AGBH,AHBE,AHBG,AHBH
*        COMPUTE THE CONVERGENCE HISTOGRAM.
         IF(IMPH.GE.1) THEN
            LMPH=IMPH.GE.1
            CALL XCOMP(IPFLUX,LL4,NUN,EVECT(1,NGRP,IMOD),LMPH,ERR(M))
            ALPH(M)=ALP
            BETA(M)=BET
         ENDIF
      ENDIF
*
      IF(M.EQ.1) TEST=DELS
      IF((M.GT.5).AND.(DELS.GT.TEST)) CALL XABORT('FLUMON: CONVERGENCE'
     1 //' FAILURE.')
      IF(M.GE.MAXX) THEN
         WRITE (6,'(/46H FLUMON: ***WARNING*** MAXIMUM NUMBER OF ITERA,
     1   17HTIONS IS REACHED.)')
         GO TO 190
      ENDIF
      IF(MOD(M,36).EQ.0) THEN
         ISTART=M+1
         NNADI=NNADI+1
         WRITE (6,700) NNADI
      ENDIF
      GO TO 50
*
* DIRECT SOLUTION EDITION.
  190 Z1=1.0D0/EVAL
      DO 200 IGR=1,NGRP
      WRITE(TEXT12,'(4HFLUX,2I3)') IGR,IMOD
      CALL LCMPUT(IPFLUX,TEXT12,NUN,2,EVECT(1,IGR,IMOD))
  200 CONTINUE
      IF(IMPX.GE.1) WRITE (6,630) 1.0D0/EVAL
      IF(IMPX.EQ.1) WRITE (6,640) M
      IF(IMPX.EQ.3) THEN
         DO 210 IGR=1,NGRP
         WRITE (6,660) 'DIRECT',IGR,(EVECT(I,IGR,IMOD),I=1,LL4)
  210    CONTINUE
      ENDIF
      IF(IMPH.GE.2) THEN
         IGRAPH=0
  215    IGRAPH=IGRAPH+1
         WRITE (TEXT12,'(5HHISTO,I3)') IGRAPH
         CALL LCMLEN (IPFLUX,TEXT12,ILENG,ITYLCM)
         IF(ILENG.EQ.0) THEN
            READ (TITR,'(18A4)') ITITR
            CALL LCMSIX (IPFLUX,TEXT12,1)
            CALL LCMPUT (IPFLUX,'HTITLE',18,3,ITITR)
            CALL LCMPUT (IPFLUX,'ALPHA',M,2,ALPH)
            CALL LCMPUT (IPFLUX,'BETA',M,2,BETA)
            CALL LCMPUT (IPFLUX,'ERROR',M,2,ERR)
            CALL LCMPUT (IPFLUX,'IMPH',1,1,IMPH)
            CALL LCMSIX (IPFLUX,' ',2)
         ELSE
            GO TO 215
         ENDIF
      ENDIF
*
* PRECONDITIONED POWER METHOD FOR THE ADJOINT PROBLEM.
      EVAL=1.0D0
      VVV=0.0
      ISTART=1
      NNADI=NADI
      IF(IMPX.GE.1) WRITE (6,600) NADI,'ADJOINT'
      IF(IMPX.GE.2) WRITE (6,610)
      CALL SETARA(BASE,LL4,IGAF)
      CALL FLUDEF(NUN,IPTRK,IPSYS,LL4,NGRP,LCM,IMOD,EVECT,ADECT,
     1 ADECT(1,1,IMOD),2,VEA1,VEB1,AGAR,BASE(IGAF))
      CALL RLSARA(BASE(IGAF))
      M=0
  220 M=M+1
*
* EIGENVALUE CALCULATION.
      AEBE=0.0D0
      BEBE=0.0D0
      DO 230 IGR=1,NGRP
      DO 230 I=1,LL4
      AEBE=AEBE+VEA1(I,IGR)*VEB1(I,IGR)
  230 BEBE=BEBE+VEB1(I,IGR)**2
      EVAL=AEBE/BEBE
*
* DIRECTION EVALUATION.
      DO 280 IGR=NGRP,1,-1
      DO 240 I=1,LL4
  240 GRAD1(I,IGR)=EVAL*VEB1(I,IGR)-VEA1(I,IGR)
      DO 270 JGR=NGRP,IGR+1,-1
      WRITE(TEXT12,'(1HA,2I3)') JGR,IGR
      CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 270
      IF(LCM) THEN
         IAGAR=LCMIOF(IPSYS,TEXT12)
         DO 250 I=1,LL4
  250    GRAD1(I,IGR)=GRAD1(I,IGR)+BASE(IAGAR+I-1)*GRAD1(I,JGR)
      ELSE
         CALL LCMGET(IPSYS,TEXT12,AGAR)
         DO 260 I=1,LL4
  260    GRAD1(I,IGR)=GRAD1(I,IGR)+AGAR(I)*GRAD1(I,JGR)
      ENDIF
  270 CONTINUE
*
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      IF(NNADI.EQ.1) THEN
         CALL MTLDLS(TEXT12,IPTRK,IPSYS,GRAD1(1,IGR))
      ELSE
         CALL TRIADI(TEXT12,IPTRK,IPSYS,LL4,GRAD1(1,IGR),NNADI)
      ENDIF
  280 CONTINUE
*
* DISPLACEMENT EVALUATION.
      F=0.0D0
      DELS=ABS((EVAL-VVV)/EVAL)
      VVV=EVAL
*
* EVALUATION OF THE TWO ACCELERATION PARAMETERS ALP AND BET.
      ALP=1.0D0
      BET=0.0D0
      N=0
      AEAE=0.0D0
      AEAG=0.0D0
      AEAH=0.0D0
      AGAG=0.0D0
      AGAH=0.0D0
      AHAH=0.0D0
      BEBG=0.0D0
      BEBH=0.0D0
      BGBG=0.0D0
      BGBH=0.0D0
      BHBH=0.0D0
      AEBG=0.0D0
      AEBH=0.0D0
      AGBE=0.0D0
      AGBG=0.0D0
      AGBH=0.0D0
      AHBE=0.0D0
      AHBG=0.0D0
      AHBH=0.0D0
      CALL SETARA(BASE,LL4,IGAF)
      CALL FLUDEF(NUN,IPTRK,IPSYS,LL4,NGRP,LCM,IMOD,EVECT,ADECT,GRAD1,
     1 2,VEA2,VEB2,AGAR,BASE(IGAF))
      CALL RLSARA(BASE(IGAF))
      IF(1+MOD(M-ISTART,ICL1+ICL2).GT.ICL1) THEN
         DO 290 IGR=1,NGRP
         DO 290 I=1,LL4
*        COMPUTE (A ,A )
         AEAE=AEAE+VEA1(I,IGR)**2
         AEAG=AEAG+VEA1(I,IGR)*VEA2(I,IGR)
         AEAH=AEAH+VEA1(I,IGR)*VEA3(I,IGR)
         AGAG=AGAG+VEA2(I,IGR)**2
         AGAH=AGAH+VEA2(I,IGR)*VEA3(I,IGR)
         AHAH=AHAH+VEA3(I,IGR)**2
*        COMPUTE (B ,B )
         BEBG=BEBG+VEB1(I,IGR)*VEB2(I,IGR)
         BEBH=BEBH+VEB1(I,IGR)*VEB3(I,IGR)
         BGBG=BGBG+VEB2(I,IGR)**2
         BGBH=BGBH+VEB2(I,IGR)*VEB3(I,IGR)
         BHBH=BHBH+VEB3(I,IGR)**2
*        COMPUTE (A ,B )
         AEBG=AEBG+VEA1(I,IGR)*VEB2(I,IGR)
         AEBH=AEBH+VEA1(I,IGR)*VEB3(I,IGR)
         AGBE=AGBE+VEA2(I,IGR)*VEB1(I,IGR)
         AGBG=AGBG+VEA2(I,IGR)*VEB2(I,IGR)
         AGBH=AGBH+VEA2(I,IGR)*VEB3(I,IGR)
         AHBE=AHBE+VEA3(I,IGR)*VEB1(I,IGR)
         AHBG=AHBG+VEA3(I,IGR)*VEB2(I,IGR)
  290    AHBH=AHBH+VEA3(I,IGR)*VEB3(I,IGR)
*
  300    N=N+1
         IF(N.GT.50) CALL XABORT('FLUMON: FAILURE OF THE OVERRELAXATIO'
     1   //'N PARAMETERS CALCULATION.')
*        COMPUTE X(M+1)
         X=BEBE+ALP*ALP*BGBG+BET*BET*BHBH+2.0D0*(ALP*BEBG+BET*BEBH
     1   +ALP*BET*BGBH)
         DXDA=2.0D0*(BEBG+ALP*BGBG+BET*BGBH)
         DXDB=2.0D0*(BEBH+ALP*BGBH+BET*BHBH)
*        COMPUTE Y(M+1)
         Y=AEAE+ALP*ALP*AGAG+BET*BET*AHAH+2.0D0*(ALP*AEAG+BET*AEAH
     1   +ALP*BET*AGAH)
         DYDA=2.0D0*(AEAG+ALP*AGAG+BET*AGAH)
         DYDB=2.0D0*(AEAH+ALP*AGAH+BET*AHAH)
*        COMPUTE Z(M+1)
         Z=AEBE+ALP*ALP*AGBG+BET*BET*AHBH+ALP*(AEBG+AGBE)
     1   +BET*(AEBH+AHBE)+ALP*BET*(AGBH+AHBG)
         DZDA=AEBG+AGBE+2.0D0*ALP*AGBG+BET*(AGBH+AHBG)
         DZDB=AEBH+AHBE+ALP*(AGBH+AHBG)+2.0D0*BET*AHBH
*        COMPUTE F(M+1)
         F=X*Y-Z*Z
         D2F(1,1)=2.0D0*(BGBG*Y+DXDA*DYDA+X*AGAG-DZDA**2-2.0D0*Z*AGBG)
         D2F(1,2)=2.0D0*BGBH*Y+DXDA*DYDB+DXDB*DYDA+2.0D0*X*AGAH
     1   -2.0D0*DZDA*DZDB-2.0D0*Z*(AGBH+AHBG)
         D2F(2,2)=2.0D0*(BHBH*Y+DXDB*DYDB+X*AHAH-DZDB**2-2.0D0*Z*AHBH)
         D2F(2,1)=D2F(1,2)
         D2F(1,3)=DXDA*Y+X*DYDA-2.0D0*Z*DZDA
         D2F(2,3)=DXDB*Y+X*DYDB-2.0D0*Z*DZDB
*        SOLUTION OF A LINEAR SYSTEM.
         CALL ALSBD(2,1,D2F,IER,2)
         IF(IER.NE.0) CALL XABORT('FLUMON: SINGULAR MATRIX.')
         ALP=ALP-D2F(1,3)
         BET=BET-D2F(2,3)
         IF((ABS(D2F(1,3)).LE.1.0D-4).AND.(ABS(D2F(2,3)).LE.1.0D-4))
     1   GO TO 310
         GO TO 300
  310    BET=BET/ALP
*
         IF((ALP.LT.1.0D0).AND.(ALP.GT.0.0D0)) THEN
            ALP=1.0D0
            BET=0.0D0
         ELSE IF(ALP.LE.0.0D0) THEN
            ISTART=M+1
            ALP=1.0D0
            BET=0.0D0
         ENDIF
         DO 320 IGR=1,NGRP
         DO 320 I=1,LL4
         GRAD1(I,IGR)=ALP*(GRAD1(I,IGR)+BET*GRAD2(I,IGR))
         VEA2(I,IGR)=ALP*(VEA2(I,IGR)+BET*VEA3(I,IGR))
  320    VEB2(I,IGR)=ALP*(VEB2(I,IGR)+BET*VEB3(I,IGR))
      ENDIF
*
      LOGTES=(M.LT.ICL1).OR.(MOD(M,ICL1+ICL2).EQ.ICL1)
      IF(LOGTES.AND.(DELS.LE.EPS1))THEN
         DELT=0.0
         DO 340 IGR=1,NGRP
         DELN=0.0
         DELD=0.0
         DO 330 I=1,LL4
         DELN=MAX(DELN,ABS(GRAD1(I,IGR)))
         ADECT(I,IGR,IMOD)=ADECT(I,IGR,IMOD)+GRAD1(I,IGR)
         DELD=MAX(DELD,ABS(ADECT(I,IGR,IMOD)))
         VEA1(I,IGR)=VEA1(I,IGR)+VEA2(I,IGR)
         VEB1(I,IGR)=VEB1(I,IGR)+VEB2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
         VEA3(I,IGR)=VEA2(I,IGR)
  330    VEB3(I,IGR)=VEB2(I,IGR)
         IF(DELD.NE.0.0) DELT=MAX(DELT,DELN/DELD)
  340    CONTINUE
         IF(IMPX.GE.2) WRITE (6,615) M,AEAE,AEAG,AEAH,AGAG,AGAH,AHAH,
     1   BEBE,ALP,BET,EVAL,F,DELS,DELT,N,BEBG,BEBH,BGBG,BGBH,BHBH,AEBE,
     2   AEBG,AEBH,AGBE,AGBG,AGBH,AHBE,AHBG,AHBH
         IF(DELT.LE.EPS2) GO TO 360
      ELSE
         DO 350 IGR=1,NGRP
         DO 350 I=1,LL4
         ADECT(I,IGR,IMOD)=ADECT(I,IGR,IMOD)+GRAD1(I,IGR)
         VEA1(I,IGR)=VEA1(I,IGR)+VEA2(I,IGR)
         VEB1(I,IGR)=VEB1(I,IGR)+VEB2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
         VEA3(I,IGR)=VEA2(I,IGR)
  350    VEB3(I,IGR)=VEB2(I,IGR)
         IF(IMPX.GE.2) WRITE (6,620) M,AEAE,AEAG,AEAH,AGAG,AGAH,AHAH,
     1   BEBE,ALP,BET,EVAL,F,DELS,N,BEBG,BEBH,BGBG,BGBH,BHBH,AEBE,
     2   AEBG,AEBH,AGBE,AGBG,AGBH,AHBE,AHBG,AHBH
      ENDIF
      IF(M.EQ.1) TEST=DELS
      IF((M.GT.5).AND.(DELS.GT.TEST)) CALL XABORT('FLUMON: CONVERGENCE'
     1 //' FAILURE.')
      IF(M.GE.MAXX) THEN
         WRITE (6,'(/46H FLUMON: ***WARNING*** MAXIMUM NUMBER OF ITERA,
     1   17HTIONS IS REACHED.)')
         GO TO 360
      ENDIF
      IF(MOD(M,36).EQ.0) THEN
         ISTART=M+1
         NNADI=NNADI+1
         WRITE (6,700) NNADI
      ENDIF
      GO TO 220
*
* ADJOINT SOLUTION EDITION.
  360 DO 370 IGR=1,NGRP
      WRITE(TEXT12,'(5HAFLUX,2I3)') IGR,IMOD
      CALL LCMPUT(IPFLUX,TEXT12,NUN,2,ADECT(1,IGR,IMOD))
  370 CONTINUE
      IF(IMPX.GE.1) WRITE (6,630) 1.0D0/EVAL
      IF(IMPX.EQ.1) WRITE (6,640) M
      IF(IMPX.EQ.3) THEN
         DO 380 IGR=1,NGRP
         WRITE (6,660) 'ADJOINT',IGR,(ADECT(I,IGR,IMOD),I=1,LL4)
  380    CONTINUE
      ENDIF
*
      IF(ABS(Z1-1.0D0/EVAL).GT.1.0E-4) CALL XABORT('FLUMON: FAILURE O'
     1 //'F HARMONIC COMPUTATION.')
      FKEFF(IMOD)=0.5*(Z1+1.0D0/EVAL)
  390 CONTINUE
      CALL LCMPUT(IPFLUX,'K-EFFECTIVE',LMOD,2,FKEFF)
      RETURN
*
  600 FORMAT(1H1/50H FLUMON: ITERATIVE PROCEDURE BASED ON PRECONDITION,
     1 17HED POWER METHOD (,I2,37H ADI ITERATIONS PER OUTER ITERATION)./
     2 9X,A7,10H EQUATION.)
  610 FORMAT(//5X,17HBILINEAR PRODUCTS,48X,5HALPHA,3X,4HBETA,3X,
     1 12HEIGENVALUE..,12X,8HACCURACY,11(1H.),2X,1HN)
  615 FORMAT(1X,I3,1P,7E9.1,0P,2F8.3,E14.6,3E10.2,I4/
     1 (4X,1P,7E9.1))
  620 FORMAT(1X,I3,1P,7E9.1,0P,2F8.3,E14.6,2E10.2,10X,I4/
     1 (4X,1P,7E9.1))
  630 FORMAT(/42H FLUMON: EFFECTIVE MULTIPLICATION FACTOR =,1P,D17.10/)
  640 FORMAT(/23H FLUMON: CONVERGENCE IN,I4,12H ITERATIONS.)
  650 FORMAT(/53H FLUMON: CPU TIME USED TO SOLVE THE TRIANGULAR LINEAR,
     1 10H SYSTEMS =,F9.3/23X,34HTO COMPUTE THE BILINEAR PRODUCTS =,
     2 F9.3,20X,16HTOTAL CPU TIME =,F9.3)
  660 FORMAT(//9H FLUMON: ,A7,37H EIGENVECTOR CORRESPONDING TO THE GRO,
     1 2HUP,I4//(5X,1P,8E14.5))
  700 FORMAT(/53H FLUMON: INCREASING THE NUMBER OF INNER ITERATIONS TO,
     1 I3,36H ADI ITERATIONS PER OUTER ITERATION./)
      END
