*DECK FLUSMB
      SUBROUTINE FLUSMB (IPTRK,IPSYS,IPFLUX,NUN,NGRP,ICL1,ICL2,IMPX,
     1 IMPH,TITR,EPS2,MAXX0,EVECT,FKEFF,GRAD1,GRAD2,GAR1,GAR2,GAR3,GAF1,
     2 GAF2,GAF3,WORK)
*
*-----------------------------------------------------------------------
*
* SOLUTION OF A MULTIGROUP EIGENVALUE SYSTEM FOR THE CALCULATION OF THE
* DIRECT NEUTRON FLUX IN BIVAC. USE THE PRECONDITIONNED POWER METHOD
* WITH A TWO PARAMETER SVAT ACCELERATION TECHNIQUE.
*
* INPUT PARAMETERS:
*  IPTRK  : L_BIVAC POINTER TO THE BIVAC TRACKING INFORMATION.
*  IPSYS  : L_SYSTEM POINTER TO SYSTEM MATRICES.
*  IPFLUX : L_FLUX POINTER TO THE SOLUTION.
*  NUN    : NUMBER OF UNKNOWNS IN EACH ENERGY GROUP.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  ICL1   : NUMBER OF FREE IRETATIONS IN ONE CYCLE OF THE INVERSE POWER
*           METHOD.
*  ICL2   : NUMBER OF ACCELERATED ITERATIONS IN ONE CYCLE.
*  IMPX   : PRINT PARAMETER. =0: NO PRINT ; =1: MINIMUM PRINTING;
*           =2: ITERATION HISTORY IS PRINTED.
*  IMPH   : =0: NO ACTION IS TAKEN.
*           =1: THE FLUX IS COMPARED TO A REFERENCE FLUX STORED ON LCM.
*           =2: THE CONVERGENCE HISTOGRAM IS PRINTED.
*           =3: THE CONVERGENCE HISTOGRAM IS PRINTED WITH AXIS AND
*               TITLES. THE PLOTTING FILE IS COMPLETED.
*           =4: THE CONVERGENCE HISTOGRAM IS PRINTED WITH AXIS, ACCE-
*               LERATION FACTORS AND TITLES. THE PLOTTING FILE IS
*               COMPLETED.
*  TITR   : CHARACTER*72 TITLE.
*  EPS2   : CONVERGENCE CRITERIA FOR THE FLUX.
*  EVECT  : INITIAL ESTIMATE OF THE UNKNOWN VECTOR.
*           DIMENSION EVECT(NUN,NGRP)
*
* OUTPUT PARAMETERS:
*  EVECT  : CONVERGED UNKNOWN VECTOR. DIMENSION EVECT(NUN,NGRP)
*  FKEFF  : EFFECTIVE MULTIPLICATION FACTOR
*
* SCRATCH STORAGE:
*  GRAD1  : DIMENSION GRAD1(NUN,NGRP)
*  GRAD2  : DIMENSION GRAD2(NUN,NGRP)
*  GAR1   : DIMENSION GAR1(NUN,NGRP)
*  GAR2   : DIMENSION GAR2(NUN,NGRP)
*  GAR3   : DIMENSION GAR3(NUN,NGRP)
*  GAF1   : DIMENSION GAF1(NUN,NGRP)
*  GAF2   : DIMENSION GAF2(NUN,NGRP)
*  GAF3   : DIMENSION GAF3(NUN,NGRP)
*  WORK   : DIMENSION WORK(NUN)
*
* REFERENCE:
*  A. HEBERT, 'PRECONDITIONING THE POWER METHOD FOR REACTOR CALCU-
*  LATIONS', NUCL. SCI. ENG., 94, 1 (1986).
*
*-------------------------------------- AUTHOR: A. HEBERT ; 21/06/94 ---
*
      CHARACTER*(*) TITR
      CHARACTER*12 TEXT12
      LOGICAL LOGTES,LMPH
      DOUBLE PRECISION AEAE,AEAG,AEAH,AGAG,AGAH,AHAH,BEBE,BEBG,BEBH,
     1 BGBG,BGBH,BHBH,AEBE,AEBG,AEBH,AGBE,AGBG,AGBH,AHBE,AHBG,AHBH,
     2 X,DXDA,DXDB,Y,DYDA,DYDB,Z,DZDA,DZDB,F,D2F(2,3),EVAL,ALP,BET
      DIMENSION EVECT(NUN,*),GRAD1(NUN,*),GRAD2(NUN,*),GAR1(NUN,*),
     1 GAR2(NUN,*),GAR3(NUN,*),GAF1(NUN,*),GAF2(NUN,*),GAF3(NUN,*),
     2 WORK(*),ERR(250),ALPH(250),BETA(250),ITITR(18)
      DATA EPS1,MMAXX/1.0E-5,250/
      MMAX=MIN(MAXX0,MMAXX)
      CALL MTOPEN(IMPX,IPTRK,LL4)
      IF(LL4.GT.NUN) CALL XABORT('FLUSMB: INVALID NUMBER OF UNKNOWNS.')
*
* PRECONDITIONED POWER METHOD.
      EVAL=1.0
      VVV=0.0
      ISTART=1
      ICL1B=ICL1
      IF(IMPX.GE.1) WRITE (6,600)
      IF(IMPX.GE.2) WRITE (6,610)
      DO 20 IGR=1,NGRP
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      CALL MTLDLM(TEXT12,IPTRK,IPSYS,EVECT(1,IGR),GAR1(1,IGR))
      DO 20 JGR=1,NGRP
      IF(JGR.EQ.IGR) GO TO 20
      WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 20
      CALL MTLDLM(TEXT12,IPTRK,IPSYS,EVECT(1,JGR),WORK)
      DO 10 I=1,LL4
   10 GAR1(I,IGR)=GAR1(I,IGR)-WORK(I)
   20 CONTINUE
*
      M=0
   30 M=M+1
*
* EIGENVALUE EVALUATION.
      AEBE=0.0D0
      BEBE=0.0D0
      DO 70 IGR=1,NGRP
      DO 40 I=1,LL4
   40 GAF1(I,IGR)=0.0
      DO 60 JGR=1,NGRP
      WRITE(TEXT12,'(1HB,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 60
      CALL MTLDLM(TEXT12,IPTRK,IPSYS,EVECT(1,JGR),WORK)
      DO 50 I=1,LL4
   50 GAF1(I,IGR)=GAF1(I,IGR)+WORK(I)
   60 CONTINUE
      DO 70 I=1,LL4
      AEBE=AEBE+GAR1(I,IGR)*GAF1(I,IGR)
   70 BEBE=BEBE+GAF1(I,IGR)**2
      EVAL=AEBE/BEBE
*
* DIRECTION EVALUATION.
      DO 110 IGR=1,NGRP
      DO 80 I=1,LL4
   80 GRAD1(I,IGR)=EVAL*GAF1(I,IGR)-GAR1(I,IGR)
      DO 100 JGR=1,IGR-1
      WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 100
      CALL MTLDLM(TEXT12,IPTRK,IPSYS,GRAD1(1,JGR),WORK)
      DO 90 I=1,LL4
   90 GRAD1(I,IGR)=GRAD1(I,IGR)+WORK(I)
  100 CONTINUE
*
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      CALL MTLDLS(TEXT12,IPTRK,IPSYS,GRAD1(1,IGR))
  110 CONTINUE
*
* DISPLACEMENT EVALUATION.
      F=0.0D0
      DELS=ABS((EVAL-VVV)/EVAL)
      VVV=EVAL
*
* EVALUATION OF THE TWO ACCELERATION PARAMETERS ALP AND BET.
      ALP=1.0D0
      BET=0.0D0
      N=0
      AEAE=0.0D0
      AEAG=0.0D0
      AEAH=0.0D0
      AGAG=0.0D0
      AGAH=0.0D0
      AHAH=0.0D0
      BEBG=0.0D0
      BEBH=0.0D0
      BGBG=0.0D0
      BGBH=0.0D0
      BHBH=0.0D0
      AEBG=0.0D0
      AEBH=0.0D0
      AGBE=0.0D0
      AGBG=0.0D0
      AGBH=0.0D0
      AHBE=0.0D0
      AHBG=0.0D0
      AHBH=0.0D0
      DO 160 IGR=1,NGRP
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      CALL MTLDLM(TEXT12,IPTRK,IPSYS,GRAD1(1,IGR),GAR2(1,IGR))
      DO 120 I=1,LL4
  120 GAF2(I,IGR)=0.0
      DO 160 JGR=1,NGRP
      IF(JGR.EQ.IGR) GO TO 140
      WRITE(TEXT12,'(1HA,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 140
      CALL MTLDLM(TEXT12,IPTRK,IPSYS,GRAD1(1,JGR),WORK)
      DO 130 I=1,LL4
  130 GAR2(I,IGR)=GAR2(I,IGR)-WORK(I)
  140 WRITE(TEXT12,'(1HB,2I3)') IGR,JGR
      CALL LCMLEN(IPSYS,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 160
      CALL MTLDLM(TEXT12,IPTRK,IPSYS,GRAD1(1,JGR),WORK)
      DO 150 I=1,LL4
  150 GAF2(I,IGR)=GAF2(I,IGR)+WORK(I)
  160 CONTINUE
      IF(1+MOD(M-ISTART,ICL1+ICL2).GT.ICL1) THEN
         DO 170 IGR=1,NGRP
         DO 170 I=1,LL4
*        COMPUTE (A ,A )
         AEAE=AEAE+GAR1(I,IGR)**2
         AEAG=AEAG+GAR1(I,IGR)*GAR2(I,IGR)
         AEAH=AEAH+GAR1(I,IGR)*GAR3(I,IGR)
         AGAG=AGAG+GAR2(I,IGR)**2
         AGAH=AGAH+GAR2(I,IGR)*GAR3(I,IGR)
         AHAH=AHAH+GAR3(I,IGR)**2
*        COMPUTE (B ,B )
         BEBG=BEBG+GAF1(I,IGR)*GAF2(I,IGR)
         BEBH=BEBH+GAF1(I,IGR)*GAF3(I,IGR)
         BGBG=BGBG+GAF2(I,IGR)**2
         BGBH=BGBH+GAF2(I,IGR)*GAF3(I,IGR)
         BHBH=BHBH+GAF3(I,IGR)**2
*        COMPUTE (A ,B )
         AEBG=AEBG+GAR1(I,IGR)*GAF2(I,IGR)
         AEBH=AEBH+GAR1(I,IGR)*GAF3(I,IGR)
         AGBE=AGBE+GAR2(I,IGR)*GAF1(I,IGR)
         AGBG=AGBG+GAR2(I,IGR)*GAF2(I,IGR)
         AGBH=AGBH+GAR2(I,IGR)*GAF3(I,IGR)
         AHBE=AHBE+GAR3(I,IGR)*GAF1(I,IGR)
         AHBG=AHBG+GAR3(I,IGR)*GAF2(I,IGR)
  170    AHBH=AHBH+GAR3(I,IGR)*GAF3(I,IGR)
*
  180    N=N+1
         IF(N.GT.10) CALL XABORT('FLUSM3: FAILURE OF THE OVERRELAXATIO'
     1   //'N PARAMETERS CALCULATION.')
*        COMPUTE X(M+1)
         X=BEBE+ALP*ALP*BGBG+BET*BET*BHBH+2.0D0*(ALP*BEBG+BET*BEBH
     1   +ALP*BET*BGBH)
         DXDA=2.0D0*(BEBG+ALP*BGBG+BET*BGBH)
         DXDB=2.0D0*(BEBH+ALP*BGBH+BET*BHBH)
*        COMPUTE Y(M+1)
         Y=AEAE+ALP*ALP*AGAG+BET*BET*AHAH+2.0D0*(ALP*AEAG+BET*AEAH
     1   +ALP*BET*AGAH)
         DYDA=2.0D0*(AEAG+ALP*AGAG+BET*AGAH)
         DYDB=2.0D0*(AEAH+ALP*AGAH+BET*AHAH)
*        COMPUTE Z(M+1)
         Z=AEBE+ALP*ALP*AGBG+BET*BET*AHBH+ALP*(AEBG+AGBE)
     1   +BET*(AEBH+AHBE)+ALP*BET*(AGBH+AHBG)
         DZDA=AEBG+AGBE+2.0D0*ALP*AGBG+BET*(AGBH+AHBG)
         DZDB=AEBH+AHBE+ALP*(AGBH+AHBG)+2.0D0*BET*AHBH
*        COMPUTE F(M+1)
         F=X*Y-Z*Z
         D2F(1,1)=2.0D0*(BGBG*Y+DXDA*DYDA+X*AGAG-DZDA**2-2.0D0*Z*AGBG)
         D2F(1,2)=2.0D0*BGBH*Y+DXDA*DYDB+DXDB*DYDA+2.0D0*X*AGAH
     1   -2.0D0*DZDA*DZDB-2.0D0*Z*(AGBH+AHBG)
         D2F(2,2)=2.0D0*(BHBH*Y+DXDB*DYDB+X*AHAH-DZDB**2-2.0D0*Z*AHBH)
         D2F(2,1)=D2F(1,2)
         D2F(1,3)=DXDA*Y+X*DYDA-2.0D0*Z*DZDA
         D2F(2,3)=DXDB*Y+X*DYDB-2.0D0*Z*DZDB
*        SOLUTION OF A LINEAR SYSTEM.
         CALL ALSBD(2,1,D2F,IER,2)
         IF(IER.NE.0) CALL XABORT('FLUSM3: SINGULAR MATRIX.')
         ALP=ALP-D2F(1,3)
         BET=BET-D2F(2,3)
         IF((ABS(D2F(1,3)).LE.1.0D-4).AND.(ABS(D2F(2,3)).LE.1.0D-4))
     1   GO TO 190
         GO TO 180
  190    BET=BET/ALP
         IF((ALP.LT.1.0D0).AND.(ALP.GT.0.0D0)) THEN
            ALP=1.0D0
            BET=0.0D0
         ELSE IF(ALP.LE.0.0D0) THEN
            ISTART=M+1
            ALP=1.0D0
            BET=0.0D0
         ENDIF
         DO 200 IGR=1,NGRP
         DO 200 I=1,LL4
         GRAD1(I,IGR)=ALP*(GRAD1(I,IGR)+BET*GRAD2(I,IGR))
         GAR2(I,IGR)=ALP*(GAR2(I,IGR)+BET*GAR3(I,IGR))
  200    GAF2(I,IGR)=ALP*(GAF2(I,IGR)+BET*GAF3(I,IGR))
      ENDIF
*
      LOGTES=(M.LT.ICL1B).OR.(MOD(M,ICL1B+ICL2).EQ.ICL1B)
      IF(LOGTES.AND.(DELS.LE.EPS1)) THEN
         DELT=0.0
         DO 220 IGR=1,NGRP
         DELN=0.0
         DELD=0.0
         DO 210 I=1,LL4
         DELN=MAX(DELN,ABS(GRAD1(I,IGR)))
         EVECT(I,IGR)=EVECT(I,IGR)+GRAD1(I,IGR)
         DELD=MAX(DELD,ABS(EVECT(I,IGR)))
         GAR1(I,IGR)=GAR1(I,IGR)+GAR2(I,IGR)
         GAF1(I,IGR)=GAF1(I,IGR)+GAF2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
         GAR3(I,IGR)=GAR2(I,IGR)
  210    GAF3(I,IGR)=GAF2(I,IGR)
         IF(DELD.NE.0.0) DELT=MAX(DELT,DELN/DELD)
  220    CONTINUE
         IF(IMPX.GE.2) WRITE (6,615) M,AEAE,AEAG,AEAH,AGAG,AGAH,AHAH,
     1   BEBE,ALP,BET,EVAL,F,DELS,DELT,N,BEBG,BEBH,BGBG,BGBH,BHBH,
     2   AEBE,AEBG,AEBH,AGBE,AGBG,AGBH,AHBE,AHBG,AHBH
*        COMPUTE THE CONVERGENCE HISTOGRAM.
         IF(IMPH.GE.1) THEN
            LMPH=IMPH.GE.1
            CALL XCOMP(IPFLUX,LL4,NUN,EVECT(1,NGRP),LMPH,ERR(M))
            ALPH(M)=ALP
            BETA(M)=BET
         ENDIF
         IF(DELT.LE.EPS2) GO TO 240
      ELSE
         DO 230 IGR=1,NGRP
         DO 230 I=1,LL4
         EVECT(I,IGR)=EVECT(I,IGR)+GRAD1(I,IGR)
         GAR1(I,IGR)=GAR1(I,IGR)+GAR2(I,IGR)
         GAF1(I,IGR)=GAF1(I,IGR)+GAF2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
         GAR3(I,IGR)=GAR2(I,IGR)
  230    GAF3(I,IGR)=GAF2(I,IGR)
         IF(IMPX.GE.2) WRITE (6,620) M,AEAE,AEAG,AEAH,AGAG,AGAH,AHAH,
     1   BEBE,ALP,BET,EVAL,F,DELS,N,BEBG,BEBH,BGBG,BGBH,BHBH,AEBE,
     2   AEBG,AEBH,AGBE,AGBG,AGBH,AHBE,AHBG,AHBH
*        COMPUTE THE CONVERGENCE HISTOGRAM.
         IF(IMPH.GE.1) THEN
            LMPH=IMPH.GE.1
            CALL XCOMP(IPFLUX,LL4,NUN,EVECT(1,NGRP),LMPH,ERR(M))
            ALPH(M)=ALP
            BETA(M)=BET
         ENDIF
      ENDIF
      IF(M.EQ.1) TEST=DELS
      IF((M.GT.5).AND.(DELS.GT.TEST)) CALL XABORT('FLUSMB: CONVERGENCE'
     1 //' FAILURE.')
      IF(M.GE.MMAX) CALL XABORT('FLUSMB: MAXIMUM NUMBER OF ITERATIONS '
     1 //'REACHED.')
      GO TO 30
*
* SOLUTION EDITION.
  240 FKEFF=1.0D0/EVAL
      IF(IMPX.EQ.1) WRITE (6,640) M
      IF(IMPX.GE.1) THEN
         WRITE (6,670) FKEFF
      ENDIF
      IF(IMPX.EQ.3) THEN
         DO 250 IGR=1,NGRP
         WRITE (6,680) IGR,(EVECT(I,IGR),I=1,LL4)
  250    CONTINUE
      ENDIF
      IF(IMPH.GE.2) THEN
         IGRAPH=0
  260    IGRAPH=IGRAPH+1
         WRITE (TEXT12,'(5HHISTO,I3)') IGRAPH
         CALL LCMLEN (IPFLUX,TEXT12,ILENG,ITYLCM)
         IF(ILENG.EQ.0) THEN
            READ (TITR,'(18A4)') ITITR
            CALL LCMSIX (IPFLUX,TEXT12,1)
            CALL LCMPUT (IPFLUX,'HTITLE',18,3,ITITR)
            CALL LCMPUT (IPFLUX,'ALPHA',M,2,ALPH)
            CALL LCMPUT (IPFLUX,'BETA',M,2,BETA)
            CALL LCMPUT (IPFLUX,'ERROR',M,2,ERR)
            CALL LCMPUT (IPFLUX,'IMPH',1,1,IMPH)
            CALL LCMSIX (IPFLUX,' ',2)
         ELSE
            GO TO 260
         ENDIF
      ENDIF
      RETURN
*
  600 FORMAT(1H1/50H FLUSMB: ITERATIVE PROCEDURE BASED ON PRECONDITION,
     1 16HED POWER METHOD./9X,16HDIRECT EQUATION.)
  610 FORMAT(//5X,17HBILINEAR PRODUCTS,48X,5HALPHA,3X,4HBETA,3X,
     1 12HEIGENVALUE..,12X,8HACCURACY,11(1H.),2X,1HN)
  615 FORMAT(1X,I3,1P,7E9.1,0P,2F8.3,E14.6,3E10.2,I4/
     1 (4X,1P,7E9.1))
  620 FORMAT(1X,I3,1P,7E9.1,0P,2F8.3,E14.6,2E10.2,10X,I4/
     1 (4X,1P,7E9.1))
  640 FORMAT(/23H FLUSMB: CONVERGENCE IN,I4,12H ITERATIONS.)
  650 FORMAT(/53H FLUSMB: CPU TIME USED TO SOLVE THE TRIANGULAR LINEAR,
     1 10H SYSTEMS =,F9.3/23X,34HTO COMPUTE THE BILINEAR PRODUCTS =,
     2 F9.3,20X,16HTOTAL CPU TIME =,F9.3)
  670 FORMAT(//42H FLUSMB: EFFECTIVE MULTIPLICATION FACTOR =,1P,E17.10/)
  680 FORMAT(//47H FLUSMB: EIGENVECTOR CORRESPONDING TO THE GROUP,I4
     1 //(5X,1P,8E14.5))
      END
