*DECK FLXAXC
      SUBROUTINE FLXAXC(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        *******************************************************       *
*        * MODULE FOR COMPUTING AVERAGE CELL FLUXES, POWERS    *       *
*        * AND AXIAL FLUX SHAPES OVER FUEL LATTICE             *       *
*        *                                                     *       *
*        * MODULE DE CALCUL DES FLUX MOYENS, DES PUISSANCES ET *       *
*        * DES FORMES AXIALES DU FLUX                          *       *
*        * SUR LE RESEAU DE COMBUSTIBLE                        *       *
*        *                                                     *       *
*        *******************************************************       *
*                                                                      *
*     INPUT/OUTPUT PARAMETERS:                                         *
*      NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.    *
*      HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.         *
*      IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;*
*               =4 SEQUENTIAL ASCII FILE.                              *
*      JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.                 *
*               =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;  *
*               =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.  *
*      KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.     *
*                                                                      *
*      DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),         *
*                KENTRY(NENTRY)                                        *
*                                                                      *
*     REQUIRED LINKED LISTS ARE:                                       *
*        KENTRY(1) : TYPE L_MAP IN MODIFICATION                        *
*        KENTRY(2) : TYPE L_FLUX IN READ-ONLY MODE.                    *
*        KENTRY(3) : TYPE L_TRIVAC IN READ-ONLY MODE.                  *
*        KENTRY(4) : TYPE L_INDEX IN READ-ONLY MODE.                   *
*        KENTRY(5) : TYPE L_MACROLIB IN READ-ONLY MODE.                *
*                                                                      *
*     PROCEDURES CALLED: INIAXL,FLXCAL, AXLCAL, AXLNRM                 *
*                                                                      *
*-------------------------------------- AUTHOR: E. VARIN ; 19/09/95 ---*
*                                                                      *
*
*
      IMPLICIT NONE
      DOUBLE PRECISION  DFLOTT
      INTEGER    NENTRY
      REAL       FLOTT,EPSF,BASE
      CHARACTER HENTRY(NENTRY)*12,HSIGN*12,TEXT*12
      INTEGER  IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
      INTEGER  IPRES,IPMAC,IPFLU,IPTRK,IPIND,RDATA(6),ISTATE(13),
     +         ITYP,NZ,KCHAR(3),NITMA,NGRP,IGPARM(4),IZC,
     +         IMESHZ,IFLU,IFLUR,IFLX,IKEYF,IVOL,IMIX,NIND,IFP(5),
     +         IZONE,NZC,KF,IDATA(7),IVOLR,FLC,
     +         IFLXZ,IPRT,IBASE
      INTEGER  KX,I1,NK,NCH,NUN,NEL,ILONG,I
      DIMENSION IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
* PARAMETER VALIDATION.
*
      IF(NENTRY.LT.1) CALL XABORT('FLXAXC: PARAMETER EXPECTED.')
      IF(JENTRY(1).NE.1) CALL XABORT('FLXAXC: MODIFICATION'
     +     //' MODE EXPECTED FOR OBJECT L_MAP .')
      IPRES = 0
      IPFLU = 0
      IPMAC = 0
      IPTRK = 0
      IPIND = 0
      DO 5 I=1,NENTRY
        IF((IENTRY(I).EQ.1).OR.(IENTRY(I).EQ.2)) THEN
         CALL LCMSIX(KENTRY(I),' ',0)
         CALL LCMGET(KENTRY(I),'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF (HSIGN.EQ.'L_MAP') THEN
           IF (IPRES.EQ.0) THEN
            IPRES=KENTRY(I)
            IF(JENTRY(I).NE.1) CALL XABORT('FLXAXC: MODIFICATION'
     +     //' MODE EXPECTED FOR OBJECT '//HSIGN//'.')
           ENDIF
         ELSEIF (HSIGN.EQ.'L_INDEX') THEN
           IF(JENTRY(I).NE.2) CALL XABORT('FLXAXC: READ-ONL'
     +     //'Y MODE EXPECTED FOR OBJECT '//HSIGN//'.')
           IF (IPIND.EQ.0) IPIND=KENTRY(I)
         ELSEIF (HSIGN.EQ.'L_MACROLIB') THEN
           IF(JENTRY(I).NE.2) CALL XABORT('FLXAXC: READ-ONL'
     +     //'Y MODE EXPECTED FOR OBJECT '//HSIGN//'.')
           IF (IPMAC.EQ.0) IPMAC=KENTRY(I)
         ELSEIF (HSIGN.EQ.'L_FLUX') THEN
           IF(JENTRY(I).NE.2) CALL XABORT('FLXAXC: READ-ONL'
     +     //'Y MODE EXPECTED FOR OBJECT '//HSIGN//'.')
           IF (IPFLU.EQ.0) IPFLU=KENTRY(I)
         ELSEIF (HSIGN.EQ.'L_IQS') THEN
           IF(JENTRY(I).NE.2) CALL XABORT('FLXAXC: READ-ONL'
     +     //'Y MODE EXPECTED FOR OBJECT '//HSIGN//'.')
           IF (IPFLU.EQ.0) IPFLU=KENTRY(I)
         ELSEIF (HSIGN.EQ.'L_TRIVAC') THEN
           IF(JENTRY(I).NE.2) CALL XABORT('FLXAXC: READ-ONL'
     +     //'Y MODE EXPECTED FOR OBJECT '//HSIGN//'.')
           IF (IPTRK.EQ.0) IPTRK=KENTRY(I)
         ELSEIF (HSIGN.EQ.'L_BIVAC') THEN
           IF(JENTRY(I).NE.2) CALL XABORT('FLXAXC: READ-ONL'
     +     //'Y MODE EXPECTED FOR OBJECT '//HSIGN//'.')
           IF (IPTRK.EQ.0) IPTRK=KENTRY(I)
         ENDIF
        ENDIF
   5  CONTINUE
      IF( IPRES.EQ.0 .OR. ( IPFLU.EQ.0 .AND. IPMAC.EQ.0 ) )
     +   CALL XABORT('FLXAXC: OBJECT MISSING ')
*
*    DATA INPUT READING
*
      IPRT = 1
      IZC = 0
      KX = 0
      KF = 0
 10   CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
      IF(ITYP.NE.3) CALL XABORT('FLXAXC: CHARACTER DATA EXPECTED')
      IF(TEXT.EQ.'AXIAL') THEN
        CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
        IF(ITYP.NE.3) CALL XABORT('FLXAXC: CHARACTER DATA EXPECTED')
        IF(TEXT.EQ.'REFR') THEN
*  INITIALIZATION CALCULATION: COSINUS SHAPE PER ENEGY GROUP
          KX = 1
          CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
          IF(ITYP.NE.3) CALL XABORT('FLXAXC: CHARACTER DATA EXPECTED')
          IF(TEXT.EQ.'ZONE') THEN
             IZC = 1
          ELSEIF(TEXT.EQ.'COMP') THEN
             IZC = 0
          ELSE
             CALL XABORT('FLXAXC: ILLEGAL KEYWORD '//TEXT)
          ENDIF
          CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
          IF(ITYP.NE.3) CALL XABORT('FLXAXC: CHARACTER DATA EXPECTED')
          IF(TEXT.EQ.'FLAT') THEN
             FLC = 1
          ELSEIF(TEXT.EQ.'COS') THEN
             FLC = 2
          ELSE
             CALL XABORT('FLXAXC: ILLEGAL KEYWORD '//TEXT)
          ENDIF
        ELSEIF(TEXT.EQ.'ZONE') THEN
*  AXIAL SHAPE PER COMBUSTION ZONE PER ENERGY GROUP
          KX = 2
        ELSEIF(TEXT.EQ.'COMP') THEN
*  AXIAL SHAPE PER CHANNEL PER ENERGY GROUP
          KX = 3
        ELSE
          CALL XABORT('FLXAXC: ILLEGAL KEYWORD '//TEXT)
        ENDIF
      ELSEIF(TEXT.EQ.'FLUX-AV') THEN
         KF = 1
      ELSEIF(TEXT.EQ.'EDIT') THEN
        CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
        IF(ITYP.NE.1) CALL XABORT('FLXAXC: INTEGER DATA EXPECTED')
        IPRT = NITMA
      ELSEIF(TEXT.EQ.';') THEN
        GOTO 20
      ELSE
        CALL XABORT('FLXAXC: ILLEGAL KEYWORD '//TEXT)
      ENDIF
      GOTO 10
*
 20   CALL LCMSIX(IPRES,' ',0)
      CALL LCMGET(IPRES,'RES-DATA',RDATA)
      NK = RDATA(1)
      NCH = RDATA(2)
      NZC = RDATA(3)
      NGRP = RDATA(4)
      IF( NGRP.EQ.0 ) THEN
        IF( IPMAC.NE.0 )THEN
          CALL LCMSIX(IPMAC,' ',0)
          CALL LCMGET(IPMAC,'IDATAXS',IDATA)
          NGRP = IDATA(1)
        ELSEIF( IPFLU.NE.0 )THEN
          CALL LCMSIX(IPFLU,' ',0)
          CALL LCMGET(IPFLU,'FLUX-PARAM',IFP)
          NGRP = IFP(1)
        ELSE
          CALL XABORT('FLXAXC: NO FLU NEITHER MAC FOR NGRP')
        ENDIF
        RDATA(4) = NGRP
        CALL LCMPUT(IPRES,'RES-DATA',6,1,RDATA)
      ENDIF
      IF( KF.EQ.1.OR.KX.GT.1 )THEN
       IF( IPTRK.EQ.0 .OR. IPFLU.EQ.0.OR. IPIND.EQ.0 )
     +   CALL XABORT('FLXAXC: MISSING OBJECT FOR FLUX CALCULATION')
       WRITE(6,*) ' CALCUL DES FLUX MOYENS SUR LE RESEAU'
*
*     COMPUTATION OF AVERAGE FLUXES IN THE FUEL LATTICE
*
       CALL LCMSIX(IPFLU,' ',0)
       CALL LCMGET(IPFLU,'FLUX-PARAM',IFP)
       CALL LCMSIX(IPTRK,' ',0)
       CALL LCMGET(IPTRK,'GPARAM',IGPARM)
       NEL = IGPARM(1)
       NUN = IGPARM(2)
       IF( IFP(1).NE.NGRP )CALL XABORT('FLXAXC: INVALID NUMBER OF ENER'
     +   //'GY GROUPS IN FUEL MAP OR FLUX')
       IF( IFP(2).NE.NUN )CALL XABORT('FLXAXC: INVALID NUMBER OF UNKNO'
     +   //'WS IN TRACKING OR FLUX')
       CALL SETARA(IBASE,NEL,IKEYF)
       CALL SETARA(BASE,NEL,IVOL)
       CALL SETARA(BASE,NUN*NGRP,IFLU)
       CALL SETARA(BASE,NCH*NK*NGRP,IFLUR)
       CALL SETARA(BASE,NCH*NK,IVOLR)
       CALL LCMSIX(IPIND,' ',0)
       CALL LCMLEN(IPIND,'MIX',ILONG,ITYP)
       IF (ILONG.NE.0) THEN
         IF(ILONG.LT.NEL) CALL XABORT('FLXAXC: INDEX INCOMPATIBLE'
     +   //' WITH TRACKING')
         NIND = ILONG
       ELSE
         CALL XABORT('FLXAXC: WRONG OBJECT L_INDEX')
       ENDIF
       CALL SETARA(IBASE,NIND,IMIX)
       CALL FLXCAL(IPFLU,IPTRK,IPIND,NEL,NUN,NCH,NK,NGRP,IBASE(IKEYF),
     +      BASE(IVOL),BASE(IFLU),BASE(IFLUR),IBASE(IMIX),BASE(IVOLR))
       CALL LCMSIX(IPRES,' ',0)
       CALL LCMPUT(IPRES,'FLUX-AV',NCH*NK*NGRP,2,BASE(IFLUR))
       CALL LCMPUT(IPRES,'VOLUME',NCH*NK,2,BASE(IVOLR))
       CALL RLSARA(BASE(IFLU))
       CALL RLSARA(BASE(IVOLR))
       CALL RLSARA(IBASE(IKEYF))
       CALL RLSARA(IBASE(IMIX))
       CALL RLSARA(BASE(IVOL))
       WRITE(6,*) 'END OF AVERAGE FLUX CALCULATION'
      ENDIF
*
      IF(KX.EQ.1) THEN
       CALL LCMSIX(IPRES,'GEO:RES',1)
       CALL LCMGET(IPRES,'STATE',ISTATE)
       IF( ISTATE(1).EQ.7.OR.ISTATE(7).EQ.9 )THEN
        NZ = ISTATE(5)
       ELSE
        CALL XABORT('FLXAXC: NO AXIAL SHAPE ON 2-D GEOMETRY')
       ENDIF
       CALL SETARA(BASE,NZ,IFLX)
       CALL SETARA(BASE,NZ+1,IMESHZ)
       CALL LCMGET(IPRES,'MESHZ',BASE(IMESHZ))
       CALL SETARA(BASE,NCH*NK*NGRP,IFLU)
       CALL INIAXL(NGRP,NZ,NCH,NK,FLC,BASE(IMESHZ),BASE(IFLU),
     +             BASE(IFLX))
       CALL LCMSIX(IPRES,' ',0)
       CALL LCMPUT(IPRES,'FLUX-AX',NCH*NK*NGRP,2,BASE(IFLU))
       CALL RLSARA(BASE(IFLU))
       CALL RLSARA(BASE(IFLX))
       CALL RLSARA(BASE(IMESHZ))
      ENDIF
*
*     AXIAL SHAPE CALCULATION
*
      IF(KX.GE.2) THEN
*  CALCULATION OF AXIAL SHAPE PER COMBUSTION ZONE PER ENERGY GROUP
         CALL SETARA(IBASE,NCH,IZONE)
         CALL LCMGET(IPRES,'ZONE',IBASE(IZONE))
         CALL LCMLEN(IPRES,'FLUX-AX',ILONG,ITYP)
         CALL SETARA(BASE,NCH*NK*NGRP,IFLX)
         IF (ILONG.NE.0) THEN
          CALL LCMGET(IPRES,'FLUX-AX',BASE(IFLX))
         ELSE
          IF(IPFLU.EQ.0)
     +       CALL XABORT('FLXAXC: PREVIOUS CALL WITH REFR REQUIRED')
          DO 40 I=1,NCH*NK*NGRP
            BASE(IFLX+I-1) = 0.0
  40      CONTINUE
         ENDIF
         CALL SETARA(BASE,NZC*NK*NGRP,IFLXZ)
         CALL AXLCAL(NGRP,NZC,NCH,NK,IBASE(IZONE),BASE(IFLUR),
     +              BASE(IFLX),BASE(IFLXZ),EPSF,KX)
         WRITE(6,*) ' AXIAL CONVERGENCE FACTOR: ',EPSF
         CALL AXLNRM(NGRP,NCH,NK,BASE(IFLX))
         CALL LCMSIX(IPRES,' ',0)
         CALL LCMPUT(IPRES,'FLUX-AX',NCH*NK*NGRP,2,BASE(IFLX))
         CALL LCMPUT(IPRES,'EPSF-AX',1,2,EPSF)
         CALL RLSARA(BASE(IFLX))
         CALL RLSARA(BASE(IFLXZ))
         CALL RLSARA(IBASE(IZONE))
      ENDIF
      IF( KF.EQ.1.OR.KX.GT.1 )THEN
       CALL RLSARA(BASE(IFLUR))
      ENDIF
      RETURN
*
      END
