*DECK FLXCAL
      SUBROUTINE FLXCAL(IPFLU,IPTRK,IPIND,NEL,NUN,NCH,NK,NGRP,KEYFLX,
     +      VOL,FLU,FLUR,MIX,VOLR)
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        *******************************************************       *
*        * MODULE TO COMPUTE AVERAGE FLUXES OVER THE FUEL      *       *
*        * LATTICE                                             *       *
*        * MODULE DE CALCUL DES FLUX MOYEN SUR LE RESEAU DE    *       *
*        * COMBUSTIBLE                                         *       *
*        *                                                     *       *
*        *******************************************************       *
*                                                                      *
* INPUT PARAMETERS:
*  IPFLU  : ADDRESS OF THE FLUX LINKED LIST OR XSM FILE.
*  IPTRK  : ADDRESS OF THE TRACKING LINKED LIST OR XSM FILE.
*  IPIND  : ADDRESS OF THE INDEX LINKED LIST OR XSM FILE.
*  NEL    : NUMBER OF FINITE ELEMENTS
*  NUN    : NUMBER OF UNKNOWNS PER ENERGY GROUP
*  NCH    : NUMBER OF CHANNELS
*  NK     : NUMBER OF BUNDLES PER CHANNEL
*  NGRP   : NUMBER OF ENERGY GROUPS
*  KEYFLX : ELEMENT-ORDERED INDICES POINTING ON AVERAGE FLUXES. DIM: NEL
*  VOL    : ELEMENT_ORDERED VOLUME. DIM: NEL
*  FLU    : FLUXES. DIM: NUN*NGRP
*  MIX    : ELEMENT-ORDERED INDICES ON GEOMETRY. DIM: => NEL
*
* OUTPUT PARAMETERS:
*  FLUR    : MEAN FLUXES ON FUEL MAP. DIM: NK*NCH*NGRP
*  VOLR    : FUEL MAP VOLUMES. DIM: NK*NCH
*
*
*
*----------------:-------------------------:---------------------------*
*                                                                      *
      IMPLICIT NONE
*
      CHARACTER  TEXT12*12
      INTEGER IPFLU,IPTRK,IPIND,NEL,NUN,NCH,NK,NGRP
      REAL    VOL(NEL),FLU(NUN,NGRP),FLUR(NCH,NK,NGRP),VOLR(NCH,NK)
      INTEGER KEYFLX(NEL),MIX(*)
*
      INTEGER IK,ICH,IG,IM,IRES
*
*     RECOVER INFORMATIONS
*
      CALL LCMGET(IPIND,'MIX',MIX)
      CALL LCMGET(IPTRK,'KEYFLX',KEYFLX)
      CALL LCMGET(IPTRK,'VOLUME',VOL)
      DO 1 IG=1,NGRP
         WRITE(TEXT12,'(4HFLUX,I3)') IG
         CALL LCMGET(IPFLU,TEXT12,FLU(1,IG))
  1   CONTINUE
*
      DO 4 IK=1,NK
        DO 5 ICH=1,NCH
          VOLR(ICH,IK) = 0.0
          DO 6 IG=1,NGRP
            FLUR(ICH,IK,IG) = 0.0
  6       CONTINUE
  5     CONTINUE
  4   CONTINUE
*
      DO 10 IK=1,NK
       DO 10 ICH=1,NCH
         IRES = (IK-1)*NCH + ICH
         DO 20 IM=1,NEL
           IF(MIX(IM).EQ.-IRES) THEN
*
             DO 21 IG=1,NGRP
               FLUR(ICH,IK,IG) = FLUR(ICH,IK,IG)
     +                          + FLU(KEYFLX(IM),IG)*VOL(IM)
 21          CONTINUE
             VOLR(ICH,IK) = VOLR(ICH,IK) + VOL(IM)
           ENDIF
 20      CONTINUE
         DO 11 IG=1,NGRP
          FLUR(ICH,IK,IG) = FLUR(ICH,IK,IG)/VOLR(ICH,IK)
 11      CONTINUE
 10   CONTINUE
*
      RETURN
*
      END
