*DECK INTFLU
      SUBROUTINE INTFLU(GEOTYP,LHEX,NXP1,NYP1,NZP1,NEL,NUN,MESHX,
     +    MESHY,MESHZ,KEYF,FLU,NGRP,SPECT,DEVPOS,NHEX,IHEX,RESP,IPRT)
*
**********************************************************************
*
*     THIS SUBROUTINE COMPUTES FLUX AT DETECTOR SITE
*
*     CE MODULE CALCULE LE FLUX MOYEN DES ZONES QUI CONTIENNENT UN
*     DETECTEUR.
*
*     INPUT :
*             GEOTYP : INTEGER
*                      GEOMETRY TYPE
*             LHEX   : LOGICAL
*                      HEXAGONAL DETECTOR FLAG
*             NXP1   : INTEGER
*                      NUMBER OF X COORDINATES
*             NYP1   : INTEGER
*                      NUMBER OF Y COORDINATES
*             NZP1   : INTEGER
*                      NUMBER OF Z COORDINATES
*             NEL    : INTEGER
*                      NUMBER OF FINITE ELEMENTS
*             NUN    : INTEGER
*                      NUMBER OF UNKNOWNS
*             NGRP   : INTEGER
*                      NUMBER OF ENERGY GROUP
*             MESHX  : REAL
*                      VECTOR OF X COORDINATES. DIM: NXP1
*             MESHY  : REAL
*                      VECTOR OF X COORDINATES. DIM: NYP1
*             MESHZ  : REAL
*                      VECTOR OF X COORDINATES. DIM: NZP1
*             KEYF   : INTEGER
*                      ELEMENT-ORDERED INDICES POINTING TO AVERAGE FLUXE
*                      DIM: NEL
*             FLU    : REAL
*                      GROUP DEPENDENT UNKNOWN VECTOR. DIM: NUN*NGRP
*             SPECT  : REAL
*                      ENERGY SPECTRAL INFORMATION OF THE DETECTOR
*             DEVPOS : REAL
*                      DETECTOR COORDINATES
*             NHEX   : INTEGER
*                      NUMBER OF HEXAGONS OF THE DETECTOR
*             IHEX   : INTEGER
*                      INDEX NUMBER OF THE HEXAGONS. DIM: NHEX
*             IPRT   : INTEGER
*                      PRINTING FLAG
*     OUTPUT :
*             RESP   : REAL
*                      FLUX AT DETECTOR SITE
*
*
*     REMARQUES: SEARCH OF ZONES IDENTICAL TO NEWMAC AND XSRES
*
*
*-------------------------------------- AUTHOR: E. VARIN  94/10/10 ---
*
*
      IMPLICIT NONE
      INTEGER NXP1,NYP1,NZP1,NEL,NUN,NGRP,NX,NY,IPRT,GEOTYP,NHEX,NT
      REAL MESHX(NXP1),MESHY(NYP1),MESHZ(NZP1)
      REAL FLU(NUN,NGRP),RESP,DEVPOS(6),SPECT(NGRP)
      INTEGER  KEYF(NEL),IHEX(NHEX)
      LOGICAL  LHEX
* LOCAL
      INTEGER I,J,K,I1,I2,J1,J2,K1,K2,IAM,IGR
      REAL    X1,X2,Y1,Y2,Z1,Z2
*
      NX = NXP1 - 1
      NY = NYP1 - 1
*
      X1=DEVPOS(1)
      X2=DEVPOS(2)
      Y1=DEVPOS(3)
      Y2=DEVPOS(4)
      Z1=DEVPOS(5)
      Z2=DEVPOS(6)
*
      IF(.NOT.LHEX) THEN
        IF(X1.LT.MESHX(1)) X1=MESHX(1)
        IF(X2.LT.MESHX(1)) X2=MESHX(1)
        IF(X2.GT.MESHX(NXP1)) X2=MESHX(NXP1)
        IF(X1.GT.MESHX(NXP1)) X1=MESHX(NXP1)
*
        IF(Y1.LT.MESHY(1)) Y1=MESHY(1)
        IF(Y2.LT.MESHY(1)) Y2=MESHY(1)
        IF(Y2.GT.MESHY(NYP1)) Y2=MESHY(NYP1)
        IF(Y1.GT.MESHY(NYP1)) Y1=MESHY(NYP1)
      ENDIF
*
      IF(Z1.LT.MESHZ(1)) Z1=MESHZ(1)
      IF(Z2.LT.MESHZ(1)) Z2=MESHZ(1)
      IF(Z2.GT.MESHZ(NZP1)) Z2=MESHZ(NZP1)
      IF(Z1.GT.MESHZ(NZP1)) Z1=MESHZ(NZP1)
*
      IF(.NOT.LHEX) THEN
        DO 20 I=1,NXP1
          IF(X1.GE.MESHX(I) .AND. X1.LE.MESHX(I+1)) THEN
            I1=I
          ENDIF
          IF(X2.GE.MESHX(I) .AND. X2.LE.MESHX(I+1)) THEN
            I2=I
            GOTO 10
          ENDIF
  20    CONTINUE
*
  10    DO 30 J=1,NYP1
          IF(Y1.GE.MESHY(J) .AND. Y1.LE.MESHY(J+1)) THEN
              J1=J
          ENDIF
          IF(Y2.GE.MESHY(J) .AND. Y2.LE.MESHY(J+1)) THEN
            J2=J
            GOTO 40
          ENDIF
  30    CONTINUE
  40    CONTINUE
      ELSE
        J1 = 1
        J2 = 1
        I1 = 1
        I2 = NHEX
      ENDIF
*
      DO 50 K=1,NZP1
          IF(Z1.GE.MESHZ(K) .AND. Z1.LE.MESHZ(K+1)) THEN
             K1=K
          ENDIF
          IF(Z2.GE.MESHZ(K) .AND. Z2.LE.MESHZ(K+1)) THEN
             K2=K
             GOTO 60
          ENDIF
  50  CONTINUE
*
  60  RESP = 0.0
      NT = 0
*
      IF(IPRT.GT.4) WRITE(6,*) 'POS GEOM ',I1,I2,J1,J2,K1,K2
      DO 70 K=K1,K2
       DO 71 J=J1,J2
        DO 72 I=I1,I2
          NT = NT+1
          IF(LHEX) THEN
            IAM = (K-1)*NX+IHEX(I)
          ELSE
            IAM=(K-1)*NX*NY+(J-1)*NX+I
          ENDIF
          DO 73 IGR=1,NGRP
            RESP = RESP + SPECT(IGR)*FLU(KEYF(IAM),IGR)
  73      CONTINUE
          IF(IPRT.GT.4) WRITE(6,*) 'INTFLU: FINITE ELEMENT NUMBER ',
     +                             IAM
  72  CONTINUE
  71  CONTINUE
  70  CONTINUE
*
      RESP = RESP / FLOAT(NT)
*
      RETURN
      END
