*DECK INTRPL
      SUBROUTINE  INTRPL(NBURN,NGRP,NBURNF,DBURN,BURNUP,TAB,SECT,IG)
*
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        *******************************************************       *
*        *   CETTE ROUTINE INTERPOLE LE TABLEAU DE SECTIONS    *       *
*        *   EFFICACE SELON LE BURNUP POUR AVOIR DELTA B       *       *
*        *   CONSTANT                                          *       *
*        *                                                     *       *
*        *   THIS SUBROUTINE INTERPOLATES A BURNUP DEPENDENT   *       *
*        *   XS VECTOR TO OBTAIN A FIX BURNUP INTERVAL         *       *
*        *                                                     *       *
*        *******************************************************       *
*                                                                      *
* :---------------------:                                              *
* :INPUT PARAMETER      :                                              *
* :---------------------:                                              *
*                                                                      *
*     NBURN   : NUMBER OF INITIAL BURNUP STEPS                         *
*     NGRP    : NUMBER OF ENRGY GROUPS                                 *
*     NBURNF  : NUMBER OF FINAL BURNUP STEPS                           *
*     DBURN   : BURNUP INCREMENT                                       *
*     BURNUP  : BURNUP STEP VECTOR                                     *
*     SECT    : CROSS SECTIONS AND DERIVATIVES VECTOR                  *
*               EX: SECT(NBURN,NGRP,2)                                 *
*                                                                      *
* :---------------------:                                              *
* :OUTPUT PARAMETERS    :                                              *
* :---------------------:                                              *
*                                                                      *
*     TAB     : INTERPOLATED CROSS SECTIONS                            *
*               EX: TAB(NGRP,NBURNF,2)                                 *
*                                                                      *
*                                                                      *
*                                                                      *
*----------------:-------------------------:------ E. VARIN 26/09/95 --*
*
      IMPLICIT   NONE
      INTEGER    NBURN,NGRP,NBURNF,I,IG,J
      INTEGER    ILOW,IHIGH
      REAL       SECT(NBURN,NGRP,2),
     +           TAB(NBURNF,NGRP,2),
     +           BURNUP(NBURN),DBURN
      REAL       U,H1,H2,H3,H4,DX,XNEW
*
*
      DO 40 I=1, NBURNF
           XNEW= (I-1)*DBURN
C          FIND XNEW(I) INTERVAL:
           ILOW = 0
           DO 30 J=1, NBURN
              IF(BURNUP(J).LE.XNEW ) ILOW = J
   30      CONTINUE
           IF( ILOW.EQ.0 )THEN
              ILOW = 1
              IHIGH= 1
           ELSE
              IHIGH= MIN0(NBURN, ILOW+1)
           ENDIF
         IF( IHIGH.EQ.ILOW )THEN
           IF( ILOW.EQ.1 )THEN
C             EXTRAPOLATED AT LEFT...
              TAB(I,IG,1)= SECT(1,IG,1)
     >               + SECT(1,IG,2) * (XNEW - BURNUP(1))
           ELSEIF( IHIGH.EQ.NBURN )THEN
C             EXTRAPOLATED AT RIGHT ...
              TAB(I,IG,1)= SECT(NBURN,IG,1)
     >               + SECT(NBURN,IG,2) * (XNEW - BURNUP(NBURN))
           ELSE
C             *** NOT NORMAL
              STOP 'ADNORMAL END'
           ENDIF
        ELSE
C          REFERENCE POSITION (-0.5,+0.5)
           DX=  BURNUP(IHIGH)-BURNUP(ILOW)
           U = (XNEW - 0.5*(BURNUP(ILOW)+BURNUP(IHIGH)))/DX
C          CALCULATION OF HERMITE POLYNOMIALS
           H1= (0.5-U)*(0.5-U)*( 3. - 2.*(0.5-U))
           H2= (0.5-U)*(0.5-U)*( 1. -    (0.5-U))
           H3= (0.5+U)*(0.5+U)*( 3. - 2.*(0.5+U))
           H4= (0.5+U)*(0.5+U)*(-1. +    (0.5+U))
           TAB(I,IG,1) = H1*SECT(ILOW,IG,1)+H2*SECT(ILOW,IG,2)*DX
     1                  +H3*SECT(IHIGH,IG,1)+H4*SECT(IHIGH,IG,2)*DX
        ENDIF
        TAB(I,IG,2) = TAB(I,IG,1)
  40  CONTINUE
*
      RETURN
      END
