*DECK IQS
      SUBROUTINE IQS(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
* IMPROVED QUASISTATIC (IQS) SOLUTION MODULE FOR TRIVAC-3.
*
* INPUT/OUTPUT PARAMETERS:
*  NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
*  HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
*  IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
*           =4 SEQUENTIAL ASCII FILE.
*  JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
*           =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
*           =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
*  KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.
*           DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
*           KENTRY(NENTRY)
*
* LINKED LIST / XSM FILE:
*  HENTRY(1) : CREATE OR MODIFICATION TYPE(L_FLUX) => IQS SOLUTION.
*  HENTRY(2) : READ-ONLY TYPE(L_FLUX) => UNPERTURBED SOLUTION.
*  HENTRY(3) : READ-ONLY TYPE(L_SYSTEM) => PERTURBED MATRICES.
*  HENTRY(4) : READ-ONLY TYPE(L_SYSTEM) => UNPERTURBED MATRICES.
*  HENTRY(5) : READ-ONLY TYPE(L_TRIVAC) => TRACKING.
*  HENTRY(6) : READ-ONLY TYPE(L_MACROLIB) => PERTURBED PROPERTIES.
*  HENTRY(7) : READ-ONLY TYPE(L_MACROLIB) => UNPERTURBED PROPERTIES.
*  HENTRY(8) : READ-ONLY TYPE(L_GEOM) => GEOMETRY.
*
*-------------------------------------- AUTHOR: A. HEBERT ; 20/12/93 ---
*                                     MODIFIED: E. VARIN  ; 02/10/96 ---
*
      CHARACTER HENTRY(*)*12,TEXT12*12,TITLE*72,HSIGN*12,TEXT4*4
      LOGICAL   REC,LBI
      DIMENSION IENTRY(*),JENTRY(*),KENTRY(*),IGP(7),IBASE(1),KCHAR(18),
     +          ITP(19),ISP(7),IFP(5)
      COMMON    BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      LBI=.FALSE.
* PARAMETER VALIDATION.
      IF(NENTRY.LT.8) CALL XABORT('IQS: EIGHT PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('IQS: LI'
     1 //'NKED LIST OR XSM FILE EXPECTED AT LHS.')
      IF((JENTRY(1).NE.0).AND.(JENTRY(1).NE.1)) CALL XABORT('IQS: EN'
     1 //'TRY IN CREATE OR MODIFICATION MODE EXPECTED.')
      IF((JENTRY(2).NE.2).OR.((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)))
     1 CALL XABORT('IQS: LINKED LIST OR XSM FILE IN READ-ONLY MODE E'
     2 //'XPECTED AT FIRST RHS.')
      IF((JENTRY(3).NE.2).OR.((IENTRY(3).NE.1).AND.(IENTRY(3).NE.2)))
     1 CALL XABORT('IQS: LINKED LIST OR XSM FILE IN READ-ONLY MODE E'
     2 //'XPECTED AT SECOND RHS.')
      IF((JENTRY(4).NE.2).OR.((IENTRY(4).NE.1).AND.(IENTRY(4).NE.2)))
     1 CALL XABORT('IQS: LINKED LIST OR XSM FILE IN READ-ONLY MODE E'
     2 //'XPECTED AT THIRD RHS.')
      IF((JENTRY(5).NE.2).OR.(IENTRY(5).NE.1))
     1 CALL XABORT('IQS: LINKED LIST IN READ-ONLY MODE E'
     2 //'XPECTED AT FOURTH RHS.')
      IF((JENTRY(6).NE.2).OR.((IENTRY(6).NE.1).AND.(IENTRY(6).NE.2)))
     1 CALL XABORT('IQS: LINKED LIST OR XSM FILE IN READ-ONLY MODE E'
     2 //'XPECTED AT FIFTH RHS.')
      IF((JENTRY(7).NE.2).OR.((IENTRY(7).NE.1).AND.(IENTRY(7).NE.2)))
     1 CALL XABORT('IQS: LINKED LIST OR XSM FILE IN READ-ONLY MODE E'
     2 //'XPECTED AT SIXTH RHS.')
      IF((JENTRY(8).NE.2).OR.((IENTRY(8).NE.1).AND.(IENTRY(8).NE.2)))
     1 CALL XABORT('IQS: LINKED LIST OR XSM FILE IN READ-ONLY MODE E'
     2 //'XPECTED AT SEVENTH RHS.')
* L_IQS
      IF(JENTRY(1).EQ.1) THEN
         CALL LCMGET(KENTRY(1),'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_IQS') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('IQS: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_IQS EXPECTED.')
         ENDIF
      ELSE
         HSIGN='L_IQS'
         READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMPUT(KENTRY(1),'SIGNATURE',3,3,KCHAR)
      ENDIF
* L_FLUX (UNPERTURBED)
      CALL LCMGET(KENTRY(2),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_FLUX') THEN
         TEXT12=HENTRY(2)
         CALL XABORT('IQS: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_FLUX EXPECTED.')
      ENDIF
* L_SYSTEM (PERTURBED)
      CALL LCMGET(KENTRY(3),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_SYSTEM') THEN
         TEXT12=HENTRY(3)
         CALL XABORT('IQS: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_SYSTEM EXPECTED (PERTURBED).')
      ENDIF
* L_SYSTEM (UNPERTURBED)
      CALL LCMGET(KENTRY(4),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_SYSTEM') THEN
         TEXT12=HENTRY(4)
         CALL XABORT('IQS: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_SYSTEM EXPECTED (UNPERTURBED).')
      ENDIF
* L_TRIVAC
      CALL LCMGET(KENTRY(5),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_TRIVAC') THEN
         TEXT12=HENTRY(5)
         CALL XABORT('IQS: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_TRIVAC EXPECTED.')
      ENDIF
* L_MACROLIB (PERTURBED)
      CALL LCMGET(KENTRY(6),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_MACROLIB') THEN
         TEXT12=HENTRY(6)
         CALL XABORT('IQS: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_MACROLIB EXPECTED (PERTURBED).')
      ENDIF
* L_MACROLIB (UNPERTURBED)
      CALL LCMGET(KENTRY(7),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_MACROLIB') THEN
         TEXT12=HENTRY(7)
         CALL XABORT('IQS: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_MACROLIB EXPECTED (UNPERTURBED).')
      ENDIF
* L_GEOM
      CALL LCMGET(KENTRY(8),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_GEOM') THEN
         TEXT12=HENTRY(8)
         CALL XABORT('IQS: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_GEOM EXPECTED.')
      ENDIF
*
      IPFLUP=KENTRY(1)
      IPFLU0=KENTRY(2)
      IPSYSP=KENTRY(3)
      IPSYS0=KENTRY(4)
      IPTRK=KENTRY(5)
      IPMACP=KENTRY(6)
      IPMAC0=KENTRY(7)
      IPGEOM=KENTRY(8)
      REC=(JENTRY(1).EQ.1)
*
*
* RECOVER GENERAL TRACKING AND MACROLIB INFORMATION.
  100 CALL LCMGET(IPTRK,'GPARAM',IGP)
      CALL LCMGET(IPTRK,'TPARAM',ITP)
      NEL=IGP(1)
      NUN=IGP(2)
      MXMIX=IGP(4)
      MAXNEL=NEL
      LL4 = ITP(8)
      CALL LCMGET(IPMAC0,'IDATAXS',IGP)
      NGRP=IGP(1)
      MAXMIX=IGP(2)
      NL=IGP(3)
      NDG = 0
      IF(IGP(7).NE.0) THEN
        NDG=IGP(7)
        LBI=.TRUE.
        WRITE(6,*) ' NUMBER OF DELAYED GROUPS ',NDG
        WRITE(6,*) ' CHI-P DIFFERENT FROM CHI-I '
      ENDIF
      CALL SETARA(IBASE,NEL,IMAT)
      CALL SETARA(BASE,NEL,IVOL)
      CALL SETARA(IBASE,NEL,IIDL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IIDL))
      CALL LCMLEN(IPTRK,'TITLE',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(IPTRK,'TITLE',KCHAR)
         WRITE(TITLE,'(18A4)') (KCHAR(I),I=1,18)
      ELSE
         TITLE='*** NO TITLE PROVIDED ***'
      ENDIF
*
* TEST COMPATIBLE INFORMATIONS BETWEEN OBJECTS
      IF( MXMIX.GT.MAXMIX )CALL XABORT('IQS: NOT ENOUGH PROPERTIES'
     +  //' IN MACROLIB TO BE ADDRESSED BY GEOMETRY')
*
      CALL LCMGET(IPGEOM,'STATE',ITP)
      IF( ITP(7).NE.MXMIX )CALL XABORT('IQS: TRACKING AND GEOMETRY'
     +   //' HAVE DIFFERENT NUMBER OF MAXIMUM MIXTURES')
      IF( ITP(6).GT.NEL )CALL XABORT('IQS: TRACKING AND GEOMETRY'
     +   //' HAVE INCOMPATIBLE NUMBER OF FINITE ELEMENTS')
*
      CALL LCMGET(IPMACP,'IDATAXS',IGP)
      IF( IGP(1).NE.NGRP )CALL XABORT('IQS: PERTURBED AND UNPERTURBED'
     +   //' MACROLIB HAVE DIFFERENT NUMBER OF ENERGY GROUPS')
      IF( IGP(2).NE.MAXMIX )CALL XABORT('IQS: PERTURBED AND UNPERTURBED'
     +   //' MACROLIB HAVE DIFFERENT NUMBER OF MATERIAL MIXTURES')
*
      CALL LCMGET(IPSYS0,'SYS-PARAM',ISP)
      IF( ISP(1).NE.NGRP )CALL XABORT('IQS: UNPERTURBED MACROLIB '
     +   //' AND SYSTEM HAVE DIFFERENT NUMBER OF ENERGY GROUPS')
      IF( ISP(2).NE.LL4 )CALL XABORT('IQS: UNPERTURBED SYSTEM '
     +   //' AND TRACKING HAVE DIFFERENT NUMBER OF LINEAR ORDER.')
      IF( ISP(3).NE.NDG )CALL XABORT('IQS: UNPERTURBED MACROLIB '
     +   //' AND SYSTEM HAVE DIFFERENT NUMBER OF DELAYED NEUTRONS')
*
      CALL LCMGET(IPSYSP,'SYS-PARAM',ISP)
      IF( ISP(1).NE.NGRP )CALL XABORT('IQS: PERTURBED MACROLIB '
     +   //' AND SYSTEM HAVE DIFFERENT NUMBER OF ENERGY GROUPS')
      IF( ISP(2).NE.LL4 )CALL XABORT('IQS: PERTURBED SYSTEM '
     +   //' AND TRACKING HAVE DIFFERENT NUMBER OF LINEAR ORDER.')
      IF( ISP(3).NE.0 )CALL XABORT('IQS: PERTURBED SYSTEM '
     +   //'CONTAINS DELAYED NEUTRON B-TYPE MATRIX')
*
      CALL LCMGET(IPFLU0,'FLUX-PARAM',IFP)
      IF( IFP(5).NE.0 )CALL XABORT('IQS: UNPERTURBED '
     +   //'FLUX CONTAINS GPT SOLUTION')
      IF( IFP(1).NE.NGRP )CALL XABORT('IQS: UNPERTURBED MACROLIB '
     +   //' AND FLUX HAVE DIFFERENT NUMBER OF ENERGY GROUPS')
      IF( IFP(2).NE.NUN )CALL XABORT('IQS: UNPERTURBED FLUX '
     +   //'AND TRACKING HAVE DIFFERENT NUMBER OF UNKNOWS')
      IF( IFP(3).NE.1 .OR. IFP(4).NE.0 )CALL XABORT('IQS: UNPERTURBED '
     +   //'FLUX DOES NOT CONTAIN ADJOINT CALCULATION')
      IF(REC) THEN
        CALL LCMGET(IPFLUP,'IQS-PARAM',IFP)
        IF( IFP(1).NE.NGRP )CALL XABORT('IQS: PERTURBED MACROLIB '
     +   //' AND FLUX HAVE DIFFERENT NUMBER OF ENERGY GROUPS')
        IF( IFP(2).NE.NUN )CALL XABORT('IQS: PERTURBED FLUX '
     +   //'AND TRACKING HAVE DIFFERENT NUMBER OF UNKNOWS')
        IF( NDG.NE.0 )THEN
          IF( IFP(3).NE.NDG )CALL XABORT('IQS: PERTURBED MACROLIB '
     +    //' AND FLUX HAVE DIFFERENT NUMBER OF DELAYED NEUTRONS')
        ELSE
          NDG = IFP(3)
        ENDIF
      ENDIF
*
* READ THE NUMBER OF DELAYED NEUTRON GROUPS.
      IMPX=1
  110 CALL REDGET (INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('IQS: CHARACTER DATA EXPECTED.')
      IF(TEXT4.EQ.'DELN') THEN
         IF( NDG.NE.0 )CALL XABORT('IQS: NDG IS ALREADY'
     +    //' DEFINED.')
         CALL REDGET(INDIC,NDG,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('IQS: INTEGER DATA EXPECTED(1).')
         WRITE(6,*) ' NUMBER OF DELAYED GROUPS ',NDG
         WRITE(6,*) ' CHI-P EQUAL TO CHI-I '
         GOTO 120
      ELSE IF(TEXT4.EQ.'EDIT') THEN
         CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('IQS: INTEGER DATA EXPECTED(2).')
         IF(NDG.NE.0) GOTO 120
      ELSE
         CALL XABORT('IQS: EDIT OR NUMBER OF DELAYED GROUPS EXPECTED.')
      ENDIF
      GO TO 110
*
* COMPUTE THE IQ SOLUTION.
  120 CALL SETARA(BASE,NGRP*NUN,IEVF)
      CALL SETARA(BASE,NGRP*NUN,IAD1)
      CALL SETARA(BASE,MAXMIX,ISGD)
      CALL SETARA(BASE,NUN,IRM)
      CALL SETARA(BASE,NGRP*NUN,IOV11)
      CALL SETARA(BASE,NGRP*NUN,ISHAPE)
      CALL SETARA(BASE,2*NGRP,IAMPG)
      CALL IQSDRV(MAXNEL,MAXMIX,IPGEOM,IPTRK,IPMAC0,IPMACP,IPSYS0,
     1 IPSYSP,IPFLU0,REC,NEL,NUN,IBASE(IMAT),BASE(IVOL),IBASE(IIDL),
     2 NGRP,NDG,IMPX,TITLE,BASE(IEVF),IPFLUP,BASE(IAD1),BASE(ISGD),
     3 BASE(IRM),BASE(IOV11),BASE(ISHAPE),BASE(IAMPG),LBI,NL)
      CALL RLSARA(BASE(IAMPG))
      CALL RLSARA(BASE(ISHAPE))
      CALL RLSARA(BASE(IOV11))
      CALL RLSARA(BASE(IRM))
      CALL RLSARA(BASE(ISGD))
      CALL RLSARA(BASE(IAD1))
      CALL RLSARA(BASE(IEVF))
*
* RELEASE GENERAL TRACKING INFORMATION.
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
