*DECK IQSDRV
      SUBROUTINE IQSDRV (MAXNEL,MAXMIX,IPGEOM,IPTRK,IPMAC0,IPMACP,
     1 IPSYS0,IPSYSP,IPFLU0,REC,NEL,NUN,MAT,VOL,IDL,NGRP,NDG,IMPX,TITR,
     2 EVECT,IPFLUP,ADECT,SGD,RM,V,SHAPE,AMPGA,LBI,NL)
*
*-----------------------------------------------------------------------
*
* DRIVER FOR THE SOLUTION OF THE IMPROVED QUASISTATIC (IQS) EQUATIONS.
* KAPS-RENTROP VERSION.
* NOTE: THIS IS THE MULTIGROUP VERSION.
*
* INPUT PARAMETERS:
*  MAXNEL : MAXIMUM NUMBER OF FINITE ELEMENTS.
*  MAXMIX : MAXIMUM NUMBER OF MATERIAL MIXTURES.
*  IPGEOM : L_GEOM POINTER TO THE GEOMETRY.
*  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPMAC0 : L_MACROLIB POINTER TO THE UNPERTURBED MACROLIB.
*  IPMACP : L_MACROLIB POINTER TO THE DELTA MACROLIB.
*  IPSYS0 : L_SYSTEM POINTER TO UNPERTURBED SYSTEM MATRICES.
*  IPSYSP : L_SYSTEM POINTER TO DELTA SYSTEM MATRICES.
*  IPFLU0 : L_FLUX POINTER TO THE UNPERTURBED SOLUTION.
*  REC    : .TRUE. RECOVER THE EXISTING SOLUTION AS GPT INITIAL
*           ESTIMATE; .FALSE. USE A UNIFORM GPT INITIAL ESTIMATE.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : TOTAL NUMBER OF UNKNOWNS PER GROUP.
*  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*           DIMENSION MAT(NEL)
*  VOL    : VOLUMES. DIMENSION VOL(NEL)
*  IDL    : POSITION OF THE AVERAGE FLUX COMPONENT ASSOCIATED WITH
*           EACH VOLUME. DIMENSION IDL(NEL)
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NDG    : NUMBER OF PRECURSOR GROUPS.
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*  TITR   : TITLE.
*   LBI   : LOGICAL VARIABLE FOR NEUTRON FISSION SPECTRUM.
*
* OUTPUT PARAMETER:
*  IPFLUP : L_IQS POINTER TO THE IQS SOLUTION.
*
* SCRATCH STORAGE
*  EVECT  : DIMENSION EVECT(NUN,NGRP)
*  SHAPE  : DIMENSION SHAPE(NUN,NGRP)
*  ADECT  : DIMENSION ADECT(NUN,NGRP)
*  SGD    : DIMENSION SGD(MAXMIX)
*  RM     : DIMENSION RM(NUN)
*  V      : DIMENSION V(NUN,NGRP)
*
* INFORMATION RECOVERED FROM LCM OR XSM :
* 'A  1  1', 'A  2  2', .... :SYSTEM MATRICES RELATED TO
*                             LEAKAGE AND REMOVAL FOR EACH ENERGY GROUP.
* 'A  1  2', 'A  2  1', .... : SYSTEM MATRICES RELATED TO UP-SCATTERING
*                              AND DOWN-SCATTERING.
* 'B  1  1', 'B  1  2', .... : SYSTEM MATRICES RELATED TO FISSION.
*
* LOCAL VARIABLES:
*  DELSRC    : DELAYED SOURCE. DIMENSION DELSRC(NUN)
*  TMACRO    : REAL TIME, ALWAYS T=0.0 A START OF A TRANSIENT
*              CALCULATION
*  AMPLD     : LOG DERIVATIVE OF THE AMPLITUDE FUNCTION
*  AMP       : VALUE OF THE AMPLITUDE FUNCTION
*  REACT     : REACTIVITY
*  DECRAT(I) : DECAY RATE OF THE I-TH PRECURSOR
*  BETA(I)   : DELAYED FRACTION OF THE I-TH PRECURSOR
*  ETA(I)    : AMPLITUDE OF THE I-TH PRECUSOR CONCENTRATION
*  EPSRHO    : CONVERGENCE CRITERIA FOR RHO ESTIMATE, IN MK <----
*  STARL     : MEAN GENERATION TIME
*  ATFO      : VALUE OF THE CONSTRAINT FUNCTION
*  IMPX      : FLAG THAT INDICATES IF OUTPUT FORM FLUX CALCUL DESIRED
***
* MODIFIED BY M.T. SISSAOUI ; 28/07/94
* MODIFIED BY E. VARIN ; 28/02/95
*---------------------------------------------- A. MONIER --------------
*
      LOGICAL REC,PKIN,QSM,RENORM,LCM1,LCM2,LCM3,VELO,LTETA,LBI,TMAC
      DOUBLE PRECISION DFLOTT,FACT,ZPOW,AMPGA(NGRP)
      CHARACTER TITR*(*),TEXT4*4,TEXT12*12,CGRPNM*12,NAMLCM*12,NAMMY*12,
     1 HTRACK*12
      REAL     POWER
      DIMENSION MAT(*),VOL(*),IDL(*),EVECT(NUN,NGRP),ADECT(NUN,NGRP),
     1 SGD(*),RM(*),V(NUN,NGRP),SHAPE(NUN,NGRP),IQSP(3)
      COMMON BASE(1)
      CHARACTER*5 METHOD
*
      IF(NGRP.EQ.0) CALL XABORT('IQSDRV: ZERO NUMBER OF ENERGY GROUPS.')
      IF(.NOT.REC) THEN
        IQSP(1) = NGRP
        IQSP(2) = NUN
        IQSP(3) = NDG
        CALL LCMPUT(IPFLUP,'IQS-PARAM',3,1,IQSP)
      ENDIF
      PKIN=.FALSE.
      RENORM=.FALSE.
      QSM=.TRUE.
      VELO =.TRUE.
      LTETA=.FALSE.
      TMAC =.FALSE.
*
* LECTURE DES OPTIONS POUR LE CALCUL DES FONCTIONS DE FORME.
*     OPTIONS PAR DEFAUT :
      ICL1=3
      ICL2=3
      NADI=1
      MAXX0=250
      EPS2=1.0E-4
      EPSRHO=.01
      ERAMP=1.0E-4
      ERCON=1.0E-4
      RHOPOU=1.0
*
      IMPPK=0
      PWRINT=1.0
      LIMM=5
      TMACRO=0.0
      DO 111 IGR=1,NGRP
        AMPGA(IGR)=1.0D0
 111  CONTINUE
      TETA=0.5001
      DEL =1.E-6
      HI=1.E-3
      METHOD='GRKA'
      HTRACK='TRIVAC'
      CALL SETARA(BASE,NDG,IBETA)
      CALL SETARA(BASE,NDG,IDECR)
      IF(REC) THEN
        CALL LCMGET(IPFLUP,'BETA',BASE(IBETA))
        CALL LCMGET(IPFLUP,'DECAY',BASE(IDECR))
      ENDIF
*
* READ AN OPTION KEY WORD.
   10 CALL REDGET (INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('IQSDRV: CHARACTER DATA EXPECTED.')
      IF(TEXT4.EQ.'TETA') THEN
         LTETA=.TRUE.
      ELSE IF(TEXT4.EQ.'VTET') THEN
         CALL REDGET (INDIC,NITMA,TETA,TEXT4,DFLOTT)
          IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'BETA') THEN
         IF(REC) WRITE(6,*) 'IQSDRV: BE AWARE THAT YOU MODIFY PREVIOUS'
     +   //'LY STORED VARIABLES FOR DELAYED NEUTRON BETA'
         DO 20 I=1,NDG
           CALL REDGET (INDIC,NITMA,BASE(IBETA-1+I),TEXT4,DFLOTT)
           IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
   20    CONTINUE
      ELSE IF(TEXT4.EQ.'DECR') THEN
         IF(REC) WRITE(6,*) 'IQSDRV: BE AWARE THAT YOU MODIFY PREVIOUS'
     +   //'LY STORED VARIABLES FOR DELAYED NEUTRON DECAY'
         DO 220 I=1,NDG
*        READ IN VALUE OF LAMBDA(I)
           CALL REDGET (INDIC,NITMA,BASE(IDECR-1+I),TEXT4,DFLOTT)
           IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
  220    CONTINUE
      ELSE IF(TEXT4.EQ.'VEL') THEN
          CALL SETARA(BASE,NGRP,IVM)
          DO 12 I=1,NGRP
         CALL REDGET (INDIC,NITMA,BASE(IVM-1+I),TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
  12      CONTINUE
          VELO=.FALSE.
      ELSE IF(TEXT4.EQ.'VAR1') THEN
        CALL REDGET(INDIC,ICL1,FLOTT,TEXT4,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('IQSDRV: INTEGER DATA EXPECTED.')
        CALL REDGET(INDIC,ICL2,FLOTT,TEXT4,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('IQSDRV: INTEGER DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'ADI') THEN
        CALL REDGET(INDIC,NADI,FLOTT,TEXT4,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('IQSDRV: INTEGER DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'MAX') THEN
        CALL REDGET(INDIC,MAXX0,FLOTT,TEXT4,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('IQSDRV: INTEGER DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'CONV') THEN
        CALL REDGET (INDIC,NITMA,EPSRHO,TEXT4,DFLOTT)
        IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
        CALL REDGET (INDIC,NITMA,EPS2,TEXT4,DFLOTT)
        IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
        CALL REDGET (INDIC,LIMM,FLOTT,TEXT4,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('IQSDRV: INTEGER DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'KAPS') THEN
*       KAPS AND RENTROP OPTIONS
251     CALL REDGET (INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT('IQSDRV: CHARACTER DATA EXPECTED.')
        IF(TEXT4.EQ.'DEL') THEN
          CALL REDGET (INDIC,NITMA,DEL,TEXT4,DFLOTT)
          IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
        ELSE IF(TEXT4.EQ.'HI') THEN
          CALL REDGET (INDIC,NITMA,HI,TEXT4,DFLOTT)
          IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
        ELSE IF(TEXT4.EQ.'METH') THEN
          CALL REDGET (INDIC,NITMA,FLOTT,METHOD,DFLOTT)
          IF(INDIC.NE.3) CALL XABORT('IQSDRV: CHARACTER DATA EXPECTED.')
        ELSE IF(TEXT4.EQ.'END') THEN
          GOTO 10
        ENDIF
        GOTO 251
      ELSE IF(TEXT4.EQ.'CON2') THEN
        CALL REDGET(INDIC,NITMA,ERAMP,TEXT4,DFLOTT)
        IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
        CALL REDGET (INDIC,NITMA,ERCON,TEXT4,DFLOTT)
        IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
        CALL REDGET (INDIC,NITMA,RHOPOU,TEXT4,DFLOTT)
        IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'RENO') THEN
        RENORM=.TRUE.
      ELSE IF(TEXT4.EQ.'PKIN') THEN
        PKIN=.TRUE.
      ELSE IF(TEXT4.EQ.'QSM') THEN
        QSM=.FALSE.
      ELSE IF(TEXT4.EQ.'POWE') THEN
        CALL REDGET (INDIC,NITMA,PWRINT,TEXT4,DFLOTT)
        IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'TMAC') THEN
        TMAC=.TRUE.
      ELSE IF(TEXT4.EQ.'BEGI') THEN
        GO TO 30
      ELSE
        CALL XABORT('IQSDRV: '//TEXT4//' IS AN INVALID KEY-WORD.')
      ENDIF
      GO TO 10
   30 ISTEP=0
      TEXT12=' '
      CALL LCMNXT(IPSYS0,TEXT12,NAMLCM,NAMMY,LCM1)
      TEXT12=' '
      CALL LCMNXT(IPMAC0,TEXT12,NAMLCM,NAMMY,LCM2)
      TEXT12=' '
      CALL LCMNXT(IPSYSP,TEXT12,NAMLCM,NAMMY,LCM3)
*
* CREATE A LINKED LIST TO CONTAIN THE UNPERTURBED SYSTEM MATRICES.
      IF(LCM1) THEN
         CALL LCMOP(IPLIS0,'COPY1',0,1,0)
      ELSE
         CALL LCMOP(IPLIS0,'COPY1',0,2,0)
      ENDIF
      CALL LCMEQU(IPSYS0,IPLIS0)
*
* CREATE A LINKED LIST TO CONTAIN THE UNPERTURBED SYSTEM MATRICES.
       IF(LCM1) THEN
         CALL LCMOP(IPLISS,'COPYS',0,1,0)
      ELSE
         CALL LCMOP(IPLISS,'COPYS',0,2,0)
      ENDIF
*
* CREATE A LINKED LIST TO CONTAIN THE UNPERTURBED CROSS SECTIONS.
      IF(LCM2) THEN
         CALL LCMOP(IPMAC,'COPY2',0,1,0)
      ELSE
         CALL LCMOP(IPMAC,'COPY2',0,2,0)
      ENDIF
      CALL LCMEQU(IPMAC0,IPMAC)
*
* RECOVER THE WEIGHTING MATRIX RM.
      CALL LCMGET(IPSYSP,'RM',RM)
C RECOVER THE VELOCITIES MATRICES
      IF(VELO) THEN
        DO 165 IGR=1,NGRP
         WRITE (CGRPNM,'(1HV,I3,I3)') IGR,IGR
         CALL LCMGET(IPSYSP,CGRPNM,V(1,IGR))
  165   CONTINUE
      ENDIF
*
* RECOVER THE BEGINNING TIME OF TRANSIENT
      IF(TMAC) THEN
       IF(.NOT.REC) CALL XABORT('IQSDRV: INVALID OPTION'
     +      //' TMAC, IPFLUP DOES NOT EXIST')
       CALL LCMGET(IPFLUP,'TMACRO',TMACRO)
      ENDIF
*
*
* BEGINNING OF THE CALCULATIONS RELATIVE TO THE PERTURBATION.
      IF((TMACRO.EQ.0.0).AND.(.NOT.TMAC)) THEN
         IF(IMPX.GT.0) WRITE(6,*) ' INITIALISATION PAR FLUX STATIQUE'
*        RECOVER THE NON-PERTURBED K-EFFECTIVE AND DIRECT FLUXES.
         CALL LCMGET(IPFLU0,'K-EFFECTIVE',FKEFF)
         DO 156 IGR=1,NGRP
           WRITE (CGRPNM,'(4HFLUX,I3)') IGR
           CALL LCMGET(IPFLU0,CGRPNM,EVECT(1,IGR))
  156    CONTINUE
*
*        NORMALIZATION OF THE FLUX TO THE INITIAL POWER.
         FACT=0.0D0
         DO 105 IGR=1,NGRP
           WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
           CALL LCMSIX(IPMAC,CGRPNM,1)
           CALL LCMLEN(IPMAC,'H-FACTORS',NBMIX,ITYLCM)
           IF(NBMIX.EQ.0) THEN
              CALL XABORT('IQSDRV: NO ''H-FACTORS'' INFORMATION.')
           ELSE IF(NBMIX.GT.MAXMIX) THEN
              CALL XABORT('IQSDRV: INVALID LENGTH FOR ''H-FACTORS'' IN'
     1         //'FORMATION.')
           ENDIF
           CALL LCMGET(IPMAC,'H-FACTORS',SGD)
           CALL LCMSIX(IPMAC,' ',2)
           DO 95 I=1,NEL
             IF(MAT(I).EQ.0) GO TO 95
             FACT=FACT+VOL(I)*EVECT(IDL(I),IGR)*SGD(MAT(I))
   95      CONTINUE
  105    CONTINUE
         IF(PWRINT.NE.0.0) THEN
            FACT=PWRINT/FACT
            DO 110 I=1,NUN
              DO 112 IGR=1,NGRP
                EVECT(I,IGR)=FACT*EVECT(I,IGR)
  112         CONTINUE
  110       CONTINUE
            WRITE(6,1000) PWRINT
         ELSE
            WRITE(6,1000) FACT
         ENDIF
*
*        RECOVER THE ADJOINT FLUXES.
         DO 157 IGR=1,NGRP
           WRITE (CGRPNM,'(5HAFLUX,I3)') IGR
           CALL LCMGET(IPFLU0,CGRPNM,ADECT(1,IGR))
 157     CONTINUE
*
*        NORMALIZATION OF THE ADJOINT FLUX.
         FACT=0.0D0
         DO 120 I=1,NEL
           IF(IDL(I).EQ.0) GO TO 120
           FACT=FACT+VOL(I)*ADECT(IDL(I),1)
  120    CONTINUE
         DO 130 I=1,NUN
          DO 131 IGR=1,NGRP
            ADECT(I,IGR)=ADECT(I,IGR)/FACT
  131     CONTINUE
  130    CONTINUE
*        STORE THE NORMALIZED ADJOINT FLUXES.
         DO 159 IGR=1,NGRP
            WRITE (CGRPNM,'(5HAFLUX,I3)') IGR
            CALL LCMPUT(IPFLUP,CGRPNM,NUN,2,ADECT(1,IGR))
 159     CONTINUE
*
      ELSE
         DO 158 IGR=1,NGRP
          WRITE (CGRPNM,'(5HSHAPE,I3)') IGR
          CALL LCMGET(IPFLUP,CGRPNM,EVECT(1,IGR))
          WRITE (CGRPNM,'(5HAFLUX,I3)') IGR
          CALL LCMGET(IPFLUP,CGRPNM,ADECT(1,IGR))
  158    CONTINUE
         CALL LCMGET(IPFLU0,'K-EFFECTIVE',FKEFF)
         CALL LCMGET(IPFLUP,'AMPGA',AMPGA)
      ENDIF
*
   40 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('IQSDRV: CHARACTER DATA EXPECTED.')
      IF(TEXT4.EQ.':::') THEN
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF((INDIC.NE.3).OR.(TEXT4.NE.'OUT:')) CALL XABORT('IQSDRV: OUT'
     1   //': EXPECTED.')
         IF(IMPX.GT.0) WRITE(6,'(/34H IQSDRV: OUTPUT REACTION RATES AT ,
     1   4HTIME,F7.2,11H SECOND(S).)') TMACRO
         CALL LCMOP(IPTEMP,'TEMP',0,1,0)
         CALL LCMEQU(IPFLUP,IPTEMP)
         WRITE(TEXT12,'(4HSTEP,I8)') ISTEP
         CALL LCMSIX(IPFLUP,TEXT12,1)
         CALL SETARA(BASE,NGRP*NUN,IEV)
         CALL SETARA(BASE,NEL,IIHOM)
         CALL SETARA(BASE,MAXMIX,ISGD)
         CALL OUTDRV (MAXNEL,MAXMIX,IPGEOM,IPMAC,IPTEMP,IPFLUP,NGRP,
     1   NEL,NUN,HTRACK,MAT,VOL,IDL,TITR,BASE(IEV),BASE(IIHOM),
     2   BASE(ISGD))
         CALL RLSARA(BASE(ISGD))
         CALL RLSARA(BASE(IIHOM))
         CALL RLSARA(BASE(IEV))
         CALL LCMSIX(IPFLUP,' ',2)
         CALL LCMCL(IPTEMP,2)
       ELSE IF(TEXT4.EQ.'PERT') THEN
         CALL REDGET (INDIC,NDIV,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('IQSDRV: INTEGER DATA EXPECTED.')
         RDIV=REAL(NDIV)
         GO TO 60
       ELSE IF(TEXT4.EQ.';') THEN
         GO TO 170
       ELSE
        CALL XABORT('IQSDRV: '//TEXT4//' IS AN INVALID KEY-WORD.')
       ENDIF
       GO TO 40
*
* TRANSIENT INITIALIZATION.
   60 IDIV=0
   70 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(TEXT4.EQ.'RAMP') THEN
        ITRANA=2
        ITRANB=2
        CALL REDGET (INDIC,NITMA,DTMACR,TEXT4,DFLOTT)
        DTMACR=DTMACR/RDIV
        IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
        IF(HI.GE.DTMACR) THEN
        CALL XABORT('IQSDRV: HI MAY NOT CROSS DTMACR.')
        ENDIF
      ELSE IF(TEXT4.EQ.'CERB') THEN
        ITRANA=3
        ITRANB=3
        CALL REDGET (INDIC,NITMA,DTMACR,TEXT4,DFLOTT)
        DTMACR=DTMACR/RDIV
        IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
        IF(HI.GE.DTMACR) THEN
        CALL XABORT('IQSDRV: HI MAY NOT CROSS DTMACR.')
        ENDIF
      ELSE IF(TEXT4.EQ.'STEP') THEN
        ITRANA=1
        ITRANB=1
        CALL REDGET (INDIC,NITMA,DTMACR,TEXT4,DFLOTT)
        DTMACR=DTMACR/RDIV
        IF(INDIC.NE.2) CALL XABORT('IQSDRV: REAL DATA EXPECTED.')
        IF(HI.GE.DTMACR) THEN
        CALL XABORT('IQSDRV: HI MAY NOT CROSS DTMACR.')
        ENDIF
      ELSE IF(TEXT4.EQ.'REAC') THEN
          IMPPK=1
      ELSE IF(TEXT4.EQ.'END') THEN
        GO TO 90
      ELSE
        CALL XABORT('IQSDRV: INVALID KEY-WORD.')
      ENDIF
      GO TO 70
*
* BEGINNING OF THE CALCULATIONS RELATIVE TO THE PERTURBATION.
 90   ISTEP=ISTEP+1
      IF(IMPX.GT.0) WRITE(6,'(/23H IQSDRV: BEGINNING STEP,I4,8H AT TIME,
     1 F7.2,11H SECOND(S).)') ISTEP,TMACRO
*
* CREATE A LINKED LIST TO CONTAIN THE PERTURBED SYSTEM MATRICES.
      WRITE(TEXT12,'(4HSTEP,I8)') ISTEP
      CALL LCMLEN(IPSYSP,TEXT12,IEXIST,ITYLCM)
      IF(IEXIST.NE.0) THEN
         CALL LCMSIX(IPMACP,TEXT12,1)
         CALL LCMSIX(IPSYSP,TEXT12,1)
         IF(LCM3) THEN
            CALL LCMOP(IPLISP,'COPY3',0,1,0)
         ELSE
            CALL LCMOP(IPLISP,'COPY3',0,2,0)
         ENDIF
         CALL LCMEQU(IPSYSP,IPLISP)
         CALL LCMSIX(IPSYSP,' ',2)
*
*        USER SPECIFIED NDIV FOR THIS STEP AND THIS A RAMP
*        --> DIVIDE THE DELTA SYSTEM MATRICES BY NDIV
         IF((ITRANA.GE.2).AND.(RDIV.NE.1.0)) THEN
            FLOTT=1.0/RDIV
            CALL LCMULT(IPLISP,FLOTT)
         ENDIF
      ELSE
         IPLISP=0
      ENDIF
*
* LOOPING OVER TRANSIENT NDIV TIMES
  140 IDIV=IDIV+1
*
* USER SPECIFIED NDIV FOR THIS STEP AND THIS A STEP
* --> ALLOW ONE MATRIX UPDATE; THEN ZERO
      IF((IDIV.GT.1).AND.(ITRANA.EQ.1).AND.(IPLISP.NE.0)) THEN
         FLOTT=0.0
         CALL LCMULT(IPLISP,FLOTT)
      ENDIF
*
* COMPUTE THE BILINEAR PRODUCTS.
      CALL IQSNIN (IPTRK,IPLIS0,IPLISP,IPFLUP,NUN,NGRP,ADECT,
     1 ITRANA,ITRANB,FKEFF,NDG,LBI)
*
* STORE THE UNPERTURBED B11 AND B12 IN BASE(IGAR1) AND BASE(IGAR2).
       CALL LCMEQU(IPLIS0,IPLISS)
*
      IF(IPLISP.NE.0) THEN
*        UPDATE THE SYSTEM MATRICES.
         CALL LCMADD(IPLISP,IPLIS0)
*
*        CHOLESKY FACTORIZATION.
         DO 19 IGR= 1,NGRP
           WRITE(CGRPNM,'(1HA,I3,I3)') IGR,IGR
           CALL MTLDLF(CGRPNM,IPTRK,IPLIS0,IMPX)
  19     CONTINUE
      ENDIF
*
      IF(IMPX.NE.0) WRITE(6,2000) TMACRO
* COMPUTE THE VECTOR LENGTH FOR THE KINETICS SOLUTIONS.
      IF(LTETA) THEN
       IF(QSM) THEN
       NT=NDG+3
       ELSE
       NT=1+(1+NDG)*NGRP
       ENDIF
      ELSE
       IF(QSM) THEN
       NT=4*NDG+3
       ELSE
       NT=1+(1+4*NDG)*NGRP
       ENDIF
      ENDIF
*
* TRANSIENT CALCULATION.
      CALL SETARA(BASE,NUN*NGRP,IDELSR)
      CALL SETARA(BASE,2*NDG*NGRP,IETA)
      CALL SETARA(BASE,NDG*NUN*NGRP,IPREC)
      CALL SETARA(BASE,NDG*NUN,IPRECW)
      CALL SETARA(BASE,2*NT,IY)
      CALL SETARA(BASE,2*NT,IYDE)
      CALL SETARA(BASE,2*NGRP*NGRP,IPA00)
      CALL SETARA(BASE,2*NGRP*NGRP,IPA01)
      CALL SETARA(BASE,2*NGRP*NGRP,IPA10)
      CALL SETARA(BASE,2*NGRP*NGRP,IPA11)
      CALL SETARA(BASE,2*NGRP*NGRP,IP00)
      CALL SETARA(BASE,2*NGRP*NGRP,IP01)
      CALL SETARA(BASE,2*NGRP*NGRP,IP10)
      CALL SETARA(BASE,2*NGRP*NGRP,IP11)
      CALL SETARA(BASE,2*NGRP*NGRP,IPB00)
      CALL SETARA(BASE,2*NGRP*NGRP,IPB01)
      CALL SETARA(BASE,2*NGRP*NGRP,IPB10)
      CALL SETARA(BASE,2*NGRP*NGRP,IPB11)
      CALL SETARA(BASE,2*NGRP,IR0C)
      CALL SETARA(BASE,2*NGRP,ITERM)
      CALL SETARA(BASE,NGRP,IAMPP)
      CALL SETARA(BASE,NGRP,IAT)
      CALL SETARA(BASE,NGRP,IATOUT)
      CALL SETARA(BASE,2*NGRP,IAMP)
      CALL SETARA(BASE,2*NGRP*NDG,IGAMA)
      CALL SETARA(BASE,2*NGRP*NDG,ISIGMA)
      CALL SETARA(BASE,2*NGRP,ICONTOB)
      CALL SETARA(BASE,2*NGRP,ICONSN)
      CALL SETARA(BASE,2*NGRP,IAMPLD)
      CALL SETARA(BASE,NDG,IRAPQ)
       CALL IQSNCA (IPTRK,IPLIS0,IPFLUP,IPLISS,NUN,ADECT,
     1 EVECT,FKEFF,NDG,BASE(IBETA),BASE(IDECR),BASE(IVM),DTMACR,
     2 ITRANA,ITRANB,EPS2,EPSRHO,ERAMP,ERCON,RHOPOU,LIMM,MAXX0,
     3 ICL1,ICL2,IMPPK,IMPX,NADI,BASE(IY),BASE(IYDE),SHAPE,
     4 BASE(IDELSR),BASE(IPREC),BASE(IETA),TMACRO,AMPGA,
     5 PKIN,QSM,RENORM,RM,DEL,DBLE(HI),
     6 METHOD,VELO,V,NGRP,BASE(IAMPP),BASE(IAT),BASE(IATOUT),
     x BASE(IAMP),BASE(IGAMA),BASE(ISIGMA),BASE(ICONTOB),
     x BASE(ICONSN),BASE(IAMPLD),BASE(IPA00),BASE(IPA01),
     x BASE(IPA10),BASE(IPA11),
     x BASE(IP00),BASE(IP01),BASE(IP10),
     7 BASE(IP11),BASE(IPB00),BASE(IPB01),BASE(IPB10),
     8 BASE(IPB11),BASE(IR0C),BASE(ITERM),BASE(IPRECW),LBI,
     x LTETA,TETA,NT,BASE(IRAPQ))
*
      DO 258 IGR=1,NGRP
         CALL TRIFLX(IPTRK,NEL,NUN,SHAPE(1,IGR),MAT,VOL,IDL)
  258  CONTINUE
*
*     CALCULATE THE REACTOR POWER.
      ZPOW=0.0D0
      DO 155 IGR=1,NGRP
        WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
        CALL LCMSIX(IPMAC,CGRPNM,1)
        CALL LCMLEN(IPMAC,'H-FACTORS',NBMIX,ITYLCM)
        IF(NBMIX.EQ.0) THEN
          CALL XABORT('IQSDRV: NO ''H-FACTORS'' INFORMATION.')
        ELSE IF(NBMIX.GT.MAXMIX) THEN
          CALL XABORT('IQSDRV: INVALID LENGTH FOR ''H-FACTORS'' IN'
     1      //'FORMATION.')
        ENDIF
        CALL LCMGET(IPMAC,'H-FACTORS',SGD)
        CALL LCMSIX(IPMAC,' ',2)
        DO 145 I=1,NEL
          IF(MAT(I).EQ.0) GO TO 145
          ZPOW=ZPOW+VOL(I)*SHAPE(IDL(I),IGR)*SGD(MAT(I))*AMPGA(IGR)
  145   CONTINUE
  155 CONTINUE
      IF(ZPOW.GT.1D+6) THEN
        WRITE(6,*) ' REACTOR POWER=',ZPOW*1.D-6,
     +        ' MW AT TIME=',TMACRO+DTMACR
      ELSEIF(ZPOW.GT.1D+3) THEN
        WRITE(6,*) ' REACTOR POWER=',ZPOW*1.D-3,
     +         ' KW AT TIME=',TMACRO+DTMACR
      ELSE
        WRITE(6,*) ' REACTOR POWER=',ZPOW,
     +         ' W AT TIME=',TMACRO+DTMACR
      ENDIF
*
      CALL RLSARA(BASE(IRAPQ))
      CALL RLSARA(BASE(IAMPLD))
      CALL RLSARA(BASE(ICONSN))
      CALL RLSARA(BASE(ICONTOB))
      CALL RLSARA(BASE(ISIGMA))
      CALL RLSARA(BASE(IGAMA))
      CALL RLSARA(BASE(IAMP))
      CALL RLSARA(BASE(IATOUT))
      CALL RLSARA(BASE(IAT))
      CALL RLSARA(BASE(IAMPP))
      CALL RLSARA(BASE(ITERM))
      CALL RLSARA(BASE(IR0C))
      CALL RLSARA(BASE(IPB11))
      CALL RLSARA(BASE(IPB10))
      CALL RLSARA(BASE(IPB01))
      CALL RLSARA(BASE(IPB00))
      CALL RLSARA(BASE(IP11))
      CALL RLSARA(BASE(IP10))
      CALL RLSARA(BASE(IP01))
      CALL RLSARA(BASE(IP00))
      CALL RLSARA(BASE(IPA11))
      CALL RLSARA(BASE(IPA10))
      CALL RLSARA(BASE(IPA01))
      CALL RLSARA(BASE(IPA00))
      CALL RLSARA(BASE(IYDE))
      CALL RLSARA(BASE(IY))
      CALL RLSARA(BASE(IPRECW))
      CALL RLSARA(BASE(IPREC))
      CALL RLSARA(BASE(IETA))
      CALL RLSARA(BASE(IDELSR))
*
* UPDATE THE TIME.
      TMACRO=TMACRO+DTMACR
*
      DO 260 IGR =1,NGRP
       DO 160 I=1,NUN
         EVECT(I,IGR)=SHAPE(I,IGR)
 160   CONTINUE
 260  CONTINUE
*
      IF(IDIV.EQ.NDIV) THEN
*        UPDATE THE CROSS SECTIONS.
         IF(IPLISP.NE.0) THEN
           IF(LCM2) THEN
             CALL LCMOP(IPMAC2,'COPY2',0,1,0)
           ELSE
             CALL LCMOP(IPMAC2,'COPY2',0,2,0)
           ENDIF
         CALL LCMEQU(IPMAC0,IPMAC2)
         CALL SETARA(BASE,MAXMIX*NGRP,ITOTAL)
         CALL SETARA(BASE,MAXMIX*NGRP,IZNUG)
         CALL SETARA(BASE,MAXMIX*NGRP,ISIGF)
         CALL SETARA(BASE,MAXMIX*NGRP,ICHI)
         CALL SETARA(BASE,MAXMIX*NGRP*NDG,ICHDL)
         CALL SETARA(BASE,MAXMIX*NGRP,IOVERV)
         CALL SETARA(BASE,MAXMIX*NGRP,IDIFFX)
         CALL SETARA(BASE,MAXMIX*NGRP,IDIFFY)
         CALL SETARA(BASE,MAXMIX*NGRP,IDIFFZ)
         CALL SETARA(BASE,MAXMIX*NGRP,IH)
         CALL SETARA(BASE,MAXMIX*NGRP,IS)
         CALL SETARA(BASE,MAXMIX*NL*NGRP,IIJJ)
         CALL SETARA(BASE,MAXMIX*NL*NGRP,INJJ)
         CALL SETARA(BASE,MAXMIX*NL*NGRP,IIJJP)
         CALL SETARA(BASE,MAXMIX*NL*NGRP,INJJP)
         CALL SETARA(BASE,MAXMIX*NL*NGRP*NGRP,ISCAT)
         CALL SETARA(BASE,MAXMIX*NGRP,IWORK)
         CALL SETARA(BASE,MAXMIX,IIPOS)
         CALL IQSADD (IPMACP,IPMAC2,IPMAC,MAXMIX,NGRP,NDG,NL,
     1   BASE(ITOTAL),BASE(IZNUG),BASE(ISIGF),BASE(ICHI),BASE(ICHDL),
     x   BASE(IOVERV),BASE(IDIFFX),BASE(IDIFFY),BASE(IDIFFZ),BASE(IH),
     x   BASE(IS),BASE(IIJJ),BASE(INJJ),BASE(ISCAT),BASE(IWORK),
     x   BASE(IIPOS),BASE(IIJJP),BASE(INJJP),.FALSE.,1.0)
         CALL LCMCL(IPMAC2,2)
         CALL RLSARA(BASE(IIPOS))
         CALL RLSARA(BASE(IWORK))
         CALL RLSARA(BASE(ISCAT))
         CALL RLSARA(BASE(INJJP))
         CALL RLSARA(BASE(IIJJP))
         CALL RLSARA(BASE(INJJ))
         CALL RLSARA(BASE(IIJJ))
         CALL RLSARA(BASE(IS))
         CALL RLSARA(BASE(IH))
         CALL RLSARA(BASE(IDIFFZ))
         CALL RLSARA(BASE(IDIFFY))
         CALL RLSARA(BASE(IDIFFX))
         CALL RLSARA(BASE(IOVERV))
         CALL RLSARA(BASE(ICHDL))
         CALL RLSARA(BASE(ICHI))
         CALL RLSARA(BASE(IZNUG))
         CALL RLSARA(BASE(ISIGF))
         CALL RLSARA(BASE(ITOTAL))
         ENDIF
*
*     DATA STORAGE
*
         DO 250 IGR =1,NGRP
           DO 252 I=1,NUN
             EVECT(I,IGR)=SHAPE(I,IGR)*AMPGA(IGR)
  252      CONTINUE
  250    CONTINUE
         CALL LCMPUT(IPFLUP,'TMACRO',1,2,TMACRO)
         DO 270 IGR =1,NGRP
           WRITE (CGRPNM,'(5HSHAPE,I3)') IGR
           CALL LCMPUT(IPFLUP,CGRPNM,NUN,2,SHAPE(1,IGR))
           WRITE (CGRPNM,'(4HFLUX,I3)') IGR
           CALL LCMPUT(IPFLUP,CGRPNM,NUN,2,EVECT(1,IGR))
 270     CONTINUE
         CALL LCMPUT(IPFLUP,'AMPGA',2*NGRP,4,AMPGA)
         POWER = REAL(ZPOW)
         CALL LCMPUT(IPFLUP,'POWER',1,2,POWER)
*
      ELSE
        GO TO 140
      ENDIF
      IF(IPLISP.NE.0) CALL LCMSIX(IPMACP,' ',2)
      IF(IPLISP.NE.0) CALL LCMCL(IPLISP,2)
      GO TO 40
*
  170 CALL LCMPUT(IPFLUP,'BETA',NDG,2,BASE(IBETA))
      CALL LCMPUT(IPFLUP,'DECAY',NDG,2,BASE(IDECR))
      CALL RLSARA(BASE(IBETA))
      CALL RLSARA(BASE(IDECR))
      IF(.NOT.VELO) THEN
        CALL RLSARA(BASE(IVM))
      ENDIF
*
      IF(IMPX.GT.1) CALL LCMLIB(IPFLUP)
      CALL LCMCL(IPMAC,2)
      CALL LCMCL(IPLISS,2)
      CALL LCMCL(IPLIS0,2)
      RETURN
*
 1000 FORMAT(/' INITIAL POWER=',E12.5)
 2000 FORMAT('1',T30,'** BEGINNING OF TRANSIENT AT TIME : ',F10.5,' **')
      END
