*DECK IQSNCA
      SUBROUTINE IQSNCA (IPTRK,IPSYSP,IPFLUP,IPSYSS,NUN,ADECT,EVECT
     1 ,FKEFF,NDG,BETA,DECRAT,VM,DTMACR,ITRANA,ITRANB,
     2 EPS2,EPSRHO,ERAMP,ERCON,RHOPOU,LIMM,MAXX0,ICL1,ICL2,IMPPK,
     3 IMPR,NADI,Y,YDE,DVECT,DELSRC,PREC,ETA,TMACRO,AMPGA,PKIN,
     4 QSM,RENORM,RM,DEL,HI,METHOD,VELO,V,NGRP,AMPP,AT,ATOUT,
     x AMP,GAMMA,SIGMA,CONTOB,CONSN,AMPLD,PA00,PA01,PA10,PA11,
     5 P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM,PRECW,LBI,
     x LTETA,TETA,NT,RAPQ)
*-----------------------------------------------------------------------
*
* THIS SUBROUTINE COMPUTE THE SOLUTION OF THE IMPROVED AND GENERALIZED
* QUASISTATIC (IQS AND GQS) EQUATIONS.
* KAPS-RENTROP VERSION.
* NOTE: THIS IS THE MULTIGROUP VERSION.
*
* INPUT PARAMETERS:
*  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPSYSP : L_SYSTEM POINTER TO DELTA SYSTEM MATRICES.
*  IPFLUP : L_IQS POINTER TO THE IQS SOLUTION.
*  IPSYSS : L_SYSTEM POINTER TO UNPERTURBED SYSTEM MATRICES.
*  NUN    : TOTAL NUMBER OF UNKNOWNS PER GROUP.
*  NDG    : NUMBER OF DELAYED NEUTRON GROUPS.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  FKEFF  : EFFECTIVE MULTIPLICATION FACTOR.
*  BETA   : DELAYED NEUTRON FRACTION FOR EACH GROUP;
*         : DIMENSION BETA(NDG).
*  DECRAT : DELAYED NEUTRON DECAY RATE FOR EACH GROUP;
*         : DIMENSION DECRAT(NDG).
*  VM     : NEUTRON VELOCITY; DIMENSION VM(NGRP).
*  CHID   : DELAYED NEUTRON FISSION SPECTRUM FOR EACH ISOTOPE;
*         : DIMENSION CHID(NDG,NGRP).
*  DTMACR : MACRO TIME STEP.
*  EVECT  : DIMENSION EVECT1(NUN.NGRP)
*  ADECT  : DIMENSION ADECT1(NUN,NGRP)
*  RM     : UNITARY MATRIX; DIMENSION RM(NUN).
*  V      : RECIPROCAL VELOCITY MATRIX; DIMENSION V(NUN,NGRP).
*  EPS2   : CONVERGENCE CRITERION FOR THE SHAPE FUNCTION.
*  EPSRHO : CONVERGENCE CRITERION FOR THE REACTIVITY.
*  ERAMP  : CONVERGENCE CRITERION FOR THE AMPLITUDE.
*  RHOPOU : CONVERGENCE CRITERION FOR (DELT.RHOU)*2/RHOU.
*  LIMM   : MAXIMUM NUMBER OF REACTIVITY ITERATION.
*  MAXX0  : MAXIMUM NUMBER OF OUTER SHAPE FUNCTION ITERATION.
*  ITRANA : TYPE OF PERTURBATION FOR A-SYSTEM MATRICES.
*  ITRANB : TYPE OF PERTURBATION FOR B-SYSTEM MATRICES.
*  ICL1   : NUMBER OF FREE OUTER ITERATION IN SVAT.
*  ICL2   : NUMBER OF ACCELERATED OUTER ITERATION IN SVAT.
*  IMPPK  : PRINTING INDEX.
*  IMPR   : PRINTING INDEX.
*  NADI   : NUMBER OF ADI INNER ITERATION PER OUTER ITERATION.
*  AMPGA  : INITIAL VALUE OF THE APMLITUDE; DIMENSION AMPGA(NGRP).
*  DEL    : KAPS-RENTROP TOLERANCE VALUE.
*  HI     : KAPS-RENTROP FIRST STEP SIZE.
*  NT     : KAPS-RENTROP NUMBER OF EQUATIONS.
*  TETA   : THETA VALUE.
*  METHOD : KAPS-RENTROP METHOD (GRKT OR GRKA).
*  LTETA  : LOGICAL VARIABLE FOR THETA METHOD.
*  VELO   : LOGICAL VARIABLE FOR VELOCITY.
*  PKIN   : LOGICAL VARIABLE FOR POINT KINETICS PROBLEM.
*  QSM    : LOGICAL VARIABLE FOR QUASISTATIC SOLUTION.
*  RENORM : LOGICAL VARIABLE FOR THE CONSTRAINT.
*  LBI    : LOGICAL VARIABLE FOR NEUTRON FISSION SPECTRUM.
**
* OUTPUT PARAMETER:
*  IPFLUP : L_IQS POINTER TO THE IQS SOLUTION.
*  TMACRO : TIME AT WICH THE SOLUTION IS COMPUTED.
**
* SCRATCH STORAGE
*  PREC   : DIMENSION PREC(NDG*NUN*NGRP)
*  ETA    : DIMENSION ETA(2*NUN*NGRP)
*  Y      : DIMENSION Y(NT)
*  YDE    : HOLDS START OF STEP VALUES FOR POINT KINETICS VARIABLES
*         : DIMENSION YDE(NT)
*  DVECT  : DIMENSION DVECT(NUN,NGRP)
*  DELSRC : DIMENSION DELSRC(NUN,NGRP)
*  RAPQ   : DIMENSION RAPQ(NDG)
**
* CONSN  : CONSTRAINT VALUE FOR END OF INTERVAL AT THE NTH ITERATION
* CONTOB : INITIAL VALUE OF THE CONSTRAINT
* ATOUT  : CONTRAINT VALUE LEAVING IQSQSM OR IQSGQS
***
* MODIFIED BY M.T. SISSAOUI ; 28/07/1994
************************************** AUTHOR: A. MONIER ; 1991 **
*
*
      DIMENSION EVECT(NUN,NGRP),ADECT(NUN,NGRP),
     x DVECT(NUN,NGRP),PREC(NUN,NDG,NGRP),BETA(NDG),RAPQ(NDG),
     x DECRAT(NDG),RM(*),AMPP(NGRP),AT(NGRP),PRECW(NUN,NDG),
     x ATOUT(NGRP),VM(NGRP),V(NUN,NGRP),DELSRC(NUN,NGRP)
C
C  GENERALIZED QUASISTATIC VARIABLES
      LOGICAL PKIN,QSM,RENORM,FIXCON,LCM,LCMF,VELO,LTETA,LBI
      CHARACTER*12 NAME,NAMLCM,NAMMY,NAMB,NAMA
      DOUBLE PRECISION PA00(NGRP,NGRP),PA01(NGRP,NGRP),
     1 PA10(NGRP,NGRP),PA11(NGRP,NGRP),
     1 PB00(NGRP,NGRP),PB01(NGRP,NGRP),PB10(NGRP,NGRP),
     1 PB11(NGRP,NGRP),
     1  P00(NGRP,NGRP), P01(NGRP,NGRP), P10(NGRP,NGRP),
     1  P11(NGRP,NGRP)
      DOUBLE PRECISION AMP(NGRP),AMPGA(NGRP),Y(NT),
     1 YDE(NT),ETA(NDG,NGRP)
      DOUBLE PRECISION GAMMA(NGRP,NDG),TERM(NGRP),SIGMA(NGRP,NDG),
     1 CONTOB(NGRP),CONSN(NGRP),AMPLD(NGRP)
       DOUBLE PRECISION DTMACL,DMCM1,R0CPC(NGRP),DRHO
C KAPS-RENTROP  VARIABLES
      CHARACTER*5 METHOD
      DOUBLE PRECISION HI,H
C  GQS VARIABLES END
      DOUBLE PRECISION DTEMP0,DTEMP1,DTEMP2,DTEMP3,DTEMP4,BETATD,TEMP1
      COMMON BASE(1)
*
      COMMON/CPCOM/DTMACL,DMCM1,IDECR,IBETA,NDGL,ITRAN,BETATD,DRHO
*     OPTION COMMON BLOCK
*
      FIXCON = .FALSE.
      ICALL = 0
      DO 222 IGR=1,NGRP
      TERM(IGR) = 0.D0
 222  CONTINUE
      IOFF1= 2 + NDG*1
      IOFF2= 2 + NDG*2
      IOFF3= 2 + NDG*3
      IOFF4= 2 + NDG*4
      IOFF5= 2 + NDG*5
      IOFF6= 2 + NDG*6
C
      BETAT = 0.0
      DO 10 I = 1, NDG
10    BETAT=BETAT + BETA(I)
      BETATD=DBLE(BETAT)
      NAME=' '
      CALL LCMNXT(IPSYSS,NAME,NAMLCM,NAMMY,LCM)
      NAME=' '
      CALL LCMNXT(IPFLUP,NAME,NAMLCM,NAMMY,LCMF)
C
C INITIALISATION DES PRECURSEURS
      IF(TMACRO.EQ.0.0D0) THEN
       DO 922 IGR=1,NGRP
       DO 922 I=1,NDG
       DO 922 J=1,NUN
 922   PREC(J,I,IGR)=0.0
      ENDIF
*
      DO 15 IGR= 1,NGRP
         IF(TMACRO.EQ.0.0D0) THEN
*
           IF(LBI) THEN
* PRECURSEUR B DANS MACROLIB => DANS SYSTEM0
             DO 970 II=1,NDG
              DO 976 JGR=1,NGRP
               WRITE(NAMB,'(1HB,I3,I3,I3)') II,IGR,JGR
               CALL LCMLEN(IPSYSS,NAMB,LENGB,ITYLCM)
               IF(LCM) THEN
*         IN-CORE STORAGE.
                IF(LENGB.GT.0) IB0=LCMIOF(IPSYSS,NAMB)
               ELSE
*         XSM STORAGE.
                IF(LENGB.GT.0) CALL SETARA(BASE,LENGB,IB0)
                IF(LENGB.GT.0) CALL LCMGET(IPSYSS,NAMB,BASE(IB0))
               ENDIF
               IF(LENGB.GT.0) THEN
                 DO 911 I=1,NUN
 911  PREC(I,II,IGR)= PREC(I,II,IGR)+BASE(IB0-1+I)*EVECT(I,JGR)/FKEFF
                 IF(.NOT.LCM) THEN
                  IF(LENGB.GT.0) CALL RLSARA(BASE(IB0))
                 ENDIF
               ENDIF
 976          CONTINUE
*
              TEMP1=AMPGA(1)*BETA(II)/DECRAT(II)
              DO 921 J=1,NUN
 921            PREC(J,II,IGR)=TEMP1*PREC(J,II,IGR)
 970         CONTINUE
*
           ELSE
* PAS DE PRECURSEUR B DANS MACROLIB => DANS SYSTEM0
*
             CALL SETARA(BASE,NUN,IWRK)
             DO 255 I=1,NUN
              BASE(IWRK-1+I)=0.
 255         CONTINUE
*
             DO 16 JGR=1,NGRP
               WRITE(NAMB,'(1HB,I3,I3)') IGR,JGR
               CALL LCMLEN(IPSYSS,NAMB,LENGB,ITYLCM)
               IF(LCM) THEN
*            IN-CORE STORAGE.
                IF(LENGB.GT.0) IB0=LCMIOF(IPSYSS,NAMB)
               ELSE
*            XSM STORAGE.
                IF(LENGB.GT.0) CALL SETARA(BASE,LENGB,IB0)
                IF(LENGB.GT.0) CALL LCMGET(IPSYSS,NAMB,BASE(IB0))
               ENDIF
               IF(LENGB.GT.0) THEN
                 DO 11 I=1,NUN
 11   BASE(IWRK-1+I)= BASE(IWRK-1+I)+BASE(IB0-1+I)*EVECT(I,JGR)/FKEFF
                 IF(.NOT.LCM) THEN
                   IF(LENGB.GT.0) CALL RLSARA(BASE(IB0))
                 ENDIF
               ENDIF
 16          CONTINUE
*
             DO 923 I=1,NDG
               TEMP1=AMPGA(1)*BETA(I)/DECRAT(I)
               DO 21 J=1,NUN
 21               PREC(J,I,IGR)=TEMP1*BASE(IWRK-1+J)
 923         CONTINUE
             CALL RLSARA(BASE(IWRK))
*
           ENDIF
* LBI
         ELSE
C          RECUPERATION DES PRECURSEURS
           DO 261 I=1,NDG
             WRITE (NAMB,'(4HPREC,I3,I3)') IGR,I
             CALL LCMLEN(IPFLUP,NAMB,LENGB,ITYLCM)
             IF(LENGB.GT.0) THEN
                CALL LCMGET(IPFLUP,NAMB,PREC(1,I,IGR))
             ENDIF
 261       CONTINUE
         ENDIF
         IF(DTMACR.EQ.0.0D0) THEN
           DO 262 I=1,NDG
              WRITE (NAMB,'(4HPREC,I3,I3)') IGR,I
              CALL LCMPUT(IPFLUP,NAMB,NUN,2,PREC(1,I,IGR))
 262       CONTINUE
           RETURN
         ENDIF
  15  CONTINUE
*
*
* APPLY THE CONSTRAINT.
      DTEMP1=0.0D0
      DTEMP2=0.0D0
      DO 301 IGR=1,NGRP
       DTEMP1=0.0D0
       IF(VELO) THEN
         DO 201 I=1,NUN
 201     DTEMP1=DTEMP1+ADECT(I,IGR)*V(I,IGR)*EVECT(I,IGR)
         CONTOB(IGR)=DTEMP1
         R0CPC(IGR) =1.0D0/DTEMP1
       ELSE
         DO 20 I=1,NUN
 20       DTEMP1=DTEMP1+ADECT(I,IGR)*RM(I)*EVECT(I,IGR)/VM(IGR)
         CONTOB(IGR)=DTEMP1
         R0CPC(IGR) =1.0D0/DTEMP1
       ENDIF
 301  CONTINUE
      IF(QSM) THEN
        DTEMP2=0.0D0
        DO 302 IGR=1,NGRP
         DTEMP2=DTEMP2 + CONTOB(IGR)
         R0CPC(IGR)=1.D0
 302    CONTINUE
        R0CPC(1)=1.D0/DTEMP2
      ENDIF
C PARAMETRES RENDUS LOCAUX POUR IQSFI
      ITRAN = ITRANA
      DTMACL= DTMACR
      NDGL  = NDG
      DMCM1 = 1.0/DTMACR
*
      CALL SETARA(BASE,NDG,IBETA)
      CALL SETARA(BASE,NDG,IDECR)
      DO 23 I=1,NDG
        BASE(IBETA-1+I)=BETA(I)
        BASE(IDECR-1+I)=DECRAT(I)
23    CONTINUE
*
C INTEGRATION DES PRECURSEURS
      DO 361 IGR=1,NGRP
        DO 362 I=1,NDG
          ETA(I,IGR)=0.0D0
 362    CONTINUE
 361  CONTINUE
      DO 291 I=1,NDG
        DTEMP1=0.0D0
        DO 310 IGR=1,NGRP
         DO 317 J=1,NUN
          ETA(I,IGR)=ETA(I,IGR)+PREC(J,I,IGR)*ADECT(J,IGR)
          DTEMP1=DTEMP1+PREC(J,I,IGR)*ADECT(J,IGR)
317      CONTINUE
310     CONTINUE
*     FOR BOTH QSM AND QGS
        IF(QSM) THEN
          DTEMP2=0.0D0
          DO 320 IGR=1,NGRP
            DTEMP2=DTEMP2 + CONTOB(IGR)
 320      CONTINUE
          ETA(I,1)=DTEMP1/DTEMP2
        ELSE
          DO 329 IGR=1,NGRP
            ETA(I,IGR)=ETA(I,IGR)/CONTOB(IGR)
 329      CONTINUE
        ENDIF
291   CONTINUE
C INFO DEBUT DE TRANSITOIRE
      IF(LTETA) THEN
        IF(QSM) THEN
          YDE(1)=AMPGA(1)
          YDE(2)=1.0D0
          L = 0
          DO 94 I=3,NDG+2
            L = L+1
94          YDE(I)=ETA(L,1)
        ELSE
          DO 95 IGR=1,NGRP
            YDE(IGR)=AMPGA(IGR)
            DO 96 I=1,NDG
             INP=NGRP+(IGR-1)*NDG
             YDE(I+INP)=ETA(I,IGR)
 96         CONTINUE
 95       CONTINUE
        ENDIF
      ELSE
        IF(QSM) THEN
          YDE(1)=AMPGA(1)
          YDE(2)=1.0D0
          L = 0
          DO 794 I=3,NDG+2
            L = L+1
 794        YDE(I)=ETA(L,1)
          INI=3+NDG
          DO 726 I=INI,NT
 726        YDE(I)=0.0D0
        ELSE
          DO 795 IGR=1,NGRP
            YDE(IGR)=AMPGA(IGR)
            DO 796 I=1,NDG
              INP=NGRP+(IGR-1)*NDG
              YDE(I+INP)=ETA(I,IGR)
 796        CONTINUE
 795      CONTINUE
          INDG=NGRP*(1+NDG)+1
          DO 26 I=INDG,NT
 26        YDE(I)=0.0D0
        ENDIF
      ENDIF
C RECUPERATION DES PRODUITS LINEAIRES
      IF(ITRANA.NE.0) THEN
        DO 190 IGR= 1,NGRP
          DO 191 JGR=1,NGRP
            WRITE(NAMA,'(4HPV0A,I3,I3)') IGR,JGR
            CALL LCMLEN(IPFLUP,NAMA,LENGA,ITYLCM)
            IF(LCMF) THEN
*        IN-CORE STORAGE.
              IF(LENGA.GT.0) IV0A=LCMIOF(IPFLUP,NAMA)
            ELSE
*        XSM STORAGE.
              IF(LENGA.GT.0) CALL SETARA(BASE,LENGA,IV0A)
              IF(LENGA.GT.0) CALL LCMGET(IPFLUP,NAMA,BASE(IV0A))
            ENDIF
            DTEMP0 = 0.0D0
            IF(LENGA.GT.0) THEN
              DO 101 I=1,NUN
                DTEMP0 = DTEMP0 + BASE(IV0A-1+I)*EVECT(I,JGR)
 101          CONTINUE
              IF(.NOT.LCMF) THEN
                IF(LENGA.GT.0) CALL RLSARA(BASE(IV0A))
              ENDIF
            ENDIF
            IF(IGR.EQ.JGR) THEN
              DTEMP0 = -DTEMP0
            ENDIF
            PA00(IGR,JGR) = DTEMP0
 191      CONTINUE
 190    CONTINUE
      ELSE
        DO 180 IGR= 1,NGRP
         DO 181 JGR=1,NGRP
           PA00(IGR,JGR) =0.0D0
 181     CONTINUE
 180    CONTINUE
      ENDIF
      IF(ITRANA.NE.0) THEN
        DO 192 IGR= 1,NGRP
          DO 193 JGR=1,NGRP
            WRITE(NAMA,'(4HPV1A,I3,I3)') IGR,JGR
            CALL LCMLEN(IPFLUP,NAMA,LENGA,ITYLCM)
            IF(LCMF) THEN
*        IN-CORE STORAGE.
              IF(LENGA.GT.0) IV1A=LCMIOF(IPFLUP,NAMA)
            ELSE
*        XSM STORAGE.
              IF(LENGA.GT.0) CALL SETARA(BASE,LENGA,IV1A)
              IF(LENGA.GT.0) CALL LCMGET(IPFLUP,NAMA,BASE(IV1A))
            ENDIF
            DTEMP0 = 0.0D0
            IF(LENGA.GT.0) THEN
              DO 102 I=1,NUN
                DTEMP0 = DTEMP0 + BASE(IV1A-1+I)*EVECT(I,JGR)
102           CONTINUE
              IF(.NOT.LCMF) THEN
                IF(LENGA.GT.0) CALL RLSARA(BASE(IV1A))
              ENDIF
            ENDIF
            IF(IGR.EQ.JGR) THEN
              DTEMP0 = -DTEMP0
            ENDIF
            PA10(IGR,JGR) =DTEMP0
 193      CONTINUE
 192    CONTINUE
      ELSE
        DO 182 IGR=1,NGRP
          DO 183 JGR=1,NGRP
            PA10(IGR,JGR) = 0.0D0
 183      CONTINUE
 182    CONTINUE
      ENDIF
      IF(ITRANB.NE.0) THEN
         DO 194 IGR= 1,NGRP
           DO 195 JGR=1,NGRP
             WRITE(NAMB,'(4HPV0B,I3,I3)') IGR,JGR
             CALL LCMLEN(IPFLUP,NAMB,LENGB,ITYLCM)
             IF(LCMF) THEN
*        IN-CORE STORAGE.
               IF(LENGB.GT.0) IV0B=LCMIOF(IPFLUP,NAMB)
             ELSE
*        XSM STORAGE.
               IF(LENGB.GT.0) CALL SETARA(BASE,LENGB,IV0B)
               IF(LENGB.GT.0) CALL LCMGET(IPFLUP,NAMB,BASE(IV0B))
             ENDIF
             DTEMP1 = 0.0D0
             IF(LENGB.GT.0) THEN
               DO 103 I=1,NUN
                 DTEMP1 = DTEMP1 + BASE(IV0B-1+I)*EVECT(I,JGR)
 103           CONTINUE
               IF(.NOT.LCMF) THEN
                 IF(LENGB.GT.0) CALL RLSARA(BASE(IV0B))
               ENDIF
             ENDIF
             PB00(IGR,JGR) = DTEMP1
 195       CONTINUE
 194     CONTINUE
      ELSE
         DO 184 IGR=1,NGRP
           DO 185 JGR=1,NGRP
             PB00(IGR,JGR) = 0.0D0
 185       CONTINUE
 184     CONTINUE
      ENDIF
      IF(ITRANB.NE.0)THEN
        DO 196 IGR= 1,NGRP
          DO 197 JGR=1,NGRP
            WRITE(NAMB,'(4HPV1B,I3,I3)') IGR,JGR
            CALL LCMLEN(IPFLUP,NAMB,LENGB,ITYLCM)
            IF(LCMF) THEN
*        IN-CORE STORAGE.
              IF(LENGB.GT.0) IV1B=LCMIOF(IPFLUP,NAMB)
            ELSE
*        XSM STORAGE.
              IF(LENGB.GT.0) CALL SETARA(BASE,LENGB,IV1B)
              IF(LENGB.GT.0) CALL LCMGET(IPFLUP,NAMB,BASE(IV1B))
            ENDIF
            DTEMP1 = 0.0D0
            IF(LENGB.GT.0) THEN
               DO 104 I=1,NUN
                 DTEMP1 = DTEMP1 + BASE(IV1B-1+I)*EVECT(I,JGR)
104            CONTINUE
               IF(.NOT.LCMF) THEN
                 IF(LENGB.GT.0) CALL RLSARA(BASE(IV1B))
               ENDIF
            ENDIF
            PB10(IGR,JGR) = DTEMP1
 197      CONTINUE
 196    CONTINUE
      ELSE
        DO 186 IGR=1,NGRP
          DO 187 JGR=1,NGRP
            PB10(IGR,JGR) = 0.0D0
 187      CONTINUE
 186    CONTINUE
      ENDIF
C CALCUL DE CERTAINS PARAMETRES CONSTANTS DE CP
      DO 460 JGR=1,NGRP
        DO 465 IGR=1,NGRP
          P00(IGR,JGR) = PA00(IGR,JGR)
          P10(IGR,JGR) = PA10(IGR,JGR)
          P00(IGR,JGR)=P00(IGR,JGR)+(1.D0-BETATD)*PB00(IGR,JGR)
          P10(IGR,JGR)=P10(IGR,JGR)+(1.D0-BETATD)*PB10(IGR,JGR)
465     CONTINUE
460   CONTINUE
      IF(TMACRO.EQ.0.0D0) THEN
        DRHO = 0.0D0
        DO 461 JGR=1,NGRP
          DO 466 IGR=1,NGRP
            DRHO = DRHO + PA00(IGR,JGR) + PB00(IGR,JGR)
466       CONTINUE
461     CONTINUE
        CALL LCMPUT(IPFLUP,'DELTA-RHO',1,2,DRHO)
      ELSE
        CALL LCMGET(IPFLUP,'DELTA-RHO',DRHO)
      ENDIF
C
C REACTIVITE CINETIQUE PONCTUELLE
C INITIALISATION DU FLUX DYNAMIQUE
C INITIALISATION DE STIFFZ
      MF=22
*      NT=7*NDG+2
      NJAC=2*(NT*(NT+11)+1)
C
C                     DEBUT DE LA PROCEDURE ITERATIVE
C                     *******************************
*  SET VARIABLES FOR ITERATIVE LOOP
      ITRAT=0
      REACT=0.0
      DO 333 IGR=1,NGRP
        AMP(IGR)=AMPGA(IGR)
 333  CONTINUE
      IF(QSM) THEN
        DTEMP2=0.0D0
        DO 330 IGR=1,NGRP
          DTEMP2=DTEMP2 + CONTOB(IGR)
          AT(IGR)= 1.D0
 330    CONTINUE
        AT(1) = DTEMP2
      ELSE
        DO 444 IGR=1,NGRP
          AT(IGR) = CONTOB(IGR)
 444    CONTINUE
      ENDIF
      DO 331 IGR=1,NGRP
        DO 30 I=1,NUN
          DVECT(I,IGR)=EVECT(I,IGR)
  30    CONTINUE
 331  CONTINUE
      IF((IMPPK.EQ.1).OR.(IMPR.GT.2)) WRITE(6,410)
C
C     ITERATIVE LOOP STARTS HERE
205   CONTINUE
      ITRAT=ITRAT+1
      DO 555 IGR=1,NGRP
        AMPP(IGR)=AMP(IGR)
 555  CONTINUE
      REACTP=REACT
C     ETAPE 5: CALCUL DES PARAMETRES ITERATIFS DE LA CP
      IF(IMPPK.EQ.1 .OR. IMPR.GT.0) WRITE(6,360) ITRAT
      IF(ITRANA.NE.0) THEN
        DO 399 IGR= 1,NGRP
          DO 391 JGR=1,NGRP
            WRITE(NAMA,'(4HPV0A,I3,I3)') IGR,JGR
            CALL LCMLEN(IPFLUP,NAMA,LENGA,ITYLCM)
            IF(LCMF) THEN
*        IN-CORE STORAGE.
              IF(LENGA.GT.0) IV0AS=LCMIOF(IPFLUP,NAMA)
            ELSE
*        XSM STORAGE.
              IF(LENGA.GT.0) CALL SETARA(BASE,LENGA,IV0AS)
              IF(LENGA.GT.0) CALL LCMGET(IPFLUP,NAMA,BASE(IV0AS))
            ENDIF
            DTEMP0 = 0.0D0
            IF(LENGA.GT.0) THEN
              DO 70  I=1,NUN
                DTEMP0 = DTEMP0 + BASE(IV0AS-1+I)*DVECT(I,JGR)
 70           CONTINUE
              IF(.NOT.LCMF) THEN
                IF(LENGA.GT.0) CALL RLSARA(BASE(IV0AS))
              ENDIF
            ENDIF
            IF(IGR.EQ.JGR) THEN
              DTEMP0 = -DTEMP0
            ENDIF
            PA01(IGR,JGR) =DTEMP0
 391      CONTINUE
 399    CONTINUE
      ELSE
        DO 389 IGR= 1,NGRP
          DO 381 JGR=1,NGRP
            PA01(IGR,JGR) =0.0D0
 381      CONTINUE
 389    CONTINUE
      ENDIF
      IF(ITRANA.NE.0) THEN
        DO 392 IGR= 1,NGRP
          DO 393 JGR=1,NGRP
            WRITE(NAMA,'(4HPV1A,I3,I3)') IGR,JGR
            CALL LCMLEN(IPFLUP,NAMA,LENGA,ITYLCM)
            IF(LCMF) THEN
*        IN-CORE STORAGE.
              IF(LENGA.GT.0) IV1AS=LCMIOF(IPFLUP,NAMA)
            ELSE
*        XSM STORAGE.
              IF(LENGA.GT.0) CALL SETARA(BASE,LENGA,IV1AS)
              IF(LENGA.GT.0) CALL LCMGET(IPFLUP,NAMA,BASE(IV1AS))
            ENDIF
            DTEMP0 = 0.0D0
            IF(LENGA.GT.0) THEN
              DO 80  I=1,NUN
                DTEMP0 = DTEMP0 + BASE(IV1AS-1+I)*DVECT(I,JGR)
 80           CONTINUE
              IF(.NOT.LCMF) THEN
                IF(LENGA.GT.0) CALL RLSARA(BASE(IV1AS))
              ENDIF
            ENDIF
            IF(IGR.EQ.JGR) THEN
               DTEMP0 = -DTEMP0
            ENDIF
            PA11(IGR,JGR) =DTEMP0
  393     CONTINUE
  392   CONTINUE
      ELSE
        DO 382 IGR= 1,NGRP
          DO 383 JGR=1,NGRP
            PA11(IGR,JGR) =0.0D0
 383      CONTINUE
 382    CONTINUE
      ENDIF
C
      IF(ITRANB.NE.0) THEN
        DO 394 IGR= 1,NGRP
          DO 395 JGR=1,NGRP
            WRITE(NAMB,'(4HPV0B,I3,I3)') IGR,JGR
            CALL LCMLEN(IPFLUP,NAMB,LENGB,ITYLCM)
            IF(LCMF) THEN
*         IN-CORE STORAGE.
              IF(LENGB.GT.0) IV0BS=LCMIOF(IPFLUP,NAMB)
            ELSE
*        XSM STORAGE.
              IF(LENGB.GT.0) CALL SETARA(BASE,LENGB,IV0BS)
              IF(LENGB.GT.0) CALL LCMGET(IPFLUP,NAMB,BASE(IV0BS))
            ENDIF
            DTEMP1=0.0D0
            IF(LENGB.GT.0) THEN
              DO 110 I=1,NUN
                DTEMP1=DTEMP1+BASE(IV0BS-1+I)*DVECT(I,JGR)
 110          CONTINUE
              IF(.NOT.LCMF) THEN
                 IF(LENGB.GT.0) CALL RLSARA(BASE(IV0BS))
              ENDIF
            ENDIF
            PB01(IGR,JGR) = DTEMP1
 395      CONTINUE
 394    CONTINUE
      ELSE
        DO 384 IGR= 1,NGRP
          DO 385 JGR=1,NGRP
            PB01(IGR,JGR) =0.0D0
 385      CONTINUE
 384    CONTINUE
      ENDIF
      IF(ITRANB.NE.0) THEN
         DO 396 IGR= 1,NGRP
           DO 397 JGR=1,NGRP
             WRITE(NAMB,'(4HPV1B,I3,I3)') IGR,JGR
             CALL LCMLEN(IPFLUP,NAMB,LENGB,ITYLCM)
             IF(LCMF) THEN
*         IN-CORE STORAGE.
               IF(LENGB.GT.0) IV1BS=LCMIOF(IPFLUP,NAMB)
             ELSE
*        XSM STORAGE.
               IF(LENGB.GT.0) CALL SETARA(BASE,LENGB,IV1BS)
               IF(LENGB.GT.0) CALL LCMGET(IPFLUP,NAMB,BASE(IV1BS))
             ENDIF
             DTEMP1=0.0D0
             IF(LENGB.GT.0) THEN
               DO 120 I=1,NUN
                 DTEMP1=DTEMP1+BASE(IV1BS-1+I)*DVECT(I,JGR)
 120           CONTINUE
               IF(.NOT.LCMF) THEN
                 IF(LENGB.GT.0) CALL RLSARA(BASE(IV1BS))
               ENDIF
             ENDIF
             PB11(IGR,JGR) = DTEMP1
  397     CONTINUE
  396   CONTINUE
      ELSE
        DO 386 IGR= 1,NGRP
          DO 387 JGR=1,NGRP
            PB11(IGR,JGR) =0.0D0
 387      CONTINUE
 386    CONTINUE
      ENDIF
C     PARAMETRES ITERATIFS RENDUS LOCAUX A IQSCPC
      DO 470 J=1,NGRP
        DO 475 I=1,NGRP
          P11(I,J) = PA11(I,J)
          P01(I,J) = PA01(I,J)
          P11(I,J)=P11(I,J)+(1.D0-BETATD)*PB11(I,J)
          P01(I,J)=P01(I,J)+(1.D0-BETATD)*PB01(I,J)
 475    CONTINUE
 470  CONTINUE
C     COMPUTE NEW CONSTRAINT
      DTEMP2=0.0D0
      DO 501 IGR=1,NGRP
        DTEMP1=0.0D0
        IF(VELO) THEN
          DO 481 I=1,NUN
            DTEMP1=DTEMP1+ADECT(I,IGR)*V(I,IGR)*DVECT(I,IGR)
 481      CONTINUE
          CONSN(IGR) = DTEMP1
          DTEMP2=DTEMP2 + CONSN(IGR)
        ELSE
          DO 480 I=1,NUN
            DTEMP1=DTEMP1+ADECT(I,IGR)*RM(I)*DVECT(I,IGR)/VM(IGR)
  480     CONTINUE
          CONSN(IGR) = DTEMP1
          DTEMP2=DTEMP2 + CONSN(IGR)
        ENDIF
 501  CONTINUE
      IF(QSM) THEN
        DO 502 IGR=1,NGRP
          IF(IGR.EQ.1) THEN
            CONSN(IGR) = DTEMP2
          ELSE
           CONSN(IGR) = 1.D0
          ENDIF
 502    CONTINUE
        IF( IMPPK.EQ.1 .OR. IMPR.GT.2  )THEN
        WRITE(6,*) ' CONSTRAINT AT END OF INTERVAL IS ', CONSN(1)
        WRITE(6,*) '                         RATIO IS ', CONSN(1)/AT(1)
        ENDIF
      ELSE
        IF( IMPPK.EQ.1 .OR. IMPR.GT.2  )THEN
        WRITE(6,*)' CONSTRAINT AT END OF INTERVAL IS ', CONSN
        WRITE(6,*)'                         RATIO IS ', CONSN(1)/AT(1),
     1                                                  CONSN(2)/AT(2)
        ENDIF
      ENDIF
C
C RESOLUTION DE LA CINETIQUE PONCTUELLE
C
C INITIALISATION DEBUT DE LA TRANSITOIRE
      IF(RENORM)THEN
       DO 666 IGR=1,NGRP
         TERM(IGR)=AT(IGR)
 666   CONTINUE
      ELSE
       DO 777 IGR=1,NGRP
         TERM(IGR)=CONSN(IGR)
 777   CONTINUE
      ENDIF
      IF( ITRAT.EQ. 1 ) THEN
        DO 888 IGR=1,NGRP
          ATOUT(IGR) = AT(IGR)
 888    CONTINUE
      ENDIF
*
      DO 161 I=1,NT
 161   Y(I)=YDE(I)
*
*
      NT2=2*NT
      NTS=NT2*NT2
      H=HI
      CALL SETARA(BASE,NTS,IFP)
      CALL SETARA(BASE,NTS,IA)
      CALL SETARA(BASE,NT2,IY4)
      CALL SETARA(BASE,NT2,IY3)
      CALL SETARA(BASE,NT2,IYDUM)
      CALL SETARA(BASE,NT2,IF)
      CALL SETARA(BASE,NT2,IF1)
      CALL SETARA(BASE,NT2,IF2)
      CALL SETARA(BASE,NT2,IF3)
      CALL SETARA(BASE,NT2,IF4)
      CALL SETARA(BASE,NT2,IFDUM)
      IF(LTETA) THEN
        IF(QSM) THEN
*       RUN POINT KINETICS (KAPS-RENTROP)
          TMICRO = 0.0D0
*
          CALL  IQSKRT(NT,Y,H,BASE(IFP),BASE(IA),
     x    BASE(IY4),
     x    BASE(IY3),BASE(IYDUM),BASE(IF),BASE(IF1),BASE(IF2),
     x    BASE(IF3),BASE(IF4),BASE(IFDUM),DBLE(TMICRO),METHOD,
     x    DEL,DBLE(DTMACR),NDG,QSM,NGRP,
     x    P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM)
*
          WRITE(*,*) ' USING KAPS-RENTROP AND THETA METHOD FOR IQS '
*
          CALL IQSFIT (NT,Y,BASE(IF),NGRP,
     x    P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM)
*
          CALL IQSVAL(AMPLD,BASE(IF),NGRP)
*
          AMPLD(1) = AMPLD(1)/Y(1)
          DO 551 IGR=1,NGRP
            AMPLD(IGR) = AMPLD(1)
            AMP(IGR)   =  Y(1)
 551      CONTINUE
*
        ELSE
*
          DO 562 I=1,NT
 562        Y(I)=YDE(I)
          TMICRO = 0.0D0
*
          CALL  IQSKRT(NT,Y,H,BASE(IFP),BASE(IA),
     x    BASE(IY4),
     x    BASE(IY3),BASE(IYDUM),BASE(IF),BASE(IF1),BASE(IF2),
     x    BASE(IF3),BASE(IF4),BASE(IFDUM),DBLE(TMICRO),METHOD,
     x    DEL,DBLE(DTMACR),NDG,QSM,NGRP,
     x    P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM)
*
          WRITE(*,*) ' USING KAPS-RENTROP AND THETA METHOD FOR GQS '
*
          CALL SETARA(BASE,NGRP*NGRP*2,IAB)
          CALL SETARA(BASE,NGRP*NGRP*2,IB)
          CALL SETARA(BASE,NGRP*2,IDERC)
          CALL SETARA(BASE,NGRP*2,IR0)
          CALL SETARA(BASE,NGRP*2,IDERL)
          CALL SETARA(BASE,NGRP*2,IPR)
*
          CALL IQSFGT (NT,Y,BASE(IF),NGRP,
     x     P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM,
     x     BASE(IAB),BASE(IB),BASE(IDERC),
     x     BASE(IR0),BASE(IDERL),BASE(IPR))
*
          CALL RLSARA(BASE(IPR))
          CALL RLSARA(BASE(IDERL))
          CALL RLSARA(BASE(IR0))
          CALL RLSARA(BASE(IDERC))
          CALL RLSARA(BASE(IB))
          CALL RLSARA(BASE(IAB))
*
          CALL IQSVAL(AMPLD,BASE(IF),NGRP)
*
          DO 712 IGR=1,NGRP
            AMPLD(IGR) = AMPLD(IGR)/Y(IGR)
            AMP(IGR)   =  Y(IGR)
 712      CONTINUE
        ENDIF
*
      ELSE
* GAMMA METHOD
       IF(QSM) THEN
*        RUN POINT KINETICS (KAPS-RENTROP)
          TMICRO = 0.0D0
*
          CALL  IQSKAR(NT,Y,H,BASE(IFP),BASE(IA),
     x      BASE(IY4),
     x      BASE(IY3),BASE(IYDUM),BASE(IF),BASE(IF1),BASE(IF2),
     x      BASE(IF3),BASE(IF4),BASE(IFDUM),DBLE(TMICRO),METHOD,
     x      DEL,DBLE(DTMACR),NDG,QSM,NGRP,
     x      P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM)
*
          WRITE(*,*) ' USING KAPS AND RENTROP METHOD FOR IQS '
*
*
          CALL IQSFI (NT,Y,BASE(IF),NGRP,
     x      P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM)
*
          CALL IQSVAL(AMPLD,BASE(IF),NGRP)
*
          AMPLD(1) = AMPLD(1)/Y(1)
          DO 511 IGR=1,NGRP
            AMPLD(IGR) = AMPLD(1)
            AMP(IGR)   =  Y(1)
 511      CONTINUE
*
       ELSE
*
*       RUN IQSKAR
          DO 162 I=1,NT
 162      Y(I)=YDE(I)
          TMICRO = 0.0D0
*
*
          CALL  IQSKAR(NT,Y,H,BASE(IFP),BASE(IA),
     x     BASE(IY4),
     x     BASE(IY3),BASE(IYDUM),BASE(IF),BASE(IF1),BASE(IF2),
     x     BASE(IF3),BASE(IF4),BASE(IFDUM),DBLE(TMICRO),METHOD,
     x     DEL,DBLE(DTMACR),NDG,QSM,NGRP,
     x     P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM)
*
          WRITE(*,*) ' USING KAPS AND RENTROP METHOD FOR GQS '
*
*
          CALL SETARA(BASE,NGRP*NGRP*2,IAB)
          CALL SETARA(BASE,NGRP*NGRP*2,IB)
          CALL SETARA(BASE,NGRP*2,IDERC)
          CALL SETARA(BASE,NGRP*2,IR0)
          CALL SETARA(BASE,NGRP*2,IDERL)
          CALL SETARA(BASE,NGRP*2,IPR)
*
          CALL IQSFG (NT,Y,BASE(IF),NGRP,
     x     P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM,
     x     BASE(IAB),BASE(IB),BASE(IDERC),
     x     BASE(IR0),BASE(IDERL),BASE(IPR))
*
          CALL RLSARA(BASE(IPR))
          CALL RLSARA(BASE(IDERL))
          CALL RLSARA(BASE(IR0))
          CALL RLSARA(BASE(IDERC))
          CALL RLSARA(BASE(IB))
          CALL RLSARA(BASE(IAB))
*
          CALL IQSVAL(AMPLD,BASE(IF),NGRP)
*
          DO 512 IGR=1,NGRP
            AMPLD(IGR) = AMPLD(IGR)/Y(IGR)
            AMP(IGR)   =  Y(IGR)
 512      CONTINUE
       ENDIF
      ENDIF
*
      CALL RLSARA(BASE(IFDUM))
      CALL RLSARA(BASE(IF4))
      CALL RLSARA(BASE(IF3))
      CALL RLSARA(BASE(IF2))
      CALL RLSARA(BASE(IF1))
      CALL RLSARA(BASE(IF))
      CALL RLSARA(BASE(IYDUM))
      CALL RLSARA(BASE(IY3))
      CALL RLSARA(BASE(IY4))
      CALL RLSARA(BASE(IA))
      CALL RLSARA(BASE(IFP))
*
      IF(IMPR.GT.2) WRITE(6,*) ' **** AMPLD =',AMPLD
*     SUM Y VALUES TO ESTIMATE GAMMA
*
      DO 891 IGR=1,NGRP
       DO 891 I=1,NDG
         GAMMA(IGR,I) = 0.0D0
         SIGMA(IGR,I) = 0.0D0
 891  CONTINUE
*
      IF(.NOT.LTETA) THEN
       IF(.NOT.LBI) THEN
        IF(QSM) THEN
          DO 893 IGR=1,NGRP
            IND2=2+2*NDG
            IND3=2+3*NDG
            DO 873 I=1,NDG
              DTEMP1 = BETA(I)*DECRAT(I)
              SIGMA(IGR,1) = DTEMP1*Y(IND2+I) + SIGMA(IGR,1)
              GAMMA(IGR,1) = DTEMP1*Y(IND3+I) + GAMMA(IGR,1)
 873        CONTINUE
 893      CONTINUE
        ELSE
          DO 892 IGR=1,NGRP
            IND2=NGRP*(NDG+1)+3*NDG*(IGR-1)+NDG
            IND3=NGRP*(NDG+1)+3*NDG*(IGR-1)+2*NDG
            DO 870 I=1,NDG
              DTEMP1 = BETA(I)*DECRAT(I)
              SIGMA(IGR,1) = DTEMP1*Y(IND2+I) + SIGMA(IGR,1)
              GAMMA(IGR,1) = DTEMP1*Y(IND3+I) + GAMMA(IGR,1)
 870        CONTINUE
 892      CONTINUE
        ENDIF
       ELSE
        IF(QSM) THEN
          DO 793 IGR=1,NGRP
            IND2=2+2*NDG
            IND3=2+3*NDG
            DO 773 I=1,NDG
              DTEMP1 = BETA(I)*DECRAT(I)
              SIGMA(IGR,I) = DTEMP1*Y(IND2+I)
              GAMMA(IGR,I) = DTEMP1*Y(IND3+I)
 773        CONTINUE
 793      CONTINUE
        ELSE
          DO 792 IGR=1,NGRP
            IND2=NGRP*(NDG+1)+3*NDG*(IGR-1)+NDG
            IND3=NGRP*(NDG+1)+3*NDG*(IGR-1)+2*NDG
            DO 770 I=1,NDG
              DTEMP1 = BETA(I)*DECRAT(I)
              SIGMA(IGR,I) = DTEMP1*Y(IND2+I)
              GAMMA(IGR,I) = DTEMP1*Y(IND3+I)
 770        CONTINUE
 792      CONTINUE
        ENDIF
       ENDIF
      ENDIF
      IF(IMPR.GT.2) WRITE(6,*) '* SIGMA =',SIGMA
      IF(IMPR.GT.2) WRITE(6,*) '* GAMMA =',GAMMA
C     PRINT OUT POINT KINETICS SOLUTION
C     COMPRESS GENERALIZED QUASISTATIC DATA TO QS
      DTEMP1 =0.0D0
      DTEMP2 =0.0D0
      DO 490 J=1,NGRP
       DO 490 I=1,NGRP
         DTEMP1 = DTEMP1 + (PB11(I,J)+PB01(I,J))*AMP(J)
         DTEMP2 = DTEMP2 + (PA11(I,J)+PA01(I,J))*AMP(J)
 490  CONTINUE
      DTEMP3 =0.0D0
      DTEMP4 =0.0D0
      DO 600 IGR =1,NGRP
        DTEMP3 = DTEMP3 + CONTOB(IGR)*AMP(IGR)
        DTEMP4 = DTEMP4 + CONTOB(IGR)
 600  CONTINUE
      REACT = (DTEMP1+DTEMP2-DRHO)/DTEMP1
      IF(IMPPK.EQ.1 .OR. IMPR.GT.0 ) THEN
          IF( QSM ) WRITE(6,*) '     SYSTEM SOLVED AS QUASISTATIC   '
          WRITE(6,485)' GQ..SYSTEM......',
     1       ((I,J, (P11(I,J)+P01(I,J))/CONSN(I),J=1,NGRP),I=1,NGRP)
          WRITE(6,*) '   PERIODE INVERSE.', AMPLD
          WRITE(6,*) '   AMPLITUDE.......', AMP
          WRITE(6,*) '   PRECURSEURS.....', ETA
          WRITE(6,*) '   CONTRAINT.......', CONTOB
          WRITE(6,*) '   SEPERATION RATIO', AMP(2)/AMP(1) - 1.D0
          WRITE(6,*) ' EQUIVALENT QUASISTATIC PARAMETERS'
          WRITE(6,*) '   REACTIVITE......', REACT
          WRITE(6,*) '   LAMBDA..........', DTEMP3/DTEMP1
          WRITE(6,*) '   AMPLITUDE.......', DTEMP3/DTEMP4
      ENDIF
*
C ETAPE 7: TEST DE CONVERGENCE
      DREACT=ABS(REACTP-REACT)
      IF(IMPPK.EQ.1) WRITE(6,380) DREACT*1000.0
      X1 = DREACT
      X2 = X1 * DREACT/REACT
      X3 =ABS(AT(1)- ATOUT(1))/AT(1)
      X4 = ABS(AMP(1)-AMPP(1))/AMP(1)
      DO 852 I=2,NGRP
        X3 = MAX(X3, ABS(AT(I)- ATOUT(I))/AT(I) )
        X4 = MAX(X4 , REAL(ABS(AMP(I)-AMPP(I))/AMP(I)) )
 852  CONTINUE
C
      IF(.NOT.RENORM) ERCON =1.E10
      IF(IMPR.GE.1) THEN
        WRITE(6,5)
        WRITE(6,7) EPSRHO,RHOPOU,ERCON,ERAMP,X1,X2,X3,X4
      ENDIF
      IF(X1.LE.EPSRHO    .AND.
     1   X2.LE.RHOPOU    .AND.
     2   X3.LE.ERCON     .AND.
     3   X4.LE.ERAMP     .AND.
     4   IERR.EQ.0       .AND.
     5   ITRAT.NE.1           ) GO TO 240
C     IF(DREACT.LT.EPSRHO) GO TO 240
      IF(ITRAT.GE.LIMM) GO TO 239
C
      IF( PKIN ) GOTO 205
C
C ETAPE 8: CALCUL DE LA SOURCE RETARDEE
      CALL SETARA(BASE,NUN*NGRP*NDG,IWRK)
      CALL SETARA(BASE,2*NGRP*NDG,IWRK1)
      CALL SETARA(BASE,2*NGRP*NDG,IWRK2)
      CALL SETARA(BASE,2*NGRP*NDG,IWRK3)
*
      IF(LTETA) THEN
        CALL IQSPRT(IPSYSS,IPSYSP,NDG,BETA,DECRAT,DTMACR,NUN,
     x    EVECT,DVECT,FKEFF,BASE(IWRK),1,PREC,DELSRC,NGRP,RAPQ,
     x    AMP,AMPGA,TETA,LBI)
      ELSE
        CALL IQSPRE(IPSYSS,IPSYSP,NDG,BETA,DECRAT,DTMACR,NUN,
     1    EVECT,DVECT,
     1    FKEFF,ITRANB,Y,BASE(IWRK1),BASE(IWRK2),BASE(IWRK3),
     3    BASE(IWRK),1,PREC,DELSRC,NGRP,QSM,NT,LBI)
       ENDIF
*
      CALL RLSARA(BASE(IWRK3))
      CALL RLSARA(BASE(IWRK2))
      CALL RLSARA(BASE(IWRK1))
      CALL RLSARA(BASE(IWRK))
C
C STEP 4: PERFORM FLUX CALCULATION WITH LAST END OF STEP FLX AS ESTIMATE
C
      CALL SETARA(BASE,NGRP*NUN,IGRAD)
      CALL SETARA(BASE,NGRP*NUN,IGRAD1)
      CALL SETARA(BASE,NGRP*NUN,IGAS)
      CALL SETARA(BASE,NGRP*NUN,IGAR)
      CALL SETARA(BASE,NGRP*NUN,IGAR1)
      CALL SETARA(BASE,NGRP*NUN,IGAR2)
      CALL SETARA(BASE,NGRP*NUN,IGAR3)
      CALL SETARA(BASE,NUN*NGRP,ISORC1)
      CALL SETARA(BASE,NUN*NGRP,ISOUR1)
      CALL SETARA(BASE,NGRP*2,IFRES)
      CALL SETARA(BASE,NGRP*2,IFRESN)
      CALL SETARA(BASE,12,ID2F)
      CALL SETARA(BASE,NGRP*2,IZNORM)
      CALL SETARA(BASE,NGRP,IFACTA)
*  TO SIMULATES QSM WITH GQS GO DIRECTLY TO GQS COMPUTATION WITH
*  QSM VALUES ; HOPEFULLY THEY WILL BE ALL SET PROPERLY
      IF(QSM) THEN
*        ATOUT(2)=1.0
         CALL IQSQSM(IPTRK,IPSYSP,IPSYSS,NUN,
     1   RM,DVECT,FKEFF,BASE(IGRAD),BASE(IGRAD1),
     2   BASE(IGAR),BASE(IGAR1),BASE(IGAR2),
     1   BASE(ISORC1),ICL1,ICL2,IMPR,
     3   TITR,EPS2,NADI,MAXX0,DELSRC,EVECT,ADECT,DTMACR,
     4   AMPLD,VM,AMP,BETAT,AT,ATOUT,BASE(ISOUR1),GAMMA,
     5   SIGMA,IERR,RENORM,FIXCON,VELO,V,NGRP,BASE(IFRES),
     x   BASE(IFRESN),BASE(ID2F),RAPQ,LTETA,NDG,LBI)
      ELSE
         CALL IQSGQS(IPTRK,IPSYSP,IPSYSS,NUN,
     1   RM,DVECT,FKEFF,BASE(IGRAD),BASE(IGRAD1),
     2   BASE(IGAR),BASE(IGAR1),BASE(IGAR2),
     1   BASE(ISORC1),ICL1,ICL2,IMPR,
     3   TITR,EPS2,NADI,MAXX0,DELSRC,EVECT,ADECT,DTMACR,
     4   AMPLD,VM,AMP,BETAT,AT,ATOUT,BASE(ISOUR1),GAMMA,
     5   SIGMA,IERR,RENORM,VELO,V,NGRP,BASE(IFRES),
     x   BASE(ID2F),BASE(IZNORM),BASE(IFACTA),RAPQ,LTETA,
     x   NDG,LBI)
      ENDIF
*
       CALL RLSARA(BASE(IFACTA))
       CALL RLSARA(BASE(IZNORM))
       CALL RLSARA(BASE(ID2F))
       CALL RLSARA(BASE(IFRESN))
       CALL RLSARA(BASE(IFRES))
       CALL RLSARA(BASE(ISOUR1))
       CALL RLSARA(BASE(ISORC1))
       CALL RLSARA(BASE(IGAR3))
       CALL RLSARA(BASE(IGAR2))
       CALL RLSARA(BASE(IGAR1))
       CALL RLSARA(BASE(IGAR))
       CALL RLSARA(BASE(IGAS))
       CALL RLSARA(BASE(IGRAD1))
       CALL RLSARA(BASE(IGRAD))
       GO TO 205
  239  WRITE(6,400)
C
C ETAPE 11: CALCUL DE LA CONCENTRATION DES PRECURSEURS FIN DE TRANS
  240 IF(IMPPK.EQ.1) WRITE(6,370) ITRAT,EPSRHO,DREACT
*
      CALL SETARA(BASE,NUN*NGRP*NDG,IWRK)
      CALL SETARA(BASE,2*NGRP*NDG,IWRK1)
      CALL SETARA(BASE,2*NGRP*NDG,IWRK2)
      CALL SETARA(BASE,2*NGRP*NDG,IWRK3)
*
      IF(LTETA) THEN
        CALL IQSPRT(IPSYSS,IPSYSP,NDG,BETA,DECRAT,DTMACR,NUN,
     x    EVECT,DVECT,FKEFF,BASE(IWRK),2,PREC,DELSRC,NGRP,RAPQ,
     x    AMP,AMPGA,TETA,LBI)
      ELSE
        CALL IQSPRE(IPSYSS,IPSYSP,NDG,BETA,DECRAT,DTMACR,NUN,
     1    EVECT,DVECT,
     1    FKEFF,ITRANB,Y,BASE(IWRK1),BASE(IWRK2),BASE(IWRK3),
     3    BASE(IWRK),2,PREC,DELSRC,NGRP,QSM,NT,LBI)
      ENDIF
*
      CALL RLSARA(BASE(IWRK3))
      CALL RLSARA(BASE(IWRK2))
      CALL RLSARA(BASE(IWRK1))
      CALL RLSARA(BASE(IWRK))
C
C CALCULATE THE NORM OF THE CHANGE IN THE FLUX.
C MISE A JOUR DE LA SOLUTION.
C SAUVEGARDE DE LA CONCENTRATION DES PRECURSEURS
       DO 709 IGR=1,NGRP
         DO 263 I=1,NDG
           WRITE (NAMB,'(4HPREC,I3,I3)') IGR,I
           CALL LCMPUT(IPFLUP,NAMB,NUN,2,PREC(1,I,IGR))
 263     CONTINUE
 709   CONTINUE
C
C MISE A JOUR DE L AMPLITUDE
      DO 66 IGR =1,NGRP
        AMPGA(IGR) = AMP(IGR)
  66  CONTINUE
*
      CALL RLSARA(BASE(IBETA))
      CALL RLSARA(BASE(IDECR))
      CALL IQSPRN(TMACRO,6,NGRP,
     x   P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM)
      RETURN
C
C FORMATS.
    5 FORMAT(10X,'ETUDE DE CONVERGENCE',/,
     1       10X,'REACT      REACTRE   CONTRA    AMPLITUDE')
    7 FORMAT(1X,'OBJECTIF:',4(2X,E8.2)/10X,4(2X,E8.2))
  360 FORMAT(////' *** REACTIVITY ITERATION : ',I5)
  370 FORMAT(/' ITERATION FOR REACTIVITY CONVERGED AFTER',I6,
     > '   ATTEMPTS'/'  THE PRECISION DEMANDED WAS:',E12.5,
     > ' MK'/ '                    ACHIEVED:',E12.5,' MK')
  380 FORMAT(/' THE CHANGE IN REACTIVITY = ',F10.5,' MK. ')
  390 FORMAT(///' START OF STEP VALUES FOR IMPORTANT PARAMETERS'
     > ,/,' REACTIVITY',T25,E12.5,/,' GENERATION TIME ',T25,E12.5
     >,/,' AMPLITUDE',T25,E12.5,/,' LOG DERIVATIVE OF',T25,E12.5
     > ,/,' SCALAR PREC CONT',T25,7(E12.5,3X))
  400 FORMAT(//,' MAXIMUM NUMBER OF ITERATIONS FOR REACTIVITY EXCEEDED')
  410 FORMAT(/,' **** INITIAL FLUX ITERATION ')
  450 FORMAT(/37H TEMPS CPU UTILISE POUR LA FORME:    ,F8.3/
     1 19X,41HPOUR LES CALCULS DE CINETIQUE PONCTUELLE:,F8.3/
     2 19X,29HPOUR LES CALCULS AUXILIAIRES:,F8.3//)
  485 FORMAT( A19,2(T20,2(' (',I1,',',I1,') ',E18.11)/))
      END
