*DECK LINKDS
      SUBROUTINE LINKDS(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*********************************************************************
*
*     THIS MODULE MODIFIES EITHER DEVICE OBJECT OR PROCEDE OBJECT
*     DEPENDING ON OPTION.
*     GEOMETRY OBJECT IS REQUIRED TO RECOVER MESHES FOR ROD
*     POSITION CALCULATION.
*
*     CETTE SOUS ROUTINE MET A JOUR L'OBJET DEVICE A PARTIR
*     DES DONNEES CONTENUES DANS L'OBJET PROCEDE.
*     OU BIEN
*     MET A JOUR L'OBJET PROCEDE A PARTIR DE DEVICE.
*     L'OBJET L_GEOM EST UTILISE POUR RECUPERER LES MESH
*     LORS DU CALCUL DES POSITIONS DES ADJUSTERS
*
*     INPUT/OUTPUT PARAMETERS:
*      HMODUL : CHARACTER*12 NAME OF THE MODULE.
*      NENTRY : NUMBER OF LINKED LISTS AND FILES USED BY THE MODULE.
*      HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
*      IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
*               =4 SEQUENTIAL ASCII FILE; =5 DIRECT ACCESS FILE.
*      JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
*               =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
*               =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
*      KENTRY : FILE UNIT NUMBER OR LINKED LIST ADDRESS.
*
*      DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
*      KENTRY(NENTRY)
*
*     REQUIRED LINKED LISTS ARE:
*        KENTRY(1-3) : OBJECTS L_DEVICE  IN MODIFICATION
*                              L_PROCEDE IN MODIFICATION
*                              L_GEOM    IN READ-ONLY MODE
*
*     PROCEDURES CALLED :  LINPER, LZCSPD, LZCNIV, LZCCAL,
*                                  ADJSPD, ADJPAD, ADJCAL
*
*-------------------------------------- AUTHORS: E. VARIN  95/02/10 ---
*
*
      IMPLICIT NONE
*
      REAL BASE
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      INTEGER NENTRY
      CHARACTER HENTRY(*)*12
      INTEGER IENTRY,JENTRY,KENTRY
      DIMENSION IENTRY(*),JENTRY(*),KENTRY(*)
*
      INTEGER      IPPROC,IPLDEV,IPGEOM,JLDEV,JPROC
C REDGET TYPES
      INTEGER NITMA
      REAL FLOTT
      DOUBLE PRECISION DFLOTT
      CHARACTER*12 TEXT
*
* DEVICE STRUCTURE
*
      CHARACTER*12 NAMTYP,NAMFAM,NAMREC
      LOGICAL      LVIT,LPOS
*
* SIGNATURE INFORMATION
*
      CHARACTER*12 SGNTUR
      INTEGER ISGNTR(3)
*
      INTEGER      ITYP,I,I1,IACTIO
      INTEGER      IPRT
      REAL         DT
      LOGICAL      INSR
*
*     INPUT PARAMTER VALIDATION
*
      IPPROC = 0
      IPLDEV = 0
      IPGEOM = 0
      IF(NENTRY.LT.3) CALL XABORT('LINKDS: PARAMETER EXPECTED.')
*
      DO 5 I=1,NENTRY
        IF((IENTRY(I).EQ.1).OR.(IENTRY(I).EQ.2)) THEN
         CALL LCMSIX(KENTRY(I),' ',0)
         CALL LCMGET(KENTRY(I),'SIGNATURE',ISGNTR)
         WRITE(SGNTUR,'(3A4)') (ISGNTR(I1),I1=1,3)
         IF (SGNTUR.EQ.'L_DEVICE') THEN
           IF (IPLDEV.EQ.0) THEN
             IPLDEV=KENTRY(I)
             IF(JENTRY(I).EQ.0) CALL XABORT('LINKDS: MODIFICATION OR '
     +       //'READ-ONLY MODE EXPECTED FOR OBJECT '//SGNTUR//' .')
             JLDEV = JENTRY(I)
           ENDIF
         ELSEIF (SGNTUR.EQ.'L_PROCEDE') THEN
           IF (IPPROC.EQ.0) THEN
             IPPROC=KENTRY(I)
             IF(JENTRY(I).EQ.0) CALL XABORT('LINKDS: MODIFICATION OR '
     +       //'READ-ONLY MODE EXPECTED FOR OBJECT '//SGNTUR//' .')
             JPROC = JENTRY(I)
           ENDIF
         ELSEIF (SGNTUR.EQ.'L_GEOM') THEN
           IF (IPGEOM.EQ.0) IPGEOM=KENTRY(I)
           IF(JENTRY(I).NE.2) CALL XABORT('LINKDS: READ-ONL'
     +       //'Y MODE EXPECTED FOR OBJECT'//SGNTUR//'.')
         ENDIF
        ENDIF
  5   CONTINUE
      IF ((IPGEOM.EQ.0).OR.(IPPROC.EQ.0).OR.(IPLDEV.EQ.0))
     +   CALL XABORT('LINKDS: ILLEGAL OR MISSING TYPE OF OBJECTS')
*
      IPRT = 1
      INSR = .TRUE.
      DT = 0.0
      IACTIO = 0
*
*     DATA INPUT READING
*
 50   CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
      IF(ITYP.EQ.3) THEN
        IF (TEXT.EQ.'EDIT') THEN
*   PRINTING FLAG READING
          CALL REDGET(ITYP,IPRT,FLOTT,TEXT,DFLOTT)
          IF (ITYP.NE.1)
     +      CALL XABORT('LINKDS: INTEGER DATA EXPECTED')
        ELSEIF (TEXT.EQ.'DEVC') THEN
*
*   DATA READING FOR DEVICE UPDATE
*
          IF( IACTIO.NE.0 .AND. IACTIO.NE.1 )CALL XABORT('LINKDS: '
     +    //'ONLY ONE OPTION DEVC/PROC PER CALL')
          IF( JLDEV.NE.1 )CALL XABORT('LINKDS: L_DEVICE OBJECT MUST'
     +    //' BE IN MODIFICATION MODE FOR DEVC OPTION')
          IF( JPROC.NE.1 )CALL XABORT('LINKDS: L_PROCEDE OBJECT MUST'
     +    //' BE IN MODIFICATION MODE FOR DEVC OPTION')
          IACTIO = 1
          GOTO 11
        ELSEIF (TEXT.EQ.'PROC') THEN
*
*  DATA READING FOR PROCEDE UPDATE
*
          IF( IACTIO.NE.0 .AND. IACTIO.NE.2 )CALL XABORT('LINKDS: '
     +    //'ONLY ONE OPTION DEVC/PROC PER CALL')
          IF( JLDEV.NE.2 )CALL XABORT('LINKDS: L_DEVICE OBJECT MUST'
     +    //' BE IN READ-ONLY MODE FOR PROC OPTION')
          IF( JPROC.NE.1 )CALL XABORT('LINKDS: L_PROCEDE OBJECT MUST'
     +    //' BE IN MODIFICATION MODE FOR PROC OPTION')
          IACTIO = 2
          GOTO 11
        ELSE
          CALL XABORT('LINKDS: INVALID KEYWORD '//TEXT)
        ENDIF
      ELSE
        CALL XABORT('LINKDS: CHARACTER DATA EXPECTED')
      ENDIF
      GOTO 50
*
 11   CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
      IF (ITYP.NE.3)
     +   CALL XABORT('LINKDS: CHARACTER DATA EXPECTED')
      IF (TEXT.EQ.'TYPE') THEN
*   DEVICE TYPE NAME READING
          CALL REDGET(ITYP,NITMA,FLOTT,NAMTYP,DFLOTT)
          IF (ITYP.NE.3)
     +         CALL XABORT('LINKDS: CHARACTER DATA EXPECTED FOR '
     +         //'DEVICE TYPE')
          CALL LCMSIX(IPLDEV,' ',0)
          CALL LCMSIX(IPLDEV,NAMTYP,1)
 10       CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
          IF (ITYP.EQ.3) THEN
            IF(TEXT.EQ.'LZC') THEN
              CALL REDGET(ITYP,NITMA,FLOTT,NAMFAM,DFLOTT)
              IF (ITYP.NE.3)
     +          CALL XABORT('LINKDS: CHARACTER DATA EXPECTED FOR LZC'
     +          //' FAMILY NAME')
*
              CALL LCMSIX(IPPROC,' ',0)
              CALL LCMSIX(IPPROC,NAMFAM,1)
              CALL LCMSIX(IPLDEV,NAMFAM,1)
              IF( IACTIO.EQ.1 )THEN
* DEVC MODE
                 LVIT = .FALSE.
                 LPOS = .FALSE.
                 CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
                 IF (ITYP.NE.3)
     +           CALL XABORT('LINKDS: CHARACTER DATA EXPECTED POS/SPD')
                 IF(TEXT.EQ.'SPEED') THEN
                   LVIT = .TRUE.
                   NAMREC = 'YVZ'
                 ELSEIF(TEXT.EQ.'POS') THEN
                   LPOS = .TRUE.
                   NAMREC = 'NIV'
                 ELSE
                    CALL XABORT('LINKDS: INVALID KEYWORD '//TEXT)
                 ENDIF
                 IF( LVIT )THEN
                   CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
                   IF (ITYP.NE.3)
     +           CALL XABORT('LINKDS: CHARACTER DATA EXPECTED TIME')
                   IF(TEXT.EQ.'TIME') THEN
                     CALL REDGET(ITYP,NITMA,DT,TEXT,DFLOTT)
                     IF (ITYP.NE.2)
     +                 CALL XABORT('LINKDS: REAL DATA EXPECTED')
                   ELSE
                     CALL XABORT('LINKDS: INVALID KEYWORD LZC '//TEXT)
                   ENDIF
                 ENDIF
                 CALL LINPER(IPPROC,NAMFAM,NAMREC)
*
                 IF( LVIT )THEN
                   CALL LZCSPD(IPPROC,IPLDEV,IPRT,DT)
                 ELSEIF (LPOS) THEN
                   CALL LZCNIV(IPPROC,IPLDEV,IPRT)
                 ENDIF
              ELSEIF( IACTIO.EQ.2 )THEN
* PROC MODE
                 CALL LZCCAL(IPPROC,IPLDEV,IPRT)
              ENDIF
              CALL LCMSIX(IPPROC,' ',2)
              CALL LCMSIX(IPLDEV,' ',2)
            ELSEIF(TEXT.EQ.'ROD') THEN
              CALL REDGET(ITYP,NITMA,FLOTT,NAMFAM,DFLOTT)
              IF (ITYP.NE.3)
     +          CALL XABORT('LINKDS: CHARACTER DATA EXPECTED FOR ROD'
     +            //' FAMILY NAME')
*
              CALL LCMSIX(IPPROC,' ',0)
              CALL LCMSIX(IPPROC,NAMFAM,1)
              CALL LCMSIX(IPLDEV,NAMFAM,1)
              IF( IACTIO.EQ.1 )THEN
* DEVC MODE
                 LVIT = .FALSE.
                 LPOS = .FALSE.
                 CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
                 IF (ITYP.NE.3)
     +           CALL XABORT('LINKDS: CHARACTER DATA EXPECTED POS/SPD')
                 IF(TEXT.EQ.'SPEED') THEN
                   LVIT = .TRUE.
                   NAMREC = 'VITBC'
                 ELSEIF(TEXT.EQ.'POS') THEN
                   LPOS = .TRUE.
                   NAMREC = 'POS'
                 ELSE
                    CALL XABORT('LINKDS: INVALID KEYWORD '//TEXT)
                 ENDIF
                 CALL LINPER(IPPROC,NAMFAM,NAMREC)
*
                 IF( LVIT )THEN
                   CALL ADJSPD(IPPROC,IPLDEV,IPGEOM,IPRT)
                 ELSEIF (LPOS) THEN
                   CALL ADJPAD(IPPROC,IPLDEV,IPGEOM,IPRT)
                 ENDIF
              ELSEIF( IACTIO.EQ.2 )THEN
* PROC MODE
                CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
                IF (ITYP.NE.3)
     +            CALL XABORT('LINKDS: CHARACTER DATA EXPECTED INSR |'
     +                //' EXTR')
                IF (TEXT.EQ.'INSR') THEN
                  INSR = .TRUE.
                ELSEIF(TEXT.EQ.'EXTR') THEN
                  INSR = .FALSE.
                ENDIF
*
                CALL ADJCAL(IPPROC,IPLDEV,IPGEOM,INSR,IPRT)
              ENDIF
              CALL LCMSIX(IPPROC,' ',2)
              CALL LCMSIX(IPLDEV,' ',2)
            ELSEIF(TEXT.EQ.'END') THEN
              GOTO 11
            ELSE
              CALL XABORT('LINKDS: INVALID KEYWORD END '//TEXT)
            ENDIF
          ELSE
            CALL XABORT('LINKDS: CHARACTER DATA EXPECTED')
          ENDIF
          GOTO 10
      ELSEIF( TEXT.EQ.';' )THEN
         GOTO 60
      ELSE
         CALL XABORT('LINKDS: INVALID KEYWORD ; '//TEXT)
      ENDIF
*
 60   CALL LCMSIX(IPPROC,' ',0)
      CALL LCMSIX(IPLDEV,' ',0)
      RETURN
      END
