*DECK MACD
      SUBROUTINE MACD(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
* MACROSCOPIC CROSS SECTIONS AND DIFFUSION COEFFICIENTS INPUT MODULE
* FOR TRIVAC-3.
*
* INPUT/OUTPUT PARAMETERS:
*  NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
*  HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
*  IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
*           =4 SEQUENTIAL ASCII FILE.
*  JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
*           =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
*           =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
*  KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.
*           DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
*           KENTRY(NENTRY)
*
* LINKED LIST / XSM FILE:
*  HENTRY(1) : CREATE OR MODIFICATION TYPE(L_MACROLIB).
*
*-------------------------------------- AUTHOR: A. HEBERT ; 26/11/93 ---
*
      CHARACTER HENTRY(*)*12,TEXT4*4,TEXT12*12,HSMG*131,HSIGN*12
      DOUBLE PRECISION DFLOTT
      DIMENSION IENTRY(*),JENTRY(*),KENTRY(*),IPAR(7),KCHAR(3)
      COMMON BASE(1)
*
* PARAMETER VALIDATION.
      IF(NENTRY.EQ.0) CALL XABORT('MACD: PARAMETER EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('MACD: LINK'
     1 //'ED LIST OR XSM FILE EXPECTED AT LHS.')
      IF((JENTRY(1).NE.0).AND.(JENTRY(1).NE.1)) CALL XABORT('MACD: ENTR'
     1 //'Y IN CREATE OR MODIFICATION MODE EXPECTED.')
      ITYPE=JENTRY(1)
      IPLIST=KENTRY(1)
*
* READ THE INPUT DATA.
*     DEFAULT OPTIONS:
      IMPX=1
      ISTEP=0
      IF(ITYPE.EQ.0) THEN
         IND=1
         NL=1
         NGRP=0
         NMIXT=0
         NDG=0
      ELSE IF(ITYPE.EQ.1) THEN
         CALL LCMGET(IPLIST,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_MACROLIB') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('MACD: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_MACROLIB EXPECTED.')
         ENDIF
         IND=2
         CALL LCMGET(IPLIST,'IDATAXS',IPAR)
         NGRP=IPAR(1)
         NMIXT=IPAR(2)
         NL=IPAR(3)
         NDG=IPAR(7)
      ENDIF
*
* READ THE MAC: MODULE OPTIONS.
   10 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('MACD: CHARACTER DATA EXPECTED(1).')
   20 IF(TEXT4.EQ.'EDIT') THEN
*        READ THE PRINT INDEX.
         CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MACD: INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT4.EQ.'NGRO') THEN
*        READ THE NUMBER OF ENERGY GROUPS.
         IF(IND.EQ.2) CALL XABORT('MACD: NGRO IS ALREADY DEFINED.')
         CALL REDGET(INDIC,NGRP,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MACD: INTEGER DATA EXPECTED(2).')
      ELSE IF(TEXT4.EQ.'NMIX') THEN
*        READ THE MAXIMUM NUMBER OF MATERIAL MIXTURES.
         IF(IND.EQ.2) CALL XABORT('MACD: NMIX IS ALREADY DEFINED.')
         CALL REDGET(INDIC,NMIXT,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MACD: INTEGER DATA EXPECTED(3).')
      ELSE IF(TEXT4.EQ.'DELP') THEN
*        READ THE MAXIMUM NUMBER OF PRECURSORS.
         IF(IND.EQ.2) CALL XABORT('MACD: DELP IS ALREADY DEFINED.')
         CALL REDGET(INDIC,NDG,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MACD: INTEGER DATA EXPECTED(3).')
      ELSE IF(TEXT4.EQ.'ANIS') THEN
*        READ THE SCATTERING ANISOTROPY FOR TRANSPORT THEORY CASES.
         IF(IND.EQ.2) CALL XABORT('ANIS: NMIX IS ALREADY DEFINED.')
         CALL REDGET(INDIC,NL,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MACD: INTEGER DATA EXPECTED(4).')
      ELSE IF(TEXT4.EQ.'ALBP') THEN
*        READ GROUP-INDEPENDENT PHYSICAL ALBEDOS
         CALL REDGET(INDIC,NALBD,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MACD: INTEGER DATA EXPECTED(5).')
         IF(NALBD.GT.0) THEN
            CALL SETARA(BASE,NALBD,IALB)
            DO 15 IAL=1,NALBD
            CALL REDGET(INDIC,NITMA,BASE(IALB+IAL-1),TEXT4,DFLOTT)
            IF(INDIC.NE.2) CALL XABORT('MACD: REAL DATA EXPECTED.')
  15        CONTINUE
            CALL LCMPUT(IPLIST,'ALBEDO',NALBD,2,BASE(IALB))
            CALL RLSARA(BASE(IALB))
         ELSE
            CALL XABORT('MACD: INVALID NUMBER OF ALBEDOS.')
         ENDIF
      ELSE IF(TEXT4.EQ.'STEP') THEN
*        STEP TO A SON DIRECTORY AND WRITE PERTURBATION VALUES.
         CALL REDGET(INDIC,ISTEP,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MACD: INTEGER DATA EXPECTED(6).')
         WRITE(TEXT12,'(4HSTEP,I8)') ISTEP
         IF(IND.EQ.1) THEN
            CALL LCMLEN(IPLIST,TEXT12,ILENG,ITYLCM)
            IF(ILENG.GT.0) THEN
               WRITE(HSMG,'(30HMACD: PERTURBATION DIRECTORY '',A12,
     1         21H'' ALREADY EXISTS IN '',A12,2H''.)') TEXT12,HENTRY(1)
               CALL XABORT(HSMG)
            ENDIF
         ENDIF
         CALL LCMSIX(IPLIST,TEXT12,1)
         IF(IMPX.GT.0) WRITE(6,'(/32H MACD: WRITE PERTURBATION VALUES
     1 15HON, DIRECTORY '',A12,6H'' OF '',A12,2H''.)') TEXT12,HENTRY(1)
      ELSE IF(TEXT4.EQ.'READ') THEN
*        INPUT NON-PERTURBED OR PERTURBED DIFFUSION COEFFICIENTS AND
*        CROSS SECTIONS PER MIXTURE.
         CALL SETARA(BASE,NMIXT*NGRP,ITOTAL)
         CALL SETARA(BASE,NMIXT*NGRP,IZNUG)
         CALL SETARA(BASE,NMIXT*NGRP,ICHI)
         CALL SETARA(BASE,NMIXT*NGRP*NDG,ICHDL)
         CALL SETARA(BASE,NMIXT*NGRP,IOVERV)
         CALL SETARA(BASE,NMIXT*NGRP,IDIFFX)
         CALL SETARA(BASE,NMIXT*NGRP,IDIFFY)
         CALL SETARA(BASE,NMIXT*NGRP,IDIFFZ)
         CALL SETARA(BASE,NMIXT*NGRP,IH)
         CALL SETARA(BASE,NMIXT*NGRP,IS)
         CALL SETARA(BASE,NMIXT*NL*NGRP,IIJJ)
         CALL SETARA(BASE,NMIXT*NL*NGRP,INJJ)
         CALL SETARA(BASE,NMIXT*NL*NGRP*NGRP,ISCAT)
         CALL SETARA(BASE,NMIXT*NGRP,IWORK)
         CALL SETARA(BASE,NMIXT,IIPOS)
         CALL MACXSI(IPLIST,IND,NMIXT,NGRP,NDG,NL,IMPX,NBMIX,JND,
     1   BASE(ITOTAL),BASE(IZNUG),BASE(ICHI),BASE(ICHDL),BASE(IOVERV),
     x   BASE(IDIFFX),BASE(IDIFFY),BASE(IDIFFZ),BASE(IH),BASE(IS),
     x   BASE(IIJJ),BASE(INJJ),BASE(ISCAT),BASE(IWORK),BASE(IIPOS))
         CALL RLSARA(BASE(IIPOS))
         CALL RLSARA(BASE(IWORK))
         CALL RLSARA(BASE(ISCAT))
         CALL RLSARA(BASE(INJJ))
         CALL RLSARA(BASE(IIJJ))
         CALL RLSARA(BASE(IS))
         CALL RLSARA(BASE(IH))
         CALL RLSARA(BASE(IDIFFZ))
         CALL RLSARA(BASE(IDIFFY))
         CALL RLSARA(BASE(IDIFFX))
         CALL RLSARA(BASE(IOVERV))
         CALL RLSARA(BASE(ICHDL))
         CALL RLSARA(BASE(ICHI))
         CALL RLSARA(BASE(IZNUG))
         CALL RLSARA(BASE(ITOTAL))
         IF(ISTEP.GT.0) CALL LCMSIX(IPLIST,' ',2)
         IF(JND.EQ.1) THEN
            GO TO 30
         ELSE IF(JND.EQ.2) THEN
            TEXT4='STEP'
            GO TO 20
         ENDIF
      ELSE
         CALL XABORT('MACD: '//TEXT4//' IS AN INVALID KEY-WORD.')
      ENDIF
      GO TO 10
*
   30 IF(ITYPE.EQ.0) THEN
         HSIGN='L_MACROLIB'
         READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMPUT(IPLIST,'SIGNATURE',3,3,KCHAR)
         IPAR(1)=NGRP
         IPAR(2)=NMIXT
         IPAR(3)=NL
         IPAR(4)=1
         IPAR(5)=0
         IPAR(6)=0
         IPAR(7)=NDG
         CALL LCMPUT(IPLIST,'IDATAXS',7,1,IPAR)
      ENDIF
      IF((IMPX.GT.1).AND.(ISTEP.GT.0)) CALL LCMLIB(IPLIST)
      RETURN
      END
