*DECK MOVDEV
      SUBROUTINE MOVDEV(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
************************************************************************
*
*     THIS MODULE ALLOWS THE MODIFICATION OF DEVICE POSITION
*     ACCORDING TO THEIR SPEED IN CM/S.
*
*     CE MODULE PERMET DE MODIFIER LES COORDONNEES DES
*     CONTROLEURS EN FONCTION DE LEUR VITESSE RESPECTIVE.
*
*
*     INPUT/OUTPUT PARAMETERS:
*      HMODUL : CHARACTER*12 NAME OF THE MODULE.
*      NENTRY : NUMBER OF LINKED LISTS AND FILES USED BY THE MODULE.
*      HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
*      IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
*               =4 SEQUENTIAL ASCII FILE; =5 DIRECT ACCESS FILE.
*      JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
*               =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
*               =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
*      KENTRY : FILE UNIT NUMBER OR LINKED LIST ADDRESS.
*
*      DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
*      KENTRY(NENTRY)
*
*     REQUIRED LINKED LISTS ARE:
*        KENTRY(1-3) : OBJECTS TYPE L_DEVICE  IEN MODIFICATION
*                                   L_PROCEDE IN READ ONLY MODE
*                                   L_GEOM    IN READ ONLY MODE
*
*     PROCEDURES CALLED:  LZCMOV, ADJMOV
*
*-------------------------------------- AUTHORS: E. VARIN  94/04/10 ---
*
*
*      IMPLICIT NONE
*
      REAL BASE
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      INTEGER NENTRY
      CHARACTER HENTRY(*)*12
      INTEGER IENTRY,JENTRY,KENTRY
      DIMENSION IENTRY(*),JENTRY(*),KENTRY(*)
*
      INTEGER      IPLDEV,IPPROC,IPGEOM
C REDGET TYPES
      INTEGER NITMA,ITYP
      REAL FLOTT
      DOUBLE PRECISION DFLOTT
      CHARACTER*12 TEXT
*
* DEVICE STRUCTURE
*
      CHARACTER*12 NAMTYP,NAMFAM
*
* SIGNATURE INFORMATION
*
      CHARACTER*12 SGNTUR
      INTEGER ISGNTR(3)
*
      INTEGER      I
      INTEGER      IPRT
      REAL         DT
*
*     INPUT PARAMETER VALIDATION
*
      IPPROC = 0
      IPGEOM = 0
      IPLDEV = 0
      IF(NENTRY.LT.3) CALL XABORT('MOVDEV: PARAMETER EXPECTED.')
*
      DO 5 I=1,NENTRY
        IF((IENTRY(I).EQ.1).OR.(IENTRY(I).EQ.2)) THEN
         CALL LCMSIX(KENTRY(I),' ',0)
         CALL LCMGET(KENTRY(I),'SIGNATURE',ISGNTR)
         WRITE(SGNTUR,'(3A4)') (ISGNTR(I1),I1=1,3)
         IF (SGNTUR.EQ.'L_DEVICE') THEN
           IF (IPLDEV.EQ.0) IPLDEV=KENTRY(I)
           IF(JENTRY(1).NE.1) CALL XABORT('MOVD'
     +      //'EV: MODIFICATION MODE EXPECTED FOR OBJECT'//SGNTUR//'.')
         ELSEIF (SGNTUR.EQ.'L_PROCEDE') THEN
           IF (IPPROC.EQ.0) IPPROC=KENTRY(I)
           IF(JENTRY(I).NE.2) CALL XABORT('MOVDEV: READ-ONL'
     +       //'Y MODE EXPECTED FOR OBJECT'//SGNTUR//'.')
         ELSEIF (SGNTUR.EQ.'L_GEOM') THEN
           IF (IPGEOM.EQ.0) IPGEOM=KENTRY(I)
           IF(JENTRY(I).NE.2) CALL XABORT('MOVDEV: READ-ONL'
     +       //'Y MODE EXPECTED FOR OBJECT'//SGNTUR//'.')
         ENDIF
        ENDIF
  5   CONTINUE
      IF ((IPGEOM.EQ.0).OR.(IPLDEV.EQ.0).OR.(IPPROC.EQ.0))
     +   CALL XABORT('MOVDEV: ILLEGAL OR MISSING TYPE OF OBJECTS')
*
      IPRT = 1
      DT = 0.0
*
*     DATA INPUT READING
*
 50   CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
      IF(ITYP.EQ.3) THEN
        IF (TEXT.EQ.'EDIT') THEN
*   PRINTING FLAG READING
          CALL REDGET(ITYP,IPRT,FLOTT,TEXT,DFLOTT)
          IF (ITYP.NE.1)
     +      CALL XABORT('MOVDEV: INTEGER DATA EXPECTED')
        ELSEIF (TEXT.EQ.'TYPE') THEN
*   DEVICE TYPE NAME READING
          CALL REDGET(ITYP,NITMA,FLOTT,NAMTYP,DFLOTT)
          IF (ITYP.NE.3)
     +      CALL XABORT('MOVDEV: CHARACTER DATA EXPECTED')
        ELSEIF (TEXT.EQ.'TIME') THEN
*   SIMULATION TIME STEP READING
          CALL REDGET(ITYP,NITMA,DT,TEXT,DFLOTT)
          IF (ITYP.NE.2)
     +      CALL XABORT('MOVDEV: REAL DATA EXPECTED')
        ELSEIF(TEXT.EQ.'LZC') THEN
          CALL REDGET(ITYP,NITMA,FLOTT,NAMFAM,DFLOTT)
          IF (ITYP.NE.3)
     +      CALL XABORT('MOVDEV: CHARACTER DATA EXPECTED')
          IF( DT.EQ.0.0 )CALL XABORT('LINKDS: TIME STEP DT'
     +            //' MUST BE SPECIFIED')
          CALL LZCMOV(IPLDEV,IPPROC,NAMTYP,NAMFAM,IPRT,DT)
*
        ELSEIF(TEXT.EQ.'ROD') THEN
          CALL REDGET(ITYP,NITMA,FLOTT,NAMFAM,DFLOTT)
          IF (ITYP.NE.3)
     +      CALL XABORT('MOVDEV: CHARACTER DATA EXPECTED')
          IF( DT.EQ.0.0 )CALL XABORT('LINKDS: TIME STEP DT'
     +            //' MUST BE SPECIFIED')
          CALL ADJMOV(IPLDEV,IPGEOM,NAMTYP,NAMFAM,IPRT,DT)
*
        ELSEIF (TEXT.EQ.';') THEN
          GOTO 60
        ELSE
          CALL XABORT('MOVDEV: CONTROLLED TYPE EXPECTED'//TEXT)
        ENDIF
      ELSE
        CALL XABORT('MOVDEV: CHARACTER DATA EXPECTED')
      ENDIF
      GOTO 50
*
 60   CALL LCMSIX(IPLDEV,' ',0)
      CALL LCMSIX(IPPROC,' ',0)
      RETURN
      END
