*DECK MTLDLM
      SUBROUTINE MTLDLM(NAMP,IPTRK,IPSYS,F2,F3)
*
*-----------------------------------------------------------------------
*
* XSM/LCM DRIVER FOR THE MULTIPLICATION OF A MATRIX BY A VECTOR.
*
* INPUT PARAMETERS:
*   NAMP : CHARACTER*12 NAME OF THE COEFFICIENT MATRIX.
*  IPTRK : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPSYS : L_SYSTEM POINTER TO SYSTEM MATRICES.
*     F2 : VECTOR TO MULTIPLY.
*
* OUTPUT PARAMETER:
*     F3 : RESULT OF THE MULTIPLICATION.
*
* INFORMATION RECOVERED FROM XSM OR LCM (NON-SPLITTED MATRIX):
*  NAMP       : COEFFICIENT MATRIX.
*  'MU'       : POSITION OF DIAGONAL ELEMENT IN COEFFICIENT MATRIX.
*
* INFORMATION RECOVERED FROM XSM OR LCM (SPLITTED MATRIX):
*  'W'//NAMP 'X'//NAMP 'Y'//NAMP 'Z'//NAMP : W-, X-, Y- AND Z-ORIENTED
*                            MATRIX COMPONENTS.
*
* SCALAR INFORMATION RECOVERED FROM LCM.
*  'MUW' 'MUX' 'MUY' 'MUZ' : POSITION OF DIAGONAL ELEMENT IN W-, X-, Y-
*                            OR Z-ORIENTED MATRIX COMPONENTS.
*  'IPW' 'IPX' 'IPY' 'IPZ' : PERMUTATION INFORMATION FOR W-, X-, Y- OR
*                            Z-ORIENTED MATRIX COMPONENTS.
*
* SUPERVECTORIZATION INFORMATION RECOVERED FROM LCM.
*  'LL4VW' 'LL4VX' 'LL4VY' 'LL4VZ' : ORDER OF THE REORDERED W-, X-, Y-
*                            AND Z-ORIENTED MATRIX COMPONENTS.
*  'MUVW' 'MUVX' 'MUVY' 'MUVZ' : POSITION OF DIAGONAL ELEMENT IN W-, X-,
*                            Y-OR Z-ORIENTED MATRIX COMPONENTS.
*  'IPVW' 'IPVX' 'IPVY' 'IPVZ' : PERMUTATION INFORMATION FOR W-, X-, Y-
*                            OR Z-ORIENTED MATRIX COMPONENTS.
*  'NBLW' 'NBLX' 'NBLY' 'NBLZ' : NUMBER OF LINEAR SYSTEMS IN EACH SUPER-
*                            VECTORIAL UNKNOWN GROUP.
*  'LBLW' 'LBLX' 'LBLY' 'LBLZ' : ORDER OF LINEAR SYSTEMS IN EACH SUPER-
*                            VECTORIAL UNKNOWN GROUP.
*
*-------------------------------------- AUTHOR: A. HEBERT ; 21/10/92 ---
*
      CHARACTER NAMP*(*),NAMT*12,TEXT12*12,NAMLCM*12,NAMMY*12
      DIMENSION F2(*),F3(*),IBASE(1),ITP(30)
      LOGICAL LMU,LMUW,LMUX,LMUY,LMUZ,DIAG,LCM
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      CALL LCMLEN(IPTRK,'MU',IDUM,ITYLCM)
      LMU=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUW',IDUM,ITYLCM)
      LMUW=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUX',IDUM,ITYLCM)
      LMUX=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUY',IDUM,ITYLCM)
      LMUY=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUZ',IDUM,ITYLCM)
      LMUZ=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      DIAG=LMUY.AND.(.NOT.LMUX)
      CALL LCMLEN(IPTRK,'TPARAM',ILENG,ITYLCM)
      IF(ILENG.GT.0) THEN
         CALL LCMGET(IPTRK,'TPARAM',ITP)
         ISEG=ITP(12)
         LTSW=ITP(14)
      ELSE
         ISEG=0
      ENDIF
      NAMT=' '
      CALL LCMNXT(IPSYS,NAMT,NAMLCM,NAMMY,LCM)
*
      NAMT=NAMP
      IF(LMU) THEN
         CALL LCMLEN(IPTRK,'MU',LL4,ITYLCM)
         IMU=LCMIOF(IPTRK,'MU')
         IF(LCM) THEN
*           IN-CORE STORAGE OF THE GLOBAL COMPRESSED MATRIX.
            IASS=LCMIOF(IPSYS,NAMT)
            CALL ALLDLM(LL4,BASE(IASS),F2,F3,IBASE(IMU),1)
         ELSE
*           XSM STORAGE OF THE GLOBAL COMPRESSED MATRIX.
            CALL SETARA(BASE,IBASE(IMU+LL4-1),IASS)
            CALL LCMGET(IPSYS,NAMT,BASE(IASS))
            CALL ALLDLM(LL4,BASE(IASS),F2,F3,IBASE(IMU),1)
            CALL RLSARA(BASE(IASS))
         ENDIF
      ELSE IF(ISEG.EQ.0) THEN
*        SCALAR MULTIPLICATION FOR A W- OR X-ORIENTED MATRIX.
         IF(LMUW) THEN
            IMU=LCMIOF(IPTRK,'MUW')
            TEXT12='W'//NAMT
            CALL LCMLEN(IPTRK,'IPW',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPW')
         ELSE IF(DIAG) THEN
            IMU=LCMIOF(IPTRK,'MUY')
            TEXT12='Y'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPX')
         ELSE
            IMU=LCMIOF(IPTRK,'MUX')
            TEXT12='X'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPX')
         ENDIF
         CALL SETARA(BASE,LL4,IGAR)
         DO 10 I=1,LL4
   10    BASE(IGAR+IBASE(IIP+I-1)-1)=F2(I)
         DO 20 I=1,LL4
   20    F2(I)=BASE(IGAR+I-1)
         IF(LCM) THEN
*           IN-CORE STORAGE.
            IASS=LCMIOF(IPSYS,TEXT12)
            CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),1)
         ELSE
*           XSM STORAGE.
            CALL SETARA(BASE,IBASE(IMU+LL4-1),IASS)
            CALL LCMGET(IPSYS,TEXT12,BASE(IASS))
            CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),1)
            CALL RLSARA(BASE(IASS))
         ENDIF
         DO 30 I=1,LL4
         II=IBASE(IIP+I-1)
         F3(I)=BASE(IGAR+II-1)
   30    BASE(IGAR+II-1)=F2(II)
         DO 40 I=1,LL4
   40    F2(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         IF(LMUW) THEN
*           SCALAR MULTIPLICATION FOR A X-ORIENTED MATRIX.
            IMU=LCMIOF(IPTRK,'MUX')
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPX')
            DO 50 I=1,LL4
   50       BASE(IGAR+IBASE(IIP+I-1)-1)=F2(I)
            DO 60 I=1,LL4
            II=IBASE(IIP+I-1)
            F2(II)=BASE(IGAR+II-1)
   60       BASE(IGAR+II-1)=F3(I)
            IF(LCM) THEN
*             IN-CORE SRORAGE.
              IASS=LCMIOF(IPSYS,'X'//NAMT)
              CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),2)
            ELSE
*             XSM STORAGE.
              CALL SETARA(BASE,IBASE(IMU+LL4-1),IASS)
              CALL LCMGET(IPSYS,'X'//NAMT,BASE(IASS))
              CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),2)
              CALL RLSARA(BASE(IASS))
            ENDIF
            DO 70 I=1,LL4
            II=IBASE(IIP+I-1)
            F3(I)=BASE(IGAR+II-1)
   70       BASE(IGAR+II-1)=F2(II)
            DO 80 I=1,LL4
   80       F2(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         IF(LMUY) THEN
*           SCALAR MULTIPLICATION FOR A Y-ORIENTED MATRIX.
            IMU=LCMIOF(IPTRK,'MUY')
            CALL LCMLEN(IPTRK,'IPY',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPY')
            DO 90 I=1,LL4
   90       BASE(IGAR+IBASE(IIP+I-1)-1)=F2(I)
            DO 100 I=1,LL4
            II=IBASE(IIP+I-1)
            F2(II)=BASE(IGAR+II-1)
  100       BASE(IGAR+II-1)=F3(I)
            IF(LCM) THEN
*             IN-CORE SRORAGE.
              IASS=LCMIOF(IPSYS,'Y'//NAMT)
              CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),2)
            ELSE
*             XSM STORAGE.
              CALL SETARA(BASE,IBASE(IMU+LL4-1),IASS)
              CALL LCMGET(IPSYS,'Y'//NAMT,BASE(IASS))
              CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),2)
              CALL RLSARA(BASE(IASS))
            ENDIF
            DO 110 I=1,LL4
            II=IBASE(IIP+I-1)
            F3(I)=BASE(IGAR+II-1)
  110       BASE(IGAR+II-1)=F2(II)
            DO 120 I=1,LL4
  120       F2(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         IF(LMUZ) THEN
*           SCALAR MULTIPLICATION FOR A Z-ORIENTED MATRIX.
            IMU=LCMIOF(IPTRK,'MUZ')
            CALL LCMLEN(IPTRK,'IPZ',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPZ')
            DO 130 I=1,LL4
  130       BASE(IGAR+IBASE(IIP+I-1)-1)=F2(I)
            DO 140 I=1,LL4
            II=IBASE(IIP+I-1)
            F2(II)=BASE(IGAR+II-1)
  140       BASE(IGAR+II-1)=F3(I)
            IF(LCM) THEN
*              IN-CORE STORAGE.
               IASS=LCMIOF(IPSYS,'Z'//NAMT)
               CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),2)
            ELSE
*              XSM STORAGE.
               CALL SETARA(BASE,IBASE(IMU+LL4-1),IASS)
               CALL LCMGET(IPSYS,'Z'//NAMT,BASE(IASS))
               CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),2)
               CALL RLSARA(BASE(IASS))
            ENDIF
            DO 150 I=1,LL4
            II=IBASE(IIP+I-1)
            F3(I)=BASE(IGAR+II-1)
  150       BASE(IGAR+II-1)=F2(II)
            DO 160 I=1,LL4
  160       F2(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         CALL RLSARA(BASE(IGAR))
      ELSE IF(ISEG.GT.0) THEN
*        SUPERVECTORIAL MULTIPLICATION FOR A W- OR X-ORIENTED MATRIX.
         IF(LMUW) THEN
            CALL LCMGET(IPTRK,'LL4VW',LL4V)
            IMU=LCMIOF(IPTRK,'MUVW')
            TEXT12='W'//NAMT
            CALL LCMLEN(IPTRK,'IPW',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPW')
            IIPV=LCMIOF(IPTRK,'IPVW')
            CALL LCMLEN(IPTRK,'NBLW',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLW')
            ILBL=LCMIOF(IPTRK,'LBLW')
         ELSE IF(DIAG) THEN
            CALL LCMGET(IPTRK,'LL4VY',LL4V)
            IMU=LCMIOF(IPTRK,'MUVY')
            TEXT12='Y'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPX')
            IIPV=LCMIOF(IPTRK,'IPVY')
            CALL LCMLEN(IPTRK,'NBLY',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLY')
            ILBL=LCMIOF(IPTRK,'LBLY')
         ELSE
            CALL LCMGET(IPTRK,'LL4VX',LL4V)
            IMU=LCMIOF(IPTRK,'MUVX')
            TEXT12='X'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPX')
            IIPV=LCMIOF(IPTRK,'IPVX')
            CALL LCMLEN(IPTRK,'NBLX',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLX')
            ILBL=LCMIOF(IPTRK,'LBLX')
         ENDIF
         CALL SETARA(IBASE,LL4,IIPB)
         DO 165 I=1,LL4
  165    IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
         CALL SETARA(BASE,LL4V,IGAR)
         CALL SETARA(BASE,LL4V,IGAF)
         DO 170 I=1,LL4V
  170    BASE(IGAR+I-1)=0.0
         DO 180 I=1,LL4
  180    BASE(IGAR+IBASE(IIPB+I-1)-1)=F2(I)
         IF(LCM) THEN
*           IN-CORE STORAGE.
            IASS=LCMIOF(IPSYS,TEXT12)
            CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1      IBASE(IMU),1,ISEG,LON,IBASE(INBL),IBASE(ILBL))
         ELSE
*           XSM STORAGE.
            CALL LCMLEN(IPSYS,TEXT12,IIMAX,ITYLCM)
            CALL SETARA(BASE,IIMAX,IASS)
            CALL LCMGET(IPSYS,TEXT12,BASE(IASS))
            CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1      IBASE(IMU),1,ISEG,LON,IBASE(INBL),IBASE(ILBL))
            CALL RLSARA(BASE(IASS))
         ENDIF
         DO 190 I=1,LL4
         II=IBASE(IIPB+I-1)
         F2(I)=BASE(IGAR+II-1)
  190    F3(I)=BASE(IGAF+II-1)
         CALL RLSARA(BASE(IGAF))
         CALL RLSARA(BASE(IGAR))
         CALL RLSARA(BASE(IIPB))
         IF(LMUW) THEN
*           SUPERVECTORIAL MULTIPLICATION FOR A X-ORIENTED MATRIX.
            CALL LCMGET(IPTRK,'LL4VX',LL4V)
            IMU=LCMIOF(IPTRK,'MUVX')
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPX')
            IIPV=LCMIOF(IPTRK,'IPVX')
            CALL LCMLEN(IPTRK,'NBLX',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLX')
            ILBL=LCMIOF(IPTRK,'LBLX')
            CALL SETARA(IBASE,LL4,IIPB)
            DO 195 I=1,LL4
  195       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            CALL SETARA(BASE,LL4V,IGAF)
            DO 200 I=1,LL4V
  200       BASE(IGAR+I-1)=0.0
            DO 210 I=1,LL4V
  210       BASE(IGAF+I-1)=0.0
            DO 220 I=1,LL4
            II=IBASE(IIPB+I-1)
            BASE(IGAR+II-1)=F2(I)
  220       BASE(IGAF+II-1)=F3(I)
            IF(LCM) THEN
*              IN-CORE STORAGE.
               IASS=LCMIOF(IPSYS,'X'//NAMT)
               CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1         IBASE(IMU),2,ISEG,LON,IBASE(INBL),IBASE(ILBL))
            ELSE
*              XSM STORAGE.
               CALL LCMLEN(IPSYS,'X'//NAMT,IIMAX,ITYLCM)
               CALL SETARA(BASE,IIMAX,IASS)
               CALL LCMGET(IPSYS,'X'//NAMT,BASE(IASS))
               CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1         IBASE(IMU),2,ISEG,LON,IBASE(INBL),IBASE(ILBL))
               CALL RLSARA(BASE(IASS))
            ENDIF
            DO 230 I=1,LL4
            II=IBASE(IIPB+I-1)
            F2(I)=BASE(IGAR+II-1)
  230       F3(I)=BASE(IGAF+II-1)
            CALL RLSARA(BASE(IGAF))
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
         IF(LMUY) THEN
*           SUPERVECTORIAL MULTIPLICATION FOR A Y-ORIENTED MATRIX.
            CALL LCMGET(IPTRK,'LL4VY',LL4V)
            IMU=LCMIOF(IPTRK,'MUVY')
            CALL LCMLEN(IPTRK,'IPY',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPY')
            IIPV=LCMIOF(IPTRK,'IPVY')
            CALL LCMLEN(IPTRK,'NBLY',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLY')
            ILBL=LCMIOF(IPTRK,'LBLY')
            CALL SETARA(IBASE,LL4,IIPB)
            DO 235 I=1,LL4
  235       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            CALL SETARA(BASE,LL4V,IGAF)
            DO 240 I=1,LL4V
  240       BASE(IGAR+I-1)=0.0
            DO 250 I=1,LL4V
  250       BASE(IGAF+I-1)=0.0
            DO 260 I=1,LL4
            II=IBASE(IIPB+I-1)
            BASE(IGAR+II-1)=F2(I)
  260       BASE(IGAF+II-1)=F3(I)
            IF(LCM) THEN
*              IN-CORE STORAGE.
               IASS=LCMIOF(IPSYS,'Y'//NAMT)
               CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1         IBASE(IMU),2,ISEG,LON,IBASE(INBL),IBASE(ILBL))
            ELSE
*              XSM STORAGE.
               CALL LCMLEN(IPSYS,'Y'//NAMT,IIMAX,ITYLCM)
               CALL SETARA(BASE,IIMAX,IASS)
               CALL LCMGET(IPSYS,'Y'//NAMT,BASE(IASS))
               CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1         IBASE(IMU),2,ISEG,LON,IBASE(INBL),IBASE(ILBL))
               CALL RLSARA(BASE(IASS))
            ENDIF
            DO 270 I=1,LL4
            II=IBASE(IIPB+I-1)
            F2(I)=BASE(IGAR+II-1)
  270       F3(I)=BASE(IGAF+II-1)
            CALL RLSARA(BASE(IGAF))
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
         IF(LMUZ) THEN
*           SUPERVECTORIAL MULTIPLICATION FOR A Z-ORIENTED MATRIX.
            CALL LCMGET(IPTRK,'LL4VZ',LL4V)
            IMU=LCMIOF(IPTRK,'MUVZ')
            CALL LCMLEN(IPTRK,'IPZ',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPZ')
            IIPV=LCMIOF(IPTRK,'IPVZ')
            CALL LCMLEN(IPTRK,'NBLZ',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLZ')
            ILBL=LCMIOF(IPTRK,'LBLZ')
            CALL SETARA(IBASE,LL4,IIPB)
            DO 275 I=1,LL4
  275       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            CALL SETARA(BASE,LL4V,IGAF)
            DO 280 I=1,LL4V
  280       BASE(IGAR+I-1)=0.0
            DO 290 I=1,LL4V
  290       BASE(IGAF+I-1)=0.0
            DO 300 I=1,LL4
            II=IBASE(IIPB+I-1)
            BASE(IGAR+II-1)=F2(I)
  300       BASE(IGAF+II-1)=F3(I)
            IF(LCM) THEN
*              IN-CORE STORAGE.
               IASS=LCMIOF(IPSYS,'Z'//NAMT)
               CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1         IBASE(IMU),2,ISEG,LON,IBASE(INBL),IBASE(ILBL))
            ELSE
*              XSM STORAGE.
               CALL LCMLEN(IPSYS,'Z'//NAMT,IIMAX,ITYLCM)
               CALL SETARA(BASE,IIMAX,IASS)
               CALL LCMGET(IPSYS,'Z'//NAMT,BASE(IASS))
               CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1         IBASE(IMU),2,ISEG,LON,IBASE(INBL),IBASE(ILBL))
               CALL RLSARA(BASE(IASS))
            ENDIF
            DO 310 I=1,LL4
            II=IBASE(IIPB+I-1)
            F2(I)=BASE(IGAR+II-1)
  310       F3(I)=BASE(IGAF+II-1)
            CALL RLSARA(BASE(IGAF))
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
      ENDIF
      RETURN
      END
