*DECK MTLDLS
      SUBROUTINE MTLDLS(NAMP,IPTRK,IPSYS,F1)
*
*-----------------------------------------------------------------------
*
* XSM/LCM DRIVER FOR THE SOLUTION OF A LINEAR SYSTEM AFTER LDL(T)
* FACTORIZATION.
*
* INPUT PARAMETERS:
*   NAMP : CHARACTER*12 NAME OF THE COEFFICIENT MATRIX.
*  IPTRK : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPSYS : L_SYSTEM POINTER TO SYSTEM MATRICES.
*     F1 : RIGHT-HAND SIDE OF THE LINEAR SYSTEM.
*
* OUTPUT PARAMETER:
*     F1 : SOLUTION OF THE LINEAR SYSTEM.
*
* INFORMATION RECOVERED FROM XSM OR LCM (NON-SPLITTED MATRIX):
*  NAMP       : COEFFICIENT MATRIX.
*  'I'//NAMP  : FACTORIZED COEFFICIENT MATRIX.
*  'MU'       : POSITION OF DIAGONAL ELEMENT IN COEFFICIENT MATRIX.
*
* INFORMATION RECOVERED FROM XSM OR LCM (SPLITTED MATRIX):
*  'W'//NAMP 'X'//NAMP 'Y'//NAMP 'Z'//NAMP : W-, X-, Y- AND Z-ORIENTED
*                            MATRIX COMPONENTS.
*  'WI'//NAMP 'XI'//NAMP 'YI'//NAMP 'ZI'//NAMP : W-, X-, Y- AND Z-
*                            ORIENTED FACTORIZED MATRIX COMPONENTS.
*
* SCALAR INFORMATION RECOVERED FROM LCM.
*  'MUW' 'MUX' 'MUY' 'MUZ' : POSITION OF DIAGONAL ELEMENT IN W-, X-, Y-
*                            OR Z-ORIENTED MATRIX COMPONENTS.
*  'IPW' 'IPX' 'IPY' 'IPZ' : PERMUTATION INFORMATION FOR W-, X-, Y- OR
*                            Z-ORIENTED MATRIX COMPONENTS.
*
* SUPERVECTORIZATION INFORMATION RECOVERED FROM LCM.
*  'DW'//NAMP 'DX'//NAMP 'DY'//NAMP 'DZ'//NAMP : DIAGONAL ELEMENTS FOR
*                           W-, X-, Y- AND Z-ORIENTED MATRIX COMPONENTS.
*  'LL4VW' 'LL4VX' 'LL4VY' 'LL4VZ' : ORDER OF THE REORDERED W-, X-, Y-
*                            AND Z-ORIENTED MATRIX COMPONENTS.
*  'MUVW' 'MUVX' 'MUVY' 'MUVZ' : POSITION OF DIAGONAL ELEMENT IN W-, X-,
*                            Y-OR Z-ORIENTED MATRIX COMPONENTS.
*  'IPVW' 'IPVX' 'IPVY' 'IPVZ' : PERMUTATION INFORMATION FOR W-, X-, Y-
*                            OR Z-ORIENTED MATRIX COMPONENTS.
*  'NBLW' 'NBLX' 'NBLY' 'NBLZ' : NUMBER OF LINEAR SYSTEMS IN EACH SUPER-
*                            VECTORIAL UNKNOWN GROUP.
*  'LBLW' 'LBLX' 'LBLY' 'LBLZ' : ORDER OF LINEAR SYSTEMS IN EACH SUPER-
*                            VECTORIAL UNKNOWN GROUP.
*
*-------------------------------------- AUTHOR: A. HEBERT ; 21/10/92 ---
*
      CHARACTER NAMP*(*),NAMT*12,TEXT12*12,NAMLCM*12,NAMMY*12
      LOGICAL LCM
      DIMENSION F1(*),IBASE(1),ITP(30)
      LOGICAL LMU,LMUW,LMUX,LMUY,LMUZ,DIAG
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      CALL LCMLEN(IPTRK,'MU',IDUM,ITYLCM)
      LMU=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUW',IDUM,ITYLCM)
      LMUW=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUX',IDUM,ITYLCM)
      LMUX=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUY',IDUM,ITYLCM)
      LMUY=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUZ',IDUM,ITYLCM)
      LMUZ=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      DIAG=LMUY.AND.(.NOT.LMUX)
      CALL LCMLEN(IPTRK,'TPARAM',ILENG,ITYLCM)
      IF(ILENG.GT.0) THEN
         CALL LCMGET(IPTRK,'TPARAM',ITP)
         ISEG=ITP(12)
         LTSW=ITP(14)
      ELSE
         ISEG=0
      ENDIF
      NAMT=' '
      CALL LCMNXT(IPSYS,NAMT,NAMLCM,NAMMY,LCM)
*
      NAMT=NAMP
      IF(LMU) THEN
         CALL LCMLEN(IPTRK,'MU',LL4,ITYLCM)
         IMU=LCMIOF(IPTRK,'MU')
         IF(LCM) THEN
*           IN-CORE STORAGE OF THE GLOBAL COMPRESSED MATRIX.
            IASS=LCMIOF(IPSYS,'I'//NAMT)
            CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),F1)
         ELSE
*           XSM STORAGE OF THE GLOBAL COMPRESSED MATRIX.
            CALL SETARA(BASE,IBASE(IMU+LL4-1),IASS)
            CALL LCMGET(IPSYS,'I'//NAMT,BASE(IASS))
            CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),F1)
            CALL RLSARA(BASE(IASS))
         ENDIF
      ELSE IF(ISEG.EQ.0) THEN
*        SCALAR SOLUTION FOR A W- OR X-ORIENTED LINEAR SYSTEM.
         IF(LMUW) THEN
            IMU=LCMIOF(IPTRK,'MUW')
            TEXT12='WI'//NAMT
            CALL LCMLEN(IPTRK,'IPW',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPW')
         ELSE IF(DIAG) THEN
            IMU=LCMIOF(IPTRK,'MUY')
            TEXT12='YI'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPX')
         ELSE
            IMU=LCMIOF(IPTRK,'MUX')
            TEXT12='XI'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPX')
         ENDIF
         CALL SETARA(BASE,LL4,IGAR)
         DO 10 I=1,LL4
   10    BASE(IGAR+IBASE(IIP+I-1)-1)=F1(I)
         IF(LCM) THEN
*           IN-CORE STORAGE.
            IASS=LCMIOF(IPSYS,TEXT12)
            CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
         ELSE
*           XSM STORAGE.
            CALL SETARA(BASE,IBASE(IMU+LL4-1),IASS)
            CALL LCMGET(IPSYS,TEXT12,BASE(IASS))
            CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
            CALL RLSARA(BASE(IASS))
         ENDIF
         DO 20 I=1,LL4
   20    F1(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         IF(LMUW) THEN
*           SCALAR SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            IMU=LCMIOF(IPTRK,'MUX')
            IIP=LCMIOF(IPTRK,'IPX')
            IF(LCM) THEN
*              IN-CORE STORAGE.
               IASS=LCMIOF(IPSYS,'X'//NAMT)
               DO 30 I=1,LL4
               II=IBASE(IIP+I-1)
   30          BASE(IGAR+II-1)=F1(I)*BASE(IASS+IBASE(IMU+II-1)-1)
               IASS=LCMIOF(IPSYS,'XI'//NAMT)
               CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
            ELSE
*              XSM STORAGE.
               CALL SETARA(BASE,IBASE(IMU+LL4-1),IASS)
               CALL LCMGET(IPSYS,'X'//NAMT,BASE(IASS))
               DO 40 I=1,LL4
               II=IBASE(IIP+I-1)
   40          BASE(IGAR+II-1)=F1(I)*BASE(IASS+IBASE(IMU+II-1)-1)
               CALL LCMGET(IPSYS,'XI'//NAMT,BASE(IASS))
               CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
               CALL RLSARA(BASE(IASS))
            ENDIF
            DO 50 I=1,LL4
   50       F1(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         IF(LMUY) THEN
*           SCALAR SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            IMU=LCMIOF(IPTRK,'MUY')
            IIP=LCMIOF(IPTRK,'IPY')
            IF(LCM) THEN
*              IN-CORE STORAGE.
               IASS=LCMIOF(IPSYS,'Y'//NAMT)
               DO 60 I=1,LL4
               II=IBASE(IIP+I-1)
   60          BASE(IGAR+II-1)=F1(I)*BASE(IASS+IBASE(IMU+II-1)-1)
               IASS=LCMIOF(IPSYS,'YI'//NAMT)
               CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
            ELSE
*              XSM STORAGE.
               CALL SETARA(BASE,IBASE(IMU+LL4-1),IASS)
               CALL LCMGET(IPSYS,'Y'//NAMT,BASE(IASS))
               DO 70 I=1,LL4
               II=IBASE(IIP+I-1)
   70          BASE(IGAR+II-1)=F1(I)*BASE(IASS+IBASE(IMU+II-1)-1)
               CALL LCMGET(IPSYS,'YI'//NAMT,BASE(IASS))
               CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
               CALL RLSARA(BASE(IASS))
            ENDIF
            DO 80 I=1,LL4
   80       F1(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         IF(LMUZ) THEN
*           SCALAR SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            IMU=LCMIOF(IPTRK,'MUZ')
            IIP=LCMIOF(IPTRK,'IPZ')
            IF(LCM) THEN
*              IN-CORE STORAGE.
               IASS=LCMIOF(IPSYS,'Z'//NAMT)
               DO 90 I=1,LL4
               II=IBASE(IIP+I-1)
   90          BASE(IGAR+II-1)=F1(I)*BASE(IASS+IBASE(IMU+II-1)-1)
               IASS=LCMIOF(IPSYS,'ZI'//NAMT)
               CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
            ELSE
*              XSM STORAGE.
               CALL SETARA(BASE,IBASE(IMU+LL4-1),IASS)
               CALL LCMGET(IPSYS,'Z'//NAMT,BASE(IASS))
               DO 100 I=1,LL4
               II=IBASE(IIP+I-1)
  100          BASE(IGAR+II-1)=F1(I)*BASE(IASS+IBASE(IMU+II-1)-1)
               CALL LCMGET(IPSYS,'ZI'//NAMT,BASE(IASS))
               CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
               CALL RLSARA(BASE(IASS))
            ENDIF
            DO 110 I=1,LL4
  110       F1(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         CALL RLSARA(BASE(IGAR))
      ELSE IF(ISEG.GT.0) THEN
*        SUPERVECTORIAL SOLUTION FOR A W- OR X-ORIENTED LINEAR SYSTEM.
         CALL SETARA(BASE,ISEG,IT)
         IF(LMUW) THEN
            CALL LCMGET(IPTRK,'LL4VW',LL4V)
            IMU=LCMIOF(IPTRK,'MUVW')
            TEXT12='WI'//NAMT
            CALL LCMLEN(IPTRK,'IPW',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPW')
            IIPV=LCMIOF(IPTRK,'IPVW')
            CALL LCMLEN(IPTRK,'NBLW',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLW')
            ILBL=LCMIOF(IPTRK,'LBLW')
         ELSE IF(DIAG) THEN
            CALL LCMGET(IPTRK,'LL4VY',LL4V)
            IMU=LCMIOF(IPTRK,'MUVY')
            TEXT12='YI'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPX')
            IIPV=LCMIOF(IPTRK,'IPVY')
            CALL LCMLEN(IPTRK,'NBLY',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLY')
            ILBL=LCMIOF(IPTRK,'LBLY')
         ELSE
            CALL LCMGET(IPTRK,'LL4VX',LL4V)
            IMU=LCMIOF(IPTRK,'MUVX')
            TEXT12='XI'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPX')
            IIPV=LCMIOF(IPTRK,'IPVX')
            CALL LCMLEN(IPTRK,'NBLX',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLX')
            ILBL=LCMIOF(IPTRK,'LBLX')
         ENDIF
         CALL SETARA(IBASE,LL4,IIPB)
         DO 115 I=1,LL4
  115    IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
         CALL SETARA(BASE,LL4V,IGAR)
         DO 120 I=1,LL4V
  120    BASE(IGAR+I-1)=0.0
         DO 130 I=1,LL4
  130    BASE(IGAR+IBASE(IIPB+I-1)-1)=F1(I)
         IF(LCM) THEN
*           IN-CORE STORAGE.
            IASS=LCMIOF(IPSYS,TEXT12)
            CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,LON,
     1      IBASE(INBL),IBASE(ILBL),BASE(IT))
         ELSE
*           XSM STORAGE.
            CALL LCMLEN(IPSYS,TEXT12,IIMAX,ITYLCM)
            CALL SETARA(BASE,IIMAX,IASS)
            CALL LCMGET(IPSYS,TEXT12,BASE(IASS))
            CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,LON,
     1      IBASE(INBL),IBASE(ILBL),BASE(IT))
            CALL RLSARA(BASE(IASS))
         ENDIF
         DO 140 I=1,LL4
  140    F1(I)=BASE(IGAR+IBASE(IIPB+I-1)-1)
         CALL RLSARA(BASE(IGAR))
         CALL RLSARA(BASE(IIPB))
         IF(LMUW) THEN
*           SUPERVECTORIAL SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VX',LL4V)
            IMU=LCMIOF(IPTRK,'MUVX')
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPX')
            IIPV=LCMIOF(IPTRK,'IPVX')
            CALL LCMLEN(IPTRK,'NBLX',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLX')
            ILBL=LCMIOF(IPTRK,'LBLX')
            CALL SETARA(IBASE,LL4,IIPB)
            DO 145 I=1,LL4
  145       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            DO 150 I=1,LL4V
  150       BASE(IGAR+I-1)=0.0
            DO 160 I=1,LL4
  160       BASE(IGAR+IBASE(IIPB+I-1)-1)=F1(I)
            IF(LCM) THEN
*              IN-CORE STORAGE.
               IASS=LCMIOF(IPSYS,'XI'//NAMT)
               IDGV=LCMIOF(IPSYS,'DX'//NAMT)
CDIR$ IVDEP
               DO 170 I=1,LL4V
  170          BASE(IGAR+I-1)=BASE(IGAR+I-1)*BASE(IDGV+I-1)
               CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,
     1         LON,IBASE(INBL),IBASE(ILBL),BASE(IT))
            ELSE
*              XSM STORAGE.
               CALL SETARA(BASE,LL4V,IGAF)
               CALL LCMGET(IPSYS,'DX'//NAMT,BASE(IGAF))
CDIR$ IVDEP
               DO 180 I=1,LL4V
  180          BASE(IGAR+I-1)=BASE(IGAR+I-1)*BASE(IGAF+I-1)
               CALL RLSARA(BASE(IGAF))
               CALL LCMLEN(IPSYS,'XI'//NAMT,IIMAX,ITYLCM)
               CALL SETARA(BASE,IIMAX,IASS)
               CALL LCMGET(IPSYS,'XI'//NAMT,BASE(IASS))
               CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,
     1         LON,IBASE(INBL),IBASE(ILBL),BASE(IT))
               CALL RLSARA(BASE(IASS))
            ENDIF
            DO 190 I=1,LL4
  190       F1(I)=BASE(IGAR+IBASE(IIPB+I-1)-1)
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
         IF(LMUY) THEN
*           SUPERVECTORIAL SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VY',LL4V)
            IMU=LCMIOF(IPTRK,'MUVY')
            CALL LCMLEN(IPTRK,'IPY',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPY')
            IIPV=LCMIOF(IPTRK,'IPVY')
            CALL LCMLEN(IPTRK,'NBLY',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLY')
            ILBL=LCMIOF(IPTRK,'LBLY')
            CALL SETARA(IBASE,LL4,IIPB)
            DO 195 I=1,LL4
  195       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            DO 200 I=1,LL4V
  200       BASE(IGAR+I-1)=0.0
            DO 210 I=1,LL4
  210       BASE(IGAR+IBASE(IIPB+I-1)-1)=F1(I)
            IF(LCM) THEN
*              IN-CORE STORAGE.
               IASS=LCMIOF(IPSYS,'YI'//NAMT)
               IDGV=LCMIOF(IPSYS,'DY'//NAMT)
CDIR$ IVDEP
               DO 220 I=1,LL4V
  220          BASE(IGAR+I-1)=BASE(IGAR+I-1)*BASE(IDGV+I-1)
               CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,
     1         LON,IBASE(INBL),IBASE(ILBL),BASE(IT))
            ELSE
*              XSM STORAGE.
               CALL SETARA(BASE,LL4V,IGAF)
               CALL LCMGET(IPSYS,'DY'//NAMT,BASE(IGAF))
CDIR$ IVDEP
               DO 230 I=1,LL4V
  230          BASE(IGAR+I-1)=BASE(IGAR+I-1)*BASE(IGAF+I-1)
               CALL RLSARA(BASE(IGAF))
               CALL LCMLEN(IPSYS,'YI'//NAMT,IIMAX,ITYLCM)
               CALL SETARA(BASE,IIMAX,IASS)
               CALL LCMGET(IPSYS,'YI'//NAMT,BASE(IASS))
               CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,
     1         LON,IBASE(INBL),IBASE(ILBL),BASE(IT))
               CALL RLSARA(BASE(IASS))
            ENDIF
            DO 240 I=1,LL4
  240       F1(I)=BASE(IGAR+IBASE(IIPB+I-1)-1)
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
         IF(LMUZ) THEN
*           SUPERVECTORIAL SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VZ',LL4V)
            IMU=LCMIOF(IPTRK,'MUVZ')
            CALL LCMLEN(IPTRK,'IPZ',LL4,ITYLCM)
            IIP=LCMIOF(IPTRK,'IPZ')
            IIPV=LCMIOF(IPTRK,'IPVZ')
            CALL LCMLEN(IPTRK,'NBLZ',LON,ITYLCM)
            INBL=LCMIOF(IPTRK,'NBLZ')
            ILBL=LCMIOF(IPTRK,'LBLZ')
            CALL SETARA(IBASE,LL4,IIPB)
            DO 245 I=1,LL4
  245       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            DO 250 I=1,LL4V
  250       BASE(IGAR+I-1)=0.0
            DO 260 I=1,LL4
  260       BASE(IGAR+IBASE(IIPB+I-1)-1)=F1(I)
            IF(LCM) THEN
*              IN-CORE STORAGE.
               IASS=LCMIOF(IPSYS,'ZI'//NAMT)
               IDGV=LCMIOF(IPSYS,'DZ'//NAMT)
CDIR$ IVDEP
               DO 270 I=1,LL4V
  270          BASE(IGAR+I-1)=BASE(IGAR+I-1)*BASE(IDGV+I-1)
               CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,
     1         LON,IBASE(INBL),IBASE(ILBL),BASE(IT))
            ELSE
*              XSM STORAGE.
               CALL SETARA(BASE,LL4V,IGAF)
               CALL LCMGET(IPSYS,'DZ'//NAMT,BASE(IGAF))
CDIR$ IVDEP
               DO 280 I=1,LL4V
  280          BASE(IGAR+I-1)=BASE(IGAR+I-1)*BASE(IGAF+I-1)
               CALL RLSARA(BASE(IGAF))
               CALL LCMLEN(IPSYS,'ZI'//NAMT,IIMAX,ITYLCM)
               CALL SETARA(BASE,IIMAX,IASS)
               CALL LCMGET(IPSYS,'ZI'//NAMT,BASE(IASS))
               CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,
     1         LON,IBASE(INBL),IBASE(ILBL),BASE(IT))
               CALL RLSARA(BASE(IASS))
            ENDIF
            DO 290 I=1,LL4
  290       F1(I)=BASE(IGAR+IBASE(IIPB+I-1)-1)
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
         CALL RLSARA(BASE(IT))
      ENDIF
      RETURN
      END
