*DECK NEIGHB
      FUNCTION NEIGHB (J,K,IHEX,NH,POIDS)
*
*-----------------------------------------------------------------------
*
* COMPUTE THE INDEX OF A NEIGHBOUR HEXAGON TAKING INTO ACCOUNT THE
* SYMMETRIES.
*
* INPUT PARAMETERS:
*       J  : INDEX OF THE CONSIDERED HEXAGON.
*       K  : INDEX OF THE SIDE:
*                                SIDE 2
*                                XXXXXX
*                     SIDE 3   X        X   SIDE 1
*                            X            X
*                          X                X
*                            X            X
*                     SIDE 4   X        X   SIDE 6
*                                XXXXXX
*                                SIDE 5
*
*    IHEX  : TYPE OF SYMMETRY:
*            =1 : S30   =2 : SA60   =3 : SB60   =4 : S90   =5 : R120
*            =6 : R180  =7 : SA180  =8 : SB180  =9 : COMPLETE
*            =10: S30 WITH HBC SYME =11: SA60 WITH HBC SYME
*      NH  : TOTAL NUMBER OF HEXAGONS.
*   POIDS  : WEIGHT OF THE HEXAGON.
*
* OUTPUT PARAMETERS:
*  NEIGHB  : INDEX OF THE NEIGHBOUR HEXAGON.
*            ABS(NEIGHB).GT.NH : EXTERNAL BOUNDARY
*            NEIGHB=J          : REFLECTION ON SIDE K
*            NEIGHB.LT.0       : AXIAL SYMMETRY OR ROTATION
*
*----------------------------------- AUTHOR: A. BENABOUD ; 15/05/90 ----
*
      COMMON AB(1)
      IF ((IHEX.EQ.1).OR.(IHEX.EQ.10)) THEN
         VI = 2.* SQRT(REAL(NH)) - 1.
         VP = SQRT(REAL(4*NH+1)) - 1.
         IF (AINT(VI).EQ.VI) THEN
            NC = INT(VI)
         ELSE IF (AINT(VP).EQ.VP) THEN
            NC = INT(VP)
         ELSE
            CALL XABORT('NEIGHB: INVALID NUMBER OF HEXAGONS(1).')
         ENDIF
      ELSE IF ((IHEX.EQ.2).OR.(IHEX.EQ.11)) THEN
         VA = (SQRT(REAL(8*NH+1)) - 1.)/2.
         IF (AINT(VA).EQ.VA) THEN
            NC = INT(VA)
         ELSE
            CALL XABORT('NEIGHB: INVALID NUMBER OF HEXAGONS(2).')
         ENDIF
      ELSE IF (IHEX.EQ.3) THEN
         VI = SQRT(REAL(2*NH-1))
         VP = SQRT(REAL(2*NH))
         IF (AINT(VI).EQ.VI) THEN
            NC = INT(VI)
         ELSE IF (AINT(VP).EQ.VP) THEN
            NC = INT(VP)
         ELSE
            CALL XABORT('NEIGHB: INVALID NUMBER OF HEXAGONS(3).')
         ENDIF
      ELSE IF (IHEX.EQ.4) THEN
         VI = SQRT(REAL((4*NH-1)/3))
         VP = SQRT(REAL(4*NH/3))
         IF (AINT(VI).EQ.VI) THEN
            NC = INT(VI)
         ELSE IF (AINT(VP).EQ.VP) THEN
            NC = INT(VP)
         ELSE
            CALL XABORT('NEIGHB: INVALID NUMBER OF HEXAGONS(4).')
         ENDIF
      ELSE IF (IHEX.EQ.5) THEN
         VA = (SQRT(REAL(4*(NH-1)+1)) + 1.)/2.
         IF (AINT(VA).EQ.VA) THEN
            NC = INT(VA)
         ELSE
            CALL XABORT('NEIGHB: INVALID NUMBER OF HEXAGONS(5).')
         ENDIF
      ELSE IF (IHEX.EQ.6) THEN
         VA = (SQRT(REAL(8*(NH-1)/3+1)) + 1)/2
         IF (AINT(VA).EQ.VA) THEN
            NC = INT(VA)
         ELSE
            CALL XABORT('NEIGHB: INVALID NUMBER OF HEXAGONS(6).')
         ENDIF
      ELSE IF (IHEX.EQ.7) THEN
         VA = (SQRT(REAL(24*NH+1)) + 1.)/6.
         IF (AINT(VA).EQ.VA) THEN
            NC = INT(VA)
         ELSE
            CALL XABORT('NEIGHB: INVALID NUMBER OF HEXAGONS(7).')
         ENDIF
      ELSE IF (IHEX.EQ.8) THEN
         VI = (1.+SQRT(REAL(3*(2*NH-1)+1)))/3.
         VP = (1.+SQRT(REAL(6*NH+1)))/3.
         IF (AINT(VI).EQ.VI) THEN
            NC = INT(VI)
         ELSE IF (AINT(VP).EQ.VP) THEN
            NC = INT(VP)
         ELSE
            CALL XABORT('NEIGHB: INVALID NUMBER OF HEXAGONS(8).')
         ENDIF
      ELSE IF (IHEX.EQ.9) THEN
         VA = (SQRT(REAL((4*NH-1)/3)) + 1.)/2.
         IF (AINT(VA).EQ.VA) THEN
            NC = INT(VA)
         ELSE
            CALL XABORT('NEIGHB: INVALID NUMBER OF HEXAGONS(9).')
         ENDIF
      ELSE
         CALL XABORT('NEIGHB: INVALID TYPE OF SYMMETRY.')
      ENDIF
      CALL SETARA (AB,NC+2,J1)
      CALL SETARA (AB,NC+2,J2)
      CALL SETARA (AB,NC+2,J3)
      CALL SETARA (AB,NC+2,J4)
*
      IF (IHEX.EQ.1) THEN
         CALL NEIGH1 (NC,J,K,N,POIDS,AB(J1),AB(J2),AB(J3),AB(J4))
      ELSE IF (IHEX.EQ.2) THEN
         CALL NEIGH2 (NC,J,K,N,POIDS,AB(J1),AB(J2),AB(J3))
      ELSE IF (IHEX.EQ.3) THEN
         CALL NEIGH3 (NC,J,K,N,POIDS,AB(J1),AB(J2),AB(J3))
      ELSE IF (IHEX.EQ.4) THEN
         CALL NEIGH4 (NC,J,K,N,POIDS,AB(J1),AB(J2),AB(J3),AB(J4))
      ELSE IF (IHEX.EQ.5) THEN
         CALL NEIGH5 (NC,J,K,N,POIDS,AB(J1),AB(J2),AB(J3))
         IF (-N.GT.NH) N=-N
      ELSE IF (IHEX.EQ.6) THEN
         CALL NEIGH6 (NC,J,K,N,POIDS,AB(J1),AB(J2),AB(J3))
         IF (-N.GT.NH) N=-N
      ELSE IF (IHEX.EQ.7) THEN
         CALL NEIGH7 (NC,J,K,N,POIDS,AB(J1),AB(J2),AB(J3))
      ELSE IF (IHEX.EQ.8) THEN
         CALL NEIGH8 (NC,J,K,N,POIDS,AB(J1),AB(J2),AB(J3))
      ELSE IF (IHEX.EQ.9) THEN
         CALL NEIGH9 (NC,J,K,N,POIDS,AB(J1),AB(J2),AB(J3))
      ELSE IF (IHEX.EQ.10) THEN
         CALL NEIG10 (NC,J,K,N,POIDS,AB(J1),AB(J2),AB(J3))
      ELSE IF (IHEX.EQ.11) THEN
         CALL NEIG11 (NC,J,K,N,POIDS,AB(J1),AB(J2),AB(J3))
      ENDIF
      CALL RLSARA (AB(J1))
      CALL RLSARA (AB(J2))
      CALL RLSARA (AB(J3))
      CALL RLSARA (AB(J4))
      NEIGHB=N
      RETURN
      END
