*DECK OUTDRV
      SUBROUTINE OUTDRV (MAXNEL,MAXMIX,IPGEOM,IPMAC1,IPFLUX,IPMAC2,
     1 NGRP,NEL,NUN,HTRACK,MAT,VOL,IDL,TITR,EVECT,IHOM,SGD)
*
*-----------------------------------------------------------------------
*
* DRIVER FOR THE PORT-TREATMENT OF REACTOR CALCULATION RESULTS.
*
* INPUT PARAMETERS:
*  MAXNEL : MAXIMUM NUMBER OF FINITE ELEMENTS.
*  MAXMIX : MAXIMUM NUMBER OF MATERIAL MIXTURES.
*  IPGEOM : L_GEOM POINTER TO THE GEOMETRY.
*  IPMAC1 : L_MACROLIB POINTER TO THE NUCLEAR PROPERTIES.
*  IPFLUX : L_FLUX POINTER TO THE SOLUTION.
*  IPMAC2 : L_MACROLIB POINTER TO THE EDITION INFORMATION.
*  NGRP   : TOTAL NUMBER OF ENERGY GROUPS.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : TOTAL NUMBER OF UNKNOWNS PER GROUP.
*  HTRACK : TYPE OF TRACKING: ='BIVAC' OR 'TRIVAC'.
*  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*           DIMENSION MAT(NEL)
*  VOL    : VOLUMES. DIMENSION VOL(NEL)
*  IDL    : POSITION OF THE AVERAGE FLUX COMPONENT ASSOCIATED WITH
*           EACH VOLUME. DIMENSION IDL(NEL)
*  TITR   : TITLE.
*
* SCRATCH STORAGE
*  EVECT  : DIMENSION EVECT(NUN,NGRP)
*  IHOM   : DIMENSION IHOM(NEL)
*  SGD    : DIMENSION SGD(MAXMIX)
*
*---------------------------------------------- A. HEBERT 13/07/93 -----
*
      CHARACTER TITR*72,HTRACK*12,TEXT4*4,TEXT12*12,CGRPNM*12
      DOUBLE PRECISION DFLOTT,ZNORM
      DIMENSION SGD(*),MAT(*),VOL(*),IDL(*),EVECT(NUN,*),IHOM(*)
      COMMON BASE(1)
*
      IMPX=1
*
* RECOVER THE SOLUTION.
      CALL LCMLEN(IPFLUX,'K-EFFECTIVE',ILEN,ITYLCM)
      IF(ILEN.GT.0) THEN
         CALL LCMGET(IPFLUX,'K-EFFECTIVE',FKEFF)
         CALL LCMPUT(IPMAC2,'K-EFFECTIVE',1,2,FKEFF)
      ENDIF
      CALL LCMLEN(IPFLUX,'FLUX  1',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         DO 10 IGR=1,NGRP
         WRITE(TEXT12,'(4HFLUX,I3)') IGR
         CALL LCMGET(IPFLUX,TEXT12,EVECT(1,IGR))
   10    CONTINUE
      ELSE
         CALL LCMLEN(IPFLUX,'FLUX  1  1',LENGT,ITYLCM)
         IF(LENGT.EQ.0) THEN
            CALL LCMLIB(IPFLUX)
            CALL XABORT('OUTDRV: UNABLE TO RECOVER A FLUX.')
         ENDIF
         DO 20 IGR=1,NGRP
         WRITE(TEXT12,'(4HFLUX,I3,3H  1)') IGR
         CALL LCMGET(IPFLUX,TEXT12,EVECT(1,IGR))
   20    CONTINUE
      ENDIF
*
   30 CALL REDGET (INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('OUTDRV: CHARACTER DATA EXPECTED.')
*
      IF(TEXT4.EQ.'POWR') THEN
*        NORMALIZATION TO A GIVEN FISSION POWER.
         CALL REDGET (INDIC,NITMA,POW,TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('OUTDRV: REAL DATA EXPECTED.')
*        NORMALIZATION FACTOR FOR THE DIRECT FLUX.
         ZNORM=0.0D0
         DO 40 IGR=1,NGRP
         WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
         CALL LCMSIX(IPMAC1,CGRPNM,1)
         CALL LCMLEN(IPMAC1,'H-FACTORS',LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            CALL LCMGET(IPMAC1,'H-FACTORS',SGD)
         ELSE
            WRITE(6,'(/44H OUTDRV: *** WARNING *** NO H-FACTORS FOUND ,
     1      28HON LCM. USE NU*SIGF INSTEAD.)')
            CALL LCMGET(IPMAC1,'NUSIGF',SGD)
         ENDIF
         CALL LCMSIX(IPMAC1,' ',2)
         DO 40 K=1,NEL
         L=MAT(K)
         IF((L.EQ.0).OR.(IDL(K).EQ.0)) GO TO 40
         ZNORM=ZNORM+EVECT(IDL(K),IGR)*VOL(K)*SGD(L)
   40    CONTINUE
         ZNORM=POW/ZNORM
         WRITE(6,300) ' DIRECT',ZNORM
         DO 50 IGR=1,NGRP
         DO 50 I=1,NUN
   50    EVECT(I,IGR)=EVECT(I,IGR)*ZNORM
      ELSE IF(TEXT4.EQ.'EDIT') THEN
         CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('OUT: INTEGER DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'INTG') THEN
*        COMPUTE AND DISPLAY THE MACRO-ZONE REACTION RATES.
*        READ THE MACRO-ZONES DEFINITION.
         CALL OUTHOM (MAXNEL,IPGEOM,IMPX,NEL,HTRACK,MAT,NZS,IHOM)
         IF(NZS.GT.NEL) CALL XABORT('OUTDRV: INVALID VALUE OF NZS.')
         CALL SETARA(BASE,MAX(MAXMIX,NZS)*NGRP,IWORK)
         CALL SETARA(BASE,MAX(MAXMIX,NZS),IIJJ)
         CALL SETARA(BASE,MAX(MAXMIX,NZS),INJJ)
         CALL SETARA(BASE,MAX(MAXMIX,NZS),IIPOS)
         CALL SETARA(BASE,NZS,IRATE)
         CALL SETARA(BASE,NZS*NGRP,IFLINT)
         CALL SETARA(BASE,NGRP,IGAR)
         CALL SETARA(BASE,NZS*NGRP,ISCAT)
         CALL SETARA(BASE,MAXMIX,IWORK2)
         CALL SETARA(BASE,(NZS+1)*12,IOUTR)
         CALL OUTAUX (MAXMIX,IPMAC1,IPMAC2,NGRP,NEL,NUN,NZS,MAT,VOL,
     1   IDL,EVECT,IHOM,IMPX,BASE(IWORK),BASE(IIJJ),BASE(INJJ),
     2   BASE(IIPOS),BASE(IRATE),BASE(IFLINT),BASE(IGAR),BASE(ISCAT),
     2   BASE(IWORK2),BASE(IOUTR))
         CALL RLSARA(BASE(IOUTR))
         CALL RLSARA(BASE(IWORK2))
         CALL RLSARA(BASE(ISCAT))
         CALL RLSARA(BASE(IGAR))
         CALL RLSARA(BASE(IFLINT))
         CALL RLSARA(BASE(IRATE))
         CALL RLSARA(BASE(IIPOS))
         CALL RLSARA(BASE(INJJ))
         CALL RLSARA(BASE(IIJJ))
         CALL RLSARA(BASE(IWORK))
      ELSE IF(TEXT4.EQ.';') THEN
         RETURN
      ELSE
         CALL XABORT('OUTDRV: '//TEXT4//' IS AN INVALID KEY WORD.')
      ENDIF
      GO TO 30
*
  300 FORMAT(/9H OUTDRV: ,A7,28H FLUX NORMALIZATION FACTOR =,1P,E13.5)
  310 FORMAT(/49H OUTDRV: CPU TIME FOR REACTION RATE CALCULATION =,F7.3)
      END
