*DECK POWER
      SUBROUTINE POWER(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        *******************************************************       *
*        * MODULE TO COMPUTE FUEL POWERS, AVERAGE COMBUSTION   *       *
*        * RATE AND CHANNEL POWER PEAKING FACTORS              *       *
*        *                                                     *       *
*        *******************************************************       *
*                                                                      *
*     INPUT/OUTPUT PARAMETERS:                                         *
*      NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.    *
*      HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.         *
*      IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;*
*               =4 SEQUENTIAL ASCII FILE.                              *
*      JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.                 *
*               =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;  *
*               =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.  *
*      KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.     *
*                                                                      *
*      DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),         *
*                KENTRY(NENTRY)                                        *
*                                                                      *
*     REQUIRED LINKED LISTS ARE:                                       *
*        KENTRY(1) : OBJECT TYPE L_MAP IN MODIFICATION MODE            *
*        KENTRY(2) : OBJECT TYPE L_MACROLIB IN READ ONLY MODE.         *
*                                                                      *
*     PROCEDURES CALLED: CALPOW, CALBEX, PWCPPF                        *
*                                                                      *
*-------------------------------------- AUTHOR: E. VARIN ; 21/11/95 ---*
*                                                                      *
*
*
      IMPLICIT NONE
      DOUBLE PRECISION  DFLOTT
      INTEGER    NENTRY
      REAL       FLOTT,BASE,PTOT,FSTH
      CHARACTER HENTRY(NENTRY)*12,HSIGN*12,TEXT*12,CGMPM*12
      INTEGER  IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
      INTEGER  IPRES,IPMAC,IPRES2,RDATA(6),RDATA2(6),ISTATE(13),
     +         ITYP,NZ,KCHAR(3),NITMA,NGRP,IPOWTA,ICPPF,
     +         IFLUR,IMIX,IDATA(7),IBPPF,IXN,IYN,
     +         IZONE,NZC,IG,IHFAC,IPOWR,NX,NY,NREG,
     +         IPCHAN,IT,IBZC,IPRT,IBASE,IFFK,ICPFZ,IVOLR
      INTEGER  KP,I1,NK,NCH,ILONG,NMIX,I
      DIMENSION IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
* PARAMETER VALIDATION.
*
      IF(NENTRY.LT.1) CALL XABORT('POWER: PARAMETER EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('POWER:'
     +   //' LINKED_LIST OR XSM_FILE EXPECTED')
      CALL LCMSIX(KENTRY(1),' ',0)
      CALL LCMGET(KENTRY(1),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF (HSIGN.NE.'L_MAP') CALL XABORT('POWER: OBJECT L_MAP EXPECTED')
      IPRES = KENTRY(1)
      IPRES2 = 0
      IPMAC = 0
      DO 5 I=2,NENTRY
        IF((IENTRY(I).EQ.1).OR.(IENTRY(I).EQ.2)) THEN
         CALL LCMSIX(KENTRY(I),' ',0)
         CALL LCMGET(KENTRY(I),'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF (HSIGN.EQ.'L_MAP') THEN
           IF(JENTRY(I).NE.2) CALL XABORT('POWER: READ-ONLY'
     +     //' MODE EXPECTED FOR OBJECT '//HSIGN//'.')
           IF (IPRES2.EQ.0) IPRES2=KENTRY(I)
         ELSEIF (HSIGN.EQ.'L_MACROLIB') THEN
           IF(JENTRY(I).NE.2) CALL XABORT('POWER: READ-ONL'
     +     //'Y MODE EXPECTED FOR OBJECT '//HSIGN//'.')
           IF (IPMAC.EQ.0) IPMAC=KENTRY(I)
         ENDIF
        ENDIF
   5  CONTINUE
      IF(IPMAC.EQ.0)
     2   CALL XABORT('POWER: OBJECT MISSING ')
*
*    DATA READING
*
      IPRT = 1
      KP = 0
      FSTH = 1.0
 10   CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
      IF(ITYP.NE.3) CALL XABORT('POWER: CHARACTER DATA EXPECTED')
      IF(TEXT.EQ.'POWER') THEN
* THERMAL POWER
        CALL REDGET(ITYP,NITMA,PTOT,TEXT,DFLOTT)
        IF(ITYP.NE.2) CALL XABORT('POWER: REAL DATA EXPECTED')
        KP = 1
      ELSEIF(TEXT.EQ.'FSTH') THEN
* THERMAL TO FISSION POWER RATIO
        CALL REDGET(ITYP,NITMA,FSTH,TEXT,DFLOTT)
        IF(ITYP.NE.2) CALL XABORT('POWER: REAL DATA EXPECTED')
      ELSEIF(TEXT.EQ.'TCOMB') THEN
        KP = 2
      ELSEIF(TEXT.EQ.'CPPF') THEN
        KP = 3
        IF(IPRES2.EQ.0) CALL XABORT('POWER: TWO L_MAP OBJECTS EXPECTED')
      ELSEIF(TEXT.EQ.'EDIT') THEN
        CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
        IF(ITYP.NE.1) CALL XABORT('POWER: INTEGER DATA EXPECTED')
        IPRT = NITMA
      ELSEIF(TEXT.EQ.';') THEN
        GOTO 20
      ELSE
        CALL XABORT('POWER: ILLEGAL KEYWORD '//TEXT)
      ENDIF
      GOTO 10
*
 20   CALL LCMSIX(IPRES,' ',0)
      CALL LCMGET(IPRES,'RES-DATA',RDATA)
      NK = RDATA(1)
      NCH = RDATA(2)
      NZC = RDATA(3)
      CALL LCMSIX(IPMAC,' ',0)
      CALL LCMGET(IPMAC,'IDATAXS',IDATA)
      NGRP = IDATA(1)
      IF( NGRP.NE.RDATA(4) )CALL XABORT('POWER: NUMBER OF ENERGY GROUPS'
     +   //' DIFFERENT IN MACROLIB THAN IN FUEL MAP')
      CALL LCMSIX(IPRES,'GEO:RES',1)
      CALL LCMGET(IPRES,'STATE',ISTATE)
      NREG = ISTATE(6)
      NX = ISTATE(3)
      NY = ISTATE(4)
      IF (ISTATE(1).EQ.7) THEN
        NZ = ISTATE(5)
      ELSE
        NZ = 1
      ENDIF
*
      IF(KP.GE.1) THEN
* COMPUTE FUEL POWERS NORMALIZED TO A TOTAL REACTOR POWER
*
*     RECOVER AVERAGE FUEL FLUXES
        CALL SETARA(BASE,NCH*NK*NGRP,IFLUR)
        CALL LCMSIX(IPRES,' ',0)
        CALL LCMLEN(IPRES,'FLUX-AV',ILONG,ITYP)
        IF(ILONG.NE.NCH*NK*NGRP)
     +    CALL XABORT('POWER: MODULE FLXAXC MUST BE EXECUTED BEFORE')
        CALL LCMGET(IPRES,'FLUX-AV',BASE(IFLUR))
        CALL SETARA(BASE,NCH*NK,IVOLR)
        CALL LCMGET(IPRES,'VOLUME',BASE(IVOLR))
        NMIX = IDATA(2)
        IF(NMIX.NE.NCH*NK) THEN
          CALL XABORT('POWER: MACROLIB OVER FUEL REQUIRED')
        ENDIF
        CALL SETARA(BASE,NMIX*NGRP,IHFAC)
        CALL LCMSIX(IPMAC,' ',0)
        DO 11 IG=1,NGRP
         WRITE(CGMPM,'(5HGROUP,I3,1H/,I3)') IG,NGRP
         CALL LCMSIX(IPMAC,CGMPM,1)
         CALL LCMLEN(IPMAC,'H-FACTORS',ILONG,ITYP)
         IF (ILONG.EQ.0)
     +    CALL XABORT('POWER: BLOCK H-FACTORS DO NOT EXIST')
         CALL LCMGET(IPMAC,'H-FACTORS',BASE(IHFAC+(IG-1)*NMIX))
         CALL LCMSIX(IPMAC,' ',2)
 11     CONTINUE
        CALL SETARA(BASE,NCH*NK,IPOWR)
        CALL CALPOW(PTOT,BASE(IFLUR),BASE(IHFAC),NGRP,NCH,NK,
     +              NMIX,BASE(IPOWR),BASE(IVOLR),FSTH)
        CALL LCMSIX(IPRES,' ',0)
        CALL LCMPUT(IPRES,'FLUX-AV',NCH*NK*NGRP,2,BASE(IFLUR))
        CALL LCMPUT(IPRES,'POWER-BUN',NCH*NK,2,BASE(IPOWR))
        CALL LCMPUT(IPRES,'POWER-TOTAL',1,2,PTOT)
        CALL RLSARA(BASE(IFLUR))
        CALL RLSARA(BASE(IVOLR))
        CALL RLSARA(BASE(IHFAC))
        IF(KP.EQ.2) THEN
*
*     COMPUTE AVERAGE EXIT BURNUP OR COMBUSTION RATE FOR TIME-AVERAGE
*
         IF( RDATA(5).LE.3 )THEN
          CALL SETARA(BASE,NZC*2,IFFK)
          CALL SETARA(BASE,NZC,IBZC)
          CALL LCMGET(IPRES,'BURN-ZONE',BASE(IBZC))
          CALL SETARA(IBASE,NCH,IZONE)
          CALL LCMGET(IPRES,'ZONE',IBASE(IZONE))
          CALL CALBEX(NCH,NK,NZC,BASE(IPOWR),IBASE(IZONE),BASE(IBZC),
     +               BASE(IFFK))
          CALL RLSARA(BASE(IFFK))
          CALL RLSARA(BASE(IBZC))
          CALL RLSARA(IBASE(IZONE))
         ENDIF
        ENDIF
        IF(KP.EQ.3) THEN
*
*      COMPUTATION OF CPPF AND BPPF
*
          CALL SETARA(BASE,NX,IT)
          CALL SETARA(IBASE,NX,IXN)
          CALL SETARA(IBASE,NY,IYN)
          CALL SETARA(IBASE,NREG,IMIX)
          CALL LCMSIX(IPRES,'GEO:RES',1)
          CALL LCMGET(IPRES,'MIX',IBASE(IMIX))
          CALL LCMSIX(IPRES,' ',0)
          CALL LCMGET(IPRES,'XNAME',IBASE(IXN))
          CALL LCMGET(IPRES,'YNAME',IBASE(IYN))
          CALL LCMSIX(IPRES2,' ',0)
          CALL LCMGET(IPRES2,'RES-DATA',RDATA2)
          CALL SETARA(IBASE,NCH,ICPFZ)
          CALL LCMGET(IPRES,'CPPF-ZONE',IBASE(ICPFZ))
          IF(RDATA2(2).NE.NCH) CALL XABORT('POWER: INVALID NUMBER OF '
     +     //'CHANNELS IN THE SECOND FUEL MAP')
          IF(RDATA2(1).NE.NK) CALL XABORT('POWER: INVALID NUMBER OF '
     +     //'BUNDLES IN THE SECOND FUEL MAP')
          IF(RDATA2(5).NE.3) CALL XABORT('POWER: TIME-AVERAGE L_MAP'
     +     //' EXPECTED')
          CALL SETARA(BASE,NCH*NK,IPOWTA)
          CALL LCMGET(IPRES2,'POWER-BUN',BASE(IPOWTA))
          CALL SETARA(BASE,NCH*NK,IBPPF)
          CALL SETARA(BASE,NCH,ICPPF)
          CALL PWCPPF(NCH,NK,BASE(IPOWR),BASE(IPOWTA),BASE(ICPPF),
     +                BASE(IBPPF),IBASE(IMIX),IBASE(IXN),IBASE(IYN),
     +                NX,NY,NREG,BASE(IT),
     +                IBASE(ICPFZ))
          CALL RLSARA(IBASE(ICPFZ))
          CALL RLSARA(BASE(ICPPF))
          CALL RLSARA(BASE(IBPPF))
          CALL RLSARA(BASE(IPOWTA))
          CALL RLSARA(IBASE(IXN))
          CALL RLSARA(IBASE(IYN))
          CALL RLSARA(BASE(IT))
          CALL RLSARA(IBASE(IMIX))
        ENDIF
        CALL RLSARA(BASE(IPOWR))
      ENDIF
*
*    POWER PRINTING
*
      IF(IPRT.GT.1) THEN
         IF (ISTATE(1).NE.5 .AND. ISTATE(1).NE.7)
     +     CALL XABORT('POWER: IMPOSSIBLE PRINT ON HEXAGONAL GEOMETRY')
         CALL SETARA(IBASE,NX,IXN)
         CALL SETARA(IBASE,NY,IYN)
         CALL SETARA(BASE,NCH*NK,IPOWR)
         CALL LCMGET(IPRES,'XNAME',IBASE(IXN))
         CALL LCMGET(IPRES,'YNAME',IBASE(IYN))
         CALL LCMLEN(IPRES,'POWER-BUN',ILONG,ITYP)
         IF(ILONG.NE.NCH*NK)
     +     CALL XABORT('POWER: BUNDLE POWERS MUST BE COMPUTED OR SET')
         CALL LCMGET(IPRES,'POWER-BUN',BASE(IPOWR))
         CALL LCMSIX(IPRES,' ',0)
         CALL LCMSIX(IPRES,'GEO:RES',1)
         CALL SETARA(IBASE,NREG,IMIX)
         CALL SETARA(BASE,NX,IT)
         CALL LCMGET(IPRES,'MIX',IBASE(IMIX))
         CALL LCMSIX(IPRES,' ',0)
         CALL SETARA(BASE,NCH,IPCHAN)
         CALL PRTPOW(NCH,NK,NX,NY,NZ,BASE(IPOWR),IBASE(IMIX),NREG,
     +              BASE(IPCHAN),BASE(IT),IBASE(IXN),IBASE(IYN),
     +              IPRT,FSTH)
         CALL RLSARA(BASE(IPCHAN))
         CALL RLSARA(BASE(IT))
         CALL RLSARA(IBASE(IMIX))
         CALL RLSARA(BASE(IPOWR))
         CALL RLSARA(IBASE(IXN))
         CALL RLSARA(IBASE(IYN))
        ENDIF
      RETURN
*
      END
