*DECK RESIND
      SUBROUTINE RESIND(IPRES,IPGEOM,NX,NY,NZ,LX,LY,LZ,NK,NCH,MESHX,
     +   MESHY,MESHZ,XX,YY,ZZ,MIX,INDEX,GEOTYP)
C
C THIS SUBOUTINE CALCULATES THE INDEX REFERENCE BETWEEN FUEL MAP
C AND SOLUTION GEOMETRY
C
C INPUTS -
C          IPGEOM : IPLIST OF L_GEOM
C          IPRES  : IPLIST OF L_MAP
C          NK  : NUMBER OF BUNDLES
C          NCH : NUMBER OF CHANNELS
C          NX : NUMBER OF NODES IN X DIRECTION IN MAP
C          NY : NUMBER OF NODES IN Y DIRECTION IN MAP
C          NZ : NUMBER OF NODES IN Z DIRECTION IN MAP
C          LX : NUMBER OF NODES IN X DIRECTION IN GEOM
C          LY : NUMBER OF NODES IN Y DIRECTION IN GEOM
C          LZ : NUMBER OF NODES IN Z DIRECTION IN GEOM
C          MESHX MESHY MESHZ : X Y AND Z COORDINATES IN MAP
C          XX YY ZZ : X Y AND Z COORDINATES IN GEOM
C          MIX :  MIX NUMBER FOR EACH NODE IN MAP
C OUTPUTS -
C          INDEX : INDEX OF NODES BETWEEN MAP AND GEOM
*
*-------------------------------------- AUTHOR: E. VARIN 25/08/95 ---
C
*
      IMPLICIT NONE
*
      INTEGER IPGEOM,IPRES,GEOTYP
      INTEGER NX,NY,NZ,LX,LY,LZ,NK,NCH,ISTATE(13)
      REAL MESHX(NX+1),MESHY(NY+1),MESHZ(NZ+1),
     +     XX(LX+1),YY(LY+1),ZZ(LZ+1)
      INTEGER MIX(NX*NY*NZ),INDEX(LX*LY*LZ)
*
C LOCAL
      INTEGER I,J,K,IO,JO,KO,I1,I2,J1,J2,K1,K2,IAM,IAMO,ILONG,
     +        ITYP,CPT,IMP,IGM
      LOGICAL FIND
*
      REAL X1,X2,Y1,Y2,Z1,Z2,PREC,DIFF
*
*     RECOVER INFORMATION
*
      CALL LCMLEN(IPRES,'GEO:RES',ILONG,ITYP)
      IF (ITYP.NE.0)
     +  CALL XABORT('RESIND: MAP GEOMETRY NEEDED')
      CALL LCMSIX(IPRES,'GEO:RES',1)
      CALL LCMGET(IPRES,'MIX',MIX)
      CALL LCMGET(IPRES,'MESHZ',MESHZ)
      CALL LCMGET(IPGEOM,'MESHZ',ZZ)
      IF(GEOTYP.EQ.7) THEN
       CALL LCMGET(IPRES,'MESHX',MESHX)
       CALL LCMGET(IPRES,'MESHY',MESHY)
       CALL LCMGET(IPGEOM,'MESHX',XX)
       CALL LCMGET(IPGEOM,'MESHY',YY)
      ENDIF
*
*     VERIFICATION OF MESH FIT BETWEEN FUEL MAP AND GEOMETRY
*
      PREC= 1.0E-4
      IF( GEOTYP.EQ.7 )THEN
        DO 20 IMP=1,NX+1
          FIND = .FALSE.
          DO 21 IGM=1,LX+1
            DIFF = ABS(MESHX(IMP)-XX(IGM))
            IF(DIFF.LT.PREC) THEN
              FIND = .TRUE.
            ENDIF
  21      CONTINUE
          IF(.NOT.FIND) THEN
            WRITE(6,*) ' MESH IN X DIRECTION IN RESEAU ',MESHX(IMP)
            CALL XABORT('RESIND: THIS MESH MUST BE IN GEOM MESH')
          ENDIF
  20    CONTINUE
        DO 22 IMP=1,NY+1
          FIND = .FALSE.
          DO 23 IGM=1,LY+1
            DIFF = ABS(MESHY(IMP)-YY(IGM))
            IF(DIFF.LT.PREC) THEN
              FIND = .TRUE.
            ENDIF
  23      CONTINUE
          IF(.NOT.FIND) THEN
             WRITE(6,*) ' MESH IN Y DIRECTION IN RESEAU ',MESHY(IMP)
             CALL XABORT('RESIND: THIS MESH MUST BE IN GEOM MESH')
          ENDIF
  22    CONTINUE
      ENDIF
      DO 24 IMP=1,NZ+1
       FIND = .FALSE.
       DO 25 IGM=1,LZ+1
         DIFF = ABS(MESHZ(IMP)-ZZ(IGM))
         IF(DIFF.LT.PREC) THEN
           FIND = .TRUE.
         ENDIF
  25   CONTINUE
       IF(.NOT.FIND) THEN
          WRITE(6,*) ' MESH IN Z DIRECTION IN RESEAU ',MESHZ(IMP)
          CALL XABORT('RESIND: THIS MESH MUST BE IN GEOM MESH')
       ENDIF
  24  CONTINUE
*
*     RENUMBERING OF MIX RECORD IN FUEL MAP GEOMETRY
*
      CPT = 0
      DO 30 I=1,NX*NY*NZ
         IF (MIX(I).NE.0) THEN
            CPT = CPT + 1
            MIX(I) = CPT
         ENDIF
  30  CONTINUE
      IF (CPT.NE.NK*NCH) THEN
       WRITE(6,*) ' NK, NCH ',NK,NCH,' CPT ',CPT
       CALL XABORT('RESIND: WRONG NUMBER OF BUNDLES OR CHANNELS')
      ENDIF
      CALL LCMGET(IPRES,'STATE',ISTATE)
      ISTATE(7) = CPT
      CALL LCMPUT(IPRES,'STATE',13,1,ISTATE)
      CALL LCMPUT(IPRES,'MIX',NX*NY*NZ,1,MIX)
      CALL LCMSIX(IPRES,' ',0)
*
      DO 10 K=1,NZ
       DO 10 J=1,NY
       DO 10 I=1,NX
           IAM = (K-1)*NX*NY + (J-1)*NX + I
           IF( GEOTYP.EQ.7 )THEN
             X1=MESHX(I)
             X2=MESHX(I+1)
             Y1=MESHY(J)
             Y2=MESHY(J+1)
*
             DO 105 IO=1,LX
               IF(X1.GE.XX(IO) .AND. X1.LE.XX(IO+1)) THEN
                 I1=IO
               ENDIF
               IF(X2.GE.XX(IO) .AND. X2.LE.XX(IO+1)) THEN
                 I2=IO
                 GO TO 110
               ENDIF
 105         CONTINUE
 110         CONTINUE
*
             DO 115 JO=1,LY
               IF(Y1.GE.YY(JO) .AND. Y1.LE.YY(JO+1)) THEN
                 J1=JO
               ENDIF
               IF(Y2.GE.YY(JO) .AND. Y2.LE.YY(JO+1)) THEN
                 J2=JO
                 GO TO 120
               ENDIF
 115         CONTINUE
 120         CONTINUE
           ELSE
             I1 = I
             I2 = I
             J1 = 1
             J2 = 1
           ENDIF
*
           Z1=MESHZ(K)
           Z2=MESHZ(K+1)
           DO 125 KO=1,LZ
            IF(Z1.GE.ZZ(KO) .AND. Z1.LE.ZZ(KO+1)) THEN
              K1=KO
            ENDIF
            IF(Z2.GE.ZZ(KO) .AND. Z2.LE.ZZ(KO+1)) THEN
              K2=KO
              GOTO 130
            ENDIF
 125       CONTINUE
 130       CONTINUE
*
*
           DO 140 KO=K1,K2
           DO 140 JO=J1,J2
           DO 140 IO=I1,I2
             IAMO = (KO-1)*LX*LY + (JO-1)*LX + IO
             INDEX(IAMO) = MIX(IAM)
 140       CONTINUE
 10   CONTINUE
*
      RETURN
      END
