*DECK SMDRPD
      SUBROUTINE SMDRPD (DT,RESPON,REF,KC,PDD,REGUL,PDF,PDT,NDDG,
     +                   AP,BP)
*
*
*-----------------------------------------------------------------------
*
*     THIS ROUTINE COMPUTES PLATINUM DETECTOR READING WITH DELAY
*     TIME CONSIDERATION AND NORMALIZED TO A REFERENCE VALUE
*
*     CETTE SOUS-ROUTINE CALCULE LA RESPONSE D'UN DETECTEUR PLATINE
*     NORMALISE PAR UNE LECTURE DE REFERENCE
*
*     INPUT:
*             NDDG    : NOMBRE DE CONSTANTES DE TEMPS DU FILTRE
*             RESPON  : REAL
*                       FLUX AT DETECTOR SITE
*             REF     : REAL
*                       REFERENCE VALUE
*             PDD     : REAL
*                       PREVIOUS VALUES OF THE FILTER COMPOSANTS.
*                       DIM: NDDG
*             PDT     : REAL
*                       INVERSE DELAY TIME CONSTANTS. DIM: NDDG
*             PDF     : REAL
*                       DELAY FRACTIONS. DIM: NDDG
*             DT      : REAL
*                       SIMULATION TIME STEP
*             KC      : INTEGER
*                       = 0 REFERENCE CALCULATION
*                       = 1 NORMAL CALCULATION
*             REGUL   : LOGICAL TRUE IF REACTOR REGULATING DETECTOR
*
*     OUTPUT:
*             RESPON  : REAL
*                       DETECTOR REPONSE
*             PDD     : REAL
*                       NEW VALUES OF THE FILTER COMPOSANTS. DIM: NDDG
*
*
*     ROUTINE FROM SMOKIN-G2, INCORPORATED IN DONJON
*
*
*---------------------------------------------------------------------
*
*
      INTEGER  NDDG,I
      REAL RESPON,REF,DT
      REAL AEP,AP(NDDG),BP(NDDG),PDT(NDDG),PDF(NDDG+1),PDD(NDDG),PDO
      LOGICAL  REGUL
*
*     FLUX AT DETECTOR SITE
*     ---------------------
      IF ( KC.EQ.0) THEN
*     REFERENCE CALCULATION
*
          PDO = RESPON
          IF (REGUL) THEN
             CALL SRFILT(PDO,RESPON,RESPON,25.,DT,1)
          ENDIF
          IF (PDO.EQ.0.0) THEN
            PDO = 1.
            WRITE(6,*)'===> DETECTOR HAS AN INITIAL ZERO VALUE <==='
          ENDIF
          REF = PDO
      ENDIF
*
*     NORMALIZATION TO REFERENCE VALUE
*
      RESPON = RESPON/REF
*
      DO 10 I = 1 , NDDG
*
        AEP   = DT*PDT(I)
        AP(I) = 1.0
*
        IF (AEP.GT.20.0) THEN
           AEP = 20.0
        ENDIF
        IF (AEP.GT.1.0E-6) THEN
           AP(I) = EXP ( -AEP )
        ENDIF
        BP(I) = 1.0 - AP(I)
*
10    CONTINUE
*
      IF (KC.EQ.0) THEN
*
*     INITIALISATION AT REFERENCE CALCULATION
*     ---------------------------------------
*
         DO 20 I = 1 , NDDG
           PDD(I) = PDF(I+1) * RESPON
20       CONTINUE
      ENDIF
*
*     CALCULATION OF DETECTOR RESPONSE
*     --------------------------------
*
      PDO = 0.0
      PDO = PDF(1)*RESPON
      DO 30 I = 1 , NDDG
           PDD(I) = AP(I)*PDD(I) + BP(I)*PDF(I+1)*RESPON
           PDO    = PDO + PDD(I)
 30   CONTINUE
*
      RESPON = PDO
*
      RETURN
      END
