*DECK SORKDV
      SUBROUTINE SORKDV(IPGPT,IPTRK,IPSYS0,IPSYSP,IPFLU0,REC,NEL,NUN,
     1 NGRP,EVECT,ADECT,SOUR)
*
*-----------------------------------------------------------------------
*
* DRIVER FOR THE SOURCE CALCULATION ON RAYLEIGH FUNCTIONAL
*
* INPUT PARAMETERS:
*  IPGPT  : L_GPT POINTER TO THE SOURCE TERM INFORMATION.
*  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IPSYS0 : L_SYSTEM POINTER TO UNPERTURBED SYSTEM MATRICES.
*  IPSYSP : L_SYSTEM POINTER TO DELTA SYSTEM MATRICES.
*  IPFLU0 : L_FLUX POINTER TO THE UNPERTURBED SOLUTION.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : TOTAL NUMBER OF UNKNOWNS PER GROUP.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*
* OUTPUT PARAMETER:
*  IPGPT  : L_GPT POINTER TO THE SOURCE TERM INFORMATION.
*
* SCRATCH STORAGE:
*  EVECT  : DIMENSION EVECT(NUN,NGRP)
*  ADECT  : DIMENSION ADECT(NUN,NGRP)
*  SOUR   : DIMENSION SOUR(NUN,NGRP)
*
* INFORMATION RECOVERED FROM L_SYSTEM AT IPSYS0:
*  'A  1  1'  : SYSTEM MATRIX RELATED TO FAST LEAKAGE AND REMOVAL.
*  'A  2  2'  : SYSTEM MATRIX RELATED TO THERMAL LEAKAGE AND REMOVAL.
*  'A  1  2'  : SYSTEM MATRIX RELATED TO UP-SCATTERING.
*  'A  2  1'  : SYSTEM MATRIX RELATED TO DOWN-SCATTERING.
*  'B  1  1'  : SYSTEM MATRIX RELATED TO FAST FISSION.
*  'B  1  2'  : SYSTEM MATRIX RELATED TO THERMAL FISSION.
*
* INFORMATION RECOVERED FROM L_SYSTEM AT IPSYSP:
*  'A  1  1' : DELTA-SYSTEM MATRIX RELATED TO FAST LEAKAGE AND REMOVAL.
*  'A  2  2' : DELTA-SYSTEM MATRIX RELATED TO THERMAL LEAKAGE AND
*              REMOVAL.
*  'A  1  2' : DELTA-SYSTEM MATRIX RELATED TO UP-SCATTERING.
*  'A  2  1' : DELTA-SYSTEM MATRIX RELATED TO DOWN-SCATTERING.
*  'B  1  1' : DELTA-SYSTEM MATRIX RELATED TO FAST FISSION.
*  'B  1  2' : DELTA-SYSTEM MATRIX RELATED TO THERMAL FISSION.
*
*---------------------------------------------- A. HEBERT 13/07/93 -----
*---------------------------------------------- E. VARIN  15/05/96 -----
*
      LOGICAL CLASIC,ADJ,REC
      DOUBLE PRECISION DFLOTT
      CHARACTER TEXT4*4,TEXT12*12
      DIMENSION EVECT(NUN,*),ADECT(NUN,*),
     1 SOUR(NUN,*),IPARM(3)
      COMMON BASE(1)
*
* READ THE INPUT DATA.
*     DEFAULT OPTIONS.
      N1 = 1
      IREFL = 1
      IMPX=1
      CLASIC=.FALSE.
      ADJ=.FALSE.
      NSRC = 1
*
   10 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.EQ.10) GO TO 160
      IF(INDIC.NE.3) CALL XABORT('SORKDV: CHARACTER DATA EXPECTED.')
*
   40 IF(TEXT4.NE.'EDIT') GO TO 50
      CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('SORKDV: INTEGER DATA EXPECTED.')
      GO TO 10
*
   50 IF(TEXT4.NE.'ADJ') GO TO 55
      ADJ=.TRUE.
      GO TO 10
*
   55 IF(TEXT4.NE.'NSRC') GO TO 60
      CALL REDGET(INDIC,NSRC,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('SORKDV: INTEGER DATA EXPECTED.')
      IF(NSRC.LE.0) CALL XABORT('SORKDV: INVALID VALUE OF NSRC')
      GO TO 10
*
   60 IF(TEXT4.NE.'CLAS') GO TO 65
      CLASIC=.TRUE.
      GO TO 10
*
   65 IF(TEXT4.NE.'CHAN') GO TO 70
      N1 = 380
      CALL REDGET(INDIC,IREFL,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('SORKDV: INTEGER DATA EXPECTED.')
      IF(IREFL.LE.0) CALL XABORT('SORKDV: INVALID VALUE OF IREFL')
      GO TO 10
*
   70 IF(TEXT4.NE.'ZONE') GO TO 150
      N1 = 14
      CALL REDGET(INDIC,IREFL,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.1) CALL XABORT('SORKDV: INTEGER DATA EXPECTED.')
      IF(IREFL.LE.0) CALL XABORT('SORKDV: INVALID VALUE OF IREFL')
      GO TO 10
*
  150 IF(TEXT4.NE.';') CALL XABORT('SORKDV: ; EXPECTED.')
*
* CHECK LENGTH OF DELKEF (N1)
      IF( IREFL.GT.N1) THEN
          CALL XABORT('SORKDV: INVALID VALUE OF IREFL')
      ELSEIF( NSRC.NE.1 .AND. N1.NE.1 )THEN
          CALL XABORT('SORKDV: NO NSRC FOR DELKEF CALCULATION')
      ENDIF
      CALL SETARA(BASE,N1,IDKEF)
      DO 152 I=1,N1
        BASE(IDKEF+I-1) = 0.0
  152 CONTINUE
      IF(REC) THEN
        CALL LCMLEN(IPGPT,'DELTA-KEFF',ILONG,ITYP)
        IF( ILONG.NE.0 )THEN
          IF( ILONG.NE.N1 )CALL XABORT('ILLEGAL VALUE
     +                                OF N1 WITH OBJECT')
          CALL LCMGET(IPGPT,'DELTA-KEFF',BASE(IDKEF))
        ENDIF
      ENDIF
*
* RECOVER UNPERTURBED K-EFFECTIVE AND FLUXES.
  160 CALL LCMGET(IPFLU0,'K-EFFECTIVE',FKEFF)
      DO 170 IGR=1,NGRP
        WRITE(TEXT12,'(4HFLUX,I3)') IGR
        CALL LCMGET(IPFLU0,TEXT12,EVECT(1,IGR))
        WRITE(TEXT12,'(5HAFLUX,I3)') IGR
        CALL LCMGET(IPFLU0,TEXT12,ADECT(1,IGR))
  170 CONTINUE
*
*
* SOURCE CALCULATION.
*
*     COMPUTE THE DIRECT OR ADJOINT FIXED SOURCES.
      DO 180 ISRC=1,NSRC
        CALL SETARA(BASE,NUN,IAGAR)
        CALL SETARA(BASE,NUN,IWORK)
        CALL DELPER(IPTRK,IPSYS0,IPSYSP,ADJ,NUN,NGRP,FKEFF,IMPX,
     1     EVECT,ADECT,DELKEF,SOUR,BASE(IAGAR),BASE(IWORK))
        CALL RLSARA(BASE(IWORK))
        CALL RLSARA(BASE(IAGAR))
*
        IF(.NOT.CLASIC) THEN
          DO 200 IGR=1,NGRP
            IF(ADJ) THEN
              WRITE(TEXT12,'(5HASOUR,I3,I3)') IGR,ISRC
            ELSE
              WRITE(TEXT12,'(5HDSOUR,I3,I3)') IGR,ISRC
            ENDIF
            CALL LCMPUT(IPGPT,TEXT12,NUN,2,SOUR(1,IGR))
  200     CONTINUE
        ENDIF
  180 CONTINUE
*
      BASE(IDKEF+IREFL-1) = DELKEF
      CALL LCMPUT(IPGPT,'DELTA-KEFF',N1,2,BASE(IDKEF))
      CALL RLSARA(BASE(IDKEF))
      IPARM(1) = NGRP
      IPARM(2) = NUN
      IPARM(3) = NSRC
      CALL LCMPUT(IPGPT,'PARAM',3,1,IPARM)
      IF( IMPX.GT.1 )CALL LCMLIB(IPGPT)
      RETURN
      END
