*DECK SURVEY
      SUBROUTINE SURVEY(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        *******************************************************       *
*        * MODULE TO COMPUTE POWERS IN INTEREST ZONES OF THE   *       *
*        *  FUEL LATTICE                                       *       *
*        *                                                     *       *
*        * MODULE DE CALCUL DES PUISSANCES DANS DES ZONES      *       *
*        * DE CONTROLE SUR LE RESEAU DE COMBUSTIBLE            *       *
*        *                                                     *       *
*        *******************************************************       *
*                                                                      *
*     INPUT/OUTPUT PARAMETERS:                                         *
*      NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.    *
*      HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.         *
*      IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;*
*               =4 SEQUENTIAL ASCII FILE.                              *
*      JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.                 *
*               =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;  *
*               =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.  *
*      KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.     *
*                                                                      *
*      DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),         *
*                KENTRY(NENTRY)                                        *
*                                                                      *
*     REQUIRED LINKED LISTS ARE:                                       *
*        KENTRY(1) : OBJECT TYPE L_MAP IN MODIFICATION MODE            *
*                                                                      *
*     PROCEDURES CALLED :  SURPOW                                      *
*                                                                      *
*-------------------------------------- AUTHOR: E. VARIN ; 01/05/96 ---*
*                                                                      *
*
*
      IMPLICIT NONE
      DOUBLE PRECISION  DFLOTT
      INTEGER    NENTRY
      REAL       FLOTT,BASE,FSTH
      CHARACTER HENTRY(NENTRY)*12,HSIGN*12,TEXT*12
      INTEGER  IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
      INTEGER  IPRES,RDATA(6),ITYP,KCHAR(3),NITMA,
     +         IZONE,IPOWR,IZCNTL,IDX,NZONES,IPSUR,IBASE,IPRT
      INTEGER  KP,I1,NK,NCH,I,K
      DIMENSION IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
* PARAMETER VALIDATION.
*
      IF(NENTRY.LT.1) CALL XABORT('SURVEY: PARAMETER EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('SURVEY:'
     +   //' LINKED_LIST OR XSM_FILE EXPECTED')
      CALL LCMSIX(KENTRY(1),' ',0)
      CALL LCMGET(KENTRY(1),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF (HSIGN.NE.'L_MAP') CALL XABORT('SURVEY: OBJECT L_MAP EXPECTED')
      IPRES = KENTRY(1)
*
*    DATA READING
*
      IPRT = 1
      KP = 1
      FSTH = 1.0
 10   CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
      IF(ITYP.NE.3) CALL XABORT('SURVEY: CHARACTER DATA EXPECTED')
      IF(TEXT.EQ.'FSTH') THEN
* THERMAL TO FISSION POWER RATIO
        CALL REDGET(ITYP,NITMA,FSTH,TEXT,DFLOTT)
        IF(ITYP.NE.2) CALL XABORT('POWER: REAL DATA EXPECTED')
      ELSEIF(TEXT.EQ.'POW-CONTR') THEN
        KP = 1
      ELSEIF(TEXT.EQ.'POW-CHAN') THEN
        KP = 2
      ELSEIF(TEXT.EQ.'EDIT') THEN
        CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
        IF(ITYP.NE.1) CALL XABORT('SURVEY: INTEGER DATA EXPECTED')
        IPRT = NITMA
      ELSEIF(TEXT.EQ.';') THEN
        GOTO 20
      ELSE
        CALL XABORT('SURVEY: ILLEGAL KEYWORD '//TEXT)
      ENDIF
      GOTO 10
*
 20   CALL LCMSIX(IPRES,' ',0)
      CALL LCMGET(IPRES,'RES-DATA',RDATA)
      NK = RDATA(1)
      NCH = RDATA(2)
      CALL SETARA(BASE,NCH*NK,IPOWR)
      CALL SETARA(IBASE,NCH,IZCNTL)
      CALL LCMGET(IPRES,'POWER-BUN',BASE(IPOWR))
      CALL LCMGET(IPRES,'CONTR-ZONE',IBASE(IZCNTL))
*
* SEARCH FOR THE NUMBER OF INTEREST ZONE
* AS THE MAXIMUM OF CONTR-ZONE VECTOR
*
      NZONES = 0
*
      DO 30 I=1,NCH
         IDX = IZCNTL+I-1
         NZONES = MAX(NZONES,IBASE(IDX))
  30  CONTINUE
*
* INCREASE VECTOR OF ZONE NUMBER TO THE TOTAL NUMBER OF
* BUNDLES (NCH*NK) AND IF REQUIRED MODIFY NUMBER OF ZONES
* FOR HALF CORE IN Z DIRECTION RENUMBERING
*
      CALL SETARA(IBASE,NCH*NK,IZONE)
      DO 40 I=1,NCH
         DO 41 K=1,NK
            IDX = IZONE + (K-1)*NCH + I - 1
            IBASE(IDX) = IBASE(IZCNTL+I-1)
   41    CONTINUE
*
*  NUMBERING TO 2*NZONES IF POW-CONTR OPTION VALID
         IF( KP.EQ.1 )THEN
            DO 42 K=NK/2+1,NK
              IDX = IZONE + (K-1)*NCH + I - 1
              IBASE(IDX) = IBASE(IZCNTL+I-1) + NZONES
   42       CONTINUE
         ENDIF
*
  40  CONTINUE
      IF( KP.EQ.1 )NZONES=NZONES*2
*
* COMPUTE POWERS AND FRACTIONS OF FULL POWER
*
      CALL SETARA(BASE,NZONES,IPSUR)
      CALL SURPOW(NK,NCH,NZONES,IBASE(IZONE),BASE(IPOWR),
     +            BASE(IPSUR),IPRT,FSTH)
*
      CALL LCMPUT(IPRES,'POW-SURV',NZONES,2,BASE(IPSUR))
      CALL RLSARA(BASE(IPSUR))
      CALL RLSARA(IBASE(IZONE))
      CALL RLSARA(IBASE(IZCNTL))
      CALL RLSARA(BASE(IPOWR))
      RETURN
*
      END
