*DECK TAWCAL
      SUBROUTINE TAWCAL(IPRES,IPTAB,NCH,NK,NZC,NBURN,NGRP,NS,
     +    NTYP,INAMT,ITYP,W1,W2,ZONE,NW,BZC,FLX,PSI,HFAC,ZH,BURNUP,
     +    CW1,CW2,DW,NITZ,INDEX,NX,NY,NZ,MIX,IPRT,NSSS,ITAB)
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        *******************************************************       *
*        * MODULE OF CALCULATION OF THE BURNUP INTEGRATION     *       *
*        * LIMITS FOR THE FUEL XS COMPUTATION AVERAGED IN TIME *       *
*        * ACCORDING TO A BI DIRECTIONAL REFUELING STRATEGY    *       *
*        * FOR A 3D GEOMTRY                                    *       *
*        *******************************************************       *
*                                                                      *
*                                                                      *
*        *******************************************************       *
*        * MODULE DE CALCUL DES LIMITES D'INTEGRATION DES      *       *
*        * SECTIONS EFFICACES DU COMBUSTIBLE EN MOYENNE        *       *
*        * DANS LE TEMPS SELON UN MODELE DE RECHARGEMENT       *       *
*        * BIDIRECTIONNEL ET DISCRET POUR UNE GEOMETRIE 3D     *       *
*        *******************************************************       *
*                                                                      *
* INPUT PARAMETERS:
*  IPRES  : ADDRESS OF THE MAP LINKED LIST OR XSM FILE.
*  IPTAB  : ADDRESS OF THE XS TABLE LINKED LIST OR XSM FILE.
*  NBURN  : NUMBER OF TABULATED BURNUP STEPS.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NZC    : NUMBER OF FUEL ZONES.
*  NCH    : NUMBER OF FUEL CHANNELS.
*  NK     : NUMBER OF FUEL BUNDLES.
*  NITZ   : NUMBER OF AXIAL ITERATIONS.
*  NS     : REFUELING SCHEME. NS(NZC)
*  ZONE   : COMBUSTION ZONE INDICES. ZONE(NCH)
*  BZC    : AVERAGE EXIT BURNUPS PER COMBUSTION ZONE. BZC(NZC)
*  FLX    : AXIAL FLUXES. FLX(NCH,NK,NGRP)
*  HFAC   : H-FACTORS. HFACT(NK,NGRP)
*  ZH     : H-FACTOR TABLES. ZH(NBURN,NGRP,2)
*  BURNUP : VECTOR OF BURNUP STEPS. BURNUP(NBURN)
*
*
* OUTPUT PARAMETERS:
*  CW1   : INITIAL BURNUPS. DIMENSION W1(NCH,NK)
*  CW2   : FINAL BURNUPS. DIMENSION W2(NCH,NK)
*
* SCRATCH STORAGE:
*  NW      : DIMENSION NW(NK)
*  PSI     : DIMENSION PSI(NK)
*  W1      : DIMENSION W1(NK)
*  W2      : DIMENSION W2(NK)
*  DW      : DIMENSION DW(NK)
*
*
*     ROUTINE FROM OPTEX-4, INCORPORATED IN DONJON
*     BY E.VARIN 96/09/10
*
*----------------:-------------------------:---------------------------*
*                                                                      *
*
      IMPLICIT NONE
*
      INTEGER   NCH,NK,NZC,NGRP,NBURN,IPRES,IPTAB,NITZ,NTYP,NX,NY,NZ,
     +          ITAB,IGR,IHDR,ITYP(NTYP),IT,ITF,KDF,INAMT(3*NTYP)
      CHARACTER NAMTYP*12
      REAL      W1(NK), W2(NK), DW(NK),PSI(NK),HFAC(NGRP,NK),
     +          FLX(NCH,NK,NGRP),CW1(NCH,NK), CW2(NCH,NK),
     +          ZH(NBURN,NGRP,2,NTYP),BURNUP(NBURN),BZC(NZC)
*
      INTEGER   NS(NZC),NW(NK),ZONE(NCH),ICH,K,ITZ,NSI,NSH,
     +          IADW,IBASE(1),INDEX(NCH,NK),IG,I,MIX(NX*NY*NZ),
     +          NJ,IPRT,ICPT,RS,TT,NF,NF1,NSSS(NCH)
      REAL      BASE,BUNPOW,SUM,BE,A1,A2,AA
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
*     RECOVERING OF INFORMATION
*
      CALL LCMSIX(IPRES,'GEO:RES',1)
      CALL LCMGET(IPRES,'MIX',MIX)
      CALL LCMSIX(IPRES,' ',0)
      CALL LCMGET(IPRES,'ZONE',ZONE)
      CALL LCMGET(IPRES,'INDEX',INDEX)
      CALL LCMGET(IPRES,'BURN-ZONE',BZC)
      CALL LCMGET(IPRES,'FLUX-AX',FLX)
      CALL LCMSIX(IPTAB,' ',0)
      WRITE(NAMTYP,'(3A4)') (INAMT(I),I=1,3)
      CALL LCMSIX(IPTAB,NAMTYP,1)
      CALL LCMGET(IPTAB,'BURNUP',BURNUP)
      DO 13 IT=1,NTYP
        CALL LCMSIX(IPTAB,' ',0)
        WRITE(NAMTYP,'(3A4)') (INAMT((IT-1)*3+I),I=1,3)
        CALL LCMSIX(IPTAB,NAMTYP,1)
        IF(ITAB.EQ.1) THEN
          CALL LCMGET(IPTAB,'H-FACTORS',ZH(1,1,1,IT))
          CALL SETARA(BASE,2*NBURN,IHDR)
          DO 20 IGR=1,NGRP
            DO 21 I=1,NBURN
              ZH(I,IGR,2,IT)=ZH(I,IGR,1,IT)
   21       CONTINUE
            CALL ALDERV(NBURN,BURNUP,ZH(1,IGR,2,IT),BASE(IHDR))
   20     CONTINUE
          CALL RLSARA(BASE(IHDR))
        ELSE
          CALL LCMGET(IPTAB,'H-FACTORS',ZH(1,1,1,IT))
        ENDIF
   13 CONTINUE
*
*---- ITERATION OVER NCH CHANNELS
*
*
      CALL SETARA(BASE,NK,IADW)
*
      ICPT = 0
      DO 10 I=1,NX*NY
        IF(MIX(I).EQ.0) GOTO 10
*
         ICPT = ICPT + 1
         ICH = MIX(I)
         A1 = I
         A2 = NX
         AA = A1/A2
         NJ = IFIX(AA-0.0010)
         BE=BZC(ZONE(ICH))
         NSI=NS(ZONE(ICH))
         IF(NSI.LE.NK) THEN
           IF(ICPT.EQ.1) THEN
              RS = (NJ+I-1)/ 2
              TT = (NJ+I-1) - 2*RS
              NF = 0
              IF(TT.EQ.0) NF = -1
           ENDIF
           NF1 = NF + NJ + I - 1
           NSSS(ICH)=((-1)**NF1)*NSI
         ENDIF
*
*
*    ESTABLISH REFUELING VECTOR
*
         CALL FLSCHM (NSI,NW,0,NK,ICH)
         NSH = 0
         DO 1 K=1,NK
           IF(NW(K).EQ.0) NSH = NSH+1
 1       CONTINUE
         IF(NSH.EQ.0) CALL XABORT('TAWCAL: PROBLEME DE NSH')
*
         SUM = 0.0
         DO 15 K=1,NK
          DO 15 IG=1,NGRP
            SUM = SUM + FLX(ICH,K,IG)
   15    CONTINUE
*
         DO 16 K=1,NK
          PSI(K) = 0.0
          DO 17 IG=1,NGRP
            PSI(K) = PSI(K) + FLX(ICH,K,IG)
   17     CONTINUE
          PSI(K) = PSI(K)/SUM
   16    CONTINUE
*
*------- COMPUTE INTEGRATION LIMITS FOR XS ALONG Z AXIS
         CALL INTLIM(NK,NSH,BE,PSI,DW,W1,W2,BASE(IADW),NW,NSSS(ICH))
*
*------- ITERATION OVER AXIAL FLUX SHAPE AND BURNUP INCREMENT
*
         DO 28 ITZ=1,NITZ
            SUM = 0.0
            DO 26 K=1,NK
              KDF=0
              DO 113 IT=1,NTYP
                IF(INDEX(ICH,K).EQ.ITYP(IT)) THEN
                    ITF=IT
                    KDF=1
                ENDIF
  113        CONTINUE
             IF(KDF.EQ.0) CALL XABORT('TAWCAL: WRONG NUMBER OF INDEX')
             CALL TAINTG(NGRP,NBURN,W1(K),W2(K),BURNUP,
     +                   HFAC(1,K),ZH(1,1,1,ITF))
*
             DO 25 IG=1,NGRP
               SUM = SUM + HFAC(IG,K)*FLX(ICH,K,IG)
   25        CONTINUE
   26       CONTINUE
*
            DO 27 K=1,NK
               BUNPOW = 0.0
             DO 24 IG=1,NGRP
               BUNPOW = BUNPOW + HFAC(IG,K)*FLX(ICH,K,IG)
   24        CONTINUE
             PSI(K) = BUNPOW/SUM
  27        CONTINUE
*
            CALL INTLIM(NK,NSH,BE,PSI,DW,W1,W2,BASE(IADW),NW,NSSS(ICH))
 28      CONTINUE
*
*------- RECOVER INTEGRATION LIMITS
*
         DO 29 K=1,NK
            CW1(ICH,K) = W1(K)
            CW2(ICH,K) = W2(K)
 29      CONTINUE
         IF( IPRT.GT.3 )THEN
           WRITE(6,*) ' CHANNEL ',ICH
           WRITE(6,110) ' B0 ',(W1(K),K=1,NK)
           WRITE(6,110) ' B1 ',(W2(K),K=1,NK)
         ENDIF
*
 10   CONTINUE
      CALL RLSARA(BASE(IADW))
*
      CALL LCMPUT(IPRES,'BURN-DEB',NK*NCH,2,CW1)
      CALL LCMPUT(IPRES,'BURN-END',NK*NCH,2,CW2)
      CALL LCMPUT(IPRES,'REFUEL-SCH',NCH,1,NSSS)
*
      RETURN
 110  FORMAT(5X,A4,12(F8.1,1X))
      END
