*DECK TRIASD
      SUBROUTINE TRIASD (IELEM,ICHX,IDIM,NEL,NUN,CYLIND,SGD,VOL,MAT,
     1 KN,VEC,IGAR)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF A DIAGONAL SYSTEM MATRIX CORRESPONDING TO A SINGLE CROSS
* SECTION TYPE (DUAL CASES).
*
* INPUT PARAMETERS:
*  IELEM  : DEGREE OF THE LAGRANGIAN FINITE ELEMENTS.
*  ICHX   : TYPE OF DISCRETIZATION METHOD.
*           =2: DUAL FINITE ELEMENT APPROXIMATIONS;
*           =3: NODAL COLLOCATION METHOD WITH FULL TENSORIAL PRODUCTS;
*           =4: NODAL COLLOCATION METHOD WITH SERENDIPITY APPROXIMATION.
*  IDIM   : NUMBER OF DIMENSIONS.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : TOTAL NUMBER OF UNKNOWNS PER GROUP.
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRIES.
*  SGD    : CROSS SECTION PER MATERIAL MIXTURE. DIMENSION SGD(IR)
*           WHERE IR IS THE MAXIMUM NUMBER OF MATERIAL MIXTURES.
*  VOL    : VOLUMES. DIMENSION VOL(NEL)
*  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*           DIMENSION MAT(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(7*NEL)
*
* OUTPUT PARAMETER:
*  VEC   : DIAGONAL SYSTEM MATRIX. DIMENSION VEC(NUN)
*
* SCRATCH STORAGE:
*  IGAR  : DIMENSION IGAR(NEL)
*
* NOTE: VECTOR VEC SHOULD BE INITIALIZED BY THE CALLING PROGRAM.
*
*------------------------------------- AUTHOR: A. HEBERT ; 08/09/87 ----
*
      LOGICAL CYLIND
      DIMENSION VOL(*),MAT(*),SGD(*),KN(*),VEC(*),IGAR(*)
*
      IORD(J,K,L,LL,IEL,IW)=(IEL*L+K)*LL*IEL+(1+IEL*(IW-1))+J
*
      IORL(J,K,L,LL,IEL,IW)=
     1 1+LL*(L*(IEL*(IEL+1))/2-(L*(L-1)*(3*IEL-L+2))/6
     2 +K*(IEL-L)-(K*(K-1))/2)+(IEL-K-L)*(IW-1)+J
*
      IF(ICHX.EQ.2) THEN
*        DUAL FINITE ELEMENT METHOD.
         NUM1=0
         DO 30 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 30
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 20
         DO 10 K3=0,IELEM-1
         DO 10 K2=0,IELEM-1
         DO 10 K1=0,IELEM-1
         IND1=KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1
   10    VEC(IND1)=VEC(IND1)+VOL0*SGD(L)
   20    NUM1=NUM1+7
   30    CONTINUE
      ELSE IF(ICHX.EQ.3) THEN
*        NODAL COLLOCATION METHOD WITH FULL TENSORIAL PRODUCTS.
         LNUN=0
         DO 40 K=1,NEL
         IF(MAT(K).EQ.0) GO TO 40
         LNUN=LNUN+1
         IGAR(K)=LNUN
   40    CONTINUE
*
         DO 70 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 70
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 70
         DO 60 I3=0,IELEM-1
         DO 60 I2=0,IELEM-1
         DO 50 I1=0,IELEM-1
         INX1=IORD(I1,I2,I3,LNUN,IELEM,IGAR(K))
   50    VEC(INX1)=VEC(INX1)+SGD(L)*VOL0
         IF((IDIM.EQ.1).AND.(I2.EQ.0)) GO TO 70
         IF((IDIM.EQ.2).AND.(I2.EQ.IELEM-1)) GO TO 70
   60    CONTINUE
   70    CONTINUE
      ELSE IF(ICHX.EQ.4) THEN
*        NODAL COLLOCATION METHOD WITH SERENDIPITY APPROXIMATION.
         LNUN=0
         DO 80 K=1,NEL
         IF(MAT(K).EQ.0) GO TO 80
         LNUN=LNUN+1
         IGAR(K)=LNUN
   80    CONTINUE
*
         DO 110 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 110
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 110
         DO 100 I3=0,IELEM-1
         DO 100 I2=0,IELEM-1-I3
         DO 90 I1=0,IELEM-1-I2-I3
         INX1=IORL(I1,I2,I3,LNUN,IELEM,IGAR(K))
   90    VEC(INX1)=VEC(INX1)+SGD(L)*VOL0
         IF((IDIM.EQ.1).AND.(I2.EQ.0)) GO TO 110
         IF((IDIM.EQ.2).AND.(I2.EQ.IELEM-1)) GO TO 110
  100    CONTINUE
  110    CONTINUE
      ENDIF
      RETURN
      END
