*DECK TRIASP
      SUBROUTINE TRIASP (NEL,LL4,CYLIND,SGD,XX,DD,VOL,MAT,KN,LC,T,TS,
     1 VEC)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF A DIAGONAL SYSTEM MATRIX CORRESPONDING TO A SINGLE CROSS
* SECTION TYPE (PRIMAL FORMULATION).
*
* INPUT PARAMETERS:
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  LL4      ORDER OF SYSTEM MATRICES.
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
*  SGD    : CROSS SECTION PER MATERIAL MIXTURE. DIMENSION SGD(IR)
*           WHERE IR IS THE NUMBER OF MATERIAL MIXTURES.
*  XX     : X-DIRECTED MESH SPACINGS. DIMENSION XX(NEL)
*  DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(NEL)
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST.
*           DIMENSION KN(NEL*(IELEM+1)**3) WHERE IELEM IS THE ORDER OF
*           THE FINITE ELEMENTS.
*  LC     : ORDER OF THE UNIT MATRICES.
*  T      : CARTESIAN LINEAR PRODUCT VECTOR. DIMENSION T(LC)
*  TS     : CYLINDRICAL LINEAR PRODUCT VECTOR. DIMENSION TS(LC)
*
* OUTPUT PARAMETER:
*  VEC    : DIAGONAL MATRIX CORRESPONDING TO THE CROSS SECTION TERM.
*           DIMENSION VEC(LL4)
*
* NOTE: VECTOR VEC SHOULD BE INITIALIZED BY THE CALLING PROGRAM.
*
*------------------------------------- AUTHOR: A. HEBERT ; 15/01/86 ----
*
      LOGICAL CYLIND
      DIMENSION SGD(*),XX(*),DD(*),VOL(*),MAT(*),KN(*),T(*),TS(*),
     1 VEC(*),R3DP(125),R3DC(125)
*
* CALCULATION OF 3-D MASS MATRICES FROM TENSORIAL PRODUCT OF 1-D
* MATRICES.
      LL=LC*LC*LC
      DO 20 L=1,LL
      L1=1+MOD(L-1,LC)
      L2=1+(L-L1)/LC
      L3=1+MOD(L2-1,LC)
      I1=L1
      I2=L3
      I3=1+(L2-L3)/LC
      R3DP(L)=T(I1)*T(I2)*T(I3)
   20 R3DC(L)=TS(I1)*T(I2)*T(I3)
*
      NUM1=0
      DO 90 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 90
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 80
      DX=XX(K)
      DO 50 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 50
      IF(CYLIND) THEN
         RR=(R3DP(I)+R3DC(I)*DX/DD(K))*VOL0
      ELSE
         RR=R3DP(I)*VOL0
      ENDIF
      VEC(IND1)=VEC(IND1)+RR*SGD(L)
   50 CONTINUE
   80 NUM1=NUM1+LL
   90 CONTINUE
      RETURN
      END
