*DECK TRICH4
      SUBROUTINE TRICH4(ISPLH,IPTRK,IDIM,LX,LZ,L4,MAT,KN,MUW,MUX,MUY,
     1 MUZ,IPW,IPX,IPY,IPZ,IWRK,IMPX)
*
*-----------------------------------------------------------------------
*
* COMPUTE THE COMPRESSED DIAGONAL STORAGE INDICES (MUW, MUX, MUY AND
* MUZ) AND THE PERMUTATION VECTORS (IPW, IPX, IPY AND IPZ) FOR AN ADI
* SPLITTING OF A MESH CENTERED FINITE DIFFERENCE DISCRETIZATION IN
* HEXAGONAL GEOMETRY.
*
* INPUT PARAMETERS:
*  ISPLH  : =1 FOR COMPLETE HEXAGONS; =2 FOR TRIANGULAR MESH-SPLITTING.
*  IPTRK  : L_TRIVAC POINTER TO THE TRACKING INFORMATION.
*  IDIM   : NUMBER OF DIMENSIONS (2 OR 3).
*  LX     : NUMBER OF HEXAGONS IN A PLANE.
*  LZ     : NUMBER OF AXIAL PLANES.
*  L4     : ORDER OF SYSTEM MATRICES.
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(7*NEL)
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*
* OUTPUT PARAMETERS:
*  MUW,MUX,MUY,MUZ : COMPRESSED STORAGE MODE INDICES.
*                    DIMENSION MUW(L4),MUX(L4),MUY(L4),MUZ(L4)
*  IPW,IPX,IPY,IPZ : PERMUTATION MATRICES.
*                    DIMENSION IPW(L4),IPX(L4),IPY(L4),IPZ(L4)
*
* SCRATCH STORAGE:
*  IWRK   : DIMENSION IWRK(LX*LZ)
*
*----------------------------------- AUTHOR: A. BENABOUD ; 15/12/86 ----
*
      DIMENSION MAT(*),KN(*),MUW(*),MUX(*),MUY(*),MUZ(*),IPW(*),IPX(*),
     >          IPY(*),IPZ(*),IWRK(*)
      COMMON IBASE(1)
*
      IF(ISPLH.EQ.1) THEN
         CALL SETARA (IBASE,LX,I1)
         CALL SETARA (IBASE,LX,I2)
         CALL SETARA (IBASE,LX,I3)
         CALL SETARA (IBASE,LX,I4)
         CALL SETARA (IBASE,LX,I5)
         LT4=L4/LZ
         MEL = 0
         DO 250 KEL=1,LX
         IBASE (I4+KEL-1) = 0
         IBASE (I5+KEL-1) = 0
         IBASE (I4+KEL-1) = MAT(KEL)
         IF(MAT(KEL).LE.0) GO TO 250
         MEL = MEL + 1
  250    IBASE (I5+KEL-1) = MEL
*
*        COMPUTE THE PERMUTATION VECTORS.
         DO 260 I=1,L4
         IPW(I)=0
         IPX(I)=0
         IPY(I)=0
  260    IPZ(I)=0
         NC = INT((SQRT(REAL((4*LX-1)/3))+1.)/2.)
         J1 = 2 + 3*(NC-1)*(NC-2)
         J2 = J1 + NC - 1
         J3 = J2 + NC - 1
         CALL TRIPER (J1,1,IBASE(I4),LX,LT4,IBASE(I1),IBASE(I5))
         CALL TRIPER (J2,2,IBASE(I4),LX,LT4,IBASE(I2),IBASE(I5))
         CALL TRIPER (J3,3,IBASE(I4),LX,LT4,IBASE(I3),IBASE(I5))
         KEL = 0
         DO 270 K0 = 1,LZ
         DO 270 K1 = 1,LT4
         KEL = KEL + 1
         IV = (K0-1)*LT4
         IPW(KEL) = IBASE(I1+K1-1)+IV
         IPX(KEL) = IBASE(I2+K1-1)+IV
  270    IPY(KEL) = IBASE(I3+K1-1)+IV
         IF(IDIM.EQ.3) THEN
            JEL = 0
            DO 280 K1=1,LT4
            DO 280 K0=1,LZ
            JEL = JEL + 1
  280       IPZ((K0-1)*LT4+IBASE(I1+K1-1)) = JEL
         ENDIF
         CALL RLSARA (IBASE(I5))
         CALL RLSARA (IBASE(I4))
         CALL RLSARA (IBASE(I3))
         CALL RLSARA (IBASE(I2))
         CALL RLSARA (IBASE(I1))
*
         DO 300 I=1,L4
         MUW(I)=0
         MUX(I)=0
         MUY(I)=0
  300    MUZ(I)=0
*
*        COMPUTE THE COMPRESSED DIAGONAL STORAGE INDICES.
         NUM1=0
         DO 320 KEL=1,L4
         KK1=KN(NUM1+6)
         KK2=KN(NUM1+3)
         INW1=IPW(KEL)
         MUW(INW1)=1
         IF(KK1.GT.0) THEN
            INW2=IPW(KK1)
            MUW(INW1)=MAX0(MUW(INW1),INW1-INW2+1)
         ENDIF
         IF(KK2.GT.0) THEN
            INW2=IPW(KK2)
            MUW(INW1)=MAX0(MUW(INW1),INW1-INW2+1)
         ENDIF
  320    NUM1=NUM1+8
*
         NUM1=0
         DO 330 KEL=1,L4
         KK3=KN(NUM1+1)
         KK4=KN(NUM1+4)
         INX1=IPX(KEL)
         MUX(INX1)=1
         IF(KK3.GT.0) THEN
            INX2=IPX(KK3)
            MUX(INX1)=MAX0(MUX(INX1),INX1-INX2+1)
         ENDIF
         IF(KK4.GT.0) THEN
            INX2=IPX(KK4)
            MUX(INX1)=MAX0(MUX(INX1),INX1-INX2+1)
         ENDIF
  330    NUM1=NUM1+8
*
         NUM1=0
         DO 340 KEL=1,L4
         KK5=KN(NUM1+2)
         KK6=KN(NUM1+5)
         INY1=IPY(KEL)
         MUY(INY1)=1
         IF(KK5.GT.0) THEN
            INY2=IPY(KK5)
            MUY(INY1)=MAX0(MUY(INY1),INY1-INY2+1)
         ENDIF
         IF(KK6.GT.0) THEN
            INY2=IPY(KK6)
            MUY(INY1)=MAX0(MUY(INY1),INY1-INY2+1)
         ENDIF
  340    NUM1=NUM1+8
         IF(IDIM.EQ.3) THEN
*
            NUM1=0
            DO 350 KEL=1,L4
            KK7=KN(NUM1+7)
            KK8=KN(NUM1+8)
            INZ1=IPZ(KEL)
            MUZ(INZ1)=1
            IF(KK7.GT.0) THEN
               INZ2=IPZ(KK7)
               MUZ(INZ1)=MAX0(MUZ(INZ1),INZ1-INZ2+1)
            ENDIF
            IF(KK8.GT.0) THEN
               INZ2=IPZ(KK8)
               MUZ(INZ1)=MAX0(MUZ(INZ1),INZ1-INZ2+1)
            ENDIF
  350       NUM1=NUM1+8
         ENDIF
*
      ELSE IF(ISPLH.GE.2) THEN
*
         NTPH = 6*(ISPLH-1)**2
         NTPL = 1+2*(ISPLH-1)
         NVT1 = NTPL + 2 * (ISPLH-2) + NTPH / 2
         NVT2 = NTPH - NTPL - (ISPLH-4) * (NTPL+2)
         NVT3 = NTPH - (ISPLH-4) * NTPL
         IVAL = 3*NTPH + 8
         IF(ISPLH.EQ.3) NVT2 = NTPH
         IF(ISPLH.LE.3) ISAU =   2*(ISPLH-2)
         IF(ISPLH.GE.4) ISAU =   6*(ISPLH-3)
         ICR = ISAU*(1+2*(ISPLH-2))
*
*        COMPUTE THE PERMUTATION VECTORS.
         DO 400 I=1,L4
         IPW(I)=I
         IPX(I)=0
         IPY(I)=0
  400    IPZ(I)=0
         LI4 = L4/LZ
         CALL SETARA(IBASE,LI4,IDX)
         CALL SETARA(IBASE,LI4,IDY)
         CALL LCMGET(IPTRK,'ILX',IBASE(IDX))
         CALL LCMGET(IPTRK,'ILY',IBASE(IDY))
         DO 410 KZ=1,LZ
         DO 410 KI=1,LI4
         IPX(KI+(KZ-1)*LI4) = IBASE(IDX+KI-1) + (KZ-1)*LI4
  410    IPY(KI+(KZ-1)*LI4) = IBASE(IDY+KI-1) + (KZ-1)*LI4
         CALL RLSARA(IBASE(IDY))
         CALL RLSARA(IBASE(IDX))
         IF(IDIM.EQ.3) THEN
            DO 420 K1=1,LI4
            DO 420 K0=1,LZ
  420       IPZ((K0-1)*LI4+K1) = K0 + (K1-1)*LZ
         ENDIF
*
         DO 500 I=1,L4
         MUW(I)=0
         MUX(I)=0
         MUY(I)=0
  500    MUZ(I)=0
*
*        COMPUTE THE COMPRESSED DIAGONAL STORAGE INDICES.
         NUM1=0
         DO 520 K0=1,LX*LZ
         IF(MAT(K0).LE.0) GO TO 520
         DO 510 I = 1,NTPH
         CALL TRINEI (3,1,2,ISPLH,ICR,I,KK1,KK2,KK3,KEL,IQF,NUM1,
     >   NTPH,NTPL,NVT1,NVT2,NVT3,IVAL,KN)
         INW1=IPW(KEL)
         MUW(INW1)=1
         IF(KK1.GT.0) THEN
            INW2=IPW(KK1)
            MUW(INW1)=MAX0(MUW(INW1),INW1-INW2+1)
         ENDIF
         IF(KK2.GT.0) THEN
            INW2=IPW(KK2)
            MUW(INW1)=MAX0(MUW(INW1),INW1-INW2+1)
         ENDIF
  510    CONTINUE
         NUM1=NUM1+IVAL
  520    CONTINUE
*
         NUM1=0
         DO 540 K0=1,LX*LZ
         IF(MAT(K0).LE.0) GO TO 540
         DO 530 I = 1,NTPH
         CALL TRINEI (3,2,2,ISPLH,ICR,I,KK1,KK2,KK3,KEL,IQF,NUM1,
     >   NTPH,NTPL,NVT1,NVT2,NVT3,IVAL,KN)
         INX1=IPX(KEL)
         MUX(INX1)=1
         IF(KK1.GT.0) THEN
            INX2=IPX(KK1)
            MUX(INX1)=MAX0(MUX(INX1),INX1-INX2+1)
         ENDIF
         IF(KK2.GT.0) THEN
            INX2=IPX(KK2)
            MUX(INX1)=MAX0(MUX(INX1),INX1-INX2+1)
         ENDIF
  530    CONTINUE
         NUM1=NUM1+IVAL
  540    CONTINUE
*
         NUM1=0
         DO 560 K0=1,LX*LZ
         IF(MAT(K0).LE.0) GO TO 560
         DO 550 I = 1,NTPH
         CALL TRINEI (3,3,2,ISPLH,ICR,I,KK1,KK2,KK3,KEL,IQF,NUM1,
     >   NTPH,NTPL,NVT1,NVT2,NVT3,IVAL,KN)
         INY1=IPY(KEL)
         MUY(INY1)=1
         IF(KK1.GT.0) THEN
            INY2=IPY(KK1)
            MUY(INY1)=MAX0(MUY(INY1),INY1-INY2+1)
         ENDIF
         IF(KK2.GT.0) THEN
            INY2=IPY(KK2)
            MUY(INY1)=MAX0(MUY(INY1),INY1-INY2+1)
         ENDIF
  550    CONTINUE
         NUM1=NUM1+IVAL
  560    CONTINUE
         IF(IDIM.EQ.3) THEN
*
            NUM1=0
            DO 580 K0=1,LX*LZ
            IF(MAT(K0).LE.0) GO TO 580
            DO 570 I = 1,NTPH
            KK1 = KN(NUM1+NTPH+I)
            KK2 = KN(NUM1+2*NTPH+I)
            KEL = KN(NUM1+I)
            INZ1=IPZ(KEL)
            MUZ(INZ1)=1
            IF(KK1.GT.0) THEN
               INZ2=IPZ(KK1)
               MUZ(INZ1)=MAX0(MUZ(INZ1),INZ1-INZ2+1)
            ENDIF
            IF(KK2.GT.0) THEN
               INZ2=IPZ(KK2)
               MUZ(INZ1)=MAX0(MUZ(INZ1),INZ1-INZ2+1)
            ENDIF
  570       CONTINUE
            NUM1=NUM1+IVAL
  580       CONTINUE
         ENDIF
      ENDIF
      IF(IMPX.GE.4) THEN
         WRITE(6,710) 'IPW :',(IPW(I),I=1,L4)
         WRITE(6,710) 'MUW :',(MUW(I),I=1,L4)
         WRITE(6,710) 'IPX :',(IPX(I),I=1,L4)
         WRITE(6,710) 'MUX :',(MUX(I),I=1,L4)
         WRITE(6,710) 'IPY :',(IPY(I),I=1,L4)
         WRITE(6,710) 'MUY :',(MUY(I),I=1,L4)
         IF(IDIM.EQ.3) THEN
            WRITE(6,710) 'IPZ :',(IPZ(I),I=1,L4)
            WRITE(6,710) 'MUZ :',(MUZ(I),I=1,L4)
         ENDIF
      ENDIF
*
      MUWMAX=0
      MUXMAX=0
      MUYMAX=0
      MUZMAX=0
      IIMAWW=0
      IIMAWX=0
      IIMAWY=0
      IIMAWZ=0
      DO 590 I=1,L4
      MUWMAX=MAX(MUWMAX,MUW(I))
      MUXMAX=MAX(MUXMAX,MUX(I))
      MUYMAX=MAX(MUYMAX,MUY(I))
      MUZMAX=MAX(MUZMAX,MUZ(I))
      IIMAWW=IIMAWW+MUW(I)
      MUW(I)=IIMAWW
      IIMAWX=IIMAWX+MUX(I)
      MUX(I)=IIMAWX
      IIMAWY=IIMAWY+MUY(I)
      MUY(I)=IIMAWY
      IIMAWZ=IIMAWZ+MUZ(I)
 590  MUZ(I)=IIMAWZ
      IF(IMPX.GE.0) WRITE (6,720) MUWMAX,MUXMAX,MUYMAX,MUZMAX
      RETURN
*
  710 FORMAT(/1X,A5/(1X,20I6))
  720 FORMAT(/41H TRICH4: MAXIMUM BANDWIDTH ALONG W AXIS =,I5/
     1 27X,14HALONG X AXIS =,I5/27X,14HALONG Y AXIS =,I5/27X,
     2 14HALONG Z AXIS =,I5)
      END
