*DECK TRICHD
      SUBROUTINE TRICHD(IMPX,LX,LY,LZ,IELEM,ICOL,L4,MAT,KN,MUX,MUY,MUZ,
     1 IPX,IPY,IPZ,IWRK)
*
*-----------------------------------------------------------------------
*
* DUAL FINITE ELEMENT UNKNOWN NUMBERING FOR ADI SOLUTION IN A 3D DOMAIN.
* COMPUTE THE STORAGE INFO FOR ADI MATRICES IN COMPRESSED DIAGONAL
* STORAGE MODE. COMPUTE THE ADI PERMUTATION VECTORS.
*
* INPUT PARAMETERS:
*  IMPX   : PRINT PARAMETER.
*  LX,LY,LZ : NUMBER OF ELEMENTS ALONG THE X, Y AND Z AXIS.
*  IELEM  : DEGREE OF THE LAGRANGIAN FINITE ELEMENTS. =1 (LINEAR);
*           =2 (PARABOLIC); =3 (CUBIC); =4 (QUARTIC).
*  ICOL   : TYPE OF QUADRATURE. =1 (ANALYTICAL INTEGRATION);
*           =2 (GAUSS-LOBATTO); =3 (GAUSS-LEGENDRE).
*  L4     : TOTAL NUMBER OF UNKNOWN (VARIATIONAL COEFFICIENTS) PER
*           ENERGY GROUP. ORDER OF SYSTEM MATRICES.
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT.
*           DIMENSION MAT(LX*LY*LZ)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(7*LX*LY*LZ)
*
* OUTPUT PARAMETERS:
*  MUX,MUY,MUZ : INDICES USED WITH COMPRESSED DIAGONAL MODE MATRICES.
*                DIMENSION MUX(L4),MUY(L4),MUZ(L4)
*  IPX,IPY,IPZ : PERMUTATION VECTORS.
*                DIMENSION IPX(L4),IPY(L4),IPZ(L4)
*
* SCRATCH STORAGE:
*  IWRK   : DIMENSION IWRK(LX*LY*LZ)
*
*------------------------------------- AUTHOR: A. HEBERT ; 19/08/92 ----
*
      INTEGER J1,J2,J3
      DIMENSION MAT(LX*LY*LZ),KN(7*LX*LY*LZ),
     1 MUX(L4),MUY(L4),MUZ(L4),IPX(L4),IPY(L4),IPZ(L4),
     2 IWRK(LX,LY,LZ)
*
      KEL=0
      NUM1=0
      DO 12 J3=1,LZ
      DO 11 J2=1,LY
      DO 10 J1=1,LX
      IWRK(J1,J2,J3)=-1
      KEL=KEL+1
      IF(MAT(KEL).EQ.0) GO TO 10
      IWRK(J1,J2,J3)=NUM1
      NUM1=NUM1+7
   10 CONTINUE
   11 CONTINUE
   12 CONTINUE
*
* COMPUTE THE X-ORIENTED PERMUTATION VECTOR.
      DO 20 I=1,L4
   20 IPX(I)=0
      INEW=0
      DO 44 J3=1,LZ
      DO 43 J2=1,LY
      DO 42 K3=0,IELEM-1
      DO 41 K2=0,IELEM-1
      DO 40 J1=1,LX
      NUM1=IWRK(J1,J2,J3)
      IF(NUM1.EQ.-1) GO TO 40
      IF(KN(NUM1+2).GT.0) THEN
         INX1=KN(NUM1+2)+K3*IELEM+K2
         IF(IPX(INX1).EQ.0) THEN
            INEW=INEW+1
            IPX(INX1)=INEW
         ENDIF
      ENDIF
      DO 30 K1=0,IELEM-1
      IND1=KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1
      INEW=INEW+1
   30 IPX(IND1)=INEW
      IF(KN(NUM1+3).GT.0) THEN
         INX2=KN(NUM1+3)+K3*IELEM+K2
         IF(IPX(INX2).EQ.0) THEN
            INEW=INEW+1
            IPX(INX2)=INEW
         ENDIF
      ENDIF
   40 CONTINUE
   41 CONTINUE
   42 CONTINUE
   43 CONTINUE
   44 CONTINUE
      DO 62 J3=1,LZ
      DO 61 J2=1,LY
      DO 60 J1=1,LX
      NUM1=IWRK(J1,J2,J3)
      IF(NUM1.EQ.-1) GO TO 60
      DO 51 K3=0,IELEM-1
      DO 50 K2=0,IELEM-1
      IF(KN(NUM1+4).GT.0) THEN
         INY1=KN(NUM1+4)+K3*IELEM+K2
         IF(IPX(INY1).EQ.0) THEN
            INEW=INEW+1
            IPX(INY1)=INEW
         ENDIF
      ENDIF
      IF(KN(NUM1+5).GT.0) THEN
         INY2=KN(NUM1+5)+K3*IELEM+K2
         IF(IPX(INY2).EQ.0) THEN
            INEW=INEW+1
            IPX(INY2)=INEW
         ENDIF
      ENDIF
      IF(KN(NUM1+6).GT.0) THEN
         INZ1=KN(NUM1+6)+K3*IELEM+K2
         IF(IPX(INZ1).EQ.0) THEN
            INEW=INEW+1
            IPX(INZ1)=INEW
         ENDIF
      ENDIF
      IF(KN(NUM1+7).GT.0) THEN
         INZ2=KN(NUM1+7)+K3*IELEM+K2
         IF(IPX(INZ2).EQ.0) THEN
            INEW=INEW+1
            IPX(INZ2)=INEW
         ENDIF
      ENDIF
   50 CONTINUE
   51 CONTINUE
   60 CONTINUE
   61 CONTINUE
   62 CONTINUE
      IF(INEW.NE.L4) CALL XABORT('TRICHD: ALGORITHM FAILURE 1.')
      DO 70 I=1,L4
      IF(IPX(I).EQ.0) CALL XABORT('TRICHD: ALGORITHM FAILURE 2.')
   70 CONTINUE
*
* COMPUTE THE Y-ORIENTED PERMUTATION VECTOR.
      DO 80 I=1,L4
   80 IPY(I)=0
      INEW=0
      DO 100 J3=1,LZ
      DO 100 J1=1,LX
      DO 100 K3=0,IELEM-1
      DO 100 K1=0,IELEM-1
      DO 100 J2=1,LY
      NUM1=IWRK(J1,J2,J3)
      IF(NUM1.EQ.-1) GO TO 100
      IF(KN(NUM1+4).GT.0) THEN
         INY1=KN(NUM1+4)+K3*IELEM+K1
         IF(IPY(INY1).EQ.0) THEN
            INEW=INEW+1
            IPY(INY1)=INEW
         ENDIF
      ENDIF
      DO 90 K2=0,IELEM-1
      IND1=KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1
      INEW=INEW+1
   90 IPY(IND1)=INEW
      IF(KN(NUM1+5).GT.0) THEN
         INY2=KN(NUM1+5)+K3*IELEM+K1
         IF(IPY(INY2).EQ.0) THEN
            INEW=INEW+1
            IPY(INY2)=INEW
         ENDIF
      ENDIF
  100 CONTINUE
      DO 122 J3=1,LZ
      DO 121 J1=1,LX
      DO 120 J2=1,LY
      NUM1=IWRK(J1,J2,J3)
      IF(NUM1.EQ.-1) GO TO 120
      DO 111 K3=0,IELEM-1
      DO 110 K1=0,IELEM-1
      IF(KN(NUM1+2).GT.0) THEN
         INX1=KN(NUM1+2)+K3*IELEM+K1
         IF(IPY(INX1).EQ.0) THEN
            INEW=INEW+1
            IPY(INX1)=INEW
         ENDIF
      ENDIF
      IF(KN(NUM1+3).GT.0) THEN
         INX2=KN(NUM1+3)+K3*IELEM+K1
         IF(IPY(INX2).EQ.0) THEN
            INEW=INEW+1
            IPY(INX2)=INEW
         ENDIF
      ENDIF
      IF(KN(NUM1+6).GT.0) THEN
         INZ1=KN(NUM1+6)+K3*IELEM+K1
         IF(IPY(INZ1).EQ.0) THEN
            INEW=INEW+1
            IPY(INZ1)=INEW
         ENDIF
      ENDIF
      IF(KN(NUM1+7).GT.0) THEN
         INZ2=KN(NUM1+7)+K3*IELEM+K1
         IF(IPY(INZ2).EQ.0) THEN
            INEW=INEW+1
            IPY(INZ2)=INEW
         ENDIF
      ENDIF
  110 CONTINUE
  111 CONTINUE
  120 CONTINUE
  121 CONTINUE
  122 CONTINUE
      IF(INEW.NE.L4) CALL XABORT('TRICHD: ALGORITHM FAILURE 3.')
      DO 130 I=1,L4
      IF(IPY(I).EQ.0) CALL XABORT('TRICHD: ALGORITHM FAILURE 4.')
  130 CONTINUE
*
* COMPUTE THE Z-ORIENTED PERMUTATION VECTOR.
      DO 140 I=1,L4
  140 IPZ(I)=0
      INEW=0
      DO 164 J2=1,LY
      DO 163 J1=1,LX
      DO 162 K2=0,IELEM-1
      DO 161 K1=0,IELEM-1
      DO 160 J3=1,LZ
      NUM1=IWRK(J1,J2,J3)
      IF(NUM1.EQ.-1) GO TO 160
      IF(KN(NUM1+6).GT.0) THEN
         INZ1=KN(NUM1+6)+K2*IELEM+K1
         IF(IPZ(INZ1).EQ.0) THEN
            INEW=INEW+1
            IPZ(INZ1)=INEW
         ENDIF
      ENDIF
      DO 150 K3=0,IELEM-1
      IND1=KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1
      INEW=INEW+1
  150 IPZ(IND1)=INEW
      IF(KN(NUM1+7).GT.0) THEN
         INZ2=KN(NUM1+7)+K2*IELEM+K1
         IF(IPZ(INZ2).EQ.0) THEN
            INEW=INEW+1
            IPZ(INZ2)=INEW
         ENDIF
      ENDIF
  160 CONTINUE
  161 CONTINUE
  162 CONTINUE
  163 CONTINUE
  164 CONTINUE
      DO 182 J2=1,LY
      DO 181 J1=1,LX
      DO 180 J3=1,LZ
      NUM1=IWRK(J1,J2,J3)
      IF(NUM1.EQ.-1) GO TO 180
      DO 171 K2=0,IELEM-1
      DO 170 K1=0,IELEM-1
      IF(KN(NUM1+2).GT.0) THEN
         INX1=KN(NUM1+2)+K2*IELEM+K1
         IF(IPZ(INX1).EQ.0) THEN
            INEW=INEW+1
            IPZ(INX1)=INEW
         ENDIF
      ENDIF
      IF(KN(NUM1+3).GT.0) THEN
         INX2=KN(NUM1+3)+K2*IELEM+K1
         IF(IPZ(INX2).EQ.0) THEN
            INEW=INEW+1
            IPZ(INX2)=INEW
         ENDIF
      ENDIF
      IF(KN(NUM1+4).GT.0) THEN
         INY1=KN(NUM1+4)+K2*IELEM+K1
         IF(IPZ(INY1).EQ.0) THEN
            INEW=INEW+1
            IPZ(INY1)=INEW
         ENDIF
      ENDIF
      IF(KN(NUM1+5).GT.0) THEN
         INY2=KN(NUM1+5)+K2*IELEM+K1
         IF(IPZ(INY2).EQ.0) THEN
            INEW=INEW+1
            IPZ(INY2)=INEW
         ENDIF
      ENDIF
  170 CONTINUE
  171 CONTINUE
  180 CONTINUE
  181 CONTINUE
  182 CONTINUE
      IF(INEW.NE.L4) CALL XABORT('TRICHD: ALGORITHM FAILURE 5.')
      DO 190 I=1,L4
      IF(IPZ(I).EQ.0) CALL XABORT('TRICHD: ALGORITHM FAILURE 6.')
  190 CONTINUE
*
* COMPUTE THE X-ORIENTED SYSTEM BANDWIDTH VECTOR.
      DO 200 I=1,L4
  200 MUX(I)=1
      NUM1=0
      DO 230 KEL=1,LX*LY*LZ
      IF(MAT(KEL).EQ.0) GO TO 230
      DO 221 K3=0,IELEM-1
      DO 220 K2=0,IELEM-1
      INX1=IPX(ABS(KN(NUM1+2))+K3*IELEM+K2)
      INX2=IPX(ABS(KN(NUM1+3))+K3*IELEM+K2)
      DO 210 K1=0,IELEM-1
      IND1=IPX(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1)
      IF(IELEM.GE.4) MUX(IND1)=MAX(MUX(IND1),K1+1)
      IF(KN(NUM1+2).NE.0) THEN
         MUX(IND1)=MAX(MUX(IND1),IND1-INX1+1)
         MUX(INX1)=MAX(MUX(INX1),INX1-IND1+1)
      ENDIF
      IF(KN(NUM1+3).NE.0) THEN
         MUX(INX2)=MAX(MUX(INX2),INX2-IND1+1)
         MUX(IND1)=MAX(MUX(IND1),IND1-INX2+1)
      ENDIF
  210 CONTINUE
      IF(ICOL.NE.2) THEN
         IF((KN(NUM1+2).NE.0).AND.(KN(NUM1+3).NE.0)) THEN
            MUX(INX2)=MAX(MUX(INX2),INX2-INX1+1)
            MUX(INX1)=MAX(MUX(INX1),INX1-INX2+1)
         ENDIF
      ENDIF
  220 CONTINUE
  221 CONTINUE
      NUM1=NUM1+7
  230 CONTINUE
*
* COMPUTE THE Y-ORIENTED SYSTEM BANDWIDTH VECTOR.
      DO 240 I=1,L4
  240 MUY(I)=1
      NUM1=0
      DO 270 KEL=1,LX*LY*LZ
      IF(MAT(KEL).EQ.0) GO TO 270
      DO 261 K3=0,IELEM-1
      DO 260 K1=0,IELEM-1
      INY1=IPY(ABS(KN(NUM1+4))+K3*IELEM+K1)
      INY2=IPY(ABS(KN(NUM1+5))+K3*IELEM+K1)
      DO 250 K2=0,IELEM-1
      IND1=IPY(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1)
      IF(IELEM.GE.4) MUY(IND1)=MAX(MUY(IND1),K2+1)
      IF(KN(NUM1+4).NE.0) THEN
         MUY(IND1)=MAX(MUY(IND1),IND1-INY1+1)
         MUY(INY1)=MAX(MUY(INY1),INY1-IND1+1)
      ENDIF
      IF(KN(NUM1+5).NE.0) THEN
         MUY(INY2)=MAX(MUY(INY2),INY2-IND1+1)
         MUY(IND1)=MAX(MUY(IND1),IND1-INY2+1)
      ENDIF
  250 CONTINUE
      IF(ICOL.NE.2) THEN
         IF((KN(NUM1+4).NE.0).AND.(KN(NUM1+5).NE.0)) THEN
            MUY(INY2)=MAX(MUY(INY2),INY2-INY1+1)
            MUY(INY1)=MAX(MUY(INY1),INY1-INY2+1)
         ENDIF
      ENDIF
  260 CONTINUE
  261 CONTINUE
      NUM1=NUM1+7
  270 CONTINUE
*
* COMPUTE THE Z-ORIENTED SYSTEM BANDWIDTH VECTOR.
      DO 280 I=1,L4
  280 MUZ(I)=1
      NUM1=0
      DO 310 KEL=1,LX*LY*LZ
      IF(MAT(KEL).EQ.0) GO TO 310
      DO 301 K2=0,IELEM-1
      DO 300 K1=0,IELEM-1
      INZ1=IPZ(ABS(KN(NUM1+6))+K2*IELEM+K1)
      INZ2=IPZ(ABS(KN(NUM1+7))+K2*IELEM+K1)
      DO 290 K3=0,IELEM-1
      IND1=IPZ(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1)
      IF(IELEM.GE.4) MUZ(IND1)=MAX(MUZ(IND1),K3+1)
      IF(KN(NUM1+6).NE.0) THEN
         MUZ(IND1)=MAX(MUZ(IND1),IND1-INZ1+1)
         MUZ(INZ1)=MAX(MUZ(INZ1),INZ1-IND1+1)
      ENDIF
      IF(KN(NUM1+7).NE.0) THEN
         MUZ(INZ2)=MAX(MUZ(INZ2),INZ2-IND1+1)
         MUZ(IND1)=MAX(MUZ(IND1),IND1-INZ2+1)
      ENDIF
  290 CONTINUE
      IF(ICOL.NE.2) THEN
         IF((KN(NUM1+6).NE.0).AND.(KN(NUM1+7).NE.0)) THEN
            MUZ(INZ2)=MAX(MUZ(INZ2),INZ2-INZ1+1)
            MUZ(INZ1)=MAX(MUZ(INZ1),INZ1-INZ2+1)
         ENDIF
      ENDIF
  300 CONTINUE
  301 CONTINUE
      NUM1=NUM1+7
  310 CONTINUE
*
      MUXMAX=0
      MUYMAX=0
      MUZMAX=0
      IIMAXX=0
      IIMAXY=0
      IIMAXZ=0
      DO 320 I=1,L4
      MUXMAX=MAX(MUXMAX,MUX(I))
      MUYMAX=MAX(MUYMAX,MUY(I))
      MUZMAX=MAX(MUZMAX,MUZ(I))
      IIMAXX=IIMAXX+MUX(I)
      MUX(I)=IIMAXX
      IIMAXY=IIMAXY+MUY(I)
      MUY(I)=IIMAXY
      IIMAXZ=IIMAXZ+MUZ(I)
  320 MUZ(I)=IIMAXZ
      IF(IMPX.GT.0) WRITE (6,500) MUXMAX,MUYMAX,MUZMAX
      RETURN
*
  500 FORMAT(/52H TRICHD: MAXIMUM BANDWIDTH FOR X-ORIENTED MATRICES =,
     1 I4/27X,25HFOR Y-ORIENTED MATRICES =,I4/27X,16HFOR Z-ORIENTED M,
     2 9HATRICES =,I4)
      END
