*DECK TRICO
      SUBROUTINE TRICO (IELEM,IR,K,VOL0,MAT,DIF,XX,YY,ZZ,DD,KN,QFR,
     1 CYLIND,A)
*
*-----------------------------------------------------------------------
*
* COMPUTE THE MESH CENTERED FINITE DIFFERENCE COEFFICIENTS IN ELEMENT K.
*
* INPUT PARAMETERS:
*  IELEM  : DEGREE OF THE POLYNOMIAL BASIS. =1 (LINEAR/FINITE DIFFEREN-
*           CES); =2 (PARABOLIC); =3 (CUBIC); =4 (QUARTIC).
*  IR     : FIRST DIMENSION OF MATRIX DIF.
*  K      : INDEX OF FINITE ELEMENT UNDER CONSIDERATION.
*  VOL0   : VOLUME OF FINITE ELEMENT UNDER CONSIDERATION.
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*           WHERE NEL IS THE TOTAL NUMBER OF FINITE ELEMENTS.
*  DIF    : DIRECTIONAL DIFFUSION COEFFICIENTS. DIMENSION DIF(IR,3)
*  XX,YY,ZZ : X-, Y- AND Z-DIRECTED MESH SPACINGS.
*           DIMENSION XX(NEL),YY(NEL),ZZ(NEL)
*  DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(6)
*           .GT.0 : NEIGHBOUR INDEX;
*           =-1   : VOID/ALBEDO BOUNDARY CONDITION;
*           =-2   : REFLECTION BOUNDARY CONDITION;
*           =-3   : ZERO FLUX BOUNDARY CONDITION;
*           =-4   : SYME BOUNDARY CONDITION (AXIAL SYMMETRY).
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(6)
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
*
* OUTPUT PARAMETER:
*  A      : MESH CENTERED FINITE DIFFERENCE COEFFICIENTS.
*           DIMENSION A(6)
*
*------------------------------------- AUTHOR: A. HEBERT ; 15/01/86 ----
*
      DOUBLE PRECISION A(6),DHARM,DIN,DOT
      LOGICAL CYLIND
      DIMENSION MAT(*),DIF(IR,*),XX(*),YY(*),ZZ(*),DD(*),KN(*),QFR(*)
      DHARM(X1,X2,DIF1,DIF2)=2.0D0*DIF1*DIF2/(X1*DIF2+X2*DIF1)
*
      DENOM=REAL((IELEM+1)*IELEM)
      L=MAT(K)
      DX=XX(K)
      DY=YY(K)
      DZ=ZZ(K)
      IF(CYLIND) THEN
         DIN=1.0D0-0.5D0*DX/DD(K)
         DOT=1.0D0+0.5D0*DX/DD(K)
      ELSE
         DIN=1.0D0
         DOT=1.0D0
      ENDIF
      KK1=KN(1)
      KK2=KN(2)
      KK3=KN(3)
      KK4=KN(4)
      KK5=KN(5)
      KK6=KN(6)
*     X- SIDE:
      IF(KK1.GT.0) THEN
         A(1)=DHARM(DX,XX(KK1),DIF(L,1),DIF(MAT(KK1),1))*DIN*VOL0/DX
      ELSE IF(KK1.EQ.-1) THEN
         A(1)=DHARM(DX,DX,DIF(L,1),DX*QFR(1)/DENOM)*DIN*VOL0/DX
      ELSE IF(KK1.EQ.-2) THEN
         A(1)=0.0D0
      ELSE IF(KK1.EQ.-3) THEN
         A(1)=2.0D0*DHARM(DX,DX,DIF(L,1),DIF(L,1))*DIN*VOL0/DX
      ENDIF
*     X+ SIDE:
      IF(KK2.GT.0) THEN
         A(2)=DHARM(DX,XX(KK2),DIF(L,1),DIF(MAT(KK2),1))*DOT*VOL0/DX
      ELSE IF(KK2.EQ.-1) THEN
         A(2)=DHARM(DX,DX,DIF(L,1),DX*QFR(2)/DENOM)*DOT*VOL0/DX
      ELSE IF(KK2.EQ.-2) THEN
         A(2)=0.0D0
      ELSE IF(KK2.EQ.-3) THEN
         A(2)=2.0D0*DHARM(DX,DX,DIF(L,1),DIF(L,1))*DOT*VOL0/DX
      ELSE IF(KK2.EQ.-4) THEN
         IF(KK1.EQ.-4) CALL XABORT('TRICO: INCONSISTENT SYME (1).')
         A(2)=A(1)
      ENDIF
      IF(KK1.EQ.-4) THEN
         IF(KK2.EQ.-4) CALL XABORT('TRICO: INCONSISTENT SYME (2).')
         A(1)=A(2)
      ENDIF
*     Y- SIDE:
      IF(KK3.GT.0) THEN
         A(3)=DHARM(DY,YY(KK3),DIF(L,2),DIF(MAT(KK3),2))*VOL0/DY
      ELSE IF(KK3.EQ.-1) THEN
         A(3)=DHARM(DY,DY,DIF(L,2),DY*QFR(3)/DENOM)*VOL0/DY
      ELSE IF(KK3.EQ.-2) THEN
         A(3)=0.0D0
      ELSE IF(KK3.EQ.-3) THEN
         A(3)=2.0D0*DHARM(DY,DY,DIF(L,2),DIF(L,2))*VOL0/DY
      ENDIF
*     Y+ SIDE:
      IF(KK4.GT.0) THEN
         A(4)=DHARM(DY,YY(KK4),DIF(L,2),DIF(MAT(KK4),2))*VOL0/DY
      ELSE IF(KK4.EQ.-1) THEN
         A(4)=DHARM(DY,DY,DIF(L,2),DY*QFR(4)/DENOM)*VOL0/DY
      ELSE IF(KK4.EQ.-2) THEN
         A(4)=0.0D0
      ELSE IF(KK4.EQ.-3) THEN
         A(4)=2.0D0*DHARM(DY,DY,DIF(L,2),DIF(L,2))*VOL0/DY
      ELSE IF(KK4.EQ.-4) THEN
         IF(KK3.EQ.-4) CALL XABORT('TRICO: INCONSISTENT SYME (3).')
         A(4)=A(3)
      ENDIF
      IF(KK3.EQ.-4) THEN
         IF(KK4.EQ.-4) CALL XABORT('TRICO: INCONSISTENT SYME (4).')
         A(3)=A(4)
      ENDIF
*     Z- SIDE:
      IF(KK5.GT.0) THEN
         A(5)=DHARM(DZ,ZZ(KK5),DIF(L,3),DIF(MAT(KK5),3))*VOL0/DZ
      ELSE IF(KK5.EQ.-1) THEN
         A(5)=DHARM(DZ,DZ,DIF(L,3),DZ*QFR(5)/DENOM)*VOL0/DZ
      ELSE IF(KK5.EQ.-2) THEN
         A(5)=0.0D0
      ELSE IF(KK5.EQ.-3) THEN
         A(5)=2.0D0*DHARM(DZ,DZ,DIF(L,3),DIF(L,3))*VOL0/DZ
      ENDIF
*     Z+ SIDE:
      IF(KK6.GT.0) THEN
         A(6)=DHARM(DZ,ZZ(KK6),DIF(L,3),DIF(MAT(KK6),3))*VOL0/DZ
      ELSE IF(KK6.EQ.-1) THEN
         A(6)=DHARM(DZ,DZ,DIF(L,3),DZ*QFR(6)/DENOM)*VOL0/DZ
      ELSE IF(KK6.EQ.-2) THEN
         A(6)=0.0D0
      ELSE IF(KK6.EQ.-3) THEN
         A(6)=2.0D0*DHARM(DZ,DZ,DIF(L,3),DIF(L,3))*VOL0/DZ
      ELSE IF(KK6.EQ.-4) THEN
         IF(KK5.EQ.-4) CALL XABORT('TRICO: INCONSISTENT SYME (5).')
         A(6)=A(5)
      ENDIF
      IF(KK5.EQ.-4) THEN
         IF(KK6.EQ.-4) CALL XABORT('TRICO: INCONSISTENT SYME (6).')
         A(5)=A(6)
      ENDIF
      RETURN
      END
