*DECK TRIDFC
      SUBROUTINE TRIDFC(IMPX,IDIM,LX,LY,LZ,CYLIND,NCODE,ZCODE,MAT,XXX,
     1 YYY,ZZZ,LL4,VOL,XX,YY,ZZ,DD,KN,QFR)
*
*-----------------------------------------------------------------------
*
* NUMBERING CORRESPONDING TO A MESH CENTERED FINITE DIFFERENCE (CHEBY
* TYPE) OR NODAL COLLOCATION DISCRETIZATION IN A 3-D GEOMETRY.
*
* INPUT PARAMETERS:
*  IMPX   : PRINT PARAMETER.
*  IDIM   : NUMBER OF DIMENSIONS.
*  LX,LY,LZ : NUMBER OF ELEMENTS ALONG THE X, Y AND Z AXIS.
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
*  NCODE  : NCODE(I) IS THE TYPE OF BOUNDARY CONDITION APPLIED ON SIDE I
*           I=1: X-  I=2: X+  I=3: Y-  I=4: Y+  I=5: Z-  I=6: Z+
*           NCODE(I)=1: VOID   NCODE(I)=2: REFL   NCODE(I)=4: TRAN
*           NCODE(I)=5: SYME   NCODE(I)=7: ZERO   NCODE(I)=8: CYLI
*  ZCODE  : ZCODE(I) IS THE ALBEDO CORRESPONDING TO BOUNDARY CONDITION
*           'VOID' ON SIDE I. ZCODE(I)=0.0 BY DEFAULT.
*           DIMENSION NCODE(6),ZCODE(6)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT.
*           DIMENSION MAT(LX*LY*LZ)
*  XXX,YYY: CARTESIAN COORDINATES ALONG THE X, Y AND Z AXIS.
*  ZZZ      DIMENSION XXX(LX+1),YYY(LY+1),ZZZ(LZ+1)
*
* OUTPUT PARAMETERS:
*  LL4    : TOTAL NUMBER OF UNKNOWN (VARIATIONAL COEFFICIENTS) PER
*           ENERGY GROUP. ORDER OF SYSTEM MATRICES.
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(LX*LY*LZ)
*  XX,YY,ZZ : X-, Y- AND Z-DIRECTED MESH SPACINGS.
*           DIMENSION XX(LX*LY*LZ),YY(LX*LY*LZ),ZZ(LX*LY*LZ)
*  DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(LX*LY*LZ)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(6*LX*LY*LZ)
*           .GT.0 : NEIGHBOUR INDEX;
*           =-1   : VOID/ALBEDO BOUNDARY CONDITION;
*           =-2   : REFLECTION BOUNDARY CONDITION;
*           =-3   : ZERO FLUX BOUNDARY CONDITION;
*           =-4   : SYME BOUNDARY CONDITION (AXIAL SYMMETRY).
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS.
*           DIMENSION QFR(6*LX*LY*LZ)
*
*------------------------------------- AUTHOR: A. HEBERT ; 15/01/86 ----
*
      LOGICAL CYLIND,LL1
      DIMENSION XXX(*),YYY(*),ZZZ(*),XX(*),YY(*),ZZ(*),DD(*),KN(*),
     1 QFR(*),VOL(*),MAT(*),NCODE(*),ZCODE(*)
*
      ALB(X)=0.5*(1.0-X)/(1.0+X)
*
* IDENTIFICATION OF THE GEOMETRY. MAIN LOOP OVER THE ELEMENTS.
      IF(IMPX.GT.0) WRITE(6,700) LX,LY,LZ
      LXY=LX*LY
      NUM1=0
      KEL=0
      DO 20 K0=1,LZ
      DO 20 K1=1,LY
      DO 20 K2=1,LX
      KEL=KEL+1
      XX(KEL)=0.0
      YY(KEL)=0.0
      ZZ(KEL)=0.0
      VOL(KEL)=0.0
      IF(MAT(KEL).LE.0) GO TO 20
      XX(KEL)=XXX(K2+1)-XXX(K2)
      YY(KEL)=YYY(K1+1)-YYY(K1)
      ZZ(KEL)=ZZZ(K0+1)-ZZZ(K0)
      IF(CYLIND) DD(KEL)=0.5*(XXX(K2)+XXX(K2+1))
      DO 10 IC=1,6
   10 QFR(NUM1+IC)=0.0
      FRX=1.0
      FRY=1.0
      FRZ=1.0
      KK1=KEL-1
      KK2=KEL+1
      KK3=KEL-LX
      KK4=KEL+LX
      KK5=KEL-LXY
      KK6=KEL+LXY
*
* VOID, REFL, ZERO OR CYLI BOUNDARY CONTITION.
      IF(K2.EQ.1) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KK1).EQ.0)
      ENDIF
      IF(LL1) THEN
         IF(NCODE(1).EQ.1) THEN
            KK1=-1
            QFR(NUM1+1)=ALB(ZCODE(1))
         ELSE IF(NCODE(1).EQ.2) THEN
            KK1=-2
         ELSE IF(NCODE(1).EQ.7) THEN
            KK1=-3
         ELSE IF(NCODE(1).EQ.8) THEN
            KK1=-1
         ENDIF
      ENDIF
*
      IF(K2.EQ.LX) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KK2).EQ.0)
      ENDIF
      IF(LL1) THEN
         IF(NCODE(2).EQ.1) THEN
            KK2=-1
            QFR(NUM1+2)=ALB(ZCODE(2))
         ELSE IF(NCODE(2).EQ.2) THEN
            KK2=-2
         ELSE IF(NCODE(2).EQ.7) THEN
            KK2=-3
         ELSE IF(NCODE(2).EQ.8) THEN
            KK2=-1
         ENDIF
      ENDIF
*
      IF(K1.EQ.1) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KK3).EQ.0)
      ENDIF
      IF(LL1) THEN
         IF(NCODE(3).EQ.1) THEN
            KK3=-1
            QFR(NUM1+3)=ALB(ZCODE(3))
         ELSE IF(NCODE(3).EQ.2) THEN
            KK3=-2
         ELSE IF(NCODE(3).EQ.7) THEN
            KK3=-3
         ELSE IF(NCODE(3).EQ.8) THEN
            KK3=-1
         ENDIF
      ENDIF
*
      IF(K1.EQ.LY) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KK4).EQ.0)
      ENDIF
      IF(LL1) THEN
         IF(NCODE(4).EQ.1) THEN
            KK4=-1
            QFR(NUM1+4)=ALB(ZCODE(4))
         ELSE IF(NCODE(4).EQ.2) THEN
            KK4=-2
         ELSE IF(NCODE(4).EQ.7) THEN
            KK4=-3
         ELSE IF(NCODE(4).EQ.8) THEN
            KK4=-1
         ENDIF
      ENDIF
*
      IF(K0.EQ.1) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KK5).EQ.0)
      ENDIF
      IF(LL1) THEN
         IF(NCODE(5).EQ.1) THEN
            KK5=-1
            QFR(NUM1+5)=ALB(ZCODE(5))
         ELSE IF(NCODE(5).EQ.2) THEN
            KK5=-2
         ELSE IF(NCODE(5).EQ.7) THEN
            KK5=-3
         ELSE IF(NCODE(5).EQ.8) THEN
            KK5=-1
         ENDIF
      ENDIF
*
      IF(K0.EQ.LZ) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KK6).EQ.0)
      ENDIF
      IF(LL1) THEN
         IF(NCODE(6).EQ.1) THEN
            KK6=-1
            QFR(NUM1+6)=ALB(ZCODE(6))
         ELSE IF(NCODE(6).EQ.2) THEN
            KK6=-2
         ELSE IF(NCODE(6).EQ.7) THEN
            KK6=-3
         ELSE IF(NCODE(6).EQ.8) THEN
            KK6=-1
         ENDIF
      ENDIF
*
* TRAN BOUNDARY CONDITION.
      IF((K2.EQ.1).AND.(NCODE(1).EQ.4)) THEN
         KK1=KEL+LX-1
      ENDIF
      IF((K2.EQ.LX).AND.(NCODE(2).EQ.4)) THEN
         KK2=KEL+1-LX
      ENDIF
      IF((K1.EQ.1).AND.(NCODE(3).EQ.4)) THEN
         KK3=KEL+(LY-1)*LX
      ENDIF
      IF((K1.EQ.LY).AND.(NCODE(4).EQ.4)) THEN
         KK4=KEL-(LY-1)*LX
      ENDIF
      IF((K0.EQ.1).AND.(NCODE(5).EQ.4)) THEN
         KK5=KEL+(LZ-1)*LXY
      ENDIF
      IF((K0.EQ.LZ).AND.(NCODE(6).EQ.4)) THEN
         KK6=KEL-(LZ-1)*LXY
      ENDIF
*
* SYME BOUNDARY CONDITION.
      IF((NCODE(1).EQ.5).AND.(K2.EQ.1)) THEN
         KK1=-4
         FRX=0.5
      ELSE IF((NCODE(2).EQ.5).AND.(K2.EQ.LX)) THEN
         KK2=-4
         FRX=0.5
      ENDIF
      IF((NCODE(3).EQ.5).AND.(K1.EQ.1)) THEN
         KK3=-4
         FRY=0.5
      ELSE IF((NCODE(4).EQ.5).AND.(K1.EQ.LY)) THEN
         KK4=-4
         FRY=0.5
      ENDIF
      IF((NCODE(5).EQ.5).AND.(K0.EQ.1)) THEN
         KK5=-4
         FRZ=0.5
      ELSE IF((NCODE(6).EQ.5).AND.(K0.EQ.LZ)) THEN
         KK6=-4
         FRZ=0.5
      ENDIF
*
      VOL0=XX(KEL)*YY(KEL)*ZZ(KEL)*FRX*FRY*FRZ
      IF(CYLIND) VOL0=6.2831853072*DD(KEL)*VOL0
      VOL(KEL)=VOL0
      KN(NUM1+1)=KK1
      KN(NUM1+2)=KK2
      KN(NUM1+3)=KK3
      KN(NUM1+4)=KK4
      KN(NUM1+5)=KK5
      KN(NUM1+6)=KK6
      NUM1=NUM1+6
   20 CONTINUE
* END OF THE MAIN LOOP OVER ELEMENTS.
*
      LL4=0
      DO 40 KEL=1,LXY*LZ
      IF(MAT(KEL).NE.0) LL4=LL4+1
   40 CONTINUE
*
      IF(IMPX.GE.2) THEN
         WRITE(6,720) (VOL(I),I=1,LXY*LZ)
         WRITE(6,750)
         NUM1=0
         DO 50 KEL=1,LXY*LZ
         IF(MAT(KEL).LE.0) GO TO 50
         WRITE (6,760) KEL,(KN(NUM1+I),I=1,6),(QFR(NUM1+I),I=1,6)
         NUM1=NUM1+6
   50    CONTINUE
      ENDIF
      RETURN
*
  700 FORMAT(/53H TRIDFC: MESH CENTERED FINITE DIFFERENCE OR NODAL COL,
     1 16HLOCATION METHOD.//34H NUMBER OF ELEMENTS ALONG X AXIS =,I3/
     2 20X,14HALONG Y AXIS =,I3/20X,14HALONG Z AXIS =,I3)
  720 FORMAT(/20H VOLUMES PER ELEMENT/(1X,1P,10E13.4))
  750 FORMAT(/22H NUMBERING OF UNKNOWNS/1X,21(1H-)//
     1 8H ELEMENT,5X,7HNUMBERS,50X,23HVOID BOUNDARY CONDITION)
  760 FORMAT (1X,I6,7X,6I8,6X,6F9.2)
      END
