*DECK TRIDIG
      SUBROUTINE TRIDIG(HNAME,IPTRK,IPSYS,IMPX,NEL,NUN,IPR,MAT,VOL,IDL,
     1 SGD)
*
*-----------------------------------------------------------------------
*
* DRIVER FOR THE ASSEMBLY OF A CROSS SECTION DIAGONAL MATRIX.
*
* INPUT PARAMETERS:
*  HNAME  : CHARACTER*12 NAME OF THE DIAGONAL MATRIX.
*  IPTRK  : L_TRIVAC POINTER TO THE TRIVAC TRACKING INFORMATION.
*  IPSYS  : L_SYSTEM POINTER TO SYSTEM MATRICES.
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : TOTAL NUMBER OF UNKNOWNS PER GROUP.
*  IPR    : IF IPR=3, THE NEW CONTRIBUTION IS ADDED TO EXISTING MATRIX.
*  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*           DIMENSION MAT(NEL)
*  VOL    : VOLUMES. DIMENSION VOL(NEL)
*  IDL    : POSITION OF THE FLUX COMPONENT ASSOCIATED WITH EACH VOLUME.
*           DIMENSION IDL(NEL)
*  SGD    : CROSS SECTION PER MATERIAL MIXTURE.DIMENSION SGD(IR)
*           WHERE IR IS THE MAXIMUM NUMBER OF MATERIAL MIXTURES.
*
*---------------------------------------------- A. HEBERT 15/05/93 -----
*
      CHARACTER HNAME*(*),TEXT12*12,NAME*12,NAMLCM*12,NAMMY*12
      LOGICAL CYLIND,CHEX,LCM
      DIMENSION MAT(*),VOL(*),SGD(*),IBASE(1),ITP(19)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
* RECOVER TRIVAC SPECIFIC TRACKING INFORMATION.
      CALL LCMGET(IPTRK,'TPARAM',ITP)
      ITYPE=ITP(1)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      CHEX=(ITYPE.EQ.8).OR.(ITYPE.EQ.9)
      IELEM=ITP(4)
      ICHX=ITP(6)
      ISPLH=ITP(7)
      LL4=ITP(8)
      LX=ITP(9)
      LY=ITP(10)
      LZ=ITP(11)
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL SETARA(BASE,LX*LY*LZ,IXORZ)
      IF(CHEX) THEN
         CALL LCMGET(IPTRK,'ZZ',BASE(IXORZ))
         CALL LCMGET(IPTRK,'SIDE',SIDE)
         CALL SETARA(IBASE,LL4,IIPW)
         CALL LCMGET(IPTRK,'IPW',IBASE(IIPW))
      ELSE
         CALL LCMGET(IPTRK,'XX',BASE(IXORZ))
         CALL SETARA(BASE,LX*LY*LZ,IDD)
         CALL LCMGET(IPTRK,'DD',BASE(IDD))
      ENDIF
      TEXT12=HNAME
      IF(IMPX.GT.0) WRITE(6,'(/37H TRIDIG: ASSEMBLY OF DIAGONAL MATRIX ,
     1 1H'',A12,2H''.)') TEXT12
*
* INITIALIZATION OF A DIAGONAL SYSTEM MATRIX.
      IF(IPR.EQ.3) THEN
         NAME=' '
         CALL LCMNXT(IPSYS,NAME,NAMLCM,NAMMY,LCM)
         IF(LCM) THEN
*           IN-CORE STORAGE.
            IVEC=LCMIOF(IPSYS,TEXT12)
         ELSE
*           XSM STORAGE.
            CALL SETARA(BASE,LL4,IVEC)
            CALL LCMGET(IPSYS,TEXT12,BASE(IVEC))
         ENDIF
      ELSE
         CALL SETARA(BASE,LL4,IVEC)
         DO 10 I=1,LL4
   10    BASE(IVEC+I-1)=0.0
      ENDIF
*
* COMPUTE THE DIAGONAL SYSTEM MATRIX.
      IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
*        VARIATIONAL COLLOCATION METHOD.
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC,IT)
         CALL SETARA(BASE,LC,ITS)
         CALL LCMGET(IPTRK,'T',BASE(IT))
         CALL LCMGET(IPTRK,'TS',BASE(ITS))
         CALL LCMSIX(IPTRK,' ',2)
         CALL TRIASP(NEL,LL4,CYLIND,SGD,BASE(IXORZ),BASE(IDD),VOL,MAT,
     1   IBASE(IKN),LC,BASE(IT),BASE(ITS),BASE(IVEC))
         CALL RLSARA(BASE(ITS))
         CALL RLSARA(BASE(IT))
      ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
*        MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL SETARA(BASE,4,IR)
         CALL SETARA(BASE,36,IRH)
         CALL SETARA(BASE,9,IRT)
         CALL LCMGET(IPTRK,'R',BASE(IR))
         CALL LCMGET(IPTRK,'RH',BASE(IRH))
         CALL LCMGET(IPTRK,'RT',BASE(IRT))
         CALL LCMSIX(IPTRK,' ',2)
         CALL TRIAHP(ISPLH,NEL,LL4,SGD,SIDE,BASE(IXORZ),VOL,MAT,
     1   IBASE(IKN),BASE(IR),BASE(IRH),BASE(IRT),BASE(IVEC))
         CALL RLSARA(BASE(IRT))
         CALL RLSARA(BASE(IRH))
         CALL RLSARA(BASE(IR))
      ELSE IF(.NOT.CHEX) THEN
*        DUAL FINITE ELEMENT METHOD.
         IDIM=1
         IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6).OR.(ITYPE.EQ.8)) IDIM=2
         IF((ITYPE.EQ.7).OR.(ITYPE.EQ.9)) IDIM=3
         CALL SETARA(BASE,NEL,IGAR)
         CALL TRIASD(IELEM,ICHX,IDIM,NEL,NUN,CYLIND,SGD,VOL,MAT,
     1   IBASE(IKN),BASE(IVEC),BASE(IGAR))
         CALL RLSARA(BASE(IGAR))
      ELSE IF(CHEX.AND.(ISPLH.EQ.1)) THEN
*        MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
         CALL TRIAHD(NEL,SGD,VOL,MAT,BASE(IVEC))
      ELSE IF(CHEX.AND.(ISPLH.GT.1)) THEN
*        MESH CENTERED FINITE DIFFERENCES IN TRIANGULAR GEOMETRY.
         CALL TRIMTD(ISPLH,NEL,VOL,MAT,SGD,IBASE(IKN),IBASE(IIPW),
     1   BASE(IVEC))
      ENDIF
*
* STORAGE OF THE DIAGONAL SYSTEM MATRIX.
      CALL LCMPOF(IPSYS,TEXT12,LL4,2,IVEC)
*
* RELEASE TRIVAC SPECIFIC TRACKING INFORMATION.
      IF(CHEX) THEN
         CALL RLSARA(IBASE(IIPW))
      ELSE
         CALL RLSARA(BASE(IDD))
      ENDIF
      CALL RLSARA(BASE(IXORZ))
      CALL RLSARA(IBASE(IKN))
      RETURN
      END
