*DECK TRIDKN
      SUBROUTINE TRIDKN(IMPX,LX,LY,LZ,CYLIND,IELEM,ICOL,L4,NCODE,ZCODE,
     1 MAT,VOL,XXX,YYY,ZZZ,XX,YY,ZZ,DD,KN,QFR,IDL,IP)
*
*-----------------------------------------------------------------------
*
* NUMBERING CORRESPONDING TO A MIXED-DUAL FORMULATION OF THE FINITE-
* ELEMENT DISCRETIZATION IN A 3-D GEOMETRY.
*
* INPUT PARAMETERS:
*  IMPX   : PRINT PARAMETER.
*  LX,LY,LZ : NUMBER OF ELEMENTS ALONG THE X, Y AND Z AXIS.
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
*  IELEM  : DEGREE OF THE LAGRANGIAN FINITE ELEMENTS. =1 (LINEAR);
*           =2 (PARABOLIC); =3 (CUBIC); =4 (QUARTIC).
*  ICOL   : TYPE OF QUADRATURE. =1 (ANALYTICAL INTEGRATION);
*           =2 (GAUSS-LOBATTO); =3 (GAUSS-LEGENDRE).
*  NCODE  : NCODE(I) IS THE TYPE OF BOUNDARY CONDITION APPLIED ON SIDE I
*           I=1: X-  I=2: X+  I=3: Y-  I=4: Y+  I=5: Z-  I=6: Z+
*           NCODE(I)=1: VOID   NCODE(I)=2: REFL   NCODE(I)=4: TRAN
*           NCODE(I)=5: SYME   NCODE(I)=7: ZERO   NCODE(I)=8: CYLI
*  ZCODE  : ZCODE(I) IS THE ALBEDO CORRESPONDING TO BOUNDARY CONDITION
*           'VOID' ON SIDE I. ZCODE(I)=0.0 BY DEFAULT.
*           DIMENSION NCODE(6),ZCODE(6)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT.
*           DIMENSION MAT(LX*LY*LZ)
*  XXX,YYY: CARTESIAN COORDINATES ALONG THE X, Y AND Z AXIS.
*  ZZZ      DIMENSION XXX(LX+1),YYY(LY+1),ZZZ(LZ+1)
*
* OUTPUT PARAMETERS:
*  L4     : TOTAL NUMBER OF UNKNOWN (VARIATIONAL COEFFICIENTS) PER
*           ENERGY GROUP. ORDER OF SYSTEM MATRICES.
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(LX*LY*LZ)
*  XX,YY,ZZ : X-, Y- AND Z-DIRECTED MESH SPACINGS.
*           DIMENSION XX(LX*LY*LZ),YY(LX*LY*LZ),ZZ(LX*LY*LZ)
*  DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(LX*LY*LZ)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(7*LX*LY*LZ)
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS.
*           DIMENSION QFR(6*LX*LY*LZ)
*  IDL    : POSITION OF INTEGRATED FLUXES INTO UNKNOWN VECTOR.
*           DIMENSION IDL(LX*LY*LZ)
*
* SCRATCH STORAGE:
*  IP     : DIMENSION IP(MAXIP) WHERE MAXIP=(LX+1)*LY*LZ*IELEM*IELEM
*           +LX*(LY+1)*LZ*IELEM*IELEM+LX*LY*(LZ+1)*IELEM*IELEM
*           +LX*LY*LZ*IELEM*IELEM*IELEM
*
*------------------------------------- AUTHOR: A. HEBERT ; 18/08/92 ----
*
      LOGICAL CYLIND,COND,LL1
      DIMENSION NCODE(*),ZCODE(*),MAT(*),VOL(*),XXX(*),YYY(*),ZZZ(*),
     1 XX(*),YY(*),ZZ(*),DD(*),KN(*),QFR(*),IDL(*),IP(*),ZALB(6)
*
* IDENTIFICATION OF THE GEOMETRY. MAIN LOOP OVER THE ELEMENTS.
      DO 5 I=1,6
      IF(ZCODE(I).NE.1.0) THEN
         ZALB(I)=2.0*(1.0+ZCODE(I))/(1.0-ZCODE(I))
      ELSE
         ZALB(I)=1.0E20
      ENDIF
    5 CONTINUE
      IF(IMPX.GT.0) WRITE(6,700) LX,LY,LZ
      L2=LX*LY*LZ
      DO 10 NUM1=1,7*L2
   10 KN(NUM1)=0
      NUM1=0
      NUM2=0
      KEL=0
      DO 150 K0=1,LZ
      DO 150 K1=1,LY
      DO 150 K2=1,LX
      KEL=KEL+1
      XX(KEL)=0.0
      YY(KEL)=0.0
      ZZ(KEL)=0.0
      VOL(KEL)=0.0
      IF(MAT(KEL).EQ.0) GO TO 150
      XX(KEL)=XXX(K2+1)-XXX(K2)
      YY(KEL)=YYY(K1+1)-YYY(K1)
      ZZ(KEL)=ZZZ(K0+1)-ZZZ(K0)
      IF(CYLIND) DD(KEL)=0.5*(XXX(K2)+XXX(K2+1))
      IND0=(K0-1)*(2*LX*LY+(LX+1)*LY+(LY+1)*LX)
      IND1=(K1-1)*(3*LX+1)
      KN(NUM1+1)=IND0+LX*LY+IND1+LX+2*K2
      KN(NUM1+2)=IND0+LX*LY+IND1+LX+2*K2-1
      KN(NUM1+3)=IND0+LX*LY+IND1+LX+2*K2+1
      KN(NUM1+4)=IND0+LX*LY+IND1+K2
      KN(NUM1+5)=IND0+LX*LY+IND1+3*LX+K2+1
      KN(NUM1+6)=IND0+(K1-1)*LX+K2
      KN(NUM1+7)=IND0+2*LX*LY+(LX+1)*LY+(LY+1)*LX+(K1-1)*LX+K2
      DO 20 IC=1,6
   20 QFR(NUM2+IC)=0.0
      FRX=1.0
      FRY=1.0
      FRZ=1.0
*
* VOID, REFL OR ZERO BOUNDARY CONTITION.
      IF(K2.EQ.1) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KEL-1).EQ.0)
      ENDIF
      IF(LL1) THEN
         COND=(NCODE(1).EQ.2).OR.((NCODE(1).EQ.1).AND.(ZCODE(1).EQ.1.0))
         IF(COND) THEN
            KN(NUM1+2)=0
         ELSE IF(NCODE(1).EQ.1) THEN
            QFR(NUM2+1)=ZALB(1)
         ENDIF
      ENDIF
*
      IF(K2.EQ.LX) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KEL+1).EQ.0)
      ENDIF
      IF(LL1) THEN
         COND=(NCODE(2).EQ.2).OR.((NCODE(2).EQ.1).AND.(ZCODE(2).EQ.1.0))
         IF(COND) THEN
            KN(NUM1+3)=0
         ELSE IF(NCODE(2).EQ.1) THEN
            QFR(NUM2+2)=ZALB(2)
         ENDIF
      ENDIF
*
      IF(K1.EQ.1) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KEL-LX).EQ.0)
      ENDIF
      IF(LL1) THEN
         COND=(NCODE(3).EQ.2).OR.((NCODE(3).EQ.1).AND.(ZCODE(3).EQ.1.0))
         IF(COND) THEN
            KN(NUM1+4)=0
         ELSE IF(NCODE(3).EQ.1) THEN
            QFR(NUM2+3)=ZALB(3)
         ENDIF
      ENDIF
*
      IF(K1.EQ.LY) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KEL+LX).EQ.0)
      ENDIF
      IF(LL1) THEN
         COND=(NCODE(4).EQ.2).OR.((NCODE(4).EQ.1).AND.(ZCODE(4).EQ.1.0))
         IF(COND) THEN
            KN(NUM1+5)=0
         ELSE IF(NCODE(4).EQ.1) THEN
            QFR(NUM2+4)=ZALB(4)
         ENDIF
      ENDIF
*
      IF(K0.EQ.1) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KEL-LX*LY).EQ.0)
      ENDIF
      IF(LL1) THEN
         COND=(NCODE(5).EQ.2).OR.((NCODE(5).EQ.1).AND.(ZCODE(5).EQ.1.0))
         IF(COND) THEN
            KN(NUM1+6)=0
         ELSE IF(NCODE(5).EQ.1) THEN
            QFR(NUM2+5)=ZALB(5)
         ENDIF
      ENDIF
*
      IF(K0.EQ.LZ) THEN
         LL1=.TRUE.
      ELSE
         LL1=(MAT(KEL+LX*LY).EQ.0)
      ENDIF
      IF(LL1) THEN
         COND=(NCODE(6).EQ.2).OR.((NCODE(6).EQ.1).AND.(ZCODE(6).EQ.1.0))
         IF(COND) THEN
            KN(NUM1+7)=0
         ELSE IF(NCODE(6).EQ.1) THEN
            QFR(NUM2+6)=ZALB(6)
         ENDIF
      ENDIF
*
* TRAN BOUNDARY CONDITION.
      IF((K2.EQ.LX).AND.(NCODE(2).EQ.4)) THEN
         KN(NUM1+3)=KN(NUM1+3)-2*LX
      ENDIF
      IF((K1.EQ.LY).AND.(NCODE(4).EQ.4)) THEN
         KN(NUM1+5)=KN(NUM1+5)-2*LX*LY-(LX+1)*LY
      ENDIF
      IF((K0.EQ.LZ).AND.(NCODE(6).EQ.4)) THEN
         KN(NUM1+7)=(K1-1)*LX+K2
      ENDIF
*
* SYME BOUNDARY CONDITION.
      IF((NCODE(1).EQ.5).AND.(K2.EQ.1)) THEN
         QFR(NUM2+1)=QFR(NUM2+2)
         FRX=0.5
         KN(NUM1+2)=-KN(NUM1+3)
      ELSE IF((NCODE(2).EQ.5).AND.(K2.EQ.LX)) THEN
         QFR(NUM2+2)=QFR(NUM2+1)
         FRX=0.5
         KN(NUM1+3)=-KN(NUM1+2)
      ENDIF
      IF((NCODE(3).EQ.5).AND.(K1.EQ.1)) THEN
         QFR(NUM2+3)=QFR(NUM2+4)
         FRY=0.5
         KN(NUM1+4)=-KN(NUM1+5)
      ELSE IF((NCODE(4).EQ.5).AND.(K1.EQ.LY)) THEN
         QFR(NUM2+4)=QFR(NUM2+3)
         FRY=0.5
         KN(NUM1+5)=-KN(NUM1+4)
      ENDIF
      IF((NCODE(5).EQ.5).AND.(K0.EQ.1)) THEN
         QFR(NUM2+5)=QFR(NUM2+6)
         FRZ=0.5
         KN(NUM1+6)=-KN(NUM1+7)
      ELSE IF((NCODE(6).EQ.5).AND.(K0.EQ.LZ)) THEN
         QFR(NUM2+6)=QFR(NUM2+5)
         FRZ=0.5
         KN(NUM1+7)=-KN(NUM1+6)
      ENDIF
*
      VOL0=XX(KEL)*YY(KEL)*ZZ(KEL)*FRX*FRY*FRZ
      IF(CYLIND) VOL0=6.2831853072*DD(KEL)*VOL0
      VOL(KEL)=VOL0
      QFR(NUM2+1)=QFR(NUM2+1)*VOL0/XX(KEL)
      QFR(NUM2+2)=QFR(NUM2+2)*VOL0/XX(KEL)
      QFR(NUM2+3)=QFR(NUM2+3)*VOL0/YY(KEL)
      QFR(NUM2+4)=QFR(NUM2+4)*VOL0/YY(KEL)
      QFR(NUM2+5)=QFR(NUM2+5)*VOL0/ZZ(KEL)
      QFR(NUM2+6)=QFR(NUM2+6)*VOL0/ZZ(KEL)
      NUM1=NUM1+7
      NUM2=NUM2+6
  150 CONTINUE
* END OF THE MAIN LOOP OVER ELEMENTS.
*
* REMOVING THE UNUSED UNKNOWNS INDICES FROM KN.
      LL4=LX*LY*LZ+(LX+1)*LY*LZ+LX*(LY+1)*LZ+LX*LY*(LZ+1)
      DO 160 IND=1,LL4
  160 IP(IND)=0
      DO 170 NUM1=1,7*L2
      IF(KN(NUM1).NE.0) IP(ABS(KN(NUM1)))=1
  170 CONTINUE
      L4=0
      DO 180 IND=1,LL4
      IF(IP(IND).EQ.1) THEN
         L4=L4+1
         IP(IND)=L4
      ENDIF
  180 CONTINUE
      DO 190 NUM1=1,7*L2
      IF(KN(NUM1).NE.0) KN(NUM1)=SIGN(IP(ABS(KN(NUM1))),KN(NUM1))
  190 CONTINUE
*
* PROCESS CASES WITH IELEM.GT.1.
      IF(IELEM.GT.1) THEN
         LL4=0
         DO 220 IND=1,L4
         IP(IND)=LL4+1
         NUM1=0
         DO 210 KEL=1,L2
         IF(MAT(KEL).EQ.0) GO TO 210
         IF(ABS(KN(NUM1+1)).EQ.IND) THEN
            LL4=LL4+IELEM**3
            GO TO 220
         ELSE
            DO 200 I=2,7
            IF(ABS(KN(NUM1+I)).EQ.IND) THEN
               LL4=LL4+IELEM**2
               GO TO 220
            ENDIF
  200       CONTINUE
         ENDIF
         NUM1=NUM1+7
  210    CONTINUE
         CALL XABORT('TRIDKN: FAILURE OF THE RENUMBERING ALGORITHM.')
  220    CONTINUE
         L4=LL4
         DO 230 NUM1=1,7*L2
         IF(KN(NUM1).NE.0) KN(NUM1)=SIGN(IP(ABS(KN(NUM1))),KN(NUM1))
  230    CONTINUE
      ENDIF
      NUM1=0
      DO 235 KEL=1,L2
      IDL(KEL)=0
      IF(MAT(KEL).EQ.0) GO TO 235
      IDL(KEL)=KN(NUM1+1)
      NUM1=NUM1+7
  235 CONTINUE
*
      IF(IMPX.GT.0) WRITE(6,710) L4
      IF(IMPX.GT.2) THEN
         WRITE(6,715) (VOL(I),I=1,L2)
         NUM1=0
         NUM2=0
         WRITE (6,750)
         DO 500 K=1,L2
         IF(MAT(K).EQ.0) GO TO 500
         WRITE (6,755) K,(KN(NUM1+I),I=1,7),(QFR(NUM2+I),I=1,6)
         NUM1=NUM1+7
         NUM2=NUM2+6
  500    CONTINUE
      ENDIF
      RETURN
*
  700 FORMAT(/42H TRIDKN: MIXED-DUAL FINITE ELEMENT METHOD.//7H NUMBER,
     1 27H OF ELEMENTS ALONG X AXIS =,I3/20X,14HALONG Y AXIS =,I3/
     2 20X,14HALONG Z AXIS =,I3)
  710 FORMAT(31H NUMBER OF UNKNOWNS PER GROUP =,I6)
  715 FORMAT(/20H VOLUMES PER ELEMENT/(1X,1P,10E13.4))
  750 FORMAT(/22H NUMBERING OF UNKNOWNS/1X,21(1H-)//
     1 8H ELEMENT,3X,7HNUMBERS,56X,23HVOID BOUNDARY CONDITION)
  755 FORMAT (1X,I6,5X,7I8,4X,1P,6E10.1)
      END
