*DECK TRIDXX
      SUBROUTINE TRIDXX(IR,CYLIND,IELEM,ICOL,IDIM,NEL,LL4,MAT,VOL,XX,
     1 YY,ZZ,DD,KN,QFR,SGD,XSGD,MUX,IPX,IPR,LC,R,V,A11X)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF SYSTEM MATRICES FOR A DUAL FINITE ELEMENT METHOD IN 3-D.
*
* INPUT PARAMETERS:
*  IR     : FIRST DIMENSION OF MATRICES SGD AND XSGD.
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
*  IELEM  : DEGREE OF THE LAGRANGIAN FINITE ELEMENTS. =1 (LINEAR);
*           =2 (PARABOLIC); =3 (CUBIC); =4 (QUARTIC).
*  ICOL   : TYPE OF QUADRATURE. =1 (ANALYTICAL INTEGRATION);
*           =2 (GAUSS-LOBATTO); =3 (GAUSS-LEGENDRE).
*  IDIM   : NUMBER OF DIMENSIONS (=1, 2 OR 3).
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  LL4    : ORDER OF SYSTEM MATRICES.
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  XX,YY,ZZ : X-, Y- AND Z-DIRECTED MESH SPACINGS.
*           DIMENSION XX(NEL),YY(NEL),ZZ(NEL)
*  DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(7*NEL)
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(6*NEL)
*  SGD    : NUCLEAR PROPERTIES BY MATERIAL MIXTURE. DIMENSION SGD(IR,4)
*           SGD(L,1) : X-ORIENTED DIFFUSION COEFFICIENTS
*           SGD(L,2) : Y-ORIENTED DIFFUSION COEFFICIENTS
*           SGD(L,3) : Z-ORIENTED DIFFUSION COEFFICIENTS
*           SGD(L,4) : REMOVAL MACROSCOPIC CROSS SECTION.
*  XSGD   : 1 OVER NUCLEAR PROPERTIES. DIMENSION XSGD(IR,4)
*  MUX,MUY,MUZ : COMPRESSED STORAGE MODE INDICES.
*                DIMENSION MUX(LL4),MUY(LL4),MUZ(LL4)
*  IPX,IPY,IPZ : PERMUTATION MATRICES.
*                DIMENSION IPX(LL4),IPY(LL4),IPZ(LL4)
*  IPR    : =0: STANDARD CALCULATION ; .GT.0: PERTURBATION CALCULATION.
*  LC     : ORDER OF THE UNIT MATRICES.
*  R,V    : UNIT MATRICES. DIMENSION R(LC,LC),V(LC,LC-1)
*
* OUTPUT PARAMETERS:
*  A11X,A11Y,A11Z : MATRICES CORRESPONDING TO THE DIVERGENCE (I.E.
*                   LEAKAGE) AND REMOVAL TERMS.
*           DIMENSION A11X(MUX(LL4)),A11Y(MUY(LL4)),A11Z(MUZ(LL4))
*
* NOTE: SYSTEM MATRICES SHOULD BE INITIALIZED BY THE CALLING PROGRAM.
*
*------------------------------------- AUTHOR: A. HEBERT ; 19/08/92 ----
*
      LOGICAL CYLIND
      DIMENSION MAT(*),VOL(*),XX(*),YY(*),ZZ(*),DD(*),KN(*),QFR(*),
     1 SGD(IR,*),XSGD(IR,*),MUX(*),MUY(*),MUZ(*),IPX(*),IPY(*),IPZ(*),
     2 A11X(*),A11Y(*),A11Z(*),R(LC,*),V(LC,*),QQ(5,5)
*
* X-ORIENTED COUPLINGS. ASSEMBLY OF MATRIX A11X.
      IF((CYLIND).AND.((IELEM.GT.1).OR.(ICOL.NE.2)))
     1 CALL XABORT('TRIDXX: TYPE OF DISCRETIZATION NOT IMPLEMENTED.')
      DO 10 I0=1,IELEM
      DO 10 J0=1,IELEM
      QQ(I0,J0)=0.0
      DO 10 K0=2,IELEM
   10 QQ(I0,J0)=QQ(I0,J0)+V(K0,I0)*V(K0,J0)/R(K0,K0)
*
      NUM1=0
      NUM2=0
      DO 120 IE=1,NEL
      L=MAT(IE)
      IF(L.EQ.0) GO TO 120
      VOL0=VOL(IE)
      IF(VOL0.EQ.0.0) GO TO 110
      DX=XX(IE)
      DY=YY(IE)
      DZ=ZZ(IE)
      IF(CYLIND) THEN
         DIN=1.0-0.5*DX/DD(IE)
         DOT=1.0+0.5*DX/DD(IE)
      ELSE
         DIN=1.0
         DOT=1.0
      ENDIF
*
      DO 40 K3=0,IELEM-1
      DO 40 K2=0,IELEM-1
      INX1=IPX(ABS(KN(NUM1+2))+K3*IELEM+K2)
      INX2=IPX(ABS(KN(NUM1+3))+K3*IELEM+K2)
      DO 30 K1=0,IELEM-1
      IND1=KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1
      JND1=IPX(IND1)
      KEY=MUX(JND1)
      A11X(KEY)=A11X(KEY)+VOL0*SGD(L,4)
      DO 20 L1=0,K1
      IF(QQ(K1+1,L1+1).EQ.0.0) GO TO 20
      KND1=IPX(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+L1)
      KEY=MUX(JND1)-JND1+KND1
      A11X(KEY)=A11X(KEY)+VOL0*QQ(K1+1,L1+1)*SGD(L,1)/(DX*DX)
   20 CONTINUE
      IF((IPR.EQ.0).AND.(KN(NUM1+2).NE.0)) THEN
         IF(JND1.GT.INX1) KEY=MUX(JND1)-JND1+INX1
         IF(JND1.LT.INX1) KEY=MUX(INX1)-INX1+JND1
         SG=REAL(SIGN(1,KN(NUM1+2)))
         A11X(KEY)=A11X(KEY)+SG*(VOL0/DX)*DIN*V(1,K1+1)
      ENDIF
      IF((IPR.EQ.0).AND.(KN(NUM1+3).NE.0)) THEN
         IF(INX2.GT.JND1) KEY=MUX(INX2)-INX2+JND1
         IF(INX2.LT.JND1) KEY=MUX(JND1)-JND1+INX2
         SG=REAL(SIGN(1,KN(NUM1+3)))
         A11X(KEY)=A11X(KEY)+SG*(VOL0/DX)*DOT*V(IELEM+1,K1+1)
      ENDIF
   30 CONTINUE
      IF(KN(NUM1+2).NE.0) THEN
         KEY=MUX(INX1)
         A11X(KEY)=A11X(KEY)-DIN*(VOL0*R(1,1)*XSGD(L,1)+QFR(NUM2+1))
      ENDIF
      IF(KN(NUM1+3).NE.0) THEN
         KEY=MUX(INX2)
         A11X(KEY)=A11X(KEY)-DOT*(VOL0*R(IELEM+1,IELEM+1)*XSGD(L,1)
     1   +QFR(NUM2+2))
      ENDIF
      IF((ICOL.NE.2).AND.(KN(NUM1+2).NE.0).AND.(KN(NUM1+3).NE.0)) THEN
         IF(INX2.GT.INX1) KEY=MUX(INX2)-INX2+INX1
         IF(INX2.LE.INX1) KEY=MUX(INX1)-INX1+INX2
         SG=REAL(SIGN(1,KN(NUM1+2))*SIGN(1,KN(NUM1+3)))
         IF(INX1.EQ.INX2) SG=2.0*SG
         A11X(KEY)=A11X(KEY)-SG*VOL0*R(IELEM+1,1)*XSGD(L,1)
      ENDIF
   40 CONTINUE
      IF(IDIM.EQ.1) GO TO 110
*
      DO 70 K3=0,IELEM-1
      DO 70 K1=0,IELEM-1
      DO 60 K2=0,IELEM-1
      KEY=MUX(IPX(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1))
   60 A11X(KEY)=A11X(KEY)+VOL0*QQ(K2+1,K2+1)*SGD(L,2)/(DY*DY)
      IF(KN(NUM1+4).NE.0) THEN
         KEY=MUX(IPX(ABS(KN(NUM1+4))+K3*IELEM+K1))
         A11X(KEY)=A11X(KEY)-VOL0*R(1,1)*XSGD(L,2)-QFR(NUM2+3)
      ENDIF
      IF(KN(NUM1+5).NE.0) THEN
         KEY=MUX(IPX(ABS(KN(NUM1+5))+K3*IELEM+K1))
         A11X(KEY)=A11X(KEY)-VOL0*R(IELEM+1,IELEM+1)*XSGD(L,2)
     1   -QFR(NUM2+4)
      ENDIF
      IF((ICOL.NE.2).AND.(KN(NUM1+4).NE.0).AND.(KN(NUM1+5).NE.0)) THEN
         INY1=IPX(ABS(KN(NUM1+4))+K3*IELEM+K1)
         INY2=IPX(ABS(KN(NUM1+5))+K3*IELEM+K1)
         IF(INY1.EQ.INY2) THEN
            KEY=MUX(INY1)
            SG=2.0*REAL(SIGN(1,KN(NUM1+4))*SIGN(1,KN(NUM1+5)))
            A11X(KEY)=A11X(KEY)-SG*VOL0*R(IELEM+1,1)*XSGD(L,2)
         ENDIF
      ENDIF
   70 CONTINUE
      IF(IDIM.EQ.2) GO TO 110
*
      DO 100 K2=0,IELEM-1
      DO 100 K1=0,IELEM-1
      DO 90 K3=0,IELEM-1
      KEY=MUX(IPX(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1))
   90 A11X(KEY)=A11X(KEY)+VOL0*QQ(K3+1,K3+1)*SGD(L,3)/(DZ*DZ)
      IF(KN(NUM1+6).NE.0) THEN
         KEY=MUX(IPX(ABS(KN(NUM1+6))+K2*IELEM+K1))
         A11X(KEY)=A11X(KEY)-VOL0*R(1,1)*XSGD(L,3)-QFR(NUM2+5)
      ENDIF
      IF(KN(NUM1+7).NE.0) THEN
         KEY=MUX(IPX(ABS(KN(NUM1+7))+K2*IELEM+K1))
         A11X(KEY)=A11X(KEY)-VOL0*R(IELEM+1,IELEM+1)*XSGD(L,3)
     1   -QFR(NUM2+6)
      ENDIF
      IF((ICOL.NE.2).AND.(KN(NUM1+6).NE.0).AND.(KN(NUM1+7).NE.0)) THEN
         INZ1=IPX(ABS(KN(NUM1+6))+K2*IELEM+K1)
         INZ2=IPX(ABS(KN(NUM1+7))+K2*IELEM+K1)
         IF(INZ1.EQ.INZ2) THEN
            KEY=MUX(INZ1)
            SG=2.0*REAL(SIGN(1,KN(NUM1+6))*SIGN(1,KN(NUM1+7)))
            A11X(KEY)=A11X(KEY)-SG*VOL0*R(IELEM+1,1)*XSGD(L,3)
         ENDIF
      ENDIF
  100 CONTINUE
  110 NUM1=NUM1+7
      NUM2=NUM2+6
  120 CONTINUE
      RETURN
*
* Y-ORIENTED COUPLINGS. ASSEMBLY OF MATRIX A11Y.
      ENTRY TRIDXY(IR,CYLIND,IELEM,ICOL,IDIM,NEL,LL4,MAT,VOL,XX,YY,ZZ,
     1 DD,KN,QFR,SGD,XSGD,MUY,IPY,IPR,LC,R,V,A11Y)
      DO 130 I0=1,IELEM
      DO 130 J0=1,IELEM
      QQ(I0,J0)=0.0
      DO 130 K0=2,IELEM
  130 QQ(I0,J0)=QQ(I0,J0)+V(K0,I0)*V(K0,J0)/R(K0,K0)
*
      NUM1=0
      NUM2=0
      DO 240 IE=1,NEL
      L=MAT(IE)
      IF(L.EQ.0) GO TO 240
      VOL0=VOL(IE)
      IF(VOL0.EQ.0.0) GO TO 230
      DX=XX(IE)
      DY=YY(IE)
      DZ=ZZ(IE)
      IF(CYLIND) THEN
         DIN=1.0-0.5*DX/DD(IE)
         DOT=1.0+0.5*DX/DD(IE)
      ELSE
         DIN=1.0
         DOT=1.0
      ENDIF
*
      DO 160 K3=0,IELEM-1
      DO 160 K2=0,IELEM-1
      DO 150 K1=0,IELEM-1
      KEY=MUY(IPY(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1))
  150 A11Y(KEY)=A11Y(KEY)+VOL0*(SGD(L,4)+QQ(K1+1,K1+1)*SGD(L,1)/DX/DX)
      IF(KN(NUM1+2).NE.0) THEN
         KEY=MUY(IPY(ABS(KN(NUM1+2))+K3*IELEM+K2))
         A11Y(KEY)=A11Y(KEY)-DIN*(VOL0*R(1,1)*XSGD(L,1)+QFR(NUM2+1))
      ENDIF
      IF(KN(NUM1+3).NE.0) THEN
         KEY=MUY(IPY(ABS(KN(NUM1+3))+K3*IELEM+K2))
         A11Y(KEY)=A11Y(KEY)-DOT*(VOL0*R(IELEM+1,IELEM+1)*XSGD(L,1)
     1   +QFR(NUM2+2))
      ENDIF
      IF((ICOL.NE.2).AND.(KN(NUM1+2).NE.0).AND.(KN(NUM1+3).NE.0)) THEN
         INX1=IPY(ABS(KN(NUM1+2))+K3*IELEM+K2)
         INX2=IPY(ABS(KN(NUM1+3))+K3*IELEM+K2)
         IF(INX1.EQ.INX2) THEN
            KEY=MUY(INX1)
            SG=2.0*REAL(SIGN(1,KN(NUM1+2))*SIGN(1,KN(NUM1+3)))
            A11Y(KEY)=A11Y(KEY)-SG*VOL0*R(IELEM+1,1)*XSGD(L,1)
         ENDIF
      ENDIF
  160 CONTINUE
*
      DO 190 K3=0,IELEM-1
      DO 190 K1=0,IELEM-1
      INY1=IPY(ABS(KN(NUM1+4))+K3*IELEM+K1)
      INY2=IPY(ABS(KN(NUM1+5))+K3*IELEM+K1)
      DO 180 K2=0,IELEM-1
      JND1=IPY(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1)
      DO 170 L2=0,K2
      IF(QQ(K2+1,L2+1).EQ.0.0) GO TO 170
      KND1=IPY(KN(NUM1+1)+(K3*IELEM+L2)*IELEM+K1)
      KEY=MUY(JND1)-JND1+KND1
      A11Y(KEY)=A11Y(KEY)+VOL0*QQ(K2+1,L2+1)*SGD(L,2)/(DY*DY)
  170 CONTINUE
      IF((IPR.EQ.0).AND.(KN(NUM1+4).NE.0)) THEN
         IF(JND1.GT.INY1) KEY=MUY(JND1)-JND1+INY1
         IF(JND1.LT.INY1) KEY=MUY(INY1)-INY1+JND1
         SG=REAL(SIGN(1,KN(NUM1+4)))
         A11Y(KEY)=A11Y(KEY)+SG*(VOL0/DY)*V(1,K2+1)
      ENDIF
      IF((IPR.EQ.0).AND.(KN(NUM1+5).NE.0)) THEN
         IF(INY2.GT.JND1) KEY=MUY(INY2)-INY2+JND1
         IF(INY2.LT.JND1) KEY=MUY(JND1)-JND1+INY2
         SG=REAL(SIGN(1,KN(NUM1+5)))
         A11Y(KEY)=A11Y(KEY)+SG*(VOL0/DY)*V(IELEM+1,K2+1)
      ENDIF
  180 CONTINUE
      IF(KN(NUM1+4).NE.0) THEN
         KEY=MUY(INY1)
         A11Y(KEY)=A11Y(KEY)-VOL0*R(1,1)*XSGD(L,2)-QFR(NUM2+3)
      ENDIF
      IF(KN(NUM1+5).NE.0) THEN
         KEY=MUY(INY2)
         A11Y(KEY)=A11Y(KEY)-VOL0*R(IELEM+1,IELEM+1)*XSGD(L,2)
     1   -QFR(NUM2+4)
      ENDIF
      IF((ICOL.NE.2).AND.(KN(NUM1+4).NE.0).AND.(KN(NUM1+5).NE.0)) THEN
         IF(INY2.GT.INY1) KEY=MUY(INY2)-INY2+INY1
         IF(INY2.LE.INY1) KEY=MUY(INY1)-INY1+INY2
         SG=REAL(SIGN(1,KN(NUM1+4))*SIGN(1,KN(NUM1+5)))
         IF(INY1.EQ.INY2) SG=2.0*SG
         A11Y(KEY)=A11Y(KEY)-SG*VOL0*R(IELEM+1,1)*XSGD(L,2)
      ENDIF
  190 CONTINUE
      IF(IDIM.EQ.2) GO TO 230
*
      DO 220 K2=0,IELEM-1
      DO 220 K1=0,IELEM-1
      DO 210 K3=0,IELEM-1
      KEY=MUY(IPY(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1))
  210 A11Y(KEY)=A11Y(KEY)+VOL0*QQ(K3+1,K3+1)*SGD(L,3)/(DZ*DZ)
      IF(KN(NUM1+6).NE.0) THEN
         KEY=MUY(IPY(ABS(KN(NUM1+6))+K2*IELEM+K1))
         A11Y(KEY)=A11Y(KEY)-VOL0*R(1,1)*XSGD(L,3)-QFR(NUM2+5)
      ENDIF
      IF(KN(NUM1+7).NE.0) THEN
         KEY=MUY(IPY(ABS(KN(NUM1+7))+K2*IELEM+K1))
         A11Y(KEY)=A11Y(KEY)-VOL0*R(IELEM+1,IELEM+1)*XSGD(L,3)
     1   -QFR(NUM2+6)
      ENDIF
      IF((ICOL.NE.2).AND.(KN(NUM1+6).NE.0).AND.(KN(NUM1+7).NE.0)) THEN
         INZ1=IPY(ABS(KN(NUM1+6))+K2*IELEM+K1)
         INZ2=IPY(ABS(KN(NUM1+7))+K2*IELEM+K1)
         IF(INZ1.EQ.INZ2) THEN
            KEY=MUY(INZ1)
            SG=2.0*REAL(SIGN(1,KN(NUM1+6))*SIGN(1,KN(NUM1+7)))
            A11Y(KEY)=A11Y(KEY)-SG*VOL0*R(IELEM+1,1)*XSGD(L,3)
         ENDIF
      ENDIF
  220 CONTINUE
  230 NUM1=NUM1+7
      NUM2=NUM2+6
  240 CONTINUE
      RETURN
*
* Z-ORIENTED COUPLINGS. ASSEMBLY OF MATRIX A11Z.
      ENTRY TRIDXZ(IR,CYLIND,IELEM,ICOL,IDIM,NEL,LL4,MAT,VOL,XX,YY,ZZ,
     1 DD,KN,QFR,SGD,XSGD,MUZ,IPZ,IPR,LC,R,V,A11Z)
      DO 250 I0=1,IELEM
      DO 250 J0=1,IELEM
      QQ(I0,J0)=0.0
      DO 250 K0=2,IELEM
  250 QQ(I0,J0)=QQ(I0,J0)+V(K0,I0)*V(K0,J0)/R(K0,K0)
*
      NUM1=0
      NUM2=0
      DO 360 IE=1,NEL
      L=MAT(IE)
      IF(L.EQ.0) GO TO 360
      VOL0=VOL(IE)
      IF(VOL0.EQ.0.0) GO TO 350
      DX=XX(IE)
      DY=YY(IE)
      DZ=ZZ(IE)
      IF(CYLIND) THEN
         DIN=1.0-0.5*DX/DD(IE)
         DOT=1.0+0.5*DX/DD(IE)
      ELSE
         DIN=1.0
         DOT=1.0
      ENDIF
*
      DO 280 K3=0,IELEM-1
      DO 280 K2=0,IELEM-1
      DO 270 K1=0,IELEM-1
      KEY=MUZ(IPZ(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1))
  270 A11Z(KEY)=A11Z(KEY)+VOL0*(SGD(L,4)+QQ(K1+1,K1+1)*SGD(L,1)/DX/DX)
      IF(KN(NUM1+2).NE.0) THEN
         KEY=MUZ(IPZ(ABS(KN(NUM1+2))+K3*IELEM+K2))
         A11Z(KEY)=A11Z(KEY)-DIN*(VOL0*R(1,1)*XSGD(L,1)+QFR(NUM2+1))
      ENDIF
      IF(KN(NUM1+3).NE.0) THEN
         KEY=MUZ(IPZ(ABS(KN(NUM1+3))+K3*IELEM+K2))
         A11Z(KEY)=A11Z(KEY)-DOT*(VOL0*R(IELEM+1,IELEM+1)*XSGD(L,1)
     1   +QFR(NUM2+2))
      ENDIF
      IF((ICOL.NE.2).AND.(KN(NUM1+2).NE.0).AND.(KN(NUM1+3).NE.0)) THEN
         INX1=IPZ(ABS(KN(NUM1+2))+K3*IELEM+K2)
         INX2=IPZ(ABS(KN(NUM1+3))+K3*IELEM+K2)
         IF(INX1.EQ.INX2) THEN
            KEY=MUZ(INX1)
            SG=2.0*REAL(SIGN(1,KN(NUM1+2))*SIGN(1,KN(NUM1+3)))
            A11Z(KEY)=A11Z(KEY)-SG*VOL0*R(IELEM+1,1)*XSGD(L,1)
         ENDIF
      ENDIF
  280 CONTINUE
*
      DO 310 K3=0,IELEM-1
      DO 310 K1=0,IELEM-1
      DO 300 K2=0,IELEM-1
      KEY=MUZ(IPZ(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1))
  300 A11Z(KEY)=A11Z(KEY)+VOL0*QQ(K2+1,K2+1)*SGD(L,2)/(DY*DY)
      IF(KN(NUM1+4).NE.0) THEN
         KEY=MUZ(IPZ(ABS(KN(NUM1+4))+K3*IELEM+K1))
         A11Z(KEY)=A11Z(KEY)-VOL0*R(1,1)*XSGD(L,2)-QFR(NUM2+3)
      ENDIF
      IF(KN(NUM1+5).NE.0) THEN
         KEY=MUZ(IPZ(ABS(KN(NUM1+5))+K3*IELEM+K1))
         A11Z(KEY)=A11Z(KEY)-VOL0*R(IELEM+1,IELEM+1)*XSGD(L,2)
     1   -QFR(NUM2+4)
      ENDIF
      IF((ICOL.NE.2).AND.(KN(NUM1+4).NE.0).AND.(KN(NUM1+5).NE.0)) THEN
         INY1=IPZ(ABS(KN(NUM1+4))+K3*IELEM+K1)
         INY2=IPZ(ABS(KN(NUM1+5))+K3*IELEM+K1)
         IF(INY1.EQ.INY2) THEN
            KEY=MUZ(INY1)
            SG=2.0*REAL(SIGN(1,KN(NUM1+4))*SIGN(1,KN(NUM1+5)))
            A11Z(KEY)=A11Z(KEY)-SG*VOL0*R(IELEM+1,1)*XSGD(L,2)
         ENDIF
      ENDIF
  310 CONTINUE
*
      DO 340 K2=0,IELEM-1
      DO 340 K1=0,IELEM-1
      INZ1=IPZ(ABS(KN(NUM1+6))+K2*IELEM+K1)
      INZ2=IPZ(ABS(KN(NUM1+7))+K2*IELEM+K1)
      DO 330 K3=0,IELEM-1
      JND1=IPZ(KN(NUM1+1)+(K3*IELEM+K2)*IELEM+K1)
      DO 320 L3=0,K3
      IF(QQ(K3+1,L3+1).EQ.0.0) GO TO 320
      KND1=IPZ(KN(NUM1+1)+(L3*IELEM+K2)*IELEM+K1)
      KEY=MUZ(JND1)-JND1+KND1
      A11Z(KEY)=A11Z(KEY)+VOL0*QQ(K3+1,L3+1)*SGD(L,3)/(DZ*DZ)
  320 CONTINUE
      IF((IPR.EQ.0).AND.(KN(NUM1+6).NE.0)) THEN
         IF(JND1.GT.INZ1) KEY=MUZ(JND1)-JND1+INZ1
         IF(JND1.LT.INZ1) KEY=MUZ(INZ1)-INZ1+JND1
         SG=REAL(SIGN(1,KN(NUM1+6)))
         A11Z(KEY)=A11Z(KEY)+SG*(VOL0/DZ)*V(1,K3+1)
      ENDIF
      IF((IPR.EQ.0).AND.(KN(NUM1+7).NE.0)) THEN
         IF(INZ2.GT.JND1) KEY=MUZ(INZ2)-INZ2+JND1
         IF(INZ2.LT.JND1) KEY=MUZ(JND1)-JND1+INZ2
         SG=REAL(SIGN(1,KN(NUM1+7)))
         A11Z(KEY)=A11Z(KEY)+SG*(VOL0/DZ)*V(IELEM+1,K3+1)
      ENDIF
  330 CONTINUE
      IF(KN(NUM1+6).NE.0) THEN
         KEY=MUZ(INZ1)
         A11Z(KEY)=A11Z(KEY)-VOL0*R(1,1)*XSGD(L,3)-QFR(NUM2+5)
      ENDIF
      IF(KN(NUM1+7).NE.0) THEN
         KEY=MUZ(INZ2)
         A11Z(KEY)=A11Z(KEY)-VOL0*R(IELEM+1,IELEM+1)*XSGD(L,3)
     1   -QFR(NUM2+6)
      ENDIF
      IF((ICOL.NE.2).AND.(KN(NUM1+6).NE.0).AND.(KN(NUM1+7).NE.0)) THEN
         IF(INZ2.GT.INZ1) KEY=MUZ(INZ2)-INZ2+INZ1
         IF(INZ2.LE.INZ1) KEY=MUZ(INZ1)-INZ1+INZ2
         SG=REAL(SIGN(1,KN(NUM1+6))*SIGN(1,KN(NUM1+7)))
         IF(INZ1.EQ.INZ2) SG=2.0*SG
         A11Z(KEY)=A11Z(KEY)-SG*VOL0*R(IELEM+1,1)*XSGD(L,3)
      ENDIF
  340 CONTINUE
  350 NUM1=NUM1+7
      NUM2=NUM2+6
  360 CONTINUE
      RETURN
      END
