*DECK TRIHCO
      SUBROUTINE TRIHCO (IR,K,VOL0,MAT,DIF,DDF,SIDE,ZZ,KN,QFR,IWRK,IPR,
     1 A)
*
*-----------------------------------------------------------------------
*
* COMPUTE THE VALUE OR THE DERIVATIVE OR VARIATION OF MESH CENTERED
* FINITE DIFFERENCE COEFFICIENTS IN ELEMENT K FOR HEXAGONAL GEOMETRY.
*
* INPUT PARAMETERS:
*  IR     : FIRST DIMENSION OF MATRIX DIF.
*  K      : INDEX OF FINITE ELEMENT UNDER CONSIDERATION.
*  VOL0   : VOLUME OF FINITE ELEMENT UNDER CONSIDERATION.
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*           WHERE NEL IS THE TOTAL NUMBER OF FINITE ELEMENTS.
*  DIF    : DIRECTIONAL DIFFUSION COEFFICIENTS. DIMENSION DIF(IR,3)
*  DDF    : DERIVATIVE OR VARIATION OF DIRECTIONAL DIFFUSION COEFFI-
*           CIENTS. DIMENSION DDF(IR,3)
*  SIDE   : SIDE OF THE HEXAGONS.
*  ZZ     : Z-DIRECTED MESH SPACINGS. DIMENSION ZZ(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(6)
*           .GT.0 : NEIGHBOUR INDEX;
*           =-1   : VOID/ALBEDO BOUNDARY CONDITION;
*           =-2   : REFLECTION BOUNDARY CONDITION;
*           =-3   : ZERO FLUX BOUNDARY CONDITION;
*           =-4   : SYME BOUNDARY CONDITION (AXIAL SYMMETRY).
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(6)
*  IPR    : .EQ.0 DIRECT MCFD COEFFICIENTS CALCULATION;
*           .EQ.1 TAKE DERIVATIVE OF MCFD COEFFICIENTS;
*           .GE.2 TAKE VARIATION OF MCFD COEFFICIENTS.
*
* OUTPUT PARAMETER:
*  A      : VALUE OR DERIVATIVE OR VARIATION OF MESH CENTERED FINITE
*           DIRRERENCE COEFFICIENTS. DIMENSION A(6)
*
*----------------------------------- AUTHOR: A. BENABOUD ; 09/10/91 ----
*
      DOUBLE PRECISION A(8),SHARM,VHARM,DHARM
      DIMENSION MAT(*),DIF(IR,*),DDF(IR,*),ZZ(*),KN(*),QFR(*),IWRK(*)
C     FORMULE DIRECTE:
      SHARM(X1,X2,DIF1,DIF2)=2.0D0*DIF1*DIF2/(X1*DIF2+X2*DIF1)
C     FORMULE DE VARIATION:
      VHARM(X1,X2,DIF1,DIF2,DDF1,DDF2)=2.0D0*((DIF1+DDF1)*(DIF2+DDF2)
     1 /(X1*(DIF2+DDF2)+X2*(DIF1+DDF1))-DIF1*DIF2/(X1*DIF2+X2*DIF1))
C     FORMULE DE DERIVEE:
      DHARM(X1,X2,DIF1,DIF2,DDF1,DDF2)=2.0D0*(X1*DIF2*DIF2*DDF1+
     1 X2*DIF1*DIF1*DDF2)/(X1*DIF2+X2*DIF1)**2
C
      DENOM=2.0
      L=MAT(K)
      DZ=ZZ(K)
      DS=SQRT(3.0)*SIDE
      IF(IPR.EQ.0) THEN
C        COTE W NEGATIF:
         KK1=KN(6)
         IF(KK1.GT.0) THEN
            A(6)=SHARM(DS,DS,DIF(L,1),DIF(MAT(IWRK(KK1)),1))*DZ*SIDE
         ELSE IF(KK1.EQ.-1) THEN
            A(6)=SHARM(DS,DS,DIF(L,1),DS*QFR(6)/DENOM)*DZ*SIDE
         ELSE IF(KK1.EQ.-2) THEN
            A(6)=0.0D0
         ELSE IF(KK1.EQ.-3) THEN
            A(6)=2.0D0*SHARM(DS,DS,DIF(L,1),DIF(L,1))*DZ*SIDE
         ENDIF
C        COTE W POSITIF:
         KK2=KN(3)
         IF(KK2.GT.0) THEN
            A(3)=SHARM(DS,DS,DIF(L,1),DIF(MAT(IWRK(KK2)),1))*DZ*SIDE
         ELSE IF(KK2.EQ.-1) THEN
            A(3)=SHARM(DS,DS,DIF(L,1),DS*QFR(3)/DENOM)*DZ*SIDE
         ELSE IF(KK2.EQ.-2) THEN
            A(3)=0.0D0
         ELSE IF(KK2.EQ.-3) THEN
            A(3)=2.0D0*SHARM(DS,DS,DIF(L,1),DIF(L,1))*DZ*SIDE
         ENDIF
C        COTE X NEGATIF:
         KK3=KN(1)
         IF(KK3.GT.0) THEN
            A(1)=SHARM(DS,DS,DIF(L,1),DIF(MAT(IWRK(KK3)),1))*DZ*SIDE
         ELSE IF(KK3.EQ.-1) THEN
            A(1)=SHARM(DS,DS,DIF(L,1),DS*QFR(1)/DENOM)*DZ*SIDE
         ELSE IF(KK3.EQ.-2) THEN
            A(1)=0.0D0
         ELSE IF(KK3.EQ.-3) THEN
            A(1)=2.0D0*SHARM(DS,DS,DIF(L,1),DIF(L,1))*DZ*SIDE
         ENDIF
C        COTE X POSITIF:
         KK4=KN(4)
         IF(KK4.GT.0) THEN
            A(4)=SHARM(DS,DS,DIF(L,1),DIF(MAT(IWRK(KK4)),1))*DZ*SIDE
         ELSE IF(KK4.EQ.-1) THEN
            A(4)=SHARM(DS,DS,DIF(L,1),DS*QFR(4)/DENOM)*DZ*SIDE
         ELSE IF(KK4.EQ.-2) THEN
            A(4)=0.0D0
         ELSE IF(KK4.EQ.-3) THEN
            A(4)=2.0D0*SHARM(DS,DS,DIF(L,1),DIF(L,1))*DZ*SIDE
         ENDIF
C        COTE Y NEGATIF:
         KK5=KN(2)
         IF(KK5.GT.0) THEN
            A(2)=SHARM(DS,DS,DIF(L,1),DIF(MAT(IWRK(KK5)),1))*DZ*SIDE
         ELSE IF(KK5.EQ.-1) THEN
            A(2)=SHARM(DS,DS,DIF(L,1),DS*QFR(2)/DENOM)*DZ*SIDE
         ELSE IF(KK5.EQ.-2) THEN
            A(2)=0.0D0
         ELSE IF(KK5.EQ.-3) THEN
            A(2)=2.0D0*SHARM(DS,DS,DIF(L,1),DIF(L,1))*DZ*SIDE
         ENDIF
C        COTE Y POSITIF:
         KK6=KN(5)
         IF(KK6.GT.0) THEN
            A(5)=SHARM(DS,DS,DIF(L,1),DIF(MAT(IWRK(KK6)),1))*DZ*SIDE
         ELSE IF(KK6.EQ.-1) THEN
            A(5)=SHARM(DS,DS,DIF(L,1),DS*QFR(5)/DENOM)*DZ*SIDE
         ELSE IF(KK6.EQ.-2) THEN
            A(5)=0.0D0
         ELSE IF(KK6.EQ.-3) THEN
            A(5)=2.0D0*SHARM(DS,DS,DIF(L,1),DIF(L,1))*DZ*SIDE
         ENDIF
C        COTE Z NEGATIF:
         KK7=KN(7)
         IF(KK7.GT.0) THEN
            A(7)=SHARM(DZ,ZZ(IWRK(KK7)),DIF(L,3),DIF(MAT(IWRK(KK7)),3))
     *      *VOL0/DZ
         ELSE IF(KK7.EQ.-1) THEN
            A(7)=SHARM(DZ,DZ,DIF(L,3),DZ*QFR(7)/DENOM)*VOL0/DZ
         ELSE IF(KK7.EQ.-2) THEN
            A(7)=0.0D0
         ELSE IF(KK7.EQ.-3) THEN
            A(7)=2.0D0*SHARM(DZ,DZ,DIF(L,3),DIF(L,3))*VOL0/DZ
         ENDIF
C        COTE Z POSITIF:
         KK8=KN(8)
         IF(KK8.GT.0) THEN
            A(8)=SHARM(DZ,ZZ(IWRK(KK8)),DIF(L,3),DIF(MAT(IWRK(KK8)),3))
     *      *VOL0/DZ
         ELSE IF(KK8.EQ.-1) THEN
            A(8)=SHARM(DZ,DZ,DIF(L,3),DZ*QFR(8)/DENOM)*VOL0/DZ
         ELSE IF(KK8.EQ.-2) THEN
            A(8)=0.0D0
         ELSE IF(KK8.EQ.-3) THEN
            A(8)=2.0D0*SHARM(DZ,DZ,DIF(L,3),DIF(L,3))*VOL0/DZ
         ENDIF
      ELSE IF(IPR.EQ.1) THEN
C        FORMULE DE DERIVEE.
C        COTE W NEGATIF:
         KK1=KN(6)
         IF(KK1.GT.0) THEN
            KK1=IWRK(KK1)
            A(6)=DHARM(DS,DS,DIF(L,1),DIF(MAT(KK1),1),DDF(L,1),
     1           DDF(MAT(KK1),1))*DZ*SIDE
         ELSE IF(KK1.EQ.-1) THEN
            A(6)=DHARM(DS,DS,DIF(L,1),DS*QFR(6)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK1.EQ.-2) THEN
            A(6)=0.0D0
         ELSE IF(KK1.EQ.-3) THEN
            A(6)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE W POSITIF:
         KK2=KN(3)
         IF(KK2.GT.0) THEN
            KK2=IWRK(KK2)
            A(3)=DHARM(DS,DS,DIF(L,1),DIF(MAT(KK2),1),DDF(L,1),
     1           DDF(MAT(KK2),1))*DZ*SIDE
         ELSE IF(KK2.EQ.-1) THEN
            A(3)=DHARM(DS,DS,DIF(L,1),DS*QFR(3)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK2.EQ.-2) THEN
            A(3)=0.0D0
         ELSE IF(KK2.EQ.-3) THEN
            A(3)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE X NEGATIF:
         KK3=KN(1)
         IF(KK3.GT.0) THEN
            KK3=IWRK(KK3)
            A(1)=DHARM(DS,DS,DIF(L,1),DIF(MAT(KK3),1),DDF(L,1),
     1           DDF(MAT(KK3),1))*DZ*SIDE
         ELSE IF(KK3.EQ.-1) THEN
            A(1)=DHARM(DS,DS,DIF(L,1),DS*QFR(1)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK3.EQ.-2) THEN
            A(3)=0.0D0
         ELSE IF(KK3.EQ.-3) THEN
            A(3)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE X POSITIF:
         KK4=KN(4)
         IF(KK4.GT.0) THEN
            KK4=IWRK(KK4)
            A(4)=DHARM(DS,DS,DIF(L,1),DIF(MAT(KK4),1),DDF(L,1),
     1           DDF(MAT(KK4),1))*DZ*SIDE
         ELSE IF(KK4.EQ.-1) THEN
            A(4)=DHARM(DS,DS,DIF(L,1),DS*QFR(4)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK4.EQ.-2) THEN
            A(4)=0.0D0
         ELSE IF(KK4.EQ.-3) THEN
            A(4)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE Y NEGATIF:
         KK5=KN(2)
         IF(KK5.GT.0) THEN
            KK5=IWRK(KK5)
            A(2)=DHARM(DS,DS,DIF(L,1),DIF(MAT(KK5),1),DDF(L,1),
     1           DDF(MAT(KK5),1))*DZ*SIDE
         ELSE IF(KK5.EQ.-1) THEN
            A(2)=DHARM(DS,DS,DIF(L,1),DZ*QFR(2)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK5.EQ.-2) THEN
            A(2)=0.0D0
         ELSE IF(KK5.EQ.-3) THEN
            A(2)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE Y POSITIF:
         KK6=KN(5)
         IF(KK6.GT.0) THEN
            KK6=IWRK(KK6)
            A(5)=DHARM(DS,DS,DIF(L,1),DIF(MAT(KK6),1),DDF(L,1),
     1           DDF(MAT(KK6),1))*DZ*SIDE
         ELSE IF(KK6.EQ.-1) THEN
            A(5)=DHARM(DS,DS,DIF(L,1),DS*QFR(5)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK6.EQ.-2) THEN
            A(5)=0.0D0
         ELSE IF(KK6.EQ.-3) THEN
            A(5)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE Z NEGATIF:
         KK7=KN(7)
         IF(KK7.GT.0) THEN
            KK7=IWRK(KK7)
            A(7)=DHARM(DZ,ZZ(KK7),DIF(L,3),DIF(MAT(KK7),3),DDF(L,3),
     1           DDF(MAT(KK7),3))*VOL0/DZ
         ELSE IF(KK7.EQ.-1) THEN
            A(7)=DHARM(DZ,DZ,DIF(L,3),DZ*QFR(7)/DENOM,DDF(L,3),0.0)
     1           *VOL0/DZ
         ELSE IF(KK7.EQ.-2) THEN
            A(7)=0.0D0
         ELSE IF(KK7.EQ.-3) THEN
            A(7)=2.0D0*DDF(L,3)*VOL0/(DZ*DZ)
         ENDIF
C        COTE Z POSITIF:
         KK8=KN(8)
         IF(KK8.GT.0) THEN
            KK8=IWRK(KK8)
            A(8)=DHARM(DZ,ZZ(KK8),DIF(L,3),DIF(MAT(KK8),3),DDF(L,3),
     1           DDF(MAT(KK8),3))*VOL0/DZ
         ELSE IF(KK8.EQ.-1) THEN
            A(8)=DHARM(DZ,DZ,DIF(L,3),DZ*QFR(8)/DENOM,DDF(L,3),0.0)
     1           *VOL0/DZ
         ELSE IF(KK8.EQ.-2) THEN
            A(8)=0.0D0
         ELSE IF(KK8.EQ.-3) THEN
            A(8)=2.0D0*DDF(L,3)*VOL0/(DZ*DZ)
         ENDIF
      ELSE
C        FORMULE DE VARIATION.
C        COTE W NEGATIF:
         KK1=KN(6)
         IF(KK1.GT.0) THEN
            KK1=IWRK(KK1)
            A(6)=VHARM(DS,DS,DIF(L,1),DIF(MAT(KK1),1),DDF(L,1),
     1           DDF(MAT(KK1),1))*DZ*SIDE
         ELSE IF(KK1.EQ.-1) THEN
            A(6)=VHARM(DS,DS,DIF(L,1),DS*QFR(6)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK1.EQ.-2) THEN
            A(6)=0.0D0
         ELSE IF(KK1.EQ.-3) THEN
            A(6)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE W POSITIF:
         KK2=KN(3)
         IF(KK2.GT.0) THEN
            KK2=IWRK(KK2)
            A(3)=VHARM(DS,DS,DIF(L,1),DIF(MAT(KK2),1),DDF(L,1),
     1           DDF(MAT(KK2),1))*DZ*SIDE
         ELSE IF(KK2.EQ.-1) THEN
            A(3)=VHARM(DS,DS,DIF(L,1),DS*QFR(3)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK2.EQ.-2) THEN
            A(3)=0.0D0
         ELSE IF(KK2.EQ.-3) THEN
            A(3)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE X NEGATIF:
         KK3=KN(1)
         IF(KK3.GT.0) THEN
            KK3=IWRK(KK3)
            A(1)=VHARM(DS,DS,DIF(L,1),DIF(MAT(KK3),1),DDF(L,1),
     1           DDF(MAT(KK3),1))*DZ*SIDE
         ELSE IF(KK3.EQ.-1) THEN
            A(1)=VHARM(DS,DS,DIF(L,1),DS*QFR(1)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK3.EQ.-2) THEN
            A(1)=0.0D0
         ELSE IF(KK3.EQ.-3) THEN
            A(1)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE X POSITIF:
         KK4=KN(4)
         IF(KK4.GT.0) THEN
            KK4=IWRK(KK4)
            A(4)=VHARM(DS,DS,DIF(L,1),DIF(MAT(KK4),1),DDF(L,1),
     1           DDF(MAT(KK4),1))*DZ*SIDE
         ELSE IF(KK4.EQ.-1) THEN
            A(4)=VHARM(DS,DS,DIF(L,1),DS*QFR(4)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK4.EQ.-2) THEN
            A(4)=0.0D0
         ELSE IF(KK4.EQ.-3) THEN
            A(4)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE Y NEGATIF:
         KK5=KN(2)
         IF(KK5.GT.0) THEN
            KK5=IWRK(KK5)
            A(2)=VHARM(DS,DS,DIF(L,1),DIF(MAT(KK5),1),DDF(L,1),
     1           DDF(MAT(KK5),1))*DZ*SIDE
         ELSE IF(KK5.EQ.-1) THEN
            A(2)=VHARM(DS,DS,DIF(L,1),DS*QFR(2)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK5.EQ.-2) THEN
            A(2)=0.0D0
         ELSE IF(KK5.EQ.-3) THEN
            A(2)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE Y POSITIF:
         KK6=KN(5)
         IF(KK6.GT.0) THEN
            KK6=IWRK(KK6)
            A(5)=VHARM(DS,DS,DIF(L,1),DIF(MAT(KK6),1),DDF(L,1),
     1           DDF(MAT(KK6),1))*DZ*SIDE
         ELSE IF(KK6.EQ.-1) THEN
            A(5)=VHARM(DS,DS,DIF(L,1),DS*QFR(5)/DENOM,DDF(L,1),0.0)
     1           *DZ*SIDE
         ELSE IF(KK6.EQ.-2) THEN
            A(5)=0.0D0
         ELSE IF(KK6.EQ.-3) THEN
            A(5)=2.0D0*DDF(L,1)*DZ*SIDE/DS
         ENDIF
C        COTE Z NEGATIF:
         KK7=KN(7)
         IF(KK7.GT.0) THEN
            KK7=IWRK(KK7)
            A(7)=VHARM(DZ,ZZ(KK7),DIF(L,3),DIF(MAT(KK7),3),DDF(L,3),
     1           DDF(MAT(KK7),3))*VOL0/DZ
         ELSE IF(KK7.EQ.-1) THEN
            A(7)=VHARM(DZ,DZ,DIF(L,3),DZ*QFR(7)/DENOM,DDF(L,3),0.0)
     1           *VOL0/DZ
         ELSE IF(KK7.EQ.-2) THEN
            A(7)=0.0D0
         ELSE IF(KK7.EQ.-3) THEN
            A(7)=2.0D0*DDF(L,3)*VOL0/(DZ*DZ)
         ENDIF
C        COTE Z POSITIF:
         KK8=KN(8)
         IF(KK8.GT.0) THEN
            KK8=IWRK(KK8)
            A(8)=VHARM(DZ,ZZ(KK8),DIF(L,3),DIF(MAT(KK8),3),DDF(L,3),
     1           DDF(MAT(KK8),3))*VOL0/DZ
         ELSE IF(KK8.EQ.-1) THEN
            A(8)=VHARM(DZ,DZ,DIF(L,3),DZ*QFR(8)/DENOM,DDF(L,3),0.0)
     1           *VOL0/DZ
         ELSE IF(KK8.EQ.-2) THEN
            A(8)=0.0D0
         ELSE IF(KK8.EQ.-3) THEN
            A(8)=2.0D0*DDF(L,3)*VOL0/(DZ*DZ)
         ENDIF
      ENDIF
      RETURN
      END
