*DECK TRIIN2
      SUBROUTINE TRIIN2 (NEL,LL4,CYLIND,EVECT,XX,DD,MAT,VOL,IDL,KN,LC,
     1 T,TS)
*
*-----------------------------------------------------------------------
*
* COMPUTE THE INTEGRATED FLUX IN EACH FINITE ELEMENT FOR A LAGRANGIAN
* FINITE ELEMENT DISCRETIZATION IN CARTESIAN OR CYLINDRICAL GEOMETRY.
*
* INPUT PARAMETERS:
*  NEL    : NUMBER OF FINITE ELEMENTS.
*  LL4    : ORDER OF SYSTEM MATRICES.
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
*  EVECT  : UNKNOWN VECTOR CONTAINING THE VARIATIONAL COEFFICIENTS IN
*           LOCATIONS 1 TO LL4. DIMENSION EVECT(LL4+NEL)
*  XX     : X-DIRECTED MESH SPACINGS. DIMENSION XX(NEL)
*  DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(NEL)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT.DIMENSION MAT(NEL)
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  IDL    : INDICES POINTING TO INTEGRATED FLUXES IN EVECT.
*           DIMENSION IDL(NEL)
*  KN     : ELEMENT-ORDERED UNKNOEN LIST. DIMENSION KN(NEL*(IELEM+1)**3)
*           WHERE IELEM IS THE DEGREE OF THE FINITE ELEMENTS.
*  LC     : ORDER OF THE FINITE ELEMENT BASIS.
*  T,TS   : LINEAR PRODUCT VECTORS. DIMENSION T(LC),TS(LC)
*
* OUTPUT PARAMETER:
*  EVECT  : UNKNOWN VECTOR CONTAINING THE INTEGRATED FLUXES IN LOCATIONS
*           IDL(I). DIMENSION EVECT(LL4+NEL)
*
*------------------------------------- AUTHOR: A. HEBERT ; 15/01/86 ----
*
      LOGICAL CYLIND
      DIMENSION EVECT(*),XX(*),DD(*),MAT(*),VOL(*),IDL(*),KN(*),T(*),
     1 TS(*),IJ1(125),IJ2(125),IJ3(125)
*
      LL=LC*LC*LC
      DO 100 L=1,LL
      L1=1+MOD(L-1,LC)
      L2=1+(L-L1)/LC
      L3=1+MOD(L2-1,LC)
      IJ1(L)=L1
      IJ2(L)=L3
  100 IJ3(L)=1+(L2-L3)/LC
*
      NUM1=0
      DO 130 K=1,NEL
      IF(MAT(K).EQ.0) GO TO 130
      EVECT(IDL(K))=0.0
      IF(VOL(K).EQ.0.0) GO TO 120
      DO 110 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 110
      I1=IJ1(I)
      I2=IJ2(I)
      I3=IJ3(I)
      IF(CYLIND) THEN
         SS=(T(I1)+TS(I1)*XX(K)/DD(K))*T(I2)*T(I3)
      ELSE
         SS=T(I1)*T(I2)*T(I3)
      ENDIF
      EVECT(IDL(K))=EVECT(IDL(K))+SS*EVECT(IND1)
  110 CONTINUE
  120 NUM1=NUM1+LL
  130 CONTINUE
      RETURN
      END
