*DECK TRIKAX
      SUBROUTINE TRIKAX (IDIM,NCODE,XXX,YYY,ZZZ,LX,LY,LZ,IAXIS,CENTER)
*
*-----------------------------------------------------------------------
*
* CALCULATES THE CENTER OF THE EXTERNAL CYLINDER OUTSIDE ELEMENTS.
*
* INPUT PARAMETERS:
*   IDIM   : NUMBER OF DIMENSIONS.
*   XXX    : CARTESIAN COORDINATES OF THE DOMAIN ALONG THE X-AXIS.
*            DIMENSION XXX(LX+1)
*   YYY    : ... ALONG THE Y-AXIS. DIMENSION YYY(LY+1)
*   ZZZ    : ... ALONG THE Z-AXIS. DIMENSION ZZZ(LZ+1)
*   LX     : NUMBER OF PARALLELEPIPEDS ALONG THE X-AXIS AFTER MESH-
*            SPLITTING.
*   LY     : ... ALONG THE Y-AXIS.
*   LZ     : ... ALONG THE Z-AXIS.
*   NCODE  : BOUNDARY CONDITION RELATIVE TO EACH SIDE OF THE DOMAIN.
*            DIMENSION NCODE(6)
*
* OUTPUT PARAMETERS:
*   CENTER : COORDINATES FOR CENTER OF CYLINDER. DIMENSION CENTER(3)
*   IAXIS  : = 0 ( NO CYLINDER AT ALL )
*            = 1,2,3 ( AXIS OF THE CYLINDER )
*
*----------------------------------------- AUTHOR: R. ROY ---- 28/06/89
*
      DIMENSION XXX(*),YYY(*),ZZZ(*),NCODE(6),CENTER(3),IFC(3)
*
      IAXIS = 0
      DO 10 IC= 1,3
         CENTER(IC)= 0.0
         IFC(IC)= 0
   10 CONTINUE
      IF( IDIM.GE.2 )THEN
*
* "X" AXIS STUDY
*
         IF( NCODE(1).EQ.8.OR.NCODE(2).EQ.8 ) THEN
*        THERE IS AT LEAST ONE "X" CIRCULAR B.C.
            IFC(1)= 1
            IF( NCODE(1).EQ.8.AND.NCODE(2).EQ.8 )THEN
*           THERE IS TWO "X" CIRCULAR B.C.
               CENTER(1)= 0.5 * (XXX(LX+1) + XXX(1))
*              TAKE THE "X" CENTER AT THE MIDDLE OF ALL ELEMENTS
            ELSEIF( NCODE(1).EQ.5.OR.NCODE(2).EQ.5 )THEN
*           THERE IS ONE "X" SYMMETRIC B.C.
               IF( NCODE(1).EQ.5 )THEN
*              "X -" SYMMETRIC B.C.
                  CENTER(1)= 0.5 * (XXX(2) + XXX(1))
*                 TAKE THE "X" CENTER AT THE MIDDLE OF FIRST ELEMENT
               ELSE
*              "X +" SYMMETRIC B.C.
                  CENTER(1)= 0.5 * (XXX(LX+1) + XXX(LX))
*                 TAKE THE "X" CENTER AT THE MIDDLE OF LAST  ELEMENT
               ENDIF
            ELSE
*           ALL OTHER CASES
               IF( NCODE(1).EQ.8 )THEN
*              "X -" CIRCULAR B.C.
                  CENTER(1)= XXX(LX+1)
*                 TAKE THE "X" CENTER AT THE END    OF LAST  ELEMENT
               ELSE
*              "X +" SYMMETRIC B.C.
                  CENTER(1)= XXX(1)
*                 TAKE THE "X" CENTER AT THE BEGIN  OF FIRST ELEMENT
               ENDIF
            ENDIF
         ENDIF
*
* "Y" AXIS STUDY
*
         IF( NCODE(3).EQ.8.OR.NCODE(4).EQ.8 ) THEN
            IFC(2)= 1
*        THERE IS AT LEAST ONE "Y" CIRCULAR B.C.
            IF( NCODE(3).EQ.8.AND.NCODE(4).EQ.8 )THEN
*           THERE IS TWO "Y" CIRCULAR B.C.
               CENTER(2)= 0.5 * (YYY(LY+1) + YYY(1))
*              TAKE THE "Y" CENTER AT THE MIDDLE OF ALL ELEMENTS
            ELSEIF( NCODE(3).EQ.5.OR.NCODE(4).EQ.5 )THEN
*           THERE IS ONE "Y" SYMMETRIC B.C.
               IF( NCODE(3).EQ.5 )THEN
*              "Y -" SYMMETRIC B.C.
                  CENTER(2)= 0.5 * (YYY(2) + YYY(1))
*                 TAKE THE "Y" CENTER AT THE MIDDLE OF FIRST ELEMENT
               ELSE
*              "Y +" SYMMETRIC B.C.
                  CENTER(2)= 0.5 * (YYY(LY+1) + YYY(LY))
*                 TAKE THE "Y" CENTER AT THE MIDDLE OF LAST  ELEMENT
               ENDIF
            ELSE
*           ALL OTHER CASES
               IF( NCODE(3).EQ.8 )THEN
*              "Y -" CIRCULAR B.C.
                  CENTER(2)= YYY(LY+1)
*                 TAKE THE "Y" CENTER AT THE END    OF LAST  ELEMENT
               ELSE
*              "Y +" SYMMETRIC B.C.
                  CENTER(2)= YYY(1)
*                 TAKE THE "Y" CENTER AT THE BEGIN  OF FIRST ELEMENT
               ENDIF
            ENDIF
         ENDIF
         IF( IDIM.EQ.2 )THEN
            NONC  = IFC(1) + IFC(2)
            IF( NONC.GT.0 )THEN
               IAXIS = 3
            ENDIF
         ELSE
*
* "Z" AXIS STUDY
*
            IF( NCODE(5).EQ.8.OR.NCODE(6).EQ.8 ) THEN
*           THERE IS AT LEAST ONE "Y" CIRCULAR B.C.
               IFC(3)= 1
               IF( NCODE(5).EQ.8.AND.NCODE(6).EQ.8 )THEN
*              THERE IS TWO "Z" CIRCULAR B.C.
                  CENTER(3)= 0.5 * (ZZZ(LZ+1) + ZZZ(1))
*                 TAKE THE "Z" CENTER AT THE MIDDLE OF ALL ELEMENTS
               ELSEIF( NCODE(5).EQ.5.OR.NCODE(6).EQ.5 )THEN
*              THERE IS ONE "Z" SYMMETRIC B.C.
                  IF( NCODE(5).EQ.5 )THEN
*                 "Z -" SYMMETRIC B.C.
                     CENTER(3)= 0.5 * (ZZZ(2) + ZZZ(1))
*                    TAKE THE "Z" CENTER AT THE MIDDLE OF FIRST ELEMENT
                  ELSE
*                 "Z +" SYMMETRIC B.C.
                     CENTER(3)= 0.5 * (ZZZ(LZ+1) + ZZZ(LZ))
*                    TAKE THE "Z" CENTER AT THE MIDDLE OF LAST  ELEMENT
                  ENDIF
               ELSE
*              ALL OTHER CASES
                  IF( NCODE(5).EQ.8 )THEN
*                 "Z -" CIRCULAR B.C.
                     CENTER(3)= ZZZ(LZ+1)
*                    TAKE THE "Z" CENTER AT THE END    OF LAST  ELEMENT
                  ELSE
*                 "Z +" SYMMETRIC B.C.
                     CENTER(3)= ZZZ(1)
*                    TAKE THE "Z" CENTER AT THE BEGIN  OF FIRST ELEMENT
                  ENDIF
               ENDIF
            ENDIF
*
*           DETERMINE PRINCIPAL AXIS
            NONC= IFC(1) + IFC(2) + IFC(3)
            IF( NONC.GT.0 )THEN
               IF( NONC.EQ.2 )THEN
                  IF( IFC(1).EQ.0 ) IAXIS = 1
                  IF( IFC(2).EQ.0 ) IAXIS = 2
                  IF( IFC(3).EQ.0 ) IAXIS = 3
               ELSE
                  WRITE(6,1000)
                  CALL XABORT('TRIKAX: ALGORITHM FAILURE.')
               ENDIF
            ENDIF
         ENDIF
      ENDIF
      RETURN
 1000 FORMAT(/1X,'*** NOT POSSIBLE TO DETERMINE THE PRINCIPAL AXIS'
     1       /1X,'***'
     2       /1X,'***  N O   C Y L I N D R I C   A L B E D O S'
     3       /1X,'***')
      END
