*DECK TRILXX
      SUBROUTINE TRILXX(IR,CYLIND,IELEM,IDIM,NEL,LL4,VOL,MAT,SGD,XSGD,
     1 XX,YY,ZZ,DD,KN,QFR,MUX,IPX,IPR,A11X,IGAR)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF SYSTEM MATRICES FOR MESH CENTERED FINITE DIFFERENCES OR
* NODAL COLLOCATION METHOD WITH SERENDIPITY APPROXIMATION.
*
* INPUT PARAMETERS:
*  IR     : FIRST DIMENSION OF MATRICES SGD AND XSGD.
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
*  IELEM  : DEGREE OF THE POLYNOMIAL BASIS. =1 (LINEAR/FINITE DIFFEREN-
*           CES); =2 (PARABOLIC); =3 (CUBIC); =4 (QUARTIC).
*  IDIM   : NUMBER OF DIMENSIONS (=1, 2 OR 3).
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  LL4      ORDER OF SYSTEM MATRICES.
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  SGD    : NUCLEAR PROPERTIES BY MATERIAL MIXTURE. DIMENSION SGD(IR,4)
*           SGD(L,1) : X-ORIENTED DIFFUSION COEFFICIENTS
*           SGD(L,2) : Y-ORIENTED DIFFUSION COEFFICIENTS
*           SGD(L,3) : Z-ORIENTED DIFFUSION COEFFICIENTS
*           SGD(L,4) : REMOVAL MACROSCOPIC CROSS SECTION.
*  XSGD   : XSGD=SGD IF IPR=0 ;
*           DERIVATIVE OF NUCLEAR PROPERTIES IF IPR=1 ;
*           VARIATION OF NUCLEAR PROPERTIES IF IPR=2 OR IPR=3.
*           DIMENSION XSGD(IR,4)
*  XX,YY,ZZ : X-, Y- AND Z-DIRECTED MESH SPACINGS.
*           DIMENSION XX(NEL),YY(NEL),ZZ(NEL)
*  DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(6*NEL)
*           .GT.0 : NEIGHBOUR INDEX;
*           =-1   : VOID/ALBEDO BOUNDARY CONDITION;
*           =-2   : REFLECTION BOUNDARY CONDITION;
*           =-3   : ZERO FLUX BOUNDARY CONDITION;
*           =-4   : SYME BOUNDARY CONDITION (AXIAL SYMMETRY).
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(6*NEL)
*  MUX,MUY,MUZ : COMPRESSED STORAGE MODE INDICES.
*                DIMENSION MUX(LL4),MUY(LL4),MUZ(LL4)
*  IPX,IPY,IPZ : PERMUTATION MATRICES.
*                DIMENSION IPX(LL4),IPY(LL4),IPZ(LL4)
*  IPR    : =0: COMPUTE THE SYSTEM MATRICES ;
*           =1: COMPUTE THE DERIVATIVE OF SYSTEM MATRICES ;
*           =2 OR =3: COMPUTE THE VARIATION OF SYSTEM MATRICES ;
*
* OUTPUT PARAMETERS:
*  A11X,A11Y,A11Z : MATRICES CORRESPONDING TO THE DIVERGENCE (I.E.
*                   LEAKAGE) AND REMOVAL TERMS.
*           DIMENSION A11X(MUX(LL4)),A11Y(MUY(LL4)),A11Z(MUZ(LL4))
*
* SCRATCH STORAGE:
*  IGAR   : DIMENSION IGAR(NEL)
*
* NOTE: SYSTEM MATRICES SHOULD BE INITIALIZED BY THE CALLING PROGRAM.
*
*------------------------------------- AUTHOR: A. HEBERT ; 15/12/86 ----
*
      LOGICAL CYLIND,LOGIC
      DOUBLE PRECISION RLL,R,S,QQ,PAIR,A1(6,10)
      DIMENSION SGD(IR,*),XSGD(IR,*),VOL(*),MAT(*),XX(*),YY(*),ZZ(*),
     1 DD(*),KN(*),QFR(*),MUX(*),MUY(*),MUZ(*),IPX(*),IPY(*),IPZ(*),
     2 A11X(*),A11Y(*),A11Z(*),IGAR(*)
*
      IORD(J,K,L,LL,IEL,IW)=
     1 1+LL*(L*(IEL*(IEL+1))/2-(L*(L-1)*(3*IEL-L+2))/6
     2 +K*(IEL-L)-(K*(K-1))/2)+(IEL-K-L)*(IW-1)+J
*
* X-ORIENTED COUPLINGS. ASSEMBLY OF MATRIX A11X.
      LL=0
      DO 10 K=1,NEL
      IF(MAT(K).EQ.0) GO TO 10
      LL=LL+1
      IGAR(K)=LL
   10 CONTINUE
      NUM1=0
      DO 70 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 70
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 60
      DX=XX(K)
      DY=YY(K)
      DZ=ZZ(K)
*
      IF(IPR.EQ.0) THEN
         DO 11 IB=1,IELEM
         CALL TRICO (IB,IR,K,VOL0,MAT,XSGD(1,1),XX,YY,ZZ,DD,
     1   KN(NUM1+1),QFR(NUM1+1),CYLIND,A1(1,IB))
   11    CONTINUE
      ELSE IF(IPR.GE.1) THEN
         DO 12 IB=1,IELEM
         CALL TRIDCO (IB,IR,K,VOL0,MAT,SGD(1,1),XSGD(1,1),XX,YY,ZZ,
     1   DD,KN(NUM1+1),QFR(NUM1+1),CYLIND,IPR,A1(1,IB))
   12    CONTINUE
      ENDIF
      KK1=KN(NUM1+1)
      KK2=KN(NUM1+2)
      IF(KK1.EQ.-4) KK1=KK2
      IF(KK2.EQ.-4) KK2=KK1
*
      IF(IELEM.EQ.1) THEN
         IND1=IGAR(K)
         INX1=IPX(IND1)
         KEY0=MUX(INX1)-INX1
         IF(KK1.GT.0) THEN
            INX2=IPX(IGAR(KK1))
            IF(INX2.LT.INX1) THEN
               KEY=KEY0+INX2
               A11X(KEY)=A11X(KEY)-A1(1,1)
            ENDIF
         ENDIF
         IF(KK2.GT.0) THEN
            INX2=IPX(IGAR(KK2))
            IF(INX2.LT.INX1) THEN
               KEY=KEY0+INX2
               A11X(KEY)=A11X(KEY)-A1(2,1)
            ENDIF
         ENDIF
         KEY=KEY0+INX1
         A11X(KEY)=A11X(KEY)+A1(1,1)+A1(2,1)+A1(3,1)+A1(4,1)+A1(5,1)
     1   +A1(6,1)+XSGD(L,4)*VOL0
      ELSE
         DO 50 I3=0,IELEM-1
         DO 50 I2=0,IELEM-1-I3
         DO 40 I1=0,IELEM-1-I2-I3
         IND1=IORD(I1,I2,I3,LL,IELEM,IGAR(K))
         INX1=IPX(IND1)
         KEY0=MUX(INX1)-INX1
         IB=IELEM-I2-I3
         RLL=REAL(IB*(IB+1))
         QQ=SQRT(REAL(2*I1+1))*(RLL-REAL(I1*(I1+1)))/RLL
         IF(KK1.GT.0) THEN
            PAIR=(-1.0D0)**I1
            DO 20 I0=0,IELEM-1-I2-I3
            LOGIC=(KN((IGAR(KK1)-1)*6+1).NE.-4).OR.(MOD(I0+1,2).NE.0)
            INX2=IPX(IORD(I0,I2,I3,LL,IELEM,IGAR(KK1)))
            IF((INX2.LT.INX1).AND.LOGIC) THEN
               KEY=KEY0+INX2
               R=REAL(I0*(I0+1))
               S=SQRT(REAL(2*I0+1))
               A11X(KEY)=A11X(KEY)-0.5D0*QQ*PAIR*S*(RLL-R)*A1(1,IB)
            ENDIF
   20       CONTINUE
         ENDIF
         IF(KK2.GT.0) THEN
            DO 25 I0=0,IELEM-1-I2-I3
            INX2=IPX(IORD(I0,I2,I3,LL,IELEM,IGAR(KK2)))
            IF(INX2.LT.INX1) THEN
               PAIR=(-1.0D0)**I0
               IF(KN(NUM1+2).EQ.-4) PAIR=1.0D0
               KEY=KEY0+INX2
               R=REAL(I0*(I0+1))
               S=SQRT(REAL(2*I0+1))
               A11X(KEY)=A11X(KEY)-0.5D0*QQ*PAIR*S*(RLL-R)*A1(2,IB)
            ENDIF
   25       CONTINUE
         ENDIF
         KEY=KEY0+INX1-I1
         DO 30 I0=0,I1
         R=REAL(I0*(I0+1))
         S=SQRT(REAL(2*I0+1))
         PAIR=1.0D0+(-1.0D0)**(I0+I1)
   30    A11X(KEY+I0)=A11X(KEY+I0)+QQ*(PAIR*S*R*XSGD(L,1)*VOL0/(DX*DX)
     1   +0.5D0*S*(RLL-R)*((-1.0D0)**(I0+I1)*A1(1,IB)+A1(2,IB)))
*
         KEY=KEY0+INX1
         IB=IELEM-I1-I3
         RLL=REAL(IB*(IB+1))
         R=REAL(I2*(I2+1))
         QQ=REAL(2*I2+1)*(RLL-R)/RLL
         A11X(KEY)=A11X(KEY)+QQ*(2.0D0*R*XSGD(L,2)*VOL0/(DY*DY)
     1   +0.5D0*(RLL-R)*(A1(3,IB)+A1(4,IB)))
*
         IB=IELEM-I1-I2
         RLL=REAL(IB*(IB+1))
         R=REAL(I3*(I3+1))
         QQ=REAL(2*I3+1)*(RLL-R)/RLL
         A11X(KEY)=A11X(KEY)+QQ*(2.0D0*R*XSGD(L,3)*VOL0/(DZ*DZ)
     1   +0.5D0*(RLL-R)*(A1(5,IB)+A1(6,IB)))+XSGD(L,4)*VOL0
   40    CONTINUE
         IF((IDIM.EQ.1).AND.(I2.EQ.0)) GO TO 60
         IF((IDIM.EQ.2).AND.(I2.EQ.IELEM-1)) GO TO 60
   50    CONTINUE
      ENDIF
   60 NUM1=NUM1+6
   70 CONTINUE
      RETURN
*
* Y-ORIENTED COUPLINGS. ASSEMBLY OF MATRIX A11Y.
      ENTRY TRILXY(IR,CYLIND,IELEM,IDIM,NEL,LL4,VOL,MAT,SGD,XSGD,XX,YY,
     1 ZZ,DD,KN,QFR,MUY,IPY,IPR,A11Y,IGAR)
      LL=0
      DO 80 K=1,NEL
      IF(MAT(K).EQ.0) GO TO 80
      LL=LL+1
      IGAR(K)=LL
   80 CONTINUE
      NUM1=0
      DO 140 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 140
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 130
      DX=XX(K)
      DY=YY(K)
      DZ=ZZ(K)
*
      IF(IPR.EQ.0) THEN
         DO 81 IB=1,IELEM
         CALL TRICO (IB,IR,K,VOL0,MAT,XSGD(1,1),XX,YY,ZZ,DD,
     1   KN(NUM1+1),QFR(NUM1+1),CYLIND,A1(1,IB))
   81    CONTINUE
      ELSE IF(IPR.GE.1) THEN
         DO 82 IB=1,IELEM
         CALL TRIDCO (IB,IR,K,VOL0,MAT,SGD(1,1),XSGD(1,1),XX,YY,ZZ,
     1   DD,KN(NUM1+1),QFR(NUM1+1),CYLIND,IPR,A1(1,IB))
   82    CONTINUE
      ENDIF
      KK3=KN(NUM1+3)
      KK4=KN(NUM1+4)
      IF(KK3.EQ.-4) KK3=KK4
      IF(KK4.EQ.-4) KK4=KK3
*
      IF(IELEM.EQ.1) THEN
         INY1=IPY(IGAR(K))
         KEY0=MUY(INY1)-INY1
         IF(KK3.GT.0) THEN
            INY2=IPY(IGAR(KK3))
            IF(INY2.LT.INY1) THEN
               KEY=KEY0+INY2
               A11Y(KEY)=A11Y(KEY)-A1(3,1)
            ENDIF
         ENDIF
         IF(KK4.GT.0) THEN
            INY2=IPY(IGAR(KK4))
            IF(INY2.LT.INY1) THEN
               KEY=KEY0+INY2
               A11Y(KEY)=A11Y(KEY)-A1(4,1)
            ENDIF
         ENDIF
         KEY=KEY0+INY1
         A11Y(KEY)=A11Y(KEY)+A1(1,1)+A1(2,1)+A1(3,1)+A1(4,1)+A1(5,1)
     1   +A1(6,1)+XSGD(L,4)*VOL0
      ELSE
         DO 120 I3=0,IELEM-1
         DO 120 I2=0,IELEM-1-I3
         DO 110 I1=0,IELEM-1-I2-I3
         INY1=IPY(IORD(I2,I1,I3,LL,IELEM,IGAR(K)))
         KEY0=MUY(INY1)-INY1
         IB=IELEM-I2-I3
         RLL=REAL(IB*(IB+1))
         QQ=SQRT(REAL(2*I1+1))*(RLL-REAL(I1*(I1+1)))/RLL
         IF(KK3.GT.0) THEN
            PAIR=(-1.0D0)**I1
            DO 90 I0=0,IELEM-1-I2-I3
            LOGIC=(KN((IGAR(KK3)-1)*6+3).NE.-4).OR.(MOD(I0+1,2).NE.0)
            INY2=IPY(IORD(I2,I0,I3,LL,IELEM,IGAR(KK3)))
            IF((INY2.LT.INY1).AND.LOGIC) THEN
               KEY=KEY0+INY2
               R=REAL(I0*(I0+1))
               S=SQRT(REAL(2*I0+1))
               A11Y(KEY)=A11Y(KEY)-0.5D0*QQ*PAIR*S*(RLL-R)*A1(3,IB)
            ENDIF
   90       CONTINUE
         ENDIF
         IF(KK4.GT.0) THEN
            DO 95 I0=0,IELEM-1-I2-I3
            INY2=IPY(IORD(I2,I0,I3,LL,IELEM,IGAR(KK4)))
            IF(INY2.LT.INY1) THEN
               PAIR=(-1.0D0)**I0
               IF(KN(NUM1+4).EQ.-4) PAIR=1.0D0
               KEY=KEY0+INY2
               R=REAL(I0*(I0+1))
               S=SQRT(REAL(2*I0+1))
               A11Y(KEY)=A11Y(KEY)-0.5D0*QQ*PAIR*S*(RLL-R)*A1(4,IB)
            ENDIF
   95       CONTINUE
         ENDIF
         KEY=KEY0+INY1-I1
         DO 100 I0=0,I1
         R=REAL(I0*(I0+1))
         S=SQRT(REAL(2*I0+1))
         PAIR=1.0D0+(-1.0D0)**(I0+I1)
  100    A11Y(KEY+I0)=A11Y(KEY+I0)+QQ*(PAIR*S*R*XSGD(L,2)*VOL0/(DY*DY)
     1   +0.5D0*S*(RLL-R)*((-1.0D0)**(I0+I1)*A1(3,IB)+A1(4,IB)))
*
         KEY=KEY0+INY1
         IB=IELEM-I1-I3
         RLL=REAL(IB*(IB+1))
         R=REAL(I2*(I2+1))
         QQ=REAL(2*I2+1)*(RLL-R)/RLL
         A11Y(KEY)=A11Y(KEY)+QQ*(2.0D0*R*XSGD(L,1)*VOL0/(DX*DX)
     1   +0.5D0*(RLL-R)*(A1(1,IB)+A1(2,IB)))
*
         IB=IELEM-I1-I2
         RLL=REAL(IB*(IB+1))
         R=REAL(I3*(I3+1))
         QQ=REAL(2*I3+1)*(RLL-R)/RLL
         A11Y(KEY)=A11Y(KEY)+QQ*(2.0D0*R*XSGD(L,3)*VOL0/(DZ*DZ)
     1   +0.5D0*(RLL-R)*(A1(5,IB)+A1(6,IB)))+XSGD(L,4)*VOL0
*
  110    CONTINUE
         IF((IDIM.EQ.2).AND.(I2.EQ.IELEM-1)) GO TO 130
  120    CONTINUE
      ENDIF
  130 NUM1=NUM1+6
  140 CONTINUE
      RETURN
*
* Z-ORIENTED COUPLINGS. ASSEMBLY OF MATRIX A11Z.
      ENTRY TRILXZ(IR,CYLIND,IELEM,IDIM,NEL,LL4,VOL,MAT,SGD,XSGD,XX,YY,
     1 ZZ,DD,KN,QFR,MUZ,IPZ,IPR,A11Z,IGAR)
      LL=0
      DO 150 K=1,NEL
      IF(MAT(K).EQ.0) GO TO 150
      LL=LL+1
      IGAR(K)=LL
  150 CONTINUE
      NUM1=0
      DO 210 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 210
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 200
      DX=XX(K)
      DY=YY(K)
      DZ=ZZ(K)
*
      IF(IPR.EQ.0) THEN
         DO 151 IB=1,IELEM
         CALL TRICO (IB,IR,K,VOL0,MAT,XSGD(1,1),XX,YY,ZZ,DD,
     1   KN(NUM1+1),QFR(NUM1+1),CYLIND,A1(1,IB))
  151    CONTINUE
      ELSE IF(IPR.GE.1) THEN
         DO 152 IB=1,IELEM
         CALL TRIDCO (IB,IR,K,VOL0,MAT,SGD(1,1),XSGD(1,1),XX,YY,ZZ,
     1   DD,KN(NUM1+1),QFR(NUM1+1),CYLIND,IPR,A1(1,IB))
  152    CONTINUE
      ENDIF
      KK5=KN(NUM1+5)
      KK6=KN(NUM1+6)
      IF(KK5.EQ.-4) KK5=KK6
      IF(KK6.EQ.-4) KK6=KK5
*
      IF(IELEM.EQ.1) THEN
         INZ1=IPZ(IGAR(K))
         KEY0=MUZ(INZ1)-INZ1
         IF(KK5.GT.0) THEN
            INZ2=IPZ(IGAR(KK5))
            IF(INZ2.LT.INZ1) THEN
               KEY=KEY0+INZ2
               A11Z(KEY)=A11Z(KEY)-A1(5,1)
            ENDIF
         ENDIF
         IF(KK6.GT.0) THEN
            INZ2=IPZ(IGAR(KK6))
            IF(INZ2.LT.INZ1) THEN
               KEY=KEY0+INZ2
               A11Z(KEY)=A11Z(KEY)-A1(6,1)
            ENDIF
         ENDIF
         KEY=KEY0+INZ1
         A11Z(KEY)=A11Z(KEY)+A1(1,1)+A1(2,1)+A1(3,1)+A1(4,1)+A1(5,1)
     1   +A1(6,1)+XSGD(L,4)*VOL0
      ELSE
         DO 190 I3=0,IELEM-1
         DO 190 I2=0,IELEM-1-I3
         DO 190 I1=0,IELEM-1-I2-I3
         INZ1=IPZ(IORD(I2,I3,I1,LL,IELEM,IGAR(K)))
         KEY0=MUZ(INZ1)-INZ1
         IB=IELEM-I2-I3
         RLL=REAL(IB*(IB+1))
         QQ=SQRT(REAL(2*I1+1))*(RLL-REAL(I1*(I1+1)))/RLL
         IF(KK5.GT.0) THEN
            PAIR=(-1.0D0)**I1
            DO 160 I0=0,IELEM-1-I2-I3
            LOGIC=(KN((IGAR(KK5)-1)*6+5).NE.-4).OR.(MOD(I0+1,2).NE.0)
            INZ2=IPZ(IORD(I2,I3,I0,LL,IELEM,IGAR(KK5)))
            IF((INZ2.LT.INZ1).AND.LOGIC) THEN
               KEY=KEY0+INZ2
               R=REAL(I0*(I0+1))
               S=SQRT(REAL(2*I0+1))
               A11Z(KEY)=A11Z(KEY)-0.5D0*QQ*PAIR*S*(RLL-R)*A1(5,IB)
            ENDIF
  160       CONTINUE
         ENDIF
         IF(KK6.GT.0) THEN
            DO 165 I0=0,IELEM-1-I2-I3
            INZ2=IPZ(IORD(I2,I3,I0,LL,IELEM,IGAR(KK6)))
            IF(INZ2.LT.INZ1) THEN
               PAIR=(-1.0D0)**I0
               IF(KN(NUM1+6).EQ.-4) PAIR=1.0D0
               KEY=KEY0+INZ2
               R=REAL(I0*(I0+1))
               S=SQRT(REAL(2*I0+1))
               A11Z(KEY)=A11Z(KEY)-0.5D0*QQ*PAIR*S*(RLL-R)*A1(6,IB)
            ENDIF
  165       CONTINUE
         ENDIF
         KEY=KEY0+INZ1-I1
         DO 170 I0=0,I1
         R=REAL(I0*(I0+1))
         S=SQRT(REAL(2*I0+1))
         PAIR=1.0D0+(-1.0D0)**(I0+I1)
  170    A11Z(KEY+I0)=A11Z(KEY+I0)+QQ*(PAIR*S*R*XSGD(L,3)*VOL0/(DZ*DZ)
     1   +0.5D0*S*(RLL-R)*((-1.0D0)**(I0+I1)*A1(5,IB)+A1(6,IB)))
*
         KEY=KEY0+INZ1
         IB=IELEM-I1-I3
         RLL=REAL(IB*(IB+1))
         R=REAL(I2*(I2+1))
         QQ=REAL(2*I2+1)*(RLL-R)/RLL
         A11Z(KEY)=A11Z(KEY)+QQ*(2.0D0*R*XSGD(L,1)*VOL0/(DX*DX)
     1   +0.5D0*(RLL-R)*(A1(1,IB)+A1(2,IB)))
*
         IB=IELEM-I1-I2
         RLL=REAL(IB*(IB+1))
         R=REAL(I3*(I3+1))
         QQ=REAL(2*I3+1)*(RLL-R)/RLL
         A11Z(KEY)=A11Z(KEY)+QQ*(2.0D0*R*XSGD(L,2)*VOL0/(DY*DY)
     1   +0.5D0*(RLL-R)*(A1(3,IB)+A1(4,IB)))+XSGD(L,4)*VOL0
*
  190    CONTINUE
      ENDIF
  200 NUM1=NUM1+6
  210 CONTINUE
      RETURN
      END
