*DECK TRIMTW
      SUBROUTINE TRIMTW(ISPLH,IR,IDIM,NEL,LL4,VOL,MAT,MATN,SGD,XSGD,
     1 SIDE,ZZ,KN,QFR,MUW,IPW,IPR,A11W)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF SYSTEM MATRICES FOR A MESH CENTERED FINITE DIFFERENCE
* DISCRETIZATION IN HEXAGONAL GEOMETRY (TRIANGULAR SUB MESHS).
*
* INPUT PARAMETERS:
*  ISPLH  : THE NUMBER OF TRIANGLES IS 6*(ISPLH-1)**2.
*  IR     : FIRST DIMENSION OF MATRIX SGD.
*  IDIM   : NUMBER OF DIMENSIONS (2 OR 3).
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  LL4      ORDER OF SYSTEM MATRICES.
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH HEXAGON. DIMENSION MAT(NEL)
*  MATN   : MIXTURE INDEX ASSIGNED TO EACH TRIANGLE.
*           DIMENSION MATN(LL4)
*  SGD    : NUCLEAR PROPERTIES PER MATERIAL MIXTURES.
*           DIMENSION SGD(IR,4)
*           SGD(L,1) : W-, X-, AND Y-ORIENTED DIFFUSION COEFFICIENTS
*           SGD(L,3) : Z-ORIENTED DIFFUSION COEFFICIENTS
*           SGD(L,4) : REMOVAL MACROSCOPIC CROSS SECTION.
*  XSGD   : NUCLEAR PROPERTIES (IPR=0), DERIVATIVES (IPR=1) OR FIRST
*           VARIATIONS (IPR=2 OR 3) OF NUCLEAR PROPERTIES PER MATERIAL
*           MIXTURE. DIMENSION XSGD(IR,4)
*  SIDE   : SIDE OF AN HEXAGON.
*  ZZ     : Z-DIRECTED MESH SPACINGS. DIMENSION ZZ(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(8*NEL)
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(8*NEL)
*  MUW,MUX,MUY,MUZ : COMPRESSED STORAGE MODE INDICES.
*                    DIMENSION MUW(LL4),MUX(LL4),MUY(LL4),MUZ(LL4)
*  IPW,IPX,IPY,IPZ : PERMUTATION MATRICES.
*                    DIMENSION IPW(LL4),IPX(LL4),IPY(LL4),IPZ(LL4)
*  IPR    : =0: COMPUTE THE SYSTEM MATRICES ;
*           =1: COMPUTE THE DERIVATIVE OF SYSTEM MATRICES ;
*           =2 OR =3: COMPUTE THE VARIATION OF SYSTEM MATRICES ;
*
* OUTPUT PARAMETERS:
*  A11W,A11X,A11Y,A11Z : MATRICES CORRESPONDING TO THE DIVERGENCE (I.E.
*                        LEAKAGE) AND REMOVAL TERMS.
*
* NOTE: SYSTEM MATRICES SHOULD BE INITIALIZED BY THE CALLING PROGRAM.
*
*----------------------------------- AUTHOR: A. BENABOUD ; 04/12/90 ----
*
      DOUBLE PRECISION A1(5),VAR1
      DIMENSION SGD(IR,*),XSGD(IR,*),VOL(*),MAT(*),ZZ(*),KN(*),QFR(*),
     1 MUW(*),MUX(*),MUY(*),MUZ(*),IPW(*),IPX(*),IPY(*),IPZ(*),A11W(*),
     2 A11X(*),A11Y(*),A11Z(*),MATN(*)
      COMMON IBASE(1)
*
* ASSEMBLY OF MATRIX A11W.
      NUM1 = 0
      NUM2 = 0
      NTPH = 6 * (ISPLH-1)**2
      NTPL = 1 + 2 * (ISPLH-1)
      NVT1 = NTPL + 2 * (ISPLH-2) + NTPH / 2
      NVT2 = NTPH - NTPL - (ISPLH-4) * (NTPL+2)
      NVT3 = NTPH - (ISPLH-4) * NTPL
      IVAL = 3*NTPH+8
      IF(ISPLH.EQ.3) NVT2 = NTPH
      IF(ISPLH.LE.3) ISAU =   2*(ISPLH-2)
      IF(ISPLH.GE.4) ISAU =   6*(ISPLH-3)
      ICR = ISAU*(1+2*(ISPLH-2))
      CALL SETARA(IBASE,NEL,IWRK)
      MEL = 0
      DO 10 M=1,NEL
         IF(MAT(M).LE.0) GO TO 10
         MEL = MEL + 1
         IBASE(IWRK+MEL-1) = M
 10   CONTINUE
      DO 40 K=1,NEL
         L = MAT(K)
         IF(L.EQ.0) GO TO 40
         VOL0 = VOL(K)/NTPH
         IF(VOL0.EQ.0.0) GO TO 30
         KK4=KN(NUM1+3*NTPH+7)
         KK5=KN(NUM1+3*NTPH+8)
         IF(KK4.GT.0) KK4 = IBASE(IWRK+KK4-1)
         IF(KK5.GT.0) KK5 = IBASE(IWRK+KK5-1)
         DO 20 I = 1,NTPH
*
            CALL TRINEI (3,1,1,ISPLH,ICR,I,KK1,KK2,KK3,KEL,IQF,NUM1,
     >                   NTPH,NTPL,NVT1,NVT2,NVT3,IVAL,KN)
*
            CALL TRITCO (ISPLH,IR,IQF,K,KK1,KK2,KK3,KK4,KK5,VOL0,MAT,
     >           MATN,SGD(1,1),XSGD(1,1),SIDE,ZZ,QFR(NUM2+1),IPR,A1)
*
            INW1=IPW(KEL)
            KEY0=MUW(INW1)-INW1
            IF(KK1.GT.0) THEN
               INW2=IPW(KK1)
               IF(INW2.LT.INW1) THEN
                  KEY=KEY0+INW2
                  A11W(KEY)=A11W(KEY)-A1(1)/2.
               ENDIF
            ENDIF
            IF(KK2.GT.0) THEN
               INW2=IPW(KK2)
               IF(INW2.LT.INW1) THEN
                  KEY=KEY0+INW2
                  A11W(KEY)=A11W(KEY)-A1(2)/2.
               ENDIF
            ENDIF
            KEY=KEY0+INW1
            VAR1 = A1(1)+A1(2)+A1(3)+A1(4)+A1(5)
            A11W(KEY)=A11W(KEY)+VAR1+XSGD(L,4)*VOL0
   20    CONTINUE
   30    NUM1=NUM1+IVAL
         NUM2=NUM2+8
   40 CONTINUE
      CALL RLSARA(IBASE(IWRK))
      RETURN
*
* ASSEMBLY OF MATRIX A11X.
      ENTRY TRIMTX (ISPLH,IR,IDIM,NEL,LL4,VOL,MAT,MATN,SGD,XSGD,SIDE,ZZ,
     1              KN,QFR,MUX,IPX,IPR,A11X)
      NUM1=0
      NUM2=0
      NTPH = 6*(ISPLH-1)**2
      NTPL = 1+2*(ISPLH-1)
      NVT1 = NTPL + 2 * (ISPLH-2) + NTPH / 2
      NVT2 = NTPH - NTPL - (ISPLH-4) * (NTPL+2)
      NVT3 = NTPH - (ISPLH-4) * NTPL
      IVAL = 3*NTPH+8
      IF(ISPLH.EQ.3) NVT2 = NTPH
      IF(ISPLH.LE.3) ISAU =   2*(ISPLH-2)
      IF(ISPLH.GE.4) ISAU =   6*(ISPLH-3)
      ICR = ISAU*(1+2*(ISPLH-2))
      CALL SETARA(IBASE,NEL,IWRK)
      MEL = 0
      DO 105 M=1,NEL
         IF(MAT(M).LE.0) GO TO 105
         MEL = MEL + 1
         IBASE(IWRK+MEL-1) = M
105   CONTINUE
      DO 130 K=1,NEL
         L = MAT(K)
         IF(L.EQ.0) GO TO 130
         VOL0 = VOL(K)/NTPH
         IF(VOL0.EQ.0.0) GO TO 120
         KK4=KN(NUM1+3*NTPH+7)
         KK5=KN(NUM1+3*NTPH+8)
         IF(KK4.GT.0) KK4 = IBASE(IWRK+KK4-1)
         IF(KK5.GT.0) KK5 = IBASE(IWRK+KK5-1)
         DO 110 I = 1,NTPH
*
            CALL TRINEI (3,2,1,ISPLH,ICR,I,KK1,KK2,KK3,KEL,IQF,NUM1,
     >                   NTPH,NTPL,NVT1,NVT2,NVT3,IVAL,KN)
*
            CALL TRITCO (ISPLH,IR,IQF,K,KK1,KK2,KK3,KK4,KK5,VOL0,MAT,
     >           MATN,SGD(1,1),XSGD(1,1),SIDE,ZZ,QFR(NUM2+1),IPR,A1)
*
            INX1=IPX(KEL)
            KEY0=MUX(INX1)-INX1
            IF(KK1.GT.0) THEN
               INX2=IPX(KK1)
               IF(INX2.LT.INX1) THEN
                  KEY=KEY0+INX2
                  A11X(KEY)=A11X(KEY)-A1(1)/2.
               ENDIF
            ENDIF
            IF(KK2.GT.0) THEN
               INX2=IPX(KK2)
               IF(INX2.LT.INX1) THEN
                  KEY=KEY0+INX2
                  A11X(KEY)=A11X(KEY)-A1(2)/2.
               ENDIF
            ENDIF
            KEY=KEY0+INX1
            VAR1 = A1(1)+A1(2)+A1(3)+A1(4)+A1(5)
            A11X(KEY)=A11X(KEY)+VAR1+XSGD(L,4)*VOL0
  110    CONTINUE
  120    NUM1=NUM1+IVAL
         NUM2=NUM2+8
  130 CONTINUE
      CALL RLSARA(IBASE(IWRK))
      RETURN
*
* ASSEMBLY OF MATRIX A11Y.
      ENTRY TRIMTY (ISPLH,IR,IDIM,NEL,LL4,VOL,MAT,MATN,SGD,XSGD,SIDE,ZZ,
     1              KN,QFR,MUY,IPY,IPR,A11Y)
      NUM1=0
      NUM2=0
      NTPH = 6*(ISPLH-1)**2
      NTPL = 1+2*(ISPLH-1)
      NVT1 = NTPL + 2 * (ISPLH-2) + NTPH / 2
      NVT2 = NTPH - NTPL - (ISPLH-4) * (NTPL+2)
      NVT3 = NTPH - (ISPLH-4) * NTPL
      IVAL = 3*NTPH+8
      IF(ISPLH.EQ.3) NVT2 = NTPH
      IF(ISPLH.LE.3) ISAU =   2*(ISPLH-2)
      IF(ISPLH.GE.4) ISAU =   6*(ISPLH-3)
      ICR = ISAU*(1+2*(ISPLH-2))
      CALL SETARA(IBASE,NEL,IWRK)
      MEL = 0
      DO 205 M=1,NEL
         IF(MAT(M).LE.0) GO TO 205
         MEL = MEL + 1
         IBASE(IWRK+MEL-1) = M
205   CONTINUE
      DO 230 K=1,NEL
         L = MAT(K)
         IF(L.EQ.0) GO TO 230
         VOL0 = VOL(K)/NTPH
         IF(VOL0.EQ.0.0) GO TO 220
         KK4=KN(NUM1+3*NTPH+7)
         KK5=KN(NUM1+3*NTPH+8)
         IF(KK4.GT.0) KK4 = IBASE(IWRK+KK4-1)
         IF(KK5.GT.0) KK5 = IBASE(IWRK+KK5-1)
         DO 210 I = 1,NTPH
*
            CALL TRINEI (3,3,1,ISPLH,ICR,I,KK1,KK2,KK3,KEL,IQF,NUM1,
     >                   NTPH,NTPL,NVT1,NVT2,NVT3,IVAL,KN)
*
            CALL TRITCO (ISPLH,IR,IQF,K,KK1,KK2,KK3,KK4,KK5,VOL0,MAT,
     >           MATN,SGD(1,1),XSGD(1,1),SIDE,ZZ,QFR(NUM2+1),IPR,A1)
*
            INY1=IPY(KEL)
            KEY0=MUY(INY1)-INY1
            IF(KK1.GT.0) THEN
               INY2=IPY(KK1)
               IF(INY2.LT.INY1) THEN
                  KEY=KEY0+INY2
                  A11Y(KEY)=A11Y(KEY)-A1(1)/2.
               ENDIF
            ENDIF
            IF(KK2.GT.0) THEN
               INY2=IPY(KK2)
               IF(INY2.LT.INY1) THEN
                  KEY=KEY0+INY2
                  A11Y(KEY)=A11Y(KEY)-A1(2)/2.
               ENDIF
            ENDIF
            KEY=KEY0+INY1
            VAR1 = A1(1)+A1(2)+A1(3)+A1(4)+A1(5)
            A11Y(KEY)=A11Y(KEY)+VAR1+XSGD(L,4)*VOL0
  210    CONTINUE
  220    NUM1=NUM1+IVAL
         NUM2=NUM2+8
  230 CONTINUE
      CALL RLSARA(IBASE(IWRK))
      RETURN
*
* ASSEMBLY OF MATRIX A11Z.
      ENTRY TRIMTZ (ISPLH,IR,IDIM,NEL,LL4,VOL,MAT,MATN,SGD,XSGD,SIDE,ZZ,
     1              KN,QFR,MUZ,IPZ,IPR,A11Z)
      NUM1=0
      NUM2=0
      NTPH = 6*(ISPLH-1)**2
      NTPL = 1+2*(ISPLH-1)
      NVT1 = NTPL + 2 * (ISPLH-2) + NTPH / 2
      NVT2 = NTPH - NTPL - (ISPLH-4) * (NTPL+2)
      NVT3 = NTPH - (ISPLH-4) * NTPL
      IVAL = 3*NTPH+8
      IF(ISPLH.EQ.3) NVT2 = NTPH
      IF(ISPLH.LE.3) ISAU =   2*(ISPLH-2)
      IF(ISPLH.GE.4) ISAU =   6*(ISPLH-3)
      ICR = ISAU*(1+2*(ISPLH-2))
      CALL SETARA(IBASE,NEL,IWRK)
      MEL = 0
      DO 305 M=1,NEL
         IF(MAT(M).LE.0) GO TO 305
         MEL = MEL + 1
         IBASE(IWRK+MEL-1) = M
305   CONTINUE
      DO 330 K=1,NEL
         L = MAT(K)
         IF(L.EQ.0) GO TO 330
         VOL0 = VOL(K)/NTPH
         IF(VOL0.EQ.0.0) GO TO 320
         DO 310 I = 1,NTPH
*
            CALL TRINEI (3,1,1,ISPLH,ICR,I,KK1,KK2,KK3,KEL,IQF,NUM1,
     >                   NTPH,NTPL,NVT1,NVT2,NVT3,IVAL,KN)
            KK4 = KN(NUM1+NTPH+I)
            KK5 = KN(NUM1+2*NTPH+I)
            LK4 = KK4
            LK5 = KK5
            IF(LK4.GT.0) LK4 = IBASE(IWRK+KN(NUM1+3*NTPH+7)-1)
            IF(LK5.GT.0) LK5 = IBASE(IWRK+KN(NUM1+3*NTPH+8)-1)
*
            CALL TRITCO (ISPLH,IR,IQF,K,KK1,KK2,KK3,LK4,LK5,VOL0,MAT,
     >           MATN,SGD(1,1),XSGD(1,1),SIDE,ZZ,QFR(NUM2+1),IPR,A1)
*
            INZ1=IPZ(KEL)
            KEY0=MUZ(INZ1)-INZ1
            IF(KK4.GT.0) THEN
               INZ2=IPZ(KK4)
               IF(INZ2.LT.INZ1) THEN
                  KEY=KEY0+INZ2
                  A11Z(KEY)=A11Z(KEY)-A1(4)
               ENDIF
            ENDIF
            IF(KK5.GT.0) THEN
               INZ2=IPZ(KK5)
               IF(INZ2.LT.INZ1) THEN
                  KEY=KEY0+INZ2
                  A11Z(KEY)=A11Z(KEY)-A1(5)
               ENDIF
            ENDIF
            KEY=KEY0+INZ1
            VAR1 = A1(1)+A1(2)+A1(3)+A1(4)+A1(5)
            A11Z(KEY)=A11Z(KEY)+VAR1+XSGD(L,4)*VOL0
  310    CONTINUE
  320    NUM1=NUM1+IVAL
         NUM2=NUM2+8
  330 CONTINUE
      CALL RLSARA(IBASE(IWRK))
      RETURN
      END
