*DECK TRIMWW
      SUBROUTINE TRIMWW(IR,IDIM,NEL,LL4,VOL,MAT,SGD,XSGD,SIDE,ZZ,KN,
     1 QFR,MUW,IPW,IPR,A11W,IGAR)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF SYSTEM MATRICES FOR A MESH CENTERED FINITE DIFFERENCE
* DISCRETIZATION IN HEXAGONAL GEOMETRY (COMPLETE HEXAGONS).
*
* INPUT PARAMETERS:
*  IR     : FIRST DIMENSION OF MATRIX SGD.
*  IDIM   : NUMBER OF DIMENSIONS (2 OR 3).
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  LL4      ORDER OF SYSTEM MATRICES.
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  SGD    : NUCLEAR PROPERTIES PER MATERIAL MIXTURES.
*           DIMENSION SGD(IR,4)
*           SGD(L,1) : W-, X-, AND Y-ORIENTED DIFFUSION COEFFICIENTS
*           SGD(L,3) : Z-ORIENTED DIFFUSION COEFFICIENTS
*           SGD(L,4) : REMOVAL MACROSCOPIC CROSS SECTION.
*  XSGD   : NUCLEAR PROPERTIES (IPR=0), DERIVATIVES (IPR=1) OR FIRST
*           VARIATIONS (IPR=2 OR 3) OF NUCLEAR PROPERTIES PER MATERIAL
*           MIXTURE. DIMENSION XSGD(IR,4)
*  SIDE   : SIDE OF AN HEXAGON.
*  ZZ     : Z-DIRECTED MESH SPACINGS. DIMENSION ZZ(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(8*NEL)
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(8*NEL)
*  MUW,MUX,MUY,MUZ : COMPRESSED STORAGE MODE INDICES.
*                    DIMENSION MUW(LL4),MUX(LL4),MUY(LL4),MUZ(LL4)
*  IPW,IPX,IPY,IPZ : PERMUTATION MATRICES.
*                    DIMENSION IPW(LL4),IPX(LL4),IPY(LL4),IPZ(LL4)
*  IPR    : =0: COMPUTE THE SYSTEM MATRICES ;
*           =1: COMPUTE THE DERIVATIVE OF SYSTEM MATRICES ;
*           =2 OR =3: COMPUTE THE VARIATION OF SYSTEM MATRICES ;
*
* OUTPUT PARAMETERS:
*  A11W,A11X,A11Y,A11Z : MATRICES CORRESPONDING TO THE DIVERGENCE (I.E.
*                        LEAKAGE) AND REMOVAL TERMS.
*
* SCRATCH STORAGE:
*
* IGAR    : DIMENSION IGAR(LL4)
*
* NOTE: SYSTEM MATRICES SHOULD BE INITIALIZED BY THE CALLING PROGRAM.
*
*----------------------------------- AUTHOR: A. BENABOUD ; 04/12/90 ----
*
      DOUBLE PRECISION A1(8),VAR1
      DIMENSION SGD(IR,*),XSGD(IR,*),VOL(*),MAT(*),ZZ(*),KN(*),QFR(*),
     1 MUW(*),MUX(*),MUY(*),MUZ(*),IPW(*),IPX(*),IPY(*),IPZ(*),A11W(*),
     2 A11X(*),A11Y(*),A11Z(*),IGAR(*)
*
* ASSEMBLY OF MATRIX A11W.
      LL=0
      DO 10 K=1,NEL
      IF(MAT(K).LE.0) GO TO 10
      LL=LL+1
      IGAR(LL)=K
   10 CONTINUE
      NUM1=0
      KEL=0
      DO 70 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 70
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 60
      KEL=KEL+1
*
      CALL TRIHCO (IR,K,VOL0,MAT,SGD(1,1),XSGD(1,1),SIDE,ZZ,KN(NUM1+1),
     1 QFR(NUM1+1),IGAR,IPR,A1)
      KK1=KN(NUM1+6)
      KK2=KN(NUM1+3)
*
      INW1=IPW(KEL)
      KEY0=MUW(INW1)-INW1
      IF(KK1.GT.0) THEN
         INW2=IPW(KK1)
         IF(INW2.LT.INW1) THEN
            KEY=KEY0+INW2
            A11W(KEY)=A11W(KEY)-A1(6)
         ENDIF
      ENDIF
      IF(KK2.GT.0) THEN
         INW2=IPW(KK2)
         IF(INW2.LT.INW1) THEN
            KEY=KEY0+INW2
            A11W(KEY)=A11W(KEY)-A1(3)
         ENDIF
      ENDIF
      KEY=KEY0+INW1
      VAR1=A1(1)+A1(2)+A1(3)+A1(4)+A1(5)+A1(6)+A1(7)+A1(8)
      A11W(KEY)=A11W(KEY)+VAR1+XSGD(L,4)*VOL0
   60 NUM1=NUM1+8
   70 CONTINUE
      RETURN
*
* ASSEMBLY OF MATRIX A11X.
      ENTRY TRIMWX (IR,IDIM,NEL,LL4,VOL,MAT,SGD,XSGD,SIDE,ZZ,KN,QFR,
     1              MUX,IPX,IPR,A11X,IGAR)
      LL=0
      DO 80 K=1,NEL
      IF(MAT(K).LE.0) GO TO 80
      LL=LL+1
      IGAR(LL)=K
   80 CONTINUE
      NUM1=0
      KEL=0
      DO 140 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 140
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 130
      KEL=KEL+1
*
      CALL TRIHCO (IR,K,VOL0,MAT,SGD(1,1),XSGD(1,1),SIDE,ZZ,KN(NUM1+1),
     1 QFR(NUM1+1),IGAR,IPR,A1)
      KK3=KN(NUM1+1)
      KK4=KN(NUM1+4)
*
      INX1=IPX(KEL)
      KEY0=MUX(INX1)-INX1
      IF(KK3.GT.0) THEN
         INX2=IPX(KK3)
         IF(INX2.LT.INX1) THEN
            KEY=KEY0+INX2
            A11X(KEY)=A11X(KEY)-A1(1)
         ENDIF
      ENDIF
      IF(KK4.GT.0) THEN
         INX2=IPX(KK4)
         IF(INX2.LT.INX1) THEN
            KEY=KEY0+INX2
            A11X(KEY)=A11X(KEY)-A1(4)
         ENDIF
      ENDIF
      KEY=KEY0+INX1
      VAR1=A1(1)+A1(2)+A1(3)+A1(4)+A1(5)+A1(6)+A1(7)+A1(8)
      A11X(KEY)=A11X(KEY)+VAR1+XSGD(L,4)*VOL0
  130 NUM1=NUM1+8
  140 CONTINUE
      RETURN
*
* ASSEMBLY OF MATRIX A11Y.
      ENTRY TRIMWY (IR,IDIM,NEL,LL4,VOL,MAT,SGD,XSGD,SIDE,ZZ,KN,QFR,
     1              MUY,IPY,IPR,A11Y,IGAR)
      LL=0
      DO 85 K=1,NEL
      IF(MAT(K).LE.0) GO TO 85
      LL=LL+1
      IGAR(LL)=K
   85 CONTINUE
      NUM1=0
      KEL=0
      DO 145 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 145
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 135
      KEL=KEL+1
*
      CALL TRIHCO (IR,K,VOL0,MAT,SGD(1,1),XSGD(1,1),SIDE,ZZ,KN(NUM1+1),
     1 QFR(NUM1+1),IGAR,IPR,A1)
      KK5=KN(NUM1+2)
      KK6=KN(NUM1+5)
*
      INY1=IPY(KEL)
      KEY0=MUY(INY1)-INY1
      IF(KK5.GT.0) THEN
         INY2=IPY(KK5)
         IF(INY2.LT.INY1) THEN
            KEY=KEY0+INY2
            A11Y(KEY)=A11Y(KEY)-A1(2)
         ENDIF
      ENDIF
      IF(KK6.GT.0) THEN
         INY2=IPY(KK6)
         IF(INY2.LT.INY1) THEN
            KEY=KEY0+INY2
            A11Y(KEY)=A11Y(KEY)-A1(5)
         ENDIF
      ENDIF
      KEY=KEY0+INY1
      VAR1=A1(1)+A1(2)+A1(3)+A1(4)+A1(5)+A1(6)+A1(7)+A1(8)
      A11Y(KEY)=A11Y(KEY)+VAR1+XSGD(L,4)*VOL0
  135 NUM1=NUM1+8
  145 CONTINUE
      RETURN
*
* ASSEMBLY OF MATRIX A11Z.
      ENTRY TRIMWZ (IR,IDIM,NEL,LL4,VOL,MAT,SGD,XSGD,SIDE,ZZ,KN,QFR,
     1              MUZ,IPZ,IPR,A11Z,IGAR)
      LL=0
      DO 150 K=1,NEL
      IF(MAT(K).LE.0) GO TO 150
      LL=LL+1
      IGAR(LL)=K
  150 CONTINUE
      NUM1=0
      KEL=0
      DO 210 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 210
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 200
      KEL=KEL+1
*
      CALL TRIHCO (IR,K,VOL0,MAT,SGD(1,1),XSGD(1,1),SIDE,ZZ,KN(NUM1+1),
     1 QFR(NUM1+1),IGAR,IPR,A1)
      KK7=KN(NUM1+7)
      KK8=KN(NUM1+8)
*
      INZ1=IPZ(KEL)
      KEY0=MUZ(INZ1)-INZ1
      IF(KK7.GT.0) THEN
         INZ2=IPZ(KK7)
         IF(INZ2.LT.INZ1) THEN
            KEY=KEY0+INZ2
            A11Z(KEY)=A11Z(KEY)-A1(7)
         ENDIF
      ENDIF
      IF(KK8.GT.0) THEN
        INZ2=IPZ(KK8)
        IF(INZ2.LT.INZ1) THEN
            KEY=KEY0+INZ2
            A11Z(KEY)=A11Z(KEY)-A1(8)
         ENDIF
      ENDIF
      KEY=KEY0+INZ1
      VAR1=A1(1)+A1(2)+A1(3)+A1(4)+A1(5)+A1(6)+A1(7)+A1(8)
      A11Z(KEY)=A11Z(KEY)+VAR1+XSGD(L,4)*VOL0
  200 NUM1=NUM1+8
  210 CONTINUE
      RETURN
      END
