*DECK TRIPRH
      SUBROUTINE TRIPRH(ISPLH,IPTRK,LX,LZ,LL4,SIDE,ZZZ,ZZ,KN,QFR,VOL,
     1 MAT,NCODE,ZCODE,IMPX)
*
*-----------------------------------------------------------------------
*
* NUMBERING CORRESPONDING TO A MESH CORNER FINITE DIFFERENCE OR
* LAGRANGIAN FINITE ELEMENT DISCRETIZATION OF A 3-D HEXAGONAL GEOMETRY.
*
* INPUT PARAMETERS:
*  ISPLH  : =1 FOR HEXAGONAL ELEMENT WITH 6 POINTS,
*           =2 FOR HEXAGONAL ELEMENT WITH 7 POINTS,
*           =3 FOR TRIANGULAR ELEMENT.
*  IPTRK  : L_TRACK POINTER TO THE TRACKING INFORMATION.
*  IMPX   : PRINT PARAMETER.
*  LX     : NUMBER OF ELEMENTS .
*  LZ     : NUMBER OF AXIAL PLANES.
*  NCODE  : NCODE(I) IS THE TYPE OF BOUNDARY CONDITION APPLIED ON SIDE I
*           I=1: HBC
*           NCODE(I)=1: VOID   NCODE(I)=2: REFL   NCODE(I)=5: SYME
*           NCODE(I)=7: ZERO
*  ZCODE  : ZCODE(I) IS THE ALBEDO CORRESPONDING TO BOUNDARY CONDITION
*           'VOID' ON SIDE I. ZCODE(I)=0.0 BY DEFAULT.
*           DIMENSION NCODE(4),ZCODE(4)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(LX)
*  SIDE   : SIDE OF THE HEXAGON.
*  ZZZ    : Z-COORDINATES OF THE AXIAL PLANES. DIMENSION ZZZ(LZ+1)
*
* OUTPUT PARAMETERS:
*  LL4    : ORDER OF SYSTEM MATRICES.
*  ZZ     : AXIAL WIDTH OF EACH ELEMENT. DIMENSION ZZ(LX*LZ)
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(LX*LZ)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(ICOEF*LX*LZ)
*           WHERE ICOEF=12 OR 14
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS.
*           DIMENSION QFR(8*LX*LZ)
*
*-------------------------------------- AUTHOR: A. BENABOUD 15/01/92 ---
*
      DIMENSION NCODE(*),ZCODE(*),MAT(*),VOL(*),KN(*),QFR(*),ZZZ(*),
     1          ZZ(*)
      COMMON IBASE(1)
*
      ALB(X)=0.5*(1.0-X)/(1.0+X)
*
      IK=12
      IPAR=ISPLH
      IF(IPAR.GE.2) THEN
         IPAR=2
         IK=14
      ENDIF
      CALL SETARA(IBASE,14*LX,IICG)
      CALL SETARA(IBASE,21*LX,INIK)
      CALL TRIHEX(IPAR+2,LX,LZ,LL4,MAT,KN,NCODE,IPTRK,IBASE(IICG),
     1 IBASE(INIK))
      CALL RLSARA(IBASE(INIK))
      CALL RLSARA(IBASE(IICG))
*
* COMPUTE BOUNDARY CONDITIONS.
      FRZ=1.
      KEL=0
      NUM1=0
      DO 10 KZ=1,LZ
      DO 10 KX=1,LX
         KEL=KEL + 1
         ZZ(KEL)=0.0
         VOL(KEL)=0.0
         IF(MAT(KEL).LE.0) GO TO 10
         ZZ(KEL)=ZZZ(KZ+1) - ZZZ(KZ)
         DO 20 IC=1,6
            QFR(NUM1+IC)=0.0
            NV=NEIGHB (KX,IC,9,LX,POIDS)
            IF(NV.GT.LX) THEN
               IF(NCODE(1).EQ.1) QFR(NUM1+IC)=ALB(ZCODE(1))
            ELSE IF(MAT(NV).LE.0) THEN
               IF(NCODE(1).EQ.1) QFR(NUM1+IC)=ALB(ZCODE(1))
            ENDIF
 20      CONTINUE
         QFR(NUM1+7)=0.0
         QFR(NUM1+8)=0.0
         IF((NCODE(5).EQ.1).AND.(KZ.EQ.1))  QFR(NUM1+7)=ALB(ZCODE(5))
         IF((NCODE(6).EQ.1).AND.(KZ.EQ.LZ)) QFR(NUM1+8)=ALB(ZCODE(6))
         IF((NCODE(5).EQ.5).AND.(KZ.EQ.1)) THEN
            QFR(NUM1+7)=QFR(NUM1+8)
            FRZ=0.5
         ELSE IF((NCODE(6).EQ.5).AND.(KZ.EQ.LZ)) THEN
            QFR(NUM1+7)=QFR(NUM1+8)
            FRZ=0.5
         ENDIF
         ZZ(KEL)=ZZ(KEL)*FRZ
*
*        COMPUTE VOLUMES.
         VOL(KEL)=2.59807587*SIDE*SIDE*ZZ(KEL)
*
         DO 30 IC=1,6
 30      QFR(NUM1+IC)=QFR(NUM1+IC)*SIDE*ZZ(KEL)
         QFR(NUM1+7)=QFR(NUM1+7)*SIDE*SIDE
         QFR(NUM1+8)=QFR(NUM1+8)*SIDE*SIDE
         NUM1=NUM1+8
 10   CONTINUE
      IF(IMPX.GT.2) WRITE(6,720) (VOL(I),I=1,LX*LZ)
*
      IF(IMPX.GT.2) THEN
         NUM1=0
         NUM2=0
         WRITE(6,730)
         DO 510 KZ=1,LZ
         WRITE(6,'(/13H PLANE NUMBER,I6)') KZ
         IF(IK.EQ.12) WRITE(6,740)
         IF(IK.EQ.14) WRITE(6,745)
         DO 500 KX=1,LX
         IF(MAT(KX+(KZ-1)*LX).LE.0) GO TO 500
         K=KX+(KZ-1)*LX
         IF(IK.EQ.12)
     >      WRITE(6,750) K,(KN(NUM1+I),I=1,12),(QFR(NUM2+I),I=1,8)
         IF(IK.EQ.14)
     >      WRITE(6,760) K,(KN(NUM1+I),I=1,14),(QFR(NUM2+I),I=1,8)
         NUM1=NUM1+IK
         NUM2=NUM2+8
 500     CONTINUE
 510     CONTINUE
      ENDIF
      RETURN
*
720   FORMAT(/20H VOLUMES PER ELEMENT/(1X,10(1X,E12.5)))
730   FORMAT(/22H NUMBERING OF UNKNOWNS/1X,21(1H-))
740   FORMAT(/8H ELEMENT,3X,7HNUMBERS,58X,23HVOID BOUNDARY CONDITION)
745   FORMAT(/8H ELEMENT,3X,7HNUMBERS,68X,23HVOID BOUNDARY CONDITION)
750   FORMAT (3X,I4,4X,12I5,3X,8F6.2)
760   FORMAT (3X,I4,4X,14I5,3X,8F6.2)
      END
