*DECK TRIPXX
      SUBROUTINE TRIPXX (IR,NEL,LL4,VOL,MAT,XSGD,XX,YY,ZZ,DD,KN,QFR,MUX,
     1 IPX,CYLIND,LC,T,TS,Q,QS,A11X)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF SYSTEM MATRICES FOR A PRIMAL FINITE ELEMENT METHOD IN 3-D.
*
* INPUT PARAMETERS:
*  IR     : FIRST DIMENSION OF MATRIX SGD.
*  CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  LL4      ORDER OF SYSTEM MATRICES.
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  XX,YY,ZZ : X-, Y- AND Z-DIRECTED MESH SPACINGS.
*           DIMENSION XX(NEL),YY(NEL),ZZ(NEL)
*  DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST.
*           DIMENSION KN(NEL*(IELEM+1)**3) WHERE IELEM IS THE ORDER OF
*           THE FINITE ELEMENTS.
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(6*NEL)
*  XSGD   : NUCLEAR PROPERTIES, DERIVATIVES OR FIRST VARIATIONS OF NU-
*           CLEAR PROPERTIES PER MATERIAL MIXTURE. DIMENSION XSGD(IR,4)
*           XSGD(L,1) : X-ORIENTED DIFFUSION COEFFICIENTS
*           XSGD(L,2) : Y-ORIENTED DIFFUSION COEFFICIENTS
*           XSGD(L,3) : Z-ORIENTED DIFFUSION COEFFICIENTS
*           XSGD(L,4) : REMOVAL MACROSCOPIC CROSS SECTION.
*  MUX,MUY,MUZ : COMPRESSED STORAGE MODE INDICES.
*                DIMENSION MUX(LL4),MUY(LL4),MUZ(LL4)
*  IPX,IPY,IPZ : PERMUTATION MATRICES.
*                DIMENSION IPX(LL4),IPY(LL4),IPZ(LL4)
*  LC     : ORDER OF THE UNIT MATRICES.
*  T      : CARTESIAN LINEAR PRODUCT VECTOR. DIMENSION T(LC)
*  TS     : CYLINDRICAL LINEAR PRODUCT VECTOR. DIMENSION TS(LC)
*  Q      : CARTESIAN STIFFNESS MATRIX. DIMENSION Q(LC,LC)
*  QS     : CYLINDRICAL STIFFNESS MATRIX. DIMENSION QS(LC,LC)
*
* OUTPUT PARAMETERS:
*  A11X,A11Y,A11Z : MATRICES CORRESPONDING TO THE DIVERGENCE (I.E.
*                   LEAKAGE) AND REMOVAL TERMS.
*           DIMENSION A11X(MUX(LL4)),A11Y(MUY(LL4)),A11Z(MUZ(LL4))
*
* NOTE: SYSTEM MATRICES SHOULD BE INITIALIZED BY THE CALLING PROGRAM.
*
*------------------------------------- AUTHOR: A. HEBERT ; 15/01/86 ----
*
      LOGICAL CYLIND
      DOUBLE PRECISION RR,QQX,QQY,QQZ,VOL0,VOL1,VOL2,VOL3,DX,DY,DZ,CRZ
      DIMENSION XSGD(IR,*),XX(*),YY(*),ZZ(*),DD(*),VOL(*),MAT(*),
     1 KN(*),QFR(*),MUX(*),MUY(*),MUZ(*),IPX(*),IPY(*),IPZ(*),A11X(*),
     2 A11Y(*),A11Z(*),T(*),TS(*),Q(LC,*),QS(LC,*)
      COMMON /ELEM2/LL,LCC,IJ1(125),IJ2(125),IJ3(125),ISR(6,25),
     1 Q3DP1(125,125),Q3DP2(125,125),Q3DP3(125,125),R3DP(125),
     2 Q3DC1(125,125),Q3DC2(125,125),Q3DC3(125,125),R3DC(125),
     3 R2DP(25),R2DC(25)
      SAVE /ELEM2/
*
* CALCULATION OF COMMON /ELEM2/
      LCC=LC*LC
      LL=LC*LC*LC
      DO 5 L=1,LL
      L1=1+MOD(L-1,LC)
      L2=1+(L-L1)/LC
      L3=1+MOD(L2-1,LC)
      IJ1(L)=L1
      IJ2(L)=L3
    5 IJ3(L)=1+(L2-L3)/LC
*
* CALCULATION OF MATRIX ISR.
      K1=0
      K2=0
      J1=0
      J2=0
      I1=0
      I2=0
      L=0
      DO 6 I=1,LC
      DO 6 J=1,LC
      DO 6 K=1,LC
      L=L+1
      IF(K.EQ.1) THEN
         K1=K1+1
         ISR(1,K1)=L
      ELSE IF(K.EQ.LC) THEN
         K2=K2+1
         ISR(2,K2)=L
      ENDIF
      IF(J.EQ.1) THEN
         J1=J1+1
         ISR(3,J1)=L
      ELSE IF(J.EQ.LC) THEN
         J2=J2+1
         ISR(4,J2)=L
      ENDIF
      IF(I.EQ.1) THEN
         I1=I1+1
         ISR(5,I1)=L
      ELSE IF(I.EQ.LC) THEN
         I2=I2+1
         ISR(6,I2)=L
      ENDIF
    6 CONTINUE
*
* CALCULATION OF 3-D MASS AND STIFFNESS MATRICES FROM TENSORIAL PRODUCT
* OF 1-D MATRICES.
      DO 20 I=1,LL
      I1=IJ1(I)
      I2=IJ2(I)
      I3=IJ3(I)
      DO 10 J=1,LL
      J1=IJ1(J)
      J2=IJ2(J)
      J3=IJ3(J)
      IF((I2.EQ.J2).AND.(I3.EQ.J3)) THEN
         Q3DP1(I,J)=Q(I1,J1)*T(I2)*T(I3)
         Q3DC1(I,J)=QS(I1,J1)*T(I2)*T(I3)
      ELSE
         Q3DP1(I,J)=0.0
         Q3DC1(I,J)=0.0
      ENDIF
      IF((I1.EQ.J1).AND.(I3.EQ.J3)) THEN
         Q3DP2(I,J)=T(I1)*Q(I2,J2)*T(I3)
         Q3DC2(I,J)=TS(I1)*Q(I2,J2)*T(I3)
      ELSE
         Q3DP2(I,J)=0.0
         Q3DC2(I,J)=0.0
      ENDIF
      IF((I1.EQ.J1).AND.(I2.EQ.J2)) THEN
         Q3DP3(I,J)=T(I1)*T(I2)*Q(I3,J3)
         Q3DC3(I,J)=TS(I1)*T(I2)*Q(I3,J3)
      ELSE
         Q3DP3(I,J)=0.0
         Q3DC3(I,J)=0.0
      ENDIF
   10 CONTINUE
      R3DP(I)=T(I1)*T(I2)*T(I3)
   20 R3DC(I)=TS(I1)*T(I2)*T(I3)
*
* CALCULATION OF 2-D MASS MATRICES FROM TENSORIAL PRODUCT OF 1-D
* MATRICES.
      DO 30 I=1,LC*LC
      I1=IJ1(I)
      I2=IJ2(I)
      R2DP(I)=T(I1)*T(I2)
   30 R2DC(I)=TS(I1)*T(I2)
*
* X-DIRECTED COUPLINGS.
*
* ASSEMBLY OF MATRIX A11X.
      NUM1=0
      NUM2=0
      DO 90 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 90
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 80
      DX=XX(K)
      DY=YY(K)
      DZ=ZZ(K)
      VOL1=VOL0/(DX*DX)
      VOL2=VOL0/(DY*DY)
      VOL3=VOL0/(DZ*DZ)
      DO 50 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 50
      INX1=IPX(IND1)
      KEY0=MUX(INX1)-INX1
      DO 40 J=1,LL
      IND2=KN(NUM1+J)
      IF(IND2.EQ.0) GO TO 40
      INX2=IPX(IND2)
      IF(INX2.EQ.INX1) THEN
         IF(CYLIND) THEN
            QQX=(Q3DP1(I,J)+Q3DC1(I,J)*DX/DD(K))*VOL1
            QQY=(Q3DP2(I,J)+Q3DC2(I,J)*DX/DD(K))*VOL2
            QQZ=(Q3DP3(I,J)+Q3DC3(I,J)*DX/DD(K))*VOL3
         ELSE
            QQX=Q3DP1(I,J)*VOL1
            QQY=Q3DP2(I,J)*VOL2
            QQZ=Q3DP3(I,J)*VOL3
         ENDIF
         KEY=KEY0+INX2
         A11X(KEY)=A11X(KEY)+QQX*XSGD(L,1)+QQY*XSGD(L,2)+QQZ*XSGD(L,3)
      ELSE IF((INX2.LT.INX1).AND.(IJ2(I).EQ.IJ2(J)).AND.
     1 (IJ3(I).EQ.IJ3(J))) THEN
         IF(CYLIND) THEN
            QQX=(Q3DP1(I,J)+Q3DC1(I,J)*DX/DD(K))*VOL1
         ELSE
            QQX=Q3DP1(I,J)*VOL1
         ENDIF
         KEY=KEY0+INX2
         A11X(KEY)=A11X(KEY)+QQX*XSGD(L,1)
      ENDIF
   40 CONTINUE
      IF(CYLIND) THEN
         RR=(R3DP(I)+R3DC(I)*DX/DD(K))*VOL0
      ELSE
         RR=R3DP(I)*VOL0
      ENDIF
      KEY=KEY0+INX1
      A11X(KEY)=A11X(KEY)+RR*XSGD(L,4)
   50 CONTINUE
      DO 70 IC=1,6
      QFR1=QFR(NUM2+IC)
      IF(QFR1.EQ.0.0) GO TO 70
      DO 60 I1=1,LCC
      I=ISR(IC,I1)
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 60
      INX1=IPX(IND1)
      KEY=MUX(INX1)
      IF(CYLIND) THEN
         IF(IC.EQ.1) THEN
            CRZ=-0.5*R2DP(I1)
         ELSE IF(IC.EQ.2) THEN
            CRZ=0.5*R2DP(I1)
         ELSE
            CRZ=R2DC(I1)
         ENDIF
         RR=(R2DP(I1)+CRZ*DX/DD(K))
      ELSE
         RR=R2DP(I1)
      ENDIF
      A11X(KEY)=A11X(KEY)+RR*QFR1
   60 CONTINUE
   70 CONTINUE
   80 NUM1=NUM1+LL
      NUM2=NUM2+6
   90 CONTINUE
      RETURN
*
* Y-DIRECTED COUPLINGS.
      ENTRY TRIPXY (IR,NEL,LL4,VOL,MAT,XSGD,XX,YY,ZZ,DD,KN,QFR,MUY,IPY,
     1 CYLIND,A11Y)
*
* ASSEMBLY OF MATRIX A11Y.
      NUM1=0
      NUM2=0
      DO 180 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 180
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 170
      DX=XX(K)
      DY=YY(K)
      DZ=ZZ(K)
      VOL1=VOL0/(DX*DX)
      VOL2=VOL0/(DY*DY)
      VOL3=VOL0/(DZ*DZ)
      DO 140 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 140
      INY1=IPY(IND1)
      KEY0=MUY(INY1)-INY1
      DO 130 J=1,LL
      IND2=KN(NUM1+J)
      IF(IND2.EQ.0) GO TO 130
      INY2=IPY(IND2)
      IF(INY2.EQ.INY1) THEN
         IF(CYLIND) THEN
            QQX=(Q3DP1(I,J)+Q3DC1(I,J)*DX/DD(K))*VOL1
            QQY=(Q3DP2(I,J)+Q3DC2(I,J)*DX/DD(K))*VOL2
            QQZ=(Q3DP3(I,J)+Q3DC3(I,J)*DX/DD(K))*VOL3
         ELSE
            QQX=Q3DP1(I,J)*VOL1
            QQY=Q3DP2(I,J)*VOL2
            QQZ=Q3DP3(I,J)*VOL3
         ENDIF
         KEY=KEY0+INY2
         A11Y(KEY)=A11Y(KEY)+QQX*XSGD(L,1)+QQY*XSGD(L,2)+QQZ*XSGD(L,3)
      ELSE IF((INY2.LT.INY1).AND.(IJ1(I).EQ.IJ1(J)).AND.
     1 (IJ3(I).EQ.IJ3(J))) THEN
         IF(CYLIND) THEN
            QQY=(Q3DP2(I,J)+Q3DC2(I,J)*DX/DD(K))*VOL2
         ELSE
            QQY=Q3DP2(I,J)*VOL2
         ENDIF
         KEY=KEY0+INY2
         A11Y(KEY)=A11Y(KEY)+QQY*XSGD(L,2)
      ENDIF
  130 CONTINUE
      IF(CYLIND) THEN
         RR=(R3DP(I)+R3DC(I)*DX/DD(K))*VOL0
      ELSE
         RR=R3DP(I)*VOL0
      ENDIF
      KEY=KEY0+INY1
      A11Y(KEY)=A11Y(KEY)+RR*XSGD(L,4)
  140 CONTINUE
      DO 160 IC=1,6
      QFR1=QFR(NUM2+IC)
      IF(QFR1.EQ.0.0) GO TO 160
      DO 150 I1=1,LCC
      I=ISR(IC,I1)
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 150
      INY1=IPY(IND1)
      KEY=MUY(INY1)
      IF(CYLIND) THEN
         IF(IC.EQ.1) THEN
            CRZ=-0.5*R2DP(I1)
         ELSE IF(IC.EQ.2) THEN
            CRZ=0.5*R2DP(I1)
         ELSE
            CRZ=R2DC(I1)
         ENDIF
         RR=(R2DP(I1)+DX*CRZ/DD(K))
      ELSE
         RR=R2DP(I1)
      ENDIF
      A11Y(KEY)=A11Y(KEY)+RR*QFR1
  150 CONTINUE
  160 CONTINUE
  170 NUM1=NUM1+LL
      NUM2=NUM2+6
  180 CONTINUE
      RETURN
*
* Z-DIRECTED COUPLINGS.
      ENTRY TRIPXZ (IR,NEL,LL4,VOL,MAT,XSGD,XX,YY,ZZ,DD,KN,QFR,MUZ,IPZ,
     1 CYLIND,A11Z)
*
* ASSEMBLY OF MATRIX A11Z.
      NUM1=0
      NUM2=0
      DO 270 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 270
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 260
      DX=XX(K)
      DY=YY(K)
      DZ=ZZ(K)
      VOL1=VOL0/(DX*DX)
      VOL2=VOL0/(DY*DY)
      VOL3=VOL0/(DZ*DZ)
      DO 230 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 230
      INZ1=IPZ(IND1)
      KEY0=MUZ(INZ1)-INZ1
      DO 220 J=1,LL
      IND2=KN(NUM1+J)
      IF(IND2.EQ.0) GO TO 220
      INZ2=IPZ(IND2)
      IF(INZ2.EQ.INZ1) THEN
         IF(CYLIND) THEN
            QQX=(Q3DP1(I,J)+Q3DC1(I,J)*DX/DD(K))*VOL1
            QQY=(Q3DP2(I,J)+Q3DC2(I,J)*DX/DD(K))*VOL2
            QQZ=(Q3DP3(I,J)+Q3DC3(I,J)*DX/DD(K))*VOL3
         ELSE
            QQX=Q3DP1(I,J)*VOL1
            QQY=Q3DP2(I,J)*VOL2
            QQZ=Q3DP3(I,J)*VOL3
         ENDIF
         KEY=KEY0+INZ2
         A11Z(KEY)=A11Z(KEY)+QQX*XSGD(L,1)+QQY*XSGD(L,2)+QQZ*XSGD(L,3)
      ELSE IF((INZ2.LT.INZ1).AND.(IJ1(I).EQ.IJ1(J)).AND.
     1 (IJ2(I).EQ.IJ2(J))) THEN
         IF(CYLIND) THEN
            QQZ=(Q3DP3(I,J)+Q3DC3(I,J)*DX/DD(K))*VOL3
         ELSE
            QQZ=Q3DP3(I,J)*VOL3
         ENDIF
         KEY=KEY0+INZ2
         A11Z(KEY)=A11Z(KEY)+QQZ*XSGD(L,3)
      ENDIF
  220 CONTINUE
      IF(CYLIND) THEN
         RR=(R3DP(I)+R3DC(I)*DX/DD(K))*VOL0
      ELSE
         RR=R3DP(I)*VOL0
      ENDIF
      KEY=KEY0+INZ1
      A11Z(KEY)=A11Z(KEY)+RR*XSGD(L,4)
  230 CONTINUE
      DO 250 IC=1,6
      QFR1=QFR(NUM2+IC)
      IF(QFR1.EQ.0.0) GO TO 250
      DO 240 I1=1,LCC
      I=ISR(IC,I1)
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 240
      INZ1=IPZ(IND1)
      KEY=MUZ(INZ1)
      IF(CYLIND) THEN
         IF(IC.EQ.1) THEN
            CRZ=-0.5*R2DP(I1)
         ELSE IF(IC.EQ.2) THEN
            CRZ=0.5*R2DP(I1)
         ELSE
            CRZ=R2DC(I1)
         ENDIF
         RR=(R2DP(I1)+DX*CRZ/DD(K))
      ELSE
         RR=R2DP(I1)
      ENDIF
      A11Z(KEY)=A11Z(KEY)+RR*QFR1
  240 CONTINUE
  250 CONTINUE
  260 NUM1=NUM1+LL
      NUM2=NUM2+6
  270 CONTINUE
      RETURN
      END
