*DECK TRIRWW
      SUBROUTINE TRIRWW (IR,NEL,LL4,VOL,MAT,XSGD,SIDE,ZZ,KN,QFR,MUW,
     1 IPW,A11W,ISPLH,R,Q,RH,QH,RT,QT)
*
*-----------------------------------------------------------------------
*
* ASSEMBLY OF SYSTEM MATRICES FOR A MESH CORNER FINITE DIFFERENCE
* DISCRETIZATION IN HEXAGONAL GEOMETRY.
*
* INPUT PARAMETERS:
*  IR     : FIRST DIMENSION OF MATRIX SGD.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  LL4      ORDER OF SYSTEM MATRICES.
*  VOL    : VOLUME OF EACH ELEMENT. DIMENSION VOL(NEL)
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH ELEMENT. DIMENSION MAT(NEL)
*  XSGD   : NUCLEAR PROPERTIES, DERIVATIVES OR FIRST VARIATIONS OF NU-
*           CLEAR PROPERTIES PER MATERIAL MIXTURE. DIMENSION XSGD(IR,4)
*           XSGD(L,1) : W-, X-, AND Y-ORIENTED DIFFUSION COEFFICIENTS
*           XSGD(L,3) : Z-ORIENTED DIFFUSION COEFFICIENTS
*           XSGD(L,4) : REMOVAL MACROSCOPIC CROSS SECTION.
*  SIDE   : SIDE OF AN HEXAGON.
*  ZZ     : Z-DIRECTED MESH SPACINGS. DIMENSION ZZ(NEL)
*  KN     : ELEMENT-ORDERED UNKNOWN LIST.
*           DIMENSION KN(ICOF*NEL) WHERE ICOF=12 OR 14.
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(8*NEL)
*  MUW,MUX,MUY,MUZ : COMPRESSED STORAGE MODE INDICES.
*                    DIMENSION MUW(LL4),MUX(LL4),MUY(LL4),MUZ(LL4)
*  IPW,IPX,IPY,IPZ : PERMUTATION MATRICES.
*                    DIMENSION IPW(LL4),IPX(LL4),IPY(LL4),IPZ(LL4)
*  ISPLH  : =1 FOR HEXAGONAL ELEMENTS ; .GT.1 FOR TRIANGULAR ELEMENTS.
*  R,Q,RH,QH,RT,QT : UNIT MATRICES. DIMENSION R(2,2),Q(2,2),RH(6,6),
*                    QH(6,6),RT(3,3),QT(3,3)
*
* OUTPUT PARAMETERS:
*  A11W,A11X,A11Y,A11Z : MATRICES CORRESPONDING TO THE DIVERGENCE (I.E.
*                        LEAKAGE) AND REMOVAL TERMS.
*
* NOTE: SYSTEM MATRICES SHOULD BE INITIALIZED BY THE CALLING PROGRAM.
*
*----------------------------------- AUTHOR: A. BENABOUD ; 15/01/92 ----
*
      DOUBLE PRECISION RR,QQP,QQZ,VOL0,VOL1,DZ
      DOUBLE PRECISION QTHP(14,14),QTHZ(14,14),RTHG(14,14),
     >           HW(14,14),HX(14,14),HY(14,14),HZ(14,14)
      DIMENSION XSGD(IR,*),ZZ(*),VOL(*),MAT(*),KN(*),QFR(*),MUW(*),
     1 MUX(*),MUY(*),MUZ(*),IPW(*),IPX(*),IPY(*),IPZ(*),A11W(*),
     2 A11X(*),A11Y(*),A11Z(*),HL(2,2),R2DP(4),ISR(8,25),IJ1(14),
     3 IJ2(14),RFAC(28,7),RH2(7,7),QH2(7,7),RF6(24,6),RF7(28,7),
     4 ILIEN(6,3),IJ27(14),ISRH(8,6),ISRT(8,7),IJ16(12),IJ26(12),
     5 IJ17(14),R(2,*),Q(2,*),RH(6,*),QH(6,*),RT(3,*),QT(3,*)
      DATA HL / 1.0,2*0.0,1.0/
      DATA R2DP / 4*0.25 /
      DATA ILIEN/6*4,2,1,5,6,7,3,1,5,6,7,3,2/
      DATA IJ16,IJ26 /1,2,3,4,5,6,1,2,3,4,5,6,6*1,6*2/
      DATA IJ17,IJ27 /1,2,3,4,5,6,7,1,2,3,4,5,6,7,7*1,7*2/
      DATA ISRT/2,1,5,6,7,3,1,8,1,5,6,7,3,2,2,9,9,8,12,13,14,10,3,10,
     >          8,12,13,14,10,9,4,11,6*0,5,12,6*0,6,13,6*0,7,14/
      DATA ISRH/2,1,4,5,6,3,1,7,1,4,5,6,3,2,2,8,8,7,10,11,12,9,3,9,
     >          7,10,11,12,9,8,4,10,6*0,5,11,6*0,6,12/
      DATA RF6/
     >1.0,0.0,0.0,0.0,0.0,0.0,1.0,1.0,0.0,0.0,1.0,0.5,
     >1.0,0.0,1.0,1.0,0.0,0.5,1.0,0.0,0.0,0.0,0.0,0.0,
     >0.0,1.0,1.0,1.0,0.5,0.0,1.0,1.0,0.0,0.0,0.5,1.0,
     >0.0,1.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,
     >0.0,1.0,1.0,0.5,1.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,
     >1.0,0.0,1.0,0.5,0.0,1.0,0.0,0.0,1.0,0.0,0.0,0.0,
     >0.0,1.0,0.5,1.0,1.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,
     >1.0,0.0,0.5,1.0,0.0,1.0,0.0,0.0,0.0,1.0,0.0,0.0,
     >0.0,0.5,1.0,1.0,1.0,0.0,1.0,0.5,0.0,0.0,1.0,1.0,
     >0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,
     >0.0,0.0,0.0,0.0,0.0,1.0,0.5,1.0,0.0,0.0,1.0,1.0,
     >0.5,0.0,1.0,1.0,0.0,1.0,0.0,0.0,0.0,0.0,0.0,1.0/
      DATA RF7/
     >1.0,0.0,0.0,0.0,0.0,0.0,0.0,1.0,1.0,0.0,0.5,0.0,1.0,0.5,
     >1.0,0.0,1.0,0.5,1.0,0.0,0.5,1.0,0.0,0.0,0.0,0.0,0.0,0.0,
     >0.0,1.0,1.0,0.5,1.0,0.5,0.0,1.0,1.0,0.0,0.5,0.0,0.5,1.0,
     >0.0,1.0,0.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,0.0,
     >0.0,1.0,1.0,0.5,0.5,1.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,
     >1.0,0.0,1.0,0.5,0.5,0.0,1.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,
     >0.0,0.5,0.5,1.0,0.5,0.5,0.0,0.5,0.5,0.0,1.0,0.0,0.5,0.5,
     >0.5,0.0,0.5,1.0,0.5,0.0,0.5,0.0,0.0,0.0,1.0,0.0,0.0,0.0,
     >0.0,1.0,0.5,0.5,1.0,1.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,
     >1.0,0.0,0.5,0.5,1.0,0.0,1.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,
     >0.0,0.5,1.0,0.5,1.0,1.0,0.0,1.0,0.5,0.0,0.5,0.0,1.0,1.0,
     >0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,
     >0.0,0.0,0.0,0.0,0.0,0.0,1.0,0.5,1.0,0.0,0.5,0.0,1.0,1.0,
     >0.5,0.0,1.0,0.5,1.0,0.0,1.0,0.0,0.0,0.0,0.0,0.0,0.0,1.0/
      SAVE LL,LC,ISR,HW,HX,HY,HZ,RTHG,QTHP,QTHZ
*
* COMPUTE THE XEHAGONAL MASS (RH2) AND STIFFNESS (QH2) MATRICES.
      IF(ISPLH.EQ.1) THEN
         LC=6
         DO 10 I=1,8
         DO 10 J=1,6
   10    ISR(I,J)=ISRH(I,J)
         DO 20 I=1,2*LC
         IJ1(I)=IJ16(I)
   20    IJ2(I)=IJ26(I)
         DO 30 I=1,4*LC
         DO 30 J=1,LC
   30    RFAC(I,J)=RF6(I,J)
         DO 40 I=1,LC
         DO 40 J=1,LC
         RH2(I,J)=RH(I,J)
   40    QH2(I,J)=QH(I,J)
      ELSE
         LC=7
         DO 50 I=1,8
         DO 50 J=1,7
   50    ISR(I,J)=ISRT(I,J)
         DO 60 I=1,2*LC
         IJ1(I)=IJ17(I)
   60    IJ2(I)=IJ27(I)
         DO 70 I=1,4*LC
         DO 70 J=1,LC
   70    RFAC(I,J)=RF7(I,J)
         DO 75 I=1,LC
         DO 75 J=1,LC
         RH2(I,J)=0.0
   75    QH2(I,J)=0.0
         DO 80 K=1,6
         DO 80 I=1,3
         NUMI=ILIEN(K,I)
         DO 80 J=1,3
         NUMJ=ILIEN(K,J)
         RH2(NUMI,NUMJ)=RH2(NUMI,NUMJ)+RT(I,J)
   80    QH2(NUMI,NUMJ)=QH2(NUMI,NUMJ)+QT(I,J)
      ENDIF
      LL=2*LC
      DO 90 I=1,LL
      I1=IJ1(I)
      I2=IJ2(I)
      DO 90 J=1,LL
      J1=IJ1(J)
      J2=IJ2(J)
      HW(I,J)  =RFAC(I1     ,J1) * HL(I2,J2)
      HX(I,J)  =RFAC(I1+LC  ,J1) * HL(I2,J2)
      HY(I,J)  =RFAC(I1+2*LC,J1) * HL(I2,J2)
      HZ(I,J)  =RFAC(I1+3*LC,J1)
      RTHG(I,J)=RH2(I1,J1) * R(I2,J2)
      QTHP(I,J)=QH2(I1,J1) * R(I2,J2)
   90 QTHZ(I,J)=RH2(I1,J1) * Q(I2,J2)
*
* ASSEMBLY OF MATRIX A11W.
      NUM1=0
      NUM2=0
      VOL1=SIDE*SIDE
      DO 160 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 160
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 150
         DZ=ZZ(K)
         DO 110 I=1,LL
            INW1=KN(NUM1+I)
            IF(INW1.EQ.0) GO TO 110
            KEY0=MUW(INW1)-INW1
            DO 100 J=1,LL
               INW2=KN(NUM1+J)
               IF(INW2.EQ.0) GO TO 100
               IF(INW2.EQ.INW1) THEN
                  QQP=QTHP(I,J)*DZ
                  QQZ=QTHZ(I,J)*VOL1/DZ
                  KEY=KEY0+INW2
                  A11W(KEY)=A11W(KEY)+QQP*XSGD(L,1)+QQZ*XSGD(L,3)
               ELSE IF((INW2.LT.INW1).AND.(HW(I,J).NE.0.0)) THEN
                  QQP=QTHP(I,J)*HW(I,J)*DZ
                  KEY=KEY0+INW2
                  A11W(KEY)=A11W(KEY)+QQP*XSGD(L,1)
               ENDIF
  100       CONTINUE
            RR=RTHG(I,I)*VOL1*DZ
            KEY=KEY0+INW1
            A11W(KEY)=A11W(KEY)+RR*XSGD(L,4)
  110    CONTINUE
         DO 140 IC=1,8
            QFR1=QFR(NUM2+IC)
            IF(QFR1.EQ.0.0) GO TO 140
            IF(IC.LT.7) THEN
               DO 120 I1=1,4
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 120
                  KEY=MUW(INW1)
                  RR=R2DP(I1)
                  A11W(KEY)=A11W(KEY)+RR*QFR1
  120          CONTINUE
            ELSE
               DO 130 I1=1,LC
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 130
                  KEY=MUW(INW1)
                  RR=RTHG(I1,I1)
                  A11W(KEY)=A11W(KEY)+RR*QFR1
  130          CONTINUE
            ENDIF
  140    CONTINUE
  150    NUM1=NUM1+LL
         NUM2=NUM2+8
  160 CONTINUE
      RETURN
*
      ENTRY TRIRWX (IR,NEL,LL4,VOL,MAT,XSGD,SIDE,ZZ,KN,QFR,MUX,IPX,A11X)
*
* ASSEMBLY OF MATRIX A11X.
      NUM1=0
      NUM2=0
      VOL1=SIDE*SIDE
      DO 230 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 230
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 220
         DZ=ZZ(K)
         DO 180 I=1,LL
            INW1=KN(NUM1+I)
            IF(INW1.EQ.0) GO TO 180
            INX1=IPX(INW1)
            KEY0=MUX(INX1)-INX1
            DO 170 J=1,LL
               INW2=KN(NUM1+J)
               IF(INW2.EQ.0) GO TO 170
               INX2=IPX(INW2)
               IF(INX2.EQ.INX1) THEN
                  QQP=QTHP(I,J)*DZ
                  QQZ=QTHZ(I,J)*VOL1/DZ
                  KEY=KEY0+INX2
                  A11X(KEY)=A11X(KEY)+QQP*XSGD(L,1)+QQZ*XSGD(L,3)
               ELSE IF((INX2.LT.INX1).AND.(HX(I,J).NE.0.0)) THEN
                  QQP=QTHP(I,J)*HX(I,J)*DZ
                  KEY=KEY0+INX2
                  A11X(KEY)=A11X(KEY)+QQP*XSGD(L,1)
               ENDIF
  170       CONTINUE
            RR=RTHG(I,I)*VOL1*DZ
            KEY=KEY0+INX1
            A11X(KEY)=A11X(KEY)+RR*XSGD(L,4)
  180    CONTINUE
         DO 210 IC=1,8
            QFR1=QFR(NUM2+IC)
            IF(QFR1.EQ.0.0) GO TO 210
            IF(IC.LT.7) THEN
               DO 190 I1=1,4
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 190
                  INX1=IPX(INW1)
                  KEY=MUX(INX1)
                  RR=R2DP(I1)
                  A11X(KEY)=A11X(KEY)+RR*QFR1
  190          CONTINUE
            ELSE
               DO 200 I1=1,LC
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 200
                  INX1=IPX(INW1)
                  KEY=MUX(INX1)
                  RR=RTHG(I1,I1)
                  A11X(KEY)=A11X(KEY)+RR*QFR1
  200          CONTINUE
            ENDIF
  210    CONTINUE
  220    NUM1=NUM1+LL
         NUM2=NUM2+8
  230 CONTINUE
      RETURN
*
      ENTRY TRIRWY (IR,NEL,LL4,VOL,MAT,XSGD,SIDE,ZZ,KN,QFR,MUY,IPY,A11Y)
*
* ASSEMBLY OF MATRIX A11Y.
      NUM1=0
      NUM2=0
      VOL1=SIDE*SIDE
      DO 300 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 300
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 290
         DZ=ZZ(K)
         DO 250 I=1,LL
            INW1=KN(NUM1+I)
            IF(INW1.EQ.0) GO TO 250
            INY1=IPY(INW1)
            KEY0=MUY(INY1)-INY1
            DO 240 J=1,LL
               INW2=KN(NUM1+J)
               IF(INW2.EQ.0) GO TO 240
               INY2=IPY(INW2)
               IF(INY2.EQ.INY1) THEN
                  QQP=QTHP(I,J)*DZ
                  QQZ=QTHZ(I,J)*VOL1/DZ
                  KEY=KEY0+INY2
                  A11Y(KEY)=A11Y(KEY)+QQP*XSGD(L,1)+QQZ*XSGD(L,3)
               ELSE IF((INY2.LT.INY1).AND.(HY(I,J).NE.0.0)) THEN
                  QQP=QTHP(I,J)*HY(I,J)*DZ
                  KEY=KEY0+INY2
                  A11Y(KEY)=A11Y(KEY)+QQP*XSGD(L,1)
               ENDIF
  240       CONTINUE
            RR=RTHG(I,I)*VOL1*DZ
            KEY=KEY0+INY1
            A11Y(KEY)=A11Y(KEY)+RR*XSGD(L,4)
  250    CONTINUE
         DO 280 IC=1,8
            QFR1=QFR(NUM2+IC)
            IF(QFR1.EQ.0.0) GO TO 280
            IF(IC.LT.7) THEN
               DO 260 I1=1,4
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 260
                  INY1=IPY(INW1)
                  KEY=MUY(INY1)
                  RR=R2DP(I1)
                  A11Y(KEY)=A11Y(KEY)+RR*QFR1
  260          CONTINUE
            ELSE
               DO 270 I1=1,LC
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 270
                  INY1=IPY(INW1)
                  KEY=MUY(INY1)
                  RR=RTHG(I1,I1)
                  A11Y(KEY)=A11Y(KEY)+RR*QFR1
  270          CONTINUE
            ENDIF
  280    CONTINUE
  290    NUM1=NUM1+LL
         NUM2=NUM2+8
  300 CONTINUE
      RETURN
*
      ENTRY TRIRWZ (IR,NEL,LL4,VOL,MAT,XSGD,SIDE,ZZ,KN,QFR,MUZ,IPZ,A11Z)
*
* ASSEMBLY OF MATRIX A11Z.
      NUM1=0
      NUM2=0
      VOL1=SIDE*SIDE
      DO 360 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 360
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 350
         DZ=ZZ(K)
         DO 320 I=1,LL
            INW1=KN(NUM1+I)
            IF(INW1.EQ.0) GO TO 320
            INZ1=IPZ(INW1)
            KEY0=MUZ(INZ1)-INZ1
            DO 310 J=1,LL
               INW2=KN(NUM1+J)
               IF(INW2.EQ.0) GO TO 310
               INZ2=IPZ(INW2)
               IF(INZ2.EQ.INZ1) THEN
                  QQP=QTHP(I,J)*DZ
                  QQZ=QTHZ(I,J)*VOL1/DZ
                  KEY=KEY0+INZ2
                  A11Z(KEY)=A11Z(KEY)+QQP*XSGD(L,1)+QQZ*XSGD(L,3)
               ELSE IF((INZ2.LT.INZ1).AND.(HZ(I,J).NE.0.0)) THEN
                  QQZ=QTHZ(I,J)*VOL1/DZ
                  KEY=KEY0+INZ2
                  A11Z(KEY)=A11Z(KEY)+QQZ*XSGD(L,1)
               ENDIF
  310       CONTINUE
            RR=RTHG(I,I)*VOL1*DZ
            KEY=KEY0+INZ1
            A11Z(KEY)=A11Z(KEY)+RR*XSGD(L,4)
  320    CONTINUE
         DO 340 IC=1,8
            QFR1=QFR(NUM2+IC)
            IF(QFR1.EQ.0.0) GO TO 340
            IF(IC.LT.7) THEN
               DO 330 I1=1,4
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 330
                  INZ1=IPZ(INW1)
                  KEY=MUZ(INZ1)
                  RR=R2DP(I1)
                  A11Z(KEY)=A11Z(KEY)+RR*QFR1
  330          CONTINUE
            ELSE
               DO 335 I1=1,LC
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 335
                  INZ1=IPZ(INW1)
                  KEY=MUZ(INZ1)
                  RR=RTHG(I1,I1)
                  A11Z(KEY)=A11Z(KEY)+RR*QFR1
  335          CONTINUE
            ENDIF
  340    CONTINUE
  350    NUM1=NUM1+LL
         NUM2=NUM2+8
  360 CONTINUE
      RETURN
      END
