*DECK TRISYS
      SUBROUTINE TRISYS (IPTRK,IPMACR,IPMACP,IPSYS,IMPX,NGRP,NEL,
     x NUN,IPR,MAT,VOL,IDL,TITLE,MAXMIX,SGD,DSGD,WORK,IJJ,NJJ,IPOS,
     x NDG)
*
*-----------------------------------------------------------------------
*
* RECOVER THE DIFFUSION COEFFICIENT AND CROSS-SECTION DATA IN THE LINKED
* LIST OR XSM FILE WITH POINTER IPMACR, COMPUTE AND STORE THE CORRES-
* PONDING SYSTEM MATRICES (OR A PERTURBATION TO THE SYSTEM MATRICES).
*
* INPUT PARAMETERS:
*  IPTRK  : L_TRIVAC POINTER TO THE TRIVAC TRACKING INFORMATION.
*  IPMACR : L_MACROLIB POINTER TO THE UNPERTURBED CROSS SECTIONS.
*  IPMACP : L_MACROLIB POINTER TO THE PERTURBED CROSS SECTIONS IF
*           IPR.GT.0. EQUAL TO IPMACR IF IPR=0.
*  IPSYS  : L_SYSTEM POINTER TO SYSTEM MATRICES.
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NEL    : TOTAL NUMBER OF FINITE ELEMENTS.
*  NUN    : TOTAL NUMBER OF UNKNOWNS PER GROUP.
*  NDG    : NUMBER OF DELAYED NEUTRON GROUPS.
*  IPR=0  : CALCULATION OF THE SYSTEM MATRICES;
*  IPR=1  : CALCULATION OF THE DERIVATIVE OF THESE MATRICES;
*  IPR=2  : CALCULATION OF THE FIRST VARIATION OF THESE MATRICES;
*  IPR=3  : IDENTICAL TO IPR=2, BUT THESE VARIATION ARE ADDED TO
*           UNPERTURBED SYSTEM MATRICES.
*  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*           DIMENSION MAT(NEL)
*  VOL    : VOLUMES. DIMENSION VOL(NEL)
*  IDL    : POSITION OF THE FLUX COMPONENT ASSOCIATED WITH EACH VOLUME.
*           DIMENSION IDL(NEL)
*  TITLE  : CHARACTER*72 TITLE.
*  MAXMIX : DIMENSION FOR SCRATCH STORAGE ALLOCATION.
*
* OUTPUT PARAMETER:
*  IPSYS : L_SYSTEM POINTER TO THE SYSTEM MATRICES.
*
* SCRATCH STORAGE:
*  SGD    : DIMENSION SGD(MAXMIX,4)
*  DSGD   : DIMENSION DSGD(MAXMIX,4) CAN BE EQUIVALENCED TO SGD IF IPR=0
*  WORK   : DIMENSION WORK(MAXMIX*NGRP)
*  IJJ    : DIMENSION IJJ(MAXMIX)
*  NJJ    : DIMENSION NJJ(MAXMIX)
*  IPOS   : DIMENSION IPOS(MAXMIX)
*
*--------------------------------------------- A. HEBERT ; 02/01/94 ----
*
      CHARACTER CGRPNM*12,TEXT12*12,TEXT*8
      LOGICAL LFIS
      DIMENSION MAT(*),VOL(*),IDL(*),SGD(MAXMIX,*),DSGD(MAXMIX,*),
     1 WORK(*),IJJ(*),NJJ(*),IPOS(*),ISPARM(7)
*
      DO 5 I=1,7
        ISPARM(I) = 0
   5  CONTINUE
      ISPARM(1) = NGRP
      ISPARM(3) = NDG
      DO 70 IGR=1,NGRP
*     PROCESS SECONDARY GROUP IGR.
      WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
      CALL LCMSIX(IPMACR,CGRPNM,1)
      IF(IPMACR.NE.IPMACP) CALL LCMSIX(IPMACP,CGRPNM,1)
*
* PROCESS LEAKAGE AND REMOVAL TERMS.
      CALL LCMLEN(IPMACR,'TOTAL',NBMIX,ITYLCM)
      IF(NBMIX.EQ.0) THEN
         CALL XABORT('TRISYS: NO TOTAL CROSS SECTIONS.')
      ELSE IF(NBMIX.GT.MAXMIX) THEN
         CALL XABORT('TRISYS: INVALID LENGTH FOR TOTAL CROSS SECTIONS.')
      ENDIF
      CALL LCMGET(IPMACR,'TOTAL',SGD(1,4))
      CALL LCMLEN(IPMACR,'SIGW 0',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         IF(LENGT.GT.MAXMIX) CALL XABORT('TRISYS: INVALID LENGTH FOR '
     1   //'''SIGW 0'' CROSS SECTIONS.')
         CALL LCMGET(IPMACR,'SIGW 0',SGD(1,1))
         DO 10 IBM=1,NBMIX
   10    SGD(IBM,4)=SGD(IBM,4)-SGD(IBM,1)
      ENDIF
      CALL LCMLEN(IPMACR,'DIFFX',NBMIX,ITYLCM)
      IF(NBMIX.EQ.0) THEN
         CALL XABORT('TRISYS: NO DIFFUSION COEFFICIENTS.')
      ELSE IF(NBMIX.GT.MAXMIX) THEN
         CALL XABORT('TRISYS: INVALID LENGTH FOR DIFFUSION COEFFS.')
      ENDIF
      CALL LCMGET(IPMACR,'DIFFX',SGD(1,1))
      CALL LCMLEN(IPMACR,'DIFFY',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(IPMACR,'DIFFY',SGD(1,2))
      ELSE
         CALL LCMGET(IPMACR,'DIFFX',SGD(1,2))
      ENDIF
      CALL LCMLEN(IPMACR,'DIFFZ',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(IPMACR,'DIFFZ',SGD(1,3))
      ELSE
         CALL LCMGET(IPMACR,'DIFFX',SGD(1,3))
      ENDIF
      WRITE(TEXT12,'(1HA,2I3)') IGR,IGR
      IF(IPR.EQ.0) THEN
*        COMPUTE UNPERTURBED SYSTEM MATRICES.
         CALL TRIASM(TEXT12,IPTRK,IPSYS,IMPX,MAXMIX,NEL,NUN,0,MAT,VOL,
     1   IDL,SGD,SGD,TITLE,ISPARM)
      ELSE
*        COMPUTE A PERTURBATION TO THE SYSTEM MATRICES.
         DO 20 J=1,4
         DO 20 IBM=1,NBMIX
  20     DSGD(IBM,J)=0.0
         CALL LCMLEN(IPMACP,'TOTAL',NBMIX,ITYLCM)
         IF(NBMIX.GT.0) THEN
            IF(NBMIX.GT.MAXMIX) CALL XABORT('TRISYS: INVALID LENGTH FOR'
     1      //' DELTA TOTAL CROSS SECTIONS.')
            CALL LCMGET(IPMACP,'TOTAL',DSGD(1,4))
         ENDIF
         CALL LCMLEN(IPMACP,'SIGW 0',LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            IF(LENGT.GT.MAXMIX) CALL XABORT('TRISYS: INVALID LENGTH FOR'
     1      //' DELTA ''SIGW 0'' CROSS SECTIONS.')
            CALL LCMGET(IPMACP,'SIGW 0',DSGD(1,1))
            DO 30 IBM=1,NBMIX
            DSGD(IBM,4)=DSGD(IBM,4)-DSGD(IBM,1)
   30       DSGD(IBM,1)=0.0
         ENDIF
         CALL LCMLEN(IPMACP,'DIFFX',NBMIX,ITYLCM)
         IF(NBMIX.GT.0) THEN
            CALL LCMGET(IPMACP,'DIFFX',DSGD(1,1))
            CALL LCMLEN(IPMACP,'DIFFY',LENGT,ITYLCM)
            IF(LENGT.GT.0) THEN
               CALL LCMGET(IPMACP,'DIFFY',DSGD(1,2))
            ELSE
            CALL LCMGET(IPMACP,'DIFFX',DSGD(1,2))
            ENDIF
            CALL LCMLEN(IPMACP,'DIFFZ',LENGT,ITYLCM)
            IF(LENGT.GT.0) THEN
               CALL LCMGET(IPMACP,'DIFFZ',DSGD(1,3))
            ELSE
               CALL LCMGET(IPMACP,'DIFFX',DSGD(1,3))
            ENDIF
         ENDIF
         CALL TRIASM(TEXT12,IPTRK,IPSYS,IMPX,MAXMIX,NEL,NUN,IPR,MAT,VOL,
     1   IDL,SGD,DSGD,TITLE,ISPARM)
      ENDIF
*
* PROCESS SCATTERING TERMS.
      CALL LCMLEN(IPMACP,'NJJ 0',NBMIX,ITYLCM)
      IF(NBMIX.GT.MAXMIX) CALL XABORT('TRISYS: INVALID LENGTH FOR ''N'
     1 //'JJ 0'' INFORMATION.')
      IF(NBMIX.GT.0) THEN
         CALL LCMGET(IPMACP,'NJJ 0',NJJ)
         CALL LCMGET(IPMACP,'IJJ 0',IJJ)
         JGRMIN=IGR
         JGRMAX=IGR
         DO 40 IBM=1,NBMIX
         JGRMIN=MIN(JGRMIN,IJJ(IBM)-NJJ(IBM)+1)
   40    JGRMAX=MAX(JGRMAX,IJJ(IBM))
         CALL LCMGET(IPMACP,'IPOS 0',IPOS)
         CALL LCMGET(IPMACP,'SCAT 0',WORK)
         DO 60 JGR=JGRMAX,JGRMIN,-1
         IF(JGR.EQ.IGR) GO TO 60
         DO 50 IBM=1,NBMIX
         IF((JGR.GT.IJJ(IBM)-NJJ(IBM)).AND.(JGR.LE.IJJ(IBM))) THEN
            SGD(IBM,1)=WORK(IPOS(IBM)+IJJ(IBM)-JGR)
         ELSE
            SGD(IBM,1)=0.0
         ENDIF
   50    CONTINUE
         WRITE (TEXT12,'(1HA,I3,I3)') IGR,JGR
         CALL TRIDIG(TEXT12,IPTRK,IPSYS,IMPX,NEL,NUN,IPR,MAT,VOL,IDL,
     1   SGD)
   60    CONTINUE
      ENDIF
      IF(IPMACR.NE.IPMACP) CALL LCMSIX(IPMACP,' ',2)
      CALL LCMSIX(IPMACR,' ',2)
   70 CONTINUE
*
* PROCESS FISSION SPECTRUM TERMS.
      WRITE (CGRPNM,'(9HGROUP  1/,I3)') NGRP
      CALL LCMSIX(IPMACP,CGRPNM,1)
      CALL LCMLEN(IPMACP,'CHI',NBMIX,ITYLCM)
      IF(NBMIX.GT.MAXMIX) CALL XABORT('TRISYS: INVALID LENGTH FOR CHI'
     1 //' INFORMATION.')
      CALL LCMSIX(IPMACP,' ',2)
      IF(NBMIX.GT.0) THEN
         DO 80 IGR=1,NGRP
         WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
         CALL LCMSIX(IPMACP,CGRPNM,1)
         CALL LCMGET(IPMACP,'CHI',WORK((IGR-1)*MAXMIX+1))
         CALL LCMSIX(IPMACP,' ',2)
   80    CONTINUE
      ELSE
         NBMIX=MAXMIX
         DO 90 IBM=1,MAXMIX
         WORK(IBM)=1.0
         DO 90 IGR=2,NGRP
   90    WORK((IGR-1)*MAXMIX+IBM)=0.0
      ENDIF
*
* PROCESS FISSION NUSIGF TERMS.
      DO 130 IGR=1,NGRP
*     PROCESS SECONDARY GROUP IGR.
      LFIS=.FALSE.
      DO 100 IBM=1,NBMIX
  100 LFIS=LFIS.OR.(WORK((IGR-1)*MAXMIX+IBM).NE.0.0)
      IF(LFIS) THEN
         DO 120 JGR=1,NGRP
         WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') JGR,NGRP
         CALL LCMSIX(IPMACP,CGRPNM,1)
         CALL LCMLEN(IPMACP,'NUSIGF',LENGT,ITYLCM)
         IF(LENGT.GT.MAXMIX) CALL XABORT('TRISYS: INVALID LENGTH FOR '
     1    //'NUSIGF INFORMATION.')
         IF(LENGT.GT.0) THEN
            CALL LCMGET(IPMACP,'NUSIGF',SGD)
            DO 110 IBM=1,LENGT
  110       SGD(IBM,1)=SGD(IBM,1)*WORK((IGR-1)*MAXMIX+IBM)
            WRITE (TEXT12,'(1HB,I3,I3)') IGR,JGR
            CALL TRIDIG(TEXT12,IPTRK,IPSYS,IMPX,NEL,NUN,IPR,MAT,VOL,IDL,
     1      SGD)
         ENDIF
         CALL LCMSIX(IPMACP,' ',2)
  120    CONTINUE
      ENDIF
  130 CONTINUE
*
* PROCESS DELAYED FISSION SPECTRUM.
      IF(NDG.NE.0) THEN
      DO 900 I=1,NDG
      WRITE (TEXT,'(4HCHDL,I3)') I
      WRITE (CGRPNM,'(9HGROUP  1/,I3)') NGRP
      CALL LCMSIX(IPMACP,CGRPNM,1)
      CALL LCMLEN(IPMACP,TEXT,NBMIX,ITYLCM)
      IF(NBMIX.GT.MAXMIX) CALL XABORT('TRISYS: INVALID LENGTH FOR CHDL'
     1 //' INFORMATION.')
      CALL LCMSIX(IPMACP,' ',2)
      IF(NBMIX.GT.0) THEN
         DO 980 IGR=1,NGRP
         WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
         CALL LCMSIX(IPMACP,CGRPNM,1)
         CALL LCMGET(IPMACP,TEXT,WORK((IGR-1)*MAXMIX+1))
         CALL LCMSIX(IPMACP,' ',2)
 980    CONTINUE
      ELSE
         NBMIX=MAXMIX
         DO 990 IBM=1,MAXMIX
         WORK(IBM)=1.0
         DO 990 IGR=2,NGRP
 990    WORK((IGR-1)*MAXMIX+IBM)=0.0
      ENDIF
*
* PROCESS FISSION NUSIGF TERMS.
      DO 930 IGR=1,NGRP
*     PROCESS SECONDARY GROUP IGR.
      LFIS=.FALSE.
      DO 800 IBM=1,NBMIX
  800 LFIS=LFIS.OR.(WORK((IGR-1)*MAXMIX+IBM).NE.0.0)
      IF(LFIS) THEN
         DO 820 JGR=1,NGRP
         WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') JGR,NGRP
         CALL LCMSIX(IPMACP,CGRPNM,1)
         CALL LCMLEN(IPMACP,'NUSIGF',LENGT,ITYLCM)
         IF(LENGT.GT.MAXMIX) CALL XABORT('TRISYS: INVALID LENGTH FOR '
     1    //'NUSIGF INFORMATION.')
         IF(LENGT.GT.0) THEN
            CALL LCMGET(IPMACP,'NUSIGF',SGD)
            DO 810 IBM=1,LENGT
  810       SGD(IBM,1)=SGD(IBM,1)*WORK((IGR-1)*MAXMIX+IBM)
            WRITE (TEXT12,'(1HB,I3,I3,I3)') I,IGR,JGR
            CALL TRIDIG(TEXT12,IPTRK,IPSYS,IMPX,NEL,NUN,IPR,MAT,VOL,IDL,
     1      SGD)
         ENDIF
         CALL LCMSIX(IPMACP,' ',2)
  820    CONTINUE
      ENDIF
  930 CONTINUE
  900 CONTINUE
      ENDIF
*
      CALL LCMSIX(IPSYS,' ',0)
      CALL LCMPUT(IPSYS,'SYS-PARAM',7,1,ISPARM)
*
      RETURN
      END
