*DECK TRITCO
      SUBROUTINE TRITCO (ISPLH,IR,IQF,K,KK1,KK2,KK3,KK4,KK5,VOL0,MAT,
     1 MATN,DIF,DDF,SIDE,ZZ,QFR,IPR,A)
*
*-----------------------------------------------------------------------
*
* MESH CENTERED FINITE DIFFERENCE COEFFICIENTS IN HEXAGONAL GEOMETRY
* WITH TRIANGULAR SUB MESHING.
*
* INPUT PARAMETERS:
*  ISPLH  : THE NUMBER OF TRIANGLES IS 6*(ISPLH-1)**2.
*  IR     : FIRST DIMENSION OF MATRICES DIF AND DDF.
*  IQF    : INDEX IN QFR.
*  K      : INDEX OF FINITE ELEMENT.
*  KK1,KK2,KK3,KK4,KK5 : NEIGHBOURS OF THE TRIANGULAR FINITE ELEMENT.
*  VOL0   : VOLUME OF THE FINITE ELEMENT.
*  MAT    : MIXTURE INDEX ASSIGNED TO EACH HEXAGON. DIMENSION MAT(NEL)
*  MATN   : MIXTURE INDEX ASSIGNED TO EACH TRIANGLE.
*           DIMENSION MATN(LL4)
*  DIF    : DIRECTIONAL DIFFUSION COEFFICIENTS. DIMENSION DIF(IR,3)
*  DDF    : VARIATION OF DIRECTIONAL DIFFUSION COEFFICIENTS.
*           DIMENSION DDF(IR,3)
*  SIDE   : SIDE OF AN HEXAGON.
*  ZZ     : Z-DIRECTED MESH SPACINGS. DIMENSION ZZ(NEL)
*  QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS. DIMENSION QFR(8*NEL)
*  IPR    : =0: COMPUTE THE SYSTEM MATRICES ;
*           =1: COMPUTE THE DERIVATIVE OF SYSTEM MATRICES ;
*           =2 OR =3: COMPUTE THE VARIATION OF SYSTEM MATRICES ;
*
* OUTPUT PARAMETERS:
*  A      : MESH CENTERED FINITE DIFFERENCE COEFFICIENTS.
*           DIMENSION A(5)
*
*----------------------------------- AUTHOR: A. BENABOUD ; 24/02/92 ----
*
      DOUBLE PRECISION A(5),SHARM,DHARM,VHARM
      DIMENSION MAT(*),MATN(*),DIF(IR,*),DDF(IR,*),ZZ(*),QFR(8)
*     FORMULA WIHOUT VARIATION OR DERIVATIVE.
      SHARM(X1,X2,DIF1,DIF2)=2.0D0*DIF1*DIF2/(X1*DIF2+X2*DIF1)
*     FORMULA WITH DERIVATIVE.
      DHARM(X1,X2,DIF1,DIF2,DDF1,DDF2)=2.0D0*(X1*DIF2*DIF2*DDF1+
     1 X2*DIF1*DIF1*DDF2)/(X1*DIF2+X2*DIF1)**2
*     FORMULA WITH VARIATION.
      VHARM(X1,X2,DIF1,DIF2,DDF1,DDF2)=2.0D0*((DIF1+DDF1)*(DIF2+DDF2)
     1 /(X1*(DIF2+DDF2)+X2*(DIF1+DDF1))-DIF1*DIF2/(X1*DIF2+X2*DIF1))
*
      L=MAT(K)
      DZ=ZZ(K)
      DS=SIDE/(SQRT(3.0)*(ISPLH-1))
      DT=SIDE/(ISPLH-1)
      IF(IPR.EQ.0) THEN
*        FORMULE DIRECTE.
         IF(KK1.GT.0) THEN
            A(1)=SHARM(DS,DS,DIF(L,1),DIF(MATN(KK1),1))*DT*DZ
         ELSE IF(KK1.EQ.-1) THEN
            A(1)=SHARM(DS,DS,DIF(L,1),DS*QFR(IQF)/2.0)*DT*DZ
         ELSE IF(KK1.EQ.-2) THEN
            A(1)=0.0D0
         ELSE IF(KK1.EQ.-3) THEN
            A(1)=2.0D0*SHARM(DS,DS,DIF(L,1),DIF(L,1))*DT*DZ
         ENDIF
*
         IF(KK2.GT.0) THEN
            A(2)=SHARM(DS,DS,DIF(L,1),DIF(MATN(KK2),1))*DT*DZ
         ELSE IF(KK2.EQ.-1) THEN
            A(2)=SHARM(DS,DS,DIF(L,1),DS*QFR(IQF)/2.0)*DT*DZ
         ELSE IF(KK2.EQ.-2) THEN
            A(2)=0.0D0
         ELSE IF(KK2.EQ.-3) THEN
            A(2)=2.0D0*SHARM(DS,DS,DIF(L,1),DIF(L,1))*DT*DZ
         ENDIF
*
         IF(KK3.GT.0) THEN
            A(3)=SHARM(DS,DS,DIF(L,1),DIF(MATN(KK3),1))*DT*DZ
         ELSE IF(KK3.EQ.-1) THEN
            A(3)=SHARM(DS,DS,DIF(L,1),DS*QFR(IQF)/2.0)*DT*DZ
         ELSE IF(KK3.EQ.-2) THEN
            A(3)=0.0D0
         ELSE IF(KK3.EQ.-3) THEN
            A(3)=2.0D0*SHARM(DS,DS,DIF(L,1),DIF(L,1))*DT*DZ
         ENDIF
*
         IF(KK4.GT.0) THEN
            A(4)=SHARM(DZ,ZZ(KK4),DIF(L,1),DIF(MAT(KK4),1))*VOL0/DZ
         ELSE IF(KK4.EQ.-1) THEN
            A(4)=SHARM(DZ,DZ,DIF(L,1),DZ*QFR(7)/2.0)*VOL0/DZ
         ELSE IF(KK4.EQ.-2) THEN
            A(4)=0.0D0
         ELSE IF(KK4.EQ.-3) THEN
            A(4)=2.0D0*SHARM(DZ,DZ,DIF(L,1),DIF(L,1))*VOL0/DZ
         ENDIF
*
         IF(KK5.GT.0) THEN
            A(5)=SHARM(DZ,ZZ(KK5),DIF(L,1),DIF(MAT(KK5),1))*VOL0/DZ
         ELSE IF(KK5.EQ.-1) THEN
            A(5)=SHARM(DZ,DZ,DIF(L,1),DZ*QFR(8)/2.0)*VOL0/DZ
         ELSE IF(KK5.EQ.-2) THEN
            A(5)=0.0D0
         ELSE IF(KK5.EQ.-3) THEN
            A(5)=2.0D0*SHARM(DZ,DZ,DIF(L,1),DIF(L,1))*VOL0/DZ
         ENDIF
*
      ELSE IF(IPR.EQ.1) THEN
*        FORMULE DE DERIVEE.
         IF(KK1.GT.0) THEN
            A(1)=DHARM(DS,DS,DIF(L,1),DIF(MATN(KK1),1),DDF(L,1),
     1           DDF(MATN(KK1),1))*DZ*DT
         ELSE IF(KK1.EQ.-1) THEN
            A(1)=DHARM(DS,DS,DIF(L,1),DS*QFR(IQF)/2.0,DDF(L,1),0.0)
     1           *DZ*DT
         ELSE IF(KK1.EQ.-2) THEN
            A(1)=0.0D0
         ELSE IF(KK1.EQ.-3) THEN
            A(1)=2.0D0*DDF(L,1)*DZ*DT/DS
         ENDIF
*
         IF(KK2.GT.0) THEN
            A(2)=DHARM(DS,DS,DIF(L,1),DIF(MATN(KK2),1),DDF(L,1),
     1           DDF(MATN(KK2),1))*DZ*DT
         ELSE IF(KK2.EQ.-1) THEN
            A(2)=DHARM(DS,DS,DIF(L,1),DS*QFR(IQF)/2.0,DDF(L,1),0.0)
     1           *DZ*DT
         ELSE IF(KK2.EQ.-2) THEN
            A(2)=0.0D0
         ELSE IF(KK2.EQ.-3) THEN
            A(2)=2.0D0*DDF(L,1)*DZ*DT/DS
         ENDIF
*
         IF(KK3.GT.0) THEN
            A(3)=DHARM(DS,DS,DIF(L,1),DIF(MATN(KK3),1),DDF(L,1),
     1           DDF(MATN(KK3),1))*DZ*DT
         ELSE IF(KK3.EQ.-1) THEN
            A(3)=DHARM(DS,DS,DIF(L,1),DS*QFR(IQF)/2.0,DDF(L,1),0.0)
     1           *DZ*DT
         ELSE IF(KK3.EQ.-2) THEN
            A(3)=0.0D0
         ELSE IF(KK3.EQ.-3) THEN
            A(3)=2.0D0*DDF(L,1)*DZ*DT/DS
         ENDIF
*
         IF(KK4.GT.0) THEN
            A(4)=DHARM(DZ,ZZ(KK4),DIF(L,3),DIF(MAT(KK4),3),DDF(L,3),
     1           DDF(MAT(KK4),3))*VOL0/DZ
         ELSE IF(KK4.EQ.-1) THEN
            A(4)=DHARM(DZ,DZ,DIF(L,3),DZ*QFR(7)/2.0,DDF(L,3),0.0)
     1           *VOL0/DZ
         ELSE IF(KK4.EQ.-2) THEN
            A(4)=0.0D0
         ELSE IF(KK4.EQ.-3) THEN
            A(4)=2.0D0*DDF(L,3)*VOL0/(DZ*DZ)
         ENDIF
*
         IF(KK5.GT.0) THEN
            A(5)=DHARM(DZ,ZZ(KK5),DIF(L,3),DIF(MAT(KK5),3),DDF(L,3),
     1           DDF(MAT(KK5),3))*VOL0/DZ
         ELSE IF(KK5.EQ.-1) THEN
            A(5)=DHARM(DZ,DZ,DIF(L,3),DZ*QFR(8)/2.0,DDF(L,3),0.0)
     1           *VOL0/DZ
         ELSE IF(KK5.EQ.-2) THEN
            A(5)=0.0D0
         ELSE IF(KK5.EQ.-3) THEN
            A(5)=2.0D0*DDF(L,3)*VOL0/(DZ*DZ)
         ENDIF
*
      ELSE IF(IPR.GE.2) THEN
*        FORMULE DE VARIATION.
         IF(KK1.GT.0) THEN
            A(1)=VHARM(DS,DS,DIF(L,1),DIF(MATN(KK1),1),DDF(L,1),
     1           DDF(MATN(KK1),1))*DZ*DT
         ELSE IF(KK1.EQ.-1) THEN
            A(1)=VHARM(DS,DS,DIF(L,1),DS*QFR(IQF)/2.0,DDF(L,1),0.0)
     1           *DZ*DT
         ELSE IF(KK1.EQ.-2) THEN
            A(1)=0.0D0
         ELSE IF(KK1.EQ.-3) THEN
            A(1)=2.0D0*DDF(L,1)*DZ*DT/DS
         ENDIF
*
         IF(KK2.GT.0) THEN
            A(2)=VHARM(DS,DS,DIF(L,1),DIF(MATN(KK2),1),DDF(L,1),
     1           DDF(MATN(KK2),1))*DZ*DT
         ELSE IF(KK2.EQ.-1) THEN
            A(2)=VHARM(DS,DS,DIF(L,1),DS*QFR(IQF)/2.0,DDF(L,1),0.0)
     1           *DZ*DT
         ELSE IF(KK2.EQ.-2) THEN
            A(2)=0.0D0
         ELSE IF(KK2.EQ.-3) THEN
            A(2)=2.0D0*DDF(L,1)*DZ*DT/DS
         ENDIF
*
         IF(KK3.GT.0) THEN
            A(3)=VHARM(DS,DS,DIF(L,1),DIF(MATN(KK3),1),DDF(L,1),
     1           DDF(MATN(KK3),1))*DZ*DT
         ELSE IF(KK3.EQ.-1) THEN
            A(3)=VHARM(DS,DS,DIF(L,1),DS*QFR(IQF)/2.0,DDF(L,1),0.0)
     1           *DZ*DT
         ELSE IF(KK3.EQ.-2) THEN
            A(3)=0.0D0
         ELSE IF(KK3.EQ.-3) THEN
            A(3)=2.0D0*DDF(L,1)*DZ*DT/DS
         ENDIF
*
         IF(KK4.GT.0) THEN
            A(4)=VHARM(DZ,ZZ(KK4),DIF(L,3),DIF(MAT(KK4),3),DDF(L,3),
     1           DDF(MAT(KK4),3))*VOL0/DZ
         ELSE IF(KK4.EQ.-1) THEN
            A(4)=VHARM(DZ,DZ,DIF(L,3),DZ*QFR(7)/2.0,DDF(L,3),0.0)
     1           *VOL0/DZ
         ELSE IF(KK4.EQ.-2) THEN
            A(4)=0.0D0
         ELSE IF(KK4.EQ.-3) THEN
            A(4)=2.0D0*DDF(L,3)*VOL0/(DZ*DZ)
         ENDIF
*
         IF(KK5.GT.0) THEN
            A(5)=VHARM(DZ,ZZ(KK5),DIF(L,3),DIF(MAT(KK5),3),DDF(L,3),
     1           DDF(MAT(KK5),3))*VOL0/DZ
         ELSE IF(KK5.EQ.-1) THEN
            A(5)=VHARM(DZ,DZ,DIF(L,3),DZ*QFR(8)/2.0,DDF(L,3),0.0)
     1           *VOL0/DZ
         ELSE IF(KK5.EQ.-2) THEN
            A(5)=0.0D0
         ELSE IF(KK5.EQ.-3) THEN
            A(5)=2.0D0*DDF(L,3)*VOL0/(DZ*DZ)
         ENDIF
*
      ENDIF
      RETURN
      END
