*DECK TRITRK
      SUBROUTINE TRITRK (MAXPTS,IPTRK,IPGEOM,IMPX,IELEM,ICOL,ICHX,ISPLH,
     1 ISEG,IMPV,MAT,VOL,IDL)
*
*-----------------------------------------------------------------------
*
* RECOVERY OF THE GEOMETRY AND TRACKING FOR TRIVAC.
*
* INPUT PARAMETERS:
*  MAXPTS : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION NEL.
*  IPTRK  : L_TRIVAC POINTER TO THE TRACKING INFORMATION.
*  IPGEOM : L_GEOM POINTER TO THE GEOMETRY.
*  IMPX   : PRINT FLAG.
*  IELEM  : DEGREE OF THE LAGRANGIAN FINITE ELEMENTS.
*           =1: LINEAR FINITE ELEMENTS OR FINITE DIFFERENCES;
*           =2: PARABOLIC FINITE ELEMENTS;
*           =3: CUBIC FINITE ELEMENTS;
*           =4: QUARTIC FINITE ELEMENTS.
*  ICOL   : TYPE OF QUADRATURE USED TO INTEGRATE THE MASS MATRIX.
*           =1: ANALYTICAL INTEGRATION;
*           =2: GAUSS-LOBATTO QUADRATURE (COLLOCATION METHOD);
*           =3: GAUSS-LEGENDRE QUADRATURE (SUPERCONVERGENT).
*           IELEM=1 AND ICOL=2 ARE FINITE DIFFERENCE APPROXIMATIONS.
*  ICHX   : TYPE OF DISCRETIZATION METHOD.
*           =1: VARIATIONAL COLLOCATION METHOD (PRIMAL FINITE ELEMENTS
*               WITH GAUSS-LOBATTO QUADRATURE);
*           =2: DUAL FINITE ELEMENT APPROXIMATIONS;
*           =3: NODAL COLLOCATION METHOD WITH FULL TENSORIAL PRODUCTS
*               (DUAL FINITE ELEMENTS WITH GAUSS-LOBATTO QUADRATURE);
*           =4: NODAL COLLOCATION METHOD WITH SERENDIPITY APPROXIMATION.
*  ISPLH  : TYPE OF HEXAGONAL MESH-SPLITTING.
*           =1 : NO MESH SPLITTING (COMPLETE HEXAGONS);
*           =K : 6*(K-1)*(K-1) TRIANGLES PER HEXAGON.
*  ISEG   : NUMBER OF ELEMENTS IN A VECTOR REGISTER. EQUAL TO ZERO FOR
*           OPERATIONS IN SCALAR MODE.
*  IMPV   : PRINT PARAMETER FOR SUPERVECTORIAL OPERATIONS.
*
* SCRATCH STORAGE:
*  MAT    : DIMENSION MAT(MAXPTS)
*  VOL    : DIMENSION VOL(MAXPTS)
*  IDL    : DIMENSION IDL(MAXPTS)
*
*---------------------------------------------- A. HEBERT 04/06/91 -----
*
      LOGICAL ILK,CYLIND,CHEX
      CHARACTER HSMG*131
      DIMENSION MAT(*),VOL(*),IDL(*),ISTATE(13),IBASE(1),IGP(4),ITP(19),
     1 NCODE(6),ZCODE(6)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
******************* TRIVAC GEOMETRICAL STRUCTURE. **********************
*                                                                      *
*   ITYPE       : =2  : CARTESIAN 1-D GEOMETRY;                        *
*                 =3  : TUBE 1-D GEOMETRY;                             *
*                 =5  : CARTESIAN 2-D GEOMETRY;                        *
*                 =6  : TUBE 2-D GEOMETRY;                             *
*                 =7  : CARTESIAN 3-D GEOMETRY;                        *
*                 =8  : HEXAGONAL 2-D GEOMETRY;                        *
*                 =9  : HEXAGONAL 3-D GEOMETRY.                        *
*   IHEX        : TYPE OF HEXAGONAL SYMMETRY.                          *
*   IDIAG       : =0 NO DIAGONAL SYMMETRY; =1 DIAGONAL SYMMETRY.       *
*   IELEM       : DEGREE OF THE LAGRANGIAN FINITE ELEMENTS.            *
*                 =1: LINEAR FINITE ELEMENTS OR FINITE DIFFERENCES;    *
*                 =2: PARABOLIC FINITE ELEMENTS;                       *
*                 =3: CUBIC FINITE ELEMENTS;                           *
*                 =4: QUARTIC FINITE ELEMENTS.                         *
*   ICOL        : TYPE OF QUADRATURE USED TO INTEGRATE THE MASS MATRIX.*
*                 =1: ANALYTICAL INTEGRATION;                          *
*                 =2: GAUSS-LOBATTO QUADRATURE (COLLOCATION METHOD);   *
*                 =3: GAUSS-LEGENDRE QUADRATURE (SUPERCONVERGENT).     *
*                 IELEM=1 AND ICOL=2 ARE FINITE DIFFERENCE APPROX.     *
*   ICHX        : TYPE OF DISCRETIZATION METHOD.                       *
*                 =1: VARIATIONAL COLLOCATION METHOD (PRIMAL FINITE    *
*                     ELEMENTS WITH GAUSS-LOBATTO QUADRATURE);         *
*                 =2: DUAL FINITE ELEMENT APPROXIMATIONS;              *
*                 =3: NODAL COLLOCATION METHOD WITH FULL TENSORIAL     *
*                     PRODUCTS (DUAL FINITE ELEMENTS WITH GAUSS-       *
*                     LOBATTO QUADRATURE);                             *
*                 =4: NODAL COLLOCATION METHOD WITH SERENDIPITY APPRO- *
*                     XIMATION.                                        *
*   ISPLH       : TYPE OF HEXAGONAL MESH-SPLITTING.                    *
*                 =1 : NO MESH SPLITTING (COMPLETE HEXAGONS);          *
*                 =K : 6*(K-1)*(K-1) TRIANGLES PER HEXAGON.            *
*   SIDE        : SIDE OF THE HEXAGONS.                                *
*   LL4         : ORDER OF THE MATRICES PER GROUP IN TRIVAC.           *
*   NCODE       : TYPES OF BOUNDARY CONDITIONS. DIMENSION=6            *
*   ZCODE       : ALBEDOS. DIMENSION=6                                 *
*   LX,LY,LZ    : NUMBER OF ELEMENTS ALONG THE X, Y AND Z AXIS.        *
*   BASE(IXX)   : X-DIRECTED MESH SPACINGS. DIMENSION=LX*LY*LZ         *
*   BASE(IYY)   : Y-DIRECTED MESH SPACINGS. DIMENSION=LX*LY*LZ         *
*   BASE(IZZ)   : Z-DIRECTED MESH SPACINGS. DIMENSION=LX*LY*LZ         *
*   BASE(IDD)   : USED WITH CYLINDRICAL GEOMETRIES. DIMENSION=LX*LY*LZ *
*  IBASE(IKN)   : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION LX*LY*LZ*ICO *
*                 WHERE ICO IS THE NUMBER OF UNKNOWN PER ELEMENT.      *
*   BASE(IQFR)  : ELEMENT-ORDERED BOUNDARY CONDITIONS.                 *
*                 DIMENSION 6*LX*LY*LZ                                 *
*  IBASE(IMUW)  : INDICES USED WITH W-DIRECTED COMPRESSED DIAGONAL     *
*                 STORAGE MODE MATRICES. DIMENSION LL4                 *
*  IBASE(IMUX)  : INDICES USED WITH X-DIRECTED COMPRESSED DIAGONAL     *
*                 STORAGE MODE MATRICES. DIMENSION LL4                 *
*  IBASE(IMUY)  : INDICES USED WITH Y-DIRECTED COMPRESSED DIAGONAL     *
*                 STORAGE MODE MATRICES. DIMENSION LL4                 *
*  IBASE(IMUZ)  : INDICES USED WITH Z-DIRECTED COMPRESSED DIAGONAL     *
*                 STORAGE MODE MATRICES. DIMENSION LL4                 *
*  IBASE(IIPW)  : W-DIRECTED PERMUTATION MATRIX. DIMENSION LL4         *
*  IBASE(IIPX)  : X-DIRECTED PERMUTATION MATRIX. DIMENSION LL4         *
*  IBASE(IIPY)  : Y-DIRECTED PERMUTATION MATRIX. DIMENSION LL4         *
*  IBASE(IIPZ)  : Z-DIRECTED PERMUTATION MATRIX. DIMENSION LL4         *
*                                                                      *
* SUPERVECTORIAL OPERATION INFORMATION:                                *
*   ISEG        : NUMBER OF ELEMENTS IN A VECTOR REGISTER. EQUAL TO    *
*                 ZERO FOR OPERATIONS IN SCALAR MODE.                  *
*   IMPV        : PRINT PARAMETER FOR SUPERVECTORIAL OPERATIONS.       *
*   LTSW        : MAXIMUM BANDWIDTH. =2 FOR TRIDIAGONAL SYSTEMS.       *
*   LONW        : NUMBER OF GROUPS OF LINEAR SYSTEMS FOR W-MATRICES.   *
*   LONX        : NUMBER OF GROUPS OF LINEAR SYSTEMS FOR X-MATRICES.   *
*   LONY        : NUMBER OF GROUPS OF LINEAR SYSTEMS FOR Y-MATRICES.   *
*   LONZ        : NUMBER OF GROUPS OF LINEAR SYSTEMS FOR Z-MATRICES.   *
*  IBASE(INBLW) : NUMBER OF LINEAR SYSTEMS PER W-GROUP. DIMENSION LONW *
*  IBASE(INBLX) : NUMBER OF LINEAR SYSTEMS PER X-GROUP. DIMENSION LONX *
*  IBASE(INBLY) : NUMBER OF LINEAR SYSTEMS PER Y-GROUP. DIMENSION LONY *
*  IBASE(INBLZ) : NUMBER OF LINEAR SYSTEMS PER Z-GROUP. DIMENSION LONZ *
*  IBASE(ILBLW) : NUMBER OF UNKNOWNS PER W-GROUP. DIMENSION LONW       *
*  IBASE(ILBLX) : NUMBER OF UNKNOWNS PER X-GROUP. DIMENSION LONX       *
*  IBASE(ILBLY) : NUMBER OF UNKNOWNS PER Y-GROUP. DIMENSION LONY       *
*  IBASE(ILBLZ) : NUMBER OF UNKNOWNS PER Z-GROUP. DIMENSION LONZ       *
*  IBASE(IMUVW) : INDICES USED WITH W-DIRECTED COMPRESSED DIAGONAL     *
*                 STORAGE MODE MATRICES IN VECTOR MODE. DIMENSION LL4  *
*  IBASE(IMUVX) : INDICES USED WITH X-DIRECTED COMPRESSED DIAGONAL     *
*                 STORAGE MODE MATRICES IN VECTOR MODE. DIMENSION LL4  *
*  IBASE(IMUVY) : INDICES USED WITH Y-DIRECTED COMPRESSED DIAGONAL     *
*                 STORAGE MODE MATRICES IN VECTOR MODE. DIMENSION LL4  *
*  IBASE(IMUVZ) : INDICES USED WITH Z-DIRECTED COMPRESSED DIAGONAL     *
*                 STORAGE MODE MATRICES IN VECTOR MODE. DIMENSION LL4  *
*  IBASE(IIPVW) : W-DIRECTED VECTOR PERMUTATION MATRIX. DIMENSION LL4  *
*  IBASE(IIPVX) : X-DIRECTED VECTOR PERMUTATION MATRIX. DIMENSION LL4  *
*  IBASE(IIPVY) : Y-DIRECTED VECTOR PERMUTATION MATRIX. DIMENSION LL4  *
*  IBASE(IIPVZ) : Z-DIRECTED VECTOR PERMUTATION MATRIX. DIMENSION LL4  *
*                                                                      *
* INFORMATION RELATED TO CYLINDRICAL CORRECTIONS IN CARTESIAN GEOMETRY *
*   NR0         : NUMBER OF RADII.                                     *
*   BASE(IRR0)  : RADII. DIMENSION NR0                                 *
*   BASE(IXR0)  : COORDINATES ON PRINCIPAL AXIS. DIMENSION NR0         *
*   BASE(IANG)  : ANGLES FOR APPLYING CIRCULAR CORRECTION.             *
*                 DIMENSION NR0                                        *
*                                                                      *
************************************************************************
*
      CALL LCMGET(IPGEOM,'STATE',ISTATE)
      ITYPE=ISTATE(1)
*
      IF(IMPX.GE.1) WRITE (6,'(/35H TRITRK: DEGREE OF FINITE ELEMENT I,
     1 6HELEM =,I3/9X,25HTYPE OF QUADRATURE ICOL =,I3/9X,10HTYPE OF DI,
     2 19HSCRETIZATION ICHX =,I3/)') IELEM,ICOL,ICHX
      IF((IMPX.GE.1).AND.(ISEG.GT.0)) WRITE (6,'(18H TRITRK: SUPERVECT,
     1 27HORIZATION OPTION ON. ISEG =,I4,8H  IMPV =,I3/)') ISEG,IMPV
      IF(ISTATE(9).EQ.0) THEN
         IF((ITYPE.NE.1).AND.(ITYPE.NE.2).AND.(ITYPE.NE.3).AND.
     1      (ITYPE.NE.5).AND.(ITYPE.NE.6).AND.(ITYPE.NE.7).AND.
     2      (ITYPE.NE.8).AND.(ITYPE.NE.9)) THEN
            CALL XABORT('TRITRK: DISCRETIZATION NOT AVAILABLE.')
         ENDIF
         CALL SETARA(BASE,MAXPTS+1,IXXX)
         CALL SETARA(BASE,MAXPTS+1,IYYY)
         CALL SETARA(BASE,MAXPTS+1,IZZZ)
*
         CALL SETARA(BASE,MAXPTS,IISPLX)
         CALL SETARA(BASE,MAXPTS,IISPLY)
         CALL SETARA(BASE,MAXPTS,IISPLZ)
         CALL READGM(MAXPTS,MAXPTS,MAXPTS,MAXPTS,IPGEOM,IHEX,IR,MAT,
     1   ILK,SIDE,BASE(IXXX),BASE(IYYY),BASE(IZZZ),IMPX,LX,LY,LZ,NEL,
     2   NCODE,ZCODE,BASE(IISPLX),BASE(IISPLY),BASE(IISPLZ))
         CALL RLSARA(BASE(IISPLZ))
         CALL RLSARA(BASE(IISPLY))
         CALL RLSARA(BASE(IISPLX))
      ELSE
         CALL XABORT('TRITRK: DISCRETIZATION NOT AVAILABLE.')
      ENDIF
*
* UNFOLD HEXAGONAL GEOMETRY CASES.
      CHEX=(ITYPE.EQ.8).OR.(ITYPE.EQ.9)
      IF(CHEX.AND.(IHEX.NE.9)) THEN
         CALL SETARA(IBASE,MAXPTS,IDP)
         CALL SETARA(IBASE,NEL,IMX)
         DO 30 I=1,NEL
   30    IBASE(IMX+I-1)=MAT(I)
         LXOLD=LX
         CALL TRIALL(IHEX,LXOLD,LX,IBASE(IDP))
         DO 40 KZ=1,LZ
         DO 40 KX=1,LX
         MAT(KX+(KZ-1)*LX)=0
         KEL=IBASE(IDP+KX-1)+(KZ-1)*LXOLD
   40    MAT(KX+(KZ-1)*LX)=IBASE(IMX+KEL-1)
         CALL RLSARA(IBASE(IMX))
         CALL RLSARA(IBASE(IDP))
         NEL=LX*LZ
      ENDIF
*
* PROCESS INFORMATION RELATED TO CYLINDRICAL CORRECTION IN CARTESIAN
* GEOMETRIES.
      NR0 = ISTATE(8)
      IF(NR0.GT.0) THEN
         IF((ITYPE.NE.5).AND.(ITYPE.NE.7)) CALL XABORT('TRITRK: CYLIND'
     1   //'RICAL CORRECTIONS ARE LIMITED TO CARTESIAN GEOMETRIES.')
         IF(IMPX.GT.0) WRITE(6,'(/33H TRITRK: PERFORM A CYLINDRICAL CO,
     2   35HRRECTION ON THE CARTESIAN BOUNDARY.)')
         CALL SETARA(BASE,NR0,IRR0)
         CALL LCMGET(IPGEOM,'RR0',BASE(IRR0))
         CALL SETARA(BASE,NR0,IXR0)
         CALL LCMGET(IPGEOM,'XR0',BASE(IXR0))
         CALL SETARA(BASE,NR0,IANG)
         CALL LCMGET(IPGEOM,'ANG',BASE(IANG))
         CALL LCMPOF(IPTRK,'RR0',NR0,2,IRR0)
         CALL LCMPOF(IPTRK,'XR0',NR0,2,IXR0)
         CALL LCMPOF(IPTRK,'ANG',NR0,2,IANG)
      ENDIF
*
      IF(LX*LY*LZ.GT.MAXPTS) THEN
         WRITE (HSMG,'(39HTRITRK: MAXPTS SHOULD BE INCREASED FROM,I7,
     1   3H TO,I7)') MAXPTS,LX*LY*LZ
         CALL XABORT(HSMG)
      ENDIF
*
* 1-D AND 2-D CASES.
      IDIM=1
      IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6).OR.(ITYPE.EQ.8)) IDIM=2
      IF((ITYPE.EQ.7).OR.(ITYPE.EQ.9)) IDIM=3
      IF((NCODE(3).EQ.0).AND.(NCODE(4).EQ.0).AND.(.NOT.CHEX)) THEN
         IF((IDIM.NE.1).OR.(LY.NE.1)) CALL XABORT('TRITRK: INVALID 1D '
     1   //'GEOMETRY.')
         NCODE(3)=2
         NCODE(4)=5
         ZCODE(3)=1.0
         ZCODE(4)=1.0
         BASE(IYYY)=0.0
         BASE(IYYY+1)=2.0
      ENDIF
      IF((NCODE(5).EQ.0).AND.(NCODE(6).EQ.0)) THEN
         IF((IDIM.EQ.3).OR.(LZ.NE.1)) CALL XABORT('TRITRK: INVALID 1D '
     1   //'OR 2D GEOMETRY.')
         NCODE(5)=2
         NCODE(6)=5
         ZCODE(5)=1.0
         ZCODE(6)=1.0
         BASE(IZZZ)=0.0
         BASE(IZZZ+1)=2.0
      ENDIF
*
* 2-D CYLINDRICAL CASES.
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      IF(ITYPE.EQ.6) THEN
         LY=LZ
         DO 45 I=1,LZ+1
   45    BASE(IYYY+I-1)=BASE(IZZZ+I-1)
         NCODE(3)=NCODE(5)
         NCODE(4)=NCODE(6)
         ZCODE(3)=ZCODE(5)
         ZCODE(4)=ZCODE(6)
         NCODE(5)=0
         NCODE(6)=0
         ZCODE(5)=0.0
         ZCODE(6)=0.0
      ENDIF
*
* UNFOLD THE DOMAIN IN DIAGONAL SYMMETRY CASES.
      IDIAG=0
      IF((NCODE(2).EQ.3).AND.(NCODE(3).EQ.3)) THEN
         IDIAG=1
         NCODE(3)=NCODE(1)
         NCODE(2)=NCODE(4)
         ZCODE(3)=ZCODE(1)
         ZCODE(2)=ZCODE(4)
         K=NEL
         DO 80 IZ=LZ,1,-1
         IOFF=(IZ-1)*LX*LY
         DO 80 IY=LY,1,-1
         DO 70 IX=LX,IY+1,-1
   70    MAT(IOFF+(IY-1)*LX+IX)=MAT(IOFF+(IX-1)*LY+IY)
         DO 80 IX=IY,1,-1
         MAT(IOFF+(IY-1)*LX+IX)=MAT(K)
   80    K=K-1
         NEL=LX*LY*LZ
         IF(K.NE.0) THEN
            CALL XABORT('TRITRK: UNABLE TO UNFOLD THE DOMAIN(1).')
         ENDIF
      ELSE IF((NCODE(1).EQ.3).AND.(NCODE(4).EQ.3)) THEN
         IDIAG=1
         NCODE(1)=NCODE(3)
         NCODE(4)=NCODE(2)
         ZCODE(1)=ZCODE(3)
         ZCODE(4)=ZCODE(2)
         K=NEL
         DO 90 IZ=LZ,1,-1
         IOFF=(IZ-1)*LX*LY
         DO 90 IY=LY,1,-1
         DO 90 IX=LX,IY,-1
         MAT(IOFF+(IY-1)*LX+IX)=MAT(K)
   90    K=K-1
         DO 100 IZ=1,LZ
         IOFF=(IZ-1)*LX*LY
         DO 100 IY=1,LY
         DO 100 IX=1,IY-1
  100    MAT(IOFF+(IY-1)*LX+IX)=MAT(IOFF+(IX-1)*LY+IY)
         NEL=LX*LY*LZ
         IF(K.NE.0) THEN
            CALL XABORT('TRITRK: UNABLE TO UNFOLD THE DOMAIN(2).')
         ENDIF
      ENDIF
      IF(IMPX.GT.5) THEN
         WRITE(6,600) 'NCODE',(NCODE(I),I=1,6)
         WRITE(6,600) 'MAT',(MAT(I),I=1,LX*LY)
      ENDIF
*
      MAXQF=6*NEL
      IF(CHEX) MAXQF=8*NEL
      IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
         MAXKN=NEL*(IELEM+1)**3
      ELSE IF((ICHX.EQ.2).AND.(.NOT.CHEX)) THEN
         MAXKN=7*NEL
      ELSE IF(((ICHX.EQ.3).OR.(ICHX.EQ.4)).AND.(.NOT.CHEX)) THEN
         MAXKN=6*NEL
      ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
         IF(ISPLH.EQ.1) THEN
            MAXKN=12*NEL
         ELSE
            MAXKN=2*(1+ISPLH*(ISPLH-1)*3)*NEL
         ENDIF
      ELSE IF(((ICHX.EQ.3).OR.(ICHX.EQ.4)).AND.CHEX) THEN
         IF(ISPLH.EQ.1) THEN
            MAXKN=8*NEL
         ELSE
            MAXKN=(18*(ISPLH-1)**2+8)*NEL
         ENDIF
      ELSE
         CALL XABORT('TRITRK: INVALID TYPE OF DISCRETIZATION.')
      ENDIF
      IF(CYLIND) THEN
         MAXDD=NEL
      ELSE
         MAXDD=1
      ENDIF
      IF(CHEX.AND.(IELEM.NE.1)) CALL XABORT('TRITRK: HEXAGONAL DISCRET'
     1 //'IZATIONS ARE LIMITED TO LINEAR ORDER.')
      IF(CHEX.AND.(ICHX.NE.1).AND.(ICHX.NE.3)) CALL XABORT('TRITRK: HE'
     1 //'XAGONAL DISCRETIZATIONS ARE LIMITED TO MESH-CENTERED OR MESH'
     2 //'-CORNER FINITE DIFFERENCES.')
      CALL SETARA(BASE,NEL,IXX)
      CALL SETARA(BASE,NEL,IYY)
      CALL SETARA(BASE,NEL,IZZ)
      CALL SETARA(BASE,MAXDD,IDD)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      DO 105 I=1,MAXKN
  105 IBASE(IKN+I-1)=0
      DO 106 I=1,MAXQF
  106 BASE(IQFR+I-1)=0.0
      IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
         MAXIP=(1+IELEM*LX)*(1+IELEM*LY)*(1+IELEM*LZ)
         CALL SETARA(BASE,MAXIP,IIP)
         CALL SETARA(BASE,MAXIP,IIWRK)
         CALL TRIPKN(IELEM,LX,LY,LZ,LL4,CYLIND,BASE(IXXX),BASE(IYYY),
     1   BASE(IZZZ),BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),IBASE(IKN),
     2   BASE(IQFR),VOL,MAT,NCODE,ZCODE,IMPX,BASE(IIP),BASE(IIWRK))
         CALL RLSARA(BASE(IIWRK))
         CALL RLSARA(BASE(IIP))
         IF((IMPX.GT.0).AND.(IELEM.EQ.1)) THEN
            WRITE (6,'(/40H TRITRK: MESH CORNER FINITE DIFFERENCES.)')
         ENDIF
      ELSE IF((ICHX.EQ.2).AND.(.NOT.CHEX)) THEN
         MAXIP=(LX+1)*LY*LZ*IELEM*IELEM+LX*(LY+1)*LZ*IELEM*IELEM
     1   +LX*LY*(LZ+1)*IELEM*IELEM+LX*LY*LZ*IELEM*IELEM*IELEM
         CALL SETARA(BASE,MAXIP,IIP)
         CALL TRIDKN(IMPX,LX,LY,LZ,CYLIND,IELEM,ICOL,LL4,NCODE,ZCODE,
     1   MAT,VOL,BASE(IXXX),BASE(IYYY),BASE(IZZZ),BASE(IXX),BASE(IYY),
     2   BASE(IZZ),BASE(IDD),IBASE(IKN),BASE(IQFR),IDL,BASE(IIP))
         CALL RLSARA(BASE(IIP))
         MAXIP=LX*LY*LZ
         NUN=LL4
      ELSE IF(((ICHX.EQ.3).OR.(ICHX.EQ.4)).AND.(.NOT.CHEX)) THEN
         MAXIP=LX*LY*LZ
         CALL TRIDFC(IMPX,IDIM,LX,LY,LZ,CYLIND,NCODE,ZCODE,MAT,
     1   BASE(IXXX),BASE(IYYY),BASE(IZZZ),LL0,VOL,BASE(IXX),BASE(IYY),
     2   BASE(IZZ),BASE(IDD),IBASE(IKN),BASE(IQFR))
         IF(IELEM.EQ.1) THEN
            LL4=LL0
            IF(IMPX.GT.0) WRITE (6,'(/29H TRITRK: MESH CENTERED FINITE,
     1      13H DIFFERENCES.)')
         ELSE IF((IELEM.GT.1).AND.(ICHX.EQ.3)) THEN
            LL4=LL0*IELEM**IDIM
            IF(IMPX.GT.0) WRITE (6,'(/29H TRITRK: NODAL COLLOCATION ME,
     1      13HTHOD OF ORDER,I3,1H.)') IELEM
         ELSE IF(ICHX.EQ.4) THEN
            IF(IDIM.EQ.1) LL4=LL0*IELEM
            IF(IDIM.EQ.2) LL4=LL0*(IELEM*(IELEM+1))/2
            IF(IDIM.EQ.3) LL4=LL0*(IELEM*(IELEM+1)*(IELEM+2))/6
            IF(IMPX.GT.0) WRITE (6,'(/29H TRITRK: NODAL COLLOCATION ME,
     1      44HTHOD WITH SERENDIPITY APPROXIMATION OF ORDER,I3,1H.)')
     2      IELEM
         ENDIF
*        COMPUTE INDICES IDL.
         IF((ICHX.EQ.3).OR.(ICHX.EQ.4)) THEN
*           NODAL COLLOCATION METHOD.
            NUN=0
            DO 110 K=1,NEL
            IDL(K)=0
            IF(MAT(K).EQ.0) GO TO 110
            NUN=NUN+1
            IDL(K)=1+IELEM*(NUN-1)
  110       CONTINUE
            NUN=LL4
         ENDIF
      ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
         MAXIP=1
         IF(IELEM.NE.1) CALL XABORT('TRITRK: INVALID DISCRETIZATION.')
         CALL TRIPRH(ISPLH,IPTRK,LX,LZ,LL4,SIDE,BASE(IZZZ),BASE(IZZ),
     1   IBASE(IKN),BASE(IQFR),VOL,MAT,NCODE,ZCODE,IMPX)
         IF(IMPX.GT.0) WRITE (6,'(/32H TRITRK: MESH CORNER FINITE DIFF,
     1   39HERENCES FOR HEXAGONAL GEOMETRY. ISPLH =,I3,1H.)') ISPLH
      ELSE IF(((ICHX.EQ.3).OR.(ICHX.EQ.4)).AND.CHEX) THEN
         MAXIP=LX*LZ
         CALL TRIDFH(ISPLH,IPTRK,IDIM,LX,LZ,LL4,NUN,SIDE,BASE(IZZZ),
     1   BASE(IZZ),IBASE(IKN),BASE(IQFR),VOL,MAT,IDL,NCODE,ZCODE,IMPX)
         IF(IMPX.GT.0) WRITE (6,'(/32H TRITRK: MESH CENTERED FINITE DI,
     1   41HFFERENCES FOR HEXAGONAL GEOMETRY. ISPLH =,I3,1H.)') ISPLH
      ENDIF
      IF(ICHX.EQ.1) THEN
*        COMPUTE INDICES IDL FOR PRIMAL FINITE ELEMENTS.
         NUN=LL4
         DO 130 K=1,NEL
         IF(MAT(K).EQ.0) THEN
            IDL(K)=0
         ELSE
            NUN=NUN+1
            IDL(K)=NUN
         ENDIF
  130    CONTINUE
      ENDIF
      IF(IMPX.GT.0) WRITE (6,'(/34H TRITRK: ORDER OF LINEAR SYSTEMS =,
     1 I7/9X,37HNUMBER OF UNKNOWNS PER ENERGY GROUP =,I7)') LL4,NUN
      CALL RLSARA(BASE(IZZZ))
      CALL RLSARA(BASE(IYYY))
      CALL RLSARA(BASE(IXXX))
*      IF(IMPX.GE.2) WRITE(6,'(/37H TRITRK: CPU TIME FOR FINITE ELEMENT
*     1 11HNUMBERING =,F7.2,2H S)') 0.0
*
* COMPUTE INDICES MUW, MUX, MUY, MUZ, IPW, IPX, IPY AND IPZ.
      IF(CHEX) CALL SETARA(IBASE,LL4,IMUW)
      CALL SETARA(IBASE,LL4,IMUX)
      CALL SETARA(IBASE,LL4,IMUY)
      CALL SETARA(IBASE,LL4,IMUZ)
      IF(CHEX) CALL SETARA(IBASE,LL4,IIPW)
      CALL SETARA(IBASE,LL4,IIPX)
      CALL SETARA(IBASE,LL4,IIPY)
      CALL SETARA(IBASE,LL4,IIPZ)
      DO 140 I=1,LL4
  140 IBASE(IIPX+I-1)=I
*
      CALL SETARA(BASE,MAXIP,IIWRK)
      IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
         CALL BIVCOL(IPTRK,IMPX,IELEM,2)
         CALL TRICHP(IELEM,LX,LY,LZ,LL4,MAT,IBASE(IKN),IBASE(IMUX),
     1   IBASE(IMUY),IBASE(IMUZ),IBASE(IIPY),IBASE(IIPZ),BASE(IIWRK),
     2   IMPX)
      ELSE IF((ICHX.EQ.2).AND.(.NOT.CHEX)) THEN
         CALL BIVCOL(IPTRK,IMPX,IELEM,ICOL)
         CALL TRICHD(IMPX,LX,LY,LZ,IELEM,ICOL,LL4,MAT,IBASE(IKN),
     1   IBASE(IMUX),IBASE(IMUY),IBASE(IMUZ),IBASE(IIPX),IBASE(IIPY),
     2   IBASE(IIPZ),BASE(IIWRK))
      ELSE IF((ICHX.EQ.3).AND.(.NOT.CHEX)) THEN
         CALL TRICH1(IELEM,IDIM,LX,LY,LZ,LL4,MAT,IBASE(IKN),IBASE(IMUX),
     1   IBASE(IMUY),IBASE(IMUZ),IBASE(IIPY),IBASE(IIPZ),BASE(IIWRK),
     2   IMPX)
      ELSE IF((ICHX.EQ.4).AND.(.NOT.CHEX)) THEN
         CALL TRICH2(IELEM,IDIM,LX,LY,LZ,LL4,MAT,IBASE(IKN),IBASE(IMUX),
     1   IBASE(IMUY),IBASE(IMUZ),IBASE(IIPY),IBASE(IIPZ),BASE(IIWRK),
     2   IMPX)
      ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
         CALL BIVCOL(IPTRK,IMPX,IELEM,2)
         CALL TRICH3(ISPLH,IPTRK,LX,LZ,LL4,MAT,IBASE(IKN),IBASE(IMUW),
     1   IBASE(IMUX),IBASE(IMUY),IBASE(IMUZ),IBASE(IIPW),IBASE(IIPX),
     2   IBASE(IIPY),IBASE(IIPZ),IMPX)
      ELSE IF(((ICHX.EQ.3).OR.(ICHX.EQ.4)).AND.CHEX) THEN
         CALL TRICH4(ISPLH,IPTRK,IDIM,LX,LZ,LL4,MAT,IBASE(IKN),
     1   IBASE(IMUW),IBASE(IMUX),IBASE(IMUY),IBASE(IMUZ),IBASE(IIPW),
     2   IBASE(IIPX),IBASE(IIPY),IBASE(IIPZ),BASE(IIWRK),IMPX)
      ENDIF
      CALL RLSARA(BASE(IIWRK))
*      IF(IMPX.GE.2) WRITE(6,'(/36H TRITRK: CPU TIME FOR ADI SPLITTING ,
*     1 11HNUMBERING =,F7.2,2H S)') 0.0
      IF(IMPX.GT.5) THEN
         I1=1
         DO 150 I=1,(NEL-1)/8+1
         I2=I1+7
         IF(I2.GT.NEL) I2=NEL
         WRITE (6,620) (J,J=I1,I2)
         WRITE (6,630) (MAT(J),J=I1,I2)
         WRITE (6,640) (IDL(J),J=I1,I2)
         WRITE (6,650) (VOL(J),J=I1,I2)
  150    I1=I1+8
      ENDIF
*
* SUPERVECTORIZATION CONTROL.
      IF(ISEG.GT.0) THEN
         CALL SETARA(IBASE,LL4,ISET)
         LTSW=0
         IF(CHEX) THEN
            IBASE(ISET)=0
            K1=IBASE(IMUW)+1
            DO 160 I=2,LL4
            IBASE(ISET+I-1)=0
            K2=IBASE(IMUW+I-1)
            DO 155 J=I-K2+K1,I-1
  155       IBASE(ISET+J-1)=1
  160       K1=K2+1
            NSYS=0
            DO 165 I=1,LL4
            IF(IBASE(ISET+I-1).EQ.0) NSYS=NSYS+1
  165       CONTINUE
            LONW=1+(NSYS-1)/ISEG
            CALL SETARA(IBASE,LONW,INBLW)
            CALL SETARA(IBASE,LONW,ILBLW)
            CALL SETARA(IBASE,LL4,IMUVW)
            CALL SETARA(IBASE,LL4,IIPVW)
            CALL VECPER('W',IMPV,ISEG,LL4,IBASE(IMUW),LONW,LTSW2,
     1      IBASE(INBLW),IBASE(ILBLW),IBASE(IMUVW),IBASE(IIPVW))
            IMU=0
            DO 166 I=1,LONW
  166       IMU=IMU+IBASE(ILBLW+I-1)
            LTSW=MAX(LTSW,LTSW2)
            CALL LCMPOF(IPTRK,'NBLW',LONW,1,INBLW)
            CALL LCMPOF(IPTRK,'LBLW',LONW,1,ILBLW)
            CALL LCMPOF(IPTRK,'MUVW',IMU,1,IMUVW)
            CALL LCMPOF(IPTRK,'IPVW',LL4,1,IIPVW)
            IMU=IMU*ISEG
            CALL LCMPUT(IPTRK,'LL4VW',1,1,IMU)
         ENDIF
         IF(IDIAG.EQ.0) THEN
            IBASE(ISET)=0
            K1=IBASE(IMUX)+1
            DO 175 I=2,LL4
            IBASE(ISET+I-1)=0
            K2=IBASE(IMUX+I-1)
            DO 170 J=I-K2+K1,I-1
  170       IBASE(ISET+J-1)=1
  175       K1=K2+1
            NSYS=0
            DO 180 I=1,LL4
            IF(IBASE(ISET+I-1).EQ.0) NSYS=NSYS+1
  180       CONTINUE
            LONX=1+(NSYS-1)/ISEG
            CALL SETARA(IBASE,LONX,INBLX)
            CALL SETARA(IBASE,LONX,ILBLX)
            CALL SETARA(IBASE,LL4,IMUVX)
            CALL SETARA(IBASE,LL4,IIPVX)
            CALL VECPER('X',IMPV,ISEG,LL4,IBASE(IMUX),LONX,LTSW2,
     1      IBASE(INBLX),IBASE(ILBLX),IBASE(IMUVX),IBASE(IIPVX))
            IMU=0
            DO 185 I=1,LONX
  185       IMU=IMU+IBASE(ILBLX+I-1)
            LTSW=MAX(LTSW,LTSW2)
            CALL LCMPOF(IPTRK,'NBLX',LONX,1,INBLX)
            CALL LCMPOF(IPTRK,'LBLX',LONX,1,ILBLX)
            CALL LCMPOF(IPTRK,'MUVX',IMU,1,IMUVX)
            CALL LCMPOF(IPTRK,'IPVX',LL4,1,IIPVX)
            IMU=IMU*ISEG
            CALL LCMPUT(IPTRK,'LL4VX',1,1,IMU)
         ENDIF
         IF(IDIM.GE.2) THEN
            IBASE(ISET)=0
            K1=IBASE(IMUY)+1
            DO 200 I=2,LL4
            IBASE(ISET+I-1)=0
            K2=IBASE(IMUY+I-1)
            DO 190 J=I-K2+K1,I-1
  190       IBASE(ISET+J-1)=1
  200       K1=K2+1
            NSYS=0
            DO 210 I=1,LL4
            IF(IBASE(ISET+I-1).EQ.0) NSYS=NSYS+1
  210       CONTINUE
            LONY=1+(NSYS-1)/ISEG
            CALL SETARA(IBASE,LONY,INBLY)
            CALL SETARA(IBASE,LONY,ILBLY)
            CALL SETARA(IBASE,LL4,IMUVY)
            CALL SETARA(IBASE,LL4,IIPVY)
            CALL VECPER('Y',IMPV,ISEG,LL4,IBASE(IMUY),LONY,LTSW2,
     1      IBASE(INBLY),IBASE(ILBLY),IBASE(IMUVY),IBASE(IIPVY))
            IMU=0
            DO 215 I=1,LONY
  215       IMU=IMU+IBASE(ILBLY+I-1)
            LTSW=MAX(LTSW,LTSW2)
            CALL LCMPOF(IPTRK,'NBLY',LONY,1,INBLY)
            CALL LCMPOF(IPTRK,'LBLY',LONY,1,ILBLY)
            CALL LCMPOF(IPTRK,'MUVY',IMU,1,IMUVY)
            CALL LCMPOF(IPTRK,'IPVY',LL4,1,IIPVY)
            IMU=IMU*ISEG
            CALL LCMPUT(IPTRK,'LL4VY',1,1,IMU)
         ENDIF
         IF(IDIM.EQ.3) THEN
            IBASE(ISET)=0
            K1=IBASE(IMUZ)+1
            DO 230 I=2,LL4
            IBASE(ISET+I-1)=0
            K2=IBASE(IMUZ+I-1)
            DO 220 J=I-K2+K1,I-1
  220       IBASE(ISET+J-1)=1
  230       K1=K2+1
            NSYS=0
            DO 240 I=1,LL4
            IF(IBASE(ISET+I-1).EQ.0) NSYS=NSYS+1
  240       CONTINUE
            LONZ=1+(NSYS-1)/ISEG
            CALL SETARA(IBASE,LONZ,INBLZ)
            CALL SETARA(IBASE,LONZ,ILBLZ)
            CALL SETARA(IBASE,LL4,IMUVZ)
            CALL SETARA(IBASE,LL4,IIPVZ)
            CALL VECPER('Z',IMPV,ISEG,LL4,IBASE(IMUZ),LONZ,LTSW2,
     1      IBASE(INBLZ),IBASE(ILBLZ),IBASE(IMUVZ),IBASE(IIPVZ))
            IMU=0
            DO 250 I=1,LONZ
  250       IMU=IMU+IBASE(ILBLZ+I-1)
            LTSW=MAX(LTSW,LTSW2)
            CALL LCMPOF(IPTRK,'NBLZ',LONZ,1,INBLZ)
            CALL LCMPOF(IPTRK,'LBLZ',LONZ,1,ILBLZ)
            CALL LCMPOF(IPTRK,'MUVZ',IMU,1,IMUVZ)
            CALL LCMPOF(IPTRK,'IPVZ',LL4,1,IIPVZ)
            IMU=IMU*ISEG
            CALL LCMPUT(IPTRK,'LL4VZ',1,1,IMU)
         ENDIF
         CALL RLSARA(IBASE(ISET))
      ELSE
        IMPV = 0
        LTSW = 0
        LONW = 0
        LONX = 0
        LONY = 0
        LONZ = 0
      ENDIF
*
* SAVE GENERAL TRACKING INFORMATION.
      IGP(1)=NEL
      IGP(2)=NUN
      IF(ILK) THEN
         IGP(3)=0
      ELSE
         IGP(3)=1
      ENDIF
      IGP(4)=ISTATE(7)
      CALL LCMPUT(IPTRK,'GPARAM',4,1,IGP)
      CALL LCMPUT(IPTRK,'MATCOD',NEL,1,MAT)
      CALL LCMPUT(IPTRK,'VOLUME',NEL,2,VOL)
      CALL LCMPUT(IPTRK,'KEYFLX',NEL,1,IDL)
*
* SAVE TRIVAC SPECIFIC TRACKING INFORMATION.
      ITP(1)=ITYPE
      ITP(2)=IHEX
      ITP(3)=IDIAG
      ITP(4)=IELEM
      ITP(5)=ICOL
      ITP(6)=ICHX
      ITP(7)=ISPLH
      ITP(8)=LL4
      ITP(9)=LX
      ITP(10)=LY
      ITP(11)=LZ
      ITP(12)=ISEG
      ITP(13)=IMPV
      ITP(14)=LTSW
      ITP(15)=LONW
      ITP(16)=LONX
      ITP(17)=LONY
      ITP(18)=LONZ
      ITP(19)=NR0
      CALL LCMPUT(IPTRK,'TPARAM',19,1,ITP)
      CALL LCMPUT(IPTRK,'NCODE',6,1,NCODE)
      CALL LCMPUT(IPTRK,'ZCODE',6,2,ZCODE)
      CALL LCMPOF(IPTRK,'ZZ',NEL,2,IZZ)
      CALL LCMPOF(IPTRK,'KN',MAXKN,1,IKN)
      CALL LCMPOF(IPTRK,'QFR',MAXQF,2,IQFR)
      CALL LCMPOF(IPTRK,'IPX',LL4,1,IIPX)
      IF(CHEX) THEN
         CALL LCMPUT(IPTRK,'SIDE',1,2,SIDE)
         CALL LCMPOF(IPTRK,'MUW',LL4,1,IMUW)
         CALL LCMPOF(IPTRK,'IPW',LL4,1,IIPW)
         CALL RLSARA(BASE(IXX))
         CALL RLSARA(BASE(IYY))
         CALL RLSARA(BASE(IDD))
      ELSE
         CALL LCMPOF(IPTRK,'XX',NEL,2,IXX)
         CALL LCMPOF(IPTRK,'YY',NEL,2,IYY)
         IF(.NOT.CYLIND) BASE(IDD)=0.0
         CALL LCMPOF(IPTRK,'DD',MAXDD,2,IDD)
      ENDIF
      IF(IDIAG.EQ.0) THEN
         CALL LCMPOF(IPTRK,'MUX',LL4,1,IMUX)
      ELSE
         CALL RLSARA(IBASE(IMUX))
      ENDIF
      IF(IDIM.GE.2) THEN
         CALL LCMPOF(IPTRK,'MUY',LL4,1,IMUY)
         CALL LCMPOF(IPTRK,'IPY',LL4,1,IIPY)
      ELSE
         CALL RLSARA(IBASE(IIPY))
         CALL RLSARA(IBASE(IMUY))
      ENDIF
      IF(IDIM.EQ.3) THEN
         CALL LCMPOF(IPTRK,'MUZ',LL4,1,IMUZ)
         CALL LCMPOF(IPTRK,'IPZ',LL4,1,IIPZ)
      ELSE
         CALL RLSARA(IBASE(IIPZ))
         CALL RLSARA(IBASE(IMUZ))
      ENDIF
      RETURN
*
  600 FORMAT(/26H TRITRK: VALUES OF VECTOR ,A6,4H ARE/(1X,1P,20I6))
  620 FORMAT (///11H REGION    ,8(I8,6X,1HI))
  630 FORMAT (   11H MIXTURE   ,8(I8,6X,1HI))
  640 FORMAT (   11H POINTER   ,8(I8,6X,1HI))
  650 FORMAT (   11H VOLUME    ,8(1P,E13.6,2H I))
      END
