*DECK VECBLD
      SUBROUTINE VECBLD(ISEG,L4,MUIN,LON,NBL,LBL,MUV,IPV,ITY,ASSIN,
     1 ASSV,DGV)
*
*-----------------------------------------------------------------------
*
* REBUILD A MATRIX STORED IN COMPRESSED DIAGONAL STORAGE MODE IN A FORM
* COMPATIBLE WITH SUPERVECTORIAL CALCULATIONS.
*
* INPUT PARAMETERS:
*    ISEG : NUMBER OF ELEMENTS IN A VECTOR REGISTER.
*      L4 : ASSIN MATRIX ORDER.
*    MUIN : POSITION OF EACH DIAGONAL ELEMENT IN MATRIX ASSIN.
*           DIMENSION MUIN(L4)
*     LON : NUMBER OF GROUPS OF LINEAR SYSTEMS.
*     NBL : NUMBER OF LINEAR SYSTEMS IN EACH GROUP. DIMENSION NBL(LON)
*     LBL : NUMBER OF UNKNOWNS IN EACH GROUP. DIMENSION LBL(LON)
*     MUV : POSITION OF EACH DIAGONAL ELEMENT IN MATRIX ASSV.
*     IPV : PERMUTATION VECTOR FOR THE ORDERED UNKNOWNS.
*           DIMENSION MUV(L4),IPV(L4)
*     ITY : =1 GATHER BACK ; =2 SCATTER FORTH
*
* INPUT/OUTPUT PARAMETERS:
*   ASSIN : INPUT (ITY=2) OR OUTPUT (ITY=1) MATRIX IN SCALAR COMPRESSED
*           DIAGONAL STORAGE MODE. DIMENSION ASSIN(MUIN(L4))
*    ASSV : INPUT (ITY=1) OR OUTPUT (ITY=2) MATRIX IN SUPERVECTORIAL
*           COMPRESSED DIAGONAL STORAGE MODE.
*           DIMENSION ASSV(ISEG,MUV(SUM(LBL(I))))
*
* OUTPUT PARAMETER (IF ITY=2):
*     DGV : DIAGONAL OF ASSV. DIMENSION DGV(ISEG,SUM(LBL(I)))
*
*-------------------------------------- AUTHOR: A. HEBERT ; 14/10/92 ---
*
      DIMENSION MUIN(*),NBL(*),LBL(*),MUV(*),IPV(*),ASSIN(*),
     1 ASSV(ISEG,*),DGV(ISEG,*)
*
* REBUILD THE MATRIX.
      IF(ITY.EQ.1) THEN
         IOF0=0
         DO 20 IND1=1,L4
         IPOS=1+(IPV(IND1)-1)/ISEG
         IBANC=1+MOD(IPV(IND1)-1,ISEG)
         IOF1=MUIN(IND1)
         DO 10 JND1=1-IOF1+IOF0,0
   10    ASSIN(IOF1+JND1)=ASSV(IBANC,MUV(IPOS)+JND1)
   20    IOF0=IOF1
      ELSE IF(ITY.EQ.2) THEN
         LBL0=0
         DO 30 I=1,LON
   30    LBL0=LBL0+LBL(I)
         DO 40 J=1,MUV(LBL0)
         DO 40 I=1,ISEG
   40    ASSV(I,J)=0.0
         LBL0=0
         DO 60 J=1,LON
         DO 50 K=1,LBL(J)
         DO 50 I=1,ISEG
         ASSV(I,MUV(LBL0+K))=1.0
   50    DGV(I,LBL0+K)=1.0
   60    LBL0=LBL0+LBL(J)
         IOF0=0
         DO 80 IND1=1,L4
         IPOS=1+(IPV(IND1)-1)/ISEG
         IBANC=1+MOD(IPV(IND1)-1,ISEG)
         IOF1=MUIN(IND1)
         DO 70 JND1=1-IOF1+IOF0,0
   70    ASSV(IBANC,MUV(IPOS)+JND1)=ASSIN(IOF1+JND1)
         DGV(IBANC,IPOS)=ASSIN(IOF1)
   80    IOF0=IOF1
      ENDIF
      RETURN
      END
