*DECK XSRES
      SUBROUTINE XSRES(IPGEO,IPLDEV,LHEX,GEOTYP,NX,NY,NZ,NXP1,NYP1,
     1 NZP1,NMXGEO,NGRP,NMIXT,NL,MESHX,MESHY,MESHZ,HX,HY,HZ,MIX,
     2 TOTAL,ZNUSIG,ZSIGF,DIFFX,DIFFY,DIFFZ,SCAT,XFACC,XFACP,LSIG,IPRT)
C
C THIS SUBOUTINE CALCULATES THE CORE CROSS SECTIONS
C AS MODIFIED BY THE PRESENCE OF DEVICES
C
C INPUTS -  IPGEO : IPLIST OF L_GEOM
C          IPLDEV : IPLIST OF L_DEVICE
C          LHEX  : FLAG FOR HEXAGONAL 3-D GEOMETRY
C          GEOTYP : TYPE OF GEOMETRY (ONLY CARTESIAN 2D 3D SUPPORTED) (F
C          NX : NUMBER OF NODES IN X DIRECTION
C          NY : NUMBER OF NODES IN Y DIRECTION
C          NZ : NUMBER OF NODES IN Z DIRECTION
C          NXP1 : NUMBER OF COORDINATES IN X DIRECTION FOR DISCRETISATIO
C          NYP1 : NUMBER OF COORDINATES IN Y DIRECTION FOR DISCRETISATIO
C          NZP1 : NUMBER OF COORDINATES IN Z DIRECTION FOR DISCRETISATIO
C          NMXGEO : NUMBER OF MIX IN L_GEOM
C          NGRP : NUMBER OF GROUPS
C          NMIXT : NUMBER OF MIX ON L_MACROLIB2
C          NL : ORDER OF LEGENDRE EXPANSION OF SCATTERING CROSS-SECTIONS
C          XFAC : FACTEUR DE CORRECTION DES DELTA SIGMA
C          IPRT : CODE D'IMPRESSION
C OUTPUTS
C          MESHX MESHY MESHZ : X Y AND Z COORDINATES OF DISCRETISATION P
C          HX HY HZ : X Y ANZD Z WIDTHS OF NODES
C          MIX :  MIX NUMBER FOR EACH NODE
C          TOTAL ... SCAT : NEW VALUES TO PUT BACK IN L_MACROLIB3
*
*-------------------------------------- AUTHOR: J. KOCLAS 94/04/10 ---
*    MODIFIED BY E. VARIN-95/03/20
C
*
      IMPLICIT NONE
*
* SETARA VARIABLES
      REAL BASE
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      INTEGER IPGEO,IPLDEV
      INTEGER GEOTYP,NGRP,NMIXT,NL,IPRT
      INTEGER NX,NY,NZ,NXP1,NYP1,NZP1,NMXGEO
      REAL MESHX(NXP1),MESHY(NYP1),MESHZ(NZP1),HX(NX),HY(NY),HZ(NZ)
      INTEGER MIX(NMXGEO)
*
      REAL TOTAL((NMIXT+1),NGRP),ZNUSIG((NMIXT+1),NGRP),
     +   ZSIGF((NMIXT+1),NGRP)
      REAL DIFFX((NMIXT+1),NGRP),DIFFY((NMIXT+1),NGRP),
     + DIFFZ((NMIXT+1),NGRP)
      REAL SCAT((NMIXT+1),NL,NGRP,NGRP)
C
C DEVICE STRUCTURE
C
      REAL DEVPOS(6)
      INTEGER MIXNB(2)
C LOCAL
      INTEGER I,MAPR,MAPD
      CHARACTER*12 NAMP
      INTEGER IDEV,NHEX,ITHEX,IHEX
      INTEGER ITOT,ILONG,INMT,ITXSM
      CHARACTER NAMT*12,NMLCM*12,MYNAM*12,FIRST*12
      LOGICAL LCM,LSIG,LHEX
*
      INTEGER ITOT2,ILONG2,INMT2,ITXSM2
      CHARACTER NAMT2*12,NMLCM2*12,MYNAM2*12,FIRST2*12
      LOGICAL LCM2
*
      INTEGER ITOT3,ILONG3,INMT3,ITXSM3
      CHARACTER NAMT3*12,NMLCM3*12,MYNAM3*12,FIRST3*12
      LOGICAL LCM3
*
      INTEGER ITOT4,ILONG4,INMT4,ITXSM4
      CHARACTER NAMT4*12,NMLCM4*12,MYNAM4*12,FIRST4*12
      LOGICAL LCM4
*
*
*     XS CORRECTION FACTORS
      REAL XFACC,XFACP,XFAC
*
      CALL LCMGET(IPGEO,'MIX',MIX)
      IF(GEOTYP.EQ.9) THEN
       CALL LCMGET(IPGEO,'MESHZ',MESHZ)
      ELSEIF((GEOTYP.EQ.7).OR.(GEOTYP.EQ.5)) THEN
        CALL LCMGET(IPGEO,'MESHX',MESHX)
        CALL LCMGET(IPGEO,'MESHY',MESHY)
        IF(GEOTYP.EQ.7) THEN
          CALL LCMGET(IPGEO,'MESHZ',MESHZ)
        ELSE IF(GEOTYP.EQ.5) THEN
*    ARBITRARY
          MESHZ(1)=0.
          MESHZ(2)=1.
        ENDIF
      ELSEIF(GEOTYP.EQ.6) THEN
       CALL LCMGET(IPGEO,'RADIUS',MESHX)
       CALL LCMGET(IPGEO,'MESHZ',MESHZ)
      ELSEIF(GEOTYP.EQ.8) THEN
*    ARBITRARY
          MESHZ(1)=0.
          MESHZ(2)=1.
      ELSE
       CALL XABORT('XSRES : WRONG GEOTYP')
      ENDIF
C
C GET MESH WIDTHS
C
      IF(.NOT.LHEX) THEN
        IF(GEOTYP.NE.6) THEN
          DO 3 I=2,NXP1
            HX(I-1)=MESHX(I)-MESHX(I-1)
 3        CONTINUE
          DO 4 I=2,NYP1
            HY(I-1)=MESHY(I)-MESHY(I-1)
 4        CONTINUE
        ELSE
          DO 2 I=2,NXP1
            HX(I-1)=MESHX(I)**2-MESHX(I-1)**2
 2        CONTINUE
        ENDIF
      ENDIF
      DO 5 I=2,NZP1
        HZ(I-1)=MESHZ(I)-MESHZ(I-1)
 5    CONTINUE
*
      IDEV=0
      NAMP = ' '
C FIRST LOOP : TYPE
       ITOT=0
       INMT=0
       CALL LCMSIX(IPLDEV,NAMP,0)
       NAMT=' '
       CALL LCMNXT(IPLDEV,NAMT,NMLCM,MYNAM,LCM)
       FIRST=NAMT
10      INMT=INMT+1
         CALL LCMNXT(IPLDEV,NAMT,NMLCM,MYNAM,LCM)
         CALL LCMLEN(IPLDEV,NAMT,ILONG,ITXSM)
         IF (ITXSM.EQ.0) THEN
C SECOND LOOP: FAMILY
          WRITE(6,*) ' TYPE ',NAMT
          ITOT=ITOT+1
          CALL LCMSIX(IPLDEV,NAMT,1)
           ITOT2=0
           INMT2=0
           NAMT2=' '
           CALL LCMNXT(IPLDEV,NAMT2,NMLCM2,MYNAM2,LCM2)
           FIRST2=NAMT2
30          INMT2=INMT2+1
            CALL LCMNXT(IPLDEV,NAMT2,NMLCM2,MYNAM2,LCM2)
            CALL LCMLEN(IPLDEV,NAMT2,ILONG2,ITXSM2)
            IF(ITXSM2.EQ.0) THEN
             ITOT2=ITOT2+1
C THIRD LOOP : SEQUENCE
             CALL LCMSIX(IPLDEV,NAMT2,1)
              ITOT3=0
              INMT3=0
              NAMT3=' '
              CALL LCMNXT(IPLDEV,NAMT3,NMLCM3,MYNAM3,LCM3)
              FIRST3=NAMT3
50            INMT3=INMT3+1
              CALL LCMNXT(IPLDEV,NAMT3,NMLCM3,MYNAM3,LCM3)
              CALL LCMLEN(IPLDEV,NAMT3,ILONG3,ITXSM3)
              IF(ITXSM3.EQ.0) THEN
C FOURTH LOOP : DESCRIPTION
               CALL LCMSIX(IPLDEV,NAMT3,1)
*  HERE IS THE REQUIRED INFORMATION
               IF(LHEX) THEN
                 CALL LCMLEN(IPLDEV,'NHEX',NHEX,ITHEX)
                 IF(NHEX.EQ.0) CALL XABORT('XSRES: HEXAGON NUMBERS'
     +              //' NOT PRESENT IN DEVICE')
                 CALL SETARA(IBASE,NHEX,IHEX)
                 CALL LCMGET(IPLDEV,'NHEX',IBASE(IHEX))
               ELSE
                 NHEX = 1
                 CALL SETARA(IBASE,NHEX,IHEX)
                 IBASE(IHEX) = 0
               ENDIF
               ITOT4=0
               INMT4=0
               NAMT4=' '
               CALL LCMNXT(IPLDEV,NAMT4,NMLCM4,MYNAM4,LCM4)
               FIRST4=NAMT4
80             INMT4=INMT4+1
               CALL LCMNXT(IPLDEV,NAMT4,NMLCM4,MYNAM4,LCM4)
               CALL LCMLEN(IPLDEV,NAMT4,ILONG4,ITXSM4)
               IF(ITXSM4.EQ.0) THEN
                  CALL LCMSIX(IPLDEV,NAMT4,1)
C
C THIS IS WHERE ALL IS AVAILABLE
C
                  IDEV=IDEV+1
                  CALL LCMGET(IPLDEV,'POSITION',DEVPOS)
                  CALL LCMGET(IPLDEV,'MIX_NUMBER',MIXNB)
                  MAPR=MIXNB(1)
                  MAPD=MIXNB(2)
                  IF (MAPD.EQ.MAPR) GOTO 141
                  IF (MAPD.EQ.0) MAPD = NMIXT+1
*
                  IF(NAMT.EQ.'CONTROLLED') THEN
                    XFAC = XFACC
                  ELSEIF (NAMT.EQ.'PERT') THEN
                    XFAC = XFACP
                  ELSEIF (NAMT.EQ.'STRUCTURAL') THEN
                    XFAC = XFACC
                  ELSE
                    XFAC = XFACC
                  ENDIF
                  IF( IPRT.GT.3 )
     +               WRITE(6,*) ' TYPE ',NAMT,' XFAC ',XFAC
*
                  CALL DVPOST(NAMT4,GEOTYP,LHEX,IBASE(IHEX),NHEX,MIX,
     +              DEVPOS,NX,NY,NZ,MESHX,MESHY,MESHZ,HX,HY,HZ,NXP1,
     +              NYP1,NZP1,TOTAL,ZNUSIG,ZSIGF,DIFFX,DIFFY,DIFFZ,
     +              SCAT,MAPR,MAPD,XFAC,NGRP,NL,NMIXT,LSIG,IPRT)
* BACK IN DIRECTORY
 141              CALL LCMSIX(IPLDEV,NAMT4,2)
                ENDIF
                IF(NAMT4.EQ.FIRST4) GO TO 70
                GOTO 80
70              CONTINUE
* BACK IN DIRECTORY
                CALL LCMSIX(IPLDEV,NAMT3,2)
                CALL RLSARA(IBASE(IHEX))
              ENDIF
              IF(NAMT3.EQ.FIRST3) GO TO 60
              GOTO 50
60            CONTINUE
* BACK IN DIRECTORY
              CALL LCMSIX(IPLDEV,NAMT2,2)
          ENDIF
          IF(NAMT2.EQ.FIRST2) GOTO 40
          GOTO 30
40        CONTINUE
          CALL LCMSIX(IPLDEV,NAMT,2)
        ENDIF
      IF (NAMT.EQ.FIRST) GO TO 20
      GO TO 10
*
   20 CONTINUE
      IF( IPRT.GT.0 )THEN
         WRITE(6,*) 'NUMBER OF DESCRIPTIONS OF DEVICES SCANNED ',IDEV
      ENDIF
*
      RETURN
      END
