*DECK XSTAB
      SUBROUTINE XSTAB(IPLIST,JPLIST,NISO,DERIV,UPS,LBURN,NBURN,NGRP,
     1 NL,IMPX,HISO,ITY,CONC,ZTOTAL,ZZNUG,ZNUGF,ZCHI,ZOVERV,ZDIFFX,
     2 ZDIFFY,ZDIFFZ,ZH,ZSCAT,ZFLUX,BURNUP,DENSIT)
*
*-----------------------------------------------------------------------
*
* RECOVER A COMPO INFORMATIONS FOR CONSTRUCTING FUEL TABLE
*
* INPUT PARAMETERS:
*  IPLIST : ADDRESS OF THE TABLE LINKED LIST OR XSM FILE.
*  JPLIST : ADDRESS OF THE COMPO LINKED LIST OR XSM FILE.
*  NISO   : 1+NUMBER OF EXTRACTED ISOTOPES.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NL     : NUMBER OF LEGENDRE ORDERS (=1 FOR ISOTROPIC SCATTERING).
*  IMPX   : PRINT PARAMETER. EQUAL TO ZERO FOR NO PRINT.
*  HISO   : HOLLERITH NAME INFORMATION FOR EXTRACTED ISOTOPES.
*           DIMENSION HISO(3*NISO)
*  DERIV  : =.TRUE.: DERIVATIVE OF MACROLIB INFO IS COMPUTED WITH
*           RESPECT TO BURN1.
*  NBURN  : NUMBER OF TABULATED BURNUP STEPS.
*  BURNUP : BURNUP OR NEUTRON EXPOSURE TABULATION POINTS.
*           DIMENSION BURNUP(NBURN)
*  ITY    : =0: DO NOT PROCESS THE ISOTOPE; =1: USE NUMBER DENSITY
*           STORED IN CONC(I); =2: USE NUMBER DENSITY STORED IN COMPO.
*           DIMENSION ITY(NISO)
*  CONC   : USER DEFINED NUMBER DENSITY. DIMENSION CONC(NISO)
*
* INTERPOLATED OUTPUT PARAMETERS:
*  ZTOTAL  : TOTAL MACROSCOPIC X-SECTIONS. DIMENSION ZTOTAL(2*NGRP*NBURN
*  ZZNUG   : NU * FISSION MACROSCOPIC X-SECTIONS.
*            DIMENSION ZZNUG(2*NGRP*NBURN)
*  ZNUGF   : FISSION MACROSCOPIC X-SECTIONS.
*            DIMENSION ZNUGF(2*NGRP*NBURN)
*  ZCHI    : FISSION SPECTRUM. DIMENSION ZCHI(2*NGRP*NBURN)
*  ZOVERV  : RECIPROCAL NEUTRON VELOCITIES.
*            DIMENSION ZOVERV(2*NGRP*NBURN)
*  ZDIFFX  : X-DIRECTED DIFFUSION COEFFICIENTS.
*            DIMENSION ZDIFFX(2*NGRP*NBURN)
*  ZDIFFY  : Y-DIRECTED DIFFUSION COEFFICIENTS.
*            DIMENSION ZDIFFY(2*NGRP*NBURN)
*  ZDIFFZ  : Z-DIRECTED DIFFUSION COEFFICIENTS.
*            DIMENSION ZDIFFZ(2*NGRP*NBURN)
*  ZH      : H-FACTORS (KAPPA * FISSION MACROSCOPIC X-SECTIONS).
*           DIMENSION ZH(2*NGRP*NBURN)
*  ZSCAT   : SCATTERING MACROSCOPIC X-SECTIONS.
*           DIMENSION ZSCAT(2*NL*NGRP*NGRP*NBURN)
*  ZFLUX   : INTEGRATED FLUX. DIMENSION ZFLUX(2*NGRP*NBURN)
*
* SCRATCH STORAGE:
*  DENSIT : DIMENSION DENSIT(NISO)
*
*-------------------------------------- AUTHOR: E. VARIN ; 11/09/95 ---
*
      LOGICAL DERIV,UPS,LBURN
      INTEGER JPLIST,IPLIST,IDIM(4)
      DIMENSION HISO(*),BURNUP(*),ITY(*),CONC(*),ZTOTAL(NBURN,NGRP,*),
     1 ZZNUG(NBURN,NGRP,*),ZNUGF(NBURN,NGRP,*),
     3 ZCHI(NBURN,NGRP,*),ZOVERV(NBURN,NGRP,*),ZDIFFX(NBURN,NGRP,*),
     4 ZDIFFY(NBURN,NGRP,*),ZDIFFZ(NBURN,NGRP,*),ZH(NBURN,NGRP,*),
     5 ZSCAT(NBURN,NL,NGRP,NGRP,*),ZFLUX(NBURN,NGRP,*),DENSIT(*)
      COMMON BASE(1)
*
* RECOVER MACROSCOPIC X-SECTION INFO FROM BURNUP DIRECTORIES:
*
      IF(LBURN) THEN
       CALL LCMGET(JPLIST,'BURNUP',BURNUP)
      ELSE
       CALL LCMGET(JPLIST,'N/KB',BURNUP)
      ENDIF
      CALL SETARA(BASE,NGRP,ITOTAL)
      CALL SETARA(BASE,NGRP,IZNUG)
      CALL SETARA(BASE,NGRP,INUGF)
      CALL SETARA(BASE,NGRP,ICHI)
      CALL SETARA(BASE,NGRP,IOVERV)
      CALL SETARA(BASE,NGRP,IDIFFX)
      CALL SETARA(BASE,NGRP,IDIFFY)
      CALL SETARA(BASE,NGRP,IDIFFZ)
      CALL SETARA(BASE,NGRP,IH)
      CALL SETARA(BASE,NL*NGRP*NGRP,ISCAT)
      CALL SETARA(BASE,NGRP,IFLUX)
*
      CALL CRETAB(JPLIST,NISO,NGRP,NL,IMPX,HISO,NBURN,
     1 BURNUP,ITY,CONC,BASE(ITOTAL),BASE(IZNUG),BASE(INUGF),BASE(ICHI),
     2 BASE(IOVERV),BASE(IDIFFX),BASE(IDIFFY),BASE(IDIFFZ),BASE(IH),
     3 BASE(ISCAT),BASE(IFLUX),ZTOTAL,ZZNUG,ZNUGF,ZCHI,ZOVERV,ZDIFFX,
     4 ZDIFFY,ZDIFFZ,ZH,ZSCAT,ZFLUX,DENSIT,UPS)
*
      CALL RLSARA(BASE(IFLUX))
      CALL RLSARA(BASE(ISCAT))
      CALL RLSARA(BASE(IH))
      CALL RLSARA(BASE(IDIFFZ))
      CALL RLSARA(BASE(IDIFFY))
      CALL RLSARA(BASE(IDIFFX))
      CALL RLSARA(BASE(IOVERV))
      CALL RLSARA(BASE(ICHI))
      CALL RLSARA(BASE(IZNUG))
      CALL RLSARA(BASE(INUGF))
      CALL RLSARA(BASE(ITOTAL))
*
* STORAGE OF TABLES
*
      IDIM(1) = NGRP
      IDIM(2) = NBURN
      IDIM(3) = NL
      IDIM(4) = 2
      CALL LCMPUT(IPLIST,'IDIM',4,1,IDIM)
      IF(LBURN) THEN
        CALL LCMPUT(IPLIST,'BURNUP',NBURN,2,BURNUP)
      ELSE
        CALL LCMPUT(IPLIST,'N/KB',NBURN,2,BURNUP)
      ENDIF
      CALL LCMPUT(IPLIST,'TOTAL',2*NGRP*NBURN,2,ZTOTAL)
      CALL LCMPUT(IPLIST,'NUSIGF',2*NGRP*NBURN,2,ZZNUG)
      CALL LCMPUT(IPLIST,'NFTOT',2*NGRP*NBURN,2,ZNUGF)
      CALL LCMPUT(IPLIST,'CHI',2*NGRP*NBURN,2,ZCHI)
      CALL LCMPUT(IPLIST,'OVERV',2*NGRP*NBURN,2,ZOVERV)
      CALL LCMPUT(IPLIST,'DIFFX',2*NGRP*NBURN,2,ZDIFFX)
      CALL LCMPUT(IPLIST,'DIFFY',2*NGRP*NBURN,2,ZDIFFY)
      CALL LCMPUT(IPLIST,'DIFFZ',2*NGRP*NBURN,2,ZDIFFZ)
      CALL LCMPUT(IPLIST,'H-FACTORS',2*NGRP*NBURN,2,ZH)
      CALL LCMPUT(IPLIST,'FLUX-INTG',2*NGRP*NBURN,2,ZFLUX)
      CALL LCMPUT(IPLIST,'SCAT',2*NL*NGRP*NGRP*NBURN,2,ZSCAT)
*
      RETURN
      END
