      SUBROUTINE KDIGET(IFILE,DATA,IOFSET,LENGTH,IRC)
*
*-----------------------------------------------------------------------
*
*                   KDIGET/PUT/CL (MULTI BUFFER VERSION)
*                   ------------------------------------
*
*  FUNCTION: THIS ROUTINE WILL PROVIDE THE POSSIBILITY TO USE A DIRECT
*            ACCESS (DA) FILE TO SIMULATE MEMORY LIKE SUPPORT. MAXBUF
*            STATIC BUFFERS ARE USED.
*
*  ENTRY POINTS:
*  . KDIGET    EXTRACT DATA FROM THE DA FILE.
*  . KDIPUT    SAVE DATA INTO THE DA FILE.
*  . KDICL     TERMINATE OPERATIONS ON THE DA FILE.
*
*  REMARK : FILE OPENING MUST BE DONE USING THE OPEN STATEMENT
*           PROVIDED IN THE FORTRAN-77 STANDARD:
*           OPEN(IFILE,ACCESS='DIRECT',RECL=LRECL*IUNIT) WHERE IUNIT
*           IS THE NUMBER OF MEMORY UNITS IN A SINGLE PRECISION WORD.
*
*  DUMMY ARGUMENTS:
*  IFILE  : UNIT NUMBER OF THE DA FILE.
*  DATA   : ADDR. IN MEMORY WHERE DATA IS TO BE MOVED OR EXTRACTED.
*  IOFSET : OFFSET, FROM START OF FILE WHERE DATA IS EXTRACTED FROM
*           OR WHERE DATA IS TO BE STORED.
*  LENGTH : NUMBER OF WORDS TO READ FROM OR WRITE INTO FILE.
*  ISTATU : =1 TO KEEP THE FILE AT CLOSE ; =2 TO DESTROY IT.
*  IRC    : RETURN CODE (ZERO IF NO ERROR IS DETECTED).
*
*  SAVED VARIABLES:
*  NBUF           : NUMBER OF BUFFERS USED. NBUF.LE.MAXBUF
*  IBUFS          : INDEX OF THE LAST BUFFER USED.
*  NRECS(IBUF)    : RELATIVE RECORD NUMBER CORRESPONDING TO THE BUFFER.
*  IFILES(IBUF)   : UNIT NUMBER CORRESPONDING TO THE BUFFER.
*  IAGE(IBUF)     : AGE OF THE BUFFER.
*  MODIF(IBUF)    : MODIFICATION FLAG FOR THE BUFFER.
*  RECORD(1,IBUF) : STATIC BUFFER. DIMENSION RECORD(LRECL,MAXBUF)
*
*------------------------------------- AUTHOR: A. HEBERT ; 21/07/88 ----
*
      IMPLICIT INTEGER(A-Z)
      PARAMETER (MAXBUF=10,LRECL=1024)
      CHARACTER TEXT72*72
      LOGICAL MODIF(MAXBUF)
      DIMENSION DATA(*),NRECS(MAXBUF),IFILES(MAXBUF),IAGE(MAXBUF),
     1 RECORD(LRECL,MAXBUF)
      SAVE NBUF,IBUFS,NRECS,IFILES,IAGE,MODIF,RECORD
      DATA NBUF/0/
*
      IRC=0
      NREC=1+IOFSET/LRECL
      N=MOD(IOFSET,LRECL)
      LMIN=1
   10 NGRO=MIN(LRECL+LMIN-N-1,LENGTH)
      KDM=-1
      DO 40 IIII=1,NBUF
      IBUF=1+MOD(IIII+IBUFS-2,NBUF)
      IF ((IFILE.EQ.IFILES(IBUF)).AND.(NREC.EQ.NRECS(IBUF))) THEN
         DO 20 L=LMIN,NGRO
         N=N+1
   20    DATA(L)=RECORD(N,IBUF)
         DO 30 I=1,NBUF
         IF (I.NE.IBUF) IAGE(I)=IAGE(I)+1
   30    CONTINUE
         IBUFS=IBUF
         IF (NGRO.EQ.LENGTH) RETURN
         NREC=NREC+1
         N=0
         LMIN=NGRO+1
         IBUFS=IBUF+1
         GO TO 10
      ENDIF
      IF (IAGE(IBUF).GT.KDM) THEN
         KDM=IAGE(IBUF)
         KBUF=IBUF
      ENDIF
   40 CONTINUE
      IF (NBUF.LT.MAXBUF) THEN
         NBUF=NBUF+1
         IBUFS=NBUF
      ELSE
         IBUFS=KBUF
         IF (MODIF(IBUFS)) THEN
*           -------------------------------------------------------
            WRITE(IFILES(IBUFS),REC=NRECS(IBUFS),ERR=50,IOSTAT=IRC)
     1      (RECORD(I,IBUFS),I=1,LRECL)
*           -------------------------------------------------------
         ENDIF
      ENDIF
      NRECS(IBUFS)=NREC
      IFILES(IBUFS)=IFILE
      IAGE(IBUFS)=0
      MODIF(IBUFS)=.FALSE.
*     ------------------------------------------------------------------
      READ(IFILE,REC=NREC,ERR=50,IOSTAT=IRC) (RECORD(I,IBUFS),I=1,LRECL)
*     ------------------------------------------------------------------
      GO TO 10
   50 RETURN
*
      ENTRY KDIPUT(IFILE,DATA,IOFSET,LENGTH,IRC)
      IRC=0
      NREC=1+IOFSET/LRECL
      N=MOD(IOFSET,LRECL)
      LMIN=1
   60 NGRO=MIN(LRECL+LMIN-N-1,LENGTH)
      KDM=-1
      DO 90 IIII=1,NBUF
      IBUF=1+MOD(IIII+IBUFS-2,NBUF)
      IF ((IFILE.EQ.IFILES(IBUF)).AND.(NREC.EQ.NRECS(IBUF))) THEN
         DO 70 L=LMIN,NGRO
         N=N+1
   70    RECORD(N,IBUF)=DATA(L)
         DO 80 I=1,MAXBUF
         IF (I.NE.IBUF) IAGE(I)=IAGE(I)+1
   80    CONTINUE
         MODIF(IBUF)=.TRUE.
         IBUFS=IBUF
         IF (NGRO.EQ.LENGTH) RETURN
         NREC=NREC+1
         N=0
         LMIN=NGRO+1
         IBUFS=IBUF+1
         GO TO 60
      ENDIF
      IF (IAGE(IBUF).GT.KDM) THEN
         KDM=IAGE(IBUF)
         KBUF=IBUF
      ENDIF
   90 CONTINUE
      IF (NBUF.LT.MAXBUF) THEN
         NBUF=NBUF+1
         IBUFS=NBUF
      ELSE
         IBUFS=KBUF
         IF (MODIF(IBUFS)) THEN
*           --------------------------------------------------------
            WRITE(IFILES(IBUFS),REC=NRECS(IBUFS),ERR=100,IOSTAT=IRC)
     1      (RECORD(I,IBUFS),I=1,LRECL)
*           --------------------------------------------------------
         ENDIF
      ENDIF
      NRECS(IBUFS)=NREC
      IFILES(IBUFS)=IFILE
      IAGE(IBUFS)=0
      MODIF(IBUFS)=.FALSE.
      IF ((N.NE.0).OR.(LRECL+LMIN-N-1.GT.LENGTH)) THEN
*        -------------------------------------------------------
         READ(IFILE,REC=NREC,ERR=95,IOSTAT=II) (RECORD(I,IBUFS),
     1   I=1,LRECL)
*        -------------------------------------------------------
      ENDIF
      GO TO 60
   95 INQUIRE(IFILE,NAME=TEXT72,RECL=NN)
      CLOSE(IFILE,STATUS='KEEP')
      OPEN(IFILE,FILE=TEXT72,ACCESS='DIRECT',RECL=NN,STATUS='OLD')
      GO TO 60
  100 RETURN
*
      ENTRY KDICL(IFILE,ISTATU,JRC)
      IF(ISTATU.EQ.1) THEN
         DO 110 IBUF=1,NBUF
         IF (MODIF(IBUF).AND.(IFILE.EQ.IFILES(IBUF))) THEN
*           -----------------------------------------------
            WRITE(IFILE,REC=NRECS(IBUF),ERR=130,IOSTAT=JRC)
     1      (RECORD(I,IBUF),I=1,LRECL)
*           -----------------------------------------------
            MODIF(IBUF)=.FALSE.
         ENDIF
         IF (IFILE.EQ.IFILES(IBUF)) IFILES(IBUF)=0
  110    CONTINUE
         CLOSE(IFILE,ERR=130,STATUS='KEEP',IOSTAT=JRC)
      ELSE IF(ISTATU.EQ.2) THEN
         DO 120 IBUF=1,NBUF
         IF (IFILE.EQ.IFILES(IBUF)) THEN
            IAGE(IBUF)=999999
            MODIF(IBUF)=.FALSE.
            IFILES(IBUF)=0
         ENDIF
  120    CONTINUE
         CLOSE(IFILE,ERR=130,STATUS='DELETE',IOSTAT=JRC)
      ELSE
         JRC=-999
      ENDIF
  130 RETURN
      END
