*DECK ALGUER
      SUBROUTINE ALGUER(A,M,X,ITS)
*
*-----------------------------------------------------------------------
*
* FIND ONE ROOT OF A POLYNOMIAL.
*
*
* INPUT PARAMETERS:
*  A   : COMPLEX POLYNOMIAL COEFFICIENTS. DIMENSION A(2,M+1)
*  M   : POLYNOMIAL ORDER.
*
* OUTPUT PARAMETER:
*  X   : COMPLEX SINGLE ROOT.
*  ITS : NUMBER OF ITERATIONS.
*
*-----------------------------------------------------------------------
*
      IMPLICIT         NONE
      INTEGER          M,ITS
      DOUBLE PRECISION A(2,M+1),X(2)
*----
*  LOCAL VARIABLES
*----
      INTEGER          MR,MT,MAXIT,IREAL,IMAGI,ITER,J
      PARAMETER       (MR=8,MT=10,MAXIT=MT*MR,IREAL=1,IMAGI=2)
      DOUBLE PRECISION EPSS,ABSX,B(2),ABSB,ERR,D(2),F(2),CDINV(2),
     >                 G(2),G2(2),H(2),SQ(2),GP(2),ABSP,GM(2),ABSM,
     >                 DX(2),ABSDX,X1(2),DPOWS2,CDTEMP(2),FRAC(MR)
      PARAMETER       (EPSS=2.D-7,DPOWS2=1.0D0/2.0D0)
      SAVE             FRAC
      DATA      FRAC   /0.50D0,0.25D0,0.75D0,0.13D0,0.38D0,0.62D0,
     >                  0.88D0,1.00D0/
*----
*  START ITERATION
*----
      DO 12 ITER=1,MAXIT
        ITS=ITER
        CALL DCAABS(ABSX,X)
        B(IREAL)=A(IREAL,M+1)
        B(IMAGI)=A(IMAGI,M+1)
        CALL DCAABS(ABSB,B)
        ERR=ABSB
        D(IREAL)=0.0D0
        D(IMAGI)=0.0D0
        F(IREAL)=0.0D0
        F(IMAGI)=0.0D0
        DO 11 J=M,1,-1
          CDTEMP(IREAL)=X(IREAL)*F(IREAL)-X(IMAGI)*F(IMAGI)
          CDTEMP(IMAGI)=X(IREAL)*F(IMAGI)+X(IMAGI)*F(IREAL)
          F(IREAL)=D(IREAL)+CDTEMP(IREAL)
          F(IMAGI)=D(IMAGI)+CDTEMP(IMAGI)
          CDTEMP(IREAL)=X(IREAL)*D(IREAL)-X(IMAGI)*D(IMAGI)
          CDTEMP(IMAGI)=X(IREAL)*D(IMAGI)+X(IMAGI)*D(IREAL)
          D(IREAL)=B(IREAL)+CDTEMP(IREAL)
          D(IMAGI)=B(IMAGI)+CDTEMP(IMAGI)
          CDTEMP(IREAL)=X(IREAL)*B(IREAL)-X(IMAGI)*B(IMAGI)
          CDTEMP(IMAGI)=X(IREAL)*B(IMAGI)+X(IMAGI)*B(IREAL)
          B(IREAL)=A(IREAL,J)+CDTEMP(IREAL)
          B(IMAGI)=A(IMAGI,J)+CDTEMP(IMAGI)
          CALL DCAABS(ABSB,B)
          ERR=ABSB+ABSX*ERR
11      CONTINUE
        ERR=EPSS*ERR
        IF(ABSB.LE.ERR) THEN
          RETURN
        ELSE
          CALL DCADIV(CDINV,B)
          G(IREAL)=D(IREAL)*CDINV(IREAL)-D(IMAGI)*CDINV(IMAGI)
          G(IMAGI)=D(IREAL)*CDINV(IMAGI)+D(IMAGI)*CDINV(IREAL)
          G2(IREAL)=G(IREAL)*G(IREAL)-G(IMAGI)*G(IMAGI)
          G2(IMAGI)=2.0D0*G(IREAL)*G(IMAGI)
          CDTEMP(IREAL)=F(IREAL)*CDINV(IREAL)-F(IMAGI)*CDINV(IMAGI)
          CDTEMP(IMAGI)=F(IREAL)*CDINV(IMAGI)+F(IMAGI)*CDINV(IREAL)
          H(IREAL)=G2(IREAL)-2.0D0*CDTEMP(IREAL)
          H(IMAGI)=G2(IMAGI)-2.0D0*CDTEMP(IMAGI)
          SQ(IREAL)=DBLE(M-1)*(DBLE(M)*H(IREAL)-G2(IREAL))
          SQ(IMAGI)=DBLE(M-1)*(DBLE(M)*H(IMAGI)-G2(IMAGI))
          CALL DCAPOW(SQ,SQ,DPOWS2)
          GP(IREAL)=G(IREAL)+SQ(IREAL)
          GP(IMAGI)=G(IMAGI)+SQ(IMAGI)
          CALL DCAABS(ABSP,GP)
          GM(IREAL)=G(IREAL)-SQ(IREAL)
          GM(IMAGI)=G(IMAGI)-SQ(IMAGI)
          CALL DCAABS(ABSM,GM)
          IF(ABSP.LT.ABSM) THEN
            GP(IREAL)=GM(IREAL)
            GP(IMAGI)=GM(IMAGI)
          ENDIF
          IF(MAX(ABSP,ABSM).GT.0.D0) THEN
            CALL DCADIV(CDINV,GP)
            DX(IREAL)=DBLE(M)*CDINV(IREAL)
            DX(IMAGI)=DBLE(M)*CDINV(IMAGI)
          ELSE
            DX(IREAL)=(1.0D0+ABSX)*COS(DBLE(ITER))
            DX(IMAGI)=(1.0D0+ABSX)*SIN(DBLE(ITER))
          ENDIF
        ENDIF
        CALL DCAABS(ABSDX,DX)
        IF(ABSDX.EQ.0.0D0) RETURN
        X1(IREAL)=X(IREAL)-DX(IREAL)
        X1(IMAGI)=X(IMAGI)-DX(IMAGI)
        IF (MOD(ITER,MT).NE.0) THEN
          X(IREAL)=X1(IREAL)
          X(IMAGI)=X1(IMAGI)
        ELSE
          X(IREAL)=X(IREAL)-FRAC(ITER/MT)*DX(IREAL)
          X(IMAGI)=X(IMAGI)-FRAC(ITER/MT)*DX(IMAGI)
        ENDIF
12    CONTINUE
      CALL XABORT('ALGUER: TOO MANY ITERATIONS.')
      END
