*DECK IQSIGV
      SUBROUTINE IQSIGV(N,H,GAM,FP,A,NDG,NGRP,AF,IND)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C      KAPS AND RENTROP INTEGRATION METHOD FOR STIFF SYSTEMS OF
C      ORDINARY DIFFERENTIAL EQUATIONS
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  THIS SUBROUTINE COMPUTES THE A MATRIX AND ITS INVERSE (THETA METHOD).
C  THE A MATRIX HAS THE FOLLOWING FORM:
C                        A=I-H*GAM*FP(Y)
C     INPUT PARAMETERS :
C
C     H                : LENGTH OF THE INTEGRATION STEP
C     GAM              : KAPS AND RENTROP COEFFICIENT
C     N                : TOTAL NUMBER OF EQUATIONS
C     FP               : JACOBIAN (N X N) MATRIX
C     NDG              : NUMBER OF DELAYED NEUTRON GROUPS
C     NGRP             : NUMBER OF ENERGY GROUPS
C
C     OUTPUT PARAMETERS:
C     FP               : (N X N) INVERSE OF A MATRIX
C     WORK SPACE MEMORY:
C     AF, IND          : DIMENSION AF(2*NGRP,2*NGRP), IND(2*NGRP)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC AUTHOR: M.T. SISSAOUI ; 15/09/94 CCCCC
C
       DOUBLE PRECISION  A(N,N), FP(N,N),AF(NGRP,NGRP),IND(NGRP),H,P
       P=GAM*H
C
C COMPUTE A MATRIX
          DO I=1,N
          DO J=1,N
          A(I,J)=-FP(I,J)*P
          ENDDO
          ENDDO
          DO I=1,N
          A(I,I)=1.D0+A(I,I)
          ENDDO
C
C COMPUTE THE COEFFCIENTS ALFA
          DO I=1,N
          DO J=1,N
          FP(I,J)=0.0D0
          ENDDO
          ENDDO
          DO IGR=1,NGRP
          LI=NGRP+(IGR-1)*NDG+1
          LF=NGRP+IGR*NDG
          DO JGR=1,NGRP
          AF(IGR,JGR)=A(IGR,JGR)
          DO L=LI,LF
          AF(IGR,JGR)=AF(IGR,JGR)-A(IGR,L)*A(L,JGR)/A(L,L)
          ENDDO
          ENDDO
          ENDDO
C  COMPUTE THE INVERT OF AF(NGRP,NGRP)
*
          CALL ALINVD(NGRP,AF,NGRP,IER,IND)
*
          DO IGR=1,NGRP
          DO JGR=1,NGRP
          LI=NGRP+(JGR-1)*NDG+1
          LF=NGRP+JGR*NDG
          DO L=LI,LF
          FP(IGR,L)=FP(IGR,L)-AF(IGR,JGR)*A(JGR,L)/A(L,L)
          FP(IGR,N)=FP(IGR,N)+AF(IGR,JGR)*A(JGR,L)*A(L,N)/A(L,L)
          ENDDO
          FP(IGR,JGR)=AF(IGR,JGR)
          FP(IGR,N)=FP(IGR,N)-AF(IGR,JGR)*A(JGR,N)
          ENDDO
          ENDDO
C COMPUTE THE INVERT OF THE PRECURSORS
          IPI=NGRP+1
          IPF=NGRP+NGRP*NDG
          DO I=IPI,IPF
          DO J=1,N
          DO IGR=1,NGRP
          FP(I,J)=FP(I,J)-A(I,IGR)*FP(IGR,J)/A(I,I)
          ENDDO
          IF(I.EQ.J) THEN
          FP(I,J)=FP(I,J)+1.0D0/A(I,I)
          ENDIF
           IF(J.EQ.N) THEN
          FP(I,J)=FP(I,J)-A(I,J)/A(I,I)
           ENDIF
          ENDDO
          ENDDO
C COMPUTE THE INVERT OF THE GAMMA TERMS
          DO IGR=1,NGRP
          IND1=NGRP*(NDG+1)+3*NDG*(IGR-1)+1
          IND3=NGRP*(NDG+1)+3*NDG*(IGR-1)+3*NDG
          DO I=IND1,IND3
          DO J=1,N
          FP(I,J)=FP(I,J)-A(I,IGR)*FP(IGR,J)/A(I,I)
          IF(I.EQ.J) THEN
          FP(I,J)=FP(I,J)+1.0D0/A(I,I)
          ENDIF
           IF(J.EQ.N) THEN
          FP(I,J)=FP(I,J)-A(I,J)/A(I,I)
           ENDIF
          ENDDO
          ENDDO
          ENDDO
          FP(N,N)=1.0D0
c          DO II=1,N
c          DO JJ=1,N
c          XX=0.0D0
c          DO LL =1,N
c          XX= XX+FP(II,LL)*A(LL,JJ)
c          ENDDO
c          WRITE(6,*)'SISSAXX'
c          WRITE(6,*)'FP*A ',XX,'I ',II,'J ',JJ
c          ENDDO
c          ENDDO
          RETURN
          END
