*DECK IQSIVT
      SUBROUTINE IQSIVT(N,H,GAM,FP,A,NDG)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C      KAPS AND RENTROP INTEGRATION METHOD FOR STIFF SYSTEMS OF
C      ORDINARY DIFFERENTIAL EQUATIONS
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  THIS SUBROUTINE COMPUTES THE A MATRIX AND ITS INVERSE (THETA METHOD).
C  THE A MATRIX HAS THE FOLLOWING FORM:
C                        A=I-H*GAM*FP(Y)
C     INPUT PARAMETERS :
C
C     H                : LENGTH OF THE INTEGRATION STEP
C     GAM              : KAPS AND RENTROP COEFFICIENT
C     N                : TOTAL NUMBER OF EQUATIONS
C     FP               : JACOBIAN (N X N) MATRIX
C     NDG              : NUMBER OF DELAYED NEUTRON GROUPS
C     OUTPUT PARAMETERS:
C     FP               : (N X N) INVERSE OF A MATRIX
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCC AUTHOR: M.T. SISSAOUI ; 15/09/94 CCCCCC
C
       DOUBLE PRECISION  A(N,N), FP(N,N),H,P
       DOUBLE PRECISION XDEL,SUM,SUM1,SUM2
       NDG1=NDG+2
          NDG3=NDG+3
          NDG4=4*NDG+3
          SUM=0.
          SUM1=0.
          SUM2=0.
       P=GAM*H
C
C COMPUTE A MATRIX
          DO I=1,N
          DO J=1,N
          A(I,J)=-FP(I,J)*P
          ENDDO
          ENDDO
          DO I=1,N
          A(I,I)=1.D0+A(I,I)
          ENDDO
C
C COMPUTE THE INVERSE OF A MATRIX
          DO I=1,N
          DO J=1,N
          FP(I,J)=0.
          ENDDO
          ENDDO
          DO I=2,NDG1
          SUM=SUM+A(1,I)*A(I,1)/A(I,I)
          ENDDO
          DO I=3,NDG1
          SUM1=SUM1+A(1,I)*A(2,1)*A(I,2)/A(I,I)/A(2,2)
          ENDDO
          DO I=3,NDG1
          SUM2=SUM2+A(1,I)*A(I,2)/A(I,I)
          ENDDO
          XDEL=A(1,1)-SUM+SUM1
          FP(1,1)=1./XDEL
          FP(1,N)=(A(1,2)*A(2,N)-A(2,2)*A(1,N))/XDEL/A(2,2)
          FP(1,N)=FP(1,N) -SUM2*A(2,N)/A(2,2)/XDEL
          FP(N,N)=1.
          DO I=2,NDG1
           FP(1,I)=-(A(1,I)/A(I,I))/XDEL
          IF(I.EQ.2) THEN
          FP(1,I)=FP(1,I)+SUM2/XDEL/A(2,2)
          ENDIF
          ENDDO
          DO I=2,NDG1
          DO J=1,N
          FP(I,J)=-A(I,1)*FP(1,J)/A(I,I)
          IF(I.EQ.J) THEN
          FP(I,J)=FP(I,J)+1./A(I,I)
          ENDIF
           IF(J.EQ.N) THEN
          FP(I,J)=FP(I,J)-A(I,J)/A(I,I)
           ENDIF
          ENDDO
          ENDDO
          DO I=3,NDG1
          DO J=1,N
          FP(I,J)=FP(I,J)-A(I,2)*FP(2,J)/A(I,I)
          ENDDO
          ENDDO
          RETURN
          END
