*DECK IQSKAR
       SUBROUTINE IQSKAR(N,Y,H,FP,A,Y4,Y3,YDUM,F,F1,F2,F3,
     x F4,FDUM,TMICRO,METHOD,DEL,DT,NDG,QSM,NGRP,
     x P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C      KAPS AND RENTROP INTEGRATION METHOD FOR STIFF SYSTEMS OF
C      ORDINARY DIFFERENTIAL EQUATIONS
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     THIS SUBROUTINE COMPUTES THE INTEGRAL OF THE KINETICS EQUATIONS.
C
C     INPUT PARAMETERS :
C
C     Y                : AMPLITUDE EQUATIONS
C     N                : TOTAL NUMBER OF EQUATIONS
C     H                : LENGTH OF THE FIRST INTEGRATION STEP
C     METHOD           : TYPE OF METHOD (GRKA OR GRKT)
C     NDG              : NUMBER OF DELAYED NEUTRON GROUPS
C     DT               : TIME AT WHICH THE SOLUTION IS DESIRED
C     DEL              : TOLERANCE VALUE
C     QSM              : LOGICAL VARIABLE USED TO INDICATE THE TYPE OF
C                        THE METHOD (GQS OR IQS)
C
C     OUTPUT PARAMETERS:
C     Y                : AMPLITUDE SOLUTION
C
C     WORK SPACE MEMORY:
C     FP,A             : DIMENSION FP(2*N,2*N),A(2*N,2*N)
C
C     F1,F2,F3,F4,F    : DIMENSION F1(2*N),F2(2*N),F3(2*N)
C                                  F4(2*N),F(2*N)
C     YDUM,Y3,Y4,FDUM  : DIMENSION YDUM(2*N),Y3(2*N),Y4(2*N)
C                                  FDUM(2*N)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC AUTHOR: M.T. SISSAOUI ; 15/01/94 CCCC
C
       INTEGER N
       DOUBLE PRECISION FP(N,N),A(N,N),Y(N),Y3(N)
     x  ,Y4(N),YDUM(N),F(N),F1(N),F2(N),
     x F3(N),F4(N),FDUM(N)
       DOUBLE PRECISION PB00(NGRP,NGRP),PB01(NGRP,NGRP),
     x PB10(NGRP,NGRP),PB11(NGRP,NGRP),
     x P00(NGRP,NGRP), P01(NGRP,NGRP), P10(NGRP,NGRP),
     x P11(NGRP,NGRP),R0CPC(NGRP),TERM(NGRP)
       DOUBLE PRECISION H,TMICRO,TMAX,DT,YSCAL,ERROR
       DOUBLE PRECISION TMICTEST,HBARNEW
       CHARACTER*5 METHOD
       LOGICAL QSM
C
C INITIAL Y
C
      DO I=1,N-1
      Y4(I)=Y(I)
      Y3(I)=Y(I)
      ENDDO
C
C INITIAL TIME
C
      Y4(N)=0.
      Y3(N)=0.
      Y(N)=0.
C
      TMAX=TMICRO+DT
      TMICTEST=H
      ISIS=0
      YSCAL=1.D0
C
 111  CONTINUE
      IF(H.LT.5.D-10) CALL XABORT('IQSKAR:STEP LESS THEN 5E-11')
      IF(QSM) THEN
      CALL IQSISI(METHOD,FP,A,Y,H,Y4,Y3,YDUM,N,F1,F2,F3,F4,F,FDUM,NDG,
     x NGRP,P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM)
      ELSE
      CALL IQSISG(METHOD,FP,A,Y,H,Y4,Y3,YDUM,N,F1,F2,F3,F4,F,FDUM,NDG,
     x NGRP,P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM)
      ENDIF
C         YSCAL=1.D0
C COMPUTE THE TRUNCATION ERROR
      ERROR=0.
      DO I=1,N
      YSCAL=1.D0
      YSCAL=MAX(YSCAL,Y4(I))
      ERROR=MAX(ERROR,ABS(Y4(I)-Y3(I))/YSCAL)
      ENDDO
C
C EVALUATION OF THE NEW STEP SIZE
         HBARNEW=0.9*H*((DEL/ERROR)**(0.25))
         IF(HBARNEW .GT. 1.5*H) THEN
         HBARNEW=1.5*H
         ELSE IF(HBARNEW .LT. 0.5*H) THEN
         HBARNEW=0.5*H
         ENDIF
C
      IF(ERROR.LT.DEL) THEN
C
C STEP SUCCEEDED
       IF(ISIS.EQ.1) THEN
       DO I=1,N
       Y(I)=Y4(I)
       ENDDO
       GO TO 112
       ENDIF
C
       DO I=1,N
       Y(I)=Y4(I)
       ENDDO
       H=HBARNEW
C
C RETURN AND COMPUTE NEXT STEP
       IF(TMICTEST+H.LT.TMAX) THEN
       TMICTEST=TMICTEST+H
       ISIS=0
       GO TO 111
       ELSE
       H=TMAX-TMICTEST
       TMICTEST=TMICTEST+H
       ISIS=1
       GO TO 111
       ENDIF
      ELSE
C
C
C GO BACK AND RE-TRY STEP
       TMICTEST=TMICTEST-H
       H=HBARNEW
       DO I=1,N
       Y4(I)=Y(I)
       Y3(I)=Y(I)
       ENDDO
       IF(TMICTEST+H.LT.TMAX) THEN
       TMICTEST=TMICTEST+H
       ISIS=0
       GO TO 111
       ELSE
       H=TMAX-TMICTEST
       TMICTEST=TMICTEST+H
       ISIS=1
       GO TO 111
       ENDIF
      ENDIF
 112      RETURN
          END
