*DECK IQSPRT
      SUBROUTINE IQSPRT(IPSYS0,IPSYSP,NDG,BETA,DECRAT,DTMACR,L4,EVECT,
     1 DVECT,FKEFF,PRECP,ISW,PREC,DELSRC,NGRP,RAPQ,AMP,AMPGA,TETA,LBI)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C   ***************************************************************    C
C   *     COMPUTE THE DELAYED SOURCE AND UPDATE THE PRECURSOR     *    C
C   ***************************************************************    C
C   THETA METHOD                                                       C
C INPUT PARAMETERS:                                                    C
C                                                                      C
C  IPSYSP : L_SYSTEM POINTER TO DELTA SYSTEM MATRICES.                 C
C  IPSYS0 : L_SYSTEM POINTER TO UNPERTURBED SYSTEM MATRICES.           C
C  NDG    : TOTAL NUMBER OF DELAYED NEUTRON GROUPS.                    C
C  LBI    : LOGICAL VARIABLE FOR NEUTRON FISSION SPECTRUM.             C
C  BETA   : DELAYED NEUTRON FRACTION FOR EACH GROUP.                   C
C  DECRAT : DELAYED NEUTRON DECAY RATE FOR EACH GROUP.                 C
C  NGRP   : NUMBER OF ENERGY GROUPS                                    C
C  TETA   : TETA VALUE                                                 C
C  AMPGA  : INITIAL VALUE OF THE AMPLITUDE; DIMENSION AMPGA(NGRP).     C
C  AMP    : AMPLITUDE VALUE AMP(NGRP).                                 C
C  L4     : TOTAL NUMBER OF UNKOWNS PER GROUP.                         C
C  EVECT  : DIMENSION EVECT(NUN,NGRP)                                  C
C  DVECT  : DIMENSION DVECT(NUN,NGRP)                                  C
C  DTMACR :                                                            C
C  ISW    :   ISW=1  SOURCE UPDATE                                     C
C         :   ISW.NE.1 PRECURSOR UPDATE                                C
C                                                                      C
C OUTPUT PARAMETERS:                                                   C
C                                                                      C
C  PREC   : PRECURSORS CONCENTRATIONS; DIMENSION PREC(L4,NDG,NGRP).    C
C  DELSRC : DELAYED SOURCE; DIMENSION DELSRC(L4,NGRP).                 C
C                                                                      C
C SCRATCH STORAGE:                                                     C
C  PRECP  : DIMENSION PRECP(L4,NDG,NGRP).                              C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC AUTHOR: M.T. SISSAOUI ; 28/09/94 CCC
C
      DIMENSION BETA(NDG),DECRAT(NDG),EVECT(L4,NGRP),PREC(L4,NDG,NGRP)
     x         ,DVECT(L4,NGRP),DELSRC(L4,NGRP),PRECP(L4,NDG,NGRP),
     x          RAPQ(NDG)
      DOUBLE PRECISION AMP(NGRP),AMPGA(NGRP)
      CHARACTER*12 NAME,NAMLCM,NAMMY,NAMB
      LOGICAL LCM,LCM0,LBI
      COMMON BASE(1)
********
       TETAD=1.-TETA
********
C
       DO 200 IGR=1,NGRP
      DO 201 I=1,NDG
      EX = EXP(-DECRAT(I)*DTMACR)
      RAP=1+TETA*DECRAT(I)*DTMACR
      RAPC=(1-TETAD*DECRAT(I)*DTMACR)/RAP
      DO 202 J=1,L4
      PRECP(J,I,IGR)=DBLE(PREC(J,I,IGR)*RAPC)
 202  CONTINUE
 201  CONTINUE
 200  CONTINUE
* SET  DELSRC TO ZERO
      DO 161 IGR=1,NGRP
      DO 151 I=1,L4
 151  DELSRC(I,IGR)  =  0.0
 161  CONTINUE
****
      NAME=' '
      CALL LCMNXT(IPSYSP,NAME,NAMLCM,NAMMY,LCM)
      NAME=' '
      CALL LCMNXT(IPSYS0,NAME,NAMLCM,NAMMY,LCM0)
*
*                  DELAYED SOURCE
*                  **************
      IF( ISW.EQ.1 ) THEN
       IF(LBI) THEN
       DO 931 IGR=1,NGRP
        DO 950 I=1,NDG
        RAP=1+TETA*DECRAT(I)*DTMACR
        RAPC=(1-TETAD*DECRAT(I)*DTMACR)*DECRAT(I)/RAP
        RAPB=BETA(I)*DTMACR*DECRAT(I)/RAP
        RAPQ(I)=  RAPB*TETA
        RAPQD =  AMPGA(1)*TETAD*RAPB
*
        DO 932 JGR=1,NGRP
        WRITE(NAMB,'(1HB,I3,I3,I3)') I,IGR,JGR
        CALL LCMLEN(IPSYS0,NAMB,LENGB0,ITYLCM)
      IF(LCM0) THEN
*        IN-CORE STORAGE.
          IF(LENGB0.GT.0) I0B=LCMIOF(IPSYS0,NAMB)
      ELSE
*        XSM STORAGE.
         IF(LENGB0.GT.0) CALL SETARA(BASE,LENGB0,I0B)
         IF(LENGB0.GT.0) CALL LCMGET(IPSYS0,NAMB,BASE(I0B))
      ENDIF
         IF(LENGB0.GT.0) THEN
        DO 955 II=1,L4
 955     DELSRC(II,IGR)= DELSRC(II,IGR) +
     x  RAPQD*DBLE(BASE(I0B-1+II)*EVECT(II,JGR))/FKEFF
         IF(.NOT.LCM0) THEN
         CALL RLSARA(BASE(I0B))
         ENDIF
        ENDIF
  932    CONTINUE
C
        DO 960 J=1,L4
        DELSRC(J,IGR) = DBLE(PREC(J,I,IGR)*RAPC)+DELSRC(J,IGR)
 960     CONTINUE
 950     CONTINUE
 931     CONTINUE
*
        ELSE
*
        DO 31 IGR=1,NGRP
        RAPQ(1)=0.0
        RAPQD=0.0
        DO 50 I=1,NDG
        RAP=1+TETA*DECRAT(I)*DTMACR
        RAPC=(1-TETAD*DECRAT(I)*DTMACR)*DECRAT(I)/RAP
        RAPB=BETA(I)*DTMACR*DECRAT(I)/RAP
        RAPQ(1)= RAPQ(1) + RAPB*TETA
        RAPQD = RAPQD + AMPGA(1)*TETAD*RAPB
 50     CONTINUE
*
        DO 32 JGR=1,NGRP
        WRITE(NAMB,'(1HB,I3,I3)') IGR,JGR
        CALL LCMLEN(IPSYS0,NAMB,LENGB0,ITYLCM)
      IF(LCM0) THEN
*        IN-CORE STORAGE.
          IF(LENGB0.GT.0) I0B=LCMIOF(IPSYS0,NAMB)
      ELSE
*        XSM STORAGE.
         IF(LENGB0.GT.0) CALL SETARA(BASE,LENGB0,I0B)
         IF(LENGB0.GT.0) CALL LCMGET(IPSYS0,NAMB,BASE(I0B))
      ENDIF
         IF(LENGB0.GT.0) THEN
        DO 55 II=1,L4
 55     DELSRC(II,IGR)= DELSRC(II,IGR) +
     x  RAPQD*DBLE(BASE(I0B-1+II)*EVECT(II,JGR))/FKEFF
         IF(.NOT.LCM0) THEN
         CALL RLSARA(BASE(I0B))
         ENDIF
        ENDIF
  32    CONTINUE
C
        DO 60 I=1,NDG
        RAP=1+TETA*DECRAT(I)*DTMACR
        RAPC=(1-TETAD*DECRAT(I)*DTMACR)*DECRAT(I)/RAP
        DO 60 J=1,L4
        DELSRC(J,IGR) = DBLE(PREC(J,I,IGR)*RAPC)+DELSRC(J,IGR)
 60     CONTINUE
 31     CONTINUE
*
        ENDIF
*
       ELSE
*                  UPDATE PRECURSOR
*                  ****************
       DO 61 IGR=1,NGRP
       DO 70 I=1,NDG
        RAP=1+TETA*DECRAT(I)*DTMACR
        RAPB=BETA(I)*DTMACR/RAP
       EX = EXP(-DECRAT(I)*DTMACR)
*
       DO 62 JGR=1,NGRP
       IF(LBI) THEN
       WRITE(NAMB,'(1HB,I3,I3,I3)') I,IGR,JGR
       ELSE
       WRITE(NAMB,'(1HB,I3,I3)') IGR,JGR
       ENDIF
       CALL LCMLEN(IPSYSP,NAMB,LENGB,ITYLCM)
       CALL LCMLEN(IPSYS0,NAMB,LENGB0,ITYLCM)
      IF(LCM) THEN
*        IN-CORE STORAGE.
         IF(LENGB.GT.0) I1B=LCMIOF(IPSYSP,NAMB)
      ELSE
*        XSM STORAGE.
         IF(LENGB.GT.0) CALL SETARA(BASE,LENGB,I1B)
         IF(LENGB.GT.0) CALL LCMGET(IPSYSP,NAMB,BASE(I1B))
      ENDIF
      IF(LCM0) THEN
*        IN-CORE STORAGE.
          IF(LENGB0.GT.0) I0B=LCMIOF(IPSYS0,NAMB)
      ELSE
*        XSM STORAGE.
         IF(LENGB0.GT.0) CALL SETARA(BASE,LENGB0,I0B)
         IF(LENGB0.GT.0) CALL LCMGET(IPSYS0,NAMB,BASE(I0B))
      ENDIF
         IF(LENGB0.GT.0) THEN
         DO 71 J=1,L4
         PRECP(J,I,IGR) = PRECP(J,I,IGR)  +
     &   AMPGA(1)*TETAD*RAPB*DBLE(BASE(I0B-1+J)*EVECT(J,JGR))/FKEFF
 71      CONTINUE
          IF(.NOT.LCM0) THEN
          CALL RLSARA(BASE(I0B))
          ENDIF
         ENDIF
         IF(LENGB.GT.0) THEN
         DO 72 J=1,L4
         PRECP(J,I,IGR) = PRECP(J,I,IGR)  +
     &   AMP(1)*TETA*RAPB*DBLE(BASE(I1B-1+J)*DVECT(J,JGR))/FKEFF
 72      CONTINUE
          IF(.NOT.LCM) THEN
          CALL RLSARA(BASE(I1B))
          ENDIF
         ENDIF
 62      CONTINUE
 70      CONTINUE
 61      CONTINUE
       DO 100  J=1,L4
       DO 100  I=1,NDG
       DO 100  IGR=1,NGRP
       PREC(J,I,IGR)=PRECP(J,I,IGR)
 100   CONTINUE
       ENDIF
      RETURN
      END
