*DECK IQSSGT
      SUBROUTINE IQSSGT(METHOD,FP,A,Y,H,Y4,Y3,YDUM,NT,F1,F2,F3,F4,F,
     1 FDUM,NDG,NGRP,P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C      KAPS AND RENTROP INTEGRATION METHOD FOR STIFF SYSTEMS OF
C      ORDINARY DIFFERENTIAL EQUATIONS
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     THIS SUBROUTINE COMPUTES THE INTEGRAL FOR ONE STEP SIZE H (GQS)
C     THETA METHOD
C     INPUT PARAMETERS :
C
C     Y                : AMPLITUDE EQUATIONS
C     NT               : TOTAL NUMBER OF EQUATIONS
C     H                : LENGTH OF THE INTEGRATION STEP
C     METHOD           : TYPE OF METHOD (GRK4A OR GRK4T)
C     NDG              : NUMBER OF DELAYED NEUTRON GROUPS
C     NGRP             : NUMBER OF ENERGY GROUPS
C
C     OUTPUT PARAMETERS:
C     Y                : AMPLITUDE SOLUTION  FOR THE NEXT STEP
C     Y3               : THIRD ORDER SOLUTION OF THE AMPLITUDE
C     Y4               : FOURTH ORDER SOLUTION OF THE AMPLITUDE
C
C     WORK SPACE MEMORY:
C     FP,A             : DIMENSION FP(2*NT,2*NT),A(2*NT,2*NT)
C     F1,F2,F3,F4,F    : DIMENSION F1(2*NT),F2(2*NT),F3(2*NT)
C                                  F4(2*NT),F(2*NT)
C     YDUM,FDUM        : DIMENSION YDUM(2*NT),FDUM(2*NT)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCC AUTHOR: M.T. SISSAOUI; 15/09/94 CCCCCC
C     IMPLICIT NONE
      INTEGER NT,NDG,I,NGRP
      DOUBLE PRECISION Y3(NT),Y4(NT),YDUM(NT),Y(NT),F(NT)
      DOUBLE PRECISION FP(NT,NT),A(NT,NT)
      DOUBLE PRECISION F1(NT),F2(NT),F3(NT),F4(NT),FDUM(NT),H
      DOUBLE PRECISION PB00(NGRP,NGRP),PB01(NGRP,NGRP),
     x PB10(NGRP,NGRP),PB11(NGRP,NGRP),
     x P00(NGRP,NGRP), P01(NGRP,NGRP), P10(NGRP,NGRP),
     x P11(NGRP,NGRP),R0CPC(NGRP),TERM(NGRP)
      CHARACTER*5 METHOD
      COMMON BASE(1)
      REAL*4 GAMMA,GAMMA21,GAMMA31,GAMMA32,GAMMA41,GAMMA42,GAMMA43
      REAL*4 BETA21,BETA31,BETA32,BETA41,BETA42,BETA43
      REAL*4 C1,C2,C3,C4,C1_HAT,C2_HAT,C3_HAT
      IF(METHOD.EQ.'GRKA') THEN
C
C KAPS AND RENTROP COEFFICIENTS FOR GRK4A METHOD
       GAMMA=0.395
           GAMMA21=-0.767672395484
           GAMMA31=-0.851675323742
           GAMMA32= 0.522967289188
           GAMMA41= 0.288463109545
           GAMMA42= 0.0880214273381
           GAMMA43=-0.337389840627
       BETA21=0.438
       BETA31=0.796920457938
       BETA32=0.0730795420615
       BETA41=BETA31
       BETA42=BETA32
       BETA43=0.
       C1=0.199293275701
       C2=0.482645235674
       C3=0.0680614886256
       C4=0.25
       C1_HAT=0.346325833758
       C2_HAT=0.285693175712
       C3_HAT=0.36798099053
      ENDIF
      IF(METHOD.EQ. 'GRKT') THEN
C
C KAPS AND RENTROP COEFFICIENTS FOR GRK4T METHOD
       GAMMA=   .231
           GAMMA21=-.270629667752
           GAMMA31= .311254483294
           GAMMA32= .852445628482E-2
           GAMMA41= .282816832044
           GAMMA42=-.457959483281
           GAMMA43=-.111208333333
       BETA21=  .462
       BETA31= -.815668168327E-1
       BETA32=  .961775150166
       BETA41=BETA31
       BETA42=BETA32
       BETA43=0.
       C1=.217487371653
       C2=.486229037990
       C3=0.
       C4=.296283590357
       C1_HAT=-.717088504499
       C2_HAT=1.77617912176E-0
       C3_HAT=-.590906172617E-1
      ENDIF
       CALL SETARA(BASE,NGRP*NGRP*2,IAB)
       CALL SETARA(BASE,NGRP*NGRP*2,IB)
       CALL SETARA(BASE,NGRP*2,IDERC)
       CALL SETARA(BASE,NGRP*2,IR0)
       CALL SETARA(BASE,NGRP*2,IDERL)
       CALL SETARA(BASE,NGRP*2,IPR)
*
       CALL SETARA(BASE,NGRP*NGRP*2,IABV)
       CALL SETARA(BASE,NGRP*NGRP*2,IBV)
       CALL SETARA(BASE,NGRP*2,IR0V)
       CALL SETARA(BASE,NGRP*2,IDERLV)
C
C COMPUTE THE JACOBIAN MATRIX
       CALL IQSPGT(NT,Y,FP,NGRP,
     x P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM,
     x BASE(IAB),BASE(IABV),BASE(IB),BASE(IBV),BASE(IDERC),
     x BASE(IR0),BASE(IDERL),BASE(IR0V),BASE(IDERLV))
      CALL RLSARA(BASE(IDERLV))
      CALL RLSARA(BASE(IR0V))
      CALL RLSARA(BASE(IBV))
      CALL RLSARA(BASE(IABV))
C
C COMPUTE THE INVERT OF A MATRIX
       CALL SETARA(BASE,NGRP*NGRP*2,IAF)
       CALL SETARA(BASE,NGRP*2,IND)
       CALL IQSIGT(NT,H,GAMMA,FP,A,NDG,NGRP,BASE(IAF),BASE(IND))
       CALL RLSARA(BASE(IND))
       CALL RLSARA(BASE(IAF))
C
C DETERMINE F
C
       CALL IQSFGT (NT,Y,F,NGRP,
     x P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM,
     x BASE(IAB),BASE(IB),BASE(IDERC),
     x BASE(IR0),BASE(IDERL),BASE(IPR))
C
C DETERMINE F1
C
       CALL IQSPMV (NT,FP,F,F1)
       DO I=1,NT
            F1(I)=H*F1(I)
       ENDDO
C
C DETERMINE F2
C
       DO I=1,NT
           YDUM(I)=Y4(I)+BETA21*F1(I)
           ENDDO
           CALL IQSFGT (NT,YDUM,FDUM,NGRP,
     x P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM,
     x BASE(IAB),BASE(IB),BASE(IDERC),
     x BASE(IR0),BASE(IDERL),BASE(IPR))
       DO I=1,NT
       FDUM(I)=H*FDUM(I)+(GAMMA21/GAMMA)*F1(I)
       ENDDO
       CALL IQSPMV (NT,FP,FDUM,F2)
       DO I=1,NT
       F2(I)=F2(I)-(GAMMA21/GAMMA)*F1(I)
       ENDDO
C
C DETERMINE F3
C
       DO I=1,NT
           YDUM(I)=Y4(I)+BETA31*F1(I)+BETA32*F2(I)
       ENDDO
           CALL IQSFGT (NT,YDUM,FDUM,NGRP,
     x P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM,
     x BASE(IAB),BASE(IB),BASE(IDERC),
     x BASE(IR0),BASE(IDERL),BASE(IPR))
       DO I=1,NT
       FDUM(I)=H*FDUM(I)+(GAMMA31/GAMMA)*F1(I)+(GAMMA32/GAMMA)*F2(I)
       ENDDO
       CALL IQSPMV (NT,FP,FDUM,F3)
       DO I=1,NT
       F3(I)=F3(I)-(GAMMA31/GAMMA)*F1(I)-(GAMMA32/GAMMA)*F2(I)
       ENDDO
C
C DETERMINE F4
C
       DO I=1,NT
           YDUM(I)=Y4(I)+BETA41*F1(I)+BETA42*F2(I)+BETA43*F3(I)
       ENDDO
           CALL IQSFGT (NT,YDUM,FDUM,NGRP,
     x P00,P01,P10,P11,PB00,PB01,PB10,PB11,R0CPC,TERM,
     x BASE(IAB),BASE(IB),BASE(IDERC),
     x BASE(IR0),BASE(IDERL),BASE(IPR))
*
       CALL RLSARA(BASE(IPR))
       CALL RLSARA(BASE(IDERL))
       CALL RLSARA(BASE(IR0))
       CALL RLSARA(BASE(IDERC))
       CALL RLSARA(BASE(IB))
       CALL RLSARA(BASE(IAB))
       DO I=1,NT
       FDUM(I)=H*FDUM(I)+(GAMMA41/GAMMA)*F1(I)+
     x         (GAMMA42/GAMMA)*F2(I)+(GAMMA43/GAMMA)*F3(I)
       ENDDO
       CALL IQSPMV (NT,FP,FDUM,F4)
       DO I=1,NT
       F4(I)=F4(I)-(GAMMA41/GAMMA)*F1(I)-
     x   (GAMMA42/GAMMA)*F2(I)-(GAMMA43/GAMMA)*F3(I)
       ENDDO
C
C COMPUTE THIRD-ORDER ESTIMATES OF Y
       DO I=1,NT
        Y3(I)=Y4(I)
           ENDDO
       DO I=1,NT
        Y3(I)=Y3(I)+C1_HAT*F1(I)+C2_HAT*F2(I)+C3_HAT*F3(I)
       ENDDO
C
C COMPUTE FOURTH-ORDER ESTIMATES OF Y
       DO I=1,NT
        Y4(I)=Y4(I)+C1*F1(I)+C2*F2(I)+C3*F3(I)+C4*F4(I)
       ENDDO
       RETURN
       END
